/* Xtensa configuration-specific ISA information.

   Copyright (c) 2003-2010 Tensilica Inc.

   Permission is hereby granted, free of charge, to any person obtaining
   a copy of this software and associated documentation files (the
   "Software"), to deal in the Software without restriction, including
   without limitation the rights to use, copy, modify, merge, publish,
   distribute, sublicense, and/or sell copies of the Software, and to
   permit persons to whom the Software is furnished to do so, subject to
   the following conditions:

   The above copyright notice and this permission notice shall be included
   in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.  */

#include "xtensa-isa.h"
#include "xtensa-isa-internal.h"


/* Sysregs.  */

static xtensa_sysreg_internal sysregs[] = {
  { "LBEG", 0, 0 },
  { "LEND", 1, 0 },
  { "LCOUNT", 2, 0 },
  { "BR", 4, 0 },
  { "MMID", 89, 0 },
  { "DDR", 104, 0 },
  { "176", 176, 0 },
  { "208", 208, 0 },
  { "INTERRUPT", 226, 0 },
  { "INTCLEAR", 227, 0 },
  { "CCOUNT", 234, 0 },
  { "PRID", 235, 0 },
  { "ICOUNT", 236, 0 },
  { "CCOMPARE0", 240, 0 },
  { "CCOMPARE1", 241, 0 },
  { "VECBASE", 231, 0 },
  { "EPC1", 177, 0 },
  { "EPC2", 178, 0 },
  { "EPC3", 179, 0 },
  { "EPC4", 180, 0 },
  { "EPC5", 181, 0 },
  { "EPC6", 182, 0 },
  { "EXCSAVE1", 209, 0 },
  { "EXCSAVE2", 210, 0 },
  { "EXCSAVE3", 211, 0 },
  { "EXCSAVE4", 212, 0 },
  { "EXCSAVE5", 213, 0 },
  { "EXCSAVE6", 214, 0 },
  { "EPS2", 194, 0 },
  { "EPS3", 195, 0 },
  { "EPS4", 196, 0 },
  { "EPS5", 197, 0 },
  { "EPS6", 198, 0 },
  { "EXCCAUSE", 232, 0 },
  { "DEPC", 192, 0 },
  { "EXCVADDR", 238, 0 },
  { "WINDOWBASE", 72, 0 },
  { "WINDOWSTART", 73, 0 },
  { "SAR", 3, 0 },
  { "LITBASE", 5, 0 },
  { "PS", 230, 0 },
  { "INTENABLE", 228, 0 },
  { "DBREAKA0", 144, 0 },
  { "DBREAKC0", 160, 0 },
  { "DBREAKA1", 145, 0 },
  { "DBREAKC1", 161, 0 },
  { "IBREAKA0", 128, 0 },
  { "IBREAKA1", 129, 0 },
  { "IBREAKENABLE", 96, 0 },
  { "ICOUNTLEVEL", 237, 0 },
  { "DEBUGCAUSE", 233, 0 },
  { "CPENABLE", 224, 0 },
  { "SCOMPARE1", 12, 0 },
  { "ATOMCTL", 99, 0 },
  { "THREADPTR", 231, 1 },
  { "FCR", 232, 1 },
  { "FSR", 233, 1 },
  { "EXPSTATE", 0, 1 },
  { "SOV", 1, 1 },
  { "SAT_MODE", 2, 1 },
  { "SAR0", 3, 1 },
  { "SAR1", 4, 1 },
  { "SAR2", 5, 1 },
  { "SAR3", 6, 1 },
  { "HSAR0", 7, 1 },
  { "HSAR1", 8, 1 },
  { "HSAR2", 9, 1 },
  { "HSAR3", 10, 1 },
  { "MAX_REG_0", 11, 1 },
  { "MAX_REG_1", 12, 1 },
  { "MAX_REG_2", 13, 1 },
  { "MAX_REG_3", 14, 1 },
  { "ARG_MAX_REG_0", 15, 1 },
  { "ARG_MAX_REG_1", 16, 1 },
  { "ARG_MAX_REG_2", 17, 1 },
  { "ARG_MAX_REG_3", 18, 1 },
  { "NCO_COUNTER_0", 19, 1 },
  { "NCO_COUNTER_1", 20, 1 },
  { "NCO_COUNTER_2", 21, 1 },
  { "NCO_COUNTER_3", 22, 1 },
  { "INTERP_EXT_N", 23, 1 },
  { "INTERP_EXT_L", 24, 1 },
  { "LLR_BUF_0", 25, 1 },
  { "LLR_BUF_1", 26, 1 },
  { "LLR_BUF_2", 27, 1 },
  { "LLR_BUF_3", 28, 1 },
  { "LLR_BUF_4", 29, 1 },
  { "LLR_BUF_5", 30, 1 },
  { "LLR_BUF_6", 31, 1 },
  { "LLR_BUF_7", 32, 1 },
  { "LLR_BUF_8", 33, 1 },
  { "LLR_BUF_9", 34, 1 },
  { "LLR_BUF_10", 35, 1 },
  { "LLR_BUF_11", 36, 1 },
  { "LLR_BUF_12", 37, 1 },
  { "LLR_BUF_13", 38, 1 },
  { "LLR_BUF_14", 39, 1 },
  { "LLR_BUF_15", 40, 1 },
  { "LLR_BUF_16", 41, 1 },
  { "LLR_BUF_17", 42, 1 },
  { "LLR_BUF_18", 43, 1 },
  { "LLR_BUF_19", 44, 1 },
  { "LLR_BUF_20", 45, 1 },
  { "LLR_BUF_21", 46, 1 },
  { "LLR_BUF_22", 47, 1 },
  { "LLR_BUF_23", 48, 1 },
  { "SMOD_BUF_0", 49, 1 },
  { "SMOD_BUF_1", 50, 1 },
  { "SMOD_BUF_2", 51, 1 },
  { "SMOD_BUF_3", 52, 1 },
  { "SMOD_BUF_4", 53, 1 },
  { "SMOD_BUF_5", 54, 1 },
  { "SMOD_BUF_6", 55, 1 },
  { "SMOD_BUF_7", 56, 1 },
  { "WEIGHT_REG", 57, 1 },
  { "SCALE_REG", 58, 1 },
  { "LLR_POS", 59, 1 },
  { "SMOD_POS", 60, 1 },
  { "PERM_REG", 61, 1 },
  { "SMOD_OFFSET_TABLE_0", 62, 1 },
  { "SMOD_OFFSET_TABLE_1", 63, 1 },
  { "SMOD_OFFSET_TABLE_2", 64, 1 },
  { "SMOD_OFFSET_TABLE_3", 65, 1 },
  { "PHASOR_N", 66, 1 },
  { "PHASOR_OFFSET", 67, 1 }
};

#define NUM_SYSREGS 125
#define MAX_SPECIAL_REG 241
#define MAX_USER_REG 233


/* Processor states.  */

static xtensa_state_internal states[] = {
  { "LCOUNT", 32, 0 },
  { "PC", 32, 0 },
  { "ICOUNT", 32, 0 },
  { "DDR", 32, 0 },
  { "INTERRUPT", 13, 0 },
  { "CCOUNT", 32, 0 },
  { "XTSYNC", 1, 0 },
  { "VECBASE", 22, 0 },
  { "EPC1", 32, 0 },
  { "EPC2", 32, 0 },
  { "EPC3", 32, 0 },
  { "EPC4", 32, 0 },
  { "EPC5", 32, 0 },
  { "EPC6", 32, 0 },
  { "EXCSAVE1", 32, 0 },
  { "EXCSAVE2", 32, 0 },
  { "EXCSAVE3", 32, 0 },
  { "EXCSAVE4", 32, 0 },
  { "EXCSAVE5", 32, 0 },
  { "EXCSAVE6", 32, 0 },
  { "EPS2", 13, 0 },
  { "EPS3", 13, 0 },
  { "EPS4", 13, 0 },
  { "EPS5", 13, 0 },
  { "EPS6", 13, 0 },
  { "EXCCAUSE", 6, 0 },
  { "PSINTLEVEL", 4, 0 },
  { "PSUM", 1, 0 },
  { "PSWOE", 1, 0 },
  { "PSEXCM", 1, 0 },
  { "DEPC", 32, 0 },
  { "EXCVADDR", 32, 0 },
  { "WindowBase", 3, 0 },
  { "WindowStart", 8, 0 },
  { "PSCALLINC", 2, 0 },
  { "PSOWB", 4, 0 },
  { "LBEG", 32, 0 },
  { "LEND", 32, 0 },
  { "SAR", 6, 0 },
  { "THREADPTR", 32, 0 },
  { "LITBADDR", 20, 0 },
  { "LITBEN", 1, 0 },
  { "InOCDMode", 1, 0 },
  { "INTENABLE", 13, 0 },
  { "DBREAKA0", 32, 0 },
  { "DBREAKC0", 8, 0 },
  { "DBREAKA1", 32, 0 },
  { "DBREAKC1", 8, 0 },
  { "IBREAKA0", 32, 0 },
  { "IBREAKA1", 32, 0 },
  { "IBREAKENABLE", 2, 0 },
  { "ICOUNTLEVEL", 4, 0 },
  { "DEBUGCAUSE", 6, 0 },
  { "DBNUM", 4, 0 },
  { "CCOMPARE0", 32, 0 },
  { "CCOMPARE1", 32, 0 },
  { "CPENABLE", 4, 0 },
  { "SCOMPARE1", 32, 0 },
  { "ATOMCTL", 6, 0 },
  { "RoundMode", 2, 0 },
  { "InvalidEnable", 1, 0 },
  { "DivZeroEnable", 1, 0 },
  { "OverflowEnable", 1, 0 },
  { "UnderflowEnable", 1, 0 },
  { "InexactEnable", 1, 0 },
  { "InvalidFlag", 1, 0 },
  { "DivZeroFlag", 1, 0 },
  { "OverflowFlag", 1, 0 },
  { "UnderflowFlag", 1, 0 },
  { "InexactFlag", 1, 0 },
  { "FPreserved20", 20, 0 },
  { "FPreserved20a", 20, 0 },
  { "FPreserved5", 5, 0 },
  { "FPreserved7", 7, 0 },
  { "EXPSTATE", 32, XTENSA_STATE_IS_EXPORTED },
  { "SOV", 4, XTENSA_STATE_IS_SHARED_OR },
  { "SAT_MODE", 1, 0 },
  { "SAR0", 6, 0 },
  { "SAR1", 6, 0 },
  { "SAR2", 6, 0 },
  { "SAR3", 6, 0 },
  { "HSAR0", 6, 0 },
  { "HSAR1", 6, 0 },
  { "HSAR2", 6, 0 },
  { "HSAR3", 6, 0 },
  { "MAX_REG", 128, 0 },
  { "ARG_MAX_REG", 128, 0 },
  { "NCO_COUNTER", 128, 0 },
  { "INTERP_EXT_N", 4, 0 },
  { "INTERP_EXT_L", 4, 0 },
  { "LLR_BUF", 768, 0 },
  { "SMOD_BUF", 256, 0 },
  { "WEIGHT_REG", 8, 0 },
  { "SCALE_REG", 5, 0 },
  { "LLR_POS", 6, 0 },
  { "SMOD_POS", 7, 0 },
  { "PERM_REG", 32, 0 },
  { "SMOD_OFFSET_TABLE", 128, 0 },
  { "PHASOR_N", 4, 0 },
  { "PHASOR_OFFSET", 16, 0 }
};

#define NUM_STATES 100

enum xtensa_state_id {
  STATE_LCOUNT,
  STATE_PC,
  STATE_ICOUNT,
  STATE_DDR,
  STATE_INTERRUPT,
  STATE_CCOUNT,
  STATE_XTSYNC,
  STATE_VECBASE,
  STATE_EPC1,
  STATE_EPC2,
  STATE_EPC3,
  STATE_EPC4,
  STATE_EPC5,
  STATE_EPC6,
  STATE_EXCSAVE1,
  STATE_EXCSAVE2,
  STATE_EXCSAVE3,
  STATE_EXCSAVE4,
  STATE_EXCSAVE5,
  STATE_EXCSAVE6,
  STATE_EPS2,
  STATE_EPS3,
  STATE_EPS4,
  STATE_EPS5,
  STATE_EPS6,
  STATE_EXCCAUSE,
  STATE_PSINTLEVEL,
  STATE_PSUM,
  STATE_PSWOE,
  STATE_PSEXCM,
  STATE_DEPC,
  STATE_EXCVADDR,
  STATE_WindowBase,
  STATE_WindowStart,
  STATE_PSCALLINC,
  STATE_PSOWB,
  STATE_LBEG,
  STATE_LEND,
  STATE_SAR,
  STATE_THREADPTR,
  STATE_LITBADDR,
  STATE_LITBEN,
  STATE_InOCDMode,
  STATE_INTENABLE,
  STATE_DBREAKA0,
  STATE_DBREAKC0,
  STATE_DBREAKA1,
  STATE_DBREAKC1,
  STATE_IBREAKA0,
  STATE_IBREAKA1,
  STATE_IBREAKENABLE,
  STATE_ICOUNTLEVEL,
  STATE_DEBUGCAUSE,
  STATE_DBNUM,
  STATE_CCOMPARE0,
  STATE_CCOMPARE1,
  STATE_CPENABLE,
  STATE_SCOMPARE1,
  STATE_ATOMCTL,
  STATE_RoundMode,
  STATE_InvalidEnable,
  STATE_DivZeroEnable,
  STATE_OverflowEnable,
  STATE_UnderflowEnable,
  STATE_InexactEnable,
  STATE_InvalidFlag,
  STATE_DivZeroFlag,
  STATE_OverflowFlag,
  STATE_UnderflowFlag,
  STATE_InexactFlag,
  STATE_FPreserved20,
  STATE_FPreserved20a,
  STATE_FPreserved5,
  STATE_FPreserved7,
  STATE_EXPSTATE,
  STATE_SOV,
  STATE_SAT_MODE,
  STATE_SAR0,
  STATE_SAR1,
  STATE_SAR2,
  STATE_SAR3,
  STATE_HSAR0,
  STATE_HSAR1,
  STATE_HSAR2,
  STATE_HSAR3,
  STATE_MAX_REG,
  STATE_ARG_MAX_REG,
  STATE_NCO_COUNTER,
  STATE_INTERP_EXT_N,
  STATE_INTERP_EXT_L,
  STATE_LLR_BUF,
  STATE_SMOD_BUF,
  STATE_WEIGHT_REG,
  STATE_SCALE_REG,
  STATE_LLR_POS,
  STATE_SMOD_POS,
  STATE_PERM_REG,
  STATE_SMOD_OFFSET_TABLE,
  STATE_PHASOR_N,
  STATE_PHASOR_OFFSET
};


/* Field definitions.  */

static unsigned
Field_t_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_t_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_s_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_s_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_r_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_r_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_op2_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_op1_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_op1_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_op0_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 28) >> 28);
  return tie_t;
}

static void
Field_op0_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf) | (tie_t << 0);
}

static unsigned
Field_n_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_n_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_m_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_m_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_sr_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_sr_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_st_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_st_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_thi3_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_thi3_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2098inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2098inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_sae4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_sae4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2019_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2019_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld2100inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2100inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2102inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2102inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2186inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2186inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2185inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2185inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2149inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2149inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3627inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 10) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3627inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0x300000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2187inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2187inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2101inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2101inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2103inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2103inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2189inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2189inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2188inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2188inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2104inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2104inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2190inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2190inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2094inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 7) | ((insn[0] << 16) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2094inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0xfe00) | (tie_t << 9);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2105inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2105inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2191inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2191inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2192inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2192inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2194inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2194inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2197inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2197inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2160inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2160inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2173inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2173inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2112inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2112inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2199inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2199inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2200inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2200inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2114inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2114inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2113inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2113inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2201inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2201inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2115inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2115inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2215inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2215inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3630inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3630inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2203inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2203inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2254_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 25) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2254_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x60) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2116inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2116inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2117inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2117inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2119inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2119inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 17) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x7000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2089inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2089inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_r2_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_r2_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_bbi4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  return tie_t;
}

static void
Field_bbi4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3631inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3631inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2085inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 11) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2085inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2088inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2088inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3633inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 14) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3633inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20000) | (tie_t << 17);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2082inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2082inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2083inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2083inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2084inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2084inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2086inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2086inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3634_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 14) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3634_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld2156inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 21) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2156inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x700) | (tie_t << 8);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2037_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 21) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2037_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x700) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2021_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2021_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_sa4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  return tie_t;
}

static void
Field_sa4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2035_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 21) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2035_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x600) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2225inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2225inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2226inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2226inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2228inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2228inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2230inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2230inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2222inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2222inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2221inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2221inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2238inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2238inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2239inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2239inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2241inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2241inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2223inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2223inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2232inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2232inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2234inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2234inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2237inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2237inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2240inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2240inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2229inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2229inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2224inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2224inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2227inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2227inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2231inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2231inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2247inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2247inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2091inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2091inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2153inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2153inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3635inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3635inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2154inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2154inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3636inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3636inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2155inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2155inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3637inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3637inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2134inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2134inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 19) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x1e00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2096inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2096inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2244inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2244inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2245inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2245inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2246inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2246inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3638inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3638inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2235inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2235inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2157inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2157inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2253inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 25) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2253inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x60) | (tie_t << 5);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3639inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3639inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2255inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2255inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3640inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3640inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2171inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2171inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2172inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2172inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2174inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2174inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2158inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2158inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2205inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2205inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2159inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2159inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2161inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2161inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2168inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2168inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2136inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2136inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2090inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2090inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2184inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2184inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3642inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3642inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2252inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2252inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3643inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3643inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2092inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 7) | ((insn[0] << 16) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2092inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0xfe00) | (tie_t << 9);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2216inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 21) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2216inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x600) | (tie_t << 9);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3644inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 10) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3644inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x300000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2217inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 21) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2217inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x600) | (tie_t << 9);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3645inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 9) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3645inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0x700000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2208inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2208inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2209inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2209inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2210inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2210inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2212inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2212inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2213inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 21) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2213inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x600) | (tie_t << 9);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3647inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3647inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2214inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2214inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3648inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3648inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2120inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2120inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2122inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2122inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2123inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2123inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2125inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2125inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2129inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 16) >> 27);
  tie_t = (tie_t << 5) | ((insn[0] << 23) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2129inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f0) | (tie_t << 4);
  tie_t = (val << 22) >> 27;
  insn[0] = (insn[0] & ~0xf800) | (tie_t << 11);
  tie_t = (val << 18) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2124inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2124inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2126inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2126inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2127inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2127inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2128inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 12) | ((insn[0] << 16) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2128inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff0) | (tie_t << 4);
  tie_t = (val << 16) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2131inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 16) >> 27);
  tie_t = (tie_t << 5) | ((insn[0] << 23) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2131inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f0) | (tie_t << 4);
  tie_t = (val << 22) >> 27;
  insn[0] = (insn[0] & ~0xf800) | (tie_t << 11);
  tie_t = (val << 18) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2138inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2138inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2146inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2146inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3649inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 19) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3649inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2147inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2147inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3650inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 19) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3650inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f00) | (tie_t << 8);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2139inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2139inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2140inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2140inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2142inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2142inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2248inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2248inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3651inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3651inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2250inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2250inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3653inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3653inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2257inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2257inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3654inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3654inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2249inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2249inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3655inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3655inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2107inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2107inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2118inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2118inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2108inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2108inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2109inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2109inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2111inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2111inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2110inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2110inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2145inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 16) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2145inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf800) | (tie_t << 11);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2141inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2141inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2143inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 16) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2143inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff00) | (tie_t << 8);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2144inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 7) | ((insn[0] << 16) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2144inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0xfe00) | (tie_t << 9);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2204inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 21) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2204inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x700) | (tie_t << 8);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3656inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3656inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2195inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2195inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2196inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2196inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2198inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2198inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2169inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2169inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2220inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2220inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2106inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2106inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2151inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2151inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3657inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 9) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3657inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x700000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2251inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2251inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3658inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3658inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2206inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2206inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2202inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2202inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2095inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 7) | ((insn[0] << 16) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2095inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0xfe00) | (tie_t << 9);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2132inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2132inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
  tie_t = (val << 26) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3659inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3659inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2099inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 7) | ((insn[0] << 16) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2099inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0xfe00) | (tie_t << 9);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2137inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2137inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2133inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2133inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3660inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3660inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2175inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2175inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2177inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2177inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2242inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2242inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3661inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3661inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2162inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2162inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2164inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2164inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2163inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2163inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2218inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2218inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2219inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2219inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2207inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2207inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2211inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2211inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2165inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2165inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2166inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2166inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2178inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2178inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2180inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2180inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2179inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2179inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2181inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2181inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2167inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2167inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2193inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2193inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3662inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3662inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2236inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2236inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2243inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2243inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2182inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2182inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2183inst_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 20) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2183inst_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0) | (tie_t << 4);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_op0_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 28) >> 28);
  return tie_t;
}

static void
Field_op0_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf) | (tie_t << 0);
}

static unsigned
Field_t_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_t_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_r_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_r_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_op0_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 28) >> 28);
  return tie_t;
}

static void
Field_op0_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf) | (tie_t << 0);
}

static unsigned
Field_z_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_z_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_i_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_i_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_s_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_s_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_sae_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_op0_s3_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s3_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld2260gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2260gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2258gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2258gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2282gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2282gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2281gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2281gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2266gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 13) >> 18);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2266gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 17) >> 18;
  insn[0] = (insn[0] & ~0x7ffe0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2302_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2302_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2312gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2312gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2386_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2386_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2283gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2283gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2286gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2286gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2287gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2287gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2289gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2289gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2293gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2293gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2288gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2288gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2290gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2290gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2359gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 11) | ((insn[0] << 19) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2359gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0x1ffc) | (tie_t << 2);
  tie_t = (val << 20) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2384_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2384_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2361gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 11) | ((insn[0] << 19) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2361gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0x1ffc) | (tie_t << 2);
  tie_t = (val << 20) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2362gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 10) | ((insn[0] << 19) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2362gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x1ff8) | (tie_t << 3);
  tie_t = (val << 21) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3663gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3663gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2364gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2364gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 23) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
  tie_t = (val << 22) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3664gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3664gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2366gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2366gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3665gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3665gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2308gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2308gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2259gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2259gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2262gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 13) | ((insn[0] << 13) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2262gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2284gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2284gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2275gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2275gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2354gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2354gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2333gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2333gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2310gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2310gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3667gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3667gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2357gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2357gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2376gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2376gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2343gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2343gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2342gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2342gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2344gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2344gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2345gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2345gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2346gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2346gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2373gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2373gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2374gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2374gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2375gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2375gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2335gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2335gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2339gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2339gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2334gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2334gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2336gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2336gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2337gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2337gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2338gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2338gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2340gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2340gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2341gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2341gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2369gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2369gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3668gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3668gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2280gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2280gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2309gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2309gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2261gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2261gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2321gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2321gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2322gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2322gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2355gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2355gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2324gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2324gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2372gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2372gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3669gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3669gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2263gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 17) | ((insn[0] << 13) >> 15);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2263gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 15) >> 15;
  insn[0] = (insn[0] & ~0x7fffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2264gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 13) >> 18);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2264gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 17) >> 18;
  insn[0] = (insn[0] & ~0x7ffe0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2368gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2368gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3670gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3670gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2291gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2291gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2292gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2292gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2294gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2294gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2295gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2295gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2297gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2297gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2296gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2296gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2301gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2301gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2272_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2272_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2277gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2277gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2279gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2279gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2278gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2278gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2267gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2267gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2268gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2268gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2269gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2269gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2271gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2271gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 21) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2305_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2305_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2270gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2270gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2273gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2273gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 21) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2298gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2298gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2299gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2299gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2300gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2300gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2303gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2303gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3671gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3671gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2379gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2379gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3673gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3673gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2274gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 13) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2274gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x7f800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2306gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2306gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2304gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2304gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3674gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3674gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2353gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2353gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2371gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2371gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3675gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3675gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2314gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2314gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2316gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2316gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2317gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2317gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2319gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2319gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2378gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2378gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3676gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3676gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2323gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2323gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2331gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2331gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2347gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2347gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2383gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2383gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3678gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 19) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3678gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2318gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2318gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2332gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2332gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2348gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2348gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2349gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2349gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2351gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2351gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2370gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2370gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2320gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2320gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2350gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2350gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2352gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2352gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2325gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2325gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2327gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2327gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2326gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2326gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2328gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2328gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2356gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2356gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2329gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2329gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2381gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2381gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3666_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3666_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2330gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2330gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2385gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2385gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2387gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2387gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2388gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2388gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3679gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3679gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2358gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2358gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2389gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2389gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3680gp_slot2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3680gp_slot2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2399gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2399gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_op0_s4_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s4_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3681gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3681gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 23) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2395gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2395gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2394gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2394gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2397gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2397gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2400gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2400gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3683gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3683gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2402gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2402gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2403gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2403gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3684gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3684gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2405gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2405gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3686gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3686gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2398gp_slot1_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2398gp_slot1_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_sae_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_op0_s5_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s5_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld2447gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2447gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3688gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3688gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2449gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 13) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2449gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x60000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3689gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3689gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2418gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2418gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2462_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2462_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2464gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2464gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2454gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2454gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2436gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2436gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 21) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3690gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3690gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2438gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 13) | ((insn[0] << 13) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2438gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2437gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2437gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2526gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2526gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2460gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2460gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2459gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2459gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2445_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2445_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2427gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2427gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 21) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2420gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2420gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2424gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2424gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2425gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2425gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2480gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2480gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2479gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2479gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2419gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2419gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 22) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2481gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2481gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2483gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2483gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2487gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2487gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2482gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2482gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2426gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2426gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2484gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2484gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2422gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2422gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2429gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2429gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2485gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2485gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2486gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2486gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2488gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2488gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2531gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2531gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3691gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 23) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3691gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x1e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2493gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2493gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2506gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2506gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3692gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3692gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2494gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2494gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2407gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2407gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2413gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2413gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2415gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2415gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2417gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2417gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2441gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2441gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3693gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3693gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2409gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2409gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2410gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2410gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2416gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2416gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2411gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2411gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2412gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2412gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2453gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2453gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2451gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2451gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2452gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2452gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2516gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2516gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2512gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2512gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2514gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2514gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2515gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2515gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2443gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 13) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2443gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x7f000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2444gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2444gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2067_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2067_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2439gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2439gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2423gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2423gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2508gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2508gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2058_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2058_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2509gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2509gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2510gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2510gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3696gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3696gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2496gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2496gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_s8_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2527gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2527gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2036_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2036_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2528gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2528gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3697gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 25) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3697gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x60) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2523gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2523gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3698gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3698gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2455gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2455gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2456gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2456gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2458gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2458gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2457gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2457gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2440gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2440gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2524gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2524gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3699gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 25) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3699gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x60) | (tie_t << 5);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2503gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2503gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2507gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 13) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2507gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0x60000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3700gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3700gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2498gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2498gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2500gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2500gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2501gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2501gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2502gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2502gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2504gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2504gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2505gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2505gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2433gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2433gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2430gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2430gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2431gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2431gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2432gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2432gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2434gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2434gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2435gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2435gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2517gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2517gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2518gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2518gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2519gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2519gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2520gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2520gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2489gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2489gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2491gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2491gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2490gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2490gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2492gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2492gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2529gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2529gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3702gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3702gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2468gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2468gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2470gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2470gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2448_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2448_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3703gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3703gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2521gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2521gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2530_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2530_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2461gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2461gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2463gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2463gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2497gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2497gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2499gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2499gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2465gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2465gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2467gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2467gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2471gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2471gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2473gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2473gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2477gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2477gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 21) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3705gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3705gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2472gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2472gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2495gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2495gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3706gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3706gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2466gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2466gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2474gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2474gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2475gp_slot0_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2475gp_slot0_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2595dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 12) >> 25);
  tie_t = (tie_t << 9) | ((insn[0] << 21) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2595dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc) | (tie_t << 2);
  tie_t = (val << 16) >> 25;
  insn[0] = (insn[0] & ~0xfe000) | (tie_t << 13);
}

static unsigned
Field_op0_s6_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s6_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3708dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3708dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2579dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2579dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2580dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2580dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2582dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2582dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2586dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2586dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2581dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2581dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2583dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2583dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2599dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 10) | ((insn[0] << 20) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2599dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc) | (tie_t << 2);
  tie_t = (val << 20) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3709dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3709dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2614_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2614_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2604dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2604dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 23) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3710dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3710dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2606dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2606dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 24) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3711dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3711dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2601dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 10) | ((insn[0] << 20) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2601dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc) | (tie_t << 2);
  tie_t = (val << 20) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2602dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2602dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 21) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3713dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3713dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2645dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2645dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3714dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3714dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2658dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2658dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3716dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3716dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3717dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3717dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 26) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld2636dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 12) >> 18);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2636dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 18) >> 18;
  insn[0] = (insn[0] & ~0xfffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2577dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2577dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2635dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2635dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3719dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3719dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2571dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2571dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2547dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2547dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2646dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2646dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3721dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3721dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2574dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2574dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2655dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2655dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2557dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2557dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2558dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2558dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2560dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2560dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2559dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2559dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2561dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2561dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2562dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2562dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2563dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2563dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2647dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2647dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2549dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2549dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2550dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2550dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2552dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2552dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2556dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2556dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2551dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2551dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2553dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2553dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2554dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2554dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2555dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2555dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2573dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2573dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2628dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2628dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3722_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3722_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2642dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2642dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 22) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2605_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2605_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2539dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2539dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2578_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2578_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2541dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2541dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2572dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2572dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2542dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2542dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3723dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 18) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3723dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x3ff0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2591dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 18) | ((insn[0] << 12) >> 14);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2591dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 14) >> 14;
  insn[0] = (insn[0] & ~0xffffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2596dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2596dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 19) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3724dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3724dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2592dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 12) >> 25);
  tie_t = (tie_t << 9) | ((insn[0] << 21) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2592dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc) | (tie_t << 2);
  tie_t = (val << 16) >> 25;
  insn[0] = (insn[0] & ~0xfe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2598dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2598dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2584dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2584dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2585dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2585dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2587dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2587dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2588dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2588dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2590dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2590dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2589dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2589dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2608dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2608dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2609dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2609dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2610dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2610dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2626dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2626dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3715_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3715_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2613dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2613dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2616dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2616dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2621dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2621dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2615dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2615dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2617dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2617dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2618dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2618dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2637dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 12) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2637dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0xfff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3725dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3725dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2641dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 11) | ((insn[0] << 12) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2641dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0xffe00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3726dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3726dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2619dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2619dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2620dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2620dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2622dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2622dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2623dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2623dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3727dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3727dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2654dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 5) | ((insn[0] << 21) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2654dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c0) | (tie_t << 6);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3728dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3728dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2611dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2611dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2612_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2612_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2624dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 12) >> 24);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2624dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 21) >> 24;
  insn[0] = (insn[0] & ~0xff000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3729dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3729dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2625_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 12) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2625_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3731dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3731dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2643dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2643dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 22) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2640dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 12) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2640dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0xfff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2569dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2569dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2632dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2632dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_t_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2532dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2532dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2644dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2644dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3732dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3732dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2533dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2533dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2534dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2534dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2536dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2536dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2652dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 5) | ((insn[0] << 21) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2652dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c0) | (tie_t << 6);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3718_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3718_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2540dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2540dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2548dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2548dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2564dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2564dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2657dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2657dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2535dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2535dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2537dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2537dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2565dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2565dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2566dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2566dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2568dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2568dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2630dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2630dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2538dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2538dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2633dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2633dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3733dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3733dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2567dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2567dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2544dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2544dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2543dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2543dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2545dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2545dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2546dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2546dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2575dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2575dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2648dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2648dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2649dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2649dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2651dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2651dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2656dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2656dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3712_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3712_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2576dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2576dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2650dot_slot2_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2650dot_slot2_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_op0_s7_Slot_dot_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s7_Slot_dot_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3734dot_slot1_Slot_dot_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 15) | ((insn[0] << 15) >> 17);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3734dot_slot1_Slot_dot_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 17) >> 17;
  insn[0] = (insn[0] & ~0x1fffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2068_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2068_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_op0_s8_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s8_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2668dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2668dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2666dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2666dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2674dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2674dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2688dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2688dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3735dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3735dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2705dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2705dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3737dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3737dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2677dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2677dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2678dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2678dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2679dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2679dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2690dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2690dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2680dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2680dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_t_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2697dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2697dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3738dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3738dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2667dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 17) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2667dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x7000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2704dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 17) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2704dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x7800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3739dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 21) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3739dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2689dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2689dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_r_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2669dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2669dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2671dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2671dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2675dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2675dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2672dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2672dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2673dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2673dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2676dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2676dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2682dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2682dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2686dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2686dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3736_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3736_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2681dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2681dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2683dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2683dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2684dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2684dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2685dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2685dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2699dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2699dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3740dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3740dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2692dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2692dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2693dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2693dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2695dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2695dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3741dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3741dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3742dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3742dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2700dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2700dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2702dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2702dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2701dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2701dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2703dot_slot0_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2703dot_slot0_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_sae_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_op0_s9_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s9_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld2707pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2707pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2739pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2739pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3744pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3744pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2717pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2717pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2713pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 13) >> 18);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2713pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 17) >> 18;
  insn[0] = (insn[0] & ~0x7ffe0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld3745pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3745pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
}

static unsigned
Field_t_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2718pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2718pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2721pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2721pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2722pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2722pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2724pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2724pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2728pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2728pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2736pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2736pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 22) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2723pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2723pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2793pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 11) | ((insn[0] << 19) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2793pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0x1ffc) | (tie_t << 2);
  tie_t = (val << 20) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2795pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 11) | ((insn[0] << 19) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2795pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0x1ffc) | (tie_t << 2);
  tie_t = (val << 20) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2796pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 10) | ((insn[0] << 19) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2796pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x1ff8) | (tie_t << 3);
  tie_t = (val << 21) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3746pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3746pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2798pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2798pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 23) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
  tie_t = (val << 22) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3747pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3747pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2801pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2801pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3749pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3749pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2743pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2743pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3750pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3750pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2706pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 13) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2706pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x7f000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2709pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 13) | ((insn[0] << 13) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2709pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2719pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2719pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2715pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2715pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2788pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2788pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2747pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2747pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2791pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2791pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2775pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2775pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2777pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2777pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2776pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2776pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2778pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2778pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2779pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2779pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2780pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2780pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2810pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2810pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2811pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2811pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2767pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2767pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2769pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2769pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2773pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2773pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2768pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2768pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2770pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2770pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2771pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2771pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2772pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2772pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2774pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2774pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2805pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2805pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3751pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3751pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2741pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2741pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2746pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2746pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3752pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3752pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2755pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2755pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2756pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2756pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2789pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2789pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2758pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2758pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2809pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2809pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3753pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3753pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2710pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 17) | ((insn[0] << 13) >> 15);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2710pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 15) >> 15;
  insn[0] = (insn[0] & ~0x7fffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2711pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 13) >> 18);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2711pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 17) >> 18;
  insn[0] = (insn[0] & ~0x7ffe0) | (tie_t << 5);
}

static unsigned
Field_s_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2803pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2803pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3754pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3754pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2725pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2725pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2726pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2726pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2727pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2727pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2729pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2729pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2730pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2730pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2732pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2732pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2731pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2731pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2733pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2733pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2734pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2734pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2735pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2735pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2807pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2807pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3756pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3756pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2742pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2742pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2738pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2738pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 22) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2708pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2708pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2714pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 13) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2714pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x7fe00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2787pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2787pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2808pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2808pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3757pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3757pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2748pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2748pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2750pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2750pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2751pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2751pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2753pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2753pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2816pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2816pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_imm7_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2757pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2757pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2765pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2765pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2781pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2781pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2814pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2814pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3758pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3758pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2752pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2752pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2766pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2766pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2782pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2782pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2783pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2783pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2785pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2785pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2806pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2806pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2025_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2025_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2754pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2754pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2784pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2784pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2786pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2786pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2759pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2759pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2761pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2761pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2760pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2760pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2762pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2762pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2790pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2790pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2763pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2763pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2812pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2812pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3748_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3748_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2764pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2764pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2818pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2818pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2737_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2737_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2820pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2820pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2821pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2821pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3759pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3759pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2792pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2792pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2823pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2823pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3760pq_slot2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3760pq_slot2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld2826pq_slot1_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2826pq_slot1_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_op0_s10_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s10_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3761pq_slot1_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 7) | ((insn[0] << 23) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3761pq_slot1_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc) | (tie_t << 2);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2825pq_slot1_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2825pq_slot1_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_sae_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_op0_s11_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s11_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld2867pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2867pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3763pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3763pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2869pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 13) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2869pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x60000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3764pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3764pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2838pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2838pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2882_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2882_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2884pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2884pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2874pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2874pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2856pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2856pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 21) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3765pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3765pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2858pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 13) | ((insn[0] << 13) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2858pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2857pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2857pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2946pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2946pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2880pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2880pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2879pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2879pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2865_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2865_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2847pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2847pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 21) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2840pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2840pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2844pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2844pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2845pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2845pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2900pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2900pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2899pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2899pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2839pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2839pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 22) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2901pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2901pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2903pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2903pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2907pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2907pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2902pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2902pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2846pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2846pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2904pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2904pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2842pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2842pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2849pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2849pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2905pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2905pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2906pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2906pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2908pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2908pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2950pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2950pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3766pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 23) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3766pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x1e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2913pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2913pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2926pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2926pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3767pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3767pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2914pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2914pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2827pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2827pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2833pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2833pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2835pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2835pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2837pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2837pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2861pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2861pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3768pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3768pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2829pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2829pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2830pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2830pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2836pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2836pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2831pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2831pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2832pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2832pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2873pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2873pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2871pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2871pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2872pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2872pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2936pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2936pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2932pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2932pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2934pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2934pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2935pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2935pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2863pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 13) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2863pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x7f000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2864pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2864pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2069_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2069_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2859pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2859pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2843pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2843pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2928pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2928pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2059_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2059_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2929pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2929pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2930pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2930pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3769pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3769pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2916pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2916pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_s8_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2947pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2947pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2036_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2036_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2948pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2948pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3770pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 25) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3770pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x60) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2942pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2942pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2875pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2875pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2876pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2876pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2878pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2878pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2877pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2877pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2860pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2860pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2941pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2941pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3771pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3771pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2923pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2923pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2927pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 13) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2927pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0x60000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3773pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3773pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2918pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2918pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2920pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2920pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2921pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2921pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2922pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2922pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2924pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2924pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2925pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2925pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2853pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2853pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2850pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2850pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2851pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2851pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2852pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2852pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2854pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2854pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2855pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2855pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2943pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2943pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3775pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3775pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3776pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3776pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3777pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3777pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2945pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2945pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2909pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2909pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2911pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2911pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2910pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2910pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2912pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2912pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2949pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2949pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3778pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3778pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2888pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2888pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2890pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2890pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2937pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2937pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2939pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2939pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3772_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3772_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2881pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2881pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2883pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2883pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2917pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2917pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2919pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2919pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2885pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2885pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2887pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2887pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2891pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2891pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2893pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2893pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2897pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2897pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 21) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3779pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3779pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2892pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2892pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2915pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2915pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3780pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3780pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2886pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2886pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2894pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2894pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2895pq_slot0_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2895pq_slot0_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2953acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2953acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
}

static unsigned
Field_op0_s12_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s12_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3782acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3782acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2956acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2956acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_s_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2967acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2967acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2966acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2966acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld3783acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3783acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2957acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2957acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2958acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2958acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2960acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2960acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3784acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3784acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2964acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2964acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3785acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3785acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2959acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2959acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2963acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2963acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2954acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2954acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3786acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3786acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2955acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2955acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3788acc2_slot2_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3788acc2_slot2_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2969acc2_slot1_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2969acc2_slot1_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_op0_s13_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s13_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3790acc2_slot1_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 8) >> 18);
  tie_t = (tie_t << 7) | ((insn[0] << 23) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3790acc2_slot1_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc) | (tie_t << 2);
  tie_t = (val << 11) >> 18;
  insn[0] = (insn[0] & ~0xfffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2968acc2_slot1_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2968acc2_slot1_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3793acc2_slot1_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 11) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3793acc2_slot1_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
  tie_t = (val << 22) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 18) >> 28;
  insn[0] = (insn[0] & ~0x1e0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld2075_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2075_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2031_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 11) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2031_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x1e0000) | (tie_t << 17);
}

static unsigned
Field_op0_s14_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s14_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_r_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2973acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2973acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3795acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3795acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2980acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 18) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2980acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x3e00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3796acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3796acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2976acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 18) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2976acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc0) | (tie_t << 6);
}

static unsigned
Field_t_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2989acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2989acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3798acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3798acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2990acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2990acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3799acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3799acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2977acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 18) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2977acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x3e00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3797_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3797_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2981acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2981acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3800acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 20) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3800acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2975acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 18) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2975acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2982acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2982acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3801acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3801acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2984acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2984acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2985acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2985acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3802acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3802acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2987acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2987acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3803acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3803acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2039_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2039_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2974acc2_slot0_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 18) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2974acc2_slot0_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x3f00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3043smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 15) | ((insn[0] << 15) >> 17);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3043smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 17) >> 17;
  insn[0] = (insn[0] & ~0x1fffc) | (tie_t << 2);
  tie_t = (val << 16) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_op0_s15_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s15_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3805smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 13) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3805smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x60000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3061smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3061smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 23) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3806smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3806smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3063smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3063smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 24) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3807smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3807smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3049smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3049smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3050smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3050smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3052smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3052smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3056smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3056smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
  tie_t = (val << 19) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3809smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3809smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3038smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 16) >> 18);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3038smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 18) >> 18;
  insn[0] = (insn[0] & ~0xfffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3091smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3091smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3102smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3102smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3810smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3810smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3116smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3116smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3812smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3812smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3813smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3813smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 26) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3092smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 12) >> 18);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3092smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 18) >> 18;
  insn[0] = (insn[0] & ~0xfffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3047smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3047smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3090smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3090smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3814smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3814smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3030smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3030smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3006smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3006smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3104smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3104smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3816smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3816smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3033smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3033smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3111smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 21) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3111smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x780) | (tie_t << 7);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3016smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3016smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3017smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3017smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3019smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3019smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3018smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3018smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3020smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3020smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3021smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3021smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3022smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3022smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3105smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3105smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3008smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3008smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3009smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3009smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3011smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3011smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3015smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3015smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3010smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3010smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3012smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3012smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3013smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3013smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3014smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3014smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3070_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3070_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3032smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3032smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3084smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3084smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3817_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3817_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3098smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3098smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 22) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3062_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3062_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2998smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2998smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3048_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3048_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3000smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3000smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3031smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3031smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3001smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3001smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3818smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 18) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3818smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x3ff0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld3039smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 18) | ((insn[0] << 12) >> 14);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3039smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 14) >> 14;
  insn[0] = (insn[0] & ~0xffffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3044smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 9) | ((insn[0] << 21) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3044smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc) | (tie_t << 2);
  tie_t = (val << 19) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3819smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3819smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3040smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 15) | ((insn[0] << 15) >> 17);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3040smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 17) >> 17;
  insn[0] = (insn[0] & ~0x1fffc) | (tie_t << 2);
  tie_t = (val << 16) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3046smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3046smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3051smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3051smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3053smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3053smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3054smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3054smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3055smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3055smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3058smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3058smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
  tie_t = (val << 19) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3059smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 14) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3059smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 21) >> 31;
  insn[0] = (insn[0] & ~0x20000) | (tie_t << 17);
  tie_t = (val << 20) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3821smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3821smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3065smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3065smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3066smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3066smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3067smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3067smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3080smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3080smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3069smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3069smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3071smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3071smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3072smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3072smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3074smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3074smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3073smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3073smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3075smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3075smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 19) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3093smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 12) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3093smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0xfff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3822smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3822smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3097smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 11) | ((insn[0] << 12) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3097smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0xffe00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3823smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3823smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3076smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3076smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3077smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3077smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3078smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3078smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3079smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 12) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3079smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3036smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 9) | ((insn[0] << 21) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3036smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc) | (tie_t << 2);
  tie_t = (val << 21) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3068smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3068smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3081smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3081smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3082smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3082smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 24) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3824smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3824smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3100smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 12) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3100smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3825smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3825smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3106smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3106smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3099smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 12) >> 23);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3099smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 22) >> 23;
  insn[0] = (insn[0] & ~0xff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3096smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 12) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3096smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0xfff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3028smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3028smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3087smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3087smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_t_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2991smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2991smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3101smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 12) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3101smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0xe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3826smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3826smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2992smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2992smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2993smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2993smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2995smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2995smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3113smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 21) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3113smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x780) | (tie_t << 7);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3827smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3827smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2999smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2999smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3007smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3007smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3023smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3023smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3115smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3115smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2994smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2994smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2996smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2996smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3024smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3024smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3025smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3025smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3027smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3027smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3085smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3085smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2997smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2997smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3088smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3088smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3828smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3828smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld3026smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3026smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3003smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3003smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3002smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3002smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3004smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3004smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3005smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3005smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3034smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3034smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3107smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3107smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3109smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3109smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3114smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3114smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3808_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3808_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3108smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3108smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3035smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3035smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3110smod_slot2_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3110smod_slot2_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3118smod_slot1_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3118smod_slot1_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_op0_s16_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s16_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3829smod_slot1_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 16) >> 26);
  tie_t = (tie_t << 7) | ((insn[0] << 23) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3829smod_slot1_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc) | (tie_t << 2);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0xfc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3117smod_slot1_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3117smod_slot1_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2080_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2080_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_sae_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_op0_s17_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s17_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3120smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 13) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3120smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x7f000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3122smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3122smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3121smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3121smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3131smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3131smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3126smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3126smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3148smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 13) >> 25);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3148smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 21) >> 25;
  insn[0] = (insn[0] & ~0x7f000) | (tie_t << 12);
}

static unsigned
Field_imm6_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 20) >> 26);
  return tie_t;
}

static void
Field_imm6_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3164smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 11) | ((insn[0] << 13) >> 21);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3164smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 20) >> 21;
  insn[0] = (insn[0] & ~0x7ff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3832smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3832smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3160smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 13) | ((insn[0] << 13) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3160smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3161smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 11) | ((insn[0] << 13) >> 21);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3161smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 20) >> 21;
  insn[0] = (insn[0] & ~0x7ff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3159smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 13) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3159smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x7ff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3165smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3165smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3135smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3135smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3133smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3133smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3119smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 13) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3119smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3155smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3155smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3833smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3833smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3149smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3149smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3137smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3137smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3138smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3138smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3184smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3184smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3836smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3836smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3182smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3182smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_sas_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_sas_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3153smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3153smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3186smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3186smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3837smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 23) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3837smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3168smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3168smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3171smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3171smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3175smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3175smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3140smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3140smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3170smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3170smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3152smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3152smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3144smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3144smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3172smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3172smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3173smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3173smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3174smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3174smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3188smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 13) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3188smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3838smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3838smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3180smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3180smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3166smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3166smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 20) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3181smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3181smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 20) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_s8_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3136smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3136smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3123smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3123smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3134smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3134smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3125smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3125smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3127smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3127smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3128smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3128smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3130smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3130smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3129smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3129smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3132smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3132smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3139smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3139smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3141smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3141smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3142smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3142smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3143smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3143smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3145smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3145smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3146smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 13) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3146smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x7f800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3150_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3150_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3156smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3156smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3157smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3157smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3158smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3158smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3176smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3176smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3177smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3177smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3179smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3179smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3178smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3178smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 19) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3841smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3841smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3842smod_slot0_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3842smod_slot0_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3208llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 14) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3208llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 18) >> 27;
  insn[0] = (insn[0] & ~0x3e000) | (tie_t << 13);
}

static unsigned
Field_op0_s18_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s18_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3843llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3843llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3193llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3193llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3194llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3194llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3196llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3196llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3200llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3200llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3213llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 14) >> 31);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3213llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x20000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3845llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3845llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3195llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3195llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3206_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 20) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3206_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3225_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3225_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3205_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 14) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3205_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3847llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3847llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3212_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3212_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld3848llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3848llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3216llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3216llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 26) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3849llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3849llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3215llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3215llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3850llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3850llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3234llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3234llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3232llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3232llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3237llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 14) >> 25);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3237llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 21) >> 25;
  insn[0] = (insn[0] & ~0x3f800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2074_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2074_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3192llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3192llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3230_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3230_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld3191llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3191llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3243llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 14) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3243llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc00) | (tie_t << 10);
}

static unsigned
Field_t_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3218llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3218llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3851llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3851llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3244llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3244llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3853llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3853llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3242llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3242llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3855llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3855llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 26) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3231llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3231llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3856llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 18) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3856llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e00) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3203llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 16) | ((insn[0] << 14) >> 16);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3203llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 16) >> 16;
  insn[0] = (insn[0] & ~0x3fffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3204llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 14) >> 27);
  tie_t = (tie_t << 10) | ((insn[0] << 20) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3204llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc) | (tie_t << 2);
  tie_t = (val << 17) >> 27;
  insn[0] = (insn[0] & ~0x3e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3844_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3844_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3207llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 14) >> 27);
  tie_t = (tie_t << 10) | ((insn[0] << 20) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3207llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc) | (tie_t << 2);
  tie_t = (val << 17) >> 27;
  insn[0] = (insn[0] & ~0x3e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3197llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3197llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3198llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3198llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3199llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3199llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3201llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3201llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3202llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3202llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3210llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3210llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 22) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3857llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3857llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3217_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3217_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld3859llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 14) >> 24);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3859llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 23) >> 24;
  insn[0] = (insn[0] & ~0x3fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3221llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3221llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3222llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3222llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3860llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3860llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3224llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3224llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3861llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3861llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3220llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3220llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3246_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3246_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3214_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 14) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3214_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3863llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 15) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3863llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x1f800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3226llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3226llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3864llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3864llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3235llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 14) >> 25);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3235llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 21) >> 25;
  insn[0] = (insn[0] & ~0x3f800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3238llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 14) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3238llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x3f000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3862_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3862_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3241llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3241llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3865llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3865llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3236_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 14) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3236_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3240llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 14) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3240llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x3e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3228llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 23) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3228llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x100) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3866llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3866llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
}

static unsigned
Field_dsp340050b49a6c_fld3247llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3247llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3867llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3867llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_s_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3233_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3233_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3245llr_slot2_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3245llr_slot2_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3868_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3868_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3250llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3250llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_op0_s19_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s19_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3869llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3869llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 22) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
  tie_t = (val << 18) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3254llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3254llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3872llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3872llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
  tie_t = (val << 21) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3252llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3252llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3875llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3875llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3253llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3253llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3876llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 18) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3876llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e00) | (tie_t << 9);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3251llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3251llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3878llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3878llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3248llr_slot1_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3248llr_slot1_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3870_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3870_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2071_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2071_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_op0_s20_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s20_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3260llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3260llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3258llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3258llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3266llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3266llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3286llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3286llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3879llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3879llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3303llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3303llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3269llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3269llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3293llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3293llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3297llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3297llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3881llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3881llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3272llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3272llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3274llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3274llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3302llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3302llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3883llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3883llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3275llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3275llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3289llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3289llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3292llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3292llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3277llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3277llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3276llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3276llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3270llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3270llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3312llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3312llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3885llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3885llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3304llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3304llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3887llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3887llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3283llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3283llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3310llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3310llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3888llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3888llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3288llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3288llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3284llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3284llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3305llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3305llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
  tie_t = (val << 24) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3306llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3306llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3890llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3890llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3259llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 17) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3259llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x7000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3291llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3291llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3261llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3261llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3263llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3263llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3267llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3267llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3264llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3264llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3265llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3265llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3268llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3268llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3308llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3308llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3892llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3892llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3294llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3294llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3295llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3295llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3296llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3296llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3298llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3298llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3299llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3299llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3300llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 17) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3300llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x7800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3893llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3893llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3278llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3278llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3279llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3279llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3280llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3280llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3282llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3282llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3281llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3281llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 26) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3311llr_slot0_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3311llr_slot0_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_r_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3364_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3364_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_op0_s21_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s21_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
}

static unsigned
Field_dsp340050b49a6c_fld3464dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3464dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3894dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3894dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3465dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3465dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3895dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3895dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3459dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3459dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3896dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 11) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3896dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3468dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 9) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3468dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3897dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3897dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3467dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 9) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3467dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x400000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3899dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3899dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3414dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3414dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3313dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3313dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3316dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3316dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3334dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3334dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_t_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3314_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3314_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3418dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3418dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3425dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3425dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3408dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 9) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3408dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x400000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3901dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3901dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3315dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3315dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3421dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3421dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3443dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3443dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3420dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3420dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3423dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3423dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3422dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3422dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3424dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3424dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3416dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3416dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3461dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3461dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3415dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3415dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3426dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3426dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3417dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3417dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3419dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3419dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3318_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3318_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3904dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3904dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3482dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3482dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3903dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 11) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3903dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x180000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 8) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0xf80000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3407_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3407_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3905dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 9) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3905dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3479dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3479dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 23) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3906dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3906dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3475dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3475dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 20) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3898dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3898dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3353dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 8) >> 25);
  tie_t = (tie_t << 13) | ((insn[0] << 17) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3353dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc) | (tie_t << 2);
  tie_t = (val << 12) >> 25;
  insn[0] = (insn[0] & ~0xfe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3907dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3907dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3388dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 13) | ((insn[0] << 8) >> 19);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3388dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 15) >> 19;
  insn[0] = (insn[0] & ~0xfff800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3913dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3913dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3387dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 18) | ((insn[0] << 8) >> 14);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3387dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 14) >> 14;
  insn[0] = (insn[0] & ~0xffffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3396dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 8) >> 25);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3396dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 21) >> 25;
  insn[0] = (insn[0] & ~0xfe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3909dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 15) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3909dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3404dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 12) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3404dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x80000) | (tie_t << 19);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3910dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 10) >> 30);
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3910dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
  tie_t = (val << 21) >> 30;
  insn[0] = (insn[0] & ~0x300000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3385dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3385dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3478dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3478dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 21) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3908dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3908dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3477dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3477dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 20) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3469dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 8) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3469dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0xff8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3484dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 13) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3484dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40000) | (tie_t << 18);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3916dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 11) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3916dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x180000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3451dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3451dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 21) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3914dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3914dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3450dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3450dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3453dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 8) >> 18);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3453dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 18) >> 18;
  insn[0] = (insn[0] & ~0xfffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3365dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3365dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3366dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3366dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3368dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3368dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3372dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3372dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3367dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3367dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3369dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3369dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3345dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 12) | ((insn[0] << 18) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3345dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x3ffc) | (tie_t << 2);
  tie_t = (val << 14) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3457dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 8) >> 25);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3457dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 21) >> 25;
  insn[0] = (insn[0] & ~0xfe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3917dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3917dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3358dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 6) | ((insn[0] << 22) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3358dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x3f0) | (tie_t << 4);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3918dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3918dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 27) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3347dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 12) | ((insn[0] << 18) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3347dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x3ffc) | (tie_t << 2);
  tie_t = (val << 14) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3348dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 12) | ((insn[0] << 18) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3348dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x3ffc) | (tie_t << 2);
  tie_t = (val << 14) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3354dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 10) | ((insn[0] << 18) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3354dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x3ff0) | (tie_t << 4);
  tie_t = (val << 16) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_s4_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3397dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3397dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3919dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3919dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3462dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3462dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3394dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 8) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3394dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0xff0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3920dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 16) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3920dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3363dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3363dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3481dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3481dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3339dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3339dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3384dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3384dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3360dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3360dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3437dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3437dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3439dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3439dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3438dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3438dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3440dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3440dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3441dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3441dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3442dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3442dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3444dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3444dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3460dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3460dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3921dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3921dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3429dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3429dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3431dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3431dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3435dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3435dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3430dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3430dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3432dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3432dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3433dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3433dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3434dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3434dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3436dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3436dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3361dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3361dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3403dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3403dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3922dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 10) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3922dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x300000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3386dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3386dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 18) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3923dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3923dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 18) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3324dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3324dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3325dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3325dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3340dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3340dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3327dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3327dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3342dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  tie_t = (tie_t << 6) | ((insn[0] << 22) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3342dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x3f0) | (tie_t << 4);
  tie_t = (val << 24) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3356dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  tie_t = (tie_t << 6) | ((insn[0] << 22) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3356dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x3f0) | (tie_t << 4);
  tie_t = (val << 23) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
  tie_t = (val << 17) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3924dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3924dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3349dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 22) | ((insn[0] << 8) >> 10);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3349dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 10) >> 10;
  insn[0] = (insn[0] & ~0xfffffc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3350dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 8) >> 25);
  tie_t = (tie_t << 13) | ((insn[0] << 17) >> 19);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3350dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 19) >> 19;
  insn[0] = (insn[0] & ~0x7ffc) | (tie_t << 2);
  tie_t = (val << 12) >> 25;
  insn[0] = (insn[0] & ~0xfe0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3341dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3341dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3370dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3370dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3371dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3371dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3373dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3373dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3374dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3374dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3376dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3376dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3375dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3375dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3377dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3377dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3378dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3378dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3379dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 8) >> 22);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3379dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 14) >> 22;
  insn[0] = (insn[0] & ~0xffc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3381dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3381dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3448dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3448dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3925dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3925dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3454dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 8) >> 24);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3454dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0xff0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3466_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3466_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3412dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3412dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3927dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3927dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3382dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 7) | ((insn[0] << 22) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3382dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8) | (tie_t << 3);
  tie_t = (val << 19) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3928dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3928dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3410_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3410_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3929dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3929dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
  tie_t = (val << 17) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3390dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 8) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3390dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0xfff000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3413dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3413dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3930dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 13) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3930dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x7fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3456dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 8) >> 24);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3456dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0xff0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3392dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 11) | ((insn[0] << 8) >> 21);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3392dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 17) >> 21;
  insn[0] = (insn[0] & ~0xffe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3900_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3900_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3337dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3337dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3446dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3446dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 20) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3317dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3317dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3458dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3458dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3319dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3319dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3320dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3320dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3322dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3322dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3399dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3399dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3931dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 11) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3931dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x1c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3326dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3326dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3427dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3427dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3470dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3470dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3411dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 9) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3411dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x400000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3933dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3933dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
  tie_t = (val << 23) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3321dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3321dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3428dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3428dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_s_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_r_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3471dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3471dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3472dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3472dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3474dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3474dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3323dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3323dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3480dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3480dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3473dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3473dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3328dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3328dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3330dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3330dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3329dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3329dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3331dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3331dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3362dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3362dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3332dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3332dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3445dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3445dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3333dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3333dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3335dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3335dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3401dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 8) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 12) >> 30);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3401dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 27) >> 30;
  insn[0] = (insn[0] & ~0xc0000) | (tie_t << 18);
  tie_t = (val << 25) >> 30;
  insn[0] = (insn[0] & ~0xc00000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3934dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 10) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3934dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x300000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3406dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 8) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3406dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x800000) | (tie_t << 23);
}

static unsigned
Field_dsp340050b49a6c_fld3935dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 9) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3935dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x7c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3380dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3380dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 18) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3336dual_slot2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 8) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3336dual_slot2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0xfc0000) | (tie_t << 18);
}

static unsigned
Field_op0_s22_Slot_dual_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 31) >> 31);
  return tie_t;
}

static void
Field_op0_s22_Slot_dual_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1) | (tie_t << 0);
}

static unsigned
Field_imm8_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  return tie_t;
}

static void
Field_imm8_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_op0_s23_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 9) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 30) >> 30);
  return tie_t;
}

static void
Field_op0_s23_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3) | (tie_t << 0);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x400000) | (tie_t << 22);
}

static unsigned
Field_dsp340050b49a6c_fld3532dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3532dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 24) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3533dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 10) >> 27);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3533dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 25) >> 27;
  insn[0] = (insn[0] & ~0x3e0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3936dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3936dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2057_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2057_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3625dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3625dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_r_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_t_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3487dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 18) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3487dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x3800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld3584_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3584_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3937dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3937dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3489dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3489dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3488dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 10) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3488dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3588dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 10) >> 27);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3588dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x3e0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3938dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3938dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3544dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3544dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3519dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3519dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 18) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3939dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3939dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3523dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 11) | ((insn[0] << 10) >> 21);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3523dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 21) >> 21;
  insn[0] = (insn[0] & ~0x3ff800) | (tie_t << 11);
}

static unsigned
Field_bbi_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 21) >> 27);
  return tie_t;
}

static void
Field_bbi_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3520dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3520dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3507dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 16) | ((insn[0] << 10) >> 16);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3507dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 16) >> 16;
  insn[0] = (insn[0] & ~0x3fffc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld3506dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 15) | ((insn[0] << 10) >> 17);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3506dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 17) >> 17;
  insn[0] = (insn[0] & ~0x3fff80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3614dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3614dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2079_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2079_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3550dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3550dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3549dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3549dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3531_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 10) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3531_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x3f8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3500dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3500dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 18) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3502dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3502dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 17) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3606dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3606dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3607dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3607dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3608dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3608dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_sae_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3619_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 10) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3619_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x200000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3940dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 11) >> 25);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3940dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3613dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 10) >> 27);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3613dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 23) >> 27;
  insn[0] = (insn[0] & ~0x3e0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3509dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3509dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3510dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3510dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3562dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3562dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3560dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3560dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3499dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3499dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3563dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3563dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3565dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3565dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3564dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3564dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3566dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3566dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3512dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3512dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3567dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3567dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3504dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3504dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 17) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3516dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3516dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3568dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3568dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3570dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3570dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3580dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3580dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3559dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3559dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3573dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3573dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3599dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3599dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3941dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3941dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3575dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3575dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3535dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3535dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2066_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2066_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3494dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3494dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3497dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3497dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3496dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3496dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3498dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 5) | ((insn[0] << 17) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3498dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c00) | (tie_t << 10);
  tie_t = (val << 23) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3490dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3490dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3491dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3491dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3493dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3493dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3527dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3527dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 26) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3492dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3492dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3589dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3589dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_s8_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3541dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3541dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2037_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2037_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld3542dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3542dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3543dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3543dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3552_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3552_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 23) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3626dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3626dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3621dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 10) >> 25);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3621dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 24) >> 25;
  insn[0] = (insn[0] & ~0x3f8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3623dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 10) >> 25);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3623dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 24) >> 25;
  insn[0] = (insn[0] & ~0x3f8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3622dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 10) >> 25);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3622dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 24) >> 25;
  insn[0] = (insn[0] & ~0x3f8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3624dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 10) >> 25);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3624dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 24) >> 25;
  insn[0] = (insn[0] & ~0x3f8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3529dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 10) | ((insn[0] << 10) >> 22);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3529dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 22) >> 22;
  insn[0] = (insn[0] & ~0x3ff000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld3530dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 10) >> 25);
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3530dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
  tie_t = (val << 23) >> 25;
  insn[0] = (insn[0] & ~0x3f8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2072_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2072_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3524dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 17) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3524dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x7800) | (tie_t << 11);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3943dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 6) | ((insn[0] << 21) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3943dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e0) | (tie_t << 5);
  tie_t = (val << 25) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3508dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3508dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3601dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3601dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3945dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3945dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3603dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3603dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3604dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3604dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3946dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 14) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3946dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x30000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3545dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3545dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3616dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 10) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3616dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x300000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld3947dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 12) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3947dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 25) >> 26;
  insn[0] = (insn[0] & ~0xfc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3618dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 10) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3618dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld3949dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 11) >> 25);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3949dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 24) >> 25;
  insn[0] = (insn[0] & ~0x1fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3609dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3609dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3546dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3546dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3548dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3548dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3554dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3554dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3547dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3547dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3522dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3522dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 19) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3950dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3950dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld3611dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3611dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3592dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3592dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3594dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3594dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3598dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3598dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3951dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3951dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3600dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 10) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3600dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 24) >> 27;
  insn[0] = (insn[0] & ~0x3e0000) | (tie_t << 17);
}

static unsigned
Field_dsp340050b49a6c_fld3952dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 15) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3952dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x18000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3593dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3593dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3595dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3595dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3596dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3596dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3597dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3597dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3511dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3511dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3513dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3513dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3514dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3514dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3515dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3515dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3517dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3517dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3518dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3518dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 17) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld3536dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3536dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld2060_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2060_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3539dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3539dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3954dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 14) >> 27);
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3954dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
  tie_t = (val << 26) >> 27;
  insn[0] = (insn[0] & ~0x3e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3537dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3537dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3538dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 10) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3538dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0000) | (tie_t << 18);
}

static unsigned
Field_dsp340050b49a6c_fld3957dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3957dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld3587dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3587dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 21) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3610_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 22) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3610_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x3e0) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld3571dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3571dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3572dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3572dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3574dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3574dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3569dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3569dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3505dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 14) | ((insn[0] << 10) >> 18);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3505dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 18) >> 18;
  insn[0] = (insn[0] & ~0x3fff00) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld3612dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 10) >> 26);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3612dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 22) >> 26;
  insn[0] = (insn[0] & ~0x3f0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld3576dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3576dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 15) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3577dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3577dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 15) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3615dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 10) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3615dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x380000) | (tie_t << 19);
}

static unsigned
Field_dsp340050b49a6c_fld3958dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 13) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3958dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x7e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3551dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3551dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3553dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3553dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3590dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3590dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3591dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3591dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
  tie_t = (val << 20) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3555dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3555dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3557dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3557dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3578dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3578dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 15) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3579dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3579dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 15) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3581dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 8) | ((insn[0] << 22) >> 24);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3581dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc) | (tie_t << 2);
  tie_t = (val << 16) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3582dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3582dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 25) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 17) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3959dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3959dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3556dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3556dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3558dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 9) | ((insn[0] << 10) >> 23);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3558dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 19) >> 23;
  insn[0] = (insn[0] & ~0x3fe000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3583dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3583dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
  tie_t = (val << 18) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3960dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3960dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
  tie_t = (val << 29) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld3585dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  tie_t = (tie_t << 1) | ((insn[0] << 22) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3585dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x200) | (tie_t << 9);
  tie_t = (val << 19) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld3961dual_slot0_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 18) >> 31);
  tie_t = (tie_t << 2) | ((insn[0] << 23) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld3961dual_slot0_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0x180) | (tie_t << 7);
  tie_t = (val << 28) >> 31;
  insn[0] = (insn[0] & ~0x2000) | (tie_t << 13);
}

static unsigned
Field_t_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_t_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_t_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_t_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_t_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_t_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_t_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_t_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_t_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_bbi_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_bbi_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_bbi_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 21) >> 27);
  return tie_t;
}

static void
Field_bbi_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c0) | (tie_t << 6);
}

static unsigned
Field_imm12_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 8) >> 20);
  return tie_t;
}

static void
Field_imm12_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0xfff000) | (tie_t << 12);
}

static unsigned
Field_imm12_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 14) >> 20);
  return tie_t;
}

static void
Field_imm12_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x3ffc0) | (tie_t << 6);
}

static unsigned
Field_imm12_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 14) >> 20);
  return tie_t;
}

static void
Field_imm12_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 20) >> 20;
  insn[0] = (insn[0] & ~0x3ffc0) | (tie_t << 6);
}

static unsigned
Field_imm8_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 8) >> 24);
  return tie_t;
}

static void
Field_imm8_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0000) | (tie_t << 16);
}

static unsigned
Field_imm8_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 10) >> 24);
  return tie_t;
}

static void
Field_imm8_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc000) | (tie_t << 14);
}

static unsigned
Field_s_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_s_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_s_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_s_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_s_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_imm12b_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  tie_t = (tie_t << 8) | ((insn[0] << 8) >> 24);
  return tie_t;
}

static void
Field_imm12b_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0000) | (tie_t << 16);
  tie_t = (val << 20) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_imm12b_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 14) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_imm12b_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc00) | (tie_t << 10);
}

static unsigned
Field_imm12b_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 14) >> 24);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_imm12b_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 20) >> 24;
  insn[0] = (insn[0] & ~0x3fc00) | (tie_t << 10);
}

static unsigned
Field_imm16_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 16) | ((insn[0] << 8) >> 16);
  return tie_t;
}

static void
Field_imm16_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 16) >> 16;
  insn[0] = (insn[0] & ~0xffff00) | (tie_t << 8);
}

static unsigned
Field_imm16_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 12) | ((insn[0] << 10) >> 20);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_imm16_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 16) >> 20;
  insn[0] = (insn[0] & ~0x3ffc00) | (tie_t << 10);
}

static unsigned
Field_offset_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 18) | ((insn[0] << 8) >> 14);
  return tie_t;
}

static void
Field_offset_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 14) >> 14;
  insn[0] = (insn[0] & ~0xffffc0) | (tie_t << 6);
}

static unsigned
Field_op2_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_op2_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_op2_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_op2_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_op2_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_op2_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_op2_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_op2_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_r_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_r_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_r_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_r_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_r_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_r_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_r_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_r_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_r_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_r_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_r_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_r_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_r_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_r_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_r_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_r_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_sae_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 15) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_sae_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x10000) | (tie_t << 16);
}

static unsigned
Field_sae_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_sae_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_sal_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sal_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_sal_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_sargt_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sargt_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_sargt_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_sas4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_sas4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
}

static unsigned
Field_sas_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_sas_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
}

static unsigned
Field_sas_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_sas_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_sas_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_sas_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_sas_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_sas_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_sas_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_sas_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_sr_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_sr_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_sr_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  return tie_t;
}

static void
Field_sr_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_st_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_st_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_st_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 20) >> 28);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_st_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 24) >> 28;
  insn[0] = (insn[0] & ~0xf00) | (tie_t << 8);
}

static unsigned
Field_imm4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_imm4_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm4_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_imm4_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm4_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_mn_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_mn_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
  tie_t = (val << 28) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_i_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_i_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_imm6lo_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm6lo_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_imm6lo_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm6lo_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_imm6hi_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_imm6hi_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_imm6hi_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  return tie_t;
}

static void
Field_imm6hi_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_imm7lo_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm7lo_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_imm7lo_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm7lo_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_imm7hi_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  return tie_t;
}

static void
Field_imm7hi_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
}

static unsigned
Field_imm7hi_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  return tie_t;
}

static void
Field_imm7hi_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
}

static unsigned
Field_z_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 25) >> 31);
  return tie_t;
}

static void
Field_z_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x40) | (tie_t << 6);
}

static unsigned
Field_imm6_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm6_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_imm6_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 26) >> 30);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm6_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 26) >> 30;
  insn[0] = (insn[0] & ~0x30) | (tie_t << 4);
}

static unsigned
Field_imm6_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 20) >> 26);
  return tie_t;
}

static void
Field_imm6_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc0) | (tie_t << 6);
}

static unsigned
Field_imm6_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 20) >> 26);
  return tie_t;
}

static void
Field_imm6_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc0) | (tie_t << 6);
}

static unsigned
Field_imm6_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 20) >> 26);
  return tie_t;
}

static void
Field_imm6_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc0) | (tie_t << 6);
}

static unsigned
Field_imm6_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 20) >> 26);
  return tie_t;
}

static void
Field_imm6_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm7_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
}

static unsigned
Field_imm7_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_imm7_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
  tie_t = (val << 25) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
}

static unsigned
Field_imm7_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_imm7_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 7) | ((insn[0] << 19) >> 25);
  return tie_t;
}

static void
Field_imm7_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 25) >> 25;
  insn[0] = (insn[0] & ~0x1fc0) | (tie_t << 6);
}

static unsigned
Field_t2_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_t2_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
}

static unsigned
Field_t2_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_t2_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
}

static unsigned
Field_t2_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 24) >> 29);
  return tie_t;
}

static void
Field_t2_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe0) | (tie_t << 5);
}

static unsigned
Field_s2_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_s2_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
}

static unsigned
Field_s2_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_s2_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
}

static unsigned
Field_s2_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 20) >> 29);
  return tie_t;
}

static void
Field_s2_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00) | (tie_t << 9);
}

static unsigned
Field_r2_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_r2_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_r2_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_r2_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static unsigned
Field_t4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_t4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_t4_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_t4_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_t4_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_t4_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_s4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_s4_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_s4_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_s4_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_s4_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_s4_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_s4_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_s4_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_r4_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_r4_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_r4_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_r4_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_r4_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  return tie_t;
}

static void
Field_r4_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_t8_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_t8_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_t8_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_t8_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_t8_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_t8_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_s8_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_s8_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_s8_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_s8_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_s8_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_r8_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_r8_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_r8_Slot_inst16a_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_r8_Slot_inst16a_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_r8_Slot_inst16b_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_r8_Slot_inst16b_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
}

static unsigned
Field_xt_wbr15_imm_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 15) | ((insn[0] << 8) >> 17);
  return tie_t;
}

static void
Field_xt_wbr15_imm_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 17) >> 17;
  insn[0] = (insn[0] & ~0xfffe00) | (tie_t << 9);
}

static unsigned
Field_xt_wbr18_imm_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 18) | ((insn[0] << 8) >> 14);
  return tie_t;
}

static void
Field_xt_wbr18_imm_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 14) >> 14;
  insn[0] = (insn[0] & ~0xffffc0) | (tie_t << 6);
}

static unsigned
Field_fimm8_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 8) >> 24);
  return tie_t;
}

static void
Field_fimm8_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0xff0000) | (tie_t << 16);
}

static unsigned
Field_fimm8_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 8) | ((insn[0] << 14) >> 24);
  return tie_t;
}

static void
Field_fimm8_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 24) >> 24;
  insn[0] = (insn[0] & ~0x3fc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2029_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2029_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2030_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2030_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 22) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x300) | (tie_t << 8);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2032_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2032_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2035_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2035_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2035_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2035_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2035_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2035_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2035_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 27) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2035_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x18) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2036_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2036_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2036_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2036_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2036_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2036_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2037_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2037_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2037_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2037_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2037_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2037_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2037_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2037_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2038_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 14) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2038_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x38000) | (tie_t << 15);
}

static unsigned
Field_dsp340050b49a6c_fld2039_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2039_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2039_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2039_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2039_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2039_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2039_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2039_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2039_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2039_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2040_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2040_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2040_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2040_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2040_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2040_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2040_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2040_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2040_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 19) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2040_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x1000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 27) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x10) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2041_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 24) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2041_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x80) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2042_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 1) | ((insn[0] << 16) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2042_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8000) | (tie_t << 15);
  tie_t = (val << 30) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2042_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2042_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2042_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2042_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2042_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 17) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2042_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x6000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 11) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x100000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2043_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2043_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 23) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x1f0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 25) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2044_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 21) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2044_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x400) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2045_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2045_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2046_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2046_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2047_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2047_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_dot_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_dot_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 8) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe00000) | (tie_t << 21);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2048_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2048_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 18) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x3000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 19) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0x1800) | (tie_t << 11);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 28) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 24) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2049_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 20) >> 30);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2049_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 30) >> 30;
  insn[0] = (insn[0] & ~0xc00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_dot_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 15) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_dot_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x1e000) | (tie_t << 13);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2050_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 25) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2050_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x78) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 22) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0x3f0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2051_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 6) | ((insn[0] << 24) >> 26);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2051_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 26) >> 26;
  insn[0] = (insn[0] & ~0xfc) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 16) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf000) | (tie_t << 12);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  tie_t = (tie_t << 1) | ((insn[0] << 29) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x4) | (tie_t << 2);
  tie_t = (val << 28) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2052_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2052_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 8) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 20) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x800) | (tie_t << 11);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0xf00000) | (tie_t << 20);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 17) >> 31);
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
  tie_t = (val << 27) >> 31;
  insn[0] = (insn[0] & ~0x4000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 5) | ((insn[0] << 13) >> 27);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 27) >> 27;
  insn[0] = (insn[0] & ~0x7c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2053_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 14) >> 28);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2053_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 27) >> 28;
  insn[0] = (insn[0] & ~0x3c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 2) | ((insn[0] << 16) >> 30);
  tie_t = (tie_t << 1) | ((insn[0] << 26) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x20) | (tie_t << 5);
  tie_t = (val << 29) >> 30;
  insn[0] = (insn[0] & ~0xc000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_llr_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_llr_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2054_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2054_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 24) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_acc2_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_acc2_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 12) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0xf0000) | (tie_t << 16);
}

static unsigned
Field_dsp340050b49a6c_fld2055_Slot_dual_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2055_Slot_dual_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_inst_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 25) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_inst_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x70) | (tie_t << 4);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_gp_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_gp_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 26) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x38) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_dot_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_dot_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_pq_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_pq_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_smod_slot0_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_smod_slot0_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 22) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x380) | (tie_t << 7);
}

static unsigned
Field_dsp340050b49a6c_fld2056_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2056_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2025_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2025_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2025_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2025_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2025_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2025_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2025_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2025_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2025_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 1) | ((insn[0] << 28) >> 31);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2025_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 31) >> 31;
  insn[0] = (insn[0] & ~0x8) | (tie_t << 3);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_gp_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_gp_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_dot_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_dot_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_pq_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_pq_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_acc2_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_acc2_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_smod_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_smod_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_llr_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_llr_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2027_Slot_dual_slot2_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2027_Slot_dual_slot2_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2026_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2026_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2026_Slot_dot_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2026_Slot_dot_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2026_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2026_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2026_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 18) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2026_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2026_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2026_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2026_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2026_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2031_Slot_gp_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2031_Slot_gp_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2031_Slot_dot_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2031_Slot_dot_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2031_Slot_pq_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2031_Slot_pq_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2031_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 26) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2031_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2031_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 4) | ((insn[0] << 22) >> 28);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2031_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 28) >> 28;
  insn[0] = (insn[0] & ~0x3c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2028_Slot_acc2_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 15) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2028_Slot_acc2_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c000) | (tie_t << 14);
}

static unsigned
Field_dsp340050b49a6c_fld2028_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 27) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2028_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c) | (tie_t << 2);
}

static unsigned
Field_dsp340050b49a6c_fld2033_Slot_smod_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 23) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2033_Slot_smod_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c0) | (tie_t << 6);
}

static unsigned
Field_dsp340050b49a6c_fld2033_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 19) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2033_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0x1c00) | (tie_t << 10);
}

static unsigned
Field_dsp340050b49a6c_fld2034_Slot_llr_slot1_get (const xtensa_insnbuf insn)
{
  unsigned tie_t = 0;
  tie_t = (tie_t << 3) | ((insn[0] << 16) >> 29);
  return tie_t;
}

static void
Field_dsp340050b49a6c_fld2034_Slot_llr_slot1_set (xtensa_insnbuf insn, uint32 val)
{
  uint32 tie_t;
  tie_t = (val << 29) >> 29;
  insn[0] = (insn[0] & ~0xe000) | (tie_t << 13);
}

static void
Implicit_Field_set (xtensa_insnbuf insn ATTRIBUTE_UNUSED,
		    uint32 val ATTRIBUTE_UNUSED)
{
  /* Do nothing.  */
}

static unsigned
Implicit_Field_ar0_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 0;
}

static unsigned
Implicit_Field_ar4_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 4;
}

static unsigned
Implicit_Field_ar8_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 8;
}

static unsigned
Implicit_Field_ar12_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 12;
}

static unsigned
Implicit_Field_bt16_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 0;
}

static unsigned
Implicit_Field_bs16_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 0;
}

static unsigned
Implicit_Field_br16_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 0;
}

static unsigned
Implicit_Field_brall_get (const xtensa_insnbuf insn ATTRIBUTE_UNUSED)
{
  return 0;
}

enum xtensa_field_id {
  FIELD_t,
  FIELD_bbi4,
  FIELD_bbi,
  FIELD_imm12,
  FIELD_imm8,
  FIELD_s,
  FIELD_imm12b,
  FIELD_imm16,
  FIELD_m,
  FIELD_n,
  FIELD_offset,
  FIELD_op0,
  FIELD_op1,
  FIELD_op2,
  FIELD_r,
  FIELD_sa4,
  FIELD_sae4,
  FIELD_sae,
  FIELD_sal,
  FIELD_sargt,
  FIELD_sas4,
  FIELD_sas,
  FIELD_sr,
  FIELD_st,
  FIELD_thi3,
  FIELD_imm4,
  FIELD_mn,
  FIELD_i,
  FIELD_imm6lo,
  FIELD_imm6hi,
  FIELD_imm7lo,
  FIELD_imm7hi,
  FIELD_z,
  FIELD_imm6,
  FIELD_imm7,
  FIELD_t2,
  FIELD_s2,
  FIELD_r2,
  FIELD_t4,
  FIELD_s4,
  FIELD_r4,
  FIELD_t8,
  FIELD_s8,
  FIELD_r8,
  FIELD_xt_wbr15_imm,
  FIELD_xt_wbr18_imm,
  FIELD_fimm8,
  FIELD_dsp340050b49a6c_fld2019,
  FIELD_dsp340050b49a6c_fld2021,
  FIELD_dsp340050b49a6c_fld2029,
  FIELD_dsp340050b49a6c_fld2030,
  FIELD_dsp340050b49a6c_fld2032,
  FIELD_dsp340050b49a6c_fld2035,
  FIELD_dsp340050b49a6c_fld2036,
  FIELD_dsp340050b49a6c_fld2037,
  FIELD_dsp340050b49a6c_fld2038,
  FIELD_dsp340050b49a6c_fld2039,
  FIELD_dsp340050b49a6c_fld2040,
  FIELD_dsp340050b49a6c_fld2041,
  FIELD_dsp340050b49a6c_fld2042,
  FIELD_dsp340050b49a6c_fld2043,
  FIELD_dsp340050b49a6c_fld2044,
  FIELD_dsp340050b49a6c_fld2045,
  FIELD_dsp340050b49a6c_fld2046,
  FIELD_dsp340050b49a6c_fld2047,
  FIELD_dsp340050b49a6c_fld2048,
  FIELD_dsp340050b49a6c_fld2049,
  FIELD_dsp340050b49a6c_fld2050,
  FIELD_dsp340050b49a6c_fld2051,
  FIELD_dsp340050b49a6c_fld2052,
  FIELD_dsp340050b49a6c_fld2053,
  FIELD_dsp340050b49a6c_fld2054,
  FIELD_dsp340050b49a6c_fld2055,
  FIELD_dsp340050b49a6c_fld2056,
  FIELD_dsp340050b49a6c_fld2082Inst,
  FIELD_dsp340050b49a6c_fld2083Inst,
  FIELD_dsp340050b49a6c_fld2084Inst,
  FIELD_dsp340050b49a6c_fld2085Inst,
  FIELD_dsp340050b49a6c_fld2086Inst,
  FIELD_dsp340050b49a6c_fld2088Inst,
  FIELD_dsp340050b49a6c_fld2089Inst,
  FIELD_dsp340050b49a6c_fld2090Inst,
  FIELD_dsp340050b49a6c_fld2091Inst,
  FIELD_dsp340050b49a6c_fld2092Inst,
  FIELD_dsp340050b49a6c_fld2094Inst,
  FIELD_dsp340050b49a6c_fld2095Inst,
  FIELD_dsp340050b49a6c_fld2096Inst,
  FIELD_dsp340050b49a6c_fld2098Inst,
  FIELD_dsp340050b49a6c_fld2099Inst,
  FIELD_dsp340050b49a6c_fld2100Inst,
  FIELD_dsp340050b49a6c_fld2101Inst,
  FIELD_dsp340050b49a6c_fld2102Inst,
  FIELD_dsp340050b49a6c_fld2103Inst,
  FIELD_dsp340050b49a6c_fld2104Inst,
  FIELD_dsp340050b49a6c_fld2105Inst,
  FIELD_dsp340050b49a6c_fld2106Inst,
  FIELD_dsp340050b49a6c_fld2107Inst,
  FIELD_dsp340050b49a6c_fld2108Inst,
  FIELD_dsp340050b49a6c_fld2109Inst,
  FIELD_dsp340050b49a6c_fld2110Inst,
  FIELD_dsp340050b49a6c_fld2111Inst,
  FIELD_dsp340050b49a6c_fld2112Inst,
  FIELD_dsp340050b49a6c_fld2113Inst,
  FIELD_dsp340050b49a6c_fld2114Inst,
  FIELD_dsp340050b49a6c_fld2115Inst,
  FIELD_dsp340050b49a6c_fld2116Inst,
  FIELD_dsp340050b49a6c_fld2117Inst,
  FIELD_dsp340050b49a6c_fld2118Inst,
  FIELD_dsp340050b49a6c_fld2119Inst,
  FIELD_dsp340050b49a6c_fld2120Inst,
  FIELD_dsp340050b49a6c_fld2122Inst,
  FIELD_dsp340050b49a6c_fld2123Inst,
  FIELD_dsp340050b49a6c_fld2124Inst,
  FIELD_dsp340050b49a6c_fld2125Inst,
  FIELD_dsp340050b49a6c_fld2126Inst,
  FIELD_dsp340050b49a6c_fld2127Inst,
  FIELD_dsp340050b49a6c_fld2128Inst,
  FIELD_dsp340050b49a6c_fld2129Inst,
  FIELD_dsp340050b49a6c_fld2131Inst,
  FIELD_dsp340050b49a6c_fld2132Inst,
  FIELD_dsp340050b49a6c_fld2133Inst,
  FIELD_dsp340050b49a6c_fld2134Inst,
  FIELD_dsp340050b49a6c_fld2136Inst,
  FIELD_dsp340050b49a6c_fld2137Inst,
  FIELD_dsp340050b49a6c_fld2138Inst,
  FIELD_dsp340050b49a6c_fld2139Inst,
  FIELD_dsp340050b49a6c_fld2140Inst,
  FIELD_dsp340050b49a6c_fld2141Inst,
  FIELD_dsp340050b49a6c_fld2142Inst,
  FIELD_dsp340050b49a6c_fld2143Inst,
  FIELD_dsp340050b49a6c_fld2144Inst,
  FIELD_dsp340050b49a6c_fld2145Inst,
  FIELD_dsp340050b49a6c_fld2146Inst,
  FIELD_dsp340050b49a6c_fld2147Inst,
  FIELD_dsp340050b49a6c_fld2149Inst,
  FIELD_dsp340050b49a6c_fld2151Inst,
  FIELD_dsp340050b49a6c_fld2153Inst,
  FIELD_dsp340050b49a6c_fld2154Inst,
  FIELD_dsp340050b49a6c_fld2155Inst,
  FIELD_dsp340050b49a6c_fld2156Inst,
  FIELD_dsp340050b49a6c_fld2157Inst,
  FIELD_dsp340050b49a6c_fld2158Inst,
  FIELD_dsp340050b49a6c_fld2159Inst,
  FIELD_dsp340050b49a6c_fld2160Inst,
  FIELD_dsp340050b49a6c_fld2161Inst,
  FIELD_dsp340050b49a6c_fld2162Inst,
  FIELD_dsp340050b49a6c_fld2163Inst,
  FIELD_dsp340050b49a6c_fld2164Inst,
  FIELD_dsp340050b49a6c_fld2165Inst,
  FIELD_dsp340050b49a6c_fld2166Inst,
  FIELD_dsp340050b49a6c_fld2167Inst,
  FIELD_dsp340050b49a6c_fld2168Inst,
  FIELD_dsp340050b49a6c_fld2169Inst,
  FIELD_dsp340050b49a6c_fld2171Inst,
  FIELD_dsp340050b49a6c_fld2172Inst,
  FIELD_dsp340050b49a6c_fld2173Inst,
  FIELD_dsp340050b49a6c_fld2174Inst,
  FIELD_dsp340050b49a6c_fld2175Inst,
  FIELD_dsp340050b49a6c_fld2177Inst,
  FIELD_dsp340050b49a6c_fld2178Inst,
  FIELD_dsp340050b49a6c_fld2179Inst,
  FIELD_dsp340050b49a6c_fld2180Inst,
  FIELD_dsp340050b49a6c_fld2181Inst,
  FIELD_dsp340050b49a6c_fld2182Inst,
  FIELD_dsp340050b49a6c_fld2183Inst,
  FIELD_dsp340050b49a6c_fld2184Inst,
  FIELD_dsp340050b49a6c_fld2185Inst,
  FIELD_dsp340050b49a6c_fld2186Inst,
  FIELD_dsp340050b49a6c_fld2187Inst,
  FIELD_dsp340050b49a6c_fld2188Inst,
  FIELD_dsp340050b49a6c_fld2189Inst,
  FIELD_dsp340050b49a6c_fld2190Inst,
  FIELD_dsp340050b49a6c_fld2191Inst,
  FIELD_dsp340050b49a6c_fld2192Inst,
  FIELD_dsp340050b49a6c_fld2193Inst,
  FIELD_dsp340050b49a6c_fld2194Inst,
  FIELD_dsp340050b49a6c_fld2195Inst,
  FIELD_dsp340050b49a6c_fld2196Inst,
  FIELD_dsp340050b49a6c_fld2197Inst,
  FIELD_dsp340050b49a6c_fld2198Inst,
  FIELD_dsp340050b49a6c_fld2199Inst,
  FIELD_dsp340050b49a6c_fld2200Inst,
  FIELD_dsp340050b49a6c_fld2201Inst,
  FIELD_dsp340050b49a6c_fld2202Inst,
  FIELD_dsp340050b49a6c_fld2203Inst,
  FIELD_dsp340050b49a6c_fld2204Inst,
  FIELD_dsp340050b49a6c_fld2205Inst,
  FIELD_dsp340050b49a6c_fld2206Inst,
  FIELD_dsp340050b49a6c_fld2207Inst,
  FIELD_dsp340050b49a6c_fld2208Inst,
  FIELD_dsp340050b49a6c_fld2209Inst,
  FIELD_dsp340050b49a6c_fld2210Inst,
  FIELD_dsp340050b49a6c_fld2211Inst,
  FIELD_dsp340050b49a6c_fld2212Inst,
  FIELD_dsp340050b49a6c_fld2213Inst,
  FIELD_dsp340050b49a6c_fld2214Inst,
  FIELD_dsp340050b49a6c_fld2215Inst,
  FIELD_dsp340050b49a6c_fld2216Inst,
  FIELD_dsp340050b49a6c_fld2217Inst,
  FIELD_dsp340050b49a6c_fld2218Inst,
  FIELD_dsp340050b49a6c_fld2219Inst,
  FIELD_dsp340050b49a6c_fld2220Inst,
  FIELD_dsp340050b49a6c_fld2221Inst,
  FIELD_dsp340050b49a6c_fld2222Inst,
  FIELD_dsp340050b49a6c_fld2223Inst,
  FIELD_dsp340050b49a6c_fld2224Inst,
  FIELD_dsp340050b49a6c_fld2225Inst,
  FIELD_dsp340050b49a6c_fld2226Inst,
  FIELD_dsp340050b49a6c_fld2227Inst,
  FIELD_dsp340050b49a6c_fld2228Inst,
  FIELD_dsp340050b49a6c_fld2229Inst,
  FIELD_dsp340050b49a6c_fld2230Inst,
  FIELD_dsp340050b49a6c_fld2231Inst,
  FIELD_dsp340050b49a6c_fld2232Inst,
  FIELD_dsp340050b49a6c_fld2234Inst,
  FIELD_dsp340050b49a6c_fld2235Inst,
  FIELD_dsp340050b49a6c_fld2236Inst,
  FIELD_dsp340050b49a6c_fld2237Inst,
  FIELD_dsp340050b49a6c_fld2238Inst,
  FIELD_dsp340050b49a6c_fld2239Inst,
  FIELD_dsp340050b49a6c_fld2240Inst,
  FIELD_dsp340050b49a6c_fld2241Inst,
  FIELD_dsp340050b49a6c_fld2242Inst,
  FIELD_dsp340050b49a6c_fld2243Inst,
  FIELD_dsp340050b49a6c_fld2244Inst,
  FIELD_dsp340050b49a6c_fld2245Inst,
  FIELD_dsp340050b49a6c_fld2246Inst,
  FIELD_dsp340050b49a6c_fld2247Inst,
  FIELD_dsp340050b49a6c_fld2248Inst,
  FIELD_dsp340050b49a6c_fld2249Inst,
  FIELD_dsp340050b49a6c_fld2250Inst,
  FIELD_dsp340050b49a6c_fld2251Inst,
  FIELD_dsp340050b49a6c_fld2252Inst,
  FIELD_dsp340050b49a6c_fld2253Inst,
  FIELD_dsp340050b49a6c_fld2254,
  FIELD_dsp340050b49a6c_fld2255Inst,
  FIELD_dsp340050b49a6c_fld2257Inst,
  FIELD_dsp340050b49a6c_fld3627Inst,
  FIELD_dsp340050b49a6c_fld3630Inst,
  FIELD_dsp340050b49a6c_fld3631Inst,
  FIELD_dsp340050b49a6c_fld3633Inst,
  FIELD_dsp340050b49a6c_fld3634,
  FIELD_dsp340050b49a6c_fld3635Inst,
  FIELD_dsp340050b49a6c_fld3636Inst,
  FIELD_dsp340050b49a6c_fld3637Inst,
  FIELD_dsp340050b49a6c_fld3638Inst,
  FIELD_dsp340050b49a6c_fld3639Inst,
  FIELD_dsp340050b49a6c_fld3640Inst,
  FIELD_dsp340050b49a6c_fld3642Inst,
  FIELD_dsp340050b49a6c_fld3643Inst,
  FIELD_dsp340050b49a6c_fld3644Inst,
  FIELD_dsp340050b49a6c_fld3645Inst,
  FIELD_dsp340050b49a6c_fld3647Inst,
  FIELD_dsp340050b49a6c_fld3648Inst,
  FIELD_dsp340050b49a6c_fld3649Inst,
  FIELD_dsp340050b49a6c_fld3650Inst,
  FIELD_dsp340050b49a6c_fld3651Inst,
  FIELD_dsp340050b49a6c_fld3653Inst,
  FIELD_dsp340050b49a6c_fld3654Inst,
  FIELD_dsp340050b49a6c_fld3655Inst,
  FIELD_dsp340050b49a6c_fld3656Inst,
  FIELD_dsp340050b49a6c_fld3657Inst,
  FIELD_dsp340050b49a6c_fld3658Inst,
  FIELD_dsp340050b49a6c_fld3659Inst,
  FIELD_dsp340050b49a6c_fld3660Inst,
  FIELD_dsp340050b49a6c_fld3661Inst,
  FIELD_dsp340050b49a6c_fld3662Inst,
  FIELD_op0_s3,
  FIELD_dsp340050b49a6c_fld2025,
  FIELD_dsp340050b49a6c_fld2027,
  FIELD_dsp340050b49a6c_fld2258GP_slot2,
  FIELD_dsp340050b49a6c_fld2259GP_slot2,
  FIELD_dsp340050b49a6c_fld2260GP_slot2,
  FIELD_dsp340050b49a6c_fld2261GP_slot2,
  FIELD_dsp340050b49a6c_fld2262GP_slot2,
  FIELD_dsp340050b49a6c_fld2263GP_slot2,
  FIELD_dsp340050b49a6c_fld2264GP_slot2,
  FIELD_dsp340050b49a6c_fld2266GP_slot2,
  FIELD_dsp340050b49a6c_fld2267GP_slot2,
  FIELD_dsp340050b49a6c_fld2268GP_slot2,
  FIELD_dsp340050b49a6c_fld2269GP_slot2,
  FIELD_dsp340050b49a6c_fld2270GP_slot2,
  FIELD_dsp340050b49a6c_fld2271GP_slot2,
  FIELD_dsp340050b49a6c_fld2272,
  FIELD_dsp340050b49a6c_fld2273GP_slot2,
  FIELD_dsp340050b49a6c_fld2274GP_slot2,
  FIELD_dsp340050b49a6c_fld2275GP_slot2,
  FIELD_dsp340050b49a6c_fld2277GP_slot2,
  FIELD_dsp340050b49a6c_fld2278GP_slot2,
  FIELD_dsp340050b49a6c_fld2279GP_slot2,
  FIELD_dsp340050b49a6c_fld2280GP_slot2,
  FIELD_dsp340050b49a6c_fld2281GP_slot2,
  FIELD_dsp340050b49a6c_fld2282GP_slot2,
  FIELD_dsp340050b49a6c_fld2283GP_slot2,
  FIELD_dsp340050b49a6c_fld2284GP_slot2,
  FIELD_dsp340050b49a6c_fld2286GP_slot2,
  FIELD_dsp340050b49a6c_fld2287GP_slot2,
  FIELD_dsp340050b49a6c_fld2288GP_slot2,
  FIELD_dsp340050b49a6c_fld2289GP_slot2,
  FIELD_dsp340050b49a6c_fld2290GP_slot2,
  FIELD_dsp340050b49a6c_fld2291GP_slot2,
  FIELD_dsp340050b49a6c_fld2292GP_slot2,
  FIELD_dsp340050b49a6c_fld2293GP_slot2,
  FIELD_dsp340050b49a6c_fld2294GP_slot2,
  FIELD_dsp340050b49a6c_fld2295GP_slot2,
  FIELD_dsp340050b49a6c_fld2296GP_slot2,
  FIELD_dsp340050b49a6c_fld2297GP_slot2,
  FIELD_dsp340050b49a6c_fld2298GP_slot2,
  FIELD_dsp340050b49a6c_fld2299GP_slot2,
  FIELD_dsp340050b49a6c_fld2300GP_slot2,
  FIELD_dsp340050b49a6c_fld2301GP_slot2,
  FIELD_dsp340050b49a6c_fld2302,
  FIELD_dsp340050b49a6c_fld2303GP_slot2,
  FIELD_dsp340050b49a6c_fld2304GP_slot2,
  FIELD_dsp340050b49a6c_fld2305,
  FIELD_dsp340050b49a6c_fld2306GP_slot2,
  FIELD_dsp340050b49a6c_fld2308GP_slot2,
  FIELD_dsp340050b49a6c_fld2309GP_slot2,
  FIELD_dsp340050b49a6c_fld2310GP_slot2,
  FIELD_dsp340050b49a6c_fld2312GP_slot2,
  FIELD_dsp340050b49a6c_fld2313,
  FIELD_dsp340050b49a6c_fld2314GP_slot2,
  FIELD_dsp340050b49a6c_fld2316GP_slot2,
  FIELD_dsp340050b49a6c_fld2317GP_slot2,
  FIELD_dsp340050b49a6c_fld2318GP_slot2,
  FIELD_dsp340050b49a6c_fld2319GP_slot2,
  FIELD_dsp340050b49a6c_fld2320GP_slot2,
  FIELD_dsp340050b49a6c_fld2321GP_slot2,
  FIELD_dsp340050b49a6c_fld2322GP_slot2,
  FIELD_dsp340050b49a6c_fld2323GP_slot2,
  FIELD_dsp340050b49a6c_fld2324GP_slot2,
  FIELD_dsp340050b49a6c_fld2325GP_slot2,
  FIELD_dsp340050b49a6c_fld2326GP_slot2,
  FIELD_dsp340050b49a6c_fld2327GP_slot2,
  FIELD_dsp340050b49a6c_fld2328GP_slot2,
  FIELD_dsp340050b49a6c_fld2329GP_slot2,
  FIELD_dsp340050b49a6c_fld2330GP_slot2,
  FIELD_dsp340050b49a6c_fld2331GP_slot2,
  FIELD_dsp340050b49a6c_fld2332GP_slot2,
  FIELD_dsp340050b49a6c_fld2333GP_slot2,
  FIELD_dsp340050b49a6c_fld2334GP_slot2,
  FIELD_dsp340050b49a6c_fld2335GP_slot2,
  FIELD_dsp340050b49a6c_fld2336GP_slot2,
  FIELD_dsp340050b49a6c_fld2337GP_slot2,
  FIELD_dsp340050b49a6c_fld2338GP_slot2,
  FIELD_dsp340050b49a6c_fld2339GP_slot2,
  FIELD_dsp340050b49a6c_fld2340GP_slot2,
  FIELD_dsp340050b49a6c_fld2341GP_slot2,
  FIELD_dsp340050b49a6c_fld2342GP_slot2,
  FIELD_dsp340050b49a6c_fld2343GP_slot2,
  FIELD_dsp340050b49a6c_fld2344GP_slot2,
  FIELD_dsp340050b49a6c_fld2345GP_slot2,
  FIELD_dsp340050b49a6c_fld2346GP_slot2,
  FIELD_dsp340050b49a6c_fld2347GP_slot2,
  FIELD_dsp340050b49a6c_fld2348GP_slot2,
  FIELD_dsp340050b49a6c_fld2349GP_slot2,
  FIELD_dsp340050b49a6c_fld2350GP_slot2,
  FIELD_dsp340050b49a6c_fld2351GP_slot2,
  FIELD_dsp340050b49a6c_fld2352GP_slot2,
  FIELD_dsp340050b49a6c_fld2353GP_slot2,
  FIELD_dsp340050b49a6c_fld2354GP_slot2,
  FIELD_dsp340050b49a6c_fld2355GP_slot2,
  FIELD_dsp340050b49a6c_fld2356GP_slot2,
  FIELD_dsp340050b49a6c_fld2357GP_slot2,
  FIELD_dsp340050b49a6c_fld2358GP_slot2,
  FIELD_dsp340050b49a6c_fld2359GP_slot2,
  FIELD_dsp340050b49a6c_fld2361GP_slot2,
  FIELD_dsp340050b49a6c_fld2362GP_slot2,
  FIELD_dsp340050b49a6c_fld2364GP_slot2,
  FIELD_dsp340050b49a6c_fld2366GP_slot2,
  FIELD_dsp340050b49a6c_fld2368GP_slot2,
  FIELD_dsp340050b49a6c_fld2369GP_slot2,
  FIELD_dsp340050b49a6c_fld2370GP_slot2,
  FIELD_dsp340050b49a6c_fld2371GP_slot2,
  FIELD_dsp340050b49a6c_fld2372GP_slot2,
  FIELD_dsp340050b49a6c_fld2373GP_slot2,
  FIELD_dsp340050b49a6c_fld2374GP_slot2,
  FIELD_dsp340050b49a6c_fld2375GP_slot2,
  FIELD_dsp340050b49a6c_fld2376GP_slot2,
  FIELD_dsp340050b49a6c_fld2378GP_slot2,
  FIELD_dsp340050b49a6c_fld2379GP_slot2,
  FIELD_dsp340050b49a6c_fld2381GP_slot2,
  FIELD_dsp340050b49a6c_fld2383GP_slot2,
  FIELD_dsp340050b49a6c_fld2384,
  FIELD_dsp340050b49a6c_fld2385GP_slot2,
  FIELD_dsp340050b49a6c_fld2386,
  FIELD_dsp340050b49a6c_fld2387GP_slot2,
  FIELD_dsp340050b49a6c_fld2388GP_slot2,
  FIELD_dsp340050b49a6c_fld2389GP_slot2,
  FIELD_dsp340050b49a6c_fld3663GP_slot2,
  FIELD_dsp340050b49a6c_fld3664GP_slot2,
  FIELD_dsp340050b49a6c_fld3665GP_slot2,
  FIELD_dsp340050b49a6c_fld3666,
  FIELD_dsp340050b49a6c_fld3667GP_slot2,
  FIELD_dsp340050b49a6c_fld3668GP_slot2,
  FIELD_dsp340050b49a6c_fld3669GP_slot2,
  FIELD_dsp340050b49a6c_fld3670GP_slot2,
  FIELD_dsp340050b49a6c_fld3671GP_slot2,
  FIELD_dsp340050b49a6c_fld3673GP_slot2,
  FIELD_dsp340050b49a6c_fld3674GP_slot2,
  FIELD_dsp340050b49a6c_fld3675GP_slot2,
  FIELD_dsp340050b49a6c_fld3676GP_slot2,
  FIELD_dsp340050b49a6c_fld3678GP_slot2,
  FIELD_dsp340050b49a6c_fld3679GP_slot2,
  FIELD_dsp340050b49a6c_fld3680GP_slot2,
  FIELD_op0_s4,
  FIELD_dsp340050b49a6c_fld2026,
  FIELD_dsp340050b49a6c_fld2031,
  FIELD_dsp340050b49a6c_fld2394GP_slot1,
  FIELD_dsp340050b49a6c_fld2395GP_slot1,
  FIELD_dsp340050b49a6c_fld2397GP_slot1,
  FIELD_dsp340050b49a6c_fld2398GP_slot1,
  FIELD_dsp340050b49a6c_fld2399GP_slot1,
  FIELD_dsp340050b49a6c_fld2400GP_slot1,
  FIELD_dsp340050b49a6c_fld2402GP_slot1,
  FIELD_dsp340050b49a6c_fld2403GP_slot1,
  FIELD_dsp340050b49a6c_fld2405GP_slot1,
  FIELD_dsp340050b49a6c_fld3681GP_slot1,
  FIELD_dsp340050b49a6c_fld3683GP_slot1,
  FIELD_dsp340050b49a6c_fld3684GP_slot1,
  FIELD_dsp340050b49a6c_fld3686GP_slot1,
  FIELD_op0_s5,
  FIELD_dsp340050b49a6c_fld2058,
  FIELD_dsp340050b49a6c_fld2067,
  FIELD_dsp340050b49a6c_fld2407GP_slot0,
  FIELD_dsp340050b49a6c_fld2409GP_slot0,
  FIELD_dsp340050b49a6c_fld2410GP_slot0,
  FIELD_dsp340050b49a6c_fld2411GP_slot0,
  FIELD_dsp340050b49a6c_fld2412GP_slot0,
  FIELD_dsp340050b49a6c_fld2413GP_slot0,
  FIELD_dsp340050b49a6c_fld2415GP_slot0,
  FIELD_dsp340050b49a6c_fld2416GP_slot0,
  FIELD_dsp340050b49a6c_fld2417GP_slot0,
  FIELD_dsp340050b49a6c_fld2418GP_slot0,
  FIELD_dsp340050b49a6c_fld2419GP_slot0,
  FIELD_dsp340050b49a6c_fld2420GP_slot0,
  FIELD_dsp340050b49a6c_fld2422GP_slot0,
  FIELD_dsp340050b49a6c_fld2423GP_slot0,
  FIELD_dsp340050b49a6c_fld2424GP_slot0,
  FIELD_dsp340050b49a6c_fld2425GP_slot0,
  FIELD_dsp340050b49a6c_fld2426GP_slot0,
  FIELD_dsp340050b49a6c_fld2427GP_slot0,
  FIELD_dsp340050b49a6c_fld2429GP_slot0,
  FIELD_dsp340050b49a6c_fld2430GP_slot0,
  FIELD_dsp340050b49a6c_fld2431GP_slot0,
  FIELD_dsp340050b49a6c_fld2432GP_slot0,
  FIELD_dsp340050b49a6c_fld2433GP_slot0,
  FIELD_dsp340050b49a6c_fld2434GP_slot0,
  FIELD_dsp340050b49a6c_fld2435GP_slot0,
  FIELD_dsp340050b49a6c_fld2436GP_slot0,
  FIELD_dsp340050b49a6c_fld2437GP_slot0,
  FIELD_dsp340050b49a6c_fld2438GP_slot0,
  FIELD_dsp340050b49a6c_fld2439GP_slot0,
  FIELD_dsp340050b49a6c_fld2440GP_slot0,
  FIELD_dsp340050b49a6c_fld2441GP_slot0,
  FIELD_dsp340050b49a6c_fld2443GP_slot0,
  FIELD_dsp340050b49a6c_fld2444GP_slot0,
  FIELD_dsp340050b49a6c_fld2445,
  FIELD_dsp340050b49a6c_fld2447GP_slot0,
  FIELD_dsp340050b49a6c_fld2448,
  FIELD_dsp340050b49a6c_fld2449GP_slot0,
  FIELD_dsp340050b49a6c_fld2451GP_slot0,
  FIELD_dsp340050b49a6c_fld2452GP_slot0,
  FIELD_dsp340050b49a6c_fld2453GP_slot0,
  FIELD_dsp340050b49a6c_fld2454GP_slot0,
  FIELD_dsp340050b49a6c_fld2455GP_slot0,
  FIELD_dsp340050b49a6c_fld2456GP_slot0,
  FIELD_dsp340050b49a6c_fld2457GP_slot0,
  FIELD_dsp340050b49a6c_fld2458GP_slot0,
  FIELD_dsp340050b49a6c_fld2459GP_slot0,
  FIELD_dsp340050b49a6c_fld2460GP_slot0,
  FIELD_dsp340050b49a6c_fld2461GP_slot0,
  FIELD_dsp340050b49a6c_fld2462,
  FIELD_dsp340050b49a6c_fld2463GP_slot0,
  FIELD_dsp340050b49a6c_fld2464GP_slot0,
  FIELD_dsp340050b49a6c_fld2465GP_slot0,
  FIELD_dsp340050b49a6c_fld2466GP_slot0,
  FIELD_dsp340050b49a6c_fld2467GP_slot0,
  FIELD_dsp340050b49a6c_fld2468GP_slot0,
  FIELD_dsp340050b49a6c_fld2470GP_slot0,
  FIELD_dsp340050b49a6c_fld2471GP_slot0,
  FIELD_dsp340050b49a6c_fld2472GP_slot0,
  FIELD_dsp340050b49a6c_fld2473GP_slot0,
  FIELD_dsp340050b49a6c_fld2474GP_slot0,
  FIELD_dsp340050b49a6c_fld2475GP_slot0,
  FIELD_dsp340050b49a6c_fld2477GP_slot0,
  FIELD_dsp340050b49a6c_fld2479GP_slot0,
  FIELD_dsp340050b49a6c_fld2480GP_slot0,
  FIELD_dsp340050b49a6c_fld2481GP_slot0,
  FIELD_dsp340050b49a6c_fld2482GP_slot0,
  FIELD_dsp340050b49a6c_fld2483GP_slot0,
  FIELD_dsp340050b49a6c_fld2484GP_slot0,
  FIELD_dsp340050b49a6c_fld2485GP_slot0,
  FIELD_dsp340050b49a6c_fld2486GP_slot0,
  FIELD_dsp340050b49a6c_fld2487GP_slot0,
  FIELD_dsp340050b49a6c_fld2488GP_slot0,
  FIELD_dsp340050b49a6c_fld2489GP_slot0,
  FIELD_dsp340050b49a6c_fld2490GP_slot0,
  FIELD_dsp340050b49a6c_fld2491GP_slot0,
  FIELD_dsp340050b49a6c_fld2492GP_slot0,
  FIELD_dsp340050b49a6c_fld2493GP_slot0,
  FIELD_dsp340050b49a6c_fld2494GP_slot0,
  FIELD_dsp340050b49a6c_fld2495GP_slot0,
  FIELD_dsp340050b49a6c_fld2496GP_slot0,
  FIELD_dsp340050b49a6c_fld2497GP_slot0,
  FIELD_dsp340050b49a6c_fld2498GP_slot0,
  FIELD_dsp340050b49a6c_fld2499GP_slot0,
  FIELD_dsp340050b49a6c_fld2500GP_slot0,
  FIELD_dsp340050b49a6c_fld2501GP_slot0,
  FIELD_dsp340050b49a6c_fld2502GP_slot0,
  FIELD_dsp340050b49a6c_fld2503GP_slot0,
  FIELD_dsp340050b49a6c_fld2504GP_slot0,
  FIELD_dsp340050b49a6c_fld2505GP_slot0,
  FIELD_dsp340050b49a6c_fld2506GP_slot0,
  FIELD_dsp340050b49a6c_fld2507GP_slot0,
  FIELD_dsp340050b49a6c_fld2508GP_slot0,
  FIELD_dsp340050b49a6c_fld2509GP_slot0,
  FIELD_dsp340050b49a6c_fld2510GP_slot0,
  FIELD_dsp340050b49a6c_fld2512GP_slot0,
  FIELD_dsp340050b49a6c_fld2514GP_slot0,
  FIELD_dsp340050b49a6c_fld2515GP_slot0,
  FIELD_dsp340050b49a6c_fld2516GP_slot0,
  FIELD_dsp340050b49a6c_fld2517GP_slot0,
  FIELD_dsp340050b49a6c_fld2518GP_slot0,
  FIELD_dsp340050b49a6c_fld2519GP_slot0,
  FIELD_dsp340050b49a6c_fld2520GP_slot0,
  FIELD_dsp340050b49a6c_fld2521GP_slot0,
  FIELD_dsp340050b49a6c_fld2523GP_slot0,
  FIELD_dsp340050b49a6c_fld2524GP_slot0,
  FIELD_dsp340050b49a6c_fld2526GP_slot0,
  FIELD_dsp340050b49a6c_fld2527GP_slot0,
  FIELD_dsp340050b49a6c_fld2528GP_slot0,
  FIELD_dsp340050b49a6c_fld2529GP_slot0,
  FIELD_dsp340050b49a6c_fld2530,
  FIELD_dsp340050b49a6c_fld2531GP_slot0,
  FIELD_dsp340050b49a6c_fld3688GP_slot0,
  FIELD_dsp340050b49a6c_fld3689GP_slot0,
  FIELD_dsp340050b49a6c_fld3690GP_slot0,
  FIELD_dsp340050b49a6c_fld3691GP_slot0,
  FIELD_dsp340050b49a6c_fld3692GP_slot0,
  FIELD_dsp340050b49a6c_fld3693GP_slot0,
  FIELD_dsp340050b49a6c_fld3695GP_slot0,
  FIELD_dsp340050b49a6c_fld3696GP_slot0,
  FIELD_dsp340050b49a6c_fld3697GP_slot0,
  FIELD_dsp340050b49a6c_fld3698GP_slot0,
  FIELD_dsp340050b49a6c_fld3699GP_slot0,
  FIELD_dsp340050b49a6c_fld3700GP_slot0,
  FIELD_dsp340050b49a6c_fld3702GP_slot0,
  FIELD_dsp340050b49a6c_fld3703GP_slot0,
  FIELD_dsp340050b49a6c_fld3705GP_slot0,
  FIELD_dsp340050b49a6c_fld3706GP_slot0,
  FIELD_op0_s6,
  FIELD_dsp340050b49a6c_fld2532DOT_slot2,
  FIELD_dsp340050b49a6c_fld2533DOT_slot2,
  FIELD_dsp340050b49a6c_fld2534DOT_slot2,
  FIELD_dsp340050b49a6c_fld2535DOT_slot2,
  FIELD_dsp340050b49a6c_fld2536DOT_slot2,
  FIELD_dsp340050b49a6c_fld2537DOT_slot2,
  FIELD_dsp340050b49a6c_fld2538DOT_slot2,
  FIELD_dsp340050b49a6c_fld2539DOT_slot2,
  FIELD_dsp340050b49a6c_fld2540DOT_slot2,
  FIELD_dsp340050b49a6c_fld2541DOT_slot2,
  FIELD_dsp340050b49a6c_fld2542DOT_slot2,
  FIELD_dsp340050b49a6c_fld2543DOT_slot2,
  FIELD_dsp340050b49a6c_fld2544DOT_slot2,
  FIELD_dsp340050b49a6c_fld2545DOT_slot2,
  FIELD_dsp340050b49a6c_fld2546DOT_slot2,
  FIELD_dsp340050b49a6c_fld2547DOT_slot2,
  FIELD_dsp340050b49a6c_fld2548DOT_slot2,
  FIELD_dsp340050b49a6c_fld2549DOT_slot2,
  FIELD_dsp340050b49a6c_fld2550DOT_slot2,
  FIELD_dsp340050b49a6c_fld2551DOT_slot2,
  FIELD_dsp340050b49a6c_fld2552DOT_slot2,
  FIELD_dsp340050b49a6c_fld2553DOT_slot2,
  FIELD_dsp340050b49a6c_fld2554DOT_slot2,
  FIELD_dsp340050b49a6c_fld2555DOT_slot2,
  FIELD_dsp340050b49a6c_fld2556DOT_slot2,
  FIELD_dsp340050b49a6c_fld2557DOT_slot2,
  FIELD_dsp340050b49a6c_fld2558DOT_slot2,
  FIELD_dsp340050b49a6c_fld2559DOT_slot2,
  FIELD_dsp340050b49a6c_fld2560DOT_slot2,
  FIELD_dsp340050b49a6c_fld2561DOT_slot2,
  FIELD_dsp340050b49a6c_fld2562DOT_slot2,
  FIELD_dsp340050b49a6c_fld2563DOT_slot2,
  FIELD_dsp340050b49a6c_fld2564DOT_slot2,
  FIELD_dsp340050b49a6c_fld2565DOT_slot2,
  FIELD_dsp340050b49a6c_fld2566DOT_slot2,
  FIELD_dsp340050b49a6c_fld2567DOT_slot2,
  FIELD_dsp340050b49a6c_fld2568DOT_slot2,
  FIELD_dsp340050b49a6c_fld2569DOT_slot2,
  FIELD_dsp340050b49a6c_fld2571DOT_slot2,
  FIELD_dsp340050b49a6c_fld2572DOT_slot2,
  FIELD_dsp340050b49a6c_fld2573DOT_slot2,
  FIELD_dsp340050b49a6c_fld2574DOT_slot2,
  FIELD_dsp340050b49a6c_fld2575DOT_slot2,
  FIELD_dsp340050b49a6c_fld2576DOT_slot2,
  FIELD_dsp340050b49a6c_fld2577DOT_slot2,
  FIELD_dsp340050b49a6c_fld2578,
  FIELD_dsp340050b49a6c_fld2579DOT_slot2,
  FIELD_dsp340050b49a6c_fld2580DOT_slot2,
  FIELD_dsp340050b49a6c_fld2581DOT_slot2,
  FIELD_dsp340050b49a6c_fld2582DOT_slot2,
  FIELD_dsp340050b49a6c_fld2583DOT_slot2,
  FIELD_dsp340050b49a6c_fld2584DOT_slot2,
  FIELD_dsp340050b49a6c_fld2585DOT_slot2,
  FIELD_dsp340050b49a6c_fld2586DOT_slot2,
  FIELD_dsp340050b49a6c_fld2587DOT_slot2,
  FIELD_dsp340050b49a6c_fld2588DOT_slot2,
  FIELD_dsp340050b49a6c_fld2589DOT_slot2,
  FIELD_dsp340050b49a6c_fld2590DOT_slot2,
  FIELD_dsp340050b49a6c_fld2591DOT_slot2,
  FIELD_dsp340050b49a6c_fld2592DOT_slot2,
  FIELD_dsp340050b49a6c_fld2595DOT_slot2,
  FIELD_dsp340050b49a6c_fld2596DOT_slot2,
  FIELD_dsp340050b49a6c_fld2598DOT_slot2,
  FIELD_dsp340050b49a6c_fld2599DOT_slot2,
  FIELD_dsp340050b49a6c_fld2601DOT_slot2,
  FIELD_dsp340050b49a6c_fld2602DOT_slot2,
  FIELD_dsp340050b49a6c_fld2604DOT_slot2,
  FIELD_dsp340050b49a6c_fld2605,
  FIELD_dsp340050b49a6c_fld2606DOT_slot2,
  FIELD_dsp340050b49a6c_fld2608DOT_slot2,
  FIELD_dsp340050b49a6c_fld2609DOT_slot2,
  FIELD_dsp340050b49a6c_fld2610DOT_slot2,
  FIELD_dsp340050b49a6c_fld2611DOT_slot2,
  FIELD_dsp340050b49a6c_fld2612,
  FIELD_dsp340050b49a6c_fld2613DOT_slot2,
  FIELD_dsp340050b49a6c_fld2614,
  FIELD_dsp340050b49a6c_fld2615DOT_slot2,
  FIELD_dsp340050b49a6c_fld2616DOT_slot2,
  FIELD_dsp340050b49a6c_fld2617DOT_slot2,
  FIELD_dsp340050b49a6c_fld2618DOT_slot2,
  FIELD_dsp340050b49a6c_fld2619DOT_slot2,
  FIELD_dsp340050b49a6c_fld2620DOT_slot2,
  FIELD_dsp340050b49a6c_fld2621DOT_slot2,
  FIELD_dsp340050b49a6c_fld2622DOT_slot2,
  FIELD_dsp340050b49a6c_fld2623DOT_slot2,
  FIELD_dsp340050b49a6c_fld2624DOT_slot2,
  FIELD_dsp340050b49a6c_fld2625,
  FIELD_dsp340050b49a6c_fld2626DOT_slot2,
  FIELD_dsp340050b49a6c_fld2628DOT_slot2,
  FIELD_dsp340050b49a6c_fld2630DOT_slot2,
  FIELD_dsp340050b49a6c_fld2632DOT_slot2,
  FIELD_dsp340050b49a6c_fld2633DOT_slot2,
  FIELD_dsp340050b49a6c_fld2635DOT_slot2,
  FIELD_dsp340050b49a6c_fld2636DOT_slot2,
  FIELD_dsp340050b49a6c_fld2637DOT_slot2,
  FIELD_dsp340050b49a6c_fld2640DOT_slot2,
  FIELD_dsp340050b49a6c_fld2641DOT_slot2,
  FIELD_dsp340050b49a6c_fld2642DOT_slot2,
  FIELD_dsp340050b49a6c_fld2643DOT_slot2,
  FIELD_dsp340050b49a6c_fld2644DOT_slot2,
  FIELD_dsp340050b49a6c_fld2645DOT_slot2,
  FIELD_dsp340050b49a6c_fld2646DOT_slot2,
  FIELD_dsp340050b49a6c_fld2647DOT_slot2,
  FIELD_dsp340050b49a6c_fld2648DOT_slot2,
  FIELD_dsp340050b49a6c_fld2649DOT_slot2,
  FIELD_dsp340050b49a6c_fld2650DOT_slot2,
  FIELD_dsp340050b49a6c_fld2651DOT_slot2,
  FIELD_dsp340050b49a6c_fld2652DOT_slot2,
  FIELD_dsp340050b49a6c_fld2654DOT_slot2,
  FIELD_dsp340050b49a6c_fld2655DOT_slot2,
  FIELD_dsp340050b49a6c_fld2656DOT_slot2,
  FIELD_dsp340050b49a6c_fld2657DOT_slot2,
  FIELD_dsp340050b49a6c_fld2658DOT_slot2,
  FIELD_dsp340050b49a6c_fld3708DOT_slot2,
  FIELD_dsp340050b49a6c_fld3709DOT_slot2,
  FIELD_dsp340050b49a6c_fld3710DOT_slot2,
  FIELD_dsp340050b49a6c_fld3711DOT_slot2,
  FIELD_dsp340050b49a6c_fld3712,
  FIELD_dsp340050b49a6c_fld3713DOT_slot2,
  FIELD_dsp340050b49a6c_fld3714DOT_slot2,
  FIELD_dsp340050b49a6c_fld3715,
  FIELD_dsp340050b49a6c_fld3716DOT_slot2,
  FIELD_dsp340050b49a6c_fld3717DOT_slot2,
  FIELD_dsp340050b49a6c_fld3718,
  FIELD_dsp340050b49a6c_fld3719DOT_slot2,
  FIELD_dsp340050b49a6c_fld3721DOT_slot2,
  FIELD_dsp340050b49a6c_fld3722,
  FIELD_dsp340050b49a6c_fld3723DOT_slot2,
  FIELD_dsp340050b49a6c_fld3724DOT_slot2,
  FIELD_dsp340050b49a6c_fld3725DOT_slot2,
  FIELD_dsp340050b49a6c_fld3726DOT_slot2,
  FIELD_dsp340050b49a6c_fld3727DOT_slot2,
  FIELD_dsp340050b49a6c_fld3728DOT_slot2,
  FIELD_dsp340050b49a6c_fld3729DOT_slot2,
  FIELD_dsp340050b49a6c_fld3731DOT_slot2,
  FIELD_dsp340050b49a6c_fld3732DOT_slot2,
  FIELD_dsp340050b49a6c_fld3733DOT_slot2,
  FIELD_op0_s7,
  FIELD_dsp340050b49a6c_fld3734DOT_slot1,
  FIELD_op0_s8,
  FIELD_dsp340050b49a6c_fld2068,
  FIELD_dsp340050b49a6c_fld2666DOT_slot0,
  FIELD_dsp340050b49a6c_fld2667DOT_slot0,
  FIELD_dsp340050b49a6c_fld2668DOT_slot0,
  FIELD_dsp340050b49a6c_fld2669DOT_slot0,
  FIELD_dsp340050b49a6c_fld2671DOT_slot0,
  FIELD_dsp340050b49a6c_fld2672DOT_slot0,
  FIELD_dsp340050b49a6c_fld2673DOT_slot0,
  FIELD_dsp340050b49a6c_fld2674DOT_slot0,
  FIELD_dsp340050b49a6c_fld2675DOT_slot0,
  FIELD_dsp340050b49a6c_fld2676DOT_slot0,
  FIELD_dsp340050b49a6c_fld2677DOT_slot0,
  FIELD_dsp340050b49a6c_fld2678DOT_slot0,
  FIELD_dsp340050b49a6c_fld2679DOT_slot0,
  FIELD_dsp340050b49a6c_fld2680DOT_slot0,
  FIELD_dsp340050b49a6c_fld2681DOT_slot0,
  FIELD_dsp340050b49a6c_fld2682DOT_slot0,
  FIELD_dsp340050b49a6c_fld2683DOT_slot0,
  FIELD_dsp340050b49a6c_fld2684DOT_slot0,
  FIELD_dsp340050b49a6c_fld2685DOT_slot0,
  FIELD_dsp340050b49a6c_fld2686DOT_slot0,
  FIELD_dsp340050b49a6c_fld2688DOT_slot0,
  FIELD_dsp340050b49a6c_fld2689DOT_slot0,
  FIELD_dsp340050b49a6c_fld2690DOT_slot0,
  FIELD_dsp340050b49a6c_fld2692DOT_slot0,
  FIELD_dsp340050b49a6c_fld2693DOT_slot0,
  FIELD_dsp340050b49a6c_fld2695DOT_slot0,
  FIELD_dsp340050b49a6c_fld2697DOT_slot0,
  FIELD_dsp340050b49a6c_fld2699DOT_slot0,
  FIELD_dsp340050b49a6c_fld2700DOT_slot0,
  FIELD_dsp340050b49a6c_fld2701DOT_slot0,
  FIELD_dsp340050b49a6c_fld2702DOT_slot0,
  FIELD_dsp340050b49a6c_fld2703DOT_slot0,
  FIELD_dsp340050b49a6c_fld2704DOT_slot0,
  FIELD_dsp340050b49a6c_fld2705DOT_slot0,
  FIELD_dsp340050b49a6c_fld3735DOT_slot0,
  FIELD_dsp340050b49a6c_fld3736,
  FIELD_dsp340050b49a6c_fld3737DOT_slot0,
  FIELD_dsp340050b49a6c_fld3738DOT_slot0,
  FIELD_dsp340050b49a6c_fld3739DOT_slot0,
  FIELD_dsp340050b49a6c_fld3740DOT_slot0,
  FIELD_dsp340050b49a6c_fld3741DOT_slot0,
  FIELD_dsp340050b49a6c_fld3742DOT_slot0,
  FIELD_op0_s9,
  FIELD_dsp340050b49a6c_fld2706PQ_slot2,
  FIELD_dsp340050b49a6c_fld2707PQ_slot2,
  FIELD_dsp340050b49a6c_fld2708PQ_slot2,
  FIELD_dsp340050b49a6c_fld2709PQ_slot2,
  FIELD_dsp340050b49a6c_fld2710PQ_slot2,
  FIELD_dsp340050b49a6c_fld2711PQ_slot2,
  FIELD_dsp340050b49a6c_fld2713PQ_slot2,
  FIELD_dsp340050b49a6c_fld2714PQ_slot2,
  FIELD_dsp340050b49a6c_fld2715PQ_slot2,
  FIELD_dsp340050b49a6c_fld2717PQ_slot2,
  FIELD_dsp340050b49a6c_fld2718PQ_slot2,
  FIELD_dsp340050b49a6c_fld2719PQ_slot2,
  FIELD_dsp340050b49a6c_fld2721PQ_slot2,
  FIELD_dsp340050b49a6c_fld2722PQ_slot2,
  FIELD_dsp340050b49a6c_fld2723PQ_slot2,
  FIELD_dsp340050b49a6c_fld2724PQ_slot2,
  FIELD_dsp340050b49a6c_fld2725PQ_slot2,
  FIELD_dsp340050b49a6c_fld2726PQ_slot2,
  FIELD_dsp340050b49a6c_fld2727PQ_slot2,
  FIELD_dsp340050b49a6c_fld2728PQ_slot2,
  FIELD_dsp340050b49a6c_fld2729PQ_slot2,
  FIELD_dsp340050b49a6c_fld2730PQ_slot2,
  FIELD_dsp340050b49a6c_fld2731PQ_slot2,
  FIELD_dsp340050b49a6c_fld2732PQ_slot2,
  FIELD_dsp340050b49a6c_fld2733PQ_slot2,
  FIELD_dsp340050b49a6c_fld2734PQ_slot2,
  FIELD_dsp340050b49a6c_fld2735PQ_slot2,
  FIELD_dsp340050b49a6c_fld2736PQ_slot2,
  FIELD_dsp340050b49a6c_fld2737,
  FIELD_dsp340050b49a6c_fld2738PQ_slot2,
  FIELD_dsp340050b49a6c_fld2739PQ_slot2,
  FIELD_dsp340050b49a6c_fld2741PQ_slot2,
  FIELD_dsp340050b49a6c_fld2742PQ_slot2,
  FIELD_dsp340050b49a6c_fld2743PQ_slot2,
  FIELD_dsp340050b49a6c_fld2746PQ_slot2,
  FIELD_dsp340050b49a6c_fld2747PQ_slot2,
  FIELD_dsp340050b49a6c_fld2748PQ_slot2,
  FIELD_dsp340050b49a6c_fld2750PQ_slot2,
  FIELD_dsp340050b49a6c_fld2751PQ_slot2,
  FIELD_dsp340050b49a6c_fld2752PQ_slot2,
  FIELD_dsp340050b49a6c_fld2753PQ_slot2,
  FIELD_dsp340050b49a6c_fld2754PQ_slot2,
  FIELD_dsp340050b49a6c_fld2755PQ_slot2,
  FIELD_dsp340050b49a6c_fld2756PQ_slot2,
  FIELD_dsp340050b49a6c_fld2757PQ_slot2,
  FIELD_dsp340050b49a6c_fld2758PQ_slot2,
  FIELD_dsp340050b49a6c_fld2759PQ_slot2,
  FIELD_dsp340050b49a6c_fld2760PQ_slot2,
  FIELD_dsp340050b49a6c_fld2761PQ_slot2,
  FIELD_dsp340050b49a6c_fld2762PQ_slot2,
  FIELD_dsp340050b49a6c_fld2763PQ_slot2,
  FIELD_dsp340050b49a6c_fld2764PQ_slot2,
  FIELD_dsp340050b49a6c_fld2765PQ_slot2,
  FIELD_dsp340050b49a6c_fld2766PQ_slot2,
  FIELD_dsp340050b49a6c_fld2767PQ_slot2,
  FIELD_dsp340050b49a6c_fld2768PQ_slot2,
  FIELD_dsp340050b49a6c_fld2769PQ_slot2,
  FIELD_dsp340050b49a6c_fld2770PQ_slot2,
  FIELD_dsp340050b49a6c_fld2771PQ_slot2,
  FIELD_dsp340050b49a6c_fld2772PQ_slot2,
  FIELD_dsp340050b49a6c_fld2773PQ_slot2,
  FIELD_dsp340050b49a6c_fld2774PQ_slot2,
  FIELD_dsp340050b49a6c_fld2775PQ_slot2,
  FIELD_dsp340050b49a6c_fld2776PQ_slot2,
  FIELD_dsp340050b49a6c_fld2777PQ_slot2,
  FIELD_dsp340050b49a6c_fld2778PQ_slot2,
  FIELD_dsp340050b49a6c_fld2779PQ_slot2,
  FIELD_dsp340050b49a6c_fld2780PQ_slot2,
  FIELD_dsp340050b49a6c_fld2781PQ_slot2,
  FIELD_dsp340050b49a6c_fld2782PQ_slot2,
  FIELD_dsp340050b49a6c_fld2783PQ_slot2,
  FIELD_dsp340050b49a6c_fld2784PQ_slot2,
  FIELD_dsp340050b49a6c_fld2785PQ_slot2,
  FIELD_dsp340050b49a6c_fld2786PQ_slot2,
  FIELD_dsp340050b49a6c_fld2787PQ_slot2,
  FIELD_dsp340050b49a6c_fld2788PQ_slot2,
  FIELD_dsp340050b49a6c_fld2789PQ_slot2,
  FIELD_dsp340050b49a6c_fld2790PQ_slot2,
  FIELD_dsp340050b49a6c_fld2791PQ_slot2,
  FIELD_dsp340050b49a6c_fld2792PQ_slot2,
  FIELD_dsp340050b49a6c_fld2793PQ_slot2,
  FIELD_dsp340050b49a6c_fld2795PQ_slot2,
  FIELD_dsp340050b49a6c_fld2796PQ_slot2,
  FIELD_dsp340050b49a6c_fld2798PQ_slot2,
  FIELD_dsp340050b49a6c_fld2801PQ_slot2,
  FIELD_dsp340050b49a6c_fld2803PQ_slot2,
  FIELD_dsp340050b49a6c_fld2805PQ_slot2,
  FIELD_dsp340050b49a6c_fld2806PQ_slot2,
  FIELD_dsp340050b49a6c_fld2807PQ_slot2,
  FIELD_dsp340050b49a6c_fld2808PQ_slot2,
  FIELD_dsp340050b49a6c_fld2809PQ_slot2,
  FIELD_dsp340050b49a6c_fld2810PQ_slot2,
  FIELD_dsp340050b49a6c_fld2811PQ_slot2,
  FIELD_dsp340050b49a6c_fld2812PQ_slot2,
  FIELD_dsp340050b49a6c_fld2814PQ_slot2,
  FIELD_dsp340050b49a6c_fld2816PQ_slot2,
  FIELD_dsp340050b49a6c_fld2817,
  FIELD_dsp340050b49a6c_fld2818PQ_slot2,
  FIELD_dsp340050b49a6c_fld2819,
  FIELD_dsp340050b49a6c_fld2820PQ_slot2,
  FIELD_dsp340050b49a6c_fld2821PQ_slot2,
  FIELD_dsp340050b49a6c_fld2823PQ_slot2,
  FIELD_dsp340050b49a6c_fld3744PQ_slot2,
  FIELD_dsp340050b49a6c_fld3745PQ_slot2,
  FIELD_dsp340050b49a6c_fld3746PQ_slot2,
  FIELD_dsp340050b49a6c_fld3747PQ_slot2,
  FIELD_dsp340050b49a6c_fld3748,
  FIELD_dsp340050b49a6c_fld3749PQ_slot2,
  FIELD_dsp340050b49a6c_fld3750PQ_slot2,
  FIELD_dsp340050b49a6c_fld3751PQ_slot2,
  FIELD_dsp340050b49a6c_fld3752PQ_slot2,
  FIELD_dsp340050b49a6c_fld3753PQ_slot2,
  FIELD_dsp340050b49a6c_fld3754PQ_slot2,
  FIELD_dsp340050b49a6c_fld3756PQ_slot2,
  FIELD_dsp340050b49a6c_fld3757PQ_slot2,
  FIELD_dsp340050b49a6c_fld3758PQ_slot2,
  FIELD_dsp340050b49a6c_fld3759PQ_slot2,
  FIELD_dsp340050b49a6c_fld3760PQ_slot2,
  FIELD_op0_s10,
  FIELD_dsp340050b49a6c_fld2825PQ_slot1,
  FIELD_dsp340050b49a6c_fld2826PQ_slot1,
  FIELD_dsp340050b49a6c_fld3761PQ_slot1,
  FIELD_op0_s11,
  FIELD_dsp340050b49a6c_fld2059,
  FIELD_dsp340050b49a6c_fld2069,
  FIELD_dsp340050b49a6c_fld2827PQ_slot0,
  FIELD_dsp340050b49a6c_fld2829PQ_slot0,
  FIELD_dsp340050b49a6c_fld2830PQ_slot0,
  FIELD_dsp340050b49a6c_fld2831PQ_slot0,
  FIELD_dsp340050b49a6c_fld2832PQ_slot0,
  FIELD_dsp340050b49a6c_fld2833PQ_slot0,
  FIELD_dsp340050b49a6c_fld2835PQ_slot0,
  FIELD_dsp340050b49a6c_fld2836PQ_slot0,
  FIELD_dsp340050b49a6c_fld2837PQ_slot0,
  FIELD_dsp340050b49a6c_fld2838PQ_slot0,
  FIELD_dsp340050b49a6c_fld2839PQ_slot0,
  FIELD_dsp340050b49a6c_fld2840PQ_slot0,
  FIELD_dsp340050b49a6c_fld2842PQ_slot0,
  FIELD_dsp340050b49a6c_fld2843PQ_slot0,
  FIELD_dsp340050b49a6c_fld2844PQ_slot0,
  FIELD_dsp340050b49a6c_fld2845PQ_slot0,
  FIELD_dsp340050b49a6c_fld2846PQ_slot0,
  FIELD_dsp340050b49a6c_fld2847PQ_slot0,
  FIELD_dsp340050b49a6c_fld2849PQ_slot0,
  FIELD_dsp340050b49a6c_fld2850PQ_slot0,
  FIELD_dsp340050b49a6c_fld2851PQ_slot0,
  FIELD_dsp340050b49a6c_fld2852PQ_slot0,
  FIELD_dsp340050b49a6c_fld2853PQ_slot0,
  FIELD_dsp340050b49a6c_fld2854PQ_slot0,
  FIELD_dsp340050b49a6c_fld2855PQ_slot0,
  FIELD_dsp340050b49a6c_fld2856PQ_slot0,
  FIELD_dsp340050b49a6c_fld2857PQ_slot0,
  FIELD_dsp340050b49a6c_fld2858PQ_slot0,
  FIELD_dsp340050b49a6c_fld2859PQ_slot0,
  FIELD_dsp340050b49a6c_fld2860PQ_slot0,
  FIELD_dsp340050b49a6c_fld2861PQ_slot0,
  FIELD_dsp340050b49a6c_fld2863PQ_slot0,
  FIELD_dsp340050b49a6c_fld2864PQ_slot0,
  FIELD_dsp340050b49a6c_fld2865,
  FIELD_dsp340050b49a6c_fld2867PQ_slot0,
  FIELD_dsp340050b49a6c_fld2869PQ_slot0,
  FIELD_dsp340050b49a6c_fld2871PQ_slot0,
  FIELD_dsp340050b49a6c_fld2872PQ_slot0,
  FIELD_dsp340050b49a6c_fld2873PQ_slot0,
  FIELD_dsp340050b49a6c_fld2874PQ_slot0,
  FIELD_dsp340050b49a6c_fld2875PQ_slot0,
  FIELD_dsp340050b49a6c_fld2876PQ_slot0,
  FIELD_dsp340050b49a6c_fld2877PQ_slot0,
  FIELD_dsp340050b49a6c_fld2878PQ_slot0,
  FIELD_dsp340050b49a6c_fld2879PQ_slot0,
  FIELD_dsp340050b49a6c_fld2880PQ_slot0,
  FIELD_dsp340050b49a6c_fld2881PQ_slot0,
  FIELD_dsp340050b49a6c_fld2882,
  FIELD_dsp340050b49a6c_fld2883PQ_slot0,
  FIELD_dsp340050b49a6c_fld2884PQ_slot0,
  FIELD_dsp340050b49a6c_fld2885PQ_slot0,
  FIELD_dsp340050b49a6c_fld2886PQ_slot0,
  FIELD_dsp340050b49a6c_fld2887PQ_slot0,
  FIELD_dsp340050b49a6c_fld2888PQ_slot0,
  FIELD_dsp340050b49a6c_fld2890PQ_slot0,
  FIELD_dsp340050b49a6c_fld2891PQ_slot0,
  FIELD_dsp340050b49a6c_fld2892PQ_slot0,
  FIELD_dsp340050b49a6c_fld2893PQ_slot0,
  FIELD_dsp340050b49a6c_fld2894PQ_slot0,
  FIELD_dsp340050b49a6c_fld2895PQ_slot0,
  FIELD_dsp340050b49a6c_fld2897PQ_slot0,
  FIELD_dsp340050b49a6c_fld2899PQ_slot0,
  FIELD_dsp340050b49a6c_fld2900PQ_slot0,
  FIELD_dsp340050b49a6c_fld2901PQ_slot0,
  FIELD_dsp340050b49a6c_fld2902PQ_slot0,
  FIELD_dsp340050b49a6c_fld2903PQ_slot0,
  FIELD_dsp340050b49a6c_fld2904PQ_slot0,
  FIELD_dsp340050b49a6c_fld2905PQ_slot0,
  FIELD_dsp340050b49a6c_fld2906PQ_slot0,
  FIELD_dsp340050b49a6c_fld2907PQ_slot0,
  FIELD_dsp340050b49a6c_fld2908PQ_slot0,
  FIELD_dsp340050b49a6c_fld2909PQ_slot0,
  FIELD_dsp340050b49a6c_fld2910PQ_slot0,
  FIELD_dsp340050b49a6c_fld2911PQ_slot0,
  FIELD_dsp340050b49a6c_fld2912PQ_slot0,
  FIELD_dsp340050b49a6c_fld2913PQ_slot0,
  FIELD_dsp340050b49a6c_fld2914PQ_slot0,
  FIELD_dsp340050b49a6c_fld2915PQ_slot0,
  FIELD_dsp340050b49a6c_fld2916PQ_slot0,
  FIELD_dsp340050b49a6c_fld2917PQ_slot0,
  FIELD_dsp340050b49a6c_fld2918PQ_slot0,
  FIELD_dsp340050b49a6c_fld2919PQ_slot0,
  FIELD_dsp340050b49a6c_fld2920PQ_slot0,
  FIELD_dsp340050b49a6c_fld2921PQ_slot0,
  FIELD_dsp340050b49a6c_fld2922PQ_slot0,
  FIELD_dsp340050b49a6c_fld2923PQ_slot0,
  FIELD_dsp340050b49a6c_fld2924PQ_slot0,
  FIELD_dsp340050b49a6c_fld2925PQ_slot0,
  FIELD_dsp340050b49a6c_fld2926PQ_slot0,
  FIELD_dsp340050b49a6c_fld2927PQ_slot0,
  FIELD_dsp340050b49a6c_fld2928PQ_slot0,
  FIELD_dsp340050b49a6c_fld2929PQ_slot0,
  FIELD_dsp340050b49a6c_fld2930PQ_slot0,
  FIELD_dsp340050b49a6c_fld2932PQ_slot0,
  FIELD_dsp340050b49a6c_fld2934PQ_slot0,
  FIELD_dsp340050b49a6c_fld2935PQ_slot0,
  FIELD_dsp340050b49a6c_fld2936PQ_slot0,
  FIELD_dsp340050b49a6c_fld2937PQ_slot0,
  FIELD_dsp340050b49a6c_fld2939PQ_slot0,
  FIELD_dsp340050b49a6c_fld2940,
  FIELD_dsp340050b49a6c_fld2941PQ_slot0,
  FIELD_dsp340050b49a6c_fld2942PQ_slot0,
  FIELD_dsp340050b49a6c_fld2943PQ_slot0,
  FIELD_dsp340050b49a6c_fld2945PQ_slot0,
  FIELD_dsp340050b49a6c_fld2946PQ_slot0,
  FIELD_dsp340050b49a6c_fld2947PQ_slot0,
  FIELD_dsp340050b49a6c_fld2948PQ_slot0,
  FIELD_dsp340050b49a6c_fld2949PQ_slot0,
  FIELD_dsp340050b49a6c_fld2950PQ_slot0,
  FIELD_dsp340050b49a6c_fld3763PQ_slot0,
  FIELD_dsp340050b49a6c_fld3764PQ_slot0,
  FIELD_dsp340050b49a6c_fld3765PQ_slot0,
  FIELD_dsp340050b49a6c_fld3766PQ_slot0,
  FIELD_dsp340050b49a6c_fld3767PQ_slot0,
  FIELD_dsp340050b49a6c_fld3768PQ_slot0,
  FIELD_dsp340050b49a6c_fld3769PQ_slot0,
  FIELD_dsp340050b49a6c_fld3770PQ_slot0,
  FIELD_dsp340050b49a6c_fld3771PQ_slot0,
  FIELD_dsp340050b49a6c_fld3772,
  FIELD_dsp340050b49a6c_fld3773PQ_slot0,
  FIELD_dsp340050b49a6c_fld3775PQ_slot0,
  FIELD_dsp340050b49a6c_fld3776PQ_slot0,
  FIELD_dsp340050b49a6c_fld3777PQ_slot0,
  FIELD_dsp340050b49a6c_fld3778PQ_slot0,
  FIELD_dsp340050b49a6c_fld3779PQ_slot0,
  FIELD_dsp340050b49a6c_fld3780PQ_slot0,
  FIELD_op0_s12,
  FIELD_dsp340050b49a6c_fld2953ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2954ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2955ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2956ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2957ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2958ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2959ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2960ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2963ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2964ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2966ACC2_slot2,
  FIELD_dsp340050b49a6c_fld2967ACC2_slot2,
  FIELD_dsp340050b49a6c_fld3782ACC2_slot2,
  FIELD_dsp340050b49a6c_fld3783ACC2_slot2,
  FIELD_dsp340050b49a6c_fld3784ACC2_slot2,
  FIELD_dsp340050b49a6c_fld3785ACC2_slot2,
  FIELD_dsp340050b49a6c_fld3786ACC2_slot2,
  FIELD_dsp340050b49a6c_fld3788ACC2_slot2,
  FIELD_op0_s13,
  FIELD_dsp340050b49a6c_fld2028,
  FIELD_dsp340050b49a6c_fld2075,
  FIELD_dsp340050b49a6c_fld2968ACC2_slot1,
  FIELD_dsp340050b49a6c_fld2969ACC2_slot1,
  FIELD_dsp340050b49a6c_fld3790ACC2_slot1,
  FIELD_dsp340050b49a6c_fld3793ACC2_slot1,
  FIELD_op0_s14,
  FIELD_dsp340050b49a6c_fld2973ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2974ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2975ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2976ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2977ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2980ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2981ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2982ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2984ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2985ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2987ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2989ACC2_slot0,
  FIELD_dsp340050b49a6c_fld2990ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3795ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3796ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3797,
  FIELD_dsp340050b49a6c_fld3798ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3799ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3800ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3801ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3802ACC2_slot0,
  FIELD_dsp340050b49a6c_fld3803ACC2_slot0,
  FIELD_op0_s15,
  FIELD_dsp340050b49a6c_fld2991SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2992SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2993SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2994SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2995SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2996SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2997SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2998SMOD_slot2,
  FIELD_dsp340050b49a6c_fld2999SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3000SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3001SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3002SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3003SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3004SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3005SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3006SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3007SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3008SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3009SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3010SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3011SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3012SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3013SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3014SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3015SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3016SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3017SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3018SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3019SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3020SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3021SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3022SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3023SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3024SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3025SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3026SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3027SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3028SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3030SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3031SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3032SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3033SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3034SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3035SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3036SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3038SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3039SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3040SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3043SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3044SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3046SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3047SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3048,
  FIELD_dsp340050b49a6c_fld3049SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3050SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3051SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3052SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3053SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3054SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3055SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3056SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3058SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3059SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3061SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3062,
  FIELD_dsp340050b49a6c_fld3063SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3065SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3066SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3067SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3068SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3069SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3070,
  FIELD_dsp340050b49a6c_fld3071SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3072SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3073SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3074SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3075SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3076SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3077SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3078SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3079SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3080SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3081SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3082SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3084SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3085SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3087SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3088SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3090SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3091SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3092SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3093SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3096SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3097SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3098SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3099SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3100SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3101SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3102SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3104SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3105SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3106SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3107SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3108SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3109SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3110SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3111SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3113SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3114SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3115SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3116SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3805SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3806SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3807SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3808,
  FIELD_dsp340050b49a6c_fld3809SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3810SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3812SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3813SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3814SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3816SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3817,
  FIELD_dsp340050b49a6c_fld3818SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3819SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3821SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3822SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3823SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3824SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3825SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3826SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3827SMOD_slot2,
  FIELD_dsp340050b49a6c_fld3828SMOD_slot2,
  FIELD_op0_s16,
  FIELD_dsp340050b49a6c_fld2033,
  FIELD_dsp340050b49a6c_fld2080,
  FIELD_dsp340050b49a6c_fld3117SMOD_slot1,
  FIELD_dsp340050b49a6c_fld3118SMOD_slot1,
  FIELD_dsp340050b49a6c_fld3829SMOD_slot1,
  FIELD_op0_s17,
  FIELD_dsp340050b49a6c_fld3119SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3120SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3121SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3122SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3123SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3125SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3126SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3127SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3128SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3129SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3130SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3131SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3132SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3133SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3134SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3135SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3136SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3137SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3138SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3139SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3140SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3141SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3142SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3143SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3144SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3145SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3146SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3148SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3149SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3150,
  FIELD_dsp340050b49a6c_fld3152SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3153SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3155SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3156SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3157SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3158SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3159SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3160SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3161SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3164SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3165SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3166SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3168SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3170SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3171SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3172SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3173SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3174SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3175SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3176SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3177SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3178SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3179SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3180SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3181SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3182SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3184SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3186SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3188SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3832SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3833SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3834,
  FIELD_dsp340050b49a6c_fld3836SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3837SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3838SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3841SMOD_slot0,
  FIELD_dsp340050b49a6c_fld3842SMOD_slot0,
  FIELD_op0_s18,
  FIELD_dsp340050b49a6c_fld2074,
  FIELD_dsp340050b49a6c_fld3191LLR_slot2,
  FIELD_dsp340050b49a6c_fld3192LLR_slot2,
  FIELD_dsp340050b49a6c_fld3193LLR_slot2,
  FIELD_dsp340050b49a6c_fld3194LLR_slot2,
  FIELD_dsp340050b49a6c_fld3195LLR_slot2,
  FIELD_dsp340050b49a6c_fld3196LLR_slot2,
  FIELD_dsp340050b49a6c_fld3197LLR_slot2,
  FIELD_dsp340050b49a6c_fld3198LLR_slot2,
  FIELD_dsp340050b49a6c_fld3199LLR_slot2,
  FIELD_dsp340050b49a6c_fld3200LLR_slot2,
  FIELD_dsp340050b49a6c_fld3201LLR_slot2,
  FIELD_dsp340050b49a6c_fld3202LLR_slot2,
  FIELD_dsp340050b49a6c_fld3203LLR_slot2,
  FIELD_dsp340050b49a6c_fld3204LLR_slot2,
  FIELD_dsp340050b49a6c_fld3205,
  FIELD_dsp340050b49a6c_fld3206,
  FIELD_dsp340050b49a6c_fld3207LLR_slot2,
  FIELD_dsp340050b49a6c_fld3208LLR_slot2,
  FIELD_dsp340050b49a6c_fld3210LLR_slot2,
  FIELD_dsp340050b49a6c_fld3212,
  FIELD_dsp340050b49a6c_fld3213LLR_slot2,
  FIELD_dsp340050b49a6c_fld3214,
  FIELD_dsp340050b49a6c_fld3215LLR_slot2,
  FIELD_dsp340050b49a6c_fld3216LLR_slot2,
  FIELD_dsp340050b49a6c_fld3217,
  FIELD_dsp340050b49a6c_fld3218LLR_slot2,
  FIELD_dsp340050b49a6c_fld3220LLR_slot2,
  FIELD_dsp340050b49a6c_fld3221LLR_slot2,
  FIELD_dsp340050b49a6c_fld3222LLR_slot2,
  FIELD_dsp340050b49a6c_fld3224LLR_slot2,
  FIELD_dsp340050b49a6c_fld3225,
  FIELD_dsp340050b49a6c_fld3226LLR_slot2,
  FIELD_dsp340050b49a6c_fld3228LLR_slot2,
  FIELD_dsp340050b49a6c_fld3230,
  FIELD_dsp340050b49a6c_fld3231LLR_slot2,
  FIELD_dsp340050b49a6c_fld3232LLR_slot2,
  FIELD_dsp340050b49a6c_fld3233,
  FIELD_dsp340050b49a6c_fld3234LLR_slot2,
  FIELD_dsp340050b49a6c_fld3235LLR_slot2,
  FIELD_dsp340050b49a6c_fld3236,
  FIELD_dsp340050b49a6c_fld3237LLR_slot2,
  FIELD_dsp340050b49a6c_fld3238LLR_slot2,
  FIELD_dsp340050b49a6c_fld3240LLR_slot2,
  FIELD_dsp340050b49a6c_fld3241LLR_slot2,
  FIELD_dsp340050b49a6c_fld3242LLR_slot2,
  FIELD_dsp340050b49a6c_fld3243LLR_slot2,
  FIELD_dsp340050b49a6c_fld3244LLR_slot2,
  FIELD_dsp340050b49a6c_fld3245LLR_slot2,
  FIELD_dsp340050b49a6c_fld3246,
  FIELD_dsp340050b49a6c_fld3247LLR_slot2,
  FIELD_dsp340050b49a6c_fld3843LLR_slot2,
  FIELD_dsp340050b49a6c_fld3844,
  FIELD_dsp340050b49a6c_fld3845LLR_slot2,
  FIELD_dsp340050b49a6c_fld3847LLR_slot2,
  FIELD_dsp340050b49a6c_fld3848LLR_slot2,
  FIELD_dsp340050b49a6c_fld3849LLR_slot2,
  FIELD_dsp340050b49a6c_fld3850LLR_slot2,
  FIELD_dsp340050b49a6c_fld3851LLR_slot2,
  FIELD_dsp340050b49a6c_fld3853LLR_slot2,
  FIELD_dsp340050b49a6c_fld3855LLR_slot2,
  FIELD_dsp340050b49a6c_fld3856LLR_slot2,
  FIELD_dsp340050b49a6c_fld3857LLR_slot2,
  FIELD_dsp340050b49a6c_fld3859LLR_slot2,
  FIELD_dsp340050b49a6c_fld3860LLR_slot2,
  FIELD_dsp340050b49a6c_fld3861LLR_slot2,
  FIELD_dsp340050b49a6c_fld3862,
  FIELD_dsp340050b49a6c_fld3863LLR_slot2,
  FIELD_dsp340050b49a6c_fld3864LLR_slot2,
  FIELD_dsp340050b49a6c_fld3865LLR_slot2,
  FIELD_dsp340050b49a6c_fld3866LLR_slot2,
  FIELD_dsp340050b49a6c_fld3867LLR_slot2,
  FIELD_dsp340050b49a6c_fld3868,
  FIELD_op0_s19,
  FIELD_dsp340050b49a6c_fld2034,
  FIELD_dsp340050b49a6c_fld3248LLR_slot1,
  FIELD_dsp340050b49a6c_fld3250LLR_slot1,
  FIELD_dsp340050b49a6c_fld3251LLR_slot1,
  FIELD_dsp340050b49a6c_fld3252LLR_slot1,
  FIELD_dsp340050b49a6c_fld3253LLR_slot1,
  FIELD_dsp340050b49a6c_fld3254LLR_slot1,
  FIELD_dsp340050b49a6c_fld3869LLR_slot1,
  FIELD_dsp340050b49a6c_fld3870,
  FIELD_dsp340050b49a6c_fld3872LLR_slot1,
  FIELD_dsp340050b49a6c_fld3875LLR_slot1,
  FIELD_dsp340050b49a6c_fld3876LLR_slot1,
  FIELD_dsp340050b49a6c_fld3878LLR_slot1,
  FIELD_op0_s20,
  FIELD_dsp340050b49a6c_fld2071,
  FIELD_dsp340050b49a6c_fld3258LLR_slot0,
  FIELD_dsp340050b49a6c_fld3259LLR_slot0,
  FIELD_dsp340050b49a6c_fld3260LLR_slot0,
  FIELD_dsp340050b49a6c_fld3261LLR_slot0,
  FIELD_dsp340050b49a6c_fld3263LLR_slot0,
  FIELD_dsp340050b49a6c_fld3264LLR_slot0,
  FIELD_dsp340050b49a6c_fld3265LLR_slot0,
  FIELD_dsp340050b49a6c_fld3266LLR_slot0,
  FIELD_dsp340050b49a6c_fld3267LLR_slot0,
  FIELD_dsp340050b49a6c_fld3268LLR_slot0,
  FIELD_dsp340050b49a6c_fld3269LLR_slot0,
  FIELD_dsp340050b49a6c_fld3270LLR_slot0,
  FIELD_dsp340050b49a6c_fld3272LLR_slot0,
  FIELD_dsp340050b49a6c_fld3274LLR_slot0,
  FIELD_dsp340050b49a6c_fld3275LLR_slot0,
  FIELD_dsp340050b49a6c_fld3276LLR_slot0,
  FIELD_dsp340050b49a6c_fld3277LLR_slot0,
  FIELD_dsp340050b49a6c_fld3278LLR_slot0,
  FIELD_dsp340050b49a6c_fld3279LLR_slot0,
  FIELD_dsp340050b49a6c_fld3280LLR_slot0,
  FIELD_dsp340050b49a6c_fld3281LLR_slot0,
  FIELD_dsp340050b49a6c_fld3282LLR_slot0,
  FIELD_dsp340050b49a6c_fld3283LLR_slot0,
  FIELD_dsp340050b49a6c_fld3284LLR_slot0,
  FIELD_dsp340050b49a6c_fld3286LLR_slot0,
  FIELD_dsp340050b49a6c_fld3288LLR_slot0,
  FIELD_dsp340050b49a6c_fld3289LLR_slot0,
  FIELD_dsp340050b49a6c_fld3291LLR_slot0,
  FIELD_dsp340050b49a6c_fld3292LLR_slot0,
  FIELD_dsp340050b49a6c_fld3293LLR_slot0,
  FIELD_dsp340050b49a6c_fld3294LLR_slot0,
  FIELD_dsp340050b49a6c_fld3295LLR_slot0,
  FIELD_dsp340050b49a6c_fld3296LLR_slot0,
  FIELD_dsp340050b49a6c_fld3297LLR_slot0,
  FIELD_dsp340050b49a6c_fld3298LLR_slot0,
  FIELD_dsp340050b49a6c_fld3299LLR_slot0,
  FIELD_dsp340050b49a6c_fld3300LLR_slot0,
  FIELD_dsp340050b49a6c_fld3302LLR_slot0,
  FIELD_dsp340050b49a6c_fld3303LLR_slot0,
  FIELD_dsp340050b49a6c_fld3304LLR_slot0,
  FIELD_dsp340050b49a6c_fld3305LLR_slot0,
  FIELD_dsp340050b49a6c_fld3306LLR_slot0,
  FIELD_dsp340050b49a6c_fld3308LLR_slot0,
  FIELD_dsp340050b49a6c_fld3310LLR_slot0,
  FIELD_dsp340050b49a6c_fld3311LLR_slot0,
  FIELD_dsp340050b49a6c_fld3312LLR_slot0,
  FIELD_dsp340050b49a6c_fld3879LLR_slot0,
  FIELD_dsp340050b49a6c_fld3881LLR_slot0,
  FIELD_dsp340050b49a6c_fld3883LLR_slot0,
  FIELD_dsp340050b49a6c_fld3885LLR_slot0,
  FIELD_dsp340050b49a6c_fld3887LLR_slot0,
  FIELD_dsp340050b49a6c_fld3888LLR_slot0,
  FIELD_dsp340050b49a6c_fld3890LLR_slot0,
  FIELD_dsp340050b49a6c_fld3892LLR_slot0,
  FIELD_dsp340050b49a6c_fld3893LLR_slot0,
  FIELD_op0_s21,
  FIELD_dsp340050b49a6c_fld3313DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3314,
  FIELD_dsp340050b49a6c_fld3315DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3316DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3317DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3318,
  FIELD_dsp340050b49a6c_fld3319DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3320DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3321DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3322DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3323DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3324DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3325DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3326DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3327DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3328DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3329DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3330DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3331DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3332DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3333DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3334DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3335DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3336DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3337DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3339DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3340DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3341DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3342DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3345DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3347DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3348DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3349DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3350DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3353DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3354DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3356DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3358DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3360DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3361DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3362DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3363DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3364,
  FIELD_dsp340050b49a6c_fld3365DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3366DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3367DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3368DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3369DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3370DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3371DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3372DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3373DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3374DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3375DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3376DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3377DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3378DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3379DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3380DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3381DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3382DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3384DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3385DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3386DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3387DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3388DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3390DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3392DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3394DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3396DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3397DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3399DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3401DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3403DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3404DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3406DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3407,
  FIELD_dsp340050b49a6c_fld3408DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3410,
  FIELD_dsp340050b49a6c_fld3411DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3412DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3413DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3414DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3415DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3416DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3417DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3418DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3419DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3420DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3421DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3422DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3423DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3424DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3425DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3426DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3427DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3428DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3429DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3430DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3431DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3432DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3433DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3434DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3435DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3436DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3437DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3438DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3439DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3440DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3441DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3442DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3443DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3444DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3445DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3446DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3448DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3450DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3451DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3453DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3454DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3456DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3457DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3458DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3459DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3460DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3461DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3462DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3464DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3465DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3466,
  FIELD_dsp340050b49a6c_fld3467DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3468DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3469DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3470DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3471DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3472DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3473DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3474DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3475DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3477DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3478DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3479DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3480DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3481DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3482DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3484DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3894DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3895DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3896DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3897DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3898DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3899DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3900,
  FIELD_dsp340050b49a6c_fld3901DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3903DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3904DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3905DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3906DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3907DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3908DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3909DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3910DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3913DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3914DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3916DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3917DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3918DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3919DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3920DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3921DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3922DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3923DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3924DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3925DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3927DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3928DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3929DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3930DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3931DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3933DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3934DUAL_slot2,
  FIELD_dsp340050b49a6c_fld3935DUAL_slot2,
  FIELD_op0_s22,
  FIELD_op0_s23,
  FIELD_dsp340050b49a6c_fld2057,
  FIELD_dsp340050b49a6c_fld2060,
  FIELD_dsp340050b49a6c_fld2066,
  FIELD_dsp340050b49a6c_fld2072,
  FIELD_dsp340050b49a6c_fld2079,
  FIELD_dsp340050b49a6c_fld3487DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3488DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3489DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3490DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3491DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3492DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3493DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3494DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3496DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3497DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3498DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3499DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3500DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3502DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3504DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3505DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3506DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3507DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3508DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3509DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3510DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3511DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3512DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3513DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3514DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3515DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3516DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3517DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3518DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3519DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3520DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3522DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3523DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3524DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3527DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3529DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3530DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3531,
  FIELD_dsp340050b49a6c_fld3532DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3533DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3535DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3536DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3537DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3538DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3539DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3541DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3542DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3543DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3544DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3545DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3546DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3547DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3548DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3549DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3550DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3551DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3552,
  FIELD_dsp340050b49a6c_fld3553DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3554DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3555DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3556DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3557DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3558DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3559DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3560DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3562DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3563DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3564DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3565DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3566DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3567DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3568DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3569DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3570DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3571DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3572DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3573DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3574DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3575DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3576DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3577DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3578DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3579DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3580DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3581DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3582DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3583DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3584,
  FIELD_dsp340050b49a6c_fld3585DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3587DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3588DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3589DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3590DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3591DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3592DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3593DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3594DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3595DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3596DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3597DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3598DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3599DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3600DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3601DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3602,
  FIELD_dsp340050b49a6c_fld3603DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3604DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3606DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3607DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3608DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3609DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3610,
  FIELD_dsp340050b49a6c_fld3611DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3612DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3613DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3614DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3615DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3616DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3618DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3619,
  FIELD_dsp340050b49a6c_fld3620DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3621DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3622DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3623DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3624DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3625DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3626DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3936DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3937DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3938DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3939DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3940DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3941DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3943DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3945DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3946DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3947DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3949DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3950DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3951DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3952DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3954DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3957DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3958DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3959DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3960DUAL_slot0,
  FIELD_dsp340050b49a6c_fld3961DUAL_slot0,
  FIELD__ar0,
  FIELD__ar4,
  FIELD__ar8,
  FIELD__ar12,
  FIELD__bt16,
  FIELD__bs16,
  FIELD__br16,
  FIELD__brall
};


/* Functional units.  */

static xtensa_funcUnit_internal funcUnits[] = {

};


/* Register files.  */

enum xtensa_regfile_id {
  REGFILE_AR,
  REGFILE_BR,
  REGFILE_FR,
  REGFILE_ACU,
  REGFILE_CM,
  REGFILE_PQ,
  REGFILE_BR2,
  REGFILE_BR4,
  REGFILE_BR8,
  REGFILE_BR16
};

static xtensa_regfile_internal regfiles[] = {
  { "AR", "a", REGFILE_AR, 32, 32 },
  { "BR", "b", REGFILE_BR, 1, 16 },
  { "FR", "f", REGFILE_FR, 32, 16 },
  { "ACU", "ACU", REGFILE_ACU, 320, 8 },
  { "CM", "CM", REGFILE_CM, 128, 16 },
  { "PQ", "PQ", REGFILE_PQ, 256, 16 },
  { "BR2", "b", REGFILE_BR, 2, 8 },
  { "BR4", "b", REGFILE_BR, 4, 4 },
  { "BR8", "b", REGFILE_BR, 8, 2 },
  { "BR16", "b", REGFILE_BR, 16, 1 }
};


/* Interfaces.  */

static xtensa_interface_internal interfaces[] = {
  { "INQ0_32_Empty", 1, 0, 0, 'i' },
  { "INQ0_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 0, 'i' },
  { "INQ0_32_NOTRDY", 1, 0, 0, 'i' },
  { "INQ0_32_KILL", 1, 0, 0, 'o' },
  { "INQ1_32_Empty", 1, 0, 1, 'i' },
  { "INQ1_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 1, 'i' },
  { "INQ1_32_NOTRDY", 1, 0, 1, 'i' },
  { "INQ1_32_KILL", 1, 0, 1, 'o' },
  { "INQ2_32_Empty", 1, 0, 2, 'i' },
  { "INQ2_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 2, 'i' },
  { "INQ2_32_NOTRDY", 1, 0, 2, 'i' },
  { "INQ2_32_KILL", 1, 0, 2, 'o' },
  { "INQ3_32_Empty", 1, 0, 3, 'i' },
  { "INQ3_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 3, 'i' },
  { "INQ3_32_NOTRDY", 1, 0, 3, 'i' },
  { "INQ3_32_KILL", 1, 0, 3, 'o' },
  { "OUTQ0_32_Full", 1, 0, 4, 'i' },
  { "OUTQ0_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 4, 'o' },
  { "OUTQ0_32_NOTRDY", 1, 0, 4, 'i' },
  { "OUTQ0_32_KILL", 1, 0, 4, 'o' },
  { "OUTQ1_32_Full", 1, 0, 5, 'i' },
  { "OUTQ1_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 5, 'o' },
  { "OUTQ1_32_NOTRDY", 1, 0, 5, 'i' },
  { "OUTQ1_32_KILL", 1, 0, 5, 'o' },
  { "OUTQ2_32_Full", 1, 0, 6, 'i' },
  { "OUTQ2_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 6, 'o' },
  { "OUTQ2_32_NOTRDY", 1, 0, 6, 'i' },
  { "OUTQ2_32_KILL", 1, 0, 6, 'o' },
  { "OUTQ3_32_Full", 1, 0, 7, 'i' },
  { "OUTQ3_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 7, 'o' },
  { "OUTQ3_32_NOTRDY", 1, 0, 7, 'i' },
  { "OUTQ3_32_KILL", 1, 0, 7, 'o' },
  { "OUTQ4_32_Full", 1, 0, 8, 'i' },
  { "OUTQ4_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 8, 'o' },
  { "OUTQ4_32_NOTRDY", 1, 0, 8, 'i' },
  { "OUTQ4_32_KILL", 1, 0, 8, 'o' },
  { "OUTQ5_32_Full", 1, 0, 9, 'i' },
  { "OUTQ5_32", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 9, 'o' },
  { "OUTQ5_32_NOTRDY", 1, 0, 9, 'i' },
  { "OUTQ5_32_KILL", 1, 0, 9, 'o' },
  { "SIGNALQ_Full", 1, 0, 10, 'i' },
  { "SIGNALQ", 32, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 10, 'o' },
  { "SIGNALQ_NOTRDY", 1, 0, 10, 'i' },
  { "SIGNALQ_KILL", 1, 0, 10, 'o' },
  { "INQ0_128_Empty", 1, 0, 11, 'i' },
  { "INQ0_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 11, 'i' },
  { "INQ0_128_NOTRDY", 1, 0, 11, 'i' },
  { "INQ0_128_KILL", 1, 0, 11, 'o' },
  { "INQ1_128_Empty", 1, 0, 12, 'i' },
  { "INQ1_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 12, 'i' },
  { "INQ1_128_NOTRDY", 1, 0, 12, 'i' },
  { "INQ1_128_KILL", 1, 0, 12, 'o' },
  { "INQ2_128_Empty", 1, 0, 13, 'i' },
  { "INQ2_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 13, 'i' },
  { "INQ2_128_NOTRDY", 1, 0, 13, 'i' },
  { "INQ2_128_KILL", 1, 0, 13, 'o' },
  { "INQ3_128_Empty", 1, 0, 14, 'i' },
  { "INQ3_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 14, 'i' },
  { "INQ3_128_NOTRDY", 1, 0, 14, 'i' },
  { "INQ3_128_KILL", 1, 0, 14, 'o' },
  { "INQ4_128_Empty", 1, 0, 15, 'i' },
  { "INQ4_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 15, 'i' },
  { "INQ4_128_NOTRDY", 1, 0, 15, 'i' },
  { "INQ4_128_KILL", 1, 0, 15, 'o' },
  { "INQ5_128_Empty", 1, 0, 16, 'i' },
  { "INQ5_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 16, 'i' },
  { "INQ5_128_NOTRDY", 1, 0, 16, 'i' },
  { "INQ5_128_KILL", 1, 0, 16, 'o' },
  { "OUTQ0_128_Full", 1, 0, 17, 'i' },
  { "OUTQ0_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 17, 'o' },
  { "OUTQ0_128_NOTRDY", 1, 0, 17, 'i' },
  { "OUTQ0_128_KILL", 1, 0, 17, 'o' },
  { "OUTQ1_128_Full", 1, 0, 18, 'i' },
  { "OUTQ1_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 18, 'o' },
  { "OUTQ1_128_NOTRDY", 1, 0, 18, 'i' },
  { "OUTQ1_128_KILL", 1, 0, 18, 'o' },
  { "OUTQ2_128_Full", 1, 0, 19, 'i' },
  { "OUTQ2_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 19, 'o' },
  { "OUTQ2_128_NOTRDY", 1, 0, 19, 'i' },
  { "OUTQ2_128_KILL", 1, 0, 19, 'o' },
  { "OUTQ3_128_Full", 1, 0, 20, 'i' },
  { "OUTQ3_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 20, 'o' },
  { "OUTQ3_128_NOTRDY", 1, 0, 20, 'i' },
  { "OUTQ3_128_KILL", 1, 0, 20, 'o' },
  { "OUTQ4_128_Full", 1, 0, 21, 'i' },
  { "OUTQ4_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 21, 'o' },
  { "OUTQ4_128_NOTRDY", 1, 0, 21, 'i' },
  { "OUTQ4_128_KILL", 1, 0, 21, 'o' },
  { "OUTQ5_128_Full", 1, 0, 22, 'i' },
  { "OUTQ5_128", 128, XTENSA_INTERFACE_HAS_SIDE_EFFECT, 22, 'o' },
  { "OUTQ5_128_NOTRDY", 1, 0, 22, 'i' },
  { "OUTQ5_128_KILL", 1, 0, 22, 'o' },
  { "IMPWIRE", 32, 0, 23, 'i' },
  { "LU128_Out", 128, 0, 24, 'o' },
  { "LU128_In", 128, 0, 25, 'i' }
};

enum xtensa_interface_id {
  INTERFACE_INQ0_32_Empty,
  INTERFACE_INQ0_32,
  INTERFACE_INQ0_32_NOTRDY,
  INTERFACE_INQ0_32_KILL,
  INTERFACE_INQ1_32_Empty,
  INTERFACE_INQ1_32,
  INTERFACE_INQ1_32_NOTRDY,
  INTERFACE_INQ1_32_KILL,
  INTERFACE_INQ2_32_Empty,
  INTERFACE_INQ2_32,
  INTERFACE_INQ2_32_NOTRDY,
  INTERFACE_INQ2_32_KILL,
  INTERFACE_INQ3_32_Empty,
  INTERFACE_INQ3_32,
  INTERFACE_INQ3_32_NOTRDY,
  INTERFACE_INQ3_32_KILL,
  INTERFACE_OUTQ0_32_Full,
  INTERFACE_OUTQ0_32,
  INTERFACE_OUTQ0_32_NOTRDY,
  INTERFACE_OUTQ0_32_KILL,
  INTERFACE_OUTQ1_32_Full,
  INTERFACE_OUTQ1_32,
  INTERFACE_OUTQ1_32_NOTRDY,
  INTERFACE_OUTQ1_32_KILL,
  INTERFACE_OUTQ2_32_Full,
  INTERFACE_OUTQ2_32,
  INTERFACE_OUTQ2_32_NOTRDY,
  INTERFACE_OUTQ2_32_KILL,
  INTERFACE_OUTQ3_32_Full,
  INTERFACE_OUTQ3_32,
  INTERFACE_OUTQ3_32_NOTRDY,
  INTERFACE_OUTQ3_32_KILL,
  INTERFACE_OUTQ4_32_Full,
  INTERFACE_OUTQ4_32,
  INTERFACE_OUTQ4_32_NOTRDY,
  INTERFACE_OUTQ4_32_KILL,
  INTERFACE_OUTQ5_32_Full,
  INTERFACE_OUTQ5_32,
  INTERFACE_OUTQ5_32_NOTRDY,
  INTERFACE_OUTQ5_32_KILL,
  INTERFACE_SIGNALQ_Full,
  INTERFACE_SIGNALQ,
  INTERFACE_SIGNALQ_NOTRDY,
  INTERFACE_SIGNALQ_KILL,
  INTERFACE_INQ0_128_Empty,
  INTERFACE_INQ0_128,
  INTERFACE_INQ0_128_NOTRDY,
  INTERFACE_INQ0_128_KILL,
  INTERFACE_INQ1_128_Empty,
  INTERFACE_INQ1_128,
  INTERFACE_INQ1_128_NOTRDY,
  INTERFACE_INQ1_128_KILL,
  INTERFACE_INQ2_128_Empty,
  INTERFACE_INQ2_128,
  INTERFACE_INQ2_128_NOTRDY,
  INTERFACE_INQ2_128_KILL,
  INTERFACE_INQ3_128_Empty,
  INTERFACE_INQ3_128,
  INTERFACE_INQ3_128_NOTRDY,
  INTERFACE_INQ3_128_KILL,
  INTERFACE_INQ4_128_Empty,
  INTERFACE_INQ4_128,
  INTERFACE_INQ4_128_NOTRDY,
  INTERFACE_INQ4_128_KILL,
  INTERFACE_INQ5_128_Empty,
  INTERFACE_INQ5_128,
  INTERFACE_INQ5_128_NOTRDY,
  INTERFACE_INQ5_128_KILL,
  INTERFACE_OUTQ0_128_Full,
  INTERFACE_OUTQ0_128,
  INTERFACE_OUTQ0_128_NOTRDY,
  INTERFACE_OUTQ0_128_KILL,
  INTERFACE_OUTQ1_128_Full,
  INTERFACE_OUTQ1_128,
  INTERFACE_OUTQ1_128_NOTRDY,
  INTERFACE_OUTQ1_128_KILL,
  INTERFACE_OUTQ2_128_Full,
  INTERFACE_OUTQ2_128,
  INTERFACE_OUTQ2_128_NOTRDY,
  INTERFACE_OUTQ2_128_KILL,
  INTERFACE_OUTQ3_128_Full,
  INTERFACE_OUTQ3_128,
  INTERFACE_OUTQ3_128_NOTRDY,
  INTERFACE_OUTQ3_128_KILL,
  INTERFACE_OUTQ4_128_Full,
  INTERFACE_OUTQ4_128,
  INTERFACE_OUTQ4_128_NOTRDY,
  INTERFACE_OUTQ4_128_KILL,
  INTERFACE_OUTQ5_128_Full,
  INTERFACE_OUTQ5_128,
  INTERFACE_OUTQ5_128_NOTRDY,
  INTERFACE_OUTQ5_128_KILL,
  INTERFACE_IMPWIRE,
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};


/* Constant tables.  */

/* constant table ai4c */
static const unsigned CONST_TBL_ai4c_0[] = {
  0xffffffff,
  0x1,
  0x2,
  0x3,
  0x4,
  0x5,
  0x6,
  0x7,
  0x8,
  0x9,
  0xa,
  0xb,
  0xc,
  0xd,
  0xe,
  0xf,
  0
};

/* constant table b4c */
static const unsigned CONST_TBL_b4c_0[] = {
  0xffffffff,
  0x1,
  0x2,
  0x3,
  0x4,
  0x5,
  0x6,
  0x7,
  0x8,
  0xa,
  0xc,
  0x10,
  0x20,
  0x40,
  0x80,
  0x100,
  0
};

/* constant table b4cu */
static const unsigned CONST_TBL_b4cu_0[] = {
  0x8000,
  0x10000,
  0x2,
  0x3,
  0x4,
  0x5,
  0x6,
  0x7,
  0x8,
  0xa,
  0xc,
  0x10,
  0x20,
  0x40,
  0x80,
  0x100,
  0
};

/* constant table LLR_CONSTELLATION_TABLE */
static const unsigned CONST_TBL_LLR_CONSTELLATION_TABLE_0[] = {
  0x1 & 0xf,
  0x2 & 0xf,
  0x8 & 0xf,
  0x4 & 0xf,
  0x8 & 0xf,
  0x6 & 0xf,
  0x8 & 0xf,
  0x8 & 0xf,
  0
};


/* Instruction operands.  */

static int
Operand_soffsetx4_decode (uint32 *valp)
{
  unsigned soffsetx4_0, offset_0;
  offset_0 = *valp & 0x3ffff;
  soffsetx4_0 = 0x4 + ((((int) offset_0 << 14) >> 14) << 2);
  *valp = soffsetx4_0;
  return 0;
}

static int
Operand_soffsetx4_encode (uint32 *valp)
{
  unsigned offset_0, soffsetx4_0;
  soffsetx4_0 = *valp;
  offset_0 = ((soffsetx4_0 - 0x4) >> 2) & 0x3ffff;
  *valp = offset_0;
  return 0;
}

static int
Operand_soffsetx4_ator (uint32 *valp, uint32 pc)
{
  *valp -= (pc & ~0x3);
  return 0;
}

static int
Operand_soffsetx4_rtoa (uint32 *valp, uint32 pc)
{
  *valp += (pc & ~0x3);
  return 0;
}

static int
Operand_uimm12x8_decode (uint32 *valp)
{
  unsigned uimm12x8_0, imm12_0;
  imm12_0 = *valp & 0xfff;
  uimm12x8_0 = imm12_0 << 3;
  *valp = uimm12x8_0;
  return 0;
}

static int
Operand_uimm12x8_encode (uint32 *valp)
{
  unsigned imm12_0, uimm12x8_0;
  uimm12x8_0 = *valp;
  imm12_0 = ((uimm12x8_0 >> 3) & 0xfff);
  *valp = imm12_0;
  return 0;
}

static int
Operand_simm4_decode (uint32 *valp)
{
  unsigned simm4_0, mn_0;
  mn_0 = *valp & 0xf;
  simm4_0 = ((int) mn_0 << 28) >> 28;
  *valp = simm4_0;
  return 0;
}

static int
Operand_simm4_encode (uint32 *valp)
{
  unsigned mn_0, simm4_0;
  simm4_0 = *valp;
  mn_0 = (simm4_0 & 0xf);
  *valp = mn_0;
  return 0;
}

static int
Operand_arr_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_arr_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_ars_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_ars_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_art_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_art_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_ar0_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_ar0_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x1f) != 0;
  return error;
}

static int
Operand_ar4_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_ar4_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x1f) != 0;
  return error;
}

static int
Operand_ar8_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_ar8_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x1f) != 0;
  return error;
}

static int
Operand_ar12_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_ar12_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x1f) != 0;
  return error;
}

static int
Operand_ars_entry_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_ars_entry_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x1f) != 0;
  return error;
}

static int
Operand_immrx4_decode (uint32 *valp)
{
  unsigned immrx4_0, r_0;
  r_0 = *valp & 0xf;
  immrx4_0 = (((0xfffffff) << 4) | r_0) << 2;
  *valp = immrx4_0;
  return 0;
}

static int
Operand_immrx4_encode (uint32 *valp)
{
  unsigned r_0, immrx4_0;
  immrx4_0 = *valp;
  r_0 = ((immrx4_0 >> 2) & 0xf);
  *valp = r_0;
  return 0;
}

static int
Operand_lsi4x4_decode (uint32 *valp)
{
  unsigned lsi4x4_0, r_0;
  r_0 = *valp & 0xf;
  lsi4x4_0 = r_0 << 2;
  *valp = lsi4x4_0;
  return 0;
}

static int
Operand_lsi4x4_encode (uint32 *valp)
{
  unsigned r_0, lsi4x4_0;
  lsi4x4_0 = *valp;
  r_0 = ((lsi4x4_0 >> 2) & 0xf);
  *valp = r_0;
  return 0;
}

static int
Operand_simm7_decode (uint32 *valp)
{
  unsigned simm7_0, imm7_0;
  imm7_0 = *valp & 0x7f;
  simm7_0 = ((((-((((imm7_0 >> 6) & 1)) & (((imm7_0 >> 5) & 1)))) & 0x1ffffff)) << 7) | imm7_0;
  *valp = simm7_0;
  return 0;
}

static int
Operand_simm7_encode (uint32 *valp)
{
  unsigned imm7_0, simm7_0;
  simm7_0 = *valp;
  imm7_0 = (simm7_0 & 0x7f);
  *valp = imm7_0;
  return 0;
}

static int
Operand_uimm6_decode (uint32 *valp)
{
  unsigned uimm6_0, imm6_0;
  imm6_0 = *valp & 0x3f;
  uimm6_0 = 0x4 + (((0) << 6) | imm6_0);
  *valp = uimm6_0;
  return 0;
}

static int
Operand_uimm6_encode (uint32 *valp)
{
  unsigned imm6_0, uimm6_0;
  uimm6_0 = *valp;
  imm6_0 = (uimm6_0 - 0x4) & 0x3f;
  *valp = imm6_0;
  return 0;
}

static int
Operand_uimm6_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_uimm6_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_ai4const_decode (uint32 *valp)
{
  unsigned ai4const_0, t_0;
  t_0 = *valp & 0xf;
  ai4const_0 = CONST_TBL_ai4c_0[t_0 & 0xf];
  *valp = ai4const_0;
  return 0;
}

static int
Operand_ai4const_encode (uint32 *valp)
{
  unsigned t_0, ai4const_0;
  ai4const_0 = *valp;
  switch (ai4const_0)
    {
    case 0xffffffff: t_0 = 0; break;
    case 0x1: t_0 = 0x1; break;
    case 0x2: t_0 = 0x2; break;
    case 0x3: t_0 = 0x3; break;
    case 0x4: t_0 = 0x4; break;
    case 0x5: t_0 = 0x5; break;
    case 0x6: t_0 = 0x6; break;
    case 0x7: t_0 = 0x7; break;
    case 0x8: t_0 = 0x8; break;
    case 0x9: t_0 = 0x9; break;
    case 0xa: t_0 = 0xa; break;
    case 0xb: t_0 = 0xb; break;
    case 0xc: t_0 = 0xc; break;
    case 0xd: t_0 = 0xd; break;
    case 0xe: t_0 = 0xe; break;
    default: t_0 = 0xf; break;
    }
  *valp = t_0;
  return 0;
}

static int
Operand_b4const_decode (uint32 *valp)
{
  unsigned b4const_0, r_0;
  r_0 = *valp & 0xf;
  b4const_0 = CONST_TBL_b4c_0[r_0 & 0xf];
  *valp = b4const_0;
  return 0;
}

static int
Operand_b4const_encode (uint32 *valp)
{
  unsigned r_0, b4const_0;
  b4const_0 = *valp;
  switch (b4const_0)
    {
    case 0xffffffff: r_0 = 0; break;
    case 0x1: r_0 = 0x1; break;
    case 0x2: r_0 = 0x2; break;
    case 0x3: r_0 = 0x3; break;
    case 0x4: r_0 = 0x4; break;
    case 0x5: r_0 = 0x5; break;
    case 0x6: r_0 = 0x6; break;
    case 0x7: r_0 = 0x7; break;
    case 0x8: r_0 = 0x8; break;
    case 0xa: r_0 = 0x9; break;
    case 0xc: r_0 = 0xa; break;
    case 0x10: r_0 = 0xb; break;
    case 0x20: r_0 = 0xc; break;
    case 0x40: r_0 = 0xd; break;
    case 0x80: r_0 = 0xe; break;
    default: r_0 = 0xf; break;
    }
  *valp = r_0;
  return 0;
}

static int
Operand_b4constu_decode (uint32 *valp)
{
  unsigned b4constu_0, r_0;
  r_0 = *valp & 0xf;
  b4constu_0 = CONST_TBL_b4cu_0[r_0 & 0xf];
  *valp = b4constu_0;
  return 0;
}

static int
Operand_b4constu_encode (uint32 *valp)
{
  unsigned r_0, b4constu_0;
  b4constu_0 = *valp;
  switch (b4constu_0)
    {
    case 0x8000: r_0 = 0; break;
    case 0x10000: r_0 = 0x1; break;
    case 0x2: r_0 = 0x2; break;
    case 0x3: r_0 = 0x3; break;
    case 0x4: r_0 = 0x4; break;
    case 0x5: r_0 = 0x5; break;
    case 0x6: r_0 = 0x6; break;
    case 0x7: r_0 = 0x7; break;
    case 0x8: r_0 = 0x8; break;
    case 0xa: r_0 = 0x9; break;
    case 0xc: r_0 = 0xa; break;
    case 0x10: r_0 = 0xb; break;
    case 0x20: r_0 = 0xc; break;
    case 0x40: r_0 = 0xd; break;
    case 0x80: r_0 = 0xe; break;
    default: r_0 = 0xf; break;
    }
  *valp = r_0;
  return 0;
}

static int
Operand_uimm8_decode (uint32 *valp)
{
  unsigned uimm8_0, imm8_0;
  imm8_0 = *valp & 0xff;
  uimm8_0 = imm8_0;
  *valp = uimm8_0;
  return 0;
}

static int
Operand_uimm8_encode (uint32 *valp)
{
  unsigned imm8_0, uimm8_0;
  uimm8_0 = *valp;
  imm8_0 = (uimm8_0 & 0xff);
  *valp = imm8_0;
  return 0;
}

static int
Operand_uimm8x2_decode (uint32 *valp)
{
  unsigned uimm8x2_0, imm8_0;
  imm8_0 = *valp & 0xff;
  uimm8x2_0 = imm8_0 << 1;
  *valp = uimm8x2_0;
  return 0;
}

static int
Operand_uimm8x2_encode (uint32 *valp)
{
  unsigned imm8_0, uimm8x2_0;
  uimm8x2_0 = *valp;
  imm8_0 = ((uimm8x2_0 >> 1) & 0xff);
  *valp = imm8_0;
  return 0;
}

static int
Operand_uimm8x4_decode (uint32 *valp)
{
  unsigned uimm8x4_0, imm8_0;
  imm8_0 = *valp & 0xff;
  uimm8x4_0 = imm8_0 << 2;
  *valp = uimm8x4_0;
  return 0;
}

static int
Operand_uimm8x4_encode (uint32 *valp)
{
  unsigned imm8_0, uimm8x4_0;
  uimm8x4_0 = *valp;
  imm8_0 = ((uimm8x4_0 >> 2) & 0xff);
  *valp = imm8_0;
  return 0;
}

static int
Operand_uimm4x16_decode (uint32 *valp)
{
  unsigned uimm4x16_0, op2_0;
  op2_0 = *valp & 0xf;
  uimm4x16_0 = op2_0 << 4;
  *valp = uimm4x16_0;
  return 0;
}

static int
Operand_uimm4x16_encode (uint32 *valp)
{
  unsigned op2_0, uimm4x16_0;
  uimm4x16_0 = *valp;
  op2_0 = ((uimm4x16_0 >> 4) & 0xf);
  *valp = op2_0;
  return 0;
}

static int
Operand_simm8_decode (uint32 *valp)
{
  unsigned simm8_0, imm8_0;
  imm8_0 = *valp & 0xff;
  simm8_0 = ((int) imm8_0 << 24) >> 24;
  *valp = simm8_0;
  return 0;
}

static int
Operand_simm8_encode (uint32 *valp)
{
  unsigned imm8_0, simm8_0;
  simm8_0 = *valp;
  imm8_0 = (simm8_0 & 0xff);
  *valp = imm8_0;
  return 0;
}

static int
Operand_simm8x256_decode (uint32 *valp)
{
  unsigned simm8x256_0, imm8_0;
  imm8_0 = *valp & 0xff;
  simm8x256_0 = (((int) imm8_0 << 24) >> 24) << 8;
  *valp = simm8x256_0;
  return 0;
}

static int
Operand_simm8x256_encode (uint32 *valp)
{
  unsigned imm8_0, simm8x256_0;
  simm8x256_0 = *valp;
  imm8_0 = ((simm8x256_0 >> 8) & 0xff);
  *valp = imm8_0;
  return 0;
}

static int
Operand_simm12b_decode (uint32 *valp)
{
  unsigned simm12b_0, imm12b_0;
  imm12b_0 = *valp & 0xfff;
  simm12b_0 = ((int) imm12b_0 << 20) >> 20;
  *valp = simm12b_0;
  return 0;
}

static int
Operand_simm12b_encode (uint32 *valp)
{
  unsigned imm12b_0, simm12b_0;
  simm12b_0 = *valp;
  imm12b_0 = (simm12b_0 & 0xfff);
  *valp = imm12b_0;
  return 0;
}

static int
Operand_msalp32_decode (uint32 *valp)
{
  unsigned msalp32_0, sal_0;
  sal_0 = *valp & 0x1f;
  msalp32_0 = 0x20 - sal_0;
  *valp = msalp32_0;
  return 0;
}

static int
Operand_msalp32_encode (uint32 *valp)
{
  unsigned sal_0, msalp32_0;
  msalp32_0 = *valp;
  sal_0 = (0x20 - msalp32_0) & 0x1f;
  *valp = sal_0;
  return 0;
}

static int
Operand_op2p1_decode (uint32 *valp)
{
  unsigned op2p1_0, op2_0;
  op2_0 = *valp & 0xf;
  op2p1_0 = op2_0 + 0x1;
  *valp = op2p1_0;
  return 0;
}

static int
Operand_op2p1_encode (uint32 *valp)
{
  unsigned op2_0, op2p1_0;
  op2p1_0 = *valp;
  op2_0 = (op2p1_0 - 0x1) & 0xf;
  *valp = op2_0;
  return 0;
}

static int
Operand_label8_decode (uint32 *valp)
{
  unsigned label8_0, imm8_0;
  imm8_0 = *valp & 0xff;
  label8_0 = 0x4 + (((int) imm8_0 << 24) >> 24);
  *valp = label8_0;
  return 0;
}

static int
Operand_label8_encode (uint32 *valp)
{
  unsigned imm8_0, label8_0;
  label8_0 = *valp;
  imm8_0 = (label8_0 - 0x4) & 0xff;
  *valp = imm8_0;
  return 0;
}

static int
Operand_label8_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_label8_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_ulabel8_decode (uint32 *valp)
{
  unsigned ulabel8_0, imm8_0;
  imm8_0 = *valp & 0xff;
  ulabel8_0 = 0x4 + (((0) << 8) | imm8_0);
  *valp = ulabel8_0;
  return 0;
}

static int
Operand_ulabel8_encode (uint32 *valp)
{
  unsigned imm8_0, ulabel8_0;
  ulabel8_0 = *valp;
  imm8_0 = (ulabel8_0 - 0x4) & 0xff;
  *valp = imm8_0;
  return 0;
}

static int
Operand_ulabel8_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_ulabel8_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_label12_decode (uint32 *valp)
{
  unsigned label12_0, imm12_0;
  imm12_0 = *valp & 0xfff;
  label12_0 = 0x4 + (((int) imm12_0 << 20) >> 20);
  *valp = label12_0;
  return 0;
}

static int
Operand_label12_encode (uint32 *valp)
{
  unsigned imm12_0, label12_0;
  label12_0 = *valp;
  imm12_0 = (label12_0 - 0x4) & 0xfff;
  *valp = imm12_0;
  return 0;
}

static int
Operand_label12_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_label12_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_soffset_decode (uint32 *valp)
{
  unsigned soffset_0, offset_0;
  offset_0 = *valp & 0x3ffff;
  soffset_0 = 0x4 + (((int) offset_0 << 14) >> 14);
  *valp = soffset_0;
  return 0;
}

static int
Operand_soffset_encode (uint32 *valp)
{
  unsigned offset_0, soffset_0;
  soffset_0 = *valp;
  offset_0 = (soffset_0 - 0x4) & 0x3ffff;
  *valp = offset_0;
  return 0;
}

static int
Operand_soffset_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_soffset_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_uimm16x4_decode (uint32 *valp)
{
  unsigned uimm16x4_0, imm16_0;
  imm16_0 = *valp & 0xffff;
  uimm16x4_0 = (((0xffff) << 16) | imm16_0) << 2;
  *valp = uimm16x4_0;
  return 0;
}

static int
Operand_uimm16x4_encode (uint32 *valp)
{
  unsigned imm16_0, uimm16x4_0;
  uimm16x4_0 = *valp;
  imm16_0 = (uimm16x4_0 >> 2) & 0xffff;
  *valp = imm16_0;
  return 0;
}

static int
Operand_uimm16x4_ator (uint32 *valp, uint32 pc)
{
  *valp -= ((pc + 3) & ~0x3);
  return 0;
}

static int
Operand_uimm16x4_rtoa (uint32 *valp, uint32 pc)
{
  *valp += ((pc + 3) & ~0x3);
  return 0;
}

static int
Operand_immt_decode (uint32 *valp)
{
  unsigned immt_0, t_0;
  t_0 = *valp & 0xf;
  immt_0 = t_0;
  *valp = immt_0;
  return 0;
}

static int
Operand_immt_encode (uint32 *valp)
{
  unsigned t_0, immt_0;
  immt_0 = *valp;
  t_0 = immt_0 & 0xf;
  *valp = t_0;
  return 0;
}

static int
Operand_imms_decode (uint32 *valp)
{
  unsigned imms_0, s_0;
  s_0 = *valp & 0xf;
  imms_0 = s_0;
  *valp = imms_0;
  return 0;
}

static int
Operand_imms_encode (uint32 *valp)
{
  unsigned s_0, imms_0;
  imms_0 = *valp;
  s_0 = imms_0 & 0xf;
  *valp = s_0;
  return 0;
}

static int
Operand_bt_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_bt_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_bs_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_bs_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_br_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_br_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_bt2_decode (uint32 *valp)
{
  *valp = *valp << 1;
  return 0;
}

static int
Operand_bt2_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x7 << 1)) != 0;
  *valp = *valp >> 1;
  return error;
}

static int
Operand_bs2_decode (uint32 *valp)
{
  *valp = *valp << 1;
  return 0;
}

static int
Operand_bs2_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x7 << 1)) != 0;
  *valp = *valp >> 1;
  return error;
}

static int
Operand_br2_decode (uint32 *valp)
{
  *valp = *valp << 1;
  return 0;
}

static int
Operand_br2_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x7 << 1)) != 0;
  *valp = *valp >> 1;
  return error;
}

static int
Operand_bt4_decode (uint32 *valp)
{
  *valp = *valp << 2;
  return 0;
}

static int
Operand_bt4_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x3 << 2)) != 0;
  *valp = *valp >> 2;
  return error;
}

static int
Operand_bs4_decode (uint32 *valp)
{
  *valp = *valp << 2;
  return 0;
}

static int
Operand_bs4_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x3 << 2)) != 0;
  *valp = *valp >> 2;
  return error;
}

static int
Operand_br4_decode (uint32 *valp)
{
  *valp = *valp << 2;
  return 0;
}

static int
Operand_br4_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x3 << 2)) != 0;
  *valp = *valp >> 2;
  return error;
}

static int
Operand_bt8_decode (uint32 *valp)
{
  *valp = *valp << 3;
  return 0;
}

static int
Operand_bt8_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x1 << 3)) != 0;
  *valp = *valp >> 3;
  return error;
}

static int
Operand_bs8_decode (uint32 *valp)
{
  *valp = *valp << 3;
  return 0;
}

static int
Operand_bs8_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x1 << 3)) != 0;
  *valp = *valp >> 3;
  return error;
}

static int
Operand_br8_decode (uint32 *valp)
{
  *valp = *valp << 3;
  return 0;
}

static int
Operand_br8_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0x1 << 3)) != 0;
  *valp = *valp >> 3;
  return error;
}

static int
Operand_bt16_decode (uint32 *valp)
{
  *valp = *valp << 4;
  return 0;
}

static int
Operand_bt16_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0 << 4)) != 0;
  *valp = *valp >> 4;
  return error;
}

static int
Operand_bs16_decode (uint32 *valp)
{
  *valp = *valp << 4;
  return 0;
}

static int
Operand_bs16_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0 << 4)) != 0;
  *valp = *valp >> 4;
  return error;
}

static int
Operand_br16_decode (uint32 *valp)
{
  *valp = *valp << 4;
  return 0;
}

static int
Operand_br16_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0 << 4)) != 0;
  *valp = *valp >> 4;
  return error;
}

static int
Operand_brall_decode (uint32 *valp)
{
  *valp = *valp << 4;
  return 0;
}

static int
Operand_brall_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~(0 << 4)) != 0;
  *valp = *valp >> 4;
  return error;
}

static int
Operand_tp7_decode (uint32 *valp)
{
  unsigned tp7_0, t_0;
  t_0 = *valp & 0xf;
  tp7_0 = t_0 + 0x7;
  *valp = tp7_0;
  return 0;
}

static int
Operand_tp7_encode (uint32 *valp)
{
  unsigned t_0, tp7_0;
  tp7_0 = *valp;
  t_0 = (tp7_0 - 0x7) & 0xf;
  *valp = t_0;
  return 0;
}

static int
Operand_xt_wbr15_label_decode (uint32 *valp)
{
  unsigned xt_wbr15_label_0, xt_wbr15_imm_0;
  xt_wbr15_imm_0 = *valp & 0x7fff;
  xt_wbr15_label_0 = 0x4 + (((int) xt_wbr15_imm_0 << 17) >> 17);
  *valp = xt_wbr15_label_0;
  return 0;
}

static int
Operand_xt_wbr15_label_encode (uint32 *valp)
{
  unsigned xt_wbr15_imm_0, xt_wbr15_label_0;
  xt_wbr15_label_0 = *valp;
  xt_wbr15_imm_0 = (xt_wbr15_label_0 - 0x4) & 0x7fff;
  *valp = xt_wbr15_imm_0;
  return 0;
}

static int
Operand_xt_wbr15_label_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_xt_wbr15_label_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_xt_wbr18_label_decode (uint32 *valp)
{
  unsigned xt_wbr18_label_0, xt_wbr18_imm_0;
  xt_wbr18_imm_0 = *valp & 0x3ffff;
  xt_wbr18_label_0 = 0x4 + (((int) xt_wbr18_imm_0 << 14) >> 14);
  *valp = xt_wbr18_label_0;
  return 0;
}

static int
Operand_xt_wbr18_label_encode (uint32 *valp)
{
  unsigned xt_wbr18_imm_0, xt_wbr18_label_0;
  xt_wbr18_label_0 = *valp;
  xt_wbr18_imm_0 = (xt_wbr18_label_0 - 0x4) & 0x3ffff;
  *valp = xt_wbr18_imm_0;
  return 0;
}

static int
Operand_xt_wbr18_label_ator (uint32 *valp, uint32 pc)
{
  *valp -= pc;
  return 0;
}

static int
Operand_xt_wbr18_label_rtoa (uint32 *valp, uint32 pc)
{
  *valp += pc;
  return 0;
}

static int
Operand_cimm8x4_decode (uint32 *valp)
{
  unsigned cimm8x4_0, fimm8_0;
  fimm8_0 = *valp & 0xff;
  cimm8x4_0 = (fimm8_0 << 2) | 0;
  *valp = cimm8x4_0;
  return 0;
}

static int
Operand_cimm8x4_encode (uint32 *valp)
{
  unsigned fimm8_0, cimm8x4_0;
  cimm8x4_0 = *valp;
  fimm8_0 = (cimm8x4_0 >> 2) & 0xff;
  *valp = fimm8_0;
  return 0;
}

static int
Operand_frr_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_frr_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_frs_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_frs_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_frt_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_frt_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper45_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper45_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper46_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper46_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper47_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper47_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper48_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper48_imm_0, dsp340050b49a6c_fld2047_0;
  dsp340050b49a6c_fld2047_0 = *valp & 0x1;
  dsp340050b49a6c_oper48_imm_0 = (0 << 1) | dsp340050b49a6c_fld2047_0;
  *valp = dsp340050b49a6c_oper48_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper48_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2047_0, dsp340050b49a6c_oper48_imm_0;
  dsp340050b49a6c_oper48_imm_0 = *valp;
  dsp340050b49a6c_fld2047_0 = (((dsp340050b49a6c_oper48_imm_0 >> 0) & 1)) & 0x1;
  *valp = dsp340050b49a6c_fld2047_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper49_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper49_imm_0, dsp340050b49a6c_fld2025_0;
  dsp340050b49a6c_fld2025_0 = *valp & 0x1;
  dsp340050b49a6c_oper49_imm_0 = (0 << 1) | dsp340050b49a6c_fld2025_0;
  *valp = dsp340050b49a6c_oper49_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper49_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2025_0, dsp340050b49a6c_oper49_imm_0;
  dsp340050b49a6c_oper49_imm_0 = *valp;
  dsp340050b49a6c_fld2025_0 = (((dsp340050b49a6c_oper49_imm_0 >> 0) & 1)) & 0x1;
  *valp = dsp340050b49a6c_fld2025_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper50_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper50_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper51_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper51_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x7) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper52_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper52_imm_0, dsp340050b49a6c_fld2049_0;
  dsp340050b49a6c_fld2049_0 = *valp & 0x3;
  dsp340050b49a6c_oper52_imm_0 = (0 << 2) | dsp340050b49a6c_fld2049_0;
  *valp = dsp340050b49a6c_oper52_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper52_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2049_0, dsp340050b49a6c_oper52_imm_0;
  dsp340050b49a6c_oper52_imm_0 = *valp;
  dsp340050b49a6c_fld2049_0 = (dsp340050b49a6c_oper52_imm_0 & 0x3);
  *valp = dsp340050b49a6c_fld2049_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper53_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper53_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper54_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper54_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x7) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper55_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper55_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper56_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper56_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper57_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper57_imm_0, dsp340050b49a6c_fld2041_0;
  dsp340050b49a6c_fld2041_0 = *valp & 0x1;
  dsp340050b49a6c_oper57_imm_0 = (0 << 1) | dsp340050b49a6c_fld2041_0;
  *valp = dsp340050b49a6c_oper57_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper57_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2041_0, dsp340050b49a6c_oper57_imm_0;
  dsp340050b49a6c_oper57_imm_0 = *valp;
  dsp340050b49a6c_fld2041_0 = (((dsp340050b49a6c_oper57_imm_0 >> 0) & 1)) & 0x1;
  *valp = dsp340050b49a6c_fld2041_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper58_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper58_imm_0, dsp340050b49a6c_fld2051_0;
  dsp340050b49a6c_fld2051_0 = *valp & 0x3f;
  dsp340050b49a6c_oper58_imm_0 = ((int) dsp340050b49a6c_fld2051_0 << 26) >> 26;
  *valp = dsp340050b49a6c_oper58_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper58_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2051_0, dsp340050b49a6c_oper58_imm_0;
  dsp340050b49a6c_oper58_imm_0 = *valp;
  dsp340050b49a6c_fld2051_0 = (dsp340050b49a6c_oper58_imm_0 & 0x3f);
  *valp = dsp340050b49a6c_fld2051_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper59_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper59_imm_0, dsp340050b49a6c_fld2052_0;
  dsp340050b49a6c_fld2052_0 = *valp & 0xf;
  dsp340050b49a6c_oper59_imm_0 = (0 << 4) | dsp340050b49a6c_fld2052_0;
  *valp = dsp340050b49a6c_oper59_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper59_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2052_0, dsp340050b49a6c_oper59_imm_0;
  dsp340050b49a6c_oper59_imm_0 = *valp;
  dsp340050b49a6c_fld2052_0 = (dsp340050b49a6c_oper59_imm_0 & 0xf);
  *valp = dsp340050b49a6c_fld2052_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper60_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper60_imm_0, dsp340050b49a6c_fld2053_0;
  dsp340050b49a6c_fld2053_0 = *valp & 0x1f;
  dsp340050b49a6c_oper60_imm_0 = (0 << 5) | dsp340050b49a6c_fld2053_0;
  *valp = dsp340050b49a6c_oper60_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper60_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2053_0, dsp340050b49a6c_oper60_imm_0;
  dsp340050b49a6c_oper60_imm_0 = *valp;
  dsp340050b49a6c_fld2053_0 = (dsp340050b49a6c_oper60_imm_0 & 0x1f);
  *valp = dsp340050b49a6c_fld2053_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper61_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper61_imm_0, dsp340050b49a6c_fld2044_0;
  dsp340050b49a6c_fld2044_0 = *valp & 0x1f;
  dsp340050b49a6c_oper61_imm_0 = (0 << 5) | dsp340050b49a6c_fld2044_0;
  *valp = dsp340050b49a6c_oper61_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper61_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2044_0, dsp340050b49a6c_oper61_imm_0;
  dsp340050b49a6c_oper61_imm_0 = *valp;
  dsp340050b49a6c_fld2044_0 = (dsp340050b49a6c_oper61_imm_0 & 0x1f);
  *valp = dsp340050b49a6c_fld2044_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper62_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper62_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper63_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper63_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x7) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper64_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper64_imm_0, dsp340050b49a6c_fld2032_0;
  dsp340050b49a6c_fld2032_0 = *valp & 0x3;
  dsp340050b49a6c_oper64_imm_0 = (0 << 2) | dsp340050b49a6c_fld2032_0;
  *valp = dsp340050b49a6c_oper64_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper64_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2032_0, dsp340050b49a6c_oper64_imm_0;
  dsp340050b49a6c_oper64_imm_0 = *valp;
  dsp340050b49a6c_fld2032_0 = (dsp340050b49a6c_oper64_imm_0 & 0x3);
  *valp = dsp340050b49a6c_fld2032_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper65_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper65_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper66_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper66_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x7) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper67_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper67_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x7) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper68_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper68_imm_0, dsp340050b49a6c_fld2035_0;
  dsp340050b49a6c_fld2035_0 = *valp & 0x3;
  dsp340050b49a6c_oper68_imm_0 = (0 << 2) | dsp340050b49a6c_fld2035_0;
  *valp = dsp340050b49a6c_oper68_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper68_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2035_0, dsp340050b49a6c_oper68_imm_0;
  dsp340050b49a6c_oper68_imm_0 = *valp;
  dsp340050b49a6c_fld2035_0 = (dsp340050b49a6c_oper68_imm_0 & 0x3);
  *valp = dsp340050b49a6c_fld2035_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper69_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper69_imm_0, dsp340050b49a6c_fld2036_0;
  dsp340050b49a6c_fld2036_0 = *valp & 0x1;
  dsp340050b49a6c_oper69_imm_0 = (0 << 1) | dsp340050b49a6c_fld2036_0;
  *valp = dsp340050b49a6c_oper69_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper69_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2036_0, dsp340050b49a6c_oper69_imm_0;
  dsp340050b49a6c_oper69_imm_0 = *valp;
  dsp340050b49a6c_fld2036_0 = (((dsp340050b49a6c_oper69_imm_0 >> 0) & 1)) & 0x1;
  *valp = dsp340050b49a6c_fld2036_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper70_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper70_imm_0, dsp340050b49a6c_fld2037_0;
  dsp340050b49a6c_fld2037_0 = *valp & 0x7;
  dsp340050b49a6c_oper70_imm_0 = (0 << 3) | dsp340050b49a6c_fld2037_0;
  *valp = dsp340050b49a6c_oper70_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper70_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2037_0, dsp340050b49a6c_oper70_imm_0;
  dsp340050b49a6c_oper70_imm_0 = *valp;
  dsp340050b49a6c_fld2037_0 = (dsp340050b49a6c_oper70_imm_0 & 0x7);
  *valp = dsp340050b49a6c_fld2037_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper71_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper71_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0x7) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper72_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper72_imm_0, dsp340050b49a6c_fld2042_0;
  dsp340050b49a6c_fld2042_0 = *valp & 0x3;
  dsp340050b49a6c_oper72_imm_0 = (0 << 2) | dsp340050b49a6c_fld2042_0;
  *valp = dsp340050b49a6c_oper72_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper72_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2042_0, dsp340050b49a6c_oper72_imm_0;
  dsp340050b49a6c_oper72_imm_0 = *valp;
  dsp340050b49a6c_fld2042_0 = (dsp340050b49a6c_oper72_imm_0 & 0x3);
  *valp = dsp340050b49a6c_fld2042_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper73_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper73_imm_0, dsp340050b49a6c_fld2038_0;
  dsp340050b49a6c_fld2038_0 = *valp & 0xf;
  dsp340050b49a6c_oper73_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2038_0) << 3;
  *valp = dsp340050b49a6c_oper73_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper73_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2038_0, dsp340050b49a6c_oper73_imm_0;
  dsp340050b49a6c_oper73_imm_0 = *valp;
  dsp340050b49a6c_fld2038_0 = ((dsp340050b49a6c_oper73_imm_0 >> 3) & 0xf);
  *valp = dsp340050b49a6c_fld2038_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper74_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper74_imm_0, dsp340050b49a6c_fld2038_0;
  dsp340050b49a6c_fld2038_0 = *valp & 0xf;
  dsp340050b49a6c_oper74_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2038_0) << 4;
  *valp = dsp340050b49a6c_oper74_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper74_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2038_0, dsp340050b49a6c_oper74_imm_0;
  dsp340050b49a6c_oper74_imm_0 = *valp;
  dsp340050b49a6c_fld2038_0 = ((dsp340050b49a6c_oper74_imm_0 >> 4) & 0xf);
  *valp = dsp340050b49a6c_fld2038_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper75_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper75_imm_0, dsp340050b49a6c_fld2038_0;
  dsp340050b49a6c_fld2038_0 = *valp & 0xf;
  dsp340050b49a6c_oper75_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2038_0) << 2;
  *valp = dsp340050b49a6c_oper75_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper75_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2038_0, dsp340050b49a6c_oper75_imm_0;
  dsp340050b49a6c_oper75_imm_0 = *valp;
  dsp340050b49a6c_fld2038_0 = ((dsp340050b49a6c_oper75_imm_0 >> 2) & 0xf);
  *valp = dsp340050b49a6c_fld2038_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper76_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper76_imm_0, dsp340050b49a6c_fld2053_0;
  dsp340050b49a6c_fld2053_0 = *valp & 0x1f;
  dsp340050b49a6c_oper76_imm_0 = ((0 << 5) | dsp340050b49a6c_fld2053_0) << 4;
  *valp = dsp340050b49a6c_oper76_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper76_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2053_0, dsp340050b49a6c_oper76_imm_0;
  dsp340050b49a6c_oper76_imm_0 = *valp;
  dsp340050b49a6c_fld2053_0 = ((dsp340050b49a6c_oper76_imm_0 >> 4) & 0x1f);
  *valp = dsp340050b49a6c_fld2053_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper77_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper77_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper78_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper78_imm_0, dsp340050b49a6c_fld2056_0;
  dsp340050b49a6c_fld2056_0 = *valp & 0x7;
  dsp340050b49a6c_oper78_imm_0 = (0 << 3) | dsp340050b49a6c_fld2056_0;
  *valp = dsp340050b49a6c_oper78_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper78_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2056_0, dsp340050b49a6c_oper78_imm_0;
  dsp340050b49a6c_oper78_imm_0 = *valp;
  dsp340050b49a6c_fld2056_0 = (dsp340050b49a6c_oper78_imm_0 & 0x7);
  *valp = dsp340050b49a6c_fld2056_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper79_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper79_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper80_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper80_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper81_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper81_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper82_reg_decode (uint32 *valp ATTRIBUTE_UNUSED)
{
  return 0;
}

static int
Operand_dsp340050b49a6c_oper82_reg_encode (uint32 *valp)
{
  int error;
  error = (*valp & ~0xf) != 0;
  return error;
}

static int
Operand_dsp340050b49a6c_oper83_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper83_imm_0, dsp340050b49a6c_fld2040_0;
  dsp340050b49a6c_fld2040_0 = *valp & 0x1f;
  dsp340050b49a6c_oper83_imm_0 = ((0 << 5) | dsp340050b49a6c_fld2040_0) << 4;
  *valp = dsp340050b49a6c_oper83_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper83_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2040_0, dsp340050b49a6c_oper83_imm_0;
  dsp340050b49a6c_oper83_imm_0 = *valp;
  dsp340050b49a6c_fld2040_0 = ((dsp340050b49a6c_oper83_imm_0 >> 4) & 0x1f);
  *valp = dsp340050b49a6c_fld2040_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper84_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper84_imm_0, dsp340050b49a6c_fld2029_0;
  dsp340050b49a6c_fld2029_0 = *valp & 0xf;
  dsp340050b49a6c_oper84_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2029_0) << 3;
  *valp = dsp340050b49a6c_oper84_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper84_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2029_0, dsp340050b49a6c_oper84_imm_0;
  dsp340050b49a6c_oper84_imm_0 = *valp;
  dsp340050b49a6c_fld2029_0 = ((dsp340050b49a6c_oper84_imm_0 >> 3) & 0xf);
  *valp = dsp340050b49a6c_fld2029_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper85_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper85_imm_0, dsp340050b49a6c_fld2029_0;
  dsp340050b49a6c_fld2029_0 = *valp & 0xf;
  dsp340050b49a6c_oper85_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2029_0) << 4;
  *valp = dsp340050b49a6c_oper85_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper85_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2029_0, dsp340050b49a6c_oper85_imm_0;
  dsp340050b49a6c_oper85_imm_0 = *valp;
  dsp340050b49a6c_fld2029_0 = ((dsp340050b49a6c_oper85_imm_0 >> 4) & 0xf);
  *valp = dsp340050b49a6c_fld2029_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper86_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper86_imm_0, dsp340050b49a6c_fld2029_0;
  dsp340050b49a6c_fld2029_0 = *valp & 0xf;
  dsp340050b49a6c_oper86_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2029_0) << 2;
  *valp = dsp340050b49a6c_oper86_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper86_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2029_0, dsp340050b49a6c_oper86_imm_0;
  dsp340050b49a6c_oper86_imm_0 = *valp;
  dsp340050b49a6c_fld2029_0 = ((dsp340050b49a6c_oper86_imm_0 >> 2) & 0xf);
  *valp = dsp340050b49a6c_fld2029_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper87_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper87_imm_0, dsp340050b49a6c_fld2043_0;
  dsp340050b49a6c_fld2043_0 = *valp & 0x1f;
  dsp340050b49a6c_oper87_imm_0 = ((0 << 5) | dsp340050b49a6c_fld2043_0) << 4;
  *valp = dsp340050b49a6c_oper87_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper87_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2043_0, dsp340050b49a6c_oper87_imm_0;
  dsp340050b49a6c_oper87_imm_0 = *valp;
  dsp340050b49a6c_fld2043_0 = ((dsp340050b49a6c_oper87_imm_0 >> 4) & 0x1f);
  *valp = dsp340050b49a6c_fld2043_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper88_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper88_imm_0, dsp340050b49a6c_fld2030_0;
  dsp340050b49a6c_fld2030_0 = *valp & 0xf;
  dsp340050b49a6c_oper88_imm_0 = (0 << 4) | dsp340050b49a6c_fld2030_0;
  *valp = dsp340050b49a6c_oper88_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper88_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2030_0, dsp340050b49a6c_oper88_imm_0;
  dsp340050b49a6c_oper88_imm_0 = *valp;
  dsp340050b49a6c_fld2030_0 = (dsp340050b49a6c_oper88_imm_0 & 0xf);
  *valp = dsp340050b49a6c_fld2030_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper89_imm_decode (uint32 *valp)
{
  unsigned dsp340050b49a6c_oper89_imm_0, dsp340050b49a6c_fld2052_0;
  dsp340050b49a6c_fld2052_0 = *valp & 0xf;
  dsp340050b49a6c_oper89_imm_0 = ((0 << 4) | dsp340050b49a6c_fld2052_0) << 2;
  *valp = dsp340050b49a6c_oper89_imm_0;
  return 0;
}

static int
Operand_dsp340050b49a6c_oper89_imm_encode (uint32 *valp)
{
  unsigned dsp340050b49a6c_fld2052_0, dsp340050b49a6c_oper89_imm_0;
  dsp340050b49a6c_oper89_imm_0 = *valp;
  dsp340050b49a6c_fld2052_0 = ((dsp340050b49a6c_oper89_imm_0 >> 2) & 0xf);
  *valp = dsp340050b49a6c_fld2052_0;
  return 0;
}

static xtensa_operand_internal operands[] = {
  { "soffsetx4", FIELD_offset, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_soffsetx4_encode, Operand_soffsetx4_decode,
    Operand_soffsetx4_ator, Operand_soffsetx4_rtoa },
  { "uimm12x8", FIELD_imm12, -1, 0,
    0,
    Operand_uimm12x8_encode, Operand_uimm12x8_decode,
    0, 0 },
  { "simm4", FIELD_mn, -1, 0,
    0,
    Operand_simm4_encode, Operand_simm4_decode,
    0, 0 },
  { "arr", FIELD_r, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_arr_encode, Operand_arr_decode,
    0, 0 },
  { "ars", FIELD_s, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_ars_encode, Operand_ars_decode,
    0, 0 },
  { "*ars_invisible", FIELD_s, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER | XTENSA_OPERAND_IS_INVISIBLE,
    Operand_ars_encode, Operand_ars_decode,
    0, 0 },
  { "art", FIELD_t, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_art_encode, Operand_art_decode,
    0, 0 },
  { "ar0", FIELD__ar0, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER | XTENSA_OPERAND_IS_INVISIBLE,
    Operand_ar0_encode, Operand_ar0_decode,
    0, 0 },
  { "ar4", FIELD__ar4, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER | XTENSA_OPERAND_IS_INVISIBLE,
    Operand_ar4_encode, Operand_ar4_decode,
    0, 0 },
  { "ar8", FIELD__ar8, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER | XTENSA_OPERAND_IS_INVISIBLE,
    Operand_ar8_encode, Operand_ar8_decode,
    0, 0 },
  { "ar12", FIELD__ar12, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER | XTENSA_OPERAND_IS_INVISIBLE,
    Operand_ar12_encode, Operand_ar12_decode,
    0, 0 },
  { "ars_entry", FIELD_s, REGFILE_AR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_ars_entry_encode, Operand_ars_entry_decode,
    0, 0 },
  { "immrx4", FIELD_r, -1, 0,
    0,
    Operand_immrx4_encode, Operand_immrx4_decode,
    0, 0 },
  { "lsi4x4", FIELD_r, -1, 0,
    0,
    Operand_lsi4x4_encode, Operand_lsi4x4_decode,
    0, 0 },
  { "simm7", FIELD_imm7, -1, 0,
    0,
    Operand_simm7_encode, Operand_simm7_decode,
    0, 0 },
  { "uimm6", FIELD_imm6, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_uimm6_encode, Operand_uimm6_decode,
    Operand_uimm6_ator, Operand_uimm6_rtoa },
  { "ai4const", FIELD_t, -1, 0,
    0,
    Operand_ai4const_encode, Operand_ai4const_decode,
    0, 0 },
  { "b4const", FIELD_r, -1, 0,
    0,
    Operand_b4const_encode, Operand_b4const_decode,
    0, 0 },
  { "b4constu", FIELD_r, -1, 0,
    0,
    Operand_b4constu_encode, Operand_b4constu_decode,
    0, 0 },
  { "uimm8", FIELD_imm8, -1, 0,
    0,
    Operand_uimm8_encode, Operand_uimm8_decode,
    0, 0 },
  { "uimm8x2", FIELD_imm8, -1, 0,
    0,
    Operand_uimm8x2_encode, Operand_uimm8x2_decode,
    0, 0 },
  { "uimm8x4", FIELD_imm8, -1, 0,
    0,
    Operand_uimm8x4_encode, Operand_uimm8x4_decode,
    0, 0 },
  { "uimm4x16", FIELD_op2, -1, 0,
    0,
    Operand_uimm4x16_encode, Operand_uimm4x16_decode,
    0, 0 },
  { "simm8", FIELD_imm8, -1, 0,
    0,
    Operand_simm8_encode, Operand_simm8_decode,
    0, 0 },
  { "simm8x256", FIELD_imm8, -1, 0,
    0,
    Operand_simm8x256_encode, Operand_simm8x256_decode,
    0, 0 },
  { "simm12b", FIELD_imm12b, -1, 0,
    0,
    Operand_simm12b_encode, Operand_simm12b_decode,
    0, 0 },
  { "msalp32", FIELD_sal, -1, 0,
    0,
    Operand_msalp32_encode, Operand_msalp32_decode,
    0, 0 },
  { "op2p1", FIELD_op2, -1, 0,
    0,
    Operand_op2p1_encode, Operand_op2p1_decode,
    0, 0 },
  { "label8", FIELD_imm8, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_label8_encode, Operand_label8_decode,
    Operand_label8_ator, Operand_label8_rtoa },
  { "ulabel8", FIELD_imm8, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_ulabel8_encode, Operand_ulabel8_decode,
    Operand_ulabel8_ator, Operand_ulabel8_rtoa },
  { "label12", FIELD_imm12, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_label12_encode, Operand_label12_decode,
    Operand_label12_ator, Operand_label12_rtoa },
  { "soffset", FIELD_offset, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_soffset_encode, Operand_soffset_decode,
    Operand_soffset_ator, Operand_soffset_rtoa },
  { "uimm16x4", FIELD_imm16, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_uimm16x4_encode, Operand_uimm16x4_decode,
    Operand_uimm16x4_ator, Operand_uimm16x4_rtoa },
  { "immt", FIELD_t, -1, 0,
    0,
    Operand_immt_encode, Operand_immt_decode,
    0, 0 },
  { "imms", FIELD_s, -1, 0,
    0,
    Operand_imms_encode, Operand_imms_decode,
    0, 0 },
  { "bt", FIELD_t, REGFILE_BR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bt_encode, Operand_bt_decode,
    0, 0 },
  { "bs", FIELD_s, REGFILE_BR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bs_encode, Operand_bs_decode,
    0, 0 },
  { "br", FIELD_r, REGFILE_BR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_br_encode, Operand_br_decode,
    0, 0 },
  { "bt2", FIELD_t2, REGFILE_BR, 2,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bt2_encode, Operand_bt2_decode,
    0, 0 },
  { "bs2", FIELD_s2, REGFILE_BR, 2,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bs2_encode, Operand_bs2_decode,
    0, 0 },
  { "br2", FIELD_r2, REGFILE_BR, 2,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_br2_encode, Operand_br2_decode,
    0, 0 },
  { "bt4", FIELD_t4, REGFILE_BR, 4,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bt4_encode, Operand_bt4_decode,
    0, 0 },
  { "bs4", FIELD_s4, REGFILE_BR, 4,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bs4_encode, Operand_bs4_decode,
    0, 0 },
  { "br4", FIELD_r4, REGFILE_BR, 4,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_br4_encode, Operand_br4_decode,
    0, 0 },
  { "bt8", FIELD_t8, REGFILE_BR, 8,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bt8_encode, Operand_bt8_decode,
    0, 0 },
  { "bs8", FIELD_s8, REGFILE_BR, 8,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bs8_encode, Operand_bs8_decode,
    0, 0 },
  { "br8", FIELD_r8, REGFILE_BR, 8,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_br8_encode, Operand_br8_decode,
    0, 0 },
  { "bt16", FIELD__bt16, REGFILE_BR, 16,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bt16_encode, Operand_bt16_decode,
    0, 0 },
  { "bs16", FIELD__bs16, REGFILE_BR, 16,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_bs16_encode, Operand_bs16_decode,
    0, 0 },
  { "br16", FIELD__br16, REGFILE_BR, 16,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_br16_encode, Operand_br16_decode,
    0, 0 },
  { "brall", FIELD__brall, REGFILE_BR, 16,
    XTENSA_OPERAND_IS_REGISTER | XTENSA_OPERAND_IS_INVISIBLE,
    Operand_brall_encode, Operand_brall_decode,
    0, 0 },
  { "tp7", FIELD_t, -1, 0,
    0,
    Operand_tp7_encode, Operand_tp7_decode,
    0, 0 },
  { "xt_wbr15_label", FIELD_xt_wbr15_imm, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_xt_wbr15_label_encode, Operand_xt_wbr15_label_decode,
    Operand_xt_wbr15_label_ator, Operand_xt_wbr15_label_rtoa },
  { "xt_wbr18_label", FIELD_xt_wbr18_imm, -1, 0,
    XTENSA_OPERAND_IS_PCRELATIVE,
    Operand_xt_wbr18_label_encode, Operand_xt_wbr18_label_decode,
    Operand_xt_wbr18_label_ator, Operand_xt_wbr18_label_rtoa },
  { "cimm8x4", FIELD_fimm8, -1, 0,
    0,
    Operand_cimm8x4_encode, Operand_cimm8x4_decode,
    0, 0 },
  { "frr", FIELD_r, REGFILE_FR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_frr_encode, Operand_frr_decode,
    0, 0 },
  { "frs", FIELD_s, REGFILE_FR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_frs_encode, Operand_frs_decode,
    0, 0 },
  { "frt", FIELD_t, REGFILE_FR, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_frt_encode, Operand_frt_decode,
    0, 0 },
  { "dsp340050b49a6c_oper45_reg", FIELD_dsp340050b49a6c_fld2045, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper45_reg_encode, Operand_dsp340050b49a6c_oper45_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper46_reg", FIELD_dsp340050b49a6c_fld2046, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper46_reg_encode, Operand_dsp340050b49a6c_oper46_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper47_reg", FIELD_dsp340050b49a6c_fld2029, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper47_reg_encode, Operand_dsp340050b49a6c_oper47_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper48_imm", FIELD_dsp340050b49a6c_fld2047, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper48_imm_encode, Operand_dsp340050b49a6c_oper48_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper49_imm", FIELD_dsp340050b49a6c_fld2025, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper49_imm_encode, Operand_dsp340050b49a6c_oper49_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper50_reg", FIELD_dsp340050b49a6c_fld2026, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper50_reg_encode, Operand_dsp340050b49a6c_oper50_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper51_reg", FIELD_dsp340050b49a6c_fld2048, REGFILE_ACU, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper51_reg_encode, Operand_dsp340050b49a6c_oper51_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper52_imm", FIELD_dsp340050b49a6c_fld2049, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper52_imm_encode, Operand_dsp340050b49a6c_oper52_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper53_reg", FIELD_dsp340050b49a6c_fld2027, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper53_reg_encode, Operand_dsp340050b49a6c_oper53_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper54_reg", FIELD_dsp340050b49a6c_fld2028, REGFILE_ACU, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper54_reg_encode, Operand_dsp340050b49a6c_oper54_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper55_reg", FIELD_dsp340050b49a6c_fld2030, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper55_reg_encode, Operand_dsp340050b49a6c_oper55_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper56_reg", FIELD_dsp340050b49a6c_fld2050, REGFILE_PQ, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper56_reg_encode, Operand_dsp340050b49a6c_oper56_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper57_imm", FIELD_dsp340050b49a6c_fld2041, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper57_imm_encode, Operand_dsp340050b49a6c_oper57_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper58_imm", FIELD_dsp340050b49a6c_fld2051, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper58_imm_encode, Operand_dsp340050b49a6c_oper58_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper59_imm", FIELD_dsp340050b49a6c_fld2052, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper59_imm_encode, Operand_dsp340050b49a6c_oper59_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper60_imm", FIELD_dsp340050b49a6c_fld2053, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper60_imm_encode, Operand_dsp340050b49a6c_oper60_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper61_imm", FIELD_dsp340050b49a6c_fld2044, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper61_imm_encode, Operand_dsp340050b49a6c_oper61_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper62_reg", FIELD_dsp340050b49a6c_fld2031, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper62_reg_encode, Operand_dsp340050b49a6c_oper62_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper63_reg", FIELD_dsp340050b49a6c_fld2054, REGFILE_ACU, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper63_reg_encode, Operand_dsp340050b49a6c_oper63_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper64_imm", FIELD_dsp340050b49a6c_fld2032, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper64_imm_encode, Operand_dsp340050b49a6c_oper64_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper65_reg", FIELD_dsp340050b49a6c_fld2055, REGFILE_PQ, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper65_reg_encode, Operand_dsp340050b49a6c_oper65_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper66_reg", FIELD_dsp340050b49a6c_fld2033, REGFILE_ACU, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper66_reg_encode, Operand_dsp340050b49a6c_oper66_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper67_reg", FIELD_dsp340050b49a6c_fld2034, REGFILE_ACU, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper67_reg_encode, Operand_dsp340050b49a6c_oper67_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper68_imm", FIELD_dsp340050b49a6c_fld2035, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper68_imm_encode, Operand_dsp340050b49a6c_oper68_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper69_imm", FIELD_dsp340050b49a6c_fld2036, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper69_imm_encode, Operand_dsp340050b49a6c_oper69_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper70_imm", FIELD_dsp340050b49a6c_fld2037, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper70_imm_encode, Operand_dsp340050b49a6c_oper70_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper71_reg", FIELD_dsp340050b49a6c_fld2037, REGFILE_ACU, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper71_reg_encode, Operand_dsp340050b49a6c_oper71_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper72_imm", FIELD_dsp340050b49a6c_fld2042, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper72_imm_encode, Operand_dsp340050b49a6c_oper72_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper73_imm", FIELD_dsp340050b49a6c_fld2038, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper73_imm_encode, Operand_dsp340050b49a6c_oper73_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper74_imm", FIELD_dsp340050b49a6c_fld2038, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper74_imm_encode, Operand_dsp340050b49a6c_oper74_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper75_imm", FIELD_dsp340050b49a6c_fld2038, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper75_imm_encode, Operand_dsp340050b49a6c_oper75_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper76_imm", FIELD_dsp340050b49a6c_fld2053, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper76_imm_encode, Operand_dsp340050b49a6c_oper76_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper77_reg", FIELD_r, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper77_reg_encode, Operand_dsp340050b49a6c_oper77_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper78_imm", FIELD_dsp340050b49a6c_fld2056, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper78_imm_encode, Operand_dsp340050b49a6c_oper78_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper79_reg", FIELD_r, REGFILE_PQ, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper79_reg_encode, Operand_dsp340050b49a6c_oper79_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper80_reg", FIELD_s, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper80_reg_encode, Operand_dsp340050b49a6c_oper80_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper81_reg", FIELD_dsp340050b49a6c_fld2039, REGFILE_PQ, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper81_reg_encode, Operand_dsp340050b49a6c_oper81_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper82_reg", FIELD_dsp340050b49a6c_fld2052, REGFILE_CM, 1,
    XTENSA_OPERAND_IS_REGISTER,
    Operand_dsp340050b49a6c_oper82_reg_encode, Operand_dsp340050b49a6c_oper82_reg_decode,
    0, 0 },
  { "dsp340050b49a6c_oper83_imm", FIELD_dsp340050b49a6c_fld2040, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper83_imm_encode, Operand_dsp340050b49a6c_oper83_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper84_imm", FIELD_dsp340050b49a6c_fld2029, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper84_imm_encode, Operand_dsp340050b49a6c_oper84_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper85_imm", FIELD_dsp340050b49a6c_fld2029, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper85_imm_encode, Operand_dsp340050b49a6c_oper85_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper86_imm", FIELD_dsp340050b49a6c_fld2029, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper86_imm_encode, Operand_dsp340050b49a6c_oper86_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper87_imm", FIELD_dsp340050b49a6c_fld2043, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper87_imm_encode, Operand_dsp340050b49a6c_oper87_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper88_imm", FIELD_dsp340050b49a6c_fld2030, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper88_imm_encode, Operand_dsp340050b49a6c_oper88_imm_decode,
    0, 0 },
  { "dsp340050b49a6c_oper89_imm", FIELD_dsp340050b49a6c_fld2052, -1, 0,
    0,
    Operand_dsp340050b49a6c_oper89_imm_encode, Operand_dsp340050b49a6c_oper89_imm_decode,
    0, 0 },
  { "t", FIELD_t, -1, 0, 0, 0, 0, 0, 0 },
  { "bbi4", FIELD_bbi4, -1, 0, 0, 0, 0, 0, 0 },
  { "bbi", FIELD_bbi, -1, 0, 0, 0, 0, 0, 0 },
  { "imm12", FIELD_imm12, -1, 0, 0, 0, 0, 0, 0 },
  { "imm8", FIELD_imm8, -1, 0, 0, 0, 0, 0, 0 },
  { "s", FIELD_s, -1, 0, 0, 0, 0, 0, 0 },
  { "imm12b", FIELD_imm12b, -1, 0, 0, 0, 0, 0, 0 },
  { "imm16", FIELD_imm16, -1, 0, 0, 0, 0, 0, 0 },
  { "m", FIELD_m, -1, 0, 0, 0, 0, 0, 0 },
  { "n", FIELD_n, -1, 0, 0, 0, 0, 0, 0 },
  { "offset", FIELD_offset, -1, 0, 0, 0, 0, 0, 0 },
  { "op0", FIELD_op0, -1, 0, 0, 0, 0, 0, 0 },
  { "op1", FIELD_op1, -1, 0, 0, 0, 0, 0, 0 },
  { "op2", FIELD_op2, -1, 0, 0, 0, 0, 0, 0 },
  { "r", FIELD_r, -1, 0, 0, 0, 0, 0, 0 },
  { "sa4", FIELD_sa4, -1, 0, 0, 0, 0, 0, 0 },
  { "sae4", FIELD_sae4, -1, 0, 0, 0, 0, 0, 0 },
  { "sae", FIELD_sae, -1, 0, 0, 0, 0, 0, 0 },
  { "sal", FIELD_sal, -1, 0, 0, 0, 0, 0, 0 },
  { "sargt", FIELD_sargt, -1, 0, 0, 0, 0, 0, 0 },
  { "sas4", FIELD_sas4, -1, 0, 0, 0, 0, 0, 0 },
  { "sas", FIELD_sas, -1, 0, 0, 0, 0, 0, 0 },
  { "sr", FIELD_sr, -1, 0, 0, 0, 0, 0, 0 },
  { "st", FIELD_st, -1, 0, 0, 0, 0, 0, 0 },
  { "thi3", FIELD_thi3, -1, 0, 0, 0, 0, 0, 0 },
  { "imm4", FIELD_imm4, -1, 0, 0, 0, 0, 0, 0 },
  { "mn", FIELD_mn, -1, 0, 0, 0, 0, 0, 0 },
  { "i", FIELD_i, -1, 0, 0, 0, 0, 0, 0 },
  { "imm6lo", FIELD_imm6lo, -1, 0, 0, 0, 0, 0, 0 },
  { "imm6hi", FIELD_imm6hi, -1, 0, 0, 0, 0, 0, 0 },
  { "imm7lo", FIELD_imm7lo, -1, 0, 0, 0, 0, 0, 0 },
  { "imm7hi", FIELD_imm7hi, -1, 0, 0, 0, 0, 0, 0 },
  { "z", FIELD_z, -1, 0, 0, 0, 0, 0, 0 },
  { "imm6", FIELD_imm6, -1, 0, 0, 0, 0, 0, 0 },
  { "imm7", FIELD_imm7, -1, 0, 0, 0, 0, 0, 0 },
  { "t2", FIELD_t2, -1, 0, 0, 0, 0, 0, 0 },
  { "s2", FIELD_s2, -1, 0, 0, 0, 0, 0, 0 },
  { "r2", FIELD_r2, -1, 0, 0, 0, 0, 0, 0 },
  { "t4", FIELD_t4, -1, 0, 0, 0, 0, 0, 0 },
  { "s4", FIELD_s4, -1, 0, 0, 0, 0, 0, 0 },
  { "r4", FIELD_r4, -1, 0, 0, 0, 0, 0, 0 },
  { "t8", FIELD_t8, -1, 0, 0, 0, 0, 0, 0 },
  { "s8", FIELD_s8, -1, 0, 0, 0, 0, 0, 0 },
  { "r8", FIELD_r8, -1, 0, 0, 0, 0, 0, 0 },
  { "xt_wbr15_imm", FIELD_xt_wbr15_imm, -1, 0, 0, 0, 0, 0, 0 },
  { "xt_wbr18_imm", FIELD_xt_wbr18_imm, -1, 0, 0, 0, 0, 0, 0 },
  { "fimm8", FIELD_fimm8, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2019", FIELD_dsp340050b49a6c_fld2019, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2021", FIELD_dsp340050b49a6c_fld2021, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2029", FIELD_dsp340050b49a6c_fld2029, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2030", FIELD_dsp340050b49a6c_fld2030, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2032", FIELD_dsp340050b49a6c_fld2032, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2035", FIELD_dsp340050b49a6c_fld2035, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2036", FIELD_dsp340050b49a6c_fld2036, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2037", FIELD_dsp340050b49a6c_fld2037, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2038", FIELD_dsp340050b49a6c_fld2038, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2039", FIELD_dsp340050b49a6c_fld2039, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2040", FIELD_dsp340050b49a6c_fld2040, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2041", FIELD_dsp340050b49a6c_fld2041, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2042", FIELD_dsp340050b49a6c_fld2042, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2043", FIELD_dsp340050b49a6c_fld2043, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2044", FIELD_dsp340050b49a6c_fld2044, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2045", FIELD_dsp340050b49a6c_fld2045, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2046", FIELD_dsp340050b49a6c_fld2046, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2047", FIELD_dsp340050b49a6c_fld2047, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2048", FIELD_dsp340050b49a6c_fld2048, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2049", FIELD_dsp340050b49a6c_fld2049, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2050", FIELD_dsp340050b49a6c_fld2050, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2051", FIELD_dsp340050b49a6c_fld2051, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2052", FIELD_dsp340050b49a6c_fld2052, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2053", FIELD_dsp340050b49a6c_fld2053, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2054", FIELD_dsp340050b49a6c_fld2054, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2055", FIELD_dsp340050b49a6c_fld2055, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2056", FIELD_dsp340050b49a6c_fld2056, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2082Inst", FIELD_dsp340050b49a6c_fld2082Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2083Inst", FIELD_dsp340050b49a6c_fld2083Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2084Inst", FIELD_dsp340050b49a6c_fld2084Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2085Inst", FIELD_dsp340050b49a6c_fld2085Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2086Inst", FIELD_dsp340050b49a6c_fld2086Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2088Inst", FIELD_dsp340050b49a6c_fld2088Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2089Inst", FIELD_dsp340050b49a6c_fld2089Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2090Inst", FIELD_dsp340050b49a6c_fld2090Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2091Inst", FIELD_dsp340050b49a6c_fld2091Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2092Inst", FIELD_dsp340050b49a6c_fld2092Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2094Inst", FIELD_dsp340050b49a6c_fld2094Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2095Inst", FIELD_dsp340050b49a6c_fld2095Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2096Inst", FIELD_dsp340050b49a6c_fld2096Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2098Inst", FIELD_dsp340050b49a6c_fld2098Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2099Inst", FIELD_dsp340050b49a6c_fld2099Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2100Inst", FIELD_dsp340050b49a6c_fld2100Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2101Inst", FIELD_dsp340050b49a6c_fld2101Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2102Inst", FIELD_dsp340050b49a6c_fld2102Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2103Inst", FIELD_dsp340050b49a6c_fld2103Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2104Inst", FIELD_dsp340050b49a6c_fld2104Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2105Inst", FIELD_dsp340050b49a6c_fld2105Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2106Inst", FIELD_dsp340050b49a6c_fld2106Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2107Inst", FIELD_dsp340050b49a6c_fld2107Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2108Inst", FIELD_dsp340050b49a6c_fld2108Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2109Inst", FIELD_dsp340050b49a6c_fld2109Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2110Inst", FIELD_dsp340050b49a6c_fld2110Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2111Inst", FIELD_dsp340050b49a6c_fld2111Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2112Inst", FIELD_dsp340050b49a6c_fld2112Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2113Inst", FIELD_dsp340050b49a6c_fld2113Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2114Inst", FIELD_dsp340050b49a6c_fld2114Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2115Inst", FIELD_dsp340050b49a6c_fld2115Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2116Inst", FIELD_dsp340050b49a6c_fld2116Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2117Inst", FIELD_dsp340050b49a6c_fld2117Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2118Inst", FIELD_dsp340050b49a6c_fld2118Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2119Inst", FIELD_dsp340050b49a6c_fld2119Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2120Inst", FIELD_dsp340050b49a6c_fld2120Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2122Inst", FIELD_dsp340050b49a6c_fld2122Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2123Inst", FIELD_dsp340050b49a6c_fld2123Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2124Inst", FIELD_dsp340050b49a6c_fld2124Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2125Inst", FIELD_dsp340050b49a6c_fld2125Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2126Inst", FIELD_dsp340050b49a6c_fld2126Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2127Inst", FIELD_dsp340050b49a6c_fld2127Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2128Inst", FIELD_dsp340050b49a6c_fld2128Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2129Inst", FIELD_dsp340050b49a6c_fld2129Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2131Inst", FIELD_dsp340050b49a6c_fld2131Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2132Inst", FIELD_dsp340050b49a6c_fld2132Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2133Inst", FIELD_dsp340050b49a6c_fld2133Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2134Inst", FIELD_dsp340050b49a6c_fld2134Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2136Inst", FIELD_dsp340050b49a6c_fld2136Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2137Inst", FIELD_dsp340050b49a6c_fld2137Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2138Inst", FIELD_dsp340050b49a6c_fld2138Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2139Inst", FIELD_dsp340050b49a6c_fld2139Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2140Inst", FIELD_dsp340050b49a6c_fld2140Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2141Inst", FIELD_dsp340050b49a6c_fld2141Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2142Inst", FIELD_dsp340050b49a6c_fld2142Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2143Inst", FIELD_dsp340050b49a6c_fld2143Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2144Inst", FIELD_dsp340050b49a6c_fld2144Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2145Inst", FIELD_dsp340050b49a6c_fld2145Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2146Inst", FIELD_dsp340050b49a6c_fld2146Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2147Inst", FIELD_dsp340050b49a6c_fld2147Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2149Inst", FIELD_dsp340050b49a6c_fld2149Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2151Inst", FIELD_dsp340050b49a6c_fld2151Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2153Inst", FIELD_dsp340050b49a6c_fld2153Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2154Inst", FIELD_dsp340050b49a6c_fld2154Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2155Inst", FIELD_dsp340050b49a6c_fld2155Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2156Inst", FIELD_dsp340050b49a6c_fld2156Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2157Inst", FIELD_dsp340050b49a6c_fld2157Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2158Inst", FIELD_dsp340050b49a6c_fld2158Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2159Inst", FIELD_dsp340050b49a6c_fld2159Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2160Inst", FIELD_dsp340050b49a6c_fld2160Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2161Inst", FIELD_dsp340050b49a6c_fld2161Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2162Inst", FIELD_dsp340050b49a6c_fld2162Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2163Inst", FIELD_dsp340050b49a6c_fld2163Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2164Inst", FIELD_dsp340050b49a6c_fld2164Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2165Inst", FIELD_dsp340050b49a6c_fld2165Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2166Inst", FIELD_dsp340050b49a6c_fld2166Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2167Inst", FIELD_dsp340050b49a6c_fld2167Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2168Inst", FIELD_dsp340050b49a6c_fld2168Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2169Inst", FIELD_dsp340050b49a6c_fld2169Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2171Inst", FIELD_dsp340050b49a6c_fld2171Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2172Inst", FIELD_dsp340050b49a6c_fld2172Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2173Inst", FIELD_dsp340050b49a6c_fld2173Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2174Inst", FIELD_dsp340050b49a6c_fld2174Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2175Inst", FIELD_dsp340050b49a6c_fld2175Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2177Inst", FIELD_dsp340050b49a6c_fld2177Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2178Inst", FIELD_dsp340050b49a6c_fld2178Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2179Inst", FIELD_dsp340050b49a6c_fld2179Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2180Inst", FIELD_dsp340050b49a6c_fld2180Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2181Inst", FIELD_dsp340050b49a6c_fld2181Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2182Inst", FIELD_dsp340050b49a6c_fld2182Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2183Inst", FIELD_dsp340050b49a6c_fld2183Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2184Inst", FIELD_dsp340050b49a6c_fld2184Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2185Inst", FIELD_dsp340050b49a6c_fld2185Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2186Inst", FIELD_dsp340050b49a6c_fld2186Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2187Inst", FIELD_dsp340050b49a6c_fld2187Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2188Inst", FIELD_dsp340050b49a6c_fld2188Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2189Inst", FIELD_dsp340050b49a6c_fld2189Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2190Inst", FIELD_dsp340050b49a6c_fld2190Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2191Inst", FIELD_dsp340050b49a6c_fld2191Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2192Inst", FIELD_dsp340050b49a6c_fld2192Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2193Inst", FIELD_dsp340050b49a6c_fld2193Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2194Inst", FIELD_dsp340050b49a6c_fld2194Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2195Inst", FIELD_dsp340050b49a6c_fld2195Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2196Inst", FIELD_dsp340050b49a6c_fld2196Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2197Inst", FIELD_dsp340050b49a6c_fld2197Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2198Inst", FIELD_dsp340050b49a6c_fld2198Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2199Inst", FIELD_dsp340050b49a6c_fld2199Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2200Inst", FIELD_dsp340050b49a6c_fld2200Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2201Inst", FIELD_dsp340050b49a6c_fld2201Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2202Inst", FIELD_dsp340050b49a6c_fld2202Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2203Inst", FIELD_dsp340050b49a6c_fld2203Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2204Inst", FIELD_dsp340050b49a6c_fld2204Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2205Inst", FIELD_dsp340050b49a6c_fld2205Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2206Inst", FIELD_dsp340050b49a6c_fld2206Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2207Inst", FIELD_dsp340050b49a6c_fld2207Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2208Inst", FIELD_dsp340050b49a6c_fld2208Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2209Inst", FIELD_dsp340050b49a6c_fld2209Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2210Inst", FIELD_dsp340050b49a6c_fld2210Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2211Inst", FIELD_dsp340050b49a6c_fld2211Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2212Inst", FIELD_dsp340050b49a6c_fld2212Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2213Inst", FIELD_dsp340050b49a6c_fld2213Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2214Inst", FIELD_dsp340050b49a6c_fld2214Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2215Inst", FIELD_dsp340050b49a6c_fld2215Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2216Inst", FIELD_dsp340050b49a6c_fld2216Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2217Inst", FIELD_dsp340050b49a6c_fld2217Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2218Inst", FIELD_dsp340050b49a6c_fld2218Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2219Inst", FIELD_dsp340050b49a6c_fld2219Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2220Inst", FIELD_dsp340050b49a6c_fld2220Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2221Inst", FIELD_dsp340050b49a6c_fld2221Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2222Inst", FIELD_dsp340050b49a6c_fld2222Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2223Inst", FIELD_dsp340050b49a6c_fld2223Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2224Inst", FIELD_dsp340050b49a6c_fld2224Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2225Inst", FIELD_dsp340050b49a6c_fld2225Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2226Inst", FIELD_dsp340050b49a6c_fld2226Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2227Inst", FIELD_dsp340050b49a6c_fld2227Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2228Inst", FIELD_dsp340050b49a6c_fld2228Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2229Inst", FIELD_dsp340050b49a6c_fld2229Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2230Inst", FIELD_dsp340050b49a6c_fld2230Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2231Inst", FIELD_dsp340050b49a6c_fld2231Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2232Inst", FIELD_dsp340050b49a6c_fld2232Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2234Inst", FIELD_dsp340050b49a6c_fld2234Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2235Inst", FIELD_dsp340050b49a6c_fld2235Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2236Inst", FIELD_dsp340050b49a6c_fld2236Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2237Inst", FIELD_dsp340050b49a6c_fld2237Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2238Inst", FIELD_dsp340050b49a6c_fld2238Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2239Inst", FIELD_dsp340050b49a6c_fld2239Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2240Inst", FIELD_dsp340050b49a6c_fld2240Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2241Inst", FIELD_dsp340050b49a6c_fld2241Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2242Inst", FIELD_dsp340050b49a6c_fld2242Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2243Inst", FIELD_dsp340050b49a6c_fld2243Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2244Inst", FIELD_dsp340050b49a6c_fld2244Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2245Inst", FIELD_dsp340050b49a6c_fld2245Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2246Inst", FIELD_dsp340050b49a6c_fld2246Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2247Inst", FIELD_dsp340050b49a6c_fld2247Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2248Inst", FIELD_dsp340050b49a6c_fld2248Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2249Inst", FIELD_dsp340050b49a6c_fld2249Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2250Inst", FIELD_dsp340050b49a6c_fld2250Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2251Inst", FIELD_dsp340050b49a6c_fld2251Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2252Inst", FIELD_dsp340050b49a6c_fld2252Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2253Inst", FIELD_dsp340050b49a6c_fld2253Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2254", FIELD_dsp340050b49a6c_fld2254, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2255Inst", FIELD_dsp340050b49a6c_fld2255Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2257Inst", FIELD_dsp340050b49a6c_fld2257Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3627Inst", FIELD_dsp340050b49a6c_fld3627Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3630Inst", FIELD_dsp340050b49a6c_fld3630Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3631Inst", FIELD_dsp340050b49a6c_fld3631Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3633Inst", FIELD_dsp340050b49a6c_fld3633Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3634", FIELD_dsp340050b49a6c_fld3634, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3635Inst", FIELD_dsp340050b49a6c_fld3635Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3636Inst", FIELD_dsp340050b49a6c_fld3636Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3637Inst", FIELD_dsp340050b49a6c_fld3637Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3638Inst", FIELD_dsp340050b49a6c_fld3638Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3639Inst", FIELD_dsp340050b49a6c_fld3639Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3640Inst", FIELD_dsp340050b49a6c_fld3640Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3642Inst", FIELD_dsp340050b49a6c_fld3642Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3643Inst", FIELD_dsp340050b49a6c_fld3643Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3644Inst", FIELD_dsp340050b49a6c_fld3644Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3645Inst", FIELD_dsp340050b49a6c_fld3645Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3647Inst", FIELD_dsp340050b49a6c_fld3647Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3648Inst", FIELD_dsp340050b49a6c_fld3648Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3649Inst", FIELD_dsp340050b49a6c_fld3649Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3650Inst", FIELD_dsp340050b49a6c_fld3650Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3651Inst", FIELD_dsp340050b49a6c_fld3651Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3653Inst", FIELD_dsp340050b49a6c_fld3653Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3654Inst", FIELD_dsp340050b49a6c_fld3654Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3655Inst", FIELD_dsp340050b49a6c_fld3655Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3656Inst", FIELD_dsp340050b49a6c_fld3656Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3657Inst", FIELD_dsp340050b49a6c_fld3657Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3658Inst", FIELD_dsp340050b49a6c_fld3658Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3659Inst", FIELD_dsp340050b49a6c_fld3659Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3660Inst", FIELD_dsp340050b49a6c_fld3660Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3661Inst", FIELD_dsp340050b49a6c_fld3661Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3662Inst", FIELD_dsp340050b49a6c_fld3662Inst, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s3", FIELD_op0_s3, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2025", FIELD_dsp340050b49a6c_fld2025, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2027", FIELD_dsp340050b49a6c_fld2027, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2258GP_slot2", FIELD_dsp340050b49a6c_fld2258GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2259GP_slot2", FIELD_dsp340050b49a6c_fld2259GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2260GP_slot2", FIELD_dsp340050b49a6c_fld2260GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2261GP_slot2", FIELD_dsp340050b49a6c_fld2261GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2262GP_slot2", FIELD_dsp340050b49a6c_fld2262GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2263GP_slot2", FIELD_dsp340050b49a6c_fld2263GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2264GP_slot2", FIELD_dsp340050b49a6c_fld2264GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2266GP_slot2", FIELD_dsp340050b49a6c_fld2266GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2267GP_slot2", FIELD_dsp340050b49a6c_fld2267GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2268GP_slot2", FIELD_dsp340050b49a6c_fld2268GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2269GP_slot2", FIELD_dsp340050b49a6c_fld2269GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2270GP_slot2", FIELD_dsp340050b49a6c_fld2270GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2271GP_slot2", FIELD_dsp340050b49a6c_fld2271GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2272", FIELD_dsp340050b49a6c_fld2272, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2273GP_slot2", FIELD_dsp340050b49a6c_fld2273GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2274GP_slot2", FIELD_dsp340050b49a6c_fld2274GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2275GP_slot2", FIELD_dsp340050b49a6c_fld2275GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2277GP_slot2", FIELD_dsp340050b49a6c_fld2277GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2278GP_slot2", FIELD_dsp340050b49a6c_fld2278GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2279GP_slot2", FIELD_dsp340050b49a6c_fld2279GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2280GP_slot2", FIELD_dsp340050b49a6c_fld2280GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2281GP_slot2", FIELD_dsp340050b49a6c_fld2281GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2282GP_slot2", FIELD_dsp340050b49a6c_fld2282GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2283GP_slot2", FIELD_dsp340050b49a6c_fld2283GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2284GP_slot2", FIELD_dsp340050b49a6c_fld2284GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2286GP_slot2", FIELD_dsp340050b49a6c_fld2286GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2287GP_slot2", FIELD_dsp340050b49a6c_fld2287GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2288GP_slot2", FIELD_dsp340050b49a6c_fld2288GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2289GP_slot2", FIELD_dsp340050b49a6c_fld2289GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2290GP_slot2", FIELD_dsp340050b49a6c_fld2290GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2291GP_slot2", FIELD_dsp340050b49a6c_fld2291GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2292GP_slot2", FIELD_dsp340050b49a6c_fld2292GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2293GP_slot2", FIELD_dsp340050b49a6c_fld2293GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2294GP_slot2", FIELD_dsp340050b49a6c_fld2294GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2295GP_slot2", FIELD_dsp340050b49a6c_fld2295GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2296GP_slot2", FIELD_dsp340050b49a6c_fld2296GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2297GP_slot2", FIELD_dsp340050b49a6c_fld2297GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2298GP_slot2", FIELD_dsp340050b49a6c_fld2298GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2299GP_slot2", FIELD_dsp340050b49a6c_fld2299GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2300GP_slot2", FIELD_dsp340050b49a6c_fld2300GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2301GP_slot2", FIELD_dsp340050b49a6c_fld2301GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2302", FIELD_dsp340050b49a6c_fld2302, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2303GP_slot2", FIELD_dsp340050b49a6c_fld2303GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2304GP_slot2", FIELD_dsp340050b49a6c_fld2304GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2305", FIELD_dsp340050b49a6c_fld2305, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2306GP_slot2", FIELD_dsp340050b49a6c_fld2306GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2308GP_slot2", FIELD_dsp340050b49a6c_fld2308GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2309GP_slot2", FIELD_dsp340050b49a6c_fld2309GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2310GP_slot2", FIELD_dsp340050b49a6c_fld2310GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2312GP_slot2", FIELD_dsp340050b49a6c_fld2312GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2313", FIELD_dsp340050b49a6c_fld2313, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2314GP_slot2", FIELD_dsp340050b49a6c_fld2314GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2316GP_slot2", FIELD_dsp340050b49a6c_fld2316GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2317GP_slot2", FIELD_dsp340050b49a6c_fld2317GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2318GP_slot2", FIELD_dsp340050b49a6c_fld2318GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2319GP_slot2", FIELD_dsp340050b49a6c_fld2319GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2320GP_slot2", FIELD_dsp340050b49a6c_fld2320GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2321GP_slot2", FIELD_dsp340050b49a6c_fld2321GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2322GP_slot2", FIELD_dsp340050b49a6c_fld2322GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2323GP_slot2", FIELD_dsp340050b49a6c_fld2323GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2324GP_slot2", FIELD_dsp340050b49a6c_fld2324GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2325GP_slot2", FIELD_dsp340050b49a6c_fld2325GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2326GP_slot2", FIELD_dsp340050b49a6c_fld2326GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2327GP_slot2", FIELD_dsp340050b49a6c_fld2327GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2328GP_slot2", FIELD_dsp340050b49a6c_fld2328GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2329GP_slot2", FIELD_dsp340050b49a6c_fld2329GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2330GP_slot2", FIELD_dsp340050b49a6c_fld2330GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2331GP_slot2", FIELD_dsp340050b49a6c_fld2331GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2332GP_slot2", FIELD_dsp340050b49a6c_fld2332GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2333GP_slot2", FIELD_dsp340050b49a6c_fld2333GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2334GP_slot2", FIELD_dsp340050b49a6c_fld2334GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2335GP_slot2", FIELD_dsp340050b49a6c_fld2335GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2336GP_slot2", FIELD_dsp340050b49a6c_fld2336GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2337GP_slot2", FIELD_dsp340050b49a6c_fld2337GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2338GP_slot2", FIELD_dsp340050b49a6c_fld2338GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2339GP_slot2", FIELD_dsp340050b49a6c_fld2339GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2340GP_slot2", FIELD_dsp340050b49a6c_fld2340GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2341GP_slot2", FIELD_dsp340050b49a6c_fld2341GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2342GP_slot2", FIELD_dsp340050b49a6c_fld2342GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2343GP_slot2", FIELD_dsp340050b49a6c_fld2343GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2344GP_slot2", FIELD_dsp340050b49a6c_fld2344GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2345GP_slot2", FIELD_dsp340050b49a6c_fld2345GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2346GP_slot2", FIELD_dsp340050b49a6c_fld2346GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2347GP_slot2", FIELD_dsp340050b49a6c_fld2347GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2348GP_slot2", FIELD_dsp340050b49a6c_fld2348GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2349GP_slot2", FIELD_dsp340050b49a6c_fld2349GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2350GP_slot2", FIELD_dsp340050b49a6c_fld2350GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2351GP_slot2", FIELD_dsp340050b49a6c_fld2351GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2352GP_slot2", FIELD_dsp340050b49a6c_fld2352GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2353GP_slot2", FIELD_dsp340050b49a6c_fld2353GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2354GP_slot2", FIELD_dsp340050b49a6c_fld2354GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2355GP_slot2", FIELD_dsp340050b49a6c_fld2355GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2356GP_slot2", FIELD_dsp340050b49a6c_fld2356GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2357GP_slot2", FIELD_dsp340050b49a6c_fld2357GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2358GP_slot2", FIELD_dsp340050b49a6c_fld2358GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2359GP_slot2", FIELD_dsp340050b49a6c_fld2359GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2361GP_slot2", FIELD_dsp340050b49a6c_fld2361GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2362GP_slot2", FIELD_dsp340050b49a6c_fld2362GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2364GP_slot2", FIELD_dsp340050b49a6c_fld2364GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2366GP_slot2", FIELD_dsp340050b49a6c_fld2366GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2368GP_slot2", FIELD_dsp340050b49a6c_fld2368GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2369GP_slot2", FIELD_dsp340050b49a6c_fld2369GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2370GP_slot2", FIELD_dsp340050b49a6c_fld2370GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2371GP_slot2", FIELD_dsp340050b49a6c_fld2371GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2372GP_slot2", FIELD_dsp340050b49a6c_fld2372GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2373GP_slot2", FIELD_dsp340050b49a6c_fld2373GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2374GP_slot2", FIELD_dsp340050b49a6c_fld2374GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2375GP_slot2", FIELD_dsp340050b49a6c_fld2375GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2376GP_slot2", FIELD_dsp340050b49a6c_fld2376GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2378GP_slot2", FIELD_dsp340050b49a6c_fld2378GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2379GP_slot2", FIELD_dsp340050b49a6c_fld2379GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2381GP_slot2", FIELD_dsp340050b49a6c_fld2381GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2383GP_slot2", FIELD_dsp340050b49a6c_fld2383GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2384", FIELD_dsp340050b49a6c_fld2384, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2385GP_slot2", FIELD_dsp340050b49a6c_fld2385GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2386", FIELD_dsp340050b49a6c_fld2386, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2387GP_slot2", FIELD_dsp340050b49a6c_fld2387GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2388GP_slot2", FIELD_dsp340050b49a6c_fld2388GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2389GP_slot2", FIELD_dsp340050b49a6c_fld2389GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3663GP_slot2", FIELD_dsp340050b49a6c_fld3663GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3664GP_slot2", FIELD_dsp340050b49a6c_fld3664GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3665GP_slot2", FIELD_dsp340050b49a6c_fld3665GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3666", FIELD_dsp340050b49a6c_fld3666, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3667GP_slot2", FIELD_dsp340050b49a6c_fld3667GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3668GP_slot2", FIELD_dsp340050b49a6c_fld3668GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3669GP_slot2", FIELD_dsp340050b49a6c_fld3669GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3670GP_slot2", FIELD_dsp340050b49a6c_fld3670GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3671GP_slot2", FIELD_dsp340050b49a6c_fld3671GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3673GP_slot2", FIELD_dsp340050b49a6c_fld3673GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3674GP_slot2", FIELD_dsp340050b49a6c_fld3674GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3675GP_slot2", FIELD_dsp340050b49a6c_fld3675GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3676GP_slot2", FIELD_dsp340050b49a6c_fld3676GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3678GP_slot2", FIELD_dsp340050b49a6c_fld3678GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3679GP_slot2", FIELD_dsp340050b49a6c_fld3679GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3680GP_slot2", FIELD_dsp340050b49a6c_fld3680GP_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s4", FIELD_op0_s4, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2026", FIELD_dsp340050b49a6c_fld2026, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2031", FIELD_dsp340050b49a6c_fld2031, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2394GP_slot1", FIELD_dsp340050b49a6c_fld2394GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2395GP_slot1", FIELD_dsp340050b49a6c_fld2395GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2397GP_slot1", FIELD_dsp340050b49a6c_fld2397GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2398GP_slot1", FIELD_dsp340050b49a6c_fld2398GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2399GP_slot1", FIELD_dsp340050b49a6c_fld2399GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2400GP_slot1", FIELD_dsp340050b49a6c_fld2400GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2402GP_slot1", FIELD_dsp340050b49a6c_fld2402GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2403GP_slot1", FIELD_dsp340050b49a6c_fld2403GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2405GP_slot1", FIELD_dsp340050b49a6c_fld2405GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3681GP_slot1", FIELD_dsp340050b49a6c_fld3681GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3683GP_slot1", FIELD_dsp340050b49a6c_fld3683GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3684GP_slot1", FIELD_dsp340050b49a6c_fld3684GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3686GP_slot1", FIELD_dsp340050b49a6c_fld3686GP_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s5", FIELD_op0_s5, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2058", FIELD_dsp340050b49a6c_fld2058, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2067", FIELD_dsp340050b49a6c_fld2067, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2407GP_slot0", FIELD_dsp340050b49a6c_fld2407GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2409GP_slot0", FIELD_dsp340050b49a6c_fld2409GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2410GP_slot0", FIELD_dsp340050b49a6c_fld2410GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2411GP_slot0", FIELD_dsp340050b49a6c_fld2411GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2412GP_slot0", FIELD_dsp340050b49a6c_fld2412GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2413GP_slot0", FIELD_dsp340050b49a6c_fld2413GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2415GP_slot0", FIELD_dsp340050b49a6c_fld2415GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2416GP_slot0", FIELD_dsp340050b49a6c_fld2416GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2417GP_slot0", FIELD_dsp340050b49a6c_fld2417GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2418GP_slot0", FIELD_dsp340050b49a6c_fld2418GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2419GP_slot0", FIELD_dsp340050b49a6c_fld2419GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2420GP_slot0", FIELD_dsp340050b49a6c_fld2420GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2422GP_slot0", FIELD_dsp340050b49a6c_fld2422GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2423GP_slot0", FIELD_dsp340050b49a6c_fld2423GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2424GP_slot0", FIELD_dsp340050b49a6c_fld2424GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2425GP_slot0", FIELD_dsp340050b49a6c_fld2425GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2426GP_slot0", FIELD_dsp340050b49a6c_fld2426GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2427GP_slot0", FIELD_dsp340050b49a6c_fld2427GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2429GP_slot0", FIELD_dsp340050b49a6c_fld2429GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2430GP_slot0", FIELD_dsp340050b49a6c_fld2430GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2431GP_slot0", FIELD_dsp340050b49a6c_fld2431GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2432GP_slot0", FIELD_dsp340050b49a6c_fld2432GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2433GP_slot0", FIELD_dsp340050b49a6c_fld2433GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2434GP_slot0", FIELD_dsp340050b49a6c_fld2434GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2435GP_slot0", FIELD_dsp340050b49a6c_fld2435GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2436GP_slot0", FIELD_dsp340050b49a6c_fld2436GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2437GP_slot0", FIELD_dsp340050b49a6c_fld2437GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2438GP_slot0", FIELD_dsp340050b49a6c_fld2438GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2439GP_slot0", FIELD_dsp340050b49a6c_fld2439GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2440GP_slot0", FIELD_dsp340050b49a6c_fld2440GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2441GP_slot0", FIELD_dsp340050b49a6c_fld2441GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2443GP_slot0", FIELD_dsp340050b49a6c_fld2443GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2444GP_slot0", FIELD_dsp340050b49a6c_fld2444GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2445", FIELD_dsp340050b49a6c_fld2445, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2447GP_slot0", FIELD_dsp340050b49a6c_fld2447GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2448", FIELD_dsp340050b49a6c_fld2448, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2449GP_slot0", FIELD_dsp340050b49a6c_fld2449GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2451GP_slot0", FIELD_dsp340050b49a6c_fld2451GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2452GP_slot0", FIELD_dsp340050b49a6c_fld2452GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2453GP_slot0", FIELD_dsp340050b49a6c_fld2453GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2454GP_slot0", FIELD_dsp340050b49a6c_fld2454GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2455GP_slot0", FIELD_dsp340050b49a6c_fld2455GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2456GP_slot0", FIELD_dsp340050b49a6c_fld2456GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2457GP_slot0", FIELD_dsp340050b49a6c_fld2457GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2458GP_slot0", FIELD_dsp340050b49a6c_fld2458GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2459GP_slot0", FIELD_dsp340050b49a6c_fld2459GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2460GP_slot0", FIELD_dsp340050b49a6c_fld2460GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2461GP_slot0", FIELD_dsp340050b49a6c_fld2461GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2462", FIELD_dsp340050b49a6c_fld2462, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2463GP_slot0", FIELD_dsp340050b49a6c_fld2463GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2464GP_slot0", FIELD_dsp340050b49a6c_fld2464GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2465GP_slot0", FIELD_dsp340050b49a6c_fld2465GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2466GP_slot0", FIELD_dsp340050b49a6c_fld2466GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2467GP_slot0", FIELD_dsp340050b49a6c_fld2467GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2468GP_slot0", FIELD_dsp340050b49a6c_fld2468GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2470GP_slot0", FIELD_dsp340050b49a6c_fld2470GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2471GP_slot0", FIELD_dsp340050b49a6c_fld2471GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2472GP_slot0", FIELD_dsp340050b49a6c_fld2472GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2473GP_slot0", FIELD_dsp340050b49a6c_fld2473GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2474GP_slot0", FIELD_dsp340050b49a6c_fld2474GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2475GP_slot0", FIELD_dsp340050b49a6c_fld2475GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2477GP_slot0", FIELD_dsp340050b49a6c_fld2477GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2479GP_slot0", FIELD_dsp340050b49a6c_fld2479GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2480GP_slot0", FIELD_dsp340050b49a6c_fld2480GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2481GP_slot0", FIELD_dsp340050b49a6c_fld2481GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2482GP_slot0", FIELD_dsp340050b49a6c_fld2482GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2483GP_slot0", FIELD_dsp340050b49a6c_fld2483GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2484GP_slot0", FIELD_dsp340050b49a6c_fld2484GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2485GP_slot0", FIELD_dsp340050b49a6c_fld2485GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2486GP_slot0", FIELD_dsp340050b49a6c_fld2486GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2487GP_slot0", FIELD_dsp340050b49a6c_fld2487GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2488GP_slot0", FIELD_dsp340050b49a6c_fld2488GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2489GP_slot0", FIELD_dsp340050b49a6c_fld2489GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2490GP_slot0", FIELD_dsp340050b49a6c_fld2490GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2491GP_slot0", FIELD_dsp340050b49a6c_fld2491GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2492GP_slot0", FIELD_dsp340050b49a6c_fld2492GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2493GP_slot0", FIELD_dsp340050b49a6c_fld2493GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2494GP_slot0", FIELD_dsp340050b49a6c_fld2494GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2495GP_slot0", FIELD_dsp340050b49a6c_fld2495GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2496GP_slot0", FIELD_dsp340050b49a6c_fld2496GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2497GP_slot0", FIELD_dsp340050b49a6c_fld2497GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2498GP_slot0", FIELD_dsp340050b49a6c_fld2498GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2499GP_slot0", FIELD_dsp340050b49a6c_fld2499GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2500GP_slot0", FIELD_dsp340050b49a6c_fld2500GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2501GP_slot0", FIELD_dsp340050b49a6c_fld2501GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2502GP_slot0", FIELD_dsp340050b49a6c_fld2502GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2503GP_slot0", FIELD_dsp340050b49a6c_fld2503GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2504GP_slot0", FIELD_dsp340050b49a6c_fld2504GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2505GP_slot0", FIELD_dsp340050b49a6c_fld2505GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2506GP_slot0", FIELD_dsp340050b49a6c_fld2506GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2507GP_slot0", FIELD_dsp340050b49a6c_fld2507GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2508GP_slot0", FIELD_dsp340050b49a6c_fld2508GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2509GP_slot0", FIELD_dsp340050b49a6c_fld2509GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2510GP_slot0", FIELD_dsp340050b49a6c_fld2510GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2512GP_slot0", FIELD_dsp340050b49a6c_fld2512GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2514GP_slot0", FIELD_dsp340050b49a6c_fld2514GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2515GP_slot0", FIELD_dsp340050b49a6c_fld2515GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2516GP_slot0", FIELD_dsp340050b49a6c_fld2516GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2517GP_slot0", FIELD_dsp340050b49a6c_fld2517GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2518GP_slot0", FIELD_dsp340050b49a6c_fld2518GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2519GP_slot0", FIELD_dsp340050b49a6c_fld2519GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2520GP_slot0", FIELD_dsp340050b49a6c_fld2520GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2521GP_slot0", FIELD_dsp340050b49a6c_fld2521GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2523GP_slot0", FIELD_dsp340050b49a6c_fld2523GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2524GP_slot0", FIELD_dsp340050b49a6c_fld2524GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2526GP_slot0", FIELD_dsp340050b49a6c_fld2526GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2527GP_slot0", FIELD_dsp340050b49a6c_fld2527GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2528GP_slot0", FIELD_dsp340050b49a6c_fld2528GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2529GP_slot0", FIELD_dsp340050b49a6c_fld2529GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2530", FIELD_dsp340050b49a6c_fld2530, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2531GP_slot0", FIELD_dsp340050b49a6c_fld2531GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3688GP_slot0", FIELD_dsp340050b49a6c_fld3688GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3689GP_slot0", FIELD_dsp340050b49a6c_fld3689GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3690GP_slot0", FIELD_dsp340050b49a6c_fld3690GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3691GP_slot0", FIELD_dsp340050b49a6c_fld3691GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3692GP_slot0", FIELD_dsp340050b49a6c_fld3692GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3693GP_slot0", FIELD_dsp340050b49a6c_fld3693GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3695GP_slot0", FIELD_dsp340050b49a6c_fld3695GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3696GP_slot0", FIELD_dsp340050b49a6c_fld3696GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3697GP_slot0", FIELD_dsp340050b49a6c_fld3697GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3698GP_slot0", FIELD_dsp340050b49a6c_fld3698GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3699GP_slot0", FIELD_dsp340050b49a6c_fld3699GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3700GP_slot0", FIELD_dsp340050b49a6c_fld3700GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3702GP_slot0", FIELD_dsp340050b49a6c_fld3702GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3703GP_slot0", FIELD_dsp340050b49a6c_fld3703GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3705GP_slot0", FIELD_dsp340050b49a6c_fld3705GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3706GP_slot0", FIELD_dsp340050b49a6c_fld3706GP_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s6", FIELD_op0_s6, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2532DOT_slot2", FIELD_dsp340050b49a6c_fld2532DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2533DOT_slot2", FIELD_dsp340050b49a6c_fld2533DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2534DOT_slot2", FIELD_dsp340050b49a6c_fld2534DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2535DOT_slot2", FIELD_dsp340050b49a6c_fld2535DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2536DOT_slot2", FIELD_dsp340050b49a6c_fld2536DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2537DOT_slot2", FIELD_dsp340050b49a6c_fld2537DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2538DOT_slot2", FIELD_dsp340050b49a6c_fld2538DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2539DOT_slot2", FIELD_dsp340050b49a6c_fld2539DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2540DOT_slot2", FIELD_dsp340050b49a6c_fld2540DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2541DOT_slot2", FIELD_dsp340050b49a6c_fld2541DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2542DOT_slot2", FIELD_dsp340050b49a6c_fld2542DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2543DOT_slot2", FIELD_dsp340050b49a6c_fld2543DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2544DOT_slot2", FIELD_dsp340050b49a6c_fld2544DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2545DOT_slot2", FIELD_dsp340050b49a6c_fld2545DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2546DOT_slot2", FIELD_dsp340050b49a6c_fld2546DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2547DOT_slot2", FIELD_dsp340050b49a6c_fld2547DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2548DOT_slot2", FIELD_dsp340050b49a6c_fld2548DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2549DOT_slot2", FIELD_dsp340050b49a6c_fld2549DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2550DOT_slot2", FIELD_dsp340050b49a6c_fld2550DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2551DOT_slot2", FIELD_dsp340050b49a6c_fld2551DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2552DOT_slot2", FIELD_dsp340050b49a6c_fld2552DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2553DOT_slot2", FIELD_dsp340050b49a6c_fld2553DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2554DOT_slot2", FIELD_dsp340050b49a6c_fld2554DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2555DOT_slot2", FIELD_dsp340050b49a6c_fld2555DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2556DOT_slot2", FIELD_dsp340050b49a6c_fld2556DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2557DOT_slot2", FIELD_dsp340050b49a6c_fld2557DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2558DOT_slot2", FIELD_dsp340050b49a6c_fld2558DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2559DOT_slot2", FIELD_dsp340050b49a6c_fld2559DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2560DOT_slot2", FIELD_dsp340050b49a6c_fld2560DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2561DOT_slot2", FIELD_dsp340050b49a6c_fld2561DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2562DOT_slot2", FIELD_dsp340050b49a6c_fld2562DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2563DOT_slot2", FIELD_dsp340050b49a6c_fld2563DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2564DOT_slot2", FIELD_dsp340050b49a6c_fld2564DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2565DOT_slot2", FIELD_dsp340050b49a6c_fld2565DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2566DOT_slot2", FIELD_dsp340050b49a6c_fld2566DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2567DOT_slot2", FIELD_dsp340050b49a6c_fld2567DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2568DOT_slot2", FIELD_dsp340050b49a6c_fld2568DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2569DOT_slot2", FIELD_dsp340050b49a6c_fld2569DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2571DOT_slot2", FIELD_dsp340050b49a6c_fld2571DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2572DOT_slot2", FIELD_dsp340050b49a6c_fld2572DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2573DOT_slot2", FIELD_dsp340050b49a6c_fld2573DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2574DOT_slot2", FIELD_dsp340050b49a6c_fld2574DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2575DOT_slot2", FIELD_dsp340050b49a6c_fld2575DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2576DOT_slot2", FIELD_dsp340050b49a6c_fld2576DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2577DOT_slot2", FIELD_dsp340050b49a6c_fld2577DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2578", FIELD_dsp340050b49a6c_fld2578, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2579DOT_slot2", FIELD_dsp340050b49a6c_fld2579DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2580DOT_slot2", FIELD_dsp340050b49a6c_fld2580DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2581DOT_slot2", FIELD_dsp340050b49a6c_fld2581DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2582DOT_slot2", FIELD_dsp340050b49a6c_fld2582DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2583DOT_slot2", FIELD_dsp340050b49a6c_fld2583DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2584DOT_slot2", FIELD_dsp340050b49a6c_fld2584DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2585DOT_slot2", FIELD_dsp340050b49a6c_fld2585DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2586DOT_slot2", FIELD_dsp340050b49a6c_fld2586DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2587DOT_slot2", FIELD_dsp340050b49a6c_fld2587DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2588DOT_slot2", FIELD_dsp340050b49a6c_fld2588DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2589DOT_slot2", FIELD_dsp340050b49a6c_fld2589DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2590DOT_slot2", FIELD_dsp340050b49a6c_fld2590DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2591DOT_slot2", FIELD_dsp340050b49a6c_fld2591DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2592DOT_slot2", FIELD_dsp340050b49a6c_fld2592DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2595DOT_slot2", FIELD_dsp340050b49a6c_fld2595DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2596DOT_slot2", FIELD_dsp340050b49a6c_fld2596DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2598DOT_slot2", FIELD_dsp340050b49a6c_fld2598DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2599DOT_slot2", FIELD_dsp340050b49a6c_fld2599DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2601DOT_slot2", FIELD_dsp340050b49a6c_fld2601DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2602DOT_slot2", FIELD_dsp340050b49a6c_fld2602DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2604DOT_slot2", FIELD_dsp340050b49a6c_fld2604DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2605", FIELD_dsp340050b49a6c_fld2605, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2606DOT_slot2", FIELD_dsp340050b49a6c_fld2606DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2608DOT_slot2", FIELD_dsp340050b49a6c_fld2608DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2609DOT_slot2", FIELD_dsp340050b49a6c_fld2609DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2610DOT_slot2", FIELD_dsp340050b49a6c_fld2610DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2611DOT_slot2", FIELD_dsp340050b49a6c_fld2611DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2612", FIELD_dsp340050b49a6c_fld2612, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2613DOT_slot2", FIELD_dsp340050b49a6c_fld2613DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2614", FIELD_dsp340050b49a6c_fld2614, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2615DOT_slot2", FIELD_dsp340050b49a6c_fld2615DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2616DOT_slot2", FIELD_dsp340050b49a6c_fld2616DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2617DOT_slot2", FIELD_dsp340050b49a6c_fld2617DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2618DOT_slot2", FIELD_dsp340050b49a6c_fld2618DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2619DOT_slot2", FIELD_dsp340050b49a6c_fld2619DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2620DOT_slot2", FIELD_dsp340050b49a6c_fld2620DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2621DOT_slot2", FIELD_dsp340050b49a6c_fld2621DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2622DOT_slot2", FIELD_dsp340050b49a6c_fld2622DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2623DOT_slot2", FIELD_dsp340050b49a6c_fld2623DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2624DOT_slot2", FIELD_dsp340050b49a6c_fld2624DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2625", FIELD_dsp340050b49a6c_fld2625, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2626DOT_slot2", FIELD_dsp340050b49a6c_fld2626DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2628DOT_slot2", FIELD_dsp340050b49a6c_fld2628DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2630DOT_slot2", FIELD_dsp340050b49a6c_fld2630DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2632DOT_slot2", FIELD_dsp340050b49a6c_fld2632DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2633DOT_slot2", FIELD_dsp340050b49a6c_fld2633DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2635DOT_slot2", FIELD_dsp340050b49a6c_fld2635DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2636DOT_slot2", FIELD_dsp340050b49a6c_fld2636DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2637DOT_slot2", FIELD_dsp340050b49a6c_fld2637DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2640DOT_slot2", FIELD_dsp340050b49a6c_fld2640DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2641DOT_slot2", FIELD_dsp340050b49a6c_fld2641DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2642DOT_slot2", FIELD_dsp340050b49a6c_fld2642DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2643DOT_slot2", FIELD_dsp340050b49a6c_fld2643DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2644DOT_slot2", FIELD_dsp340050b49a6c_fld2644DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2645DOT_slot2", FIELD_dsp340050b49a6c_fld2645DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2646DOT_slot2", FIELD_dsp340050b49a6c_fld2646DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2647DOT_slot2", FIELD_dsp340050b49a6c_fld2647DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2648DOT_slot2", FIELD_dsp340050b49a6c_fld2648DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2649DOT_slot2", FIELD_dsp340050b49a6c_fld2649DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2650DOT_slot2", FIELD_dsp340050b49a6c_fld2650DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2651DOT_slot2", FIELD_dsp340050b49a6c_fld2651DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2652DOT_slot2", FIELD_dsp340050b49a6c_fld2652DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2654DOT_slot2", FIELD_dsp340050b49a6c_fld2654DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2655DOT_slot2", FIELD_dsp340050b49a6c_fld2655DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2656DOT_slot2", FIELD_dsp340050b49a6c_fld2656DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2657DOT_slot2", FIELD_dsp340050b49a6c_fld2657DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2658DOT_slot2", FIELD_dsp340050b49a6c_fld2658DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3708DOT_slot2", FIELD_dsp340050b49a6c_fld3708DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3709DOT_slot2", FIELD_dsp340050b49a6c_fld3709DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3710DOT_slot2", FIELD_dsp340050b49a6c_fld3710DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3711DOT_slot2", FIELD_dsp340050b49a6c_fld3711DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3712", FIELD_dsp340050b49a6c_fld3712, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3713DOT_slot2", FIELD_dsp340050b49a6c_fld3713DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3714DOT_slot2", FIELD_dsp340050b49a6c_fld3714DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3715", FIELD_dsp340050b49a6c_fld3715, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3716DOT_slot2", FIELD_dsp340050b49a6c_fld3716DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3717DOT_slot2", FIELD_dsp340050b49a6c_fld3717DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3718", FIELD_dsp340050b49a6c_fld3718, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3719DOT_slot2", FIELD_dsp340050b49a6c_fld3719DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3721DOT_slot2", FIELD_dsp340050b49a6c_fld3721DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3722", FIELD_dsp340050b49a6c_fld3722, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3723DOT_slot2", FIELD_dsp340050b49a6c_fld3723DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3724DOT_slot2", FIELD_dsp340050b49a6c_fld3724DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3725DOT_slot2", FIELD_dsp340050b49a6c_fld3725DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3726DOT_slot2", FIELD_dsp340050b49a6c_fld3726DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3727DOT_slot2", FIELD_dsp340050b49a6c_fld3727DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3728DOT_slot2", FIELD_dsp340050b49a6c_fld3728DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3729DOT_slot2", FIELD_dsp340050b49a6c_fld3729DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3731DOT_slot2", FIELD_dsp340050b49a6c_fld3731DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3732DOT_slot2", FIELD_dsp340050b49a6c_fld3732DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3733DOT_slot2", FIELD_dsp340050b49a6c_fld3733DOT_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s7", FIELD_op0_s7, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3734DOT_slot1", FIELD_dsp340050b49a6c_fld3734DOT_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s8", FIELD_op0_s8, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2068", FIELD_dsp340050b49a6c_fld2068, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2666DOT_slot0", FIELD_dsp340050b49a6c_fld2666DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2667DOT_slot0", FIELD_dsp340050b49a6c_fld2667DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2668DOT_slot0", FIELD_dsp340050b49a6c_fld2668DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2669DOT_slot0", FIELD_dsp340050b49a6c_fld2669DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2671DOT_slot0", FIELD_dsp340050b49a6c_fld2671DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2672DOT_slot0", FIELD_dsp340050b49a6c_fld2672DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2673DOT_slot0", FIELD_dsp340050b49a6c_fld2673DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2674DOT_slot0", FIELD_dsp340050b49a6c_fld2674DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2675DOT_slot0", FIELD_dsp340050b49a6c_fld2675DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2676DOT_slot0", FIELD_dsp340050b49a6c_fld2676DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2677DOT_slot0", FIELD_dsp340050b49a6c_fld2677DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2678DOT_slot0", FIELD_dsp340050b49a6c_fld2678DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2679DOT_slot0", FIELD_dsp340050b49a6c_fld2679DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2680DOT_slot0", FIELD_dsp340050b49a6c_fld2680DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2681DOT_slot0", FIELD_dsp340050b49a6c_fld2681DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2682DOT_slot0", FIELD_dsp340050b49a6c_fld2682DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2683DOT_slot0", FIELD_dsp340050b49a6c_fld2683DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2684DOT_slot0", FIELD_dsp340050b49a6c_fld2684DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2685DOT_slot0", FIELD_dsp340050b49a6c_fld2685DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2686DOT_slot0", FIELD_dsp340050b49a6c_fld2686DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2688DOT_slot0", FIELD_dsp340050b49a6c_fld2688DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2689DOT_slot0", FIELD_dsp340050b49a6c_fld2689DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2690DOT_slot0", FIELD_dsp340050b49a6c_fld2690DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2692DOT_slot0", FIELD_dsp340050b49a6c_fld2692DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2693DOT_slot0", FIELD_dsp340050b49a6c_fld2693DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2695DOT_slot0", FIELD_dsp340050b49a6c_fld2695DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2697DOT_slot0", FIELD_dsp340050b49a6c_fld2697DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2699DOT_slot0", FIELD_dsp340050b49a6c_fld2699DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2700DOT_slot0", FIELD_dsp340050b49a6c_fld2700DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2701DOT_slot0", FIELD_dsp340050b49a6c_fld2701DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2702DOT_slot0", FIELD_dsp340050b49a6c_fld2702DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2703DOT_slot0", FIELD_dsp340050b49a6c_fld2703DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2704DOT_slot0", FIELD_dsp340050b49a6c_fld2704DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2705DOT_slot0", FIELD_dsp340050b49a6c_fld2705DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3735DOT_slot0", FIELD_dsp340050b49a6c_fld3735DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3736", FIELD_dsp340050b49a6c_fld3736, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3737DOT_slot0", FIELD_dsp340050b49a6c_fld3737DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3738DOT_slot0", FIELD_dsp340050b49a6c_fld3738DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3739DOT_slot0", FIELD_dsp340050b49a6c_fld3739DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3740DOT_slot0", FIELD_dsp340050b49a6c_fld3740DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3741DOT_slot0", FIELD_dsp340050b49a6c_fld3741DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3742DOT_slot0", FIELD_dsp340050b49a6c_fld3742DOT_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s9", FIELD_op0_s9, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2706PQ_slot2", FIELD_dsp340050b49a6c_fld2706PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2707PQ_slot2", FIELD_dsp340050b49a6c_fld2707PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2708PQ_slot2", FIELD_dsp340050b49a6c_fld2708PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2709PQ_slot2", FIELD_dsp340050b49a6c_fld2709PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2710PQ_slot2", FIELD_dsp340050b49a6c_fld2710PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2711PQ_slot2", FIELD_dsp340050b49a6c_fld2711PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2713PQ_slot2", FIELD_dsp340050b49a6c_fld2713PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2714PQ_slot2", FIELD_dsp340050b49a6c_fld2714PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2715PQ_slot2", FIELD_dsp340050b49a6c_fld2715PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2717PQ_slot2", FIELD_dsp340050b49a6c_fld2717PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2718PQ_slot2", FIELD_dsp340050b49a6c_fld2718PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2719PQ_slot2", FIELD_dsp340050b49a6c_fld2719PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2721PQ_slot2", FIELD_dsp340050b49a6c_fld2721PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2722PQ_slot2", FIELD_dsp340050b49a6c_fld2722PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2723PQ_slot2", FIELD_dsp340050b49a6c_fld2723PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2724PQ_slot2", FIELD_dsp340050b49a6c_fld2724PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2725PQ_slot2", FIELD_dsp340050b49a6c_fld2725PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2726PQ_slot2", FIELD_dsp340050b49a6c_fld2726PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2727PQ_slot2", FIELD_dsp340050b49a6c_fld2727PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2728PQ_slot2", FIELD_dsp340050b49a6c_fld2728PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2729PQ_slot2", FIELD_dsp340050b49a6c_fld2729PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2730PQ_slot2", FIELD_dsp340050b49a6c_fld2730PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2731PQ_slot2", FIELD_dsp340050b49a6c_fld2731PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2732PQ_slot2", FIELD_dsp340050b49a6c_fld2732PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2733PQ_slot2", FIELD_dsp340050b49a6c_fld2733PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2734PQ_slot2", FIELD_dsp340050b49a6c_fld2734PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2735PQ_slot2", FIELD_dsp340050b49a6c_fld2735PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2736PQ_slot2", FIELD_dsp340050b49a6c_fld2736PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2737", FIELD_dsp340050b49a6c_fld2737, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2738PQ_slot2", FIELD_dsp340050b49a6c_fld2738PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2739PQ_slot2", FIELD_dsp340050b49a6c_fld2739PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2741PQ_slot2", FIELD_dsp340050b49a6c_fld2741PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2742PQ_slot2", FIELD_dsp340050b49a6c_fld2742PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2743PQ_slot2", FIELD_dsp340050b49a6c_fld2743PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2746PQ_slot2", FIELD_dsp340050b49a6c_fld2746PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2747PQ_slot2", FIELD_dsp340050b49a6c_fld2747PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2748PQ_slot2", FIELD_dsp340050b49a6c_fld2748PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2750PQ_slot2", FIELD_dsp340050b49a6c_fld2750PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2751PQ_slot2", FIELD_dsp340050b49a6c_fld2751PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2752PQ_slot2", FIELD_dsp340050b49a6c_fld2752PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2753PQ_slot2", FIELD_dsp340050b49a6c_fld2753PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2754PQ_slot2", FIELD_dsp340050b49a6c_fld2754PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2755PQ_slot2", FIELD_dsp340050b49a6c_fld2755PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2756PQ_slot2", FIELD_dsp340050b49a6c_fld2756PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2757PQ_slot2", FIELD_dsp340050b49a6c_fld2757PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2758PQ_slot2", FIELD_dsp340050b49a6c_fld2758PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2759PQ_slot2", FIELD_dsp340050b49a6c_fld2759PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2760PQ_slot2", FIELD_dsp340050b49a6c_fld2760PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2761PQ_slot2", FIELD_dsp340050b49a6c_fld2761PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2762PQ_slot2", FIELD_dsp340050b49a6c_fld2762PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2763PQ_slot2", FIELD_dsp340050b49a6c_fld2763PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2764PQ_slot2", FIELD_dsp340050b49a6c_fld2764PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2765PQ_slot2", FIELD_dsp340050b49a6c_fld2765PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2766PQ_slot2", FIELD_dsp340050b49a6c_fld2766PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2767PQ_slot2", FIELD_dsp340050b49a6c_fld2767PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2768PQ_slot2", FIELD_dsp340050b49a6c_fld2768PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2769PQ_slot2", FIELD_dsp340050b49a6c_fld2769PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2770PQ_slot2", FIELD_dsp340050b49a6c_fld2770PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2771PQ_slot2", FIELD_dsp340050b49a6c_fld2771PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2772PQ_slot2", FIELD_dsp340050b49a6c_fld2772PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2773PQ_slot2", FIELD_dsp340050b49a6c_fld2773PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2774PQ_slot2", FIELD_dsp340050b49a6c_fld2774PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2775PQ_slot2", FIELD_dsp340050b49a6c_fld2775PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2776PQ_slot2", FIELD_dsp340050b49a6c_fld2776PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2777PQ_slot2", FIELD_dsp340050b49a6c_fld2777PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2778PQ_slot2", FIELD_dsp340050b49a6c_fld2778PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2779PQ_slot2", FIELD_dsp340050b49a6c_fld2779PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2780PQ_slot2", FIELD_dsp340050b49a6c_fld2780PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2781PQ_slot2", FIELD_dsp340050b49a6c_fld2781PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2782PQ_slot2", FIELD_dsp340050b49a6c_fld2782PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2783PQ_slot2", FIELD_dsp340050b49a6c_fld2783PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2784PQ_slot2", FIELD_dsp340050b49a6c_fld2784PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2785PQ_slot2", FIELD_dsp340050b49a6c_fld2785PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2786PQ_slot2", FIELD_dsp340050b49a6c_fld2786PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2787PQ_slot2", FIELD_dsp340050b49a6c_fld2787PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2788PQ_slot2", FIELD_dsp340050b49a6c_fld2788PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2789PQ_slot2", FIELD_dsp340050b49a6c_fld2789PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2790PQ_slot2", FIELD_dsp340050b49a6c_fld2790PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2791PQ_slot2", FIELD_dsp340050b49a6c_fld2791PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2792PQ_slot2", FIELD_dsp340050b49a6c_fld2792PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2793PQ_slot2", FIELD_dsp340050b49a6c_fld2793PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2795PQ_slot2", FIELD_dsp340050b49a6c_fld2795PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2796PQ_slot2", FIELD_dsp340050b49a6c_fld2796PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2798PQ_slot2", FIELD_dsp340050b49a6c_fld2798PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2801PQ_slot2", FIELD_dsp340050b49a6c_fld2801PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2803PQ_slot2", FIELD_dsp340050b49a6c_fld2803PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2805PQ_slot2", FIELD_dsp340050b49a6c_fld2805PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2806PQ_slot2", FIELD_dsp340050b49a6c_fld2806PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2807PQ_slot2", FIELD_dsp340050b49a6c_fld2807PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2808PQ_slot2", FIELD_dsp340050b49a6c_fld2808PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2809PQ_slot2", FIELD_dsp340050b49a6c_fld2809PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2810PQ_slot2", FIELD_dsp340050b49a6c_fld2810PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2811PQ_slot2", FIELD_dsp340050b49a6c_fld2811PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2812PQ_slot2", FIELD_dsp340050b49a6c_fld2812PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2814PQ_slot2", FIELD_dsp340050b49a6c_fld2814PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2816PQ_slot2", FIELD_dsp340050b49a6c_fld2816PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2817", FIELD_dsp340050b49a6c_fld2817, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2818PQ_slot2", FIELD_dsp340050b49a6c_fld2818PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2819", FIELD_dsp340050b49a6c_fld2819, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2820PQ_slot2", FIELD_dsp340050b49a6c_fld2820PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2821PQ_slot2", FIELD_dsp340050b49a6c_fld2821PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2823PQ_slot2", FIELD_dsp340050b49a6c_fld2823PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3744PQ_slot2", FIELD_dsp340050b49a6c_fld3744PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3745PQ_slot2", FIELD_dsp340050b49a6c_fld3745PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3746PQ_slot2", FIELD_dsp340050b49a6c_fld3746PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3747PQ_slot2", FIELD_dsp340050b49a6c_fld3747PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3748", FIELD_dsp340050b49a6c_fld3748, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3749PQ_slot2", FIELD_dsp340050b49a6c_fld3749PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3750PQ_slot2", FIELD_dsp340050b49a6c_fld3750PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3751PQ_slot2", FIELD_dsp340050b49a6c_fld3751PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3752PQ_slot2", FIELD_dsp340050b49a6c_fld3752PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3753PQ_slot2", FIELD_dsp340050b49a6c_fld3753PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3754PQ_slot2", FIELD_dsp340050b49a6c_fld3754PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3756PQ_slot2", FIELD_dsp340050b49a6c_fld3756PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3757PQ_slot2", FIELD_dsp340050b49a6c_fld3757PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3758PQ_slot2", FIELD_dsp340050b49a6c_fld3758PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3759PQ_slot2", FIELD_dsp340050b49a6c_fld3759PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3760PQ_slot2", FIELD_dsp340050b49a6c_fld3760PQ_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s10", FIELD_op0_s10, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2825PQ_slot1", FIELD_dsp340050b49a6c_fld2825PQ_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2826PQ_slot1", FIELD_dsp340050b49a6c_fld2826PQ_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3761PQ_slot1", FIELD_dsp340050b49a6c_fld3761PQ_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s11", FIELD_op0_s11, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2059", FIELD_dsp340050b49a6c_fld2059, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2069", FIELD_dsp340050b49a6c_fld2069, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2827PQ_slot0", FIELD_dsp340050b49a6c_fld2827PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2829PQ_slot0", FIELD_dsp340050b49a6c_fld2829PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2830PQ_slot0", FIELD_dsp340050b49a6c_fld2830PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2831PQ_slot0", FIELD_dsp340050b49a6c_fld2831PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2832PQ_slot0", FIELD_dsp340050b49a6c_fld2832PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2833PQ_slot0", FIELD_dsp340050b49a6c_fld2833PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2835PQ_slot0", FIELD_dsp340050b49a6c_fld2835PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2836PQ_slot0", FIELD_dsp340050b49a6c_fld2836PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2837PQ_slot0", FIELD_dsp340050b49a6c_fld2837PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2838PQ_slot0", FIELD_dsp340050b49a6c_fld2838PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2839PQ_slot0", FIELD_dsp340050b49a6c_fld2839PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2840PQ_slot0", FIELD_dsp340050b49a6c_fld2840PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2842PQ_slot0", FIELD_dsp340050b49a6c_fld2842PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2843PQ_slot0", FIELD_dsp340050b49a6c_fld2843PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2844PQ_slot0", FIELD_dsp340050b49a6c_fld2844PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2845PQ_slot0", FIELD_dsp340050b49a6c_fld2845PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2846PQ_slot0", FIELD_dsp340050b49a6c_fld2846PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2847PQ_slot0", FIELD_dsp340050b49a6c_fld2847PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2849PQ_slot0", FIELD_dsp340050b49a6c_fld2849PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2850PQ_slot0", FIELD_dsp340050b49a6c_fld2850PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2851PQ_slot0", FIELD_dsp340050b49a6c_fld2851PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2852PQ_slot0", FIELD_dsp340050b49a6c_fld2852PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2853PQ_slot0", FIELD_dsp340050b49a6c_fld2853PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2854PQ_slot0", FIELD_dsp340050b49a6c_fld2854PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2855PQ_slot0", FIELD_dsp340050b49a6c_fld2855PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2856PQ_slot0", FIELD_dsp340050b49a6c_fld2856PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2857PQ_slot0", FIELD_dsp340050b49a6c_fld2857PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2858PQ_slot0", FIELD_dsp340050b49a6c_fld2858PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2859PQ_slot0", FIELD_dsp340050b49a6c_fld2859PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2860PQ_slot0", FIELD_dsp340050b49a6c_fld2860PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2861PQ_slot0", FIELD_dsp340050b49a6c_fld2861PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2863PQ_slot0", FIELD_dsp340050b49a6c_fld2863PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2864PQ_slot0", FIELD_dsp340050b49a6c_fld2864PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2865", FIELD_dsp340050b49a6c_fld2865, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2867PQ_slot0", FIELD_dsp340050b49a6c_fld2867PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2869PQ_slot0", FIELD_dsp340050b49a6c_fld2869PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2871PQ_slot0", FIELD_dsp340050b49a6c_fld2871PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2872PQ_slot0", FIELD_dsp340050b49a6c_fld2872PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2873PQ_slot0", FIELD_dsp340050b49a6c_fld2873PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2874PQ_slot0", FIELD_dsp340050b49a6c_fld2874PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2875PQ_slot0", FIELD_dsp340050b49a6c_fld2875PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2876PQ_slot0", FIELD_dsp340050b49a6c_fld2876PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2877PQ_slot0", FIELD_dsp340050b49a6c_fld2877PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2878PQ_slot0", FIELD_dsp340050b49a6c_fld2878PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2879PQ_slot0", FIELD_dsp340050b49a6c_fld2879PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2880PQ_slot0", FIELD_dsp340050b49a6c_fld2880PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2881PQ_slot0", FIELD_dsp340050b49a6c_fld2881PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2882", FIELD_dsp340050b49a6c_fld2882, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2883PQ_slot0", FIELD_dsp340050b49a6c_fld2883PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2884PQ_slot0", FIELD_dsp340050b49a6c_fld2884PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2885PQ_slot0", FIELD_dsp340050b49a6c_fld2885PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2886PQ_slot0", FIELD_dsp340050b49a6c_fld2886PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2887PQ_slot0", FIELD_dsp340050b49a6c_fld2887PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2888PQ_slot0", FIELD_dsp340050b49a6c_fld2888PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2890PQ_slot0", FIELD_dsp340050b49a6c_fld2890PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2891PQ_slot0", FIELD_dsp340050b49a6c_fld2891PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2892PQ_slot0", FIELD_dsp340050b49a6c_fld2892PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2893PQ_slot0", FIELD_dsp340050b49a6c_fld2893PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2894PQ_slot0", FIELD_dsp340050b49a6c_fld2894PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2895PQ_slot0", FIELD_dsp340050b49a6c_fld2895PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2897PQ_slot0", FIELD_dsp340050b49a6c_fld2897PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2899PQ_slot0", FIELD_dsp340050b49a6c_fld2899PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2900PQ_slot0", FIELD_dsp340050b49a6c_fld2900PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2901PQ_slot0", FIELD_dsp340050b49a6c_fld2901PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2902PQ_slot0", FIELD_dsp340050b49a6c_fld2902PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2903PQ_slot0", FIELD_dsp340050b49a6c_fld2903PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2904PQ_slot0", FIELD_dsp340050b49a6c_fld2904PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2905PQ_slot0", FIELD_dsp340050b49a6c_fld2905PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2906PQ_slot0", FIELD_dsp340050b49a6c_fld2906PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2907PQ_slot0", FIELD_dsp340050b49a6c_fld2907PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2908PQ_slot0", FIELD_dsp340050b49a6c_fld2908PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2909PQ_slot0", FIELD_dsp340050b49a6c_fld2909PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2910PQ_slot0", FIELD_dsp340050b49a6c_fld2910PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2911PQ_slot0", FIELD_dsp340050b49a6c_fld2911PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2912PQ_slot0", FIELD_dsp340050b49a6c_fld2912PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2913PQ_slot0", FIELD_dsp340050b49a6c_fld2913PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2914PQ_slot0", FIELD_dsp340050b49a6c_fld2914PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2915PQ_slot0", FIELD_dsp340050b49a6c_fld2915PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2916PQ_slot0", FIELD_dsp340050b49a6c_fld2916PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2917PQ_slot0", FIELD_dsp340050b49a6c_fld2917PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2918PQ_slot0", FIELD_dsp340050b49a6c_fld2918PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2919PQ_slot0", FIELD_dsp340050b49a6c_fld2919PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2920PQ_slot0", FIELD_dsp340050b49a6c_fld2920PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2921PQ_slot0", FIELD_dsp340050b49a6c_fld2921PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2922PQ_slot0", FIELD_dsp340050b49a6c_fld2922PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2923PQ_slot0", FIELD_dsp340050b49a6c_fld2923PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2924PQ_slot0", FIELD_dsp340050b49a6c_fld2924PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2925PQ_slot0", FIELD_dsp340050b49a6c_fld2925PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2926PQ_slot0", FIELD_dsp340050b49a6c_fld2926PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2927PQ_slot0", FIELD_dsp340050b49a6c_fld2927PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2928PQ_slot0", FIELD_dsp340050b49a6c_fld2928PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2929PQ_slot0", FIELD_dsp340050b49a6c_fld2929PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2930PQ_slot0", FIELD_dsp340050b49a6c_fld2930PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2932PQ_slot0", FIELD_dsp340050b49a6c_fld2932PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2934PQ_slot0", FIELD_dsp340050b49a6c_fld2934PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2935PQ_slot0", FIELD_dsp340050b49a6c_fld2935PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2936PQ_slot0", FIELD_dsp340050b49a6c_fld2936PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2937PQ_slot0", FIELD_dsp340050b49a6c_fld2937PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2939PQ_slot0", FIELD_dsp340050b49a6c_fld2939PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2940", FIELD_dsp340050b49a6c_fld2940, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2941PQ_slot0", FIELD_dsp340050b49a6c_fld2941PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2942PQ_slot0", FIELD_dsp340050b49a6c_fld2942PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2943PQ_slot0", FIELD_dsp340050b49a6c_fld2943PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2945PQ_slot0", FIELD_dsp340050b49a6c_fld2945PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2946PQ_slot0", FIELD_dsp340050b49a6c_fld2946PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2947PQ_slot0", FIELD_dsp340050b49a6c_fld2947PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2948PQ_slot0", FIELD_dsp340050b49a6c_fld2948PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2949PQ_slot0", FIELD_dsp340050b49a6c_fld2949PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2950PQ_slot0", FIELD_dsp340050b49a6c_fld2950PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3763PQ_slot0", FIELD_dsp340050b49a6c_fld3763PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3764PQ_slot0", FIELD_dsp340050b49a6c_fld3764PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3765PQ_slot0", FIELD_dsp340050b49a6c_fld3765PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3766PQ_slot0", FIELD_dsp340050b49a6c_fld3766PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3767PQ_slot0", FIELD_dsp340050b49a6c_fld3767PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3768PQ_slot0", FIELD_dsp340050b49a6c_fld3768PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3769PQ_slot0", FIELD_dsp340050b49a6c_fld3769PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3770PQ_slot0", FIELD_dsp340050b49a6c_fld3770PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3771PQ_slot0", FIELD_dsp340050b49a6c_fld3771PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3772", FIELD_dsp340050b49a6c_fld3772, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3773PQ_slot0", FIELD_dsp340050b49a6c_fld3773PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3775PQ_slot0", FIELD_dsp340050b49a6c_fld3775PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3776PQ_slot0", FIELD_dsp340050b49a6c_fld3776PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3777PQ_slot0", FIELD_dsp340050b49a6c_fld3777PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3778PQ_slot0", FIELD_dsp340050b49a6c_fld3778PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3779PQ_slot0", FIELD_dsp340050b49a6c_fld3779PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3780PQ_slot0", FIELD_dsp340050b49a6c_fld3780PQ_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s12", FIELD_op0_s12, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2953ACC2_slot2", FIELD_dsp340050b49a6c_fld2953ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2954ACC2_slot2", FIELD_dsp340050b49a6c_fld2954ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2955ACC2_slot2", FIELD_dsp340050b49a6c_fld2955ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2956ACC2_slot2", FIELD_dsp340050b49a6c_fld2956ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2957ACC2_slot2", FIELD_dsp340050b49a6c_fld2957ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2958ACC2_slot2", FIELD_dsp340050b49a6c_fld2958ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2959ACC2_slot2", FIELD_dsp340050b49a6c_fld2959ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2960ACC2_slot2", FIELD_dsp340050b49a6c_fld2960ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2963ACC2_slot2", FIELD_dsp340050b49a6c_fld2963ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2964ACC2_slot2", FIELD_dsp340050b49a6c_fld2964ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2966ACC2_slot2", FIELD_dsp340050b49a6c_fld2966ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2967ACC2_slot2", FIELD_dsp340050b49a6c_fld2967ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3782ACC2_slot2", FIELD_dsp340050b49a6c_fld3782ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3783ACC2_slot2", FIELD_dsp340050b49a6c_fld3783ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3784ACC2_slot2", FIELD_dsp340050b49a6c_fld3784ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3785ACC2_slot2", FIELD_dsp340050b49a6c_fld3785ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3786ACC2_slot2", FIELD_dsp340050b49a6c_fld3786ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3788ACC2_slot2", FIELD_dsp340050b49a6c_fld3788ACC2_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s13", FIELD_op0_s13, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2028", FIELD_dsp340050b49a6c_fld2028, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2075", FIELD_dsp340050b49a6c_fld2075, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2968ACC2_slot1", FIELD_dsp340050b49a6c_fld2968ACC2_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2969ACC2_slot1", FIELD_dsp340050b49a6c_fld2969ACC2_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3790ACC2_slot1", FIELD_dsp340050b49a6c_fld3790ACC2_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3793ACC2_slot1", FIELD_dsp340050b49a6c_fld3793ACC2_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s14", FIELD_op0_s14, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2973ACC2_slot0", FIELD_dsp340050b49a6c_fld2973ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2974ACC2_slot0", FIELD_dsp340050b49a6c_fld2974ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2975ACC2_slot0", FIELD_dsp340050b49a6c_fld2975ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2976ACC2_slot0", FIELD_dsp340050b49a6c_fld2976ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2977ACC2_slot0", FIELD_dsp340050b49a6c_fld2977ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2980ACC2_slot0", FIELD_dsp340050b49a6c_fld2980ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2981ACC2_slot0", FIELD_dsp340050b49a6c_fld2981ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2982ACC2_slot0", FIELD_dsp340050b49a6c_fld2982ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2984ACC2_slot0", FIELD_dsp340050b49a6c_fld2984ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2985ACC2_slot0", FIELD_dsp340050b49a6c_fld2985ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2987ACC2_slot0", FIELD_dsp340050b49a6c_fld2987ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2989ACC2_slot0", FIELD_dsp340050b49a6c_fld2989ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2990ACC2_slot0", FIELD_dsp340050b49a6c_fld2990ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3795ACC2_slot0", FIELD_dsp340050b49a6c_fld3795ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3796ACC2_slot0", FIELD_dsp340050b49a6c_fld3796ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3797", FIELD_dsp340050b49a6c_fld3797, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3798ACC2_slot0", FIELD_dsp340050b49a6c_fld3798ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3799ACC2_slot0", FIELD_dsp340050b49a6c_fld3799ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3800ACC2_slot0", FIELD_dsp340050b49a6c_fld3800ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3801ACC2_slot0", FIELD_dsp340050b49a6c_fld3801ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3802ACC2_slot0", FIELD_dsp340050b49a6c_fld3802ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3803ACC2_slot0", FIELD_dsp340050b49a6c_fld3803ACC2_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s15", FIELD_op0_s15, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2991SMOD_slot2", FIELD_dsp340050b49a6c_fld2991SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2992SMOD_slot2", FIELD_dsp340050b49a6c_fld2992SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2993SMOD_slot2", FIELD_dsp340050b49a6c_fld2993SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2994SMOD_slot2", FIELD_dsp340050b49a6c_fld2994SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2995SMOD_slot2", FIELD_dsp340050b49a6c_fld2995SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2996SMOD_slot2", FIELD_dsp340050b49a6c_fld2996SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2997SMOD_slot2", FIELD_dsp340050b49a6c_fld2997SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2998SMOD_slot2", FIELD_dsp340050b49a6c_fld2998SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2999SMOD_slot2", FIELD_dsp340050b49a6c_fld2999SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3000SMOD_slot2", FIELD_dsp340050b49a6c_fld3000SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3001SMOD_slot2", FIELD_dsp340050b49a6c_fld3001SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3002SMOD_slot2", FIELD_dsp340050b49a6c_fld3002SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3003SMOD_slot2", FIELD_dsp340050b49a6c_fld3003SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3004SMOD_slot2", FIELD_dsp340050b49a6c_fld3004SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3005SMOD_slot2", FIELD_dsp340050b49a6c_fld3005SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3006SMOD_slot2", FIELD_dsp340050b49a6c_fld3006SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3007SMOD_slot2", FIELD_dsp340050b49a6c_fld3007SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3008SMOD_slot2", FIELD_dsp340050b49a6c_fld3008SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3009SMOD_slot2", FIELD_dsp340050b49a6c_fld3009SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3010SMOD_slot2", FIELD_dsp340050b49a6c_fld3010SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3011SMOD_slot2", FIELD_dsp340050b49a6c_fld3011SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3012SMOD_slot2", FIELD_dsp340050b49a6c_fld3012SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3013SMOD_slot2", FIELD_dsp340050b49a6c_fld3013SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3014SMOD_slot2", FIELD_dsp340050b49a6c_fld3014SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3015SMOD_slot2", FIELD_dsp340050b49a6c_fld3015SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3016SMOD_slot2", FIELD_dsp340050b49a6c_fld3016SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3017SMOD_slot2", FIELD_dsp340050b49a6c_fld3017SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3018SMOD_slot2", FIELD_dsp340050b49a6c_fld3018SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3019SMOD_slot2", FIELD_dsp340050b49a6c_fld3019SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3020SMOD_slot2", FIELD_dsp340050b49a6c_fld3020SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3021SMOD_slot2", FIELD_dsp340050b49a6c_fld3021SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3022SMOD_slot2", FIELD_dsp340050b49a6c_fld3022SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3023SMOD_slot2", FIELD_dsp340050b49a6c_fld3023SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3024SMOD_slot2", FIELD_dsp340050b49a6c_fld3024SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3025SMOD_slot2", FIELD_dsp340050b49a6c_fld3025SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3026SMOD_slot2", FIELD_dsp340050b49a6c_fld3026SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3027SMOD_slot2", FIELD_dsp340050b49a6c_fld3027SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3028SMOD_slot2", FIELD_dsp340050b49a6c_fld3028SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3030SMOD_slot2", FIELD_dsp340050b49a6c_fld3030SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3031SMOD_slot2", FIELD_dsp340050b49a6c_fld3031SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3032SMOD_slot2", FIELD_dsp340050b49a6c_fld3032SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3033SMOD_slot2", FIELD_dsp340050b49a6c_fld3033SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3034SMOD_slot2", FIELD_dsp340050b49a6c_fld3034SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3035SMOD_slot2", FIELD_dsp340050b49a6c_fld3035SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3036SMOD_slot2", FIELD_dsp340050b49a6c_fld3036SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3038SMOD_slot2", FIELD_dsp340050b49a6c_fld3038SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3039SMOD_slot2", FIELD_dsp340050b49a6c_fld3039SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3040SMOD_slot2", FIELD_dsp340050b49a6c_fld3040SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3043SMOD_slot2", FIELD_dsp340050b49a6c_fld3043SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3044SMOD_slot2", FIELD_dsp340050b49a6c_fld3044SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3046SMOD_slot2", FIELD_dsp340050b49a6c_fld3046SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3047SMOD_slot2", FIELD_dsp340050b49a6c_fld3047SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3048", FIELD_dsp340050b49a6c_fld3048, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3049SMOD_slot2", FIELD_dsp340050b49a6c_fld3049SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3050SMOD_slot2", FIELD_dsp340050b49a6c_fld3050SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3051SMOD_slot2", FIELD_dsp340050b49a6c_fld3051SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3052SMOD_slot2", FIELD_dsp340050b49a6c_fld3052SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3053SMOD_slot2", FIELD_dsp340050b49a6c_fld3053SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3054SMOD_slot2", FIELD_dsp340050b49a6c_fld3054SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3055SMOD_slot2", FIELD_dsp340050b49a6c_fld3055SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3056SMOD_slot2", FIELD_dsp340050b49a6c_fld3056SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3058SMOD_slot2", FIELD_dsp340050b49a6c_fld3058SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3059SMOD_slot2", FIELD_dsp340050b49a6c_fld3059SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3061SMOD_slot2", FIELD_dsp340050b49a6c_fld3061SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3062", FIELD_dsp340050b49a6c_fld3062, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3063SMOD_slot2", FIELD_dsp340050b49a6c_fld3063SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3065SMOD_slot2", FIELD_dsp340050b49a6c_fld3065SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3066SMOD_slot2", FIELD_dsp340050b49a6c_fld3066SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3067SMOD_slot2", FIELD_dsp340050b49a6c_fld3067SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3068SMOD_slot2", FIELD_dsp340050b49a6c_fld3068SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3069SMOD_slot2", FIELD_dsp340050b49a6c_fld3069SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3070", FIELD_dsp340050b49a6c_fld3070, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3071SMOD_slot2", FIELD_dsp340050b49a6c_fld3071SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3072SMOD_slot2", FIELD_dsp340050b49a6c_fld3072SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3073SMOD_slot2", FIELD_dsp340050b49a6c_fld3073SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3074SMOD_slot2", FIELD_dsp340050b49a6c_fld3074SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3075SMOD_slot2", FIELD_dsp340050b49a6c_fld3075SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3076SMOD_slot2", FIELD_dsp340050b49a6c_fld3076SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3077SMOD_slot2", FIELD_dsp340050b49a6c_fld3077SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3078SMOD_slot2", FIELD_dsp340050b49a6c_fld3078SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3079SMOD_slot2", FIELD_dsp340050b49a6c_fld3079SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3080SMOD_slot2", FIELD_dsp340050b49a6c_fld3080SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3081SMOD_slot2", FIELD_dsp340050b49a6c_fld3081SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3082SMOD_slot2", FIELD_dsp340050b49a6c_fld3082SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3084SMOD_slot2", FIELD_dsp340050b49a6c_fld3084SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3085SMOD_slot2", FIELD_dsp340050b49a6c_fld3085SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3087SMOD_slot2", FIELD_dsp340050b49a6c_fld3087SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3088SMOD_slot2", FIELD_dsp340050b49a6c_fld3088SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3090SMOD_slot2", FIELD_dsp340050b49a6c_fld3090SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3091SMOD_slot2", FIELD_dsp340050b49a6c_fld3091SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3092SMOD_slot2", FIELD_dsp340050b49a6c_fld3092SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3093SMOD_slot2", FIELD_dsp340050b49a6c_fld3093SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3096SMOD_slot2", FIELD_dsp340050b49a6c_fld3096SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3097SMOD_slot2", FIELD_dsp340050b49a6c_fld3097SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3098SMOD_slot2", FIELD_dsp340050b49a6c_fld3098SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3099SMOD_slot2", FIELD_dsp340050b49a6c_fld3099SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3100SMOD_slot2", FIELD_dsp340050b49a6c_fld3100SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3101SMOD_slot2", FIELD_dsp340050b49a6c_fld3101SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3102SMOD_slot2", FIELD_dsp340050b49a6c_fld3102SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3104SMOD_slot2", FIELD_dsp340050b49a6c_fld3104SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3105SMOD_slot2", FIELD_dsp340050b49a6c_fld3105SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3106SMOD_slot2", FIELD_dsp340050b49a6c_fld3106SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3107SMOD_slot2", FIELD_dsp340050b49a6c_fld3107SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3108SMOD_slot2", FIELD_dsp340050b49a6c_fld3108SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3109SMOD_slot2", FIELD_dsp340050b49a6c_fld3109SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3110SMOD_slot2", FIELD_dsp340050b49a6c_fld3110SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3111SMOD_slot2", FIELD_dsp340050b49a6c_fld3111SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3113SMOD_slot2", FIELD_dsp340050b49a6c_fld3113SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3114SMOD_slot2", FIELD_dsp340050b49a6c_fld3114SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3115SMOD_slot2", FIELD_dsp340050b49a6c_fld3115SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3116SMOD_slot2", FIELD_dsp340050b49a6c_fld3116SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3805SMOD_slot2", FIELD_dsp340050b49a6c_fld3805SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3806SMOD_slot2", FIELD_dsp340050b49a6c_fld3806SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3807SMOD_slot2", FIELD_dsp340050b49a6c_fld3807SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3808", FIELD_dsp340050b49a6c_fld3808, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3809SMOD_slot2", FIELD_dsp340050b49a6c_fld3809SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3810SMOD_slot2", FIELD_dsp340050b49a6c_fld3810SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3812SMOD_slot2", FIELD_dsp340050b49a6c_fld3812SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3813SMOD_slot2", FIELD_dsp340050b49a6c_fld3813SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3814SMOD_slot2", FIELD_dsp340050b49a6c_fld3814SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3816SMOD_slot2", FIELD_dsp340050b49a6c_fld3816SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3817", FIELD_dsp340050b49a6c_fld3817, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3818SMOD_slot2", FIELD_dsp340050b49a6c_fld3818SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3819SMOD_slot2", FIELD_dsp340050b49a6c_fld3819SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3821SMOD_slot2", FIELD_dsp340050b49a6c_fld3821SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3822SMOD_slot2", FIELD_dsp340050b49a6c_fld3822SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3823SMOD_slot2", FIELD_dsp340050b49a6c_fld3823SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3824SMOD_slot2", FIELD_dsp340050b49a6c_fld3824SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3825SMOD_slot2", FIELD_dsp340050b49a6c_fld3825SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3826SMOD_slot2", FIELD_dsp340050b49a6c_fld3826SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3827SMOD_slot2", FIELD_dsp340050b49a6c_fld3827SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3828SMOD_slot2", FIELD_dsp340050b49a6c_fld3828SMOD_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s16", FIELD_op0_s16, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2033", FIELD_dsp340050b49a6c_fld2033, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2080", FIELD_dsp340050b49a6c_fld2080, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3117SMOD_slot1", FIELD_dsp340050b49a6c_fld3117SMOD_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3118SMOD_slot1", FIELD_dsp340050b49a6c_fld3118SMOD_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3829SMOD_slot1", FIELD_dsp340050b49a6c_fld3829SMOD_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s17", FIELD_op0_s17, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3119SMOD_slot0", FIELD_dsp340050b49a6c_fld3119SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3120SMOD_slot0", FIELD_dsp340050b49a6c_fld3120SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3121SMOD_slot0", FIELD_dsp340050b49a6c_fld3121SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3122SMOD_slot0", FIELD_dsp340050b49a6c_fld3122SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3123SMOD_slot0", FIELD_dsp340050b49a6c_fld3123SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3125SMOD_slot0", FIELD_dsp340050b49a6c_fld3125SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3126SMOD_slot0", FIELD_dsp340050b49a6c_fld3126SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3127SMOD_slot0", FIELD_dsp340050b49a6c_fld3127SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3128SMOD_slot0", FIELD_dsp340050b49a6c_fld3128SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3129SMOD_slot0", FIELD_dsp340050b49a6c_fld3129SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3130SMOD_slot0", FIELD_dsp340050b49a6c_fld3130SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3131SMOD_slot0", FIELD_dsp340050b49a6c_fld3131SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3132SMOD_slot0", FIELD_dsp340050b49a6c_fld3132SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3133SMOD_slot0", FIELD_dsp340050b49a6c_fld3133SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3134SMOD_slot0", FIELD_dsp340050b49a6c_fld3134SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3135SMOD_slot0", FIELD_dsp340050b49a6c_fld3135SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3136SMOD_slot0", FIELD_dsp340050b49a6c_fld3136SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3137SMOD_slot0", FIELD_dsp340050b49a6c_fld3137SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3138SMOD_slot0", FIELD_dsp340050b49a6c_fld3138SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3139SMOD_slot0", FIELD_dsp340050b49a6c_fld3139SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3140SMOD_slot0", FIELD_dsp340050b49a6c_fld3140SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3141SMOD_slot0", FIELD_dsp340050b49a6c_fld3141SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3142SMOD_slot0", FIELD_dsp340050b49a6c_fld3142SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3143SMOD_slot0", FIELD_dsp340050b49a6c_fld3143SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3144SMOD_slot0", FIELD_dsp340050b49a6c_fld3144SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3145SMOD_slot0", FIELD_dsp340050b49a6c_fld3145SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3146SMOD_slot0", FIELD_dsp340050b49a6c_fld3146SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3148SMOD_slot0", FIELD_dsp340050b49a6c_fld3148SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3149SMOD_slot0", FIELD_dsp340050b49a6c_fld3149SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3150", FIELD_dsp340050b49a6c_fld3150, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3152SMOD_slot0", FIELD_dsp340050b49a6c_fld3152SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3153SMOD_slot0", FIELD_dsp340050b49a6c_fld3153SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3155SMOD_slot0", FIELD_dsp340050b49a6c_fld3155SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3156SMOD_slot0", FIELD_dsp340050b49a6c_fld3156SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3157SMOD_slot0", FIELD_dsp340050b49a6c_fld3157SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3158SMOD_slot0", FIELD_dsp340050b49a6c_fld3158SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3159SMOD_slot0", FIELD_dsp340050b49a6c_fld3159SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3160SMOD_slot0", FIELD_dsp340050b49a6c_fld3160SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3161SMOD_slot0", FIELD_dsp340050b49a6c_fld3161SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3164SMOD_slot0", FIELD_dsp340050b49a6c_fld3164SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3165SMOD_slot0", FIELD_dsp340050b49a6c_fld3165SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3166SMOD_slot0", FIELD_dsp340050b49a6c_fld3166SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3168SMOD_slot0", FIELD_dsp340050b49a6c_fld3168SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3170SMOD_slot0", FIELD_dsp340050b49a6c_fld3170SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3171SMOD_slot0", FIELD_dsp340050b49a6c_fld3171SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3172SMOD_slot0", FIELD_dsp340050b49a6c_fld3172SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3173SMOD_slot0", FIELD_dsp340050b49a6c_fld3173SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3174SMOD_slot0", FIELD_dsp340050b49a6c_fld3174SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3175SMOD_slot0", FIELD_dsp340050b49a6c_fld3175SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3176SMOD_slot0", FIELD_dsp340050b49a6c_fld3176SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3177SMOD_slot0", FIELD_dsp340050b49a6c_fld3177SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3178SMOD_slot0", FIELD_dsp340050b49a6c_fld3178SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3179SMOD_slot0", FIELD_dsp340050b49a6c_fld3179SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3180SMOD_slot0", FIELD_dsp340050b49a6c_fld3180SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3181SMOD_slot0", FIELD_dsp340050b49a6c_fld3181SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3182SMOD_slot0", FIELD_dsp340050b49a6c_fld3182SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3184SMOD_slot0", FIELD_dsp340050b49a6c_fld3184SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3186SMOD_slot0", FIELD_dsp340050b49a6c_fld3186SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3188SMOD_slot0", FIELD_dsp340050b49a6c_fld3188SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3832SMOD_slot0", FIELD_dsp340050b49a6c_fld3832SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3833SMOD_slot0", FIELD_dsp340050b49a6c_fld3833SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3834", FIELD_dsp340050b49a6c_fld3834, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3836SMOD_slot0", FIELD_dsp340050b49a6c_fld3836SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3837SMOD_slot0", FIELD_dsp340050b49a6c_fld3837SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3838SMOD_slot0", FIELD_dsp340050b49a6c_fld3838SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3841SMOD_slot0", FIELD_dsp340050b49a6c_fld3841SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3842SMOD_slot0", FIELD_dsp340050b49a6c_fld3842SMOD_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s18", FIELD_op0_s18, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2074", FIELD_dsp340050b49a6c_fld2074, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3191LLR_slot2", FIELD_dsp340050b49a6c_fld3191LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3192LLR_slot2", FIELD_dsp340050b49a6c_fld3192LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3193LLR_slot2", FIELD_dsp340050b49a6c_fld3193LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3194LLR_slot2", FIELD_dsp340050b49a6c_fld3194LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3195LLR_slot2", FIELD_dsp340050b49a6c_fld3195LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3196LLR_slot2", FIELD_dsp340050b49a6c_fld3196LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3197LLR_slot2", FIELD_dsp340050b49a6c_fld3197LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3198LLR_slot2", FIELD_dsp340050b49a6c_fld3198LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3199LLR_slot2", FIELD_dsp340050b49a6c_fld3199LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3200LLR_slot2", FIELD_dsp340050b49a6c_fld3200LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3201LLR_slot2", FIELD_dsp340050b49a6c_fld3201LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3202LLR_slot2", FIELD_dsp340050b49a6c_fld3202LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3203LLR_slot2", FIELD_dsp340050b49a6c_fld3203LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3204LLR_slot2", FIELD_dsp340050b49a6c_fld3204LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3205", FIELD_dsp340050b49a6c_fld3205, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3206", FIELD_dsp340050b49a6c_fld3206, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3207LLR_slot2", FIELD_dsp340050b49a6c_fld3207LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3208LLR_slot2", FIELD_dsp340050b49a6c_fld3208LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3210LLR_slot2", FIELD_dsp340050b49a6c_fld3210LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3212", FIELD_dsp340050b49a6c_fld3212, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3213LLR_slot2", FIELD_dsp340050b49a6c_fld3213LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3214", FIELD_dsp340050b49a6c_fld3214, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3215LLR_slot2", FIELD_dsp340050b49a6c_fld3215LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3216LLR_slot2", FIELD_dsp340050b49a6c_fld3216LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3217", FIELD_dsp340050b49a6c_fld3217, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3218LLR_slot2", FIELD_dsp340050b49a6c_fld3218LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3220LLR_slot2", FIELD_dsp340050b49a6c_fld3220LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3221LLR_slot2", FIELD_dsp340050b49a6c_fld3221LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3222LLR_slot2", FIELD_dsp340050b49a6c_fld3222LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3224LLR_slot2", FIELD_dsp340050b49a6c_fld3224LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3225", FIELD_dsp340050b49a6c_fld3225, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3226LLR_slot2", FIELD_dsp340050b49a6c_fld3226LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3228LLR_slot2", FIELD_dsp340050b49a6c_fld3228LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3230", FIELD_dsp340050b49a6c_fld3230, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3231LLR_slot2", FIELD_dsp340050b49a6c_fld3231LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3232LLR_slot2", FIELD_dsp340050b49a6c_fld3232LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3233", FIELD_dsp340050b49a6c_fld3233, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3234LLR_slot2", FIELD_dsp340050b49a6c_fld3234LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3235LLR_slot2", FIELD_dsp340050b49a6c_fld3235LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3236", FIELD_dsp340050b49a6c_fld3236, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3237LLR_slot2", FIELD_dsp340050b49a6c_fld3237LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3238LLR_slot2", FIELD_dsp340050b49a6c_fld3238LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3240LLR_slot2", FIELD_dsp340050b49a6c_fld3240LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3241LLR_slot2", FIELD_dsp340050b49a6c_fld3241LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3242LLR_slot2", FIELD_dsp340050b49a6c_fld3242LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3243LLR_slot2", FIELD_dsp340050b49a6c_fld3243LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3244LLR_slot2", FIELD_dsp340050b49a6c_fld3244LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3245LLR_slot2", FIELD_dsp340050b49a6c_fld3245LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3246", FIELD_dsp340050b49a6c_fld3246, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3247LLR_slot2", FIELD_dsp340050b49a6c_fld3247LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3843LLR_slot2", FIELD_dsp340050b49a6c_fld3843LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3844", FIELD_dsp340050b49a6c_fld3844, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3845LLR_slot2", FIELD_dsp340050b49a6c_fld3845LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3847LLR_slot2", FIELD_dsp340050b49a6c_fld3847LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3848LLR_slot2", FIELD_dsp340050b49a6c_fld3848LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3849LLR_slot2", FIELD_dsp340050b49a6c_fld3849LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3850LLR_slot2", FIELD_dsp340050b49a6c_fld3850LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3851LLR_slot2", FIELD_dsp340050b49a6c_fld3851LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3853LLR_slot2", FIELD_dsp340050b49a6c_fld3853LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3855LLR_slot2", FIELD_dsp340050b49a6c_fld3855LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3856LLR_slot2", FIELD_dsp340050b49a6c_fld3856LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3857LLR_slot2", FIELD_dsp340050b49a6c_fld3857LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3859LLR_slot2", FIELD_dsp340050b49a6c_fld3859LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3860LLR_slot2", FIELD_dsp340050b49a6c_fld3860LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3861LLR_slot2", FIELD_dsp340050b49a6c_fld3861LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3862", FIELD_dsp340050b49a6c_fld3862, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3863LLR_slot2", FIELD_dsp340050b49a6c_fld3863LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3864LLR_slot2", FIELD_dsp340050b49a6c_fld3864LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3865LLR_slot2", FIELD_dsp340050b49a6c_fld3865LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3866LLR_slot2", FIELD_dsp340050b49a6c_fld3866LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3867LLR_slot2", FIELD_dsp340050b49a6c_fld3867LLR_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3868", FIELD_dsp340050b49a6c_fld3868, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s19", FIELD_op0_s19, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2034", FIELD_dsp340050b49a6c_fld2034, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3248LLR_slot1", FIELD_dsp340050b49a6c_fld3248LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3250LLR_slot1", FIELD_dsp340050b49a6c_fld3250LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3251LLR_slot1", FIELD_dsp340050b49a6c_fld3251LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3252LLR_slot1", FIELD_dsp340050b49a6c_fld3252LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3253LLR_slot1", FIELD_dsp340050b49a6c_fld3253LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3254LLR_slot1", FIELD_dsp340050b49a6c_fld3254LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3869LLR_slot1", FIELD_dsp340050b49a6c_fld3869LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3870", FIELD_dsp340050b49a6c_fld3870, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3872LLR_slot1", FIELD_dsp340050b49a6c_fld3872LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3875LLR_slot1", FIELD_dsp340050b49a6c_fld3875LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3876LLR_slot1", FIELD_dsp340050b49a6c_fld3876LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3878LLR_slot1", FIELD_dsp340050b49a6c_fld3878LLR_slot1, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s20", FIELD_op0_s20, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2071", FIELD_dsp340050b49a6c_fld2071, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3258LLR_slot0", FIELD_dsp340050b49a6c_fld3258LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3259LLR_slot0", FIELD_dsp340050b49a6c_fld3259LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3260LLR_slot0", FIELD_dsp340050b49a6c_fld3260LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3261LLR_slot0", FIELD_dsp340050b49a6c_fld3261LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3263LLR_slot0", FIELD_dsp340050b49a6c_fld3263LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3264LLR_slot0", FIELD_dsp340050b49a6c_fld3264LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3265LLR_slot0", FIELD_dsp340050b49a6c_fld3265LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3266LLR_slot0", FIELD_dsp340050b49a6c_fld3266LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3267LLR_slot0", FIELD_dsp340050b49a6c_fld3267LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3268LLR_slot0", FIELD_dsp340050b49a6c_fld3268LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3269LLR_slot0", FIELD_dsp340050b49a6c_fld3269LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3270LLR_slot0", FIELD_dsp340050b49a6c_fld3270LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3272LLR_slot0", FIELD_dsp340050b49a6c_fld3272LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3274LLR_slot0", FIELD_dsp340050b49a6c_fld3274LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3275LLR_slot0", FIELD_dsp340050b49a6c_fld3275LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3276LLR_slot0", FIELD_dsp340050b49a6c_fld3276LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3277LLR_slot0", FIELD_dsp340050b49a6c_fld3277LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3278LLR_slot0", FIELD_dsp340050b49a6c_fld3278LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3279LLR_slot0", FIELD_dsp340050b49a6c_fld3279LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3280LLR_slot0", FIELD_dsp340050b49a6c_fld3280LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3281LLR_slot0", FIELD_dsp340050b49a6c_fld3281LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3282LLR_slot0", FIELD_dsp340050b49a6c_fld3282LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3283LLR_slot0", FIELD_dsp340050b49a6c_fld3283LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3284LLR_slot0", FIELD_dsp340050b49a6c_fld3284LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3286LLR_slot0", FIELD_dsp340050b49a6c_fld3286LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3288LLR_slot0", FIELD_dsp340050b49a6c_fld3288LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3289LLR_slot0", FIELD_dsp340050b49a6c_fld3289LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3291LLR_slot0", FIELD_dsp340050b49a6c_fld3291LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3292LLR_slot0", FIELD_dsp340050b49a6c_fld3292LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3293LLR_slot0", FIELD_dsp340050b49a6c_fld3293LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3294LLR_slot0", FIELD_dsp340050b49a6c_fld3294LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3295LLR_slot0", FIELD_dsp340050b49a6c_fld3295LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3296LLR_slot0", FIELD_dsp340050b49a6c_fld3296LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3297LLR_slot0", FIELD_dsp340050b49a6c_fld3297LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3298LLR_slot0", FIELD_dsp340050b49a6c_fld3298LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3299LLR_slot0", FIELD_dsp340050b49a6c_fld3299LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3300LLR_slot0", FIELD_dsp340050b49a6c_fld3300LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3302LLR_slot0", FIELD_dsp340050b49a6c_fld3302LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3303LLR_slot0", FIELD_dsp340050b49a6c_fld3303LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3304LLR_slot0", FIELD_dsp340050b49a6c_fld3304LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3305LLR_slot0", FIELD_dsp340050b49a6c_fld3305LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3306LLR_slot0", FIELD_dsp340050b49a6c_fld3306LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3308LLR_slot0", FIELD_dsp340050b49a6c_fld3308LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3310LLR_slot0", FIELD_dsp340050b49a6c_fld3310LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3311LLR_slot0", FIELD_dsp340050b49a6c_fld3311LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3312LLR_slot0", FIELD_dsp340050b49a6c_fld3312LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3879LLR_slot0", FIELD_dsp340050b49a6c_fld3879LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3881LLR_slot0", FIELD_dsp340050b49a6c_fld3881LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3883LLR_slot0", FIELD_dsp340050b49a6c_fld3883LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3885LLR_slot0", FIELD_dsp340050b49a6c_fld3885LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3887LLR_slot0", FIELD_dsp340050b49a6c_fld3887LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3888LLR_slot0", FIELD_dsp340050b49a6c_fld3888LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3890LLR_slot0", FIELD_dsp340050b49a6c_fld3890LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3892LLR_slot0", FIELD_dsp340050b49a6c_fld3892LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3893LLR_slot0", FIELD_dsp340050b49a6c_fld3893LLR_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s21", FIELD_op0_s21, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3313DUAL_slot2", FIELD_dsp340050b49a6c_fld3313DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3314", FIELD_dsp340050b49a6c_fld3314, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3315DUAL_slot2", FIELD_dsp340050b49a6c_fld3315DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3316DUAL_slot2", FIELD_dsp340050b49a6c_fld3316DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3317DUAL_slot2", FIELD_dsp340050b49a6c_fld3317DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3318", FIELD_dsp340050b49a6c_fld3318, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3319DUAL_slot2", FIELD_dsp340050b49a6c_fld3319DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3320DUAL_slot2", FIELD_dsp340050b49a6c_fld3320DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3321DUAL_slot2", FIELD_dsp340050b49a6c_fld3321DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3322DUAL_slot2", FIELD_dsp340050b49a6c_fld3322DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3323DUAL_slot2", FIELD_dsp340050b49a6c_fld3323DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3324DUAL_slot2", FIELD_dsp340050b49a6c_fld3324DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3325DUAL_slot2", FIELD_dsp340050b49a6c_fld3325DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3326DUAL_slot2", FIELD_dsp340050b49a6c_fld3326DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3327DUAL_slot2", FIELD_dsp340050b49a6c_fld3327DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3328DUAL_slot2", FIELD_dsp340050b49a6c_fld3328DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3329DUAL_slot2", FIELD_dsp340050b49a6c_fld3329DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3330DUAL_slot2", FIELD_dsp340050b49a6c_fld3330DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3331DUAL_slot2", FIELD_dsp340050b49a6c_fld3331DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3332DUAL_slot2", FIELD_dsp340050b49a6c_fld3332DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3333DUAL_slot2", FIELD_dsp340050b49a6c_fld3333DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3334DUAL_slot2", FIELD_dsp340050b49a6c_fld3334DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3335DUAL_slot2", FIELD_dsp340050b49a6c_fld3335DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3336DUAL_slot2", FIELD_dsp340050b49a6c_fld3336DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3337DUAL_slot2", FIELD_dsp340050b49a6c_fld3337DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3339DUAL_slot2", FIELD_dsp340050b49a6c_fld3339DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3340DUAL_slot2", FIELD_dsp340050b49a6c_fld3340DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3341DUAL_slot2", FIELD_dsp340050b49a6c_fld3341DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3342DUAL_slot2", FIELD_dsp340050b49a6c_fld3342DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3345DUAL_slot2", FIELD_dsp340050b49a6c_fld3345DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3347DUAL_slot2", FIELD_dsp340050b49a6c_fld3347DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3348DUAL_slot2", FIELD_dsp340050b49a6c_fld3348DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3349DUAL_slot2", FIELD_dsp340050b49a6c_fld3349DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3350DUAL_slot2", FIELD_dsp340050b49a6c_fld3350DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3353DUAL_slot2", FIELD_dsp340050b49a6c_fld3353DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3354DUAL_slot2", FIELD_dsp340050b49a6c_fld3354DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3356DUAL_slot2", FIELD_dsp340050b49a6c_fld3356DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3358DUAL_slot2", FIELD_dsp340050b49a6c_fld3358DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3360DUAL_slot2", FIELD_dsp340050b49a6c_fld3360DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3361DUAL_slot2", FIELD_dsp340050b49a6c_fld3361DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3362DUAL_slot2", FIELD_dsp340050b49a6c_fld3362DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3363DUAL_slot2", FIELD_dsp340050b49a6c_fld3363DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3364", FIELD_dsp340050b49a6c_fld3364, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3365DUAL_slot2", FIELD_dsp340050b49a6c_fld3365DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3366DUAL_slot2", FIELD_dsp340050b49a6c_fld3366DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3367DUAL_slot2", FIELD_dsp340050b49a6c_fld3367DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3368DUAL_slot2", FIELD_dsp340050b49a6c_fld3368DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3369DUAL_slot2", FIELD_dsp340050b49a6c_fld3369DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3370DUAL_slot2", FIELD_dsp340050b49a6c_fld3370DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3371DUAL_slot2", FIELD_dsp340050b49a6c_fld3371DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3372DUAL_slot2", FIELD_dsp340050b49a6c_fld3372DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3373DUAL_slot2", FIELD_dsp340050b49a6c_fld3373DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3374DUAL_slot2", FIELD_dsp340050b49a6c_fld3374DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3375DUAL_slot2", FIELD_dsp340050b49a6c_fld3375DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3376DUAL_slot2", FIELD_dsp340050b49a6c_fld3376DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3377DUAL_slot2", FIELD_dsp340050b49a6c_fld3377DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3378DUAL_slot2", FIELD_dsp340050b49a6c_fld3378DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3379DUAL_slot2", FIELD_dsp340050b49a6c_fld3379DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3380DUAL_slot2", FIELD_dsp340050b49a6c_fld3380DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3381DUAL_slot2", FIELD_dsp340050b49a6c_fld3381DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3382DUAL_slot2", FIELD_dsp340050b49a6c_fld3382DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3384DUAL_slot2", FIELD_dsp340050b49a6c_fld3384DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3385DUAL_slot2", FIELD_dsp340050b49a6c_fld3385DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3386DUAL_slot2", FIELD_dsp340050b49a6c_fld3386DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3387DUAL_slot2", FIELD_dsp340050b49a6c_fld3387DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3388DUAL_slot2", FIELD_dsp340050b49a6c_fld3388DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3390DUAL_slot2", FIELD_dsp340050b49a6c_fld3390DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3392DUAL_slot2", FIELD_dsp340050b49a6c_fld3392DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3394DUAL_slot2", FIELD_dsp340050b49a6c_fld3394DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3396DUAL_slot2", FIELD_dsp340050b49a6c_fld3396DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3397DUAL_slot2", FIELD_dsp340050b49a6c_fld3397DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3399DUAL_slot2", FIELD_dsp340050b49a6c_fld3399DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3401DUAL_slot2", FIELD_dsp340050b49a6c_fld3401DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3403DUAL_slot2", FIELD_dsp340050b49a6c_fld3403DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3404DUAL_slot2", FIELD_dsp340050b49a6c_fld3404DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3406DUAL_slot2", FIELD_dsp340050b49a6c_fld3406DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3407", FIELD_dsp340050b49a6c_fld3407, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3408DUAL_slot2", FIELD_dsp340050b49a6c_fld3408DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3410", FIELD_dsp340050b49a6c_fld3410, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3411DUAL_slot2", FIELD_dsp340050b49a6c_fld3411DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3412DUAL_slot2", FIELD_dsp340050b49a6c_fld3412DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3413DUAL_slot2", FIELD_dsp340050b49a6c_fld3413DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3414DUAL_slot2", FIELD_dsp340050b49a6c_fld3414DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3415DUAL_slot2", FIELD_dsp340050b49a6c_fld3415DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3416DUAL_slot2", FIELD_dsp340050b49a6c_fld3416DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3417DUAL_slot2", FIELD_dsp340050b49a6c_fld3417DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3418DUAL_slot2", FIELD_dsp340050b49a6c_fld3418DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3419DUAL_slot2", FIELD_dsp340050b49a6c_fld3419DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3420DUAL_slot2", FIELD_dsp340050b49a6c_fld3420DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3421DUAL_slot2", FIELD_dsp340050b49a6c_fld3421DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3422DUAL_slot2", FIELD_dsp340050b49a6c_fld3422DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3423DUAL_slot2", FIELD_dsp340050b49a6c_fld3423DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3424DUAL_slot2", FIELD_dsp340050b49a6c_fld3424DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3425DUAL_slot2", FIELD_dsp340050b49a6c_fld3425DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3426DUAL_slot2", FIELD_dsp340050b49a6c_fld3426DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3427DUAL_slot2", FIELD_dsp340050b49a6c_fld3427DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3428DUAL_slot2", FIELD_dsp340050b49a6c_fld3428DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3429DUAL_slot2", FIELD_dsp340050b49a6c_fld3429DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3430DUAL_slot2", FIELD_dsp340050b49a6c_fld3430DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3431DUAL_slot2", FIELD_dsp340050b49a6c_fld3431DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3432DUAL_slot2", FIELD_dsp340050b49a6c_fld3432DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3433DUAL_slot2", FIELD_dsp340050b49a6c_fld3433DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3434DUAL_slot2", FIELD_dsp340050b49a6c_fld3434DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3435DUAL_slot2", FIELD_dsp340050b49a6c_fld3435DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3436DUAL_slot2", FIELD_dsp340050b49a6c_fld3436DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3437DUAL_slot2", FIELD_dsp340050b49a6c_fld3437DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3438DUAL_slot2", FIELD_dsp340050b49a6c_fld3438DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3439DUAL_slot2", FIELD_dsp340050b49a6c_fld3439DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3440DUAL_slot2", FIELD_dsp340050b49a6c_fld3440DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3441DUAL_slot2", FIELD_dsp340050b49a6c_fld3441DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3442DUAL_slot2", FIELD_dsp340050b49a6c_fld3442DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3443DUAL_slot2", FIELD_dsp340050b49a6c_fld3443DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3444DUAL_slot2", FIELD_dsp340050b49a6c_fld3444DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3445DUAL_slot2", FIELD_dsp340050b49a6c_fld3445DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3446DUAL_slot2", FIELD_dsp340050b49a6c_fld3446DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3448DUAL_slot2", FIELD_dsp340050b49a6c_fld3448DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3450DUAL_slot2", FIELD_dsp340050b49a6c_fld3450DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3451DUAL_slot2", FIELD_dsp340050b49a6c_fld3451DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3453DUAL_slot2", FIELD_dsp340050b49a6c_fld3453DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3454DUAL_slot2", FIELD_dsp340050b49a6c_fld3454DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3456DUAL_slot2", FIELD_dsp340050b49a6c_fld3456DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3457DUAL_slot2", FIELD_dsp340050b49a6c_fld3457DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3458DUAL_slot2", FIELD_dsp340050b49a6c_fld3458DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3459DUAL_slot2", FIELD_dsp340050b49a6c_fld3459DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3460DUAL_slot2", FIELD_dsp340050b49a6c_fld3460DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3461DUAL_slot2", FIELD_dsp340050b49a6c_fld3461DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3462DUAL_slot2", FIELD_dsp340050b49a6c_fld3462DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3464DUAL_slot2", FIELD_dsp340050b49a6c_fld3464DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3465DUAL_slot2", FIELD_dsp340050b49a6c_fld3465DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3466", FIELD_dsp340050b49a6c_fld3466, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3467DUAL_slot2", FIELD_dsp340050b49a6c_fld3467DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3468DUAL_slot2", FIELD_dsp340050b49a6c_fld3468DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3469DUAL_slot2", FIELD_dsp340050b49a6c_fld3469DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3470DUAL_slot2", FIELD_dsp340050b49a6c_fld3470DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3471DUAL_slot2", FIELD_dsp340050b49a6c_fld3471DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3472DUAL_slot2", FIELD_dsp340050b49a6c_fld3472DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3473DUAL_slot2", FIELD_dsp340050b49a6c_fld3473DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3474DUAL_slot2", FIELD_dsp340050b49a6c_fld3474DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3475DUAL_slot2", FIELD_dsp340050b49a6c_fld3475DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3477DUAL_slot2", FIELD_dsp340050b49a6c_fld3477DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3478DUAL_slot2", FIELD_dsp340050b49a6c_fld3478DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3479DUAL_slot2", FIELD_dsp340050b49a6c_fld3479DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3480DUAL_slot2", FIELD_dsp340050b49a6c_fld3480DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3481DUAL_slot2", FIELD_dsp340050b49a6c_fld3481DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3482DUAL_slot2", FIELD_dsp340050b49a6c_fld3482DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3484DUAL_slot2", FIELD_dsp340050b49a6c_fld3484DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3894DUAL_slot2", FIELD_dsp340050b49a6c_fld3894DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3895DUAL_slot2", FIELD_dsp340050b49a6c_fld3895DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3896DUAL_slot2", FIELD_dsp340050b49a6c_fld3896DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3897DUAL_slot2", FIELD_dsp340050b49a6c_fld3897DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3898DUAL_slot2", FIELD_dsp340050b49a6c_fld3898DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3899DUAL_slot2", FIELD_dsp340050b49a6c_fld3899DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3900", FIELD_dsp340050b49a6c_fld3900, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3901DUAL_slot2", FIELD_dsp340050b49a6c_fld3901DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3903DUAL_slot2", FIELD_dsp340050b49a6c_fld3903DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3904DUAL_slot2", FIELD_dsp340050b49a6c_fld3904DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3905DUAL_slot2", FIELD_dsp340050b49a6c_fld3905DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3906DUAL_slot2", FIELD_dsp340050b49a6c_fld3906DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3907DUAL_slot2", FIELD_dsp340050b49a6c_fld3907DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3908DUAL_slot2", FIELD_dsp340050b49a6c_fld3908DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3909DUAL_slot2", FIELD_dsp340050b49a6c_fld3909DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3910DUAL_slot2", FIELD_dsp340050b49a6c_fld3910DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3913DUAL_slot2", FIELD_dsp340050b49a6c_fld3913DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3914DUAL_slot2", FIELD_dsp340050b49a6c_fld3914DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3916DUAL_slot2", FIELD_dsp340050b49a6c_fld3916DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3917DUAL_slot2", FIELD_dsp340050b49a6c_fld3917DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3918DUAL_slot2", FIELD_dsp340050b49a6c_fld3918DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3919DUAL_slot2", FIELD_dsp340050b49a6c_fld3919DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3920DUAL_slot2", FIELD_dsp340050b49a6c_fld3920DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3921DUAL_slot2", FIELD_dsp340050b49a6c_fld3921DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3922DUAL_slot2", FIELD_dsp340050b49a6c_fld3922DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3923DUAL_slot2", FIELD_dsp340050b49a6c_fld3923DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3924DUAL_slot2", FIELD_dsp340050b49a6c_fld3924DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3925DUAL_slot2", FIELD_dsp340050b49a6c_fld3925DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3927DUAL_slot2", FIELD_dsp340050b49a6c_fld3927DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3928DUAL_slot2", FIELD_dsp340050b49a6c_fld3928DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3929DUAL_slot2", FIELD_dsp340050b49a6c_fld3929DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3930DUAL_slot2", FIELD_dsp340050b49a6c_fld3930DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3931DUAL_slot2", FIELD_dsp340050b49a6c_fld3931DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3933DUAL_slot2", FIELD_dsp340050b49a6c_fld3933DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3934DUAL_slot2", FIELD_dsp340050b49a6c_fld3934DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3935DUAL_slot2", FIELD_dsp340050b49a6c_fld3935DUAL_slot2, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s22", FIELD_op0_s22, -1, 0, 0, 0, 0, 0, 0 },
  { "op0_s23", FIELD_op0_s23, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2057", FIELD_dsp340050b49a6c_fld2057, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2060", FIELD_dsp340050b49a6c_fld2060, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2066", FIELD_dsp340050b49a6c_fld2066, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2072", FIELD_dsp340050b49a6c_fld2072, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld2079", FIELD_dsp340050b49a6c_fld2079, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3487DUAL_slot0", FIELD_dsp340050b49a6c_fld3487DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3488DUAL_slot0", FIELD_dsp340050b49a6c_fld3488DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3489DUAL_slot0", FIELD_dsp340050b49a6c_fld3489DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3490DUAL_slot0", FIELD_dsp340050b49a6c_fld3490DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3491DUAL_slot0", FIELD_dsp340050b49a6c_fld3491DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3492DUAL_slot0", FIELD_dsp340050b49a6c_fld3492DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3493DUAL_slot0", FIELD_dsp340050b49a6c_fld3493DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3494DUAL_slot0", FIELD_dsp340050b49a6c_fld3494DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3496DUAL_slot0", FIELD_dsp340050b49a6c_fld3496DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3497DUAL_slot0", FIELD_dsp340050b49a6c_fld3497DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3498DUAL_slot0", FIELD_dsp340050b49a6c_fld3498DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3499DUAL_slot0", FIELD_dsp340050b49a6c_fld3499DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3500DUAL_slot0", FIELD_dsp340050b49a6c_fld3500DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3502DUAL_slot0", FIELD_dsp340050b49a6c_fld3502DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3504DUAL_slot0", FIELD_dsp340050b49a6c_fld3504DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3505DUAL_slot0", FIELD_dsp340050b49a6c_fld3505DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3506DUAL_slot0", FIELD_dsp340050b49a6c_fld3506DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3507DUAL_slot0", FIELD_dsp340050b49a6c_fld3507DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3508DUAL_slot0", FIELD_dsp340050b49a6c_fld3508DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3509DUAL_slot0", FIELD_dsp340050b49a6c_fld3509DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3510DUAL_slot0", FIELD_dsp340050b49a6c_fld3510DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3511DUAL_slot0", FIELD_dsp340050b49a6c_fld3511DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3512DUAL_slot0", FIELD_dsp340050b49a6c_fld3512DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3513DUAL_slot0", FIELD_dsp340050b49a6c_fld3513DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3514DUAL_slot0", FIELD_dsp340050b49a6c_fld3514DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3515DUAL_slot0", FIELD_dsp340050b49a6c_fld3515DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3516DUAL_slot0", FIELD_dsp340050b49a6c_fld3516DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3517DUAL_slot0", FIELD_dsp340050b49a6c_fld3517DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3518DUAL_slot0", FIELD_dsp340050b49a6c_fld3518DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3519DUAL_slot0", FIELD_dsp340050b49a6c_fld3519DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3520DUAL_slot0", FIELD_dsp340050b49a6c_fld3520DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3522DUAL_slot0", FIELD_dsp340050b49a6c_fld3522DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3523DUAL_slot0", FIELD_dsp340050b49a6c_fld3523DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3524DUAL_slot0", FIELD_dsp340050b49a6c_fld3524DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3527DUAL_slot0", FIELD_dsp340050b49a6c_fld3527DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3529DUAL_slot0", FIELD_dsp340050b49a6c_fld3529DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3530DUAL_slot0", FIELD_dsp340050b49a6c_fld3530DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3531", FIELD_dsp340050b49a6c_fld3531, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3532DUAL_slot0", FIELD_dsp340050b49a6c_fld3532DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3533DUAL_slot0", FIELD_dsp340050b49a6c_fld3533DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3535DUAL_slot0", FIELD_dsp340050b49a6c_fld3535DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3536DUAL_slot0", FIELD_dsp340050b49a6c_fld3536DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3537DUAL_slot0", FIELD_dsp340050b49a6c_fld3537DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3538DUAL_slot0", FIELD_dsp340050b49a6c_fld3538DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3539DUAL_slot0", FIELD_dsp340050b49a6c_fld3539DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3541DUAL_slot0", FIELD_dsp340050b49a6c_fld3541DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3542DUAL_slot0", FIELD_dsp340050b49a6c_fld3542DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3543DUAL_slot0", FIELD_dsp340050b49a6c_fld3543DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3544DUAL_slot0", FIELD_dsp340050b49a6c_fld3544DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3545DUAL_slot0", FIELD_dsp340050b49a6c_fld3545DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3546DUAL_slot0", FIELD_dsp340050b49a6c_fld3546DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3547DUAL_slot0", FIELD_dsp340050b49a6c_fld3547DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3548DUAL_slot0", FIELD_dsp340050b49a6c_fld3548DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3549DUAL_slot0", FIELD_dsp340050b49a6c_fld3549DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3550DUAL_slot0", FIELD_dsp340050b49a6c_fld3550DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3551DUAL_slot0", FIELD_dsp340050b49a6c_fld3551DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3552", FIELD_dsp340050b49a6c_fld3552, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3553DUAL_slot0", FIELD_dsp340050b49a6c_fld3553DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3554DUAL_slot0", FIELD_dsp340050b49a6c_fld3554DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3555DUAL_slot0", FIELD_dsp340050b49a6c_fld3555DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3556DUAL_slot0", FIELD_dsp340050b49a6c_fld3556DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3557DUAL_slot0", FIELD_dsp340050b49a6c_fld3557DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3558DUAL_slot0", FIELD_dsp340050b49a6c_fld3558DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3559DUAL_slot0", FIELD_dsp340050b49a6c_fld3559DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3560DUAL_slot0", FIELD_dsp340050b49a6c_fld3560DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3562DUAL_slot0", FIELD_dsp340050b49a6c_fld3562DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3563DUAL_slot0", FIELD_dsp340050b49a6c_fld3563DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3564DUAL_slot0", FIELD_dsp340050b49a6c_fld3564DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3565DUAL_slot0", FIELD_dsp340050b49a6c_fld3565DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3566DUAL_slot0", FIELD_dsp340050b49a6c_fld3566DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3567DUAL_slot0", FIELD_dsp340050b49a6c_fld3567DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3568DUAL_slot0", FIELD_dsp340050b49a6c_fld3568DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3569DUAL_slot0", FIELD_dsp340050b49a6c_fld3569DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3570DUAL_slot0", FIELD_dsp340050b49a6c_fld3570DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3571DUAL_slot0", FIELD_dsp340050b49a6c_fld3571DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3572DUAL_slot0", FIELD_dsp340050b49a6c_fld3572DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3573DUAL_slot0", FIELD_dsp340050b49a6c_fld3573DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3574DUAL_slot0", FIELD_dsp340050b49a6c_fld3574DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3575DUAL_slot0", FIELD_dsp340050b49a6c_fld3575DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3576DUAL_slot0", FIELD_dsp340050b49a6c_fld3576DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3577DUAL_slot0", FIELD_dsp340050b49a6c_fld3577DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3578DUAL_slot0", FIELD_dsp340050b49a6c_fld3578DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3579DUAL_slot0", FIELD_dsp340050b49a6c_fld3579DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3580DUAL_slot0", FIELD_dsp340050b49a6c_fld3580DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3581DUAL_slot0", FIELD_dsp340050b49a6c_fld3581DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3582DUAL_slot0", FIELD_dsp340050b49a6c_fld3582DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3583DUAL_slot0", FIELD_dsp340050b49a6c_fld3583DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3584", FIELD_dsp340050b49a6c_fld3584, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3585DUAL_slot0", FIELD_dsp340050b49a6c_fld3585DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3587DUAL_slot0", FIELD_dsp340050b49a6c_fld3587DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3588DUAL_slot0", FIELD_dsp340050b49a6c_fld3588DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3589DUAL_slot0", FIELD_dsp340050b49a6c_fld3589DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3590DUAL_slot0", FIELD_dsp340050b49a6c_fld3590DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3591DUAL_slot0", FIELD_dsp340050b49a6c_fld3591DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3592DUAL_slot0", FIELD_dsp340050b49a6c_fld3592DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3593DUAL_slot0", FIELD_dsp340050b49a6c_fld3593DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3594DUAL_slot0", FIELD_dsp340050b49a6c_fld3594DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3595DUAL_slot0", FIELD_dsp340050b49a6c_fld3595DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3596DUAL_slot0", FIELD_dsp340050b49a6c_fld3596DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3597DUAL_slot0", FIELD_dsp340050b49a6c_fld3597DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3598DUAL_slot0", FIELD_dsp340050b49a6c_fld3598DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3599DUAL_slot0", FIELD_dsp340050b49a6c_fld3599DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3600DUAL_slot0", FIELD_dsp340050b49a6c_fld3600DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3601DUAL_slot0", FIELD_dsp340050b49a6c_fld3601DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3602", FIELD_dsp340050b49a6c_fld3602, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3603DUAL_slot0", FIELD_dsp340050b49a6c_fld3603DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3604DUAL_slot0", FIELD_dsp340050b49a6c_fld3604DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3606DUAL_slot0", FIELD_dsp340050b49a6c_fld3606DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3607DUAL_slot0", FIELD_dsp340050b49a6c_fld3607DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3608DUAL_slot0", FIELD_dsp340050b49a6c_fld3608DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3609DUAL_slot0", FIELD_dsp340050b49a6c_fld3609DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3610", FIELD_dsp340050b49a6c_fld3610, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3611DUAL_slot0", FIELD_dsp340050b49a6c_fld3611DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3612DUAL_slot0", FIELD_dsp340050b49a6c_fld3612DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3613DUAL_slot0", FIELD_dsp340050b49a6c_fld3613DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3614DUAL_slot0", FIELD_dsp340050b49a6c_fld3614DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3615DUAL_slot0", FIELD_dsp340050b49a6c_fld3615DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3616DUAL_slot0", FIELD_dsp340050b49a6c_fld3616DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3618DUAL_slot0", FIELD_dsp340050b49a6c_fld3618DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3619", FIELD_dsp340050b49a6c_fld3619, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3620DUAL_slot0", FIELD_dsp340050b49a6c_fld3620DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3621DUAL_slot0", FIELD_dsp340050b49a6c_fld3621DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3622DUAL_slot0", FIELD_dsp340050b49a6c_fld3622DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3623DUAL_slot0", FIELD_dsp340050b49a6c_fld3623DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3624DUAL_slot0", FIELD_dsp340050b49a6c_fld3624DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3625DUAL_slot0", FIELD_dsp340050b49a6c_fld3625DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3626DUAL_slot0", FIELD_dsp340050b49a6c_fld3626DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3936DUAL_slot0", FIELD_dsp340050b49a6c_fld3936DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3937DUAL_slot0", FIELD_dsp340050b49a6c_fld3937DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3938DUAL_slot0", FIELD_dsp340050b49a6c_fld3938DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3939DUAL_slot0", FIELD_dsp340050b49a6c_fld3939DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3940DUAL_slot0", FIELD_dsp340050b49a6c_fld3940DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3941DUAL_slot0", FIELD_dsp340050b49a6c_fld3941DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3943DUAL_slot0", FIELD_dsp340050b49a6c_fld3943DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3945DUAL_slot0", FIELD_dsp340050b49a6c_fld3945DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3946DUAL_slot0", FIELD_dsp340050b49a6c_fld3946DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3947DUAL_slot0", FIELD_dsp340050b49a6c_fld3947DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3949DUAL_slot0", FIELD_dsp340050b49a6c_fld3949DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3950DUAL_slot0", FIELD_dsp340050b49a6c_fld3950DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3951DUAL_slot0", FIELD_dsp340050b49a6c_fld3951DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3952DUAL_slot0", FIELD_dsp340050b49a6c_fld3952DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3954DUAL_slot0", FIELD_dsp340050b49a6c_fld3954DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3957DUAL_slot0", FIELD_dsp340050b49a6c_fld3957DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3958DUAL_slot0", FIELD_dsp340050b49a6c_fld3958DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3959DUAL_slot0", FIELD_dsp340050b49a6c_fld3959DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3960DUAL_slot0", FIELD_dsp340050b49a6c_fld3960DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 },
  { "dsp340050b49a6c_fld3961DUAL_slot0", FIELD_dsp340050b49a6c_fld3961DUAL_slot0, -1, 0, 0, 0, 0, 0, 0 }
};

enum xtensa_operand_id {
  OPERAND_soffsetx4,
  OPERAND_uimm12x8,
  OPERAND_simm4,
  OPERAND_arr,
  OPERAND_ars,
  OPERAND__ars_invisible,
  OPERAND_art,
  OPERAND_ar0,
  OPERAND_ar4,
  OPERAND_ar8,
  OPERAND_ar12,
  OPERAND_ars_entry,
  OPERAND_immrx4,
  OPERAND_lsi4x4,
  OPERAND_simm7,
  OPERAND_uimm6,
  OPERAND_ai4const,
  OPERAND_b4const,
  OPERAND_b4constu,
  OPERAND_uimm8,
  OPERAND_uimm8x2,
  OPERAND_uimm8x4,
  OPERAND_uimm4x16,
  OPERAND_simm8,
  OPERAND_simm8x256,
  OPERAND_simm12b,
  OPERAND_msalp32,
  OPERAND_op2p1,
  OPERAND_label8,
  OPERAND_ulabel8,
  OPERAND_label12,
  OPERAND_soffset,
  OPERAND_uimm16x4,
  OPERAND_immt,
  OPERAND_imms,
  OPERAND_bt,
  OPERAND_bs,
  OPERAND_br,
  OPERAND_bt2,
  OPERAND_bs2,
  OPERAND_br2,
  OPERAND_bt4,
  OPERAND_bs4,
  OPERAND_br4,
  OPERAND_bt8,
  OPERAND_bs8,
  OPERAND_br8,
  OPERAND_bt16,
  OPERAND_bs16,
  OPERAND_br16,
  OPERAND_brall,
  OPERAND_tp7,
  OPERAND_xt_wbr15_label,
  OPERAND_xt_wbr18_label,
  OPERAND_cimm8x4,
  OPERAND_frr,
  OPERAND_frs,
  OPERAND_frt,
  OPERAND_dsp340050b49a6c_oper45_reg,
  OPERAND_dsp340050b49a6c_oper46_reg,
  OPERAND_dsp340050b49a6c_oper47_reg,
  OPERAND_dsp340050b49a6c_oper48_imm,
  OPERAND_dsp340050b49a6c_oper49_imm,
  OPERAND_dsp340050b49a6c_oper50_reg,
  OPERAND_dsp340050b49a6c_oper51_reg,
  OPERAND_dsp340050b49a6c_oper52_imm,
  OPERAND_dsp340050b49a6c_oper53_reg,
  OPERAND_dsp340050b49a6c_oper54_reg,
  OPERAND_dsp340050b49a6c_oper55_reg,
  OPERAND_dsp340050b49a6c_oper56_reg,
  OPERAND_dsp340050b49a6c_oper57_imm,
  OPERAND_dsp340050b49a6c_oper58_imm,
  OPERAND_dsp340050b49a6c_oper59_imm,
  OPERAND_dsp340050b49a6c_oper60_imm,
  OPERAND_dsp340050b49a6c_oper61_imm,
  OPERAND_dsp340050b49a6c_oper62_reg,
  OPERAND_dsp340050b49a6c_oper63_reg,
  OPERAND_dsp340050b49a6c_oper64_imm,
  OPERAND_dsp340050b49a6c_oper65_reg,
  OPERAND_dsp340050b49a6c_oper66_reg,
  OPERAND_dsp340050b49a6c_oper67_reg,
  OPERAND_dsp340050b49a6c_oper68_imm,
  OPERAND_dsp340050b49a6c_oper69_imm,
  OPERAND_dsp340050b49a6c_oper70_imm,
  OPERAND_dsp340050b49a6c_oper71_reg,
  OPERAND_dsp340050b49a6c_oper72_imm,
  OPERAND_dsp340050b49a6c_oper73_imm,
  OPERAND_dsp340050b49a6c_oper74_imm,
  OPERAND_dsp340050b49a6c_oper75_imm,
  OPERAND_dsp340050b49a6c_oper76_imm,
  OPERAND_dsp340050b49a6c_oper77_reg,
  OPERAND_dsp340050b49a6c_oper78_imm,
  OPERAND_dsp340050b49a6c_oper79_reg,
  OPERAND_dsp340050b49a6c_oper80_reg,
  OPERAND_dsp340050b49a6c_oper81_reg,
  OPERAND_dsp340050b49a6c_oper82_reg,
  OPERAND_dsp340050b49a6c_oper83_imm,
  OPERAND_dsp340050b49a6c_oper84_imm,
  OPERAND_dsp340050b49a6c_oper85_imm,
  OPERAND_dsp340050b49a6c_oper86_imm,
  OPERAND_dsp340050b49a6c_oper87_imm,
  OPERAND_dsp340050b49a6c_oper88_imm,
  OPERAND_dsp340050b49a6c_oper89_imm,
  OPERAND_t,
  OPERAND_bbi4,
  OPERAND_bbi,
  OPERAND_imm12,
  OPERAND_imm8,
  OPERAND_s,
  OPERAND_imm12b,
  OPERAND_imm16,
  OPERAND_m,
  OPERAND_n,
  OPERAND_offset,
  OPERAND_op0,
  OPERAND_op1,
  OPERAND_op2,
  OPERAND_r,
  OPERAND_sa4,
  OPERAND_sae4,
  OPERAND_sae,
  OPERAND_sal,
  OPERAND_sargt,
  OPERAND_sas4,
  OPERAND_sas,
  OPERAND_sr,
  OPERAND_st,
  OPERAND_thi3,
  OPERAND_imm4,
  OPERAND_mn,
  OPERAND_i,
  OPERAND_imm6lo,
  OPERAND_imm6hi,
  OPERAND_imm7lo,
  OPERAND_imm7hi,
  OPERAND_z,
  OPERAND_imm6,
  OPERAND_imm7,
  OPERAND_t2,
  OPERAND_s2,
  OPERAND_r2,
  OPERAND_t4,
  OPERAND_s4,
  OPERAND_r4,
  OPERAND_t8,
  OPERAND_s8,
  OPERAND_r8,
  OPERAND_xt_wbr15_imm,
  OPERAND_xt_wbr18_imm,
  OPERAND_fimm8,
  OPERAND_dsp340050b49a6c_fld2019,
  OPERAND_dsp340050b49a6c_fld2021,
  OPERAND_dsp340050b49a6c_fld2029,
  OPERAND_dsp340050b49a6c_fld2030,
  OPERAND_dsp340050b49a6c_fld2032,
  OPERAND_dsp340050b49a6c_fld2035,
  OPERAND_dsp340050b49a6c_fld2036,
  OPERAND_dsp340050b49a6c_fld2037,
  OPERAND_dsp340050b49a6c_fld2038,
  OPERAND_dsp340050b49a6c_fld2039,
  OPERAND_dsp340050b49a6c_fld2040,
  OPERAND_dsp340050b49a6c_fld2041,
  OPERAND_dsp340050b49a6c_fld2042,
  OPERAND_dsp340050b49a6c_fld2043,
  OPERAND_dsp340050b49a6c_fld2044,
  OPERAND_dsp340050b49a6c_fld2045,
  OPERAND_dsp340050b49a6c_fld2046,
  OPERAND_dsp340050b49a6c_fld2047,
  OPERAND_dsp340050b49a6c_fld2048,
  OPERAND_dsp340050b49a6c_fld2049,
  OPERAND_dsp340050b49a6c_fld2050,
  OPERAND_dsp340050b49a6c_fld2051,
  OPERAND_dsp340050b49a6c_fld2052,
  OPERAND_dsp340050b49a6c_fld2053,
  OPERAND_dsp340050b49a6c_fld2054,
  OPERAND_dsp340050b49a6c_fld2055,
  OPERAND_dsp340050b49a6c_fld2056,
  OPERAND_dsp340050b49a6c_fld2082Inst,
  OPERAND_dsp340050b49a6c_fld2083Inst,
  OPERAND_dsp340050b49a6c_fld2084Inst,
  OPERAND_dsp340050b49a6c_fld2085Inst,
  OPERAND_dsp340050b49a6c_fld2086Inst,
  OPERAND_dsp340050b49a6c_fld2088Inst,
  OPERAND_dsp340050b49a6c_fld2089Inst,
  OPERAND_dsp340050b49a6c_fld2090Inst,
  OPERAND_dsp340050b49a6c_fld2091Inst,
  OPERAND_dsp340050b49a6c_fld2092Inst,
  OPERAND_dsp340050b49a6c_fld2094Inst,
  OPERAND_dsp340050b49a6c_fld2095Inst,
  OPERAND_dsp340050b49a6c_fld2096Inst,
  OPERAND_dsp340050b49a6c_fld2098Inst,
  OPERAND_dsp340050b49a6c_fld2099Inst,
  OPERAND_dsp340050b49a6c_fld2100Inst,
  OPERAND_dsp340050b49a6c_fld2101Inst,
  OPERAND_dsp340050b49a6c_fld2102Inst,
  OPERAND_dsp340050b49a6c_fld2103Inst,
  OPERAND_dsp340050b49a6c_fld2104Inst,
  OPERAND_dsp340050b49a6c_fld2105Inst,
  OPERAND_dsp340050b49a6c_fld2106Inst,
  OPERAND_dsp340050b49a6c_fld2107Inst,
  OPERAND_dsp340050b49a6c_fld2108Inst,
  OPERAND_dsp340050b49a6c_fld2109Inst,
  OPERAND_dsp340050b49a6c_fld2110Inst,
  OPERAND_dsp340050b49a6c_fld2111Inst,
  OPERAND_dsp340050b49a6c_fld2112Inst,
  OPERAND_dsp340050b49a6c_fld2113Inst,
  OPERAND_dsp340050b49a6c_fld2114Inst,
  OPERAND_dsp340050b49a6c_fld2115Inst,
  OPERAND_dsp340050b49a6c_fld2116Inst,
  OPERAND_dsp340050b49a6c_fld2117Inst,
  OPERAND_dsp340050b49a6c_fld2118Inst,
  OPERAND_dsp340050b49a6c_fld2119Inst,
  OPERAND_dsp340050b49a6c_fld2120Inst,
  OPERAND_dsp340050b49a6c_fld2122Inst,
  OPERAND_dsp340050b49a6c_fld2123Inst,
  OPERAND_dsp340050b49a6c_fld2124Inst,
  OPERAND_dsp340050b49a6c_fld2125Inst,
  OPERAND_dsp340050b49a6c_fld2126Inst,
  OPERAND_dsp340050b49a6c_fld2127Inst,
  OPERAND_dsp340050b49a6c_fld2128Inst,
  OPERAND_dsp340050b49a6c_fld2129Inst,
  OPERAND_dsp340050b49a6c_fld2131Inst,
  OPERAND_dsp340050b49a6c_fld2132Inst,
  OPERAND_dsp340050b49a6c_fld2133Inst,
  OPERAND_dsp340050b49a6c_fld2134Inst,
  OPERAND_dsp340050b49a6c_fld2136Inst,
  OPERAND_dsp340050b49a6c_fld2137Inst,
  OPERAND_dsp340050b49a6c_fld2138Inst,
  OPERAND_dsp340050b49a6c_fld2139Inst,
  OPERAND_dsp340050b49a6c_fld2140Inst,
  OPERAND_dsp340050b49a6c_fld2141Inst,
  OPERAND_dsp340050b49a6c_fld2142Inst,
  OPERAND_dsp340050b49a6c_fld2143Inst,
  OPERAND_dsp340050b49a6c_fld2144Inst,
  OPERAND_dsp340050b49a6c_fld2145Inst,
  OPERAND_dsp340050b49a6c_fld2146Inst,
  OPERAND_dsp340050b49a6c_fld2147Inst,
  OPERAND_dsp340050b49a6c_fld2149Inst,
  OPERAND_dsp340050b49a6c_fld2151Inst,
  OPERAND_dsp340050b49a6c_fld2153Inst,
  OPERAND_dsp340050b49a6c_fld2154Inst,
  OPERAND_dsp340050b49a6c_fld2155Inst,
  OPERAND_dsp340050b49a6c_fld2156Inst,
  OPERAND_dsp340050b49a6c_fld2157Inst,
  OPERAND_dsp340050b49a6c_fld2158Inst,
  OPERAND_dsp340050b49a6c_fld2159Inst,
  OPERAND_dsp340050b49a6c_fld2160Inst,
  OPERAND_dsp340050b49a6c_fld2161Inst,
  OPERAND_dsp340050b49a6c_fld2162Inst,
  OPERAND_dsp340050b49a6c_fld2163Inst,
  OPERAND_dsp340050b49a6c_fld2164Inst,
  OPERAND_dsp340050b49a6c_fld2165Inst,
  OPERAND_dsp340050b49a6c_fld2166Inst,
  OPERAND_dsp340050b49a6c_fld2167Inst,
  OPERAND_dsp340050b49a6c_fld2168Inst,
  OPERAND_dsp340050b49a6c_fld2169Inst,
  OPERAND_dsp340050b49a6c_fld2171Inst,
  OPERAND_dsp340050b49a6c_fld2172Inst,
  OPERAND_dsp340050b49a6c_fld2173Inst,
  OPERAND_dsp340050b49a6c_fld2174Inst,
  OPERAND_dsp340050b49a6c_fld2175Inst,
  OPERAND_dsp340050b49a6c_fld2177Inst,
  OPERAND_dsp340050b49a6c_fld2178Inst,
  OPERAND_dsp340050b49a6c_fld2179Inst,
  OPERAND_dsp340050b49a6c_fld2180Inst,
  OPERAND_dsp340050b49a6c_fld2181Inst,
  OPERAND_dsp340050b49a6c_fld2182Inst,
  OPERAND_dsp340050b49a6c_fld2183Inst,
  OPERAND_dsp340050b49a6c_fld2184Inst,
  OPERAND_dsp340050b49a6c_fld2185Inst,
  OPERAND_dsp340050b49a6c_fld2186Inst,
  OPERAND_dsp340050b49a6c_fld2187Inst,
  OPERAND_dsp340050b49a6c_fld2188Inst,
  OPERAND_dsp340050b49a6c_fld2189Inst,
  OPERAND_dsp340050b49a6c_fld2190Inst,
  OPERAND_dsp340050b49a6c_fld2191Inst,
  OPERAND_dsp340050b49a6c_fld2192Inst,
  OPERAND_dsp340050b49a6c_fld2193Inst,
  OPERAND_dsp340050b49a6c_fld2194Inst,
  OPERAND_dsp340050b49a6c_fld2195Inst,
  OPERAND_dsp340050b49a6c_fld2196Inst,
  OPERAND_dsp340050b49a6c_fld2197Inst,
  OPERAND_dsp340050b49a6c_fld2198Inst,
  OPERAND_dsp340050b49a6c_fld2199Inst,
  OPERAND_dsp340050b49a6c_fld2200Inst,
  OPERAND_dsp340050b49a6c_fld2201Inst,
  OPERAND_dsp340050b49a6c_fld2202Inst,
  OPERAND_dsp340050b49a6c_fld2203Inst,
  OPERAND_dsp340050b49a6c_fld2204Inst,
  OPERAND_dsp340050b49a6c_fld2205Inst,
  OPERAND_dsp340050b49a6c_fld2206Inst,
  OPERAND_dsp340050b49a6c_fld2207Inst,
  OPERAND_dsp340050b49a6c_fld2208Inst,
  OPERAND_dsp340050b49a6c_fld2209Inst,
  OPERAND_dsp340050b49a6c_fld2210Inst,
  OPERAND_dsp340050b49a6c_fld2211Inst,
  OPERAND_dsp340050b49a6c_fld2212Inst,
  OPERAND_dsp340050b49a6c_fld2213Inst,
  OPERAND_dsp340050b49a6c_fld2214Inst,
  OPERAND_dsp340050b49a6c_fld2215Inst,
  OPERAND_dsp340050b49a6c_fld2216Inst,
  OPERAND_dsp340050b49a6c_fld2217Inst,
  OPERAND_dsp340050b49a6c_fld2218Inst,
  OPERAND_dsp340050b49a6c_fld2219Inst,
  OPERAND_dsp340050b49a6c_fld2220Inst,
  OPERAND_dsp340050b49a6c_fld2221Inst,
  OPERAND_dsp340050b49a6c_fld2222Inst,
  OPERAND_dsp340050b49a6c_fld2223Inst,
  OPERAND_dsp340050b49a6c_fld2224Inst,
  OPERAND_dsp340050b49a6c_fld2225Inst,
  OPERAND_dsp340050b49a6c_fld2226Inst,
  OPERAND_dsp340050b49a6c_fld2227Inst,
  OPERAND_dsp340050b49a6c_fld2228Inst,
  OPERAND_dsp340050b49a6c_fld2229Inst,
  OPERAND_dsp340050b49a6c_fld2230Inst,
  OPERAND_dsp340050b49a6c_fld2231Inst,
  OPERAND_dsp340050b49a6c_fld2232Inst,
  OPERAND_dsp340050b49a6c_fld2234Inst,
  OPERAND_dsp340050b49a6c_fld2235Inst,
  OPERAND_dsp340050b49a6c_fld2236Inst,
  OPERAND_dsp340050b49a6c_fld2237Inst,
  OPERAND_dsp340050b49a6c_fld2238Inst,
  OPERAND_dsp340050b49a6c_fld2239Inst,
  OPERAND_dsp340050b49a6c_fld2240Inst,
  OPERAND_dsp340050b49a6c_fld2241Inst,
  OPERAND_dsp340050b49a6c_fld2242Inst,
  OPERAND_dsp340050b49a6c_fld2243Inst,
  OPERAND_dsp340050b49a6c_fld2244Inst,
  OPERAND_dsp340050b49a6c_fld2245Inst,
  OPERAND_dsp340050b49a6c_fld2246Inst,
  OPERAND_dsp340050b49a6c_fld2247Inst,
  OPERAND_dsp340050b49a6c_fld2248Inst,
  OPERAND_dsp340050b49a6c_fld2249Inst,
  OPERAND_dsp340050b49a6c_fld2250Inst,
  OPERAND_dsp340050b49a6c_fld2251Inst,
  OPERAND_dsp340050b49a6c_fld2252Inst,
  OPERAND_dsp340050b49a6c_fld2253Inst,
  OPERAND_dsp340050b49a6c_fld2254,
  OPERAND_dsp340050b49a6c_fld2255Inst,
  OPERAND_dsp340050b49a6c_fld2257Inst,
  OPERAND_dsp340050b49a6c_fld3627Inst,
  OPERAND_dsp340050b49a6c_fld3630Inst,
  OPERAND_dsp340050b49a6c_fld3631Inst,
  OPERAND_dsp340050b49a6c_fld3633Inst,
  OPERAND_dsp340050b49a6c_fld3634,
  OPERAND_dsp340050b49a6c_fld3635Inst,
  OPERAND_dsp340050b49a6c_fld3636Inst,
  OPERAND_dsp340050b49a6c_fld3637Inst,
  OPERAND_dsp340050b49a6c_fld3638Inst,
  OPERAND_dsp340050b49a6c_fld3639Inst,
  OPERAND_dsp340050b49a6c_fld3640Inst,
  OPERAND_dsp340050b49a6c_fld3642Inst,
  OPERAND_dsp340050b49a6c_fld3643Inst,
  OPERAND_dsp340050b49a6c_fld3644Inst,
  OPERAND_dsp340050b49a6c_fld3645Inst,
  OPERAND_dsp340050b49a6c_fld3647Inst,
  OPERAND_dsp340050b49a6c_fld3648Inst,
  OPERAND_dsp340050b49a6c_fld3649Inst,
  OPERAND_dsp340050b49a6c_fld3650Inst,
  OPERAND_dsp340050b49a6c_fld3651Inst,
  OPERAND_dsp340050b49a6c_fld3653Inst,
  OPERAND_dsp340050b49a6c_fld3654Inst,
  OPERAND_dsp340050b49a6c_fld3655Inst,
  OPERAND_dsp340050b49a6c_fld3656Inst,
  OPERAND_dsp340050b49a6c_fld3657Inst,
  OPERAND_dsp340050b49a6c_fld3658Inst,
  OPERAND_dsp340050b49a6c_fld3659Inst,
  OPERAND_dsp340050b49a6c_fld3660Inst,
  OPERAND_dsp340050b49a6c_fld3661Inst,
  OPERAND_dsp340050b49a6c_fld3662Inst,
  OPERAND_op0_s3,
  OPERAND_dsp340050b49a6c_fld2025,
  OPERAND_dsp340050b49a6c_fld2027,
  OPERAND_dsp340050b49a6c_fld2258GP_slot2,
  OPERAND_dsp340050b49a6c_fld2259GP_slot2,
  OPERAND_dsp340050b49a6c_fld2260GP_slot2,
  OPERAND_dsp340050b49a6c_fld2261GP_slot2,
  OPERAND_dsp340050b49a6c_fld2262GP_slot2,
  OPERAND_dsp340050b49a6c_fld2263GP_slot2,
  OPERAND_dsp340050b49a6c_fld2264GP_slot2,
  OPERAND_dsp340050b49a6c_fld2266GP_slot2,
  OPERAND_dsp340050b49a6c_fld2267GP_slot2,
  OPERAND_dsp340050b49a6c_fld2268GP_slot2,
  OPERAND_dsp340050b49a6c_fld2269GP_slot2,
  OPERAND_dsp340050b49a6c_fld2270GP_slot2,
  OPERAND_dsp340050b49a6c_fld2271GP_slot2,
  OPERAND_dsp340050b49a6c_fld2272,
  OPERAND_dsp340050b49a6c_fld2273GP_slot2,
  OPERAND_dsp340050b49a6c_fld2274GP_slot2,
  OPERAND_dsp340050b49a6c_fld2275GP_slot2,
  OPERAND_dsp340050b49a6c_fld2277GP_slot2,
  OPERAND_dsp340050b49a6c_fld2278GP_slot2,
  OPERAND_dsp340050b49a6c_fld2279GP_slot2,
  OPERAND_dsp340050b49a6c_fld2280GP_slot2,
  OPERAND_dsp340050b49a6c_fld2281GP_slot2,
  OPERAND_dsp340050b49a6c_fld2282GP_slot2,
  OPERAND_dsp340050b49a6c_fld2283GP_slot2,
  OPERAND_dsp340050b49a6c_fld2284GP_slot2,
  OPERAND_dsp340050b49a6c_fld2286GP_slot2,
  OPERAND_dsp340050b49a6c_fld2287GP_slot2,
  OPERAND_dsp340050b49a6c_fld2288GP_slot2,
  OPERAND_dsp340050b49a6c_fld2289GP_slot2,
  OPERAND_dsp340050b49a6c_fld2290GP_slot2,
  OPERAND_dsp340050b49a6c_fld2291GP_slot2,
  OPERAND_dsp340050b49a6c_fld2292GP_slot2,
  OPERAND_dsp340050b49a6c_fld2293GP_slot2,
  OPERAND_dsp340050b49a6c_fld2294GP_slot2,
  OPERAND_dsp340050b49a6c_fld2295GP_slot2,
  OPERAND_dsp340050b49a6c_fld2296GP_slot2,
  OPERAND_dsp340050b49a6c_fld2297GP_slot2,
  OPERAND_dsp340050b49a6c_fld2298GP_slot2,
  OPERAND_dsp340050b49a6c_fld2299GP_slot2,
  OPERAND_dsp340050b49a6c_fld2300GP_slot2,
  OPERAND_dsp340050b49a6c_fld2301GP_slot2,
  OPERAND_dsp340050b49a6c_fld2302,
  OPERAND_dsp340050b49a6c_fld2303GP_slot2,
  OPERAND_dsp340050b49a6c_fld2304GP_slot2,
  OPERAND_dsp340050b49a6c_fld2305,
  OPERAND_dsp340050b49a6c_fld2306GP_slot2,
  OPERAND_dsp340050b49a6c_fld2308GP_slot2,
  OPERAND_dsp340050b49a6c_fld2309GP_slot2,
  OPERAND_dsp340050b49a6c_fld2310GP_slot2,
  OPERAND_dsp340050b49a6c_fld2312GP_slot2,
  OPERAND_dsp340050b49a6c_fld2313,
  OPERAND_dsp340050b49a6c_fld2314GP_slot2,
  OPERAND_dsp340050b49a6c_fld2316GP_slot2,
  OPERAND_dsp340050b49a6c_fld2317GP_slot2,
  OPERAND_dsp340050b49a6c_fld2318GP_slot2,
  OPERAND_dsp340050b49a6c_fld2319GP_slot2,
  OPERAND_dsp340050b49a6c_fld2320GP_slot2,
  OPERAND_dsp340050b49a6c_fld2321GP_slot2,
  OPERAND_dsp340050b49a6c_fld2322GP_slot2,
  OPERAND_dsp340050b49a6c_fld2323GP_slot2,
  OPERAND_dsp340050b49a6c_fld2324GP_slot2,
  OPERAND_dsp340050b49a6c_fld2325GP_slot2,
  OPERAND_dsp340050b49a6c_fld2326GP_slot2,
  OPERAND_dsp340050b49a6c_fld2327GP_slot2,
  OPERAND_dsp340050b49a6c_fld2328GP_slot2,
  OPERAND_dsp340050b49a6c_fld2329GP_slot2,
  OPERAND_dsp340050b49a6c_fld2330GP_slot2,
  OPERAND_dsp340050b49a6c_fld2331GP_slot2,
  OPERAND_dsp340050b49a6c_fld2332GP_slot2,
  OPERAND_dsp340050b49a6c_fld2333GP_slot2,
  OPERAND_dsp340050b49a6c_fld2334GP_slot2,
  OPERAND_dsp340050b49a6c_fld2335GP_slot2,
  OPERAND_dsp340050b49a6c_fld2336GP_slot2,
  OPERAND_dsp340050b49a6c_fld2337GP_slot2,
  OPERAND_dsp340050b49a6c_fld2338GP_slot2,
  OPERAND_dsp340050b49a6c_fld2339GP_slot2,
  OPERAND_dsp340050b49a6c_fld2340GP_slot2,
  OPERAND_dsp340050b49a6c_fld2341GP_slot2,
  OPERAND_dsp340050b49a6c_fld2342GP_slot2,
  OPERAND_dsp340050b49a6c_fld2343GP_slot2,
  OPERAND_dsp340050b49a6c_fld2344GP_slot2,
  OPERAND_dsp340050b49a6c_fld2345GP_slot2,
  OPERAND_dsp340050b49a6c_fld2346GP_slot2,
  OPERAND_dsp340050b49a6c_fld2347GP_slot2,
  OPERAND_dsp340050b49a6c_fld2348GP_slot2,
  OPERAND_dsp340050b49a6c_fld2349GP_slot2,
  OPERAND_dsp340050b49a6c_fld2350GP_slot2,
  OPERAND_dsp340050b49a6c_fld2351GP_slot2,
  OPERAND_dsp340050b49a6c_fld2352GP_slot2,
  OPERAND_dsp340050b49a6c_fld2353GP_slot2,
  OPERAND_dsp340050b49a6c_fld2354GP_slot2,
  OPERAND_dsp340050b49a6c_fld2355GP_slot2,
  OPERAND_dsp340050b49a6c_fld2356GP_slot2,
  OPERAND_dsp340050b49a6c_fld2357GP_slot2,
  OPERAND_dsp340050b49a6c_fld2358GP_slot2,
  OPERAND_dsp340050b49a6c_fld2359GP_slot2,
  OPERAND_dsp340050b49a6c_fld2361GP_slot2,
  OPERAND_dsp340050b49a6c_fld2362GP_slot2,
  OPERAND_dsp340050b49a6c_fld2364GP_slot2,
  OPERAND_dsp340050b49a6c_fld2366GP_slot2,
  OPERAND_dsp340050b49a6c_fld2368GP_slot2,
  OPERAND_dsp340050b49a6c_fld2369GP_slot2,
  OPERAND_dsp340050b49a6c_fld2370GP_slot2,
  OPERAND_dsp340050b49a6c_fld2371GP_slot2,
  OPERAND_dsp340050b49a6c_fld2372GP_slot2,
  OPERAND_dsp340050b49a6c_fld2373GP_slot2,
  OPERAND_dsp340050b49a6c_fld2374GP_slot2,
  OPERAND_dsp340050b49a6c_fld2375GP_slot2,
  OPERAND_dsp340050b49a6c_fld2376GP_slot2,
  OPERAND_dsp340050b49a6c_fld2378GP_slot2,
  OPERAND_dsp340050b49a6c_fld2379GP_slot2,
  OPERAND_dsp340050b49a6c_fld2381GP_slot2,
  OPERAND_dsp340050b49a6c_fld2383GP_slot2,
  OPERAND_dsp340050b49a6c_fld2384,
  OPERAND_dsp340050b49a6c_fld2385GP_slot2,
  OPERAND_dsp340050b49a6c_fld2386,
  OPERAND_dsp340050b49a6c_fld2387GP_slot2,
  OPERAND_dsp340050b49a6c_fld2388GP_slot2,
  OPERAND_dsp340050b49a6c_fld2389GP_slot2,
  OPERAND_dsp340050b49a6c_fld3663GP_slot2,
  OPERAND_dsp340050b49a6c_fld3664GP_slot2,
  OPERAND_dsp340050b49a6c_fld3665GP_slot2,
  OPERAND_dsp340050b49a6c_fld3666,
  OPERAND_dsp340050b49a6c_fld3667GP_slot2,
  OPERAND_dsp340050b49a6c_fld3668GP_slot2,
  OPERAND_dsp340050b49a6c_fld3669GP_slot2,
  OPERAND_dsp340050b49a6c_fld3670GP_slot2,
  OPERAND_dsp340050b49a6c_fld3671GP_slot2,
  OPERAND_dsp340050b49a6c_fld3673GP_slot2,
  OPERAND_dsp340050b49a6c_fld3674GP_slot2,
  OPERAND_dsp340050b49a6c_fld3675GP_slot2,
  OPERAND_dsp340050b49a6c_fld3676GP_slot2,
  OPERAND_dsp340050b49a6c_fld3678GP_slot2,
  OPERAND_dsp340050b49a6c_fld3679GP_slot2,
  OPERAND_dsp340050b49a6c_fld3680GP_slot2,
  OPERAND_op0_s4,
  OPERAND_dsp340050b49a6c_fld2026,
  OPERAND_dsp340050b49a6c_fld2031,
  OPERAND_dsp340050b49a6c_fld2394GP_slot1,
  OPERAND_dsp340050b49a6c_fld2395GP_slot1,
  OPERAND_dsp340050b49a6c_fld2397GP_slot1,
  OPERAND_dsp340050b49a6c_fld2398GP_slot1,
  OPERAND_dsp340050b49a6c_fld2399GP_slot1,
  OPERAND_dsp340050b49a6c_fld2400GP_slot1,
  OPERAND_dsp340050b49a6c_fld2402GP_slot1,
  OPERAND_dsp340050b49a6c_fld2403GP_slot1,
  OPERAND_dsp340050b49a6c_fld2405GP_slot1,
  OPERAND_dsp340050b49a6c_fld3681GP_slot1,
  OPERAND_dsp340050b49a6c_fld3683GP_slot1,
  OPERAND_dsp340050b49a6c_fld3684GP_slot1,
  OPERAND_dsp340050b49a6c_fld3686GP_slot1,
  OPERAND_op0_s5,
  OPERAND_dsp340050b49a6c_fld2058,
  OPERAND_dsp340050b49a6c_fld2067,
  OPERAND_dsp340050b49a6c_fld2407GP_slot0,
  OPERAND_dsp340050b49a6c_fld2409GP_slot0,
  OPERAND_dsp340050b49a6c_fld2410GP_slot0,
  OPERAND_dsp340050b49a6c_fld2411GP_slot0,
  OPERAND_dsp340050b49a6c_fld2412GP_slot0,
  OPERAND_dsp340050b49a6c_fld2413GP_slot0,
  OPERAND_dsp340050b49a6c_fld2415GP_slot0,
  OPERAND_dsp340050b49a6c_fld2416GP_slot0,
  OPERAND_dsp340050b49a6c_fld2417GP_slot0,
  OPERAND_dsp340050b49a6c_fld2418GP_slot0,
  OPERAND_dsp340050b49a6c_fld2419GP_slot0,
  OPERAND_dsp340050b49a6c_fld2420GP_slot0,
  OPERAND_dsp340050b49a6c_fld2422GP_slot0,
  OPERAND_dsp340050b49a6c_fld2423GP_slot0,
  OPERAND_dsp340050b49a6c_fld2424GP_slot0,
  OPERAND_dsp340050b49a6c_fld2425GP_slot0,
  OPERAND_dsp340050b49a6c_fld2426GP_slot0,
  OPERAND_dsp340050b49a6c_fld2427GP_slot0,
  OPERAND_dsp340050b49a6c_fld2429GP_slot0,
  OPERAND_dsp340050b49a6c_fld2430GP_slot0,
  OPERAND_dsp340050b49a6c_fld2431GP_slot0,
  OPERAND_dsp340050b49a6c_fld2432GP_slot0,
  OPERAND_dsp340050b49a6c_fld2433GP_slot0,
  OPERAND_dsp340050b49a6c_fld2434GP_slot0,
  OPERAND_dsp340050b49a6c_fld2435GP_slot0,
  OPERAND_dsp340050b49a6c_fld2436GP_slot0,
  OPERAND_dsp340050b49a6c_fld2437GP_slot0,
  OPERAND_dsp340050b49a6c_fld2438GP_slot0,
  OPERAND_dsp340050b49a6c_fld2439GP_slot0,
  OPERAND_dsp340050b49a6c_fld2440GP_slot0,
  OPERAND_dsp340050b49a6c_fld2441GP_slot0,
  OPERAND_dsp340050b49a6c_fld2443GP_slot0,
  OPERAND_dsp340050b49a6c_fld2444GP_slot0,
  OPERAND_dsp340050b49a6c_fld2445,
  OPERAND_dsp340050b49a6c_fld2447GP_slot0,
  OPERAND_dsp340050b49a6c_fld2448,
  OPERAND_dsp340050b49a6c_fld2449GP_slot0,
  OPERAND_dsp340050b49a6c_fld2451GP_slot0,
  OPERAND_dsp340050b49a6c_fld2452GP_slot0,
  OPERAND_dsp340050b49a6c_fld2453GP_slot0,
  OPERAND_dsp340050b49a6c_fld2454GP_slot0,
  OPERAND_dsp340050b49a6c_fld2455GP_slot0,
  OPERAND_dsp340050b49a6c_fld2456GP_slot0,
  OPERAND_dsp340050b49a6c_fld2457GP_slot0,
  OPERAND_dsp340050b49a6c_fld2458GP_slot0,
  OPERAND_dsp340050b49a6c_fld2459GP_slot0,
  OPERAND_dsp340050b49a6c_fld2460GP_slot0,
  OPERAND_dsp340050b49a6c_fld2461GP_slot0,
  OPERAND_dsp340050b49a6c_fld2462,
  OPERAND_dsp340050b49a6c_fld2463GP_slot0,
  OPERAND_dsp340050b49a6c_fld2464GP_slot0,
  OPERAND_dsp340050b49a6c_fld2465GP_slot0,
  OPERAND_dsp340050b49a6c_fld2466GP_slot0,
  OPERAND_dsp340050b49a6c_fld2467GP_slot0,
  OPERAND_dsp340050b49a6c_fld2468GP_slot0,
  OPERAND_dsp340050b49a6c_fld2470GP_slot0,
  OPERAND_dsp340050b49a6c_fld2471GP_slot0,
  OPERAND_dsp340050b49a6c_fld2472GP_slot0,
  OPERAND_dsp340050b49a6c_fld2473GP_slot0,
  OPERAND_dsp340050b49a6c_fld2474GP_slot0,
  OPERAND_dsp340050b49a6c_fld2475GP_slot0,
  OPERAND_dsp340050b49a6c_fld2477GP_slot0,
  OPERAND_dsp340050b49a6c_fld2479GP_slot0,
  OPERAND_dsp340050b49a6c_fld2480GP_slot0,
  OPERAND_dsp340050b49a6c_fld2481GP_slot0,
  OPERAND_dsp340050b49a6c_fld2482GP_slot0,
  OPERAND_dsp340050b49a6c_fld2483GP_slot0,
  OPERAND_dsp340050b49a6c_fld2484GP_slot0,
  OPERAND_dsp340050b49a6c_fld2485GP_slot0,
  OPERAND_dsp340050b49a6c_fld2486GP_slot0,
  OPERAND_dsp340050b49a6c_fld2487GP_slot0,
  OPERAND_dsp340050b49a6c_fld2488GP_slot0,
  OPERAND_dsp340050b49a6c_fld2489GP_slot0,
  OPERAND_dsp340050b49a6c_fld2490GP_slot0,
  OPERAND_dsp340050b49a6c_fld2491GP_slot0,
  OPERAND_dsp340050b49a6c_fld2492GP_slot0,
  OPERAND_dsp340050b49a6c_fld2493GP_slot0,
  OPERAND_dsp340050b49a6c_fld2494GP_slot0,
  OPERAND_dsp340050b49a6c_fld2495GP_slot0,
  OPERAND_dsp340050b49a6c_fld2496GP_slot0,
  OPERAND_dsp340050b49a6c_fld2497GP_slot0,
  OPERAND_dsp340050b49a6c_fld2498GP_slot0,
  OPERAND_dsp340050b49a6c_fld2499GP_slot0,
  OPERAND_dsp340050b49a6c_fld2500GP_slot0,
  OPERAND_dsp340050b49a6c_fld2501GP_slot0,
  OPERAND_dsp340050b49a6c_fld2502GP_slot0,
  OPERAND_dsp340050b49a6c_fld2503GP_slot0,
  OPERAND_dsp340050b49a6c_fld2504GP_slot0,
  OPERAND_dsp340050b49a6c_fld2505GP_slot0,
  OPERAND_dsp340050b49a6c_fld2506GP_slot0,
  OPERAND_dsp340050b49a6c_fld2507GP_slot0,
  OPERAND_dsp340050b49a6c_fld2508GP_slot0,
  OPERAND_dsp340050b49a6c_fld2509GP_slot0,
  OPERAND_dsp340050b49a6c_fld2510GP_slot0,
  OPERAND_dsp340050b49a6c_fld2512GP_slot0,
  OPERAND_dsp340050b49a6c_fld2514GP_slot0,
  OPERAND_dsp340050b49a6c_fld2515GP_slot0,
  OPERAND_dsp340050b49a6c_fld2516GP_slot0,
  OPERAND_dsp340050b49a6c_fld2517GP_slot0,
  OPERAND_dsp340050b49a6c_fld2518GP_slot0,
  OPERAND_dsp340050b49a6c_fld2519GP_slot0,
  OPERAND_dsp340050b49a6c_fld2520GP_slot0,
  OPERAND_dsp340050b49a6c_fld2521GP_slot0,
  OPERAND_dsp340050b49a6c_fld2523GP_slot0,
  OPERAND_dsp340050b49a6c_fld2524GP_slot0,
  OPERAND_dsp340050b49a6c_fld2526GP_slot0,
  OPERAND_dsp340050b49a6c_fld2527GP_slot0,
  OPERAND_dsp340050b49a6c_fld2528GP_slot0,
  OPERAND_dsp340050b49a6c_fld2529GP_slot0,
  OPERAND_dsp340050b49a6c_fld2530,
  OPERAND_dsp340050b49a6c_fld2531GP_slot0,
  OPERAND_dsp340050b49a6c_fld3688GP_slot0,
  OPERAND_dsp340050b49a6c_fld3689GP_slot0,
  OPERAND_dsp340050b49a6c_fld3690GP_slot0,
  OPERAND_dsp340050b49a6c_fld3691GP_slot0,
  OPERAND_dsp340050b49a6c_fld3692GP_slot0,
  OPERAND_dsp340050b49a6c_fld3693GP_slot0,
  OPERAND_dsp340050b49a6c_fld3695GP_slot0,
  OPERAND_dsp340050b49a6c_fld3696GP_slot0,
  OPERAND_dsp340050b49a6c_fld3697GP_slot0,
  OPERAND_dsp340050b49a6c_fld3698GP_slot0,
  OPERAND_dsp340050b49a6c_fld3699GP_slot0,
  OPERAND_dsp340050b49a6c_fld3700GP_slot0,
  OPERAND_dsp340050b49a6c_fld3702GP_slot0,
  OPERAND_dsp340050b49a6c_fld3703GP_slot0,
  OPERAND_dsp340050b49a6c_fld3705GP_slot0,
  OPERAND_dsp340050b49a6c_fld3706GP_slot0,
  OPERAND_op0_s6,
  OPERAND_dsp340050b49a6c_fld2532DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2533DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2534DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2535DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2536DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2537DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2538DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2539DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2540DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2541DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2542DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2543DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2544DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2545DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2546DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2547DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2548DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2549DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2550DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2551DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2552DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2553DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2554DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2555DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2556DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2557DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2558DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2559DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2560DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2561DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2562DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2563DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2564DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2565DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2566DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2567DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2568DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2569DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2571DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2572DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2573DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2574DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2575DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2576DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2577DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2578,
  OPERAND_dsp340050b49a6c_fld2579DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2580DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2581DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2582DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2583DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2584DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2585DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2586DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2587DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2588DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2589DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2590DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2591DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2592DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2595DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2596DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2598DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2599DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2601DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2602DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2604DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2605,
  OPERAND_dsp340050b49a6c_fld2606DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2608DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2609DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2610DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2611DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2612,
  OPERAND_dsp340050b49a6c_fld2613DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2614,
  OPERAND_dsp340050b49a6c_fld2615DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2616DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2617DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2618DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2619DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2620DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2621DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2622DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2623DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2624DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2625,
  OPERAND_dsp340050b49a6c_fld2626DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2628DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2630DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2632DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2633DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2635DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2636DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2637DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2640DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2641DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2642DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2643DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2644DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2645DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2646DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2647DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2648DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2649DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2650DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2651DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2652DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2654DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2655DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2656DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2657DOT_slot2,
  OPERAND_dsp340050b49a6c_fld2658DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3708DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3709DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3710DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3711DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3712,
  OPERAND_dsp340050b49a6c_fld3713DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3714DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3715,
  OPERAND_dsp340050b49a6c_fld3716DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3717DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3718,
  OPERAND_dsp340050b49a6c_fld3719DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3721DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3722,
  OPERAND_dsp340050b49a6c_fld3723DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3724DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3725DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3726DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3727DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3728DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3729DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3731DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3732DOT_slot2,
  OPERAND_dsp340050b49a6c_fld3733DOT_slot2,
  OPERAND_op0_s7,
  OPERAND_dsp340050b49a6c_fld3734DOT_slot1,
  OPERAND_op0_s8,
  OPERAND_dsp340050b49a6c_fld2068,
  OPERAND_dsp340050b49a6c_fld2666DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2667DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2668DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2669DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2671DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2672DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2673DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2674DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2675DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2676DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2677DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2678DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2679DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2680DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2681DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2682DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2683DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2684DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2685DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2686DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2688DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2689DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2690DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2692DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2693DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2695DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2697DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2699DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2700DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2701DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2702DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2703DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2704DOT_slot0,
  OPERAND_dsp340050b49a6c_fld2705DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3735DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3736,
  OPERAND_dsp340050b49a6c_fld3737DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3738DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3739DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3740DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3741DOT_slot0,
  OPERAND_dsp340050b49a6c_fld3742DOT_slot0,
  OPERAND_op0_s9,
  OPERAND_dsp340050b49a6c_fld2706PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2707PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2708PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2709PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2710PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2711PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2713PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2714PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2715PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2717PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2718PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2719PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2721PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2722PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2723PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2724PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2725PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2726PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2727PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2728PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2729PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2730PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2731PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2732PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2733PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2734PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2735PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2736PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2737,
  OPERAND_dsp340050b49a6c_fld2738PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2739PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2741PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2742PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2743PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2746PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2747PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2748PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2750PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2751PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2752PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2753PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2754PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2755PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2756PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2757PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2758PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2759PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2760PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2761PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2762PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2763PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2764PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2765PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2766PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2767PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2768PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2769PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2770PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2771PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2772PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2773PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2774PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2775PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2776PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2777PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2778PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2779PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2780PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2781PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2782PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2783PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2784PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2785PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2786PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2787PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2788PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2789PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2790PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2791PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2792PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2793PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2795PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2796PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2798PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2801PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2803PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2805PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2806PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2807PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2808PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2809PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2810PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2811PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2812PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2814PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2816PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2817,
  OPERAND_dsp340050b49a6c_fld2818PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2819,
  OPERAND_dsp340050b49a6c_fld2820PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2821PQ_slot2,
  OPERAND_dsp340050b49a6c_fld2823PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3744PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3745PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3746PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3747PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3748,
  OPERAND_dsp340050b49a6c_fld3749PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3750PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3751PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3752PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3753PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3754PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3756PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3757PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3758PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3759PQ_slot2,
  OPERAND_dsp340050b49a6c_fld3760PQ_slot2,
  OPERAND_op0_s10,
  OPERAND_dsp340050b49a6c_fld2825PQ_slot1,
  OPERAND_dsp340050b49a6c_fld2826PQ_slot1,
  OPERAND_dsp340050b49a6c_fld3761PQ_slot1,
  OPERAND_op0_s11,
  OPERAND_dsp340050b49a6c_fld2059,
  OPERAND_dsp340050b49a6c_fld2069,
  OPERAND_dsp340050b49a6c_fld2827PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2829PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2830PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2831PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2832PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2833PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2835PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2836PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2837PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2838PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2839PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2840PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2842PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2843PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2844PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2845PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2846PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2847PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2849PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2850PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2851PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2852PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2853PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2854PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2855PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2856PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2857PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2858PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2859PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2860PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2861PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2863PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2864PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2865,
  OPERAND_dsp340050b49a6c_fld2867PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2869PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2871PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2872PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2873PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2874PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2875PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2876PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2877PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2878PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2879PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2880PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2881PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2882,
  OPERAND_dsp340050b49a6c_fld2883PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2884PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2885PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2886PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2887PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2888PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2890PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2891PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2892PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2893PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2894PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2895PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2897PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2899PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2900PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2901PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2902PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2903PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2904PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2905PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2906PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2907PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2908PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2909PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2910PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2911PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2912PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2913PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2914PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2915PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2916PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2917PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2918PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2919PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2920PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2921PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2922PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2923PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2924PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2925PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2926PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2927PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2928PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2929PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2930PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2932PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2934PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2935PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2936PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2937PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2939PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2940,
  OPERAND_dsp340050b49a6c_fld2941PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2942PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2943PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2945PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2946PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2947PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2948PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2949PQ_slot0,
  OPERAND_dsp340050b49a6c_fld2950PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3763PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3764PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3765PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3766PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3767PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3768PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3769PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3770PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3771PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3772,
  OPERAND_dsp340050b49a6c_fld3773PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3775PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3776PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3777PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3778PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3779PQ_slot0,
  OPERAND_dsp340050b49a6c_fld3780PQ_slot0,
  OPERAND_op0_s12,
  OPERAND_dsp340050b49a6c_fld2953ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2954ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2955ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2956ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2957ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2958ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2959ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2960ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2963ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2964ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2966ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld2967ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld3782ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld3783ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld3784ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld3785ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld3786ACC2_slot2,
  OPERAND_dsp340050b49a6c_fld3788ACC2_slot2,
  OPERAND_op0_s13,
  OPERAND_dsp340050b49a6c_fld2028,
  OPERAND_dsp340050b49a6c_fld2075,
  OPERAND_dsp340050b49a6c_fld2968ACC2_slot1,
  OPERAND_dsp340050b49a6c_fld2969ACC2_slot1,
  OPERAND_dsp340050b49a6c_fld3790ACC2_slot1,
  OPERAND_dsp340050b49a6c_fld3793ACC2_slot1,
  OPERAND_op0_s14,
  OPERAND_dsp340050b49a6c_fld2973ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2974ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2975ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2976ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2977ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2980ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2981ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2982ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2984ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2985ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2987ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2989ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld2990ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3795ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3796ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3797,
  OPERAND_dsp340050b49a6c_fld3798ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3799ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3800ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3801ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3802ACC2_slot0,
  OPERAND_dsp340050b49a6c_fld3803ACC2_slot0,
  OPERAND_op0_s15,
  OPERAND_dsp340050b49a6c_fld2991SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2992SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2993SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2994SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2995SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2996SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2997SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2998SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld2999SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3000SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3001SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3002SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3003SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3004SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3005SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3006SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3007SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3008SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3009SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3010SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3011SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3012SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3013SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3014SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3015SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3016SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3017SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3018SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3019SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3020SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3021SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3022SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3023SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3024SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3025SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3026SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3027SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3028SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3030SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3031SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3032SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3033SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3034SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3035SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3036SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3038SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3039SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3040SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3043SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3044SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3046SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3047SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3048,
  OPERAND_dsp340050b49a6c_fld3049SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3050SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3051SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3052SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3053SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3054SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3055SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3056SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3058SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3059SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3061SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3062,
  OPERAND_dsp340050b49a6c_fld3063SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3065SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3066SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3067SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3068SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3069SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3070,
  OPERAND_dsp340050b49a6c_fld3071SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3072SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3073SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3074SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3075SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3076SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3077SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3078SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3079SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3080SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3081SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3082SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3084SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3085SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3087SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3088SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3090SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3091SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3092SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3093SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3096SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3097SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3098SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3099SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3100SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3101SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3102SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3104SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3105SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3106SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3107SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3108SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3109SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3110SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3111SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3113SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3114SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3115SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3116SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3805SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3806SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3807SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3808,
  OPERAND_dsp340050b49a6c_fld3809SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3810SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3812SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3813SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3814SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3816SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3817,
  OPERAND_dsp340050b49a6c_fld3818SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3819SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3821SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3822SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3823SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3824SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3825SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3826SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3827SMOD_slot2,
  OPERAND_dsp340050b49a6c_fld3828SMOD_slot2,
  OPERAND_op0_s16,
  OPERAND_dsp340050b49a6c_fld2033,
  OPERAND_dsp340050b49a6c_fld2080,
  OPERAND_dsp340050b49a6c_fld3117SMOD_slot1,
  OPERAND_dsp340050b49a6c_fld3118SMOD_slot1,
  OPERAND_dsp340050b49a6c_fld3829SMOD_slot1,
  OPERAND_op0_s17,
  OPERAND_dsp340050b49a6c_fld3119SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3120SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3121SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3122SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3123SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3125SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3126SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3127SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3128SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3129SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3130SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3131SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3132SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3133SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3134SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3135SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3136SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3137SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3138SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3139SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3140SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3141SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3142SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3143SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3144SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3145SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3146SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3148SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3149SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3150,
  OPERAND_dsp340050b49a6c_fld3152SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3153SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3155SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3156SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3157SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3158SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3159SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3160SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3161SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3164SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3165SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3166SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3168SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3170SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3171SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3172SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3173SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3174SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3175SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3176SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3177SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3178SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3179SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3180SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3181SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3182SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3184SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3186SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3188SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3832SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3833SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3834,
  OPERAND_dsp340050b49a6c_fld3836SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3837SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3838SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3841SMOD_slot0,
  OPERAND_dsp340050b49a6c_fld3842SMOD_slot0,
  OPERAND_op0_s18,
  OPERAND_dsp340050b49a6c_fld2074,
  OPERAND_dsp340050b49a6c_fld3191LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3192LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3193LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3194LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3195LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3196LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3197LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3198LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3199LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3200LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3201LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3202LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3203LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3204LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3205,
  OPERAND_dsp340050b49a6c_fld3206,
  OPERAND_dsp340050b49a6c_fld3207LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3208LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3210LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3212,
  OPERAND_dsp340050b49a6c_fld3213LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3214,
  OPERAND_dsp340050b49a6c_fld3215LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3216LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3217,
  OPERAND_dsp340050b49a6c_fld3218LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3220LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3221LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3222LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3224LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3225,
  OPERAND_dsp340050b49a6c_fld3226LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3228LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3230,
  OPERAND_dsp340050b49a6c_fld3231LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3232LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3233,
  OPERAND_dsp340050b49a6c_fld3234LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3235LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3236,
  OPERAND_dsp340050b49a6c_fld3237LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3238LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3240LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3241LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3242LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3243LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3244LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3245LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3246,
  OPERAND_dsp340050b49a6c_fld3247LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3843LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3844,
  OPERAND_dsp340050b49a6c_fld3845LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3847LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3848LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3849LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3850LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3851LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3853LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3855LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3856LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3857LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3859LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3860LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3861LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3862,
  OPERAND_dsp340050b49a6c_fld3863LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3864LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3865LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3866LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3867LLR_slot2,
  OPERAND_dsp340050b49a6c_fld3868,
  OPERAND_op0_s19,
  OPERAND_dsp340050b49a6c_fld2034,
  OPERAND_dsp340050b49a6c_fld3248LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3250LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3251LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3252LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3253LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3254LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3869LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3870,
  OPERAND_dsp340050b49a6c_fld3872LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3875LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3876LLR_slot1,
  OPERAND_dsp340050b49a6c_fld3878LLR_slot1,
  OPERAND_op0_s20,
  OPERAND_dsp340050b49a6c_fld2071,
  OPERAND_dsp340050b49a6c_fld3258LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3259LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3260LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3261LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3263LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3264LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3265LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3266LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3267LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3268LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3269LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3270LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3272LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3274LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3275LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3276LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3277LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3278LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3279LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3280LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3281LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3282LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3283LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3284LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3286LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3288LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3289LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3291LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3292LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3293LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3294LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3295LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3296LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3297LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3298LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3299LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3300LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3302LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3303LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3304LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3305LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3306LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3308LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3310LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3311LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3312LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3879LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3881LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3883LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3885LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3887LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3888LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3890LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3892LLR_slot0,
  OPERAND_dsp340050b49a6c_fld3893LLR_slot0,
  OPERAND_op0_s21,
  OPERAND_dsp340050b49a6c_fld3313DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3314,
  OPERAND_dsp340050b49a6c_fld3315DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3316DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3317DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3318,
  OPERAND_dsp340050b49a6c_fld3319DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3320DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3321DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3322DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3323DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3324DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3325DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3326DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3327DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3328DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3329DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3330DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3331DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3332DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3333DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3334DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3335DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3336DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3337DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3339DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3340DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3341DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3342DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3345DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3347DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3348DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3349DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3350DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3353DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3354DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3356DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3358DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3360DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3361DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3362DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3363DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3364,
  OPERAND_dsp340050b49a6c_fld3365DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3366DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3367DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3368DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3369DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3370DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3371DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3372DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3373DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3374DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3375DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3376DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3377DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3378DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3379DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3380DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3381DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3382DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3384DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3385DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3386DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3387DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3388DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3390DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3392DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3394DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3396DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3397DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3399DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3401DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3403DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3404DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3406DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3407,
  OPERAND_dsp340050b49a6c_fld3408DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3410,
  OPERAND_dsp340050b49a6c_fld3411DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3412DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3413DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3414DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3415DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3416DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3417DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3418DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3419DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3420DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3421DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3422DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3423DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3424DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3425DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3426DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3427DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3428DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3429DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3430DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3431DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3432DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3433DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3434DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3435DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3436DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3437DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3438DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3439DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3440DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3441DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3442DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3443DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3444DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3445DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3446DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3448DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3450DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3451DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3453DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3454DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3456DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3457DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3458DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3459DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3460DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3461DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3462DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3464DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3465DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3466,
  OPERAND_dsp340050b49a6c_fld3467DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3468DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3469DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3470DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3471DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3472DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3473DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3474DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3475DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3477DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3478DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3479DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3480DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3481DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3482DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3484DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3894DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3895DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3896DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3897DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3898DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3899DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3900,
  OPERAND_dsp340050b49a6c_fld3901DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3903DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3904DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3905DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3906DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3907DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3908DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3909DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3910DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3913DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3914DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3916DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3917DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3918DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3919DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3920DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3921DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3922DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3923DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3924DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3925DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3927DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3928DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3929DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3930DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3931DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3933DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3934DUAL_slot2,
  OPERAND_dsp340050b49a6c_fld3935DUAL_slot2,
  OPERAND_op0_s22,
  OPERAND_op0_s23,
  OPERAND_dsp340050b49a6c_fld2057,
  OPERAND_dsp340050b49a6c_fld2060,
  OPERAND_dsp340050b49a6c_fld2066,
  OPERAND_dsp340050b49a6c_fld2072,
  OPERAND_dsp340050b49a6c_fld2079,
  OPERAND_dsp340050b49a6c_fld3487DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3488DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3489DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3490DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3491DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3492DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3493DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3494DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3496DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3497DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3498DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3499DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3500DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3502DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3504DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3505DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3506DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3507DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3508DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3509DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3510DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3511DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3512DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3513DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3514DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3515DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3516DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3517DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3518DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3519DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3520DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3522DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3523DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3524DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3527DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3529DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3530DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3531,
  OPERAND_dsp340050b49a6c_fld3532DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3533DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3535DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3536DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3537DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3538DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3539DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3541DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3542DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3543DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3544DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3545DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3546DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3547DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3548DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3549DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3550DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3551DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3552,
  OPERAND_dsp340050b49a6c_fld3553DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3554DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3555DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3556DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3557DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3558DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3559DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3560DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3562DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3563DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3564DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3565DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3566DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3567DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3568DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3569DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3570DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3571DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3572DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3573DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3574DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3575DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3576DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3577DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3578DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3579DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3580DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3581DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3582DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3583DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3584,
  OPERAND_dsp340050b49a6c_fld3585DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3587DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3588DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3589DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3590DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3591DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3592DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3593DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3594DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3595DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3596DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3597DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3598DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3599DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3600DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3601DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3602,
  OPERAND_dsp340050b49a6c_fld3603DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3604DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3606DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3607DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3608DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3609DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3610,
  OPERAND_dsp340050b49a6c_fld3611DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3612DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3613DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3614DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3615DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3616DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3618DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3619,
  OPERAND_dsp340050b49a6c_fld3620DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3621DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3622DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3623DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3624DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3625DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3626DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3936DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3937DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3938DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3939DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3940DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3941DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3943DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3945DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3946DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3947DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3949DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3950DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3951DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3952DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3954DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3957DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3958DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3959DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3960DUAL_slot0,
  OPERAND_dsp340050b49a6c_fld3961DUAL_slot0
};


/* Iclass table.  */

static xtensa_arg_internal Iclass_xt_iclass_rfe_stateArgs[] = {
  { { STATE_PSEXCM }, 'o' },
  { { STATE_EPC1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfde_stateArgs[] = {
  { { STATE_DEPC }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_call12_args[] = {
  { { OPERAND_soffsetx4 }, 'i' },
  { { OPERAND_ar12 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_call12_stateArgs[] = {
  { { STATE_PSCALLINC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_call8_args[] = {
  { { OPERAND_soffsetx4 }, 'i' },
  { { OPERAND_ar8 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_call8_stateArgs[] = {
  { { STATE_PSCALLINC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_call4_args[] = {
  { { OPERAND_soffsetx4 }, 'i' },
  { { OPERAND_ar4 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_call4_stateArgs[] = {
  { { STATE_PSCALLINC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx12_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ar12 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx12_stateArgs[] = {
  { { STATE_PSCALLINC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx8_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ar8 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx8_stateArgs[] = {
  { { STATE_PSCALLINC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx4_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ar4 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx4_stateArgs[] = {
  { { STATE_PSCALLINC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_entry_args[] = {
  { { OPERAND_ars_entry }, 's' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm12x8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_entry_stateArgs[] = {
  { { STATE_PSCALLINC }, 'i' },
  { { STATE_PSEXCM }, 'i' },
  { { STATE_PSWOE }, 'i' },
  { { STATE_WindowBase }, 'm' },
  { { STATE_WindowStart }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_movsp_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_movsp_stateArgs[] = {
  { { STATE_WindowBase }, 'i' },
  { { STATE_WindowStart }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rotw_args[] = {
  { { OPERAND_simm4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rotw_stateArgs[] = {
  { { STATE_WindowBase }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_retw_args[] = {
  { { OPERAND__ars_invisible }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_retw_stateArgs[] = {
  { { STATE_WindowBase }, 'm' },
  { { STATE_WindowStart }, 'm' },
  { { STATE_PSEXCM }, 'i' },
  { { STATE_PSWOE }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfwou_stateArgs[] = {
  { { STATE_EPC1 }, 'i' },
  { { STATE_PSEXCM }, 'o' },
  { { STATE_WindowBase }, 'm' },
  { { STATE_WindowStart }, 'm' },
  { { STATE_PSOWB }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l32e_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_immrx4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s32e_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_immrx4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_windowbase_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_windowbase_stateArgs[] = {
  { { STATE_WindowBase }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_windowbase_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_windowbase_stateArgs[] = {
  { { STATE_WindowBase }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_windowbase_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_windowbase_stateArgs[] = {
  { { STATE_WindowBase }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_windowstart_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_windowstart_stateArgs[] = {
  { { STATE_WindowStart }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_windowstart_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_windowstart_stateArgs[] = {
  { { STATE_WindowStart }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_windowstart_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_windowstart_stateArgs[] = {
  { { STATE_WindowStart }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_add_n_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_addi_n_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ai4const }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bz6_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm6 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_loadi4_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_lsi4x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_mov_n_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_movi_n_args[] = {
  { { OPERAND_ars }, 'o' },
  { { OPERAND_simm7 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_retn_args[] = {
  { { OPERAND__ars_invisible }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_storei4_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_lsi4x4 }, 'i' }
};

static xtensa_arg_internal Iclass_rur_threadptr_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_threadptr_stateArgs[] = {
  { { STATE_THREADPTR }, 'i' }
};

static xtensa_arg_internal Iclass_wur_threadptr_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_threadptr_stateArgs[] = {
  { { STATE_THREADPTR }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_addi_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_simm8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_addmi_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_simm8x256 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_addsub_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bit_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bsi8_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_b4const }, 'i' },
  { { OPERAND_label8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bsi8b_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_bbi }, 'i' },
  { { OPERAND_label8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bsi8u_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_b4constu }, 'i' },
  { { OPERAND_label8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bst8_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_label8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bsz12_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_label12 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_call0_args[] = {
  { { OPERAND_soffsetx4 }, 'i' },
  { { OPERAND_ar0 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_callx0_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ar0 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_exti_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_sae }, 'i' },
  { { OPERAND_op2p1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_jump_args[] = {
  { { OPERAND_soffset }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_jumpx_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l16ui_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x2 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l16si_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x2 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l32i_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l32r_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_uimm16x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l32r_stateArgs[] = {
  { { STATE_LITBADDR }, 'i' },
  { { STATE_LITBEN }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l8i_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_loop_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ulabel8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_loop_stateArgs[] = {
  { { STATE_LBEG }, 'o' },
  { { STATE_LEND }, 'o' },
  { { STATE_LCOUNT }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_loopz_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_ulabel8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_loopz_stateArgs[] = {
  { { STATE_LBEG }, 'o' },
  { { STATE_LEND }, 'o' },
  { { STATE_LCOUNT }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_movi_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_simm12b }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_movz_args[] = {
  { { OPERAND_arr }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_neg_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_return_args[] = {
  { { OPERAND__ars_invisible }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s16i_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x2 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s32i_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s8i_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sar_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sar_stateArgs[] = {
  { { STATE_SAR }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_sari_args[] = {
  { { OPERAND_sas }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sari_stateArgs[] = {
  { { STATE_SAR }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_shifts_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_shifts_stateArgs[] = {
  { { STATE_SAR }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_shiftst_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_shiftst_stateArgs[] = {
  { { STATE_SAR }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_shiftt_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_shiftt_stateArgs[] = {
  { { STATE_SAR }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_slli_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_msalp32 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_srai_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_sargt }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_srli_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_s }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sync_stateArgs[] = {
  { { STATE_XTSYNC }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsil_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_s }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsil_stateArgs[] = {
  { { STATE_PSWOE }, 'i' },
  { { STATE_PSCALLINC }, 'i' },
  { { STATE_PSOWB }, 'i' },
  { { STATE_PSUM }, 'i' },
  { { STATE_PSEXCM }, 'i' },
  { { STATE_PSINTLEVEL }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_lend_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_lend_stateArgs[] = {
  { { STATE_LEND }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_lend_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_lend_stateArgs[] = {
  { { STATE_LEND }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_lend_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_lend_stateArgs[] = {
  { { STATE_LEND }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_lcount_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_lcount_stateArgs[] = {
  { { STATE_LCOUNT }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_lcount_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_lcount_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_LCOUNT }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_lcount_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_lcount_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_LCOUNT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_lbeg_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_lbeg_stateArgs[] = {
  { { STATE_LBEG }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_lbeg_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_lbeg_stateArgs[] = {
  { { STATE_LBEG }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_lbeg_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_lbeg_stateArgs[] = {
  { { STATE_LBEG }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_sar_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_sar_stateArgs[] = {
  { { STATE_SAR }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_sar_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_sar_stateArgs[] = {
  { { STATE_SAR }, 'o' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_sar_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_sar_stateArgs[] = {
  { { STATE_SAR }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_litbase_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_litbase_stateArgs[] = {
  { { STATE_LITBADDR }, 'i' },
  { { STATE_LITBEN }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_litbase_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_litbase_stateArgs[] = {
  { { STATE_LITBADDR }, 'o' },
  { { STATE_LITBEN }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_litbase_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_litbase_stateArgs[] = {
  { { STATE_LITBADDR }, 'm' },
  { { STATE_LITBEN }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_176_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_176_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_208_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ps_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ps_stateArgs[] = {
  { { STATE_PSWOE }, 'i' },
  { { STATE_PSCALLINC }, 'i' },
  { { STATE_PSOWB }, 'i' },
  { { STATE_PSUM }, 'i' },
  { { STATE_PSEXCM }, 'i' },
  { { STATE_PSINTLEVEL }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ps_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ps_stateArgs[] = {
  { { STATE_PSWOE }, 'o' },
  { { STATE_PSCALLINC }, 'o' },
  { { STATE_PSOWB }, 'o' },
  { { STATE_PSUM }, 'o' },
  { { STATE_PSEXCM }, 'o' },
  { { STATE_PSINTLEVEL }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ps_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ps_stateArgs[] = {
  { { STATE_PSWOE }, 'm' },
  { { STATE_PSCALLINC }, 'm' },
  { { STATE_PSOWB }, 'm' },
  { { STATE_PSUM }, 'm' },
  { { STATE_PSEXCM }, 'm' },
  { { STATE_PSINTLEVEL }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc1_stateArgs[] = {
  { { STATE_EPC1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc1_stateArgs[] = {
  { { STATE_EPC1 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc1_stateArgs[] = {
  { { STATE_EPC1 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave1_stateArgs[] = {
  { { STATE_EXCSAVE1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave1_stateArgs[] = {
  { { STATE_EXCSAVE1 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave1_stateArgs[] = {
  { { STATE_EXCSAVE1 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc2_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc2_stateArgs[] = {
  { { STATE_EPC2 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc2_stateArgs[] = {
  { { STATE_EPC2 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc2_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc2_stateArgs[] = {
  { { STATE_EPC2 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave2_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave2_stateArgs[] = {
  { { STATE_EXCSAVE2 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave2_stateArgs[] = {
  { { STATE_EXCSAVE2 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave2_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave2_stateArgs[] = {
  { { STATE_EXCSAVE2 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc3_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc3_stateArgs[] = {
  { { STATE_EPC3 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc3_stateArgs[] = {
  { { STATE_EPC3 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc3_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc3_stateArgs[] = {
  { { STATE_EPC3 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave3_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave3_stateArgs[] = {
  { { STATE_EXCSAVE3 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave3_stateArgs[] = {
  { { STATE_EXCSAVE3 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave3_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave3_stateArgs[] = {
  { { STATE_EXCSAVE3 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc4_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc4_stateArgs[] = {
  { { STATE_EPC4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc4_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc4_stateArgs[] = {
  { { STATE_EPC4 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc4_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc4_stateArgs[] = {
  { { STATE_EPC4 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave4_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave4_stateArgs[] = {
  { { STATE_EXCSAVE4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave4_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave4_stateArgs[] = {
  { { STATE_EXCSAVE4 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave4_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave4_stateArgs[] = {
  { { STATE_EXCSAVE4 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc5_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc5_stateArgs[] = {
  { { STATE_EPC5 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc5_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc5_stateArgs[] = {
  { { STATE_EPC5 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc5_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc5_stateArgs[] = {
  { { STATE_EPC5 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave5_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave5_stateArgs[] = {
  { { STATE_EXCSAVE5 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave5_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave5_stateArgs[] = {
  { { STATE_EXCSAVE5 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave5_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave5_stateArgs[] = {
  { { STATE_EXCSAVE5 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc6_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_epc6_stateArgs[] = {
  { { STATE_EPC6 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc6_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_epc6_stateArgs[] = {
  { { STATE_EPC6 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc6_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_epc6_stateArgs[] = {
  { { STATE_EPC6 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave6_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excsave6_stateArgs[] = {
  { { STATE_EXCSAVE6 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave6_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excsave6_stateArgs[] = {
  { { STATE_EXCSAVE6 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave6_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excsave6_stateArgs[] = {
  { { STATE_EXCSAVE6 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps2_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps2_stateArgs[] = {
  { { STATE_EPS2 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps2_stateArgs[] = {
  { { STATE_EPS2 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps2_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps2_stateArgs[] = {
  { { STATE_EPS2 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps3_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps3_stateArgs[] = {
  { { STATE_EPS3 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps3_stateArgs[] = {
  { { STATE_EPS3 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps3_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps3_stateArgs[] = {
  { { STATE_EPS3 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps4_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps4_stateArgs[] = {
  { { STATE_EPS4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps4_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps4_stateArgs[] = {
  { { STATE_EPS4 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps4_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps4_stateArgs[] = {
  { { STATE_EPS4 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps5_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps5_stateArgs[] = {
  { { STATE_EPS5 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps5_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps5_stateArgs[] = {
  { { STATE_EPS5 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps5_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps5_stateArgs[] = {
  { { STATE_EPS5 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps6_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_eps6_stateArgs[] = {
  { { STATE_EPS6 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps6_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_eps6_stateArgs[] = {
  { { STATE_EPS6 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps6_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_eps6_stateArgs[] = {
  { { STATE_EPS6 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excvaddr_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_excvaddr_stateArgs[] = {
  { { STATE_EXCVADDR }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excvaddr_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_excvaddr_stateArgs[] = {
  { { STATE_EXCVADDR }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excvaddr_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_excvaddr_stateArgs[] = {
  { { STATE_EXCVADDR }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_depc_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_depc_stateArgs[] = {
  { { STATE_DEPC }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_depc_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_depc_stateArgs[] = {
  { { STATE_DEPC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_depc_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_depc_stateArgs[] = {
  { { STATE_DEPC }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_exccause_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_exccause_stateArgs[] = {
  { { STATE_EXCCAUSE }, 'i' },
  { { STATE_XTSYNC }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_exccause_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_exccause_stateArgs[] = {
  { { STATE_EXCCAUSE }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_exccause_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_exccause_stateArgs[] = {
  { { STATE_EXCCAUSE }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_prid_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_vecbase_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_vecbase_stateArgs[] = {
  { { STATE_VECBASE }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_vecbase_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_vecbase_stateArgs[] = {
  { { STATE_VECBASE }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_vecbase_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_vecbase_stateArgs[] = {
  { { STATE_VECBASE }, 'm' }
};

static xtensa_arg_internal Iclass_xt_mul16_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfi_args[] = {
  { { OPERAND_s }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfi_stateArgs[] = {
  { { STATE_PSWOE }, 'o' },
  { { STATE_PSCALLINC }, 'o' },
  { { STATE_PSOWB }, 'o' },
  { { STATE_PSUM }, 'o' },
  { { STATE_PSEXCM }, 'o' },
  { { STATE_PSINTLEVEL }, 'o' },
  { { STATE_EPC1 }, 'i' },
  { { STATE_EPC2 }, 'i' },
  { { STATE_EPC3 }, 'i' },
  { { STATE_EPC4 }, 'i' },
  { { STATE_EPC5 }, 'i' },
  { { STATE_EPC6 }, 'i' },
  { { STATE_EPS2 }, 'i' },
  { { STATE_EPS3 }, 'i' },
  { { STATE_EPS4 }, 'i' },
  { { STATE_EPS5 }, 'i' },
  { { STATE_EPS6 }, 'i' },
  { { STATE_InOCDMode }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_wait_args[] = {
  { { OPERAND_s }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wait_stateArgs[] = {
  { { STATE_PSINTLEVEL }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_interrupt_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_interrupt_stateArgs[] = {
  { { STATE_INTERRUPT }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_intset_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_intset_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_INTERRUPT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_intclear_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_intclear_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_INTERRUPT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_intenable_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_intenable_stateArgs[] = {
  { { STATE_INTENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_intenable_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_intenable_stateArgs[] = {
  { { STATE_INTENABLE }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_intenable_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_intenable_stateArgs[] = {
  { { STATE_INTENABLE }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_break_args[] = {
  { { OPERAND_imms }, 'i' },
  { { OPERAND_immt }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_break_stateArgs[] = {
  { { STATE_PSEXCM }, 'i' },
  { { STATE_PSINTLEVEL }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_break_n_args[] = {
  { { OPERAND_imms }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_break_n_stateArgs[] = {
  { { STATE_PSEXCM }, 'i' },
  { { STATE_PSINTLEVEL }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreaka0_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreaka0_stateArgs[] = {
  { { STATE_DBREAKA0 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreaka0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreaka0_stateArgs[] = {
  { { STATE_DBREAKA0 }, 'o' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreaka0_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreaka0_stateArgs[] = {
  { { STATE_DBREAKA0 }, 'm' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreakc0_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreakc0_stateArgs[] = {
  { { STATE_DBREAKC0 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreakc0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreakc0_stateArgs[] = {
  { { STATE_DBREAKC0 }, 'o' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreakc0_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreakc0_stateArgs[] = {
  { { STATE_DBREAKC0 }, 'm' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreaka1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreaka1_stateArgs[] = {
  { { STATE_DBREAKA1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreaka1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreaka1_stateArgs[] = {
  { { STATE_DBREAKA1 }, 'o' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreaka1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreaka1_stateArgs[] = {
  { { STATE_DBREAKA1 }, 'm' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreakc1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_dbreakc1_stateArgs[] = {
  { { STATE_DBREAKC1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreakc1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_dbreakc1_stateArgs[] = {
  { { STATE_DBREAKC1 }, 'o' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreakc1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_dbreakc1_stateArgs[] = {
  { { STATE_DBREAKC1 }, 'm' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ibreaka0_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ibreaka0_stateArgs[] = {
  { { STATE_IBREAKA0 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ibreaka0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ibreaka0_stateArgs[] = {
  { { STATE_IBREAKA0 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ibreaka0_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ibreaka0_stateArgs[] = {
  { { STATE_IBREAKA0 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ibreaka1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ibreaka1_stateArgs[] = {
  { { STATE_IBREAKA1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ibreaka1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ibreaka1_stateArgs[] = {
  { { STATE_IBREAKA1 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ibreaka1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ibreaka1_stateArgs[] = {
  { { STATE_IBREAKA1 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ibreakenable_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ibreakenable_stateArgs[] = {
  { { STATE_IBREAKENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ibreakenable_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ibreakenable_stateArgs[] = {
  { { STATE_IBREAKENABLE }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ibreakenable_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ibreakenable_stateArgs[] = {
  { { STATE_IBREAKENABLE }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_debugcause_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_debugcause_stateArgs[] = {
  { { STATE_DEBUGCAUSE }, 'i' },
  { { STATE_DBNUM }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_debugcause_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_debugcause_stateArgs[] = {
  { { STATE_DEBUGCAUSE }, 'o' },
  { { STATE_DBNUM }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_debugcause_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_debugcause_stateArgs[] = {
  { { STATE_DEBUGCAUSE }, 'm' },
  { { STATE_DBNUM }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_icount_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_icount_stateArgs[] = {
  { { STATE_ICOUNT }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_icount_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_icount_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_ICOUNT }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_icount_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_icount_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_ICOUNT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_icountlevel_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_icountlevel_stateArgs[] = {
  { { STATE_ICOUNTLEVEL }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_icountlevel_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_icountlevel_stateArgs[] = {
  { { STATE_ICOUNTLEVEL }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_icountlevel_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_icountlevel_stateArgs[] = {
  { { STATE_ICOUNTLEVEL }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ddr_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ddr_stateArgs[] = {
  { { STATE_DDR }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ddr_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ddr_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_DDR }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ddr_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ddr_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_DDR }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfdo_args[] = {
  { { OPERAND_imms }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfdo_stateArgs[] = {
  { { STATE_InOCDMode }, 'm' },
  { { STATE_EPC6 }, 'i' },
  { { STATE_PSWOE }, 'o' },
  { { STATE_PSCALLINC }, 'o' },
  { { STATE_PSOWB }, 'o' },
  { { STATE_PSUM }, 'o' },
  { { STATE_PSEXCM }, 'o' },
  { { STATE_PSINTLEVEL }, 'o' },
  { { STATE_EPS6 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rfdd_stateArgs[] = {
  { { STATE_InOCDMode }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_mmid_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_mmid_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_bbool1_args[] = {
  { { OPERAND_br }, 'o' },
  { { OPERAND_bs }, 'i' },
  { { OPERAND_bt }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bbool4_args[] = {
  { { OPERAND_bt }, 'o' },
  { { OPERAND_bs4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bbool8_args[] = {
  { { OPERAND_bt }, 'o' },
  { { OPERAND_bs8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bbranch_args[] = {
  { { OPERAND_bs }, 'i' },
  { { OPERAND_label8 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_bmove_args[] = {
  { { OPERAND_arr }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_bt }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_RSR_BR_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_brall }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_WSR_BR_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_brall }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_XSR_BR_args[] = {
  { { OPERAND_art }, 'm' },
  { { OPERAND_brall }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ccount_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ccount_stateArgs[] = {
  { { STATE_CCOUNT }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ccount_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ccount_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_CCOUNT }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ccount_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ccount_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' },
  { { STATE_CCOUNT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ccompare0_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ccompare0_stateArgs[] = {
  { { STATE_CCOMPARE0 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ccompare0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ccompare0_stateArgs[] = {
  { { STATE_CCOMPARE0 }, 'o' },
  { { STATE_INTERRUPT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ccompare0_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ccompare0_stateArgs[] = {
  { { STATE_CCOMPARE0 }, 'm' },
  { { STATE_INTERRUPT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ccompare1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_ccompare1_stateArgs[] = {
  { { STATE_CCOMPARE1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ccompare1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_ccompare1_stateArgs[] = {
  { { STATE_CCOMPARE1 }, 'o' },
  { { STATE_INTERRUPT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ccompare1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_ccompare1_stateArgs[] = {
  { { STATE_CCOMPARE1 }, 'm' },
  { { STATE_INTERRUPT }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_icache_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_icache_lock_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm4x16 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_icache_inv_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_licx_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sicx_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_dcache_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_dcache_ind_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm4x16 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_dcache_inv_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_dpf_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_dcache_lock_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm4x16 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sdct_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_ldct_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_idtlb_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_idtlb_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rdtlb_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wdtlb_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wdtlb_stateArgs[] = {
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_iitlb_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_ritlb_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_witlb_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_cpenable_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_cpenable_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_cpenable_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_cpenable_stateArgs[] = {
  { { STATE_CPENABLE }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_cpenable_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_cpenable_stateArgs[] = {
  { { STATE_CPENABLE }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_clamp_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_tp7 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_minmax_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_nsa_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_sx_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_tp7 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_l32ai_args[] = {
  { { OPERAND_art }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s32ri_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s32c1i_args[] = {
  { { OPERAND_art }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_uimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_s32c1i_stateArgs[] = {
  { { STATE_SCOMPARE1 }, 'i' },
  { { STATE_XTSYNC }, 'i' },
  { { STATE_SCOMPARE1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_scompare1_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_scompare1_stateArgs[] = {
  { { STATE_SCOMPARE1 }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_scompare1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_scompare1_stateArgs[] = {
  { { STATE_SCOMPARE1 }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_scompare1_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_scompare1_stateArgs[] = {
  { { STATE_SCOMPARE1 }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_atomctl_args[] = {
  { { OPERAND_art }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_rsr_atomctl_stateArgs[] = {
  { { STATE_ATOMCTL }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_atomctl_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_xt_iclass_wsr_atomctl_stateArgs[] = {
  { { STATE_ATOMCTL }, 'o' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_atomctl_args[] = {
  { { OPERAND_art }, 'm' }
};

static xtensa_arg_internal Iclass_xt_iclass_xsr_atomctl_stateArgs[] = {
  { { STATE_ATOMCTL }, 'm' },
  { { STATE_XTSYNC }, 'o' }
};

static xtensa_arg_internal Iclass_rur_fcr_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_fcr_stateArgs[] = {
  { { STATE_RoundMode }, 'i' },
  { { STATE_InvalidEnable }, 'i' },
  { { STATE_DivZeroEnable }, 'i' },
  { { STATE_OverflowEnable }, 'i' },
  { { STATE_UnderflowEnable }, 'i' },
  { { STATE_InexactEnable }, 'i' },
  { { STATE_FPreserved20 }, 'i' },
  { { STATE_FPreserved5 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_fcr_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_fcr_stateArgs[] = {
  { { STATE_RoundMode }, 'o' },
  { { STATE_InvalidEnable }, 'o' },
  { { STATE_DivZeroEnable }, 'o' },
  { { STATE_OverflowEnable }, 'o' },
  { { STATE_UnderflowEnable }, 'o' },
  { { STATE_InexactEnable }, 'o' },
  { { STATE_FPreserved20 }, 'o' },
  { { STATE_FPreserved5 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_fsr_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_fsr_stateArgs[] = {
  { { STATE_InvalidFlag }, 'i' },
  { { STATE_DivZeroFlag }, 'i' },
  { { STATE_OverflowFlag }, 'i' },
  { { STATE_UnderflowFlag }, 'i' },
  { { STATE_InexactFlag }, 'i' },
  { { STATE_FPreserved20a }, 'i' },
  { { STATE_FPreserved7 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_fsr_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_fsr_stateArgs[] = {
  { { STATE_InvalidFlag }, 'o' },
  { { STATE_DivZeroFlag }, 'o' },
  { { STATE_OverflowFlag }, 'o' },
  { { STATE_UnderflowFlag }, 'o' },
  { { STATE_InexactFlag }, 'o' },
  { { STATE_FPreserved20a }, 'o' },
  { { STATE_FPreserved7 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_args[] = {
  { { OPERAND_frr }, 'o' },
  { { OPERAND_frs }, 'i' },
  { { OPERAND_frt }, 'i' }
};

static xtensa_arg_internal Iclass_fp_stateArgs[] = {
  { { STATE_RoundMode }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_mac_args[] = {
  { { OPERAND_frr }, 'm' },
  { { OPERAND_frs }, 'i' },
  { { OPERAND_frt }, 'i' }
};

static xtensa_arg_internal Iclass_fp_mac_stateArgs[] = {
  { { STATE_RoundMode }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_cmov_args[] = {
  { { OPERAND_frr }, 'm' },
  { { OPERAND_frs }, 'i' },
  { { OPERAND_bt }, 'i' }
};

static xtensa_arg_internal Iclass_fp_cmov_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_mov_args[] = {
  { { OPERAND_frr }, 'm' },
  { { OPERAND_frs }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_fp_mov_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_mov2_args[] = {
  { { OPERAND_frr }, 'o' },
  { { OPERAND_frs }, 'i' }
};

static xtensa_arg_internal Iclass_fp_mov2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_cmp_args[] = {
  { { OPERAND_br }, 'o' },
  { { OPERAND_frs }, 'i' },
  { { OPERAND_frt }, 'i' }
};

static xtensa_arg_internal Iclass_fp_cmp_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_float_args[] = {
  { { OPERAND_frr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_t }, 'i' }
};

static xtensa_arg_internal Iclass_fp_float_stateArgs[] = {
  { { STATE_RoundMode }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_int_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_frs }, 'i' },
  { { OPERAND_t }, 'i' }
};

static xtensa_arg_internal Iclass_fp_int_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_rfr_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_frs }, 'i' }
};

static xtensa_arg_internal Iclass_fp_rfr_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_wfr_args[] = {
  { { OPERAND_frr }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_fp_wfr_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsi_args[] = {
  { { OPERAND_frt }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_cimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsi_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsiu_args[] = {
  { { OPERAND_frt }, 'o' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_cimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsiu_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsx_args[] = {
  { { OPERAND_frr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsx_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsxu_args[] = {
  { { OPERAND_frr }, 'o' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_fp_lsxu_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssi_args[] = {
  { { OPERAND_frt }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_cimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssi_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssiu_args[] = {
  { { OPERAND_frt }, 'i' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_cimm8x4 }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssiu_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssx_args[] = {
  { { OPERAND_frr }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssx_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssxu_args[] = {
  { { OPERAND_frr }, 'i' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_fp_ssxu_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_ARGMAX_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_ARGMAX_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_HSAR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_HSAR_stateArgs[] = {
  { { STATE_HSAR3 }, 'i' },
  { { STATE_HSAR2 }, 'i' },
  { { STATE_HSAR1 }, 'i' },
  { { STATE_HSAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_HSAR2SAR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_HSAR2SAR_stateArgs[] = {
  { { STATE_SAR3 }, 'o' },
  { { STATE_SAR2 }, 'o' },
  { { STATE_SAR1 }, 'o' },
  { { STATE_SAR0 }, 'o' },
  { { STATE_HSAR3 }, 'i' },
  { { STATE_HSAR2 }, 'i' },
  { { STATE_HSAR1 }, 'i' },
  { { STATE_HSAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_INTERP_EXT_N_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_INTERP_EXT_N_stateArgs[] = {
  { { STATE_INTERP_EXT_N }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_INTERP_EXT_L_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_INTERP_EXT_L_stateArgs[] = {
  { { STATE_INTERP_EXT_L }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_LLR_BUF_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper70_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_LLR_BUF_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_LLR_POS_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_LLR_POS_stateArgs[] = {
  { { STATE_LLR_POS }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_MAX_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_MAX_stateArgs[] = {
  { { STATE_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_NCO_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_NCO_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_PERM_REG_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_PERM_REG_stateArgs[] = {
  { { STATE_PERM_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_PHASOR_N_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_PHASOR_N_stateArgs[] = {
  { { STATE_PHASOR_N }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_PHASOR_OFFSET_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_PHASOR_OFFSET_stateArgs[] = {
  { { STATE_PHASOR_OFFSET }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SAR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_SAR_stateArgs[] = {
  { { STATE_SAR3 }, 'i' },
  { { STATE_SAR2 }, 'i' },
  { { STATE_SAR1 }, 'i' },
  { { STATE_SAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SCALE_REG_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_SCALE_REG_stateArgs[] = {
  { { STATE_SCALE_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SMOD_BUF_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SMOD_BUF_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SMOD_OFFSET_TABLE_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_SMOD_OFFSET_TABLE_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SMOD_POS_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_SMOD_POS_stateArgs[] = {
  { { STATE_SMOD_POS }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_SOV_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_SOV_stateArgs[] = {
  { { STATE_SOV }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_GET_WGHT_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_GET_WGHT_stateArgs[] = {
  { { STATE_WEIGHT_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_ARGMAX_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_ARGMAX_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_EXT_REGS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper88_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_EXT_REGS_stateArgs[] = {
  { { STATE_INTERP_EXT_N }, 'o' },
  { { STATE_INTERP_EXT_L }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_HSAR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_HSAR_stateArgs[] = {
  { { STATE_HSAR0 }, 'o' },
  { { STATE_HSAR1 }, 'o' },
  { { STATE_HSAR2 }, 'o' },
  { { STATE_HSAR3 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_LLR_BUF_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper78_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_LLR_BUF_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_LLR_POS_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_LLR_POS_stateArgs[] = {
  { { STATE_LLR_POS }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_MAX_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_MAX_stateArgs[] = {
  { { STATE_MAX_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_NCO_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_NCO_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_PERM_REG_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_PERM_REG_stateArgs[] = {
  { { STATE_PERM_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_PHASOR_N_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_PHASOR_N_stateArgs[] = {
  { { STATE_PHASOR_N }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_PHASOR_OFFSET_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_PHASOR_OFFSET_stateArgs[] = {
  { { STATE_PHASOR_OFFSET }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SAR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SAR_stateArgs[] = {
  { { STATE_SAR0 }, 'o' },
  { { STATE_SAR1 }, 'o' },
  { { STATE_SAR2 }, 'o' },
  { { STATE_SAR3 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SCALE_REG_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SCALE_REG_stateArgs[] = {
  { { STATE_SCALE_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SMOD_BUF_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SMOD_BUF_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SMOD_OFFSET_TABLE_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SMOD_OFFSET_TABLE_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SMOD_POS_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SMOD_POS_stateArgs[] = {
  { { STATE_SMOD_POS }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SOV_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_SOV_stateArgs[] = {
  { { STATE_SOV }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_WGHT_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SET_WGHT_stateArgs[] = {
  { { STATE_WEIGHT_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper72_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper73_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X32_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper74_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper74_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper74_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper74_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC2X64_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC32_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper72_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper75_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC32_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_IH_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_IH_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_IL_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_IL_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_RH_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_RH_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_RL_args[] = {
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LAC_RL_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'o' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper74_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_PINC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'o' },
  { { OPERAND_art }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_PINC_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_PINC_X_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'o' },
  { { OPERAND_art }, 'm' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_PINC_X_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_U_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'o' },
  { { OPERAND_art }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_U_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_X_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'o' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_X_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_XU_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'o' },
  { { OPERAND_art }, 'm' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LCM_XU_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LP_args[] = {
  { { OPERAND_dsp340050b49a6c_oper79_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LP_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LP_X_args[] = {
  { { OPERAND_dsp340050b49a6c_oper79_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LP_X_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper79_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper76_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LQ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LQ_X_args[] = {
  { { OPERAND_dsp340050b49a6c_oper79_reg }, 'm' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LQ_X_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper80_reg }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper80_reg }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper80_reg }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper77_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper80_reg }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper72_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper84_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X32_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC2X64_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC32_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper72_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper86_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC32_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_IH_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper87_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_IH_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_IL_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper87_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_IL_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_RH_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper87_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_RH_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_RL_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper87_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SAC_RL_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_PINC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper87_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_PINC_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_PINC_X_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_PINC_X_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_U_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper87_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_U_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_X_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_X_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_XU_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'm' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SCM_XU_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_STORE_P_args[] = {
  { { OPERAND_dsp340050b49a6c_oper79_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_STORE_P_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_STORE_Q_args[] = {
  { { OPERAND_dsp340050b49a6c_oper79_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper85_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_STORE_Q_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_DUP_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_DUP_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_LN_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_LN_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_LN_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_LN_I_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_LN_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2CM_LN_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2PQ_LN_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper57_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2PQ_LN_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2SAR_DUP_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AR2SAR_DUP_stateArgs[] = {
  { { STATE_SAR3 }, 'o' },
  { { STATE_SAR2 }, 'o' },
  { { STATE_SAR1 }, 'o' },
  { { STATE_SAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLRAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_CLRAC_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLRCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_CLRCM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CM2AR_LN_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper64_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CM2AR_LN_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CM2AR_LN_I_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper64_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CM2AR_LN_I_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CM2AR_LN_R_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper64_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CM2AR_LN_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_COMB_AR_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CONJ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CONJ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV2AC32_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MOV2AC32_I_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV2AC32_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper71_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MOV2AC32_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV2CM2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV2CM2PQ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper66_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_I_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_I2R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_I2R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper66_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_R2I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAC_R2I_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVAR2_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCM2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper81_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCM2PQ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_4_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_4_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_5_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_5_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_6_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_6_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_7_args[] = {
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND_7_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_4_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_4_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_5_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_5_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_6_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_6_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_7_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVCND8_7_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV_I_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVPQ2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOVPQ2PQ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NEGCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NEGCM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP16LLR_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP16LLR_1_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_LLR_POS }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PQ2CM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PQ2CM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SWAPAC_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_SWAPAC_R_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SWAPAC_RI_args[] = {
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SWAPAC_RI_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SWAPB_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADD2AC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADD2AC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAR0 }, 'i' },
  { { STATE_SAR1 }, 'i' },
  { { STATE_SAR2 }, 'i' },
  { { STATE_SAR3 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CDOT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_CDOT_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CDOTAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_CDOTAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CDOTACS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_CDOTACS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_CMAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMACS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_CMACS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPY_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_CMPY_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPY2CM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPY2CM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPY2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_CMPY2PQ_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPYS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_CMPYS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPYXP2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMPYXP2PQ_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_COMB32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper66_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper67_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_COMB32_stateArgs[] = {
  { { STATE_WEIGHT_REG }, 'i' },
  { { STATE_SCALE_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_DOT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_DOT_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_DOTAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_DOTAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_DOTACS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_DOTACS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LIN_INT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LIN_INT_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_INTERP_EXT_L }, 'i' },
  { { STATE_INTERP_EXT_N }, 'i' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_SCALE_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LLRPRE1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_LLRPRE1_stateArgs[] = {
  { { STATE_PERM_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LLRPRE2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper66_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper78_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LLRPRE2_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_LLR_POS }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MAC8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MAC8_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACD8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACD8_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_0_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_1_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_2_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACPQXP_3_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACXP2_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACXP2_0_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACXP2_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACXP2_1_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_0_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_1_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_2_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MACXP_3_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV2AC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MOV2AC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAR0 }, 'i' },
  { { STATE_SAR1 }, 'i' },
  { { STATE_SAR2 }, 'i' },
  { { STATE_SAR3 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPY_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPY_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPY2CM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPY2CM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPY2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_MPY2PQ_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPY8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPY8_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYADD8_2CM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYADD8_2CM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYD8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYD8_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_0_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_1_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_2_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYPQXP_3_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP2PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP2PQ_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP2_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP2_0_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP2_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP2_1_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_0_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_1_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_2_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_MPYXP_3_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NORMACD_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_NORMACD_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NORMACPQ_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_NORMACPQ_I_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NORMACPQ_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_NORMACPQ_R_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NORMD_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_NORMD_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NORMPYPQ_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_NORMPYPQ_I_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NORMPYPQ_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_NORMPYPQ_R_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RCMAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_RCMAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RCMPY_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_RCMPY_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RCMPY2CM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RCMPY2CM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIR_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIRA_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIRA_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIRD_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIRD_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIRDA_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper54_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RFIRDA_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RMAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_RMAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RMPY_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_RMPY_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RMPY2CM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_RMPY2CM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMOD_ALIGN_args[] = {
  { { OPERAND_dsp340050b49a6c_oper63_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper78_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMOD_ALIGN_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_SMOD_POS }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMOD_SCR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper66_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMOD_SCR_stateArgs[] = {
  { { STATE_PERM_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUB2AC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUB2AC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAR0 }, 'i' },
  { { STATE_SAR1 }, 'i' },
  { { STATE_SAR2 }, 'i' },
  { { STATE_SAR3 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_WGHT32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper50_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper62_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_WGHT32_stateArgs[] = {
  { { STATE_WEIGHT_REG }, 'i' },
  { { STATE_SCALE_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLRTIEP_args[] = {
  { { OPERAND_dsp340050b49a6c_oper61_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLRTIEP_stateArgs[] = {
  { { STATE_EXPSTATE }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_2FIFO_0_intfArgs[] = {
  INTERFACE_OUTQ0_128
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_2FIFO_1_intfArgs[] = {
  INTERFACE_OUTQ1_128
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_2FIFO_2_intfArgs[] = {
  INTERFACE_OUTQ2_128
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_2FIFO_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_2FIFO_3_intfArgs[] = {
  INTERFACE_OUTQ3_128
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_R2FIFO_0_intfArgs[] = {
  INTERFACE_OUTQ0_128
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_R2FIFO_1_intfArgs[] = {
  INTERFACE_OUTQ1_128
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_R2FIFO_2_intfArgs[] = {
  INTERFACE_OUTQ2_128
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_R2FIFO_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_EXT_R2FIFO_3_intfArgs[] = {
  INTERFACE_OUTQ3_128
};

static xtensa_arg_internal Iclass_iclass_LUT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_LUT_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_LUT_AR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT_AR_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_LUT_AR_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_LUT_IEXT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT_IEXT_stateArgs[] = {
  { { STATE_INTERP_EXT_L }, 'i' },
  { { STATE_INTERP_EXT_N }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_LUT_IEXT_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_LUT_PHASOR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT_PHASOR_stateArgs[] = {
  { { STATE_PHASOR_N }, 'i' },
  { { STATE_PHASOR_OFFSET }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_LUT_PHASOR_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_LUT_REXT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT_REXT_stateArgs[] = {
  { { STATE_INTERP_EXT_L }, 'i' },
  { { STATE_INTERP_EXT_N }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_LUT_REXT_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_LUT_WRITE_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper64_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LUT_WRITE_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_LUT_WRITE_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_interface Iclass_iclass_MOVEQ128_0_intfArgs[] = {
  INTERFACE_OUTQ0_128,
  INTERFACE_INQ0_128
};

static xtensa_interface Iclass_iclass_MOVEQ128_1_intfArgs[] = {
  INTERFACE_OUTQ1_128,
  INTERFACE_INQ1_128
};

static xtensa_interface Iclass_iclass_MOVEQ128_2_intfArgs[] = {
  INTERFACE_OUTQ2_128,
  INTERFACE_INQ2_128
};

static xtensa_interface Iclass_iclass_MOVEQ128_3_intfArgs[] = {
  INTERFACE_OUTQ3_128,
  INTERFACE_INQ3_128
};

static xtensa_interface Iclass_iclass_MOVEQ128_4_intfArgs[] = {
  INTERFACE_OUTQ4_128,
  INTERFACE_INQ4_128
};

static xtensa_interface Iclass_iclass_MOVEQ128_5_intfArgs[] = {
  INTERFACE_OUTQ5_128,
  INTERFACE_INQ5_128
};

static xtensa_interface Iclass_iclass_MOVEQ32_0_intfArgs[] = {
  INTERFACE_OUTQ0_32,
  INTERFACE_INQ0_32
};

static xtensa_interface Iclass_iclass_MOVEQ32_1_intfArgs[] = {
  INTERFACE_OUTQ1_32,
  INTERFACE_INQ1_32
};

static xtensa_interface Iclass_iclass_MOVEQ32_2_intfArgs[] = {
  INTERFACE_OUTQ2_32,
  INTERFACE_INQ2_32
};

static xtensa_interface Iclass_iclass_MOVEQ32_3_intfArgs[] = {
  INTERFACE_OUTQ3_32,
  INTERFACE_INQ3_32
};

static xtensa_arg_internal Iclass_iclass_NCO_UPDATE_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NCO_UPDATE_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'm' },
  { { STATE_PHASOR_N }, 'i' },
  { { STATE_PHASOR_OFFSET }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_NCO_UPDATE_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_POP128_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP128_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_0_intfArgs[] = {
  INTERFACE_INQ0_128
};

static xtensa_arg_internal Iclass_iclass_POP128_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP128_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_1_intfArgs[] = {
  INTERFACE_INQ1_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2_intfArgs[] = {
  INTERFACE_INQ2_128
};

static xtensa_arg_internal Iclass_iclass_POP128_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP128_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_3_intfArgs[] = {
  INTERFACE_INQ3_128
};

static xtensa_arg_internal Iclass_iclass_POP128_4_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP128_4_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_4_intfArgs[] = {
  INTERFACE_INQ4_128
};

static xtensa_arg_internal Iclass_iclass_POP128_5_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP128_5_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_5_intfArgs[] = {
  INTERFACE_INQ5_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2CMPQ_0_intfArgs[] = {
  INTERFACE_INQ0_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2CMPQ_1_intfArgs[] = {
  INTERFACE_INQ1_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2CMPQ_2_intfArgs[] = {
  INTERFACE_INQ2_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2CMPQ_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2CMPQ_3_intfArgs[] = {
  INTERFACE_INQ3_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2M_0_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper83_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2M_0_intfArgs[] = {
  INTERFACE_INQ0_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2M_1_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper83_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2M_1_intfArgs[] = {
  INTERFACE_INQ1_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2M_2_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper83_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2M_2_intfArgs[] = {
  INTERFACE_INQ2_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2M_3_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper83_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2M_3_intfArgs[] = {
  INTERFACE_INQ3_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_0_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_0_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2PQ_0_intfArgs[] = {
  INTERFACE_INQ0_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_1_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_1_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2PQ_1_intfArgs[] = {
  INTERFACE_INQ1_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_2_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2PQ_2_intfArgs[] = {
  INTERFACE_INQ2_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_3_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_3_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2PQ_3_intfArgs[] = {
  INTERFACE_INQ3_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_4_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_4_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2PQ_4_intfArgs[] = {
  INTERFACE_INQ4_128
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_5_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_POP128_2PQ_5_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP128_2PQ_5_intfArgs[] = {
  INTERFACE_INQ5_128
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_01_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_01_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP2X128_2PQ_01_intfArgs[] = {
  INTERFACE_INQ0_128,
  INTERFACE_INQ1_128
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_03_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_03_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP2X128_2PQ_03_intfArgs[] = {
  INTERFACE_INQ0_128,
  INTERFACE_INQ3_128
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_21_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_21_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP2X128_2PQ_21_intfArgs[] = {
  INTERFACE_INQ2_128,
  INTERFACE_INQ1_128
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_23_args[] = {
  { { OPERAND_dsp340050b49a6c_oper65_reg }, 'o' }
};

static xtensa_arg_internal Iclass_iclass_POP2X128_2PQ_23_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_POP2X128_2PQ_23_intfArgs[] = {
  INTERFACE_INQ2_128,
  INTERFACE_INQ3_128
};

static xtensa_arg_internal Iclass_iclass_POP32_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_interface Iclass_iclass_POP32_0_intfArgs[] = {
  INTERFACE_INQ0_32
};

static xtensa_arg_internal Iclass_iclass_POP32_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_interface Iclass_iclass_POP32_1_intfArgs[] = {
  INTERFACE_INQ1_32
};

static xtensa_arg_internal Iclass_iclass_POP32_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_interface Iclass_iclass_POP32_2_intfArgs[] = {
  INTERFACE_INQ2_32
};

static xtensa_arg_internal Iclass_iclass_POP32_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_interface Iclass_iclass_POP32_3_intfArgs[] = {
  INTERFACE_INQ3_32
};

static xtensa_arg_internal Iclass_iclass_PUSH128_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper78_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PUSH128_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_PUSH128_intfArgs[] = {
  INTERFACE_OUTQ0_128,
  INTERFACE_OUTQ0_128_KILL,
  INTERFACE_OUTQ1_128,
  INTERFACE_OUTQ1_128_KILL,
  INTERFACE_OUTQ2_128,
  INTERFACE_OUTQ2_128_KILL,
  INTERFACE_OUTQ3_128,
  INTERFACE_OUTQ3_128_KILL,
  INTERFACE_OUTQ4_128,
  INTERFACE_OUTQ4_128_KILL,
  INTERFACE_OUTQ5_128,
  INTERFACE_OUTQ5_128_KILL
};

static xtensa_arg_internal Iclass_iclass_PUSH128_M_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper52_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper83_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_PUSH128_M_intfArgs[] = {
  INTERFACE_OUTQ0_128,
  INTERFACE_OUTQ0_128_KILL,
  INTERFACE_OUTQ1_128,
  INTERFACE_OUTQ1_128_KILL,
  INTERFACE_OUTQ2_128,
  INTERFACE_OUTQ2_128_KILL,
  INTERFACE_OUTQ3_128,
  INTERFACE_OUTQ3_128_KILL
};

static xtensa_arg_internal Iclass_iclass_PUSH128_PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper78_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PUSH128_PQ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_PUSH128_PQ_intfArgs[] = {
  INTERFACE_OUTQ0_128,
  INTERFACE_OUTQ0_128_KILL,
  INTERFACE_OUTQ1_128,
  INTERFACE_OUTQ1_128_KILL,
  INTERFACE_OUTQ2_128,
  INTERFACE_OUTQ2_128_KILL,
  INTERFACE_OUTQ3_128,
  INTERFACE_OUTQ3_128_KILL,
  INTERFACE_OUTQ4_128,
  INTERFACE_OUTQ4_128_KILL,
  INTERFACE_OUTQ5_128,
  INTERFACE_OUTQ5_128_KILL
};

static xtensa_arg_internal Iclass_iclass_PUSH2X128_PQ_args[] = {
  { { OPERAND_dsp340050b49a6c_oper56_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper57_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PUSH2X128_PQ_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_PUSH2X128_PQ_intfArgs[] = {
  INTERFACE_OUTQ0_128,
  INTERFACE_OUTQ1_128,
  INTERFACE_OUTQ2_128,
  INTERFACE_OUTQ3_128,
  INTERFACE_OUTQ0_128_KILL,
  INTERFACE_OUTQ1_128_KILL,
  INTERFACE_OUTQ2_128_KILL,
  INTERFACE_OUTQ3_128_KILL
};

static xtensa_arg_internal Iclass_iclass_PUSH32_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_PUSH32_intfArgs[] = {
  INTERFACE_OUTQ0_32,
  INTERFACE_OUTQ0_32_KILL,
  INTERFACE_OUTQ1_32,
  INTERFACE_OUTQ1_32_KILL,
  INTERFACE_OUTQ2_32,
  INTERFACE_OUTQ2_32_KILL,
  INTERFACE_OUTQ3_32,
  INTERFACE_OUTQ3_32_KILL,
  INTERFACE_OUTQ4_32,
  INTERFACE_OUTQ4_32_KILL,
  INTERFACE_OUTQ5_32,
  INTERFACE_OUTQ5_32_KILL
};

static xtensa_arg_internal Iclass_iclass_QREADY_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper61_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_QREADY_intfArgs[] = {
  INTERFACE_INQ0_32_NOTRDY,
  INTERFACE_INQ1_32_NOTRDY,
  INTERFACE_INQ2_32_NOTRDY,
  INTERFACE_INQ3_32_NOTRDY,
  INTERFACE_OUTQ0_32_NOTRDY,
  INTERFACE_OUTQ1_32_NOTRDY,
  INTERFACE_OUTQ2_32_NOTRDY,
  INTERFACE_OUTQ3_32_NOTRDY,
  INTERFACE_OUTQ4_32_NOTRDY,
  INTERFACE_OUTQ5_32_NOTRDY,
  INTERFACE_INQ0_128_NOTRDY,
  INTERFACE_INQ1_128_NOTRDY,
  INTERFACE_INQ2_128_NOTRDY,
  INTERFACE_INQ3_128_NOTRDY,
  INTERFACE_INQ4_128_NOTRDY,
  INTERFACE_INQ5_128_NOTRDY,
  INTERFACE_OUTQ0_128_NOTRDY,
  INTERFACE_OUTQ1_128_NOTRDY,
  INTERFACE_OUTQ2_128_NOTRDY,
  INTERFACE_OUTQ3_128_NOTRDY,
  INTERFACE_OUTQ4_128_NOTRDY,
  INTERFACE_OUTQ5_128_NOTRDY,
  INTERFACE_SIGNALQ_NOTRDY
};

static xtensa_arg_internal Iclass_iclass_RDTIEP_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_interface Iclass_iclass_RDTIEP_intfArgs[] = {
  INTERFACE_IMPWIRE
};

static xtensa_arg_internal Iclass_iclass_SETTIEP_args[] = {
  { { OPERAND_dsp340050b49a6c_oper61_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SETTIEP_stateArgs[] = {
  { { STATE_EXPSTATE }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_SMOD_LUT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper78_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper57_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMOD_LUT_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_interface Iclass_iclass_SMOD_LUT_intfArgs[] = {
  INTERFACE_LU128_Out,
  INTERFACE_LU128_In
};

static xtensa_arg_internal Iclass_iclass_WRTBSIGQ_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_interface Iclass_iclass_WRTBSIGQ_intfArgs[] = {
  INTERFACE_SIGNALQ
};

static xtensa_arg_internal Iclass_iclass_WRTBSIGQM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper61_imm }, 'i' }
};

static xtensa_interface Iclass_iclass_WRTBSIGQM_intfArgs[] = {
  INTERFACE_SIGNALQ
};

static xtensa_arg_internal Iclass_iclass_WRTIEP_args[] = {
  { { OPERAND_art }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_WRTIEP_stateArgs[] = {
  { { STATE_EXPSTATE }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_WRTSIGQ_args[] = {
  { { OPERAND_ars }, 'i' }
};

static xtensa_interface Iclass_iclass_WRTSIGQ_intfArgs[] = {
  INTERFACE_SIGNALQ
};

static xtensa_arg_internal Iclass_iclass_ABS8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ABS8_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADD16_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper49_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADD16_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADD32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADD32_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDAC_I2R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_ADDAC_I2R_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDAC_R2I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_ADDAC_R2I_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDAR2_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDCM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDWRP_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ADDWRP_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AND128_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_AND128_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ARGMAX8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ARGMAX8_stateArgs[] = {
  { { STATE_MAX_REG }, 'm' },
  { { STATE_ARG_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASL_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASL_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASL32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASL32_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASLACM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper58_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASLACM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASLM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASLM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASLM32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper58_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASLM32_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASR_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASR_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASR32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASR32_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASRAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASRAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASRM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_ASRM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_BITFEXT_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper60_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper61_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_BITFINS_args[] = {
  { { OPERAND_art }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper61_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper60_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLB_C_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLB_C_stateArgs[] = {
  { { STATE_SAR3 }, 'o' },
  { { STATE_SAR2 }, 'o' },
  { { STATE_SAR1 }, 'o' },
  { { STATE_SAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLB_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CLB_R_stateArgs[] = {
  { { STATE_SAR3 }, 'o' },
  { { STATE_SAR2 }, 'o' },
  { { STATE_SAR1 }, 'o' },
  { { STATE_SAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMP8_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMP8_stateArgs[] = {
  { { STATE_SOV }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMP_I_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMP_I_stateArgs[] = {
  { { STATE_SOV }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMP_R_args[] = {
  { { OPERAND_arr }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper47_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_CMP_R_stateArgs[] = {
  { { STATE_SOV }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper68_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper69_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAR3 }, 'i' },
  { { STATE_SAR2 }, 'i' },
  { { STATE_SAR1 }, 'i' },
  { { STATE_SAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper68_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper69_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT_R_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAR3 }, 'i' },
  { { STATE_SAR2 }, 'i' },
  { { STATE_SAR1 }, 'i' },
  { { STATE_SAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT32_I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT32_I_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT32_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper48_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXT32_R_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXTUI4_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_EXTUI4_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LSLM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LSLM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LSRM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper59_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_LSRM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MAX8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MAX8_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MEAN_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MEAN_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MEAN32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MEAN32_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MIN8_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MIN8_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MINCLB_C_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MINCLB_C_stateArgs[] = {
  { { STATE_HSAR3 }, 'm' },
  { { STATE_HSAR2 }, 'm' },
  { { STATE_HSAR1 }, 'm' },
  { { STATE_HSAR0 }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MINCLB_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_MINCLB_R_stateArgs[] = {
  { { STATE_HSAR3 }, 'm' },
  { { STATE_HSAR2 }, 'm' },
  { { STATE_HSAR1 }, 'm' },
  { { STATE_HSAR0 }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NOT128_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_NOT128_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_OR128_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_OR128_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PERM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper82_reg }, 'm' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_ars }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_PERM_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_REDAC_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_REDAC_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_REDAC2_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_REDAC2_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_REDAC4_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_REDAC4_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_REDACS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_REDACS_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMINCLB_C_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMINCLB_C_stateArgs[] = {
  { { STATE_HSAR3 }, 'o' },
  { { STATE_HSAR2 }, 'o' },
  { { STATE_HSAR1 }, 'o' },
  { { STATE_HSAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMINCLB_R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SMINCLB_R_stateArgs[] = {
  { { STATE_HSAR3 }, 'o' },
  { { STATE_HSAR2 }, 'o' },
  { { STATE_HSAR1 }, 'o' },
  { { STATE_HSAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_STSWAPBM_args[] = {
  { { OPERAND_ars }, 'i' },
  { { OPERAND_art }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper89_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_STSWAPBMU_args[] = {
  { { OPERAND_art }, 'm' },
  { { OPERAND_ars }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper89_imm }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUB32_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper53_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUB32_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBAC_I2R_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_SUBAC_I2R_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBAC_R2I_args[] = {
  { { OPERAND_dsp340050b49a6c_oper51_reg }, 'm' }
};

static xtensa_arg_internal Iclass_iclass_SUBAC_R2I_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBARX_args[] = {
  { { OPERAND_ars }, 'm' },
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBCM_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBCM_stateArgs[] = {
  { { STATE_SOV }, 'm' },
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBMEAN_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBMEAN_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBWRP_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_SUBWRP_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_TRANS_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_TRANS_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_XOR128_args[] = {
  { { OPERAND_dsp340050b49a6c_oper45_reg }, 'o' },
  { { OPERAND_dsp340050b49a6c_oper55_reg }, 'i' },
  { { OPERAND_dsp340050b49a6c_oper46_reg }, 'i' }
};

static xtensa_arg_internal Iclass_iclass_XOR128_stateArgs[] = {
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_expstate_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_expstate_stateArgs[] = {
  { { STATE_EXPSTATE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_expstate_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_expstate_stateArgs[] = {
  { { STATE_EXPSTATE }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sov_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sov_stateArgs[] = {
  { { STATE_SOV }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sov_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sov_stateArgs[] = {
  { { STATE_SOV }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_sat_mode_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sat_mode_stateArgs[] = {
  { { STATE_SAT_MODE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sat_mode_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sat_mode_stateArgs[] = {
  { { STATE_SAT_MODE }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_sar0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sar0_stateArgs[] = {
  { { STATE_SAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar0_stateArgs[] = {
  { { STATE_SAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_sar1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sar1_stateArgs[] = {
  { { STATE_SAR1 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar1_stateArgs[] = {
  { { STATE_SAR1 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_sar2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sar2_stateArgs[] = {
  { { STATE_SAR2 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar2_stateArgs[] = {
  { { STATE_SAR2 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_sar3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_sar3_stateArgs[] = {
  { { STATE_SAR3 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_sar3_stateArgs[] = {
  { { STATE_SAR3 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_hsar0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_hsar0_stateArgs[] = {
  { { STATE_HSAR0 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar0_stateArgs[] = {
  { { STATE_HSAR0 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_hsar1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_hsar1_stateArgs[] = {
  { { STATE_HSAR1 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar1_stateArgs[] = {
  { { STATE_HSAR1 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_hsar2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_hsar2_stateArgs[] = {
  { { STATE_HSAR2 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar2_stateArgs[] = {
  { { STATE_HSAR2 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_hsar3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_hsar3_stateArgs[] = {
  { { STATE_HSAR3 }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_hsar3_stateArgs[] = {
  { { STATE_HSAR3 }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_max_reg_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_max_reg_0_stateArgs[] = {
  { { STATE_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_0_stateArgs[] = {
  { { STATE_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_max_reg_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_max_reg_1_stateArgs[] = {
  { { STATE_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_1_stateArgs[] = {
  { { STATE_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_max_reg_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_max_reg_2_stateArgs[] = {
  { { STATE_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_2_stateArgs[] = {
  { { STATE_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_max_reg_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_max_reg_3_stateArgs[] = {
  { { STATE_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_max_reg_3_stateArgs[] = {
  { { STATE_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_0_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_0_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_1_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_1_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_2_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_2_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_arg_max_reg_3_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_arg_max_reg_3_stateArgs[] = {
  { { STATE_ARG_MAX_REG }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_0_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_0_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_1_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_1_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_2_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_2_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_nco_counter_3_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_nco_counter_3_stateArgs[] = {
  { { STATE_NCO_COUNTER }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_interp_ext_n_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_interp_ext_n_stateArgs[] = {
  { { STATE_INTERP_EXT_N }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_interp_ext_n_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_interp_ext_n_stateArgs[] = {
  { { STATE_INTERP_EXT_N }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_interp_ext_l_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_interp_ext_l_stateArgs[] = {
  { { STATE_INTERP_EXT_L }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_interp_ext_l_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_interp_ext_l_stateArgs[] = {
  { { STATE_INTERP_EXT_L }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_0_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_0_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_1_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_1_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_2_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_2_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_3_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_3_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_4_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_4_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_4_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_4_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_5_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_5_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_5_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_5_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_6_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_6_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_6_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_6_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_7_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_7_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_7_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_7_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_8_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_8_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_8_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_8_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_9_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_9_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_9_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_9_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_10_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_10_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_10_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_10_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_11_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_11_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_11_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_11_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_12_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_12_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_12_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_12_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_13_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_13_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_13_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_13_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_14_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_14_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_14_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_14_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_15_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_15_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_15_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_15_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_16_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_16_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_16_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_16_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_17_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_17_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_17_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_17_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_18_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_18_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_18_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_18_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_19_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_19_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_19_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_19_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_20_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_20_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_20_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_20_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_21_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_21_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_21_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_21_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_22_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_22_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_22_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_22_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_23_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_buf_23_stateArgs[] = {
  { { STATE_LLR_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_23_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_buf_23_stateArgs[] = {
  { { STATE_LLR_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_0_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_0_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_1_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_1_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_2_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_2_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_3_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_3_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_4_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_4_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_4_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_4_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_5_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_5_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_5_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_5_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_6_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_6_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_6_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_6_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_7_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_buf_7_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_7_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_buf_7_stateArgs[] = {
  { { STATE_SMOD_BUF }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_weight_reg_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_weight_reg_stateArgs[] = {
  { { STATE_WEIGHT_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_weight_reg_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_weight_reg_stateArgs[] = {
  { { STATE_WEIGHT_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_scale_reg_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_scale_reg_stateArgs[] = {
  { { STATE_SCALE_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_scale_reg_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_scale_reg_stateArgs[] = {
  { { STATE_SCALE_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_llr_pos_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_llr_pos_stateArgs[] = {
  { { STATE_LLR_POS }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_pos_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_llr_pos_stateArgs[] = {
  { { STATE_LLR_POS }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_pos_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_pos_stateArgs[] = {
  { { STATE_SMOD_POS }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_pos_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_pos_stateArgs[] = {
  { { STATE_SMOD_POS }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_perm_reg_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_perm_reg_stateArgs[] = {
  { { STATE_PERM_REG }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_perm_reg_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_perm_reg_stateArgs[] = {
  { { STATE_PERM_REG }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_0_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_0_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_0_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_0_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_1_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_1_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_1_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_1_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_2_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_2_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_2_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_2_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_3_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_smod_offset_table_3_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_3_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_smod_offset_table_3_stateArgs[] = {
  { { STATE_SMOD_OFFSET_TABLE }, 'm' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_phasor_n_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_phasor_n_stateArgs[] = {
  { { STATE_PHASOR_N }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_phasor_n_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_phasor_n_stateArgs[] = {
  { { STATE_PHASOR_N }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_rur_phasor_offset_args[] = {
  { { OPERAND_arr }, 'o' }
};

static xtensa_arg_internal Iclass_rur_phasor_offset_stateArgs[] = {
  { { STATE_PHASOR_OFFSET }, 'i' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_arg_internal Iclass_wur_phasor_offset_args[] = {
  { { OPERAND_art }, 'i' }
};

static xtensa_arg_internal Iclass_wur_phasor_offset_stateArgs[] = {
  { { STATE_PHASOR_OFFSET }, 'o' },
  { { STATE_CPENABLE }, 'i' }
};

static xtensa_iclass_internal iclasses[] = {
  { 0, 0 /* xt_iclass_excw */,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_rfe */,
    2, Iclass_xt_iclass_rfe_stateArgs, 0, 0 },
  { 0, 0 /* xt_iclass_rfde */,
    1, Iclass_xt_iclass_rfde_stateArgs, 0, 0 },
  { 0, 0 /* xt_iclass_syscall */,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_simcall */,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_call12_args,
    1, Iclass_xt_iclass_call12_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_call8_args,
    1, Iclass_xt_iclass_call8_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_call4_args,
    1, Iclass_xt_iclass_call4_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_callx12_args,
    1, Iclass_xt_iclass_callx12_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_callx8_args,
    1, Iclass_xt_iclass_callx8_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_callx4_args,
    1, Iclass_xt_iclass_callx4_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_entry_args,
    5, Iclass_xt_iclass_entry_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_movsp_args,
    2, Iclass_xt_iclass_movsp_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rotw_args,
    1, Iclass_xt_iclass_rotw_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_retw_args,
    4, Iclass_xt_iclass_retw_stateArgs, 0, 0 },
  { 0, 0 /* xt_iclass_rfwou */,
    5, Iclass_xt_iclass_rfwou_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_l32e_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_s32e_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_windowbase_args,
    1, Iclass_xt_iclass_rsr_windowbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_windowbase_args,
    1, Iclass_xt_iclass_wsr_windowbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_windowbase_args,
    1, Iclass_xt_iclass_xsr_windowbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_windowstart_args,
    1, Iclass_xt_iclass_rsr_windowstart_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_windowstart_args,
    1, Iclass_xt_iclass_wsr_windowstart_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_windowstart_args,
    1, Iclass_xt_iclass_xsr_windowstart_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_add_n_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_addi_n_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_bz6_args,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_ill_n */,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_loadi4_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_mov_n_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_movi_n_args,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_nopn */,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_retn_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_storei4_args,
    0, 0, 0, 0 },
  { 1, Iclass_rur_threadptr_args,
    1, Iclass_rur_threadptr_stateArgs, 0, 0 },
  { 1, Iclass_wur_threadptr_args,
    1, Iclass_wur_threadptr_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_addi_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_addmi_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_addsub_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_bit_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_bsi8_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_bsi8b_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_bsi8u_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_bst8_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_bsz12_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_call0_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_callx0_args,
    0, 0, 0, 0 },
  { 4, Iclass_xt_iclass_exti_args,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_ill */,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_jump_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_jumpx_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_l16ui_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_l16si_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_l32i_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_l32r_args,
    2, Iclass_xt_iclass_l32r_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_l8i_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_loop_args,
    3, Iclass_xt_iclass_loop_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_loopz_args,
    3, Iclass_xt_iclass_loopz_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_movi_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_movz_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_neg_args,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_nop */,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_return_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_s16i_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_s32i_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_s8i_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_sar_args,
    1, Iclass_xt_iclass_sar_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_sari_args,
    1, Iclass_xt_iclass_sari_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_shifts_args,
    1, Iclass_xt_iclass_shifts_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_shiftst_args,
    1, Iclass_xt_iclass_shiftst_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_shiftt_args,
    1, Iclass_xt_iclass_shiftt_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_slli_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_srai_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_srli_args,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_memw */,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_extw */,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_isync */,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_sync */,
    1, Iclass_xt_iclass_sync_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_rsil_args,
    6, Iclass_xt_iclass_rsil_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_lend_args,
    1, Iclass_xt_iclass_rsr_lend_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_lend_args,
    1, Iclass_xt_iclass_wsr_lend_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_lend_args,
    1, Iclass_xt_iclass_xsr_lend_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_lcount_args,
    1, Iclass_xt_iclass_rsr_lcount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_lcount_args,
    2, Iclass_xt_iclass_wsr_lcount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_lcount_args,
    2, Iclass_xt_iclass_xsr_lcount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_lbeg_args,
    1, Iclass_xt_iclass_rsr_lbeg_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_lbeg_args,
    1, Iclass_xt_iclass_wsr_lbeg_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_lbeg_args,
    1, Iclass_xt_iclass_xsr_lbeg_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_sar_args,
    1, Iclass_xt_iclass_rsr_sar_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_sar_args,
    2, Iclass_xt_iclass_wsr_sar_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_sar_args,
    1, Iclass_xt_iclass_xsr_sar_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_litbase_args,
    2, Iclass_xt_iclass_rsr_litbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_litbase_args,
    2, Iclass_xt_iclass_wsr_litbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_litbase_args,
    2, Iclass_xt_iclass_xsr_litbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_176_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_176_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_208_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ps_args,
    6, Iclass_xt_iclass_rsr_ps_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ps_args,
    6, Iclass_xt_iclass_wsr_ps_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ps_args,
    6, Iclass_xt_iclass_xsr_ps_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_epc1_args,
    1, Iclass_xt_iclass_rsr_epc1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_epc1_args,
    1, Iclass_xt_iclass_wsr_epc1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_epc1_args,
    1, Iclass_xt_iclass_xsr_epc1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excsave1_args,
    1, Iclass_xt_iclass_rsr_excsave1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excsave1_args,
    1, Iclass_xt_iclass_wsr_excsave1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excsave1_args,
    1, Iclass_xt_iclass_xsr_excsave1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_epc2_args,
    1, Iclass_xt_iclass_rsr_epc2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_epc2_args,
    1, Iclass_xt_iclass_wsr_epc2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_epc2_args,
    1, Iclass_xt_iclass_xsr_epc2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excsave2_args,
    1, Iclass_xt_iclass_rsr_excsave2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excsave2_args,
    1, Iclass_xt_iclass_wsr_excsave2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excsave2_args,
    1, Iclass_xt_iclass_xsr_excsave2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_epc3_args,
    1, Iclass_xt_iclass_rsr_epc3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_epc3_args,
    1, Iclass_xt_iclass_wsr_epc3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_epc3_args,
    1, Iclass_xt_iclass_xsr_epc3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excsave3_args,
    1, Iclass_xt_iclass_rsr_excsave3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excsave3_args,
    1, Iclass_xt_iclass_wsr_excsave3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excsave3_args,
    1, Iclass_xt_iclass_xsr_excsave3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_epc4_args,
    1, Iclass_xt_iclass_rsr_epc4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_epc4_args,
    1, Iclass_xt_iclass_wsr_epc4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_epc4_args,
    1, Iclass_xt_iclass_xsr_epc4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excsave4_args,
    1, Iclass_xt_iclass_rsr_excsave4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excsave4_args,
    1, Iclass_xt_iclass_wsr_excsave4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excsave4_args,
    1, Iclass_xt_iclass_xsr_excsave4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_epc5_args,
    1, Iclass_xt_iclass_rsr_epc5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_epc5_args,
    1, Iclass_xt_iclass_wsr_epc5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_epc5_args,
    1, Iclass_xt_iclass_xsr_epc5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excsave5_args,
    1, Iclass_xt_iclass_rsr_excsave5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excsave5_args,
    1, Iclass_xt_iclass_wsr_excsave5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excsave5_args,
    1, Iclass_xt_iclass_xsr_excsave5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_epc6_args,
    1, Iclass_xt_iclass_rsr_epc6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_epc6_args,
    1, Iclass_xt_iclass_wsr_epc6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_epc6_args,
    1, Iclass_xt_iclass_xsr_epc6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excsave6_args,
    1, Iclass_xt_iclass_rsr_excsave6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excsave6_args,
    1, Iclass_xt_iclass_wsr_excsave6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excsave6_args,
    1, Iclass_xt_iclass_xsr_excsave6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_eps2_args,
    1, Iclass_xt_iclass_rsr_eps2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_eps2_args,
    1, Iclass_xt_iclass_wsr_eps2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_eps2_args,
    1, Iclass_xt_iclass_xsr_eps2_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_eps3_args,
    1, Iclass_xt_iclass_rsr_eps3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_eps3_args,
    1, Iclass_xt_iclass_wsr_eps3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_eps3_args,
    1, Iclass_xt_iclass_xsr_eps3_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_eps4_args,
    1, Iclass_xt_iclass_rsr_eps4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_eps4_args,
    1, Iclass_xt_iclass_wsr_eps4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_eps4_args,
    1, Iclass_xt_iclass_xsr_eps4_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_eps5_args,
    1, Iclass_xt_iclass_rsr_eps5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_eps5_args,
    1, Iclass_xt_iclass_wsr_eps5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_eps5_args,
    1, Iclass_xt_iclass_xsr_eps5_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_eps6_args,
    1, Iclass_xt_iclass_rsr_eps6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_eps6_args,
    1, Iclass_xt_iclass_wsr_eps6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_eps6_args,
    1, Iclass_xt_iclass_xsr_eps6_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_excvaddr_args,
    1, Iclass_xt_iclass_rsr_excvaddr_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_excvaddr_args,
    1, Iclass_xt_iclass_wsr_excvaddr_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_excvaddr_args,
    1, Iclass_xt_iclass_xsr_excvaddr_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_depc_args,
    1, Iclass_xt_iclass_rsr_depc_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_depc_args,
    1, Iclass_xt_iclass_wsr_depc_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_depc_args,
    1, Iclass_xt_iclass_xsr_depc_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_exccause_args,
    2, Iclass_xt_iclass_rsr_exccause_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_exccause_args,
    1, Iclass_xt_iclass_wsr_exccause_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_exccause_args,
    1, Iclass_xt_iclass_xsr_exccause_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_prid_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_vecbase_args,
    1, Iclass_xt_iclass_rsr_vecbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_vecbase_args,
    1, Iclass_xt_iclass_wsr_vecbase_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_vecbase_args,
    1, Iclass_xt_iclass_xsr_vecbase_stateArgs, 0, 0 },
  { 3, Iclass_xt_mul16_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rfi_args,
    18, Iclass_xt_iclass_rfi_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wait_args,
    1, Iclass_xt_iclass_wait_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_interrupt_args,
    1, Iclass_xt_iclass_rsr_interrupt_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_intset_args,
    2, Iclass_xt_iclass_wsr_intset_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_intclear_args,
    2, Iclass_xt_iclass_wsr_intclear_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_intenable_args,
    1, Iclass_xt_iclass_rsr_intenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_intenable_args,
    1, Iclass_xt_iclass_wsr_intenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_intenable_args,
    1, Iclass_xt_iclass_xsr_intenable_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_break_args,
    2, Iclass_xt_iclass_break_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_break_n_args,
    2, Iclass_xt_iclass_break_n_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_dbreaka0_args,
    1, Iclass_xt_iclass_rsr_dbreaka0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_dbreaka0_args,
    2, Iclass_xt_iclass_wsr_dbreaka0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_dbreaka0_args,
    2, Iclass_xt_iclass_xsr_dbreaka0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_dbreakc0_args,
    1, Iclass_xt_iclass_rsr_dbreakc0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_dbreakc0_args,
    2, Iclass_xt_iclass_wsr_dbreakc0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_dbreakc0_args,
    2, Iclass_xt_iclass_xsr_dbreakc0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_dbreaka1_args,
    1, Iclass_xt_iclass_rsr_dbreaka1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_dbreaka1_args,
    2, Iclass_xt_iclass_wsr_dbreaka1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_dbreaka1_args,
    2, Iclass_xt_iclass_xsr_dbreaka1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_dbreakc1_args,
    1, Iclass_xt_iclass_rsr_dbreakc1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_dbreakc1_args,
    2, Iclass_xt_iclass_wsr_dbreakc1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_dbreakc1_args,
    2, Iclass_xt_iclass_xsr_dbreakc1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ibreaka0_args,
    1, Iclass_xt_iclass_rsr_ibreaka0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ibreaka0_args,
    1, Iclass_xt_iclass_wsr_ibreaka0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ibreaka0_args,
    1, Iclass_xt_iclass_xsr_ibreaka0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ibreaka1_args,
    1, Iclass_xt_iclass_rsr_ibreaka1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ibreaka1_args,
    1, Iclass_xt_iclass_wsr_ibreaka1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ibreaka1_args,
    1, Iclass_xt_iclass_xsr_ibreaka1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ibreakenable_args,
    1, Iclass_xt_iclass_rsr_ibreakenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ibreakenable_args,
    1, Iclass_xt_iclass_wsr_ibreakenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ibreakenable_args,
    1, Iclass_xt_iclass_xsr_ibreakenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_debugcause_args,
    2, Iclass_xt_iclass_rsr_debugcause_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_debugcause_args,
    2, Iclass_xt_iclass_wsr_debugcause_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_debugcause_args,
    2, Iclass_xt_iclass_xsr_debugcause_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_icount_args,
    1, Iclass_xt_iclass_rsr_icount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_icount_args,
    2, Iclass_xt_iclass_wsr_icount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_icount_args,
    2, Iclass_xt_iclass_xsr_icount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_icountlevel_args,
    1, Iclass_xt_iclass_rsr_icountlevel_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_icountlevel_args,
    1, Iclass_xt_iclass_wsr_icountlevel_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_icountlevel_args,
    1, Iclass_xt_iclass_xsr_icountlevel_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ddr_args,
    1, Iclass_xt_iclass_rsr_ddr_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ddr_args,
    2, Iclass_xt_iclass_wsr_ddr_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ddr_args,
    2, Iclass_xt_iclass_xsr_ddr_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rfdo_args,
    9, Iclass_xt_iclass_rfdo_stateArgs, 0, 0 },
  { 0, 0 /* xt_iclass_rfdd */,
    1, Iclass_xt_iclass_rfdd_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_mmid_args,
    1, Iclass_xt_iclass_wsr_mmid_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_bbool1_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_bbool4_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_bbool8_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_bbranch_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_bmove_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_RSR_BR_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_WSR_BR_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_XSR_BR_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ccount_args,
    1, Iclass_xt_iclass_rsr_ccount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ccount_args,
    2, Iclass_xt_iclass_wsr_ccount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ccount_args,
    2, Iclass_xt_iclass_xsr_ccount_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ccompare0_args,
    1, Iclass_xt_iclass_rsr_ccompare0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ccompare0_args,
    2, Iclass_xt_iclass_wsr_ccompare0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ccompare0_args,
    2, Iclass_xt_iclass_xsr_ccompare0_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_ccompare1_args,
    1, Iclass_xt_iclass_rsr_ccompare1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_ccompare1_args,
    2, Iclass_xt_iclass_wsr_ccompare1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_ccompare1_args,
    2, Iclass_xt_iclass_xsr_ccompare1_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_icache_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_icache_lock_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_icache_inv_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_licx_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_sicx_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_dcache_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_dcache_ind_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_dcache_inv_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_dpf_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_dcache_lock_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_sdct_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_ldct_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_idtlb_args,
    1, Iclass_xt_iclass_idtlb_stateArgs, 0, 0 },
  { 2, Iclass_xt_iclass_rdtlb_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_wdtlb_args,
    1, Iclass_xt_iclass_wdtlb_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_iitlb_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_ritlb_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_witlb_args,
    0, 0, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_cpenable_args,
    1, Iclass_xt_iclass_rsr_cpenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_cpenable_args,
    1, Iclass_xt_iclass_wsr_cpenable_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_cpenable_args,
    1, Iclass_xt_iclass_xsr_cpenable_stateArgs, 0, 0 },
  { 3, Iclass_xt_iclass_clamp_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_minmax_args,
    0, 0, 0, 0 },
  { 2, Iclass_xt_iclass_nsa_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_sx_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_l32ai_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_s32ri_args,
    0, 0, 0, 0 },
  { 3, Iclass_xt_iclass_s32c1i_args,
    3, Iclass_xt_iclass_s32c1i_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_scompare1_args,
    1, Iclass_xt_iclass_rsr_scompare1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_scompare1_args,
    1, Iclass_xt_iclass_wsr_scompare1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_scompare1_args,
    1, Iclass_xt_iclass_xsr_scompare1_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_rsr_atomctl_args,
    1, Iclass_xt_iclass_rsr_atomctl_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_wsr_atomctl_args,
    2, Iclass_xt_iclass_wsr_atomctl_stateArgs, 0, 0 },
  { 1, Iclass_xt_iclass_xsr_atomctl_args,
    2, Iclass_xt_iclass_xsr_atomctl_stateArgs, 0, 0 },
  { 0, 0 /* xt_iclass_rer */,
    0, 0, 0, 0 },
  { 0, 0 /* xt_iclass_wer */,
    0, 0, 0, 0 },
  { 1, Iclass_rur_fcr_args,
    9, Iclass_rur_fcr_stateArgs, 0, 0 },
  { 1, Iclass_wur_fcr_args,
    9, Iclass_wur_fcr_stateArgs, 0, 0 },
  { 1, Iclass_rur_fsr_args,
    8, Iclass_rur_fsr_stateArgs, 0, 0 },
  { 1, Iclass_wur_fsr_args,
    8, Iclass_wur_fsr_stateArgs, 0, 0 },
  { 3, Iclass_fp_args,
    2, Iclass_fp_stateArgs, 0, 0 },
  { 3, Iclass_fp_mac_args,
    2, Iclass_fp_mac_stateArgs, 0, 0 },
  { 3, Iclass_fp_cmov_args,
    1, Iclass_fp_cmov_stateArgs, 0, 0 },
  { 3, Iclass_fp_mov_args,
    1, Iclass_fp_mov_stateArgs, 0, 0 },
  { 2, Iclass_fp_mov2_args,
    1, Iclass_fp_mov2_stateArgs, 0, 0 },
  { 3, Iclass_fp_cmp_args,
    1, Iclass_fp_cmp_stateArgs, 0, 0 },
  { 3, Iclass_fp_float_args,
    2, Iclass_fp_float_stateArgs, 0, 0 },
  { 3, Iclass_fp_int_args,
    1, Iclass_fp_int_stateArgs, 0, 0 },
  { 2, Iclass_fp_rfr_args,
    1, Iclass_fp_rfr_stateArgs, 0, 0 },
  { 2, Iclass_fp_wfr_args,
    1, Iclass_fp_wfr_stateArgs, 0, 0 },
  { 3, Iclass_fp_lsi_args,
    1, Iclass_fp_lsi_stateArgs, 0, 0 },
  { 3, Iclass_fp_lsiu_args,
    1, Iclass_fp_lsiu_stateArgs, 0, 0 },
  { 3, Iclass_fp_lsx_args,
    1, Iclass_fp_lsx_stateArgs, 0, 0 },
  { 3, Iclass_fp_lsxu_args,
    1, Iclass_fp_lsxu_stateArgs, 0, 0 },
  { 3, Iclass_fp_ssi_args,
    1, Iclass_fp_ssi_stateArgs, 0, 0 },
  { 3, Iclass_fp_ssiu_args,
    1, Iclass_fp_ssiu_stateArgs, 0, 0 },
  { 3, Iclass_fp_ssx_args,
    1, Iclass_fp_ssx_stateArgs, 0, 0 },
  { 3, Iclass_fp_ssxu_args,
    1, Iclass_fp_ssxu_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_ARGMAX_args,
    2, Iclass_iclass_GET_ARGMAX_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_HSAR_args,
    5, Iclass_iclass_GET_HSAR_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_HSAR2SAR_args,
    9, Iclass_iclass_GET_HSAR2SAR_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_INTERP_EXT_N_args,
    2, Iclass_iclass_GET_INTERP_EXT_N_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_INTERP_EXT_L_args,
    2, Iclass_iclass_GET_INTERP_EXT_L_stateArgs, 0, 0 },
  { 2, Iclass_iclass_GET_LLR_BUF_args,
    2, Iclass_iclass_GET_LLR_BUF_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_LLR_POS_args,
    2, Iclass_iclass_GET_LLR_POS_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_MAX_args,
    2, Iclass_iclass_GET_MAX_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_NCO_args,
    2, Iclass_iclass_GET_NCO_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_PERM_REG_args,
    2, Iclass_iclass_GET_PERM_REG_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_PHASOR_N_args,
    2, Iclass_iclass_GET_PHASOR_N_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_PHASOR_OFFSET_args,
    2, Iclass_iclass_GET_PHASOR_OFFSET_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_SAR_args,
    5, Iclass_iclass_GET_SAR_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_SCALE_REG_args,
    2, Iclass_iclass_GET_SCALE_REG_stateArgs, 0, 0 },
  { 2, Iclass_iclass_GET_SMOD_BUF_args,
    2, Iclass_iclass_GET_SMOD_BUF_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_SMOD_OFFSET_TABLE_args,
    2, Iclass_iclass_GET_SMOD_OFFSET_TABLE_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_SMOD_POS_args,
    2, Iclass_iclass_GET_SMOD_POS_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_SOV_args,
    2, Iclass_iclass_GET_SOV_stateArgs, 0, 0 },
  { 1, Iclass_iclass_GET_WGHT_args,
    2, Iclass_iclass_GET_WGHT_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_ARGMAX_args,
    2, Iclass_iclass_SET_ARGMAX_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SET_EXT_REGS_args,
    3, Iclass_iclass_SET_EXT_REGS_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_HSAR_args,
    5, Iclass_iclass_SET_HSAR_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SET_LLR_BUF_args,
    2, Iclass_iclass_SET_LLR_BUF_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_LLR_POS_args,
    2, Iclass_iclass_SET_LLR_POS_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_MAX_args,
    2, Iclass_iclass_SET_MAX_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_NCO_args,
    2, Iclass_iclass_SET_NCO_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_PERM_REG_args,
    2, Iclass_iclass_SET_PERM_REG_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_PHASOR_N_args,
    2, Iclass_iclass_SET_PHASOR_N_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_PHASOR_OFFSET_args,
    2, Iclass_iclass_SET_PHASOR_OFFSET_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_SAR_args,
    5, Iclass_iclass_SET_SAR_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_SCALE_REG_args,
    2, Iclass_iclass_SET_SCALE_REG_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SET_SMOD_BUF_args,
    2, Iclass_iclass_SET_SMOD_BUF_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_SMOD_OFFSET_TABLE_args,
    2, Iclass_iclass_SET_SMOD_OFFSET_TABLE_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_SMOD_POS_args,
    2, Iclass_iclass_SET_SMOD_POS_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_SOV_args,
    2, Iclass_iclass_SET_SOV_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SET_WGHT_args,
    2, Iclass_iclass_SET_WGHT_stateArgs, 0, 0 },
  { 4, Iclass_iclass_LAC2X32_args,
    1, Iclass_iclass_LAC2X32_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC2X64_0_args,
    1, Iclass_iclass_LAC2X64_0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC2X64_1_args,
    1, Iclass_iclass_LAC2X64_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC2X64_2_args,
    1, Iclass_iclass_LAC2X64_2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC2X64_3_args,
    1, Iclass_iclass_LAC2X64_3_stateArgs, 0, 0 },
  { 4, Iclass_iclass_LAC32_R_args,
    1, Iclass_iclass_LAC32_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC_IH_args,
    1, Iclass_iclass_LAC_IH_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC_IL_args,
    1, Iclass_iclass_LAC_IL_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC_RH_args,
    1, Iclass_iclass_LAC_RH_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LAC_RL_args,
    1, Iclass_iclass_LAC_RL_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LCM_args,
    1, Iclass_iclass_LCM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LCM_PINC_args,
    1, Iclass_iclass_LCM_PINC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LCM_PINC_X_args,
    1, Iclass_iclass_LCM_PINC_X_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LCM_U_args,
    1, Iclass_iclass_LCM_U_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LCM_X_args,
    1, Iclass_iclass_LCM_X_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LCM_XU_args,
    1, Iclass_iclass_LCM_XU_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LP_args,
    1, Iclass_iclass_LP_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LP_X_args,
    1, Iclass_iclass_LP_X_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LQ_args,
    1, Iclass_iclass_LQ_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LQ_X_args,
    1, Iclass_iclass_LQ_X_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LUT0_args,
    1, Iclass_iclass_LUT0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LUT1_args,
    1, Iclass_iclass_LUT1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LUT2_args,
    1, Iclass_iclass_LUT2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LUT3_args,
    1, Iclass_iclass_LUT3_stateArgs, 0, 0 },
  { 4, Iclass_iclass_SAC2X32_args,
    1, Iclass_iclass_SAC2X32_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC2X64_0_args,
    1, Iclass_iclass_SAC2X64_0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC2X64_1_args,
    1, Iclass_iclass_SAC2X64_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC2X64_2_args,
    1, Iclass_iclass_SAC2X64_2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC2X64_3_args,
    1, Iclass_iclass_SAC2X64_3_stateArgs, 0, 0 },
  { 4, Iclass_iclass_SAC32_R_args,
    1, Iclass_iclass_SAC32_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC_IH_args,
    1, Iclass_iclass_SAC_IH_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC_IL_args,
    1, Iclass_iclass_SAC_IL_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC_RH_args,
    1, Iclass_iclass_SAC_RH_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SAC_RL_args,
    1, Iclass_iclass_SAC_RL_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SCM_args,
    1, Iclass_iclass_SCM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SCM_PINC_args,
    1, Iclass_iclass_SCM_PINC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SCM_PINC_X_args,
    1, Iclass_iclass_SCM_PINC_X_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SCM_U_args,
    1, Iclass_iclass_SCM_U_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SCM_X_args,
    1, Iclass_iclass_SCM_X_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SCM_XU_args,
    1, Iclass_iclass_SCM_XU_stateArgs, 0, 0 },
  { 3, Iclass_iclass_STORE_P_args,
    1, Iclass_iclass_STORE_P_stateArgs, 0, 0 },
  { 3, Iclass_iclass_STORE_Q_args,
    1, Iclass_iclass_STORE_Q_stateArgs, 0, 0 },
  { 2, Iclass_iclass_AR2CM_DUP_args,
    1, Iclass_iclass_AR2CM_DUP_stateArgs, 0, 0 },
  { 3, Iclass_iclass_AR2CM_LN_args,
    1, Iclass_iclass_AR2CM_LN_stateArgs, 0, 0 },
  { 3, Iclass_iclass_AR2CM_LN_I_args,
    1, Iclass_iclass_AR2CM_LN_I_stateArgs, 0, 0 },
  { 3, Iclass_iclass_AR2CM_LN_R_args,
    1, Iclass_iclass_AR2CM_LN_R_stateArgs, 0, 0 },
  { 4, Iclass_iclass_AR2PQ_LN_args,
    1, Iclass_iclass_AR2PQ_LN_stateArgs, 0, 0 },
  { 1, Iclass_iclass_AR2SAR_DUP_args,
    5, Iclass_iclass_AR2SAR_DUP_stateArgs, 0, 0 },
  { 1, Iclass_iclass_CLRAC_args,
    1, Iclass_iclass_CLRAC_stateArgs, 0, 0 },
  { 1, Iclass_iclass_CLRCM_args,
    1, Iclass_iclass_CLRCM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CM2AR_LN_args,
    1, Iclass_iclass_CM2AR_LN_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CM2AR_LN_I_args,
    1, Iclass_iclass_CM2AR_LN_I_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CM2AR_LN_R_args,
    1, Iclass_iclass_CM2AR_LN_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_COMB_AR_args,
    0, 0, 0, 0 },
  { 2, Iclass_iclass_CONJ_args,
    1, Iclass_iclass_CONJ_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOV2AC32_I_args,
    1, Iclass_iclass_MOV2AC32_I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOV2AC32_R_args,
    1, Iclass_iclass_MOV2AC32_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOV2CM2PQ_args,
    1, Iclass_iclass_MOV2CM2PQ_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVAC_args,
    1, Iclass_iclass_MOVAC_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVAC_I_args,
    1, Iclass_iclass_MOVAC_I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVAC_I2R_args,
    1, Iclass_iclass_MOVAC_I2R_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVAC_R_args,
    1, Iclass_iclass_MOVAC_R_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVAC_R2I_args,
    1, Iclass_iclass_MOVAC_R2I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVAR2_args,
    0, 0, 0, 0 },
  { 2, Iclass_iclass_MOVCM_args,
    1, Iclass_iclass_MOVCM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCM2PQ_args,
    1, Iclass_iclass_MOVCM2PQ_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_0_args,
    1, Iclass_iclass_MOVCND_0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_1_args,
    1, Iclass_iclass_MOVCND_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_2_args,
    1, Iclass_iclass_MOVCND_2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_3_args,
    1, Iclass_iclass_MOVCND_3_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_4_args,
    1, Iclass_iclass_MOVCND_4_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_5_args,
    1, Iclass_iclass_MOVCND_5_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_6_args,
    1, Iclass_iclass_MOVCND_6_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND_7_args,
    1, Iclass_iclass_MOVCND_7_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_0_args,
    1, Iclass_iclass_MOVCND8_0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_1_args,
    1, Iclass_iclass_MOVCND8_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_2_args,
    1, Iclass_iclass_MOVCND8_2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_3_args,
    1, Iclass_iclass_MOVCND8_3_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_4_args,
    1, Iclass_iclass_MOVCND8_4_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_5_args,
    1, Iclass_iclass_MOVCND8_5_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_6_args,
    1, Iclass_iclass_MOVCND8_6_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOVCND8_7_args,
    1, Iclass_iclass_MOVCND8_7_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOV_I_args,
    1, Iclass_iclass_MOV_I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOVPQ2PQ_args,
    1, Iclass_iclass_MOVPQ2PQ_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MOV_R_args,
    1, Iclass_iclass_MOV_R_stateArgs, 0, 0 },
  { 2, Iclass_iclass_NEGCM_args,
    1, Iclass_iclass_NEGCM_stateArgs, 0, 0 },
  { 1, Iclass_iclass_POP16LLR_1_args,
    3, Iclass_iclass_POP16LLR_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_PQ2CM_args,
    1, Iclass_iclass_PQ2CM_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SWAPAC_R_args,
    1, Iclass_iclass_SWAPAC_R_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SWAPAC_RI_args,
    1, Iclass_iclass_SWAPAC_RI_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SWAPB_args,
    0, 0, 0, 0 },
  { 3, Iclass_iclass_ADD2AC_args,
    6, Iclass_iclass_ADD2AC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ADDAC_args,
    3, Iclass_iclass_ADDAC_stateArgs, 0, 0 },
  { 4, Iclass_iclass_CDOT_args,
    2, Iclass_iclass_CDOT_stateArgs, 0, 0 },
  { 4, Iclass_iclass_CDOTAC_args,
    3, Iclass_iclass_CDOTAC_stateArgs, 0, 0 },
  { 4, Iclass_iclass_CDOTACS_args,
    3, Iclass_iclass_CDOTACS_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMAC_args,
    3, Iclass_iclass_CMAC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMACS_args,
    3, Iclass_iclass_CMACS_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMPY_args,
    2, Iclass_iclass_CMPY_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMPY2CM_args,
    3, Iclass_iclass_CMPY2CM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMPY2PQ_args,
    3, Iclass_iclass_CMPY2PQ_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMPYS_args,
    2, Iclass_iclass_CMPYS_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMPYXP2PQ_args,
    3, Iclass_iclass_CMPYXP2PQ_stateArgs, 0, 0 },
  { 5, Iclass_iclass_COMB32_args,
    3, Iclass_iclass_COMB32_stateArgs, 0, 0 },
  { 4, Iclass_iclass_DOT_args,
    2, Iclass_iclass_DOT_stateArgs, 0, 0 },
  { 4, Iclass_iclass_DOTAC_args,
    3, Iclass_iclass_DOTAC_stateArgs, 0, 0 },
  { 4, Iclass_iclass_DOTACS_args,
    3, Iclass_iclass_DOTACS_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LIN_INT_args,
    6, Iclass_iclass_LIN_INT_stateArgs, 0, 0 },
  { 6, Iclass_iclass_LLRPRE1_args,
    2, Iclass_iclass_LLRPRE1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LLRPRE2_args,
    3, Iclass_iclass_LLRPRE2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MAC_args,
    3, Iclass_iclass_MAC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MAC8_args,
    3, Iclass_iclass_MAC8_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACD8_args,
    3, Iclass_iclass_MACD8_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACPQXP_0_args,
    3, Iclass_iclass_MACPQXP_0_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACPQXP_1_args,
    3, Iclass_iclass_MACPQXP_1_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACPQXP_2_args,
    3, Iclass_iclass_MACPQXP_2_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACPQXP_3_args,
    3, Iclass_iclass_MACPQXP_3_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MACS_args,
    3, Iclass_iclass_MACS_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACXP2_0_args,
    3, Iclass_iclass_MACXP2_0_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MACXP2_1_args,
    3, Iclass_iclass_MACXP2_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MACXP_0_args,
    3, Iclass_iclass_MACXP_0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MACXP_1_args,
    3, Iclass_iclass_MACXP_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MACXP_2_args,
    3, Iclass_iclass_MACXP_2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MACXP_3_args,
    3, Iclass_iclass_MACXP_3_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MOV2AC_args,
    6, Iclass_iclass_MOV2AC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPY_args,
    2, Iclass_iclass_MPY_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPY2CM_args,
    3, Iclass_iclass_MPY2CM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPY2PQ_args,
    3, Iclass_iclass_MPY2PQ_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPY8_args,
    2, Iclass_iclass_MPY8_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYADD8_2CM_args,
    3, Iclass_iclass_MPYADD8_2CM_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYD8_args,
    2, Iclass_iclass_MPYD8_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYPQXP_0_args,
    2, Iclass_iclass_MPYPQXP_0_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYPQXP_1_args,
    2, Iclass_iclass_MPYPQXP_1_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYPQXP_2_args,
    2, Iclass_iclass_MPYPQXP_2_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYPQXP_3_args,
    2, Iclass_iclass_MPYPQXP_3_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYS_args,
    2, Iclass_iclass_MPYS_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYXP2PQ_args,
    3, Iclass_iclass_MPYXP2PQ_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYXP2_0_args,
    2, Iclass_iclass_MPYXP2_0_stateArgs, 0, 0 },
  { 4, Iclass_iclass_MPYXP2_1_args,
    2, Iclass_iclass_MPYXP2_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYXP_0_args,
    2, Iclass_iclass_MPYXP_0_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYXP_1_args,
    2, Iclass_iclass_MPYXP_1_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYXP_2_args,
    2, Iclass_iclass_MPYXP_2_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MPYXP_3_args,
    2, Iclass_iclass_MPYXP_3_stateArgs, 0, 0 },
  { 3, Iclass_iclass_NORMACD_args,
    3, Iclass_iclass_NORMACD_stateArgs, 0, 0 },
  { 2, Iclass_iclass_NORMACPQ_I_args,
    3, Iclass_iclass_NORMACPQ_I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_NORMACPQ_R_args,
    3, Iclass_iclass_NORMACPQ_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_NORMD_args,
    2, Iclass_iclass_NORMD_stateArgs, 0, 0 },
  { 2, Iclass_iclass_NORMPYPQ_I_args,
    2, Iclass_iclass_NORMPYPQ_I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_NORMPYPQ_R_args,
    2, Iclass_iclass_NORMPYPQ_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_RCMAC_args,
    3, Iclass_iclass_RCMAC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_RCMPY_args,
    2, Iclass_iclass_RCMPY_stateArgs, 0, 0 },
  { 3, Iclass_iclass_RCMPY2CM_args,
    1, Iclass_iclass_RCMPY2CM_stateArgs, 0, 0 },
  { 6, Iclass_iclass_RFIR_args,
    3, Iclass_iclass_RFIR_stateArgs, 0, 0 },
  { 6, Iclass_iclass_RFIRA_args,
    3, Iclass_iclass_RFIRA_stateArgs, 0, 0 },
  { 5, Iclass_iclass_RFIRD_args,
    3, Iclass_iclass_RFIRD_stateArgs, 0, 0 },
  { 5, Iclass_iclass_RFIRDA_args,
    3, Iclass_iclass_RFIRDA_stateArgs, 0, 0 },
  { 3, Iclass_iclass_RMAC_args,
    3, Iclass_iclass_RMAC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_RMPY_args,
    2, Iclass_iclass_RMPY_stateArgs, 0, 0 },
  { 3, Iclass_iclass_RMPY2CM_args,
    1, Iclass_iclass_RMPY2CM_stateArgs, 0, 0 },
  { 4, Iclass_iclass_SMOD_ALIGN_args,
    3, Iclass_iclass_SMOD_ALIGN_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SMOD_SCR_args,
    2, Iclass_iclass_SMOD_SCR_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SUB2AC_args,
    6, Iclass_iclass_SUB2AC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_WGHT32_args,
    3, Iclass_iclass_WGHT32_stateArgs, 0, 0 },
  { 1, Iclass_iclass_CLRTIEP_args,
    1, Iclass_iclass_CLRTIEP_stateArgs, 0, 0 },
  { 3, Iclass_iclass_EXT_2FIFO_0_args,
    1, Iclass_iclass_EXT_2FIFO_0_stateArgs, 1, Iclass_iclass_EXT_2FIFO_0_intfArgs },
  { 3, Iclass_iclass_EXT_2FIFO_1_args,
    1, Iclass_iclass_EXT_2FIFO_1_stateArgs, 1, Iclass_iclass_EXT_2FIFO_1_intfArgs },
  { 3, Iclass_iclass_EXT_2FIFO_2_args,
    1, Iclass_iclass_EXT_2FIFO_2_stateArgs, 1, Iclass_iclass_EXT_2FIFO_2_intfArgs },
  { 3, Iclass_iclass_EXT_2FIFO_3_args,
    1, Iclass_iclass_EXT_2FIFO_3_stateArgs, 1, Iclass_iclass_EXT_2FIFO_3_intfArgs },
  { 3, Iclass_iclass_EXT_R2FIFO_0_args,
    1, Iclass_iclass_EXT_R2FIFO_0_stateArgs, 1, Iclass_iclass_EXT_R2FIFO_0_intfArgs },
  { 3, Iclass_iclass_EXT_R2FIFO_1_args,
    1, Iclass_iclass_EXT_R2FIFO_1_stateArgs, 1, Iclass_iclass_EXT_R2FIFO_1_intfArgs },
  { 3, Iclass_iclass_EXT_R2FIFO_2_args,
    1, Iclass_iclass_EXT_R2FIFO_2_stateArgs, 1, Iclass_iclass_EXT_R2FIFO_2_intfArgs },
  { 3, Iclass_iclass_EXT_R2FIFO_3_args,
    1, Iclass_iclass_EXT_R2FIFO_3_stateArgs, 1, Iclass_iclass_EXT_R2FIFO_3_intfArgs },
  { 3, Iclass_iclass_LUT_args,
    1, Iclass_iclass_LUT_stateArgs, 2, Iclass_iclass_LUT_intfArgs },
  { 3, Iclass_iclass_LUT_AR_args,
    1, Iclass_iclass_LUT_AR_stateArgs, 2, Iclass_iclass_LUT_AR_intfArgs },
  { 3, Iclass_iclass_LUT_IEXT_args,
    3, Iclass_iclass_LUT_IEXT_stateArgs, 2, Iclass_iclass_LUT_IEXT_intfArgs },
  { 2, Iclass_iclass_LUT_PHASOR_args,
    3, Iclass_iclass_LUT_PHASOR_stateArgs, 2, Iclass_iclass_LUT_PHASOR_intfArgs },
  { 3, Iclass_iclass_LUT_REXT_args,
    3, Iclass_iclass_LUT_REXT_stateArgs, 2, Iclass_iclass_LUT_REXT_intfArgs },
  { 3, Iclass_iclass_LUT_WRITE_args,
    1, Iclass_iclass_LUT_WRITE_stateArgs, 2, Iclass_iclass_LUT_WRITE_intfArgs },
  { 0, 0 /* iclass_MOVEQ128_0 */,
    0, 0, 2, Iclass_iclass_MOVEQ128_0_intfArgs },
  { 0, 0 /* iclass_MOVEQ128_1 */,
    0, 0, 2, Iclass_iclass_MOVEQ128_1_intfArgs },
  { 0, 0 /* iclass_MOVEQ128_2 */,
    0, 0, 2, Iclass_iclass_MOVEQ128_2_intfArgs },
  { 0, 0 /* iclass_MOVEQ128_3 */,
    0, 0, 2, Iclass_iclass_MOVEQ128_3_intfArgs },
  { 0, 0 /* iclass_MOVEQ128_4 */,
    0, 0, 2, Iclass_iclass_MOVEQ128_4_intfArgs },
  { 0, 0 /* iclass_MOVEQ128_5 */,
    0, 0, 2, Iclass_iclass_MOVEQ128_5_intfArgs },
  { 0, 0 /* iclass_MOVEQ32_0 */,
    0, 0, 2, Iclass_iclass_MOVEQ32_0_intfArgs },
  { 0, 0 /* iclass_MOVEQ32_1 */,
    0, 0, 2, Iclass_iclass_MOVEQ32_1_intfArgs },
  { 0, 0 /* iclass_MOVEQ32_2 */,
    0, 0, 2, Iclass_iclass_MOVEQ32_2_intfArgs },
  { 0, 0 /* iclass_MOVEQ32_3 */,
    0, 0, 2, Iclass_iclass_MOVEQ32_3_intfArgs },
  { 2, Iclass_iclass_NCO_UPDATE_args,
    4, Iclass_iclass_NCO_UPDATE_stateArgs, 2, Iclass_iclass_NCO_UPDATE_intfArgs },
  { 1, Iclass_iclass_POP128_0_args,
    1, Iclass_iclass_POP128_0_stateArgs, 1, Iclass_iclass_POP128_0_intfArgs },
  { 1, Iclass_iclass_POP128_1_args,
    1, Iclass_iclass_POP128_1_stateArgs, 1, Iclass_iclass_POP128_1_intfArgs },
  { 1, Iclass_iclass_POP128_2_args,
    1, Iclass_iclass_POP128_2_stateArgs, 1, Iclass_iclass_POP128_2_intfArgs },
  { 1, Iclass_iclass_POP128_3_args,
    1, Iclass_iclass_POP128_3_stateArgs, 1, Iclass_iclass_POP128_3_intfArgs },
  { 1, Iclass_iclass_POP128_4_args,
    1, Iclass_iclass_POP128_4_stateArgs, 1, Iclass_iclass_POP128_4_intfArgs },
  { 1, Iclass_iclass_POP128_5_args,
    1, Iclass_iclass_POP128_5_stateArgs, 1, Iclass_iclass_POP128_5_intfArgs },
  { 3, Iclass_iclass_POP128_2CMPQ_0_args,
    1, Iclass_iclass_POP128_2CMPQ_0_stateArgs, 1, Iclass_iclass_POP128_2CMPQ_0_intfArgs },
  { 3, Iclass_iclass_POP128_2CMPQ_1_args,
    1, Iclass_iclass_POP128_2CMPQ_1_stateArgs, 1, Iclass_iclass_POP128_2CMPQ_1_intfArgs },
  { 3, Iclass_iclass_POP128_2CMPQ_2_args,
    1, Iclass_iclass_POP128_2CMPQ_2_stateArgs, 1, Iclass_iclass_POP128_2CMPQ_2_intfArgs },
  { 3, Iclass_iclass_POP128_2CMPQ_3_args,
    1, Iclass_iclass_POP128_2CMPQ_3_stateArgs, 1, Iclass_iclass_POP128_2CMPQ_3_intfArgs },
  { 2, Iclass_iclass_POP128_2M_0_args,
    0, 0, 1, Iclass_iclass_POP128_2M_0_intfArgs },
  { 2, Iclass_iclass_POP128_2M_1_args,
    0, 0, 1, Iclass_iclass_POP128_2M_1_intfArgs },
  { 2, Iclass_iclass_POP128_2M_2_args,
    0, 0, 1, Iclass_iclass_POP128_2M_2_intfArgs },
  { 2, Iclass_iclass_POP128_2M_3_args,
    0, 0, 1, Iclass_iclass_POP128_2M_3_intfArgs },
  { 2, Iclass_iclass_POP128_2PQ_0_args,
    1, Iclass_iclass_POP128_2PQ_0_stateArgs, 1, Iclass_iclass_POP128_2PQ_0_intfArgs },
  { 2, Iclass_iclass_POP128_2PQ_1_args,
    1, Iclass_iclass_POP128_2PQ_1_stateArgs, 1, Iclass_iclass_POP128_2PQ_1_intfArgs },
  { 2, Iclass_iclass_POP128_2PQ_2_args,
    1, Iclass_iclass_POP128_2PQ_2_stateArgs, 1, Iclass_iclass_POP128_2PQ_2_intfArgs },
  { 2, Iclass_iclass_POP128_2PQ_3_args,
    1, Iclass_iclass_POP128_2PQ_3_stateArgs, 1, Iclass_iclass_POP128_2PQ_3_intfArgs },
  { 2, Iclass_iclass_POP128_2PQ_4_args,
    1, Iclass_iclass_POP128_2PQ_4_stateArgs, 1, Iclass_iclass_POP128_2PQ_4_intfArgs },
  { 2, Iclass_iclass_POP128_2PQ_5_args,
    1, Iclass_iclass_POP128_2PQ_5_stateArgs, 1, Iclass_iclass_POP128_2PQ_5_intfArgs },
  { 1, Iclass_iclass_POP2X128_2PQ_01_args,
    1, Iclass_iclass_POP2X128_2PQ_01_stateArgs, 2, Iclass_iclass_POP2X128_2PQ_01_intfArgs },
  { 1, Iclass_iclass_POP2X128_2PQ_03_args,
    1, Iclass_iclass_POP2X128_2PQ_03_stateArgs, 2, Iclass_iclass_POP2X128_2PQ_03_intfArgs },
  { 1, Iclass_iclass_POP2X128_2PQ_21_args,
    1, Iclass_iclass_POP2X128_2PQ_21_stateArgs, 2, Iclass_iclass_POP2X128_2PQ_21_intfArgs },
  { 1, Iclass_iclass_POP2X128_2PQ_23_args,
    1, Iclass_iclass_POP2X128_2PQ_23_stateArgs, 2, Iclass_iclass_POP2X128_2PQ_23_intfArgs },
  { 1, Iclass_iclass_POP32_0_args,
    0, 0, 1, Iclass_iclass_POP32_0_intfArgs },
  { 1, Iclass_iclass_POP32_1_args,
    0, 0, 1, Iclass_iclass_POP32_1_intfArgs },
  { 1, Iclass_iclass_POP32_2_args,
    0, 0, 1, Iclass_iclass_POP32_2_intfArgs },
  { 1, Iclass_iclass_POP32_3_args,
    0, 0, 1, Iclass_iclass_POP32_3_intfArgs },
  { 2, Iclass_iclass_PUSH128_args,
    1, Iclass_iclass_PUSH128_stateArgs, 12, Iclass_iclass_PUSH128_intfArgs },
  { 3, Iclass_iclass_PUSH128_M_args,
    0, 0, 8, Iclass_iclass_PUSH128_M_intfArgs },
  { 3, Iclass_iclass_PUSH128_PQ_args,
    1, Iclass_iclass_PUSH128_PQ_stateArgs, 12, Iclass_iclass_PUSH128_PQ_intfArgs },
  { 3, Iclass_iclass_PUSH2X128_PQ_args,
    1, Iclass_iclass_PUSH2X128_PQ_stateArgs, 8, Iclass_iclass_PUSH2X128_PQ_intfArgs },
  { 2, Iclass_iclass_PUSH32_args,
    0, 0, 12, Iclass_iclass_PUSH32_intfArgs },
  { 2, Iclass_iclass_QREADY_args,
    0, 0, 23, Iclass_iclass_QREADY_intfArgs },
  { 1, Iclass_iclass_RDTIEP_args,
    0, 0, 1, Iclass_iclass_RDTIEP_intfArgs },
  { 1, Iclass_iclass_SETTIEP_args,
    1, Iclass_iclass_SETTIEP_stateArgs, 0, 0 },
  { 5, Iclass_iclass_SMOD_LUT_args,
    2, Iclass_iclass_SMOD_LUT_stateArgs, 2, Iclass_iclass_SMOD_LUT_intfArgs },
  { 1, Iclass_iclass_WRTBSIGQ_args,
    0, 0, 1, Iclass_iclass_WRTBSIGQ_intfArgs },
  { 1, Iclass_iclass_WRTBSIGQM_args,
    0, 0, 1, Iclass_iclass_WRTBSIGQM_intfArgs },
  { 2, Iclass_iclass_WRTIEP_args,
    1, Iclass_iclass_WRTIEP_stateArgs, 0, 0 },
  { 1, Iclass_iclass_WRTSIGQ_args,
    0, 0, 1, Iclass_iclass_WRTSIGQ_intfArgs },
  { 2, Iclass_iclass_ABS8_args,
    1, Iclass_iclass_ABS8_stateArgs, 0, 0 },
  { 4, Iclass_iclass_ADD16_args,
    3, Iclass_iclass_ADD16_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ADD32_args,
    3, Iclass_iclass_ADD32_stateArgs, 0, 0 },
  { 1, Iclass_iclass_ADDAC_I2R_args,
    3, Iclass_iclass_ADDAC_I2R_stateArgs, 0, 0 },
  { 1, Iclass_iclass_ADDAC_R2I_args,
    3, Iclass_iclass_ADDAC_R2I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_ADDAR2_args,
    0, 0, 0, 0 },
  { 3, Iclass_iclass_ADDCM_args,
    3, Iclass_iclass_ADDCM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ADDWRP_args,
    1, Iclass_iclass_ADDWRP_stateArgs, 0, 0 },
  { 3, Iclass_iclass_AND128_args,
    1, Iclass_iclass_AND128_stateArgs, 0, 0 },
  { 2, Iclass_iclass_ARGMAX8_args,
    3, Iclass_iclass_ARGMAX8_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ASL_args,
    3, Iclass_iclass_ASL_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ASL32_args,
    3, Iclass_iclass_ASL32_stateArgs, 0, 0 },
  { 2, Iclass_iclass_ASLACM_args,
    3, Iclass_iclass_ASLACM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ASLM_args,
    3, Iclass_iclass_ASLM_stateArgs, 0, 0 },
  { 2, Iclass_iclass_ASLM32_args,
    3, Iclass_iclass_ASLM32_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ASR_args,
    3, Iclass_iclass_ASR_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ASR32_args,
    3, Iclass_iclass_ASR32_stateArgs, 0, 0 },
  { 2, Iclass_iclass_ASRAC_args,
    3, Iclass_iclass_ASRAC_stateArgs, 0, 0 },
  { 3, Iclass_iclass_ASRM_args,
    3, Iclass_iclass_ASRM_stateArgs, 0, 0 },
  { 4, Iclass_iclass_BITFEXT_args,
    0, 0, 0, 0 },
  { 4, Iclass_iclass_BITFINS_args,
    0, 0, 0, 0 },
  { 2, Iclass_iclass_CLB_C_args,
    5, Iclass_iclass_CLB_C_stateArgs, 0, 0 },
  { 2, Iclass_iclass_CLB_R_args,
    5, Iclass_iclass_CLB_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMP8_args,
    2, Iclass_iclass_CMP8_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMP_I_args,
    2, Iclass_iclass_CMP_I_stateArgs, 0, 0 },
  { 3, Iclass_iclass_CMP_R_args,
    2, Iclass_iclass_CMP_R_stateArgs, 0, 0 },
  { 5, Iclass_iclass_EXT_args,
    6, Iclass_iclass_EXT_stateArgs, 0, 0 },
  { 5, Iclass_iclass_EXT_R_args,
    6, Iclass_iclass_EXT_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_EXT32_I_args,
    2, Iclass_iclass_EXT32_I_stateArgs, 0, 0 },
  { 3, Iclass_iclass_EXT32_R_args,
    2, Iclass_iclass_EXT32_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_EXTUI4_args,
    1, Iclass_iclass_EXTUI4_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LSLM_args,
    1, Iclass_iclass_LSLM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_LSRM_args,
    1, Iclass_iclass_LSRM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MAX8_args,
    1, Iclass_iclass_MAX8_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MEAN_args,
    1, Iclass_iclass_MEAN_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MEAN32_args,
    1, Iclass_iclass_MEAN32_stateArgs, 0, 0 },
  { 3, Iclass_iclass_MIN8_args,
    1, Iclass_iclass_MIN8_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MINCLB_C_args,
    5, Iclass_iclass_MINCLB_C_stateArgs, 0, 0 },
  { 2, Iclass_iclass_MINCLB_R_args,
    5, Iclass_iclass_MINCLB_R_stateArgs, 0, 0 },
  { 2, Iclass_iclass_NOT128_args,
    1, Iclass_iclass_NOT128_stateArgs, 0, 0 },
  { 3, Iclass_iclass_OR128_args,
    1, Iclass_iclass_OR128_stateArgs, 0, 0 },
  { 3, Iclass_iclass_PERM_args,
    1, Iclass_iclass_PERM_stateArgs, 0, 0 },
  { 1, Iclass_iclass_REDAC_args,
    3, Iclass_iclass_REDAC_stateArgs, 0, 0 },
  { 1, Iclass_iclass_REDAC2_args,
    3, Iclass_iclass_REDAC2_stateArgs, 0, 0 },
  { 1, Iclass_iclass_REDAC4_args,
    3, Iclass_iclass_REDAC4_stateArgs, 0, 0 },
  { 1, Iclass_iclass_REDACS_args,
    3, Iclass_iclass_REDACS_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SMINCLB_C_args,
    5, Iclass_iclass_SMINCLB_C_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SMINCLB_R_args,
    5, Iclass_iclass_SMINCLB_R_stateArgs, 0, 0 },
  { 3, Iclass_iclass_STSWAPBM_args,
    0, 0, 0, 0 },
  { 3, Iclass_iclass_STSWAPBMU_args,
    0, 0, 0, 0 },
  { 3, Iclass_iclass_SUB32_args,
    3, Iclass_iclass_SUB32_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SUBAC_I2R_args,
    3, Iclass_iclass_SUBAC_I2R_stateArgs, 0, 0 },
  { 1, Iclass_iclass_SUBAC_R2I_args,
    3, Iclass_iclass_SUBAC_R2I_stateArgs, 0, 0 },
  { 2, Iclass_iclass_SUBARX_args,
    0, 0, 0, 0 },
  { 3, Iclass_iclass_SUBCM_args,
    3, Iclass_iclass_SUBCM_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SUBMEAN_args,
    1, Iclass_iclass_SUBMEAN_stateArgs, 0, 0 },
  { 3, Iclass_iclass_SUBWRP_args,
    1, Iclass_iclass_SUBWRP_stateArgs, 0, 0 },
  { 2, Iclass_iclass_TRANS_args,
    1, Iclass_iclass_TRANS_stateArgs, 0, 0 },
  { 3, Iclass_iclass_XOR128_args,
    1, Iclass_iclass_XOR128_stateArgs, 0, 0 },
  { 1, Iclass_rur_expstate_args,
    1, Iclass_rur_expstate_stateArgs, 0, 0 },
  { 1, Iclass_wur_expstate_args,
    1, Iclass_wur_expstate_stateArgs, 0, 0 },
  { 1, Iclass_rur_sov_args,
    2, Iclass_rur_sov_stateArgs, 0, 0 },
  { 1, Iclass_wur_sov_args,
    2, Iclass_wur_sov_stateArgs, 0, 0 },
  { 1, Iclass_rur_sat_mode_args,
    2, Iclass_rur_sat_mode_stateArgs, 0, 0 },
  { 1, Iclass_wur_sat_mode_args,
    2, Iclass_wur_sat_mode_stateArgs, 0, 0 },
  { 1, Iclass_rur_sar0_args,
    2, Iclass_rur_sar0_stateArgs, 0, 0 },
  { 1, Iclass_wur_sar0_args,
    2, Iclass_wur_sar0_stateArgs, 0, 0 },
  { 1, Iclass_rur_sar1_args,
    2, Iclass_rur_sar1_stateArgs, 0, 0 },
  { 1, Iclass_wur_sar1_args,
    2, Iclass_wur_sar1_stateArgs, 0, 0 },
  { 1, Iclass_rur_sar2_args,
    2, Iclass_rur_sar2_stateArgs, 0, 0 },
  { 1, Iclass_wur_sar2_args,
    2, Iclass_wur_sar2_stateArgs, 0, 0 },
  { 1, Iclass_rur_sar3_args,
    2, Iclass_rur_sar3_stateArgs, 0, 0 },
  { 1, Iclass_wur_sar3_args,
    2, Iclass_wur_sar3_stateArgs, 0, 0 },
  { 1, Iclass_rur_hsar0_args,
    2, Iclass_rur_hsar0_stateArgs, 0, 0 },
  { 1, Iclass_wur_hsar0_args,
    2, Iclass_wur_hsar0_stateArgs, 0, 0 },
  { 1, Iclass_rur_hsar1_args,
    2, Iclass_rur_hsar1_stateArgs, 0, 0 },
  { 1, Iclass_wur_hsar1_args,
    2, Iclass_wur_hsar1_stateArgs, 0, 0 },
  { 1, Iclass_rur_hsar2_args,
    2, Iclass_rur_hsar2_stateArgs, 0, 0 },
  { 1, Iclass_wur_hsar2_args,
    2, Iclass_wur_hsar2_stateArgs, 0, 0 },
  { 1, Iclass_rur_hsar3_args,
    2, Iclass_rur_hsar3_stateArgs, 0, 0 },
  { 1, Iclass_wur_hsar3_args,
    2, Iclass_wur_hsar3_stateArgs, 0, 0 },
  { 1, Iclass_rur_max_reg_0_args,
    2, Iclass_rur_max_reg_0_stateArgs, 0, 0 },
  { 1, Iclass_wur_max_reg_0_args,
    2, Iclass_wur_max_reg_0_stateArgs, 0, 0 },
  { 1, Iclass_rur_max_reg_1_args,
    2, Iclass_rur_max_reg_1_stateArgs, 0, 0 },
  { 1, Iclass_wur_max_reg_1_args,
    2, Iclass_wur_max_reg_1_stateArgs, 0, 0 },
  { 1, Iclass_rur_max_reg_2_args,
    2, Iclass_rur_max_reg_2_stateArgs, 0, 0 },
  { 1, Iclass_wur_max_reg_2_args,
    2, Iclass_wur_max_reg_2_stateArgs, 0, 0 },
  { 1, Iclass_rur_max_reg_3_args,
    2, Iclass_rur_max_reg_3_stateArgs, 0, 0 },
  { 1, Iclass_wur_max_reg_3_args,
    2, Iclass_wur_max_reg_3_stateArgs, 0, 0 },
  { 1, Iclass_rur_arg_max_reg_0_args,
    2, Iclass_rur_arg_max_reg_0_stateArgs, 0, 0 },
  { 1, Iclass_wur_arg_max_reg_0_args,
    2, Iclass_wur_arg_max_reg_0_stateArgs, 0, 0 },
  { 1, Iclass_rur_arg_max_reg_1_args,
    2, Iclass_rur_arg_max_reg_1_stateArgs, 0, 0 },
  { 1, Iclass_wur_arg_max_reg_1_args,
    2, Iclass_wur_arg_max_reg_1_stateArgs, 0, 0 },
  { 1, Iclass_rur_arg_max_reg_2_args,
    2, Iclass_rur_arg_max_reg_2_stateArgs, 0, 0 },
  { 1, Iclass_wur_arg_max_reg_2_args,
    2, Iclass_wur_arg_max_reg_2_stateArgs, 0, 0 },
  { 1, Iclass_rur_arg_max_reg_3_args,
    2, Iclass_rur_arg_max_reg_3_stateArgs, 0, 0 },
  { 1, Iclass_wur_arg_max_reg_3_args,
    2, Iclass_wur_arg_max_reg_3_stateArgs, 0, 0 },
  { 1, Iclass_rur_nco_counter_0_args,
    2, Iclass_rur_nco_counter_0_stateArgs, 0, 0 },
  { 1, Iclass_wur_nco_counter_0_args,
    2, Iclass_wur_nco_counter_0_stateArgs, 0, 0 },
  { 1, Iclass_rur_nco_counter_1_args,
    2, Iclass_rur_nco_counter_1_stateArgs, 0, 0 },
  { 1, Iclass_wur_nco_counter_1_args,
    2, Iclass_wur_nco_counter_1_stateArgs, 0, 0 },
  { 1, Iclass_rur_nco_counter_2_args,
    2, Iclass_rur_nco_counter_2_stateArgs, 0, 0 },
  { 1, Iclass_wur_nco_counter_2_args,
    2, Iclass_wur_nco_counter_2_stateArgs, 0, 0 },
  { 1, Iclass_rur_nco_counter_3_args,
    2, Iclass_rur_nco_counter_3_stateArgs, 0, 0 },
  { 1, Iclass_wur_nco_counter_3_args,
    2, Iclass_wur_nco_counter_3_stateArgs, 0, 0 },
  { 1, Iclass_rur_interp_ext_n_args,
    2, Iclass_rur_interp_ext_n_stateArgs, 0, 0 },
  { 1, Iclass_wur_interp_ext_n_args,
    2, Iclass_wur_interp_ext_n_stateArgs, 0, 0 },
  { 1, Iclass_rur_interp_ext_l_args,
    2, Iclass_rur_interp_ext_l_stateArgs, 0, 0 },
  { 1, Iclass_wur_interp_ext_l_args,
    2, Iclass_wur_interp_ext_l_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_0_args,
    2, Iclass_rur_llr_buf_0_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_0_args,
    2, Iclass_wur_llr_buf_0_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_1_args,
    2, Iclass_rur_llr_buf_1_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_1_args,
    2, Iclass_wur_llr_buf_1_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_2_args,
    2, Iclass_rur_llr_buf_2_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_2_args,
    2, Iclass_wur_llr_buf_2_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_3_args,
    2, Iclass_rur_llr_buf_3_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_3_args,
    2, Iclass_wur_llr_buf_3_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_4_args,
    2, Iclass_rur_llr_buf_4_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_4_args,
    2, Iclass_wur_llr_buf_4_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_5_args,
    2, Iclass_rur_llr_buf_5_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_5_args,
    2, Iclass_wur_llr_buf_5_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_6_args,
    2, Iclass_rur_llr_buf_6_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_6_args,
    2, Iclass_wur_llr_buf_6_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_7_args,
    2, Iclass_rur_llr_buf_7_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_7_args,
    2, Iclass_wur_llr_buf_7_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_8_args,
    2, Iclass_rur_llr_buf_8_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_8_args,
    2, Iclass_wur_llr_buf_8_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_9_args,
    2, Iclass_rur_llr_buf_9_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_9_args,
    2, Iclass_wur_llr_buf_9_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_10_args,
    2, Iclass_rur_llr_buf_10_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_10_args,
    2, Iclass_wur_llr_buf_10_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_11_args,
    2, Iclass_rur_llr_buf_11_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_11_args,
    2, Iclass_wur_llr_buf_11_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_12_args,
    2, Iclass_rur_llr_buf_12_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_12_args,
    2, Iclass_wur_llr_buf_12_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_13_args,
    2, Iclass_rur_llr_buf_13_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_13_args,
    2, Iclass_wur_llr_buf_13_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_14_args,
    2, Iclass_rur_llr_buf_14_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_14_args,
    2, Iclass_wur_llr_buf_14_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_15_args,
    2, Iclass_rur_llr_buf_15_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_15_args,
    2, Iclass_wur_llr_buf_15_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_16_args,
    2, Iclass_rur_llr_buf_16_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_16_args,
    2, Iclass_wur_llr_buf_16_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_17_args,
    2, Iclass_rur_llr_buf_17_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_17_args,
    2, Iclass_wur_llr_buf_17_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_18_args,
    2, Iclass_rur_llr_buf_18_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_18_args,
    2, Iclass_wur_llr_buf_18_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_19_args,
    2, Iclass_rur_llr_buf_19_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_19_args,
    2, Iclass_wur_llr_buf_19_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_20_args,
    2, Iclass_rur_llr_buf_20_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_20_args,
    2, Iclass_wur_llr_buf_20_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_21_args,
    2, Iclass_rur_llr_buf_21_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_21_args,
    2, Iclass_wur_llr_buf_21_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_22_args,
    2, Iclass_rur_llr_buf_22_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_22_args,
    2, Iclass_wur_llr_buf_22_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_buf_23_args,
    2, Iclass_rur_llr_buf_23_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_buf_23_args,
    2, Iclass_wur_llr_buf_23_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_0_args,
    2, Iclass_rur_smod_buf_0_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_0_args,
    2, Iclass_wur_smod_buf_0_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_1_args,
    2, Iclass_rur_smod_buf_1_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_1_args,
    2, Iclass_wur_smod_buf_1_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_2_args,
    2, Iclass_rur_smod_buf_2_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_2_args,
    2, Iclass_wur_smod_buf_2_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_3_args,
    2, Iclass_rur_smod_buf_3_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_3_args,
    2, Iclass_wur_smod_buf_3_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_4_args,
    2, Iclass_rur_smod_buf_4_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_4_args,
    2, Iclass_wur_smod_buf_4_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_5_args,
    2, Iclass_rur_smod_buf_5_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_5_args,
    2, Iclass_wur_smod_buf_5_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_6_args,
    2, Iclass_rur_smod_buf_6_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_6_args,
    2, Iclass_wur_smod_buf_6_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_buf_7_args,
    2, Iclass_rur_smod_buf_7_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_buf_7_args,
    2, Iclass_wur_smod_buf_7_stateArgs, 0, 0 },
  { 1, Iclass_rur_weight_reg_args,
    2, Iclass_rur_weight_reg_stateArgs, 0, 0 },
  { 1, Iclass_wur_weight_reg_args,
    2, Iclass_wur_weight_reg_stateArgs, 0, 0 },
  { 1, Iclass_rur_scale_reg_args,
    2, Iclass_rur_scale_reg_stateArgs, 0, 0 },
  { 1, Iclass_wur_scale_reg_args,
    2, Iclass_wur_scale_reg_stateArgs, 0, 0 },
  { 1, Iclass_rur_llr_pos_args,
    2, Iclass_rur_llr_pos_stateArgs, 0, 0 },
  { 1, Iclass_wur_llr_pos_args,
    2, Iclass_wur_llr_pos_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_pos_args,
    2, Iclass_rur_smod_pos_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_pos_args,
    2, Iclass_wur_smod_pos_stateArgs, 0, 0 },
  { 1, Iclass_rur_perm_reg_args,
    2, Iclass_rur_perm_reg_stateArgs, 0, 0 },
  { 1, Iclass_wur_perm_reg_args,
    2, Iclass_wur_perm_reg_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_offset_table_0_args,
    2, Iclass_rur_smod_offset_table_0_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_offset_table_0_args,
    2, Iclass_wur_smod_offset_table_0_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_offset_table_1_args,
    2, Iclass_rur_smod_offset_table_1_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_offset_table_1_args,
    2, Iclass_wur_smod_offset_table_1_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_offset_table_2_args,
    2, Iclass_rur_smod_offset_table_2_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_offset_table_2_args,
    2, Iclass_wur_smod_offset_table_2_stateArgs, 0, 0 },
  { 1, Iclass_rur_smod_offset_table_3_args,
    2, Iclass_rur_smod_offset_table_3_stateArgs, 0, 0 },
  { 1, Iclass_wur_smod_offset_table_3_args,
    2, Iclass_wur_smod_offset_table_3_stateArgs, 0, 0 },
  { 1, Iclass_rur_phasor_n_args,
    2, Iclass_rur_phasor_n_stateArgs, 0, 0 },
  { 1, Iclass_wur_phasor_n_args,
    2, Iclass_wur_phasor_n_stateArgs, 0, 0 },
  { 1, Iclass_rur_phasor_offset_args,
    2, Iclass_rur_phasor_offset_stateArgs, 0, 0 },
  { 1, Iclass_wur_phasor_offset_args,
    2, Iclass_wur_phasor_offset_stateArgs, 0, 0 }
};

enum xtensa_iclass_id {
  ICLASS_xt_iclass_excw,
  ICLASS_xt_iclass_rfe,
  ICLASS_xt_iclass_rfde,
  ICLASS_xt_iclass_syscall,
  ICLASS_xt_iclass_simcall,
  ICLASS_xt_iclass_call12,
  ICLASS_xt_iclass_call8,
  ICLASS_xt_iclass_call4,
  ICLASS_xt_iclass_callx12,
  ICLASS_xt_iclass_callx8,
  ICLASS_xt_iclass_callx4,
  ICLASS_xt_iclass_entry,
  ICLASS_xt_iclass_movsp,
  ICLASS_xt_iclass_rotw,
  ICLASS_xt_iclass_retw,
  ICLASS_xt_iclass_rfwou,
  ICLASS_xt_iclass_l32e,
  ICLASS_xt_iclass_s32e,
  ICLASS_xt_iclass_rsr_windowbase,
  ICLASS_xt_iclass_wsr_windowbase,
  ICLASS_xt_iclass_xsr_windowbase,
  ICLASS_xt_iclass_rsr_windowstart,
  ICLASS_xt_iclass_wsr_windowstart,
  ICLASS_xt_iclass_xsr_windowstart,
  ICLASS_xt_iclass_add_n,
  ICLASS_xt_iclass_addi_n,
  ICLASS_xt_iclass_bz6,
  ICLASS_xt_iclass_ill_n,
  ICLASS_xt_iclass_loadi4,
  ICLASS_xt_iclass_mov_n,
  ICLASS_xt_iclass_movi_n,
  ICLASS_xt_iclass_nopn,
  ICLASS_xt_iclass_retn,
  ICLASS_xt_iclass_storei4,
  ICLASS_rur_threadptr,
  ICLASS_wur_threadptr,
  ICLASS_xt_iclass_addi,
  ICLASS_xt_iclass_addmi,
  ICLASS_xt_iclass_addsub,
  ICLASS_xt_iclass_bit,
  ICLASS_xt_iclass_bsi8,
  ICLASS_xt_iclass_bsi8b,
  ICLASS_xt_iclass_bsi8u,
  ICLASS_xt_iclass_bst8,
  ICLASS_xt_iclass_bsz12,
  ICLASS_xt_iclass_call0,
  ICLASS_xt_iclass_callx0,
  ICLASS_xt_iclass_exti,
  ICLASS_xt_iclass_ill,
  ICLASS_xt_iclass_jump,
  ICLASS_xt_iclass_jumpx,
  ICLASS_xt_iclass_l16ui,
  ICLASS_xt_iclass_l16si,
  ICLASS_xt_iclass_l32i,
  ICLASS_xt_iclass_l32r,
  ICLASS_xt_iclass_l8i,
  ICLASS_xt_iclass_loop,
  ICLASS_xt_iclass_loopz,
  ICLASS_xt_iclass_movi,
  ICLASS_xt_iclass_movz,
  ICLASS_xt_iclass_neg,
  ICLASS_xt_iclass_nop,
  ICLASS_xt_iclass_return,
  ICLASS_xt_iclass_s16i,
  ICLASS_xt_iclass_s32i,
  ICLASS_xt_iclass_s8i,
  ICLASS_xt_iclass_sar,
  ICLASS_xt_iclass_sari,
  ICLASS_xt_iclass_shifts,
  ICLASS_xt_iclass_shiftst,
  ICLASS_xt_iclass_shiftt,
  ICLASS_xt_iclass_slli,
  ICLASS_xt_iclass_srai,
  ICLASS_xt_iclass_srli,
  ICLASS_xt_iclass_memw,
  ICLASS_xt_iclass_extw,
  ICLASS_xt_iclass_isync,
  ICLASS_xt_iclass_sync,
  ICLASS_xt_iclass_rsil,
  ICLASS_xt_iclass_rsr_lend,
  ICLASS_xt_iclass_wsr_lend,
  ICLASS_xt_iclass_xsr_lend,
  ICLASS_xt_iclass_rsr_lcount,
  ICLASS_xt_iclass_wsr_lcount,
  ICLASS_xt_iclass_xsr_lcount,
  ICLASS_xt_iclass_rsr_lbeg,
  ICLASS_xt_iclass_wsr_lbeg,
  ICLASS_xt_iclass_xsr_lbeg,
  ICLASS_xt_iclass_rsr_sar,
  ICLASS_xt_iclass_wsr_sar,
  ICLASS_xt_iclass_xsr_sar,
  ICLASS_xt_iclass_rsr_litbase,
  ICLASS_xt_iclass_wsr_litbase,
  ICLASS_xt_iclass_xsr_litbase,
  ICLASS_xt_iclass_rsr_176,
  ICLASS_xt_iclass_wsr_176,
  ICLASS_xt_iclass_rsr_208,
  ICLASS_xt_iclass_rsr_ps,
  ICLASS_xt_iclass_wsr_ps,
  ICLASS_xt_iclass_xsr_ps,
  ICLASS_xt_iclass_rsr_epc1,
  ICLASS_xt_iclass_wsr_epc1,
  ICLASS_xt_iclass_xsr_epc1,
  ICLASS_xt_iclass_rsr_excsave1,
  ICLASS_xt_iclass_wsr_excsave1,
  ICLASS_xt_iclass_xsr_excsave1,
  ICLASS_xt_iclass_rsr_epc2,
  ICLASS_xt_iclass_wsr_epc2,
  ICLASS_xt_iclass_xsr_epc2,
  ICLASS_xt_iclass_rsr_excsave2,
  ICLASS_xt_iclass_wsr_excsave2,
  ICLASS_xt_iclass_xsr_excsave2,
  ICLASS_xt_iclass_rsr_epc3,
  ICLASS_xt_iclass_wsr_epc3,
  ICLASS_xt_iclass_xsr_epc3,
  ICLASS_xt_iclass_rsr_excsave3,
  ICLASS_xt_iclass_wsr_excsave3,
  ICLASS_xt_iclass_xsr_excsave3,
  ICLASS_xt_iclass_rsr_epc4,
  ICLASS_xt_iclass_wsr_epc4,
  ICLASS_xt_iclass_xsr_epc4,
  ICLASS_xt_iclass_rsr_excsave4,
  ICLASS_xt_iclass_wsr_excsave4,
  ICLASS_xt_iclass_xsr_excsave4,
  ICLASS_xt_iclass_rsr_epc5,
  ICLASS_xt_iclass_wsr_epc5,
  ICLASS_xt_iclass_xsr_epc5,
  ICLASS_xt_iclass_rsr_excsave5,
  ICLASS_xt_iclass_wsr_excsave5,
  ICLASS_xt_iclass_xsr_excsave5,
  ICLASS_xt_iclass_rsr_epc6,
  ICLASS_xt_iclass_wsr_epc6,
  ICLASS_xt_iclass_xsr_epc6,
  ICLASS_xt_iclass_rsr_excsave6,
  ICLASS_xt_iclass_wsr_excsave6,
  ICLASS_xt_iclass_xsr_excsave6,
  ICLASS_xt_iclass_rsr_eps2,
  ICLASS_xt_iclass_wsr_eps2,
  ICLASS_xt_iclass_xsr_eps2,
  ICLASS_xt_iclass_rsr_eps3,
  ICLASS_xt_iclass_wsr_eps3,
  ICLASS_xt_iclass_xsr_eps3,
  ICLASS_xt_iclass_rsr_eps4,
  ICLASS_xt_iclass_wsr_eps4,
  ICLASS_xt_iclass_xsr_eps4,
  ICLASS_xt_iclass_rsr_eps5,
  ICLASS_xt_iclass_wsr_eps5,
  ICLASS_xt_iclass_xsr_eps5,
  ICLASS_xt_iclass_rsr_eps6,
  ICLASS_xt_iclass_wsr_eps6,
  ICLASS_xt_iclass_xsr_eps6,
  ICLASS_xt_iclass_rsr_excvaddr,
  ICLASS_xt_iclass_wsr_excvaddr,
  ICLASS_xt_iclass_xsr_excvaddr,
  ICLASS_xt_iclass_rsr_depc,
  ICLASS_xt_iclass_wsr_depc,
  ICLASS_xt_iclass_xsr_depc,
  ICLASS_xt_iclass_rsr_exccause,
  ICLASS_xt_iclass_wsr_exccause,
  ICLASS_xt_iclass_xsr_exccause,
  ICLASS_xt_iclass_rsr_prid,
  ICLASS_xt_iclass_rsr_vecbase,
  ICLASS_xt_iclass_wsr_vecbase,
  ICLASS_xt_iclass_xsr_vecbase,
  ICLASS_xt_mul16,
  ICLASS_xt_iclass_rfi,
  ICLASS_xt_iclass_wait,
  ICLASS_xt_iclass_rsr_interrupt,
  ICLASS_xt_iclass_wsr_intset,
  ICLASS_xt_iclass_wsr_intclear,
  ICLASS_xt_iclass_rsr_intenable,
  ICLASS_xt_iclass_wsr_intenable,
  ICLASS_xt_iclass_xsr_intenable,
  ICLASS_xt_iclass_break,
  ICLASS_xt_iclass_break_n,
  ICLASS_xt_iclass_rsr_dbreaka0,
  ICLASS_xt_iclass_wsr_dbreaka0,
  ICLASS_xt_iclass_xsr_dbreaka0,
  ICLASS_xt_iclass_rsr_dbreakc0,
  ICLASS_xt_iclass_wsr_dbreakc0,
  ICLASS_xt_iclass_xsr_dbreakc0,
  ICLASS_xt_iclass_rsr_dbreaka1,
  ICLASS_xt_iclass_wsr_dbreaka1,
  ICLASS_xt_iclass_xsr_dbreaka1,
  ICLASS_xt_iclass_rsr_dbreakc1,
  ICLASS_xt_iclass_wsr_dbreakc1,
  ICLASS_xt_iclass_xsr_dbreakc1,
  ICLASS_xt_iclass_rsr_ibreaka0,
  ICLASS_xt_iclass_wsr_ibreaka0,
  ICLASS_xt_iclass_xsr_ibreaka0,
  ICLASS_xt_iclass_rsr_ibreaka1,
  ICLASS_xt_iclass_wsr_ibreaka1,
  ICLASS_xt_iclass_xsr_ibreaka1,
  ICLASS_xt_iclass_rsr_ibreakenable,
  ICLASS_xt_iclass_wsr_ibreakenable,
  ICLASS_xt_iclass_xsr_ibreakenable,
  ICLASS_xt_iclass_rsr_debugcause,
  ICLASS_xt_iclass_wsr_debugcause,
  ICLASS_xt_iclass_xsr_debugcause,
  ICLASS_xt_iclass_rsr_icount,
  ICLASS_xt_iclass_wsr_icount,
  ICLASS_xt_iclass_xsr_icount,
  ICLASS_xt_iclass_rsr_icountlevel,
  ICLASS_xt_iclass_wsr_icountlevel,
  ICLASS_xt_iclass_xsr_icountlevel,
  ICLASS_xt_iclass_rsr_ddr,
  ICLASS_xt_iclass_wsr_ddr,
  ICLASS_xt_iclass_xsr_ddr,
  ICLASS_xt_iclass_rfdo,
  ICLASS_xt_iclass_rfdd,
  ICLASS_xt_iclass_wsr_mmid,
  ICLASS_xt_iclass_bbool1,
  ICLASS_xt_iclass_bbool4,
  ICLASS_xt_iclass_bbool8,
  ICLASS_xt_iclass_bbranch,
  ICLASS_xt_iclass_bmove,
  ICLASS_xt_iclass_RSR_BR,
  ICLASS_xt_iclass_WSR_BR,
  ICLASS_xt_iclass_XSR_BR,
  ICLASS_xt_iclass_rsr_ccount,
  ICLASS_xt_iclass_wsr_ccount,
  ICLASS_xt_iclass_xsr_ccount,
  ICLASS_xt_iclass_rsr_ccompare0,
  ICLASS_xt_iclass_wsr_ccompare0,
  ICLASS_xt_iclass_xsr_ccompare0,
  ICLASS_xt_iclass_rsr_ccompare1,
  ICLASS_xt_iclass_wsr_ccompare1,
  ICLASS_xt_iclass_xsr_ccompare1,
  ICLASS_xt_iclass_icache,
  ICLASS_xt_iclass_icache_lock,
  ICLASS_xt_iclass_icache_inv,
  ICLASS_xt_iclass_licx,
  ICLASS_xt_iclass_sicx,
  ICLASS_xt_iclass_dcache,
  ICLASS_xt_iclass_dcache_ind,
  ICLASS_xt_iclass_dcache_inv,
  ICLASS_xt_iclass_dpf,
  ICLASS_xt_iclass_dcache_lock,
  ICLASS_xt_iclass_sdct,
  ICLASS_xt_iclass_ldct,
  ICLASS_xt_iclass_idtlb,
  ICLASS_xt_iclass_rdtlb,
  ICLASS_xt_iclass_wdtlb,
  ICLASS_xt_iclass_iitlb,
  ICLASS_xt_iclass_ritlb,
  ICLASS_xt_iclass_witlb,
  ICLASS_xt_iclass_rsr_cpenable,
  ICLASS_xt_iclass_wsr_cpenable,
  ICLASS_xt_iclass_xsr_cpenable,
  ICLASS_xt_iclass_clamp,
  ICLASS_xt_iclass_minmax,
  ICLASS_xt_iclass_nsa,
  ICLASS_xt_iclass_sx,
  ICLASS_xt_iclass_l32ai,
  ICLASS_xt_iclass_s32ri,
  ICLASS_xt_iclass_s32c1i,
  ICLASS_xt_iclass_rsr_scompare1,
  ICLASS_xt_iclass_wsr_scompare1,
  ICLASS_xt_iclass_xsr_scompare1,
  ICLASS_xt_iclass_rsr_atomctl,
  ICLASS_xt_iclass_wsr_atomctl,
  ICLASS_xt_iclass_xsr_atomctl,
  ICLASS_xt_iclass_rer,
  ICLASS_xt_iclass_wer,
  ICLASS_rur_fcr,
  ICLASS_wur_fcr,
  ICLASS_rur_fsr,
  ICLASS_wur_fsr,
  ICLASS_fp,
  ICLASS_fp_mac,
  ICLASS_fp_cmov,
  ICLASS_fp_mov,
  ICLASS_fp_mov2,
  ICLASS_fp_cmp,
  ICLASS_fp_float,
  ICLASS_fp_int,
  ICLASS_fp_rfr,
  ICLASS_fp_wfr,
  ICLASS_fp_lsi,
  ICLASS_fp_lsiu,
  ICLASS_fp_lsx,
  ICLASS_fp_lsxu,
  ICLASS_fp_ssi,
  ICLASS_fp_ssiu,
  ICLASS_fp_ssx,
  ICLASS_fp_ssxu,
  ICLASS_iclass_GET_ARGMAX,
  ICLASS_iclass_GET_HSAR,
  ICLASS_iclass_GET_HSAR2SAR,
  ICLASS_iclass_GET_INTERP_EXT_N,
  ICLASS_iclass_GET_INTERP_EXT_L,
  ICLASS_iclass_GET_LLR_BUF,
  ICLASS_iclass_GET_LLR_POS,
  ICLASS_iclass_GET_MAX,
  ICLASS_iclass_GET_NCO,
  ICLASS_iclass_GET_PERM_REG,
  ICLASS_iclass_GET_PHASOR_N,
  ICLASS_iclass_GET_PHASOR_OFFSET,
  ICLASS_iclass_GET_SAR,
  ICLASS_iclass_GET_SCALE_REG,
  ICLASS_iclass_GET_SMOD_BUF,
  ICLASS_iclass_GET_SMOD_OFFSET_TABLE,
  ICLASS_iclass_GET_SMOD_POS,
  ICLASS_iclass_GET_SOV,
  ICLASS_iclass_GET_WGHT,
  ICLASS_iclass_SET_ARGMAX,
  ICLASS_iclass_SET_EXT_REGS,
  ICLASS_iclass_SET_HSAR,
  ICLASS_iclass_SET_LLR_BUF,
  ICLASS_iclass_SET_LLR_POS,
  ICLASS_iclass_SET_MAX,
  ICLASS_iclass_SET_NCO,
  ICLASS_iclass_SET_PERM_REG,
  ICLASS_iclass_SET_PHASOR_N,
  ICLASS_iclass_SET_PHASOR_OFFSET,
  ICLASS_iclass_SET_SAR,
  ICLASS_iclass_SET_SCALE_REG,
  ICLASS_iclass_SET_SMOD_BUF,
  ICLASS_iclass_SET_SMOD_OFFSET_TABLE,
  ICLASS_iclass_SET_SMOD_POS,
  ICLASS_iclass_SET_SOV,
  ICLASS_iclass_SET_WGHT,
  ICLASS_iclass_LAC2X32,
  ICLASS_iclass_LAC2X64_0,
  ICLASS_iclass_LAC2X64_1,
  ICLASS_iclass_LAC2X64_2,
  ICLASS_iclass_LAC2X64_3,
  ICLASS_iclass_LAC32_R,
  ICLASS_iclass_LAC_IH,
  ICLASS_iclass_LAC_IL,
  ICLASS_iclass_LAC_RH,
  ICLASS_iclass_LAC_RL,
  ICLASS_iclass_LCM,
  ICLASS_iclass_LCM_PINC,
  ICLASS_iclass_LCM_PINC_X,
  ICLASS_iclass_LCM_U,
  ICLASS_iclass_LCM_X,
  ICLASS_iclass_LCM_XU,
  ICLASS_iclass_LP,
  ICLASS_iclass_LP_X,
  ICLASS_iclass_LQ,
  ICLASS_iclass_LQ_X,
  ICLASS_iclass_LUT0,
  ICLASS_iclass_LUT1,
  ICLASS_iclass_LUT2,
  ICLASS_iclass_LUT3,
  ICLASS_iclass_SAC2X32,
  ICLASS_iclass_SAC2X64_0,
  ICLASS_iclass_SAC2X64_1,
  ICLASS_iclass_SAC2X64_2,
  ICLASS_iclass_SAC2X64_3,
  ICLASS_iclass_SAC32_R,
  ICLASS_iclass_SAC_IH,
  ICLASS_iclass_SAC_IL,
  ICLASS_iclass_SAC_RH,
  ICLASS_iclass_SAC_RL,
  ICLASS_iclass_SCM,
  ICLASS_iclass_SCM_PINC,
  ICLASS_iclass_SCM_PINC_X,
  ICLASS_iclass_SCM_U,
  ICLASS_iclass_SCM_X,
  ICLASS_iclass_SCM_XU,
  ICLASS_iclass_STORE_P,
  ICLASS_iclass_STORE_Q,
  ICLASS_iclass_AR2CM_DUP,
  ICLASS_iclass_AR2CM_LN,
  ICLASS_iclass_AR2CM_LN_I,
  ICLASS_iclass_AR2CM_LN_R,
  ICLASS_iclass_AR2PQ_LN,
  ICLASS_iclass_AR2SAR_DUP,
  ICLASS_iclass_CLRAC,
  ICLASS_iclass_CLRCM,
  ICLASS_iclass_CM2AR_LN,
  ICLASS_iclass_CM2AR_LN_I,
  ICLASS_iclass_CM2AR_LN_R,
  ICLASS_iclass_COMB_AR,
  ICLASS_iclass_CONJ,
  ICLASS_iclass_MOV2AC32_I,
  ICLASS_iclass_MOV2AC32_R,
  ICLASS_iclass_MOV2CM2PQ,
  ICLASS_iclass_MOVAC,
  ICLASS_iclass_MOVAC_I,
  ICLASS_iclass_MOVAC_I2R,
  ICLASS_iclass_MOVAC_R,
  ICLASS_iclass_MOVAC_R2I,
  ICLASS_iclass_MOVAR2,
  ICLASS_iclass_MOVCM,
  ICLASS_iclass_MOVCM2PQ,
  ICLASS_iclass_MOVCND_0,
  ICLASS_iclass_MOVCND_1,
  ICLASS_iclass_MOVCND_2,
  ICLASS_iclass_MOVCND_3,
  ICLASS_iclass_MOVCND_4,
  ICLASS_iclass_MOVCND_5,
  ICLASS_iclass_MOVCND_6,
  ICLASS_iclass_MOVCND_7,
  ICLASS_iclass_MOVCND8_0,
  ICLASS_iclass_MOVCND8_1,
  ICLASS_iclass_MOVCND8_2,
  ICLASS_iclass_MOVCND8_3,
  ICLASS_iclass_MOVCND8_4,
  ICLASS_iclass_MOVCND8_5,
  ICLASS_iclass_MOVCND8_6,
  ICLASS_iclass_MOVCND8_7,
  ICLASS_iclass_MOV_I,
  ICLASS_iclass_MOVPQ2PQ,
  ICLASS_iclass_MOV_R,
  ICLASS_iclass_NEGCM,
  ICLASS_iclass_POP16LLR_1,
  ICLASS_iclass_PQ2CM,
  ICLASS_iclass_SWAPAC_R,
  ICLASS_iclass_SWAPAC_RI,
  ICLASS_iclass_SWAPB,
  ICLASS_iclass_ADD2AC,
  ICLASS_iclass_ADDAC,
  ICLASS_iclass_CDOT,
  ICLASS_iclass_CDOTAC,
  ICLASS_iclass_CDOTACS,
  ICLASS_iclass_CMAC,
  ICLASS_iclass_CMACS,
  ICLASS_iclass_CMPY,
  ICLASS_iclass_CMPY2CM,
  ICLASS_iclass_CMPY2PQ,
  ICLASS_iclass_CMPYS,
  ICLASS_iclass_CMPYXP2PQ,
  ICLASS_iclass_COMB32,
  ICLASS_iclass_DOT,
  ICLASS_iclass_DOTAC,
  ICLASS_iclass_DOTACS,
  ICLASS_iclass_LIN_INT,
  ICLASS_iclass_LLRPRE1,
  ICLASS_iclass_LLRPRE2,
  ICLASS_iclass_MAC,
  ICLASS_iclass_MAC8,
  ICLASS_iclass_MACD8,
  ICLASS_iclass_MACPQXP_0,
  ICLASS_iclass_MACPQXP_1,
  ICLASS_iclass_MACPQXP_2,
  ICLASS_iclass_MACPQXP_3,
  ICLASS_iclass_MACS,
  ICLASS_iclass_MACXP2_0,
  ICLASS_iclass_MACXP2_1,
  ICLASS_iclass_MACXP_0,
  ICLASS_iclass_MACXP_1,
  ICLASS_iclass_MACXP_2,
  ICLASS_iclass_MACXP_3,
  ICLASS_iclass_MOV2AC,
  ICLASS_iclass_MPY,
  ICLASS_iclass_MPY2CM,
  ICLASS_iclass_MPY2PQ,
  ICLASS_iclass_MPY8,
  ICLASS_iclass_MPYADD8_2CM,
  ICLASS_iclass_MPYD8,
  ICLASS_iclass_MPYPQXP_0,
  ICLASS_iclass_MPYPQXP_1,
  ICLASS_iclass_MPYPQXP_2,
  ICLASS_iclass_MPYPQXP_3,
  ICLASS_iclass_MPYS,
  ICLASS_iclass_MPYXP2PQ,
  ICLASS_iclass_MPYXP2_0,
  ICLASS_iclass_MPYXP2_1,
  ICLASS_iclass_MPYXP_0,
  ICLASS_iclass_MPYXP_1,
  ICLASS_iclass_MPYXP_2,
  ICLASS_iclass_MPYXP_3,
  ICLASS_iclass_NORMACD,
  ICLASS_iclass_NORMACPQ_I,
  ICLASS_iclass_NORMACPQ_R,
  ICLASS_iclass_NORMD,
  ICLASS_iclass_NORMPYPQ_I,
  ICLASS_iclass_NORMPYPQ_R,
  ICLASS_iclass_RCMAC,
  ICLASS_iclass_RCMPY,
  ICLASS_iclass_RCMPY2CM,
  ICLASS_iclass_RFIR,
  ICLASS_iclass_RFIRA,
  ICLASS_iclass_RFIRD,
  ICLASS_iclass_RFIRDA,
  ICLASS_iclass_RMAC,
  ICLASS_iclass_RMPY,
  ICLASS_iclass_RMPY2CM,
  ICLASS_iclass_SMOD_ALIGN,
  ICLASS_iclass_SMOD_SCR,
  ICLASS_iclass_SUB2AC,
  ICLASS_iclass_WGHT32,
  ICLASS_iclass_CLRTIEP,
  ICLASS_iclass_EXT_2FIFO_0,
  ICLASS_iclass_EXT_2FIFO_1,
  ICLASS_iclass_EXT_2FIFO_2,
  ICLASS_iclass_EXT_2FIFO_3,
  ICLASS_iclass_EXT_R2FIFO_0,
  ICLASS_iclass_EXT_R2FIFO_1,
  ICLASS_iclass_EXT_R2FIFO_2,
  ICLASS_iclass_EXT_R2FIFO_3,
  ICLASS_iclass_LUT,
  ICLASS_iclass_LUT_AR,
  ICLASS_iclass_LUT_IEXT,
  ICLASS_iclass_LUT_PHASOR,
  ICLASS_iclass_LUT_REXT,
  ICLASS_iclass_LUT_WRITE,
  ICLASS_iclass_MOVEQ128_0,
  ICLASS_iclass_MOVEQ128_1,
  ICLASS_iclass_MOVEQ128_2,
  ICLASS_iclass_MOVEQ128_3,
  ICLASS_iclass_MOVEQ128_4,
  ICLASS_iclass_MOVEQ128_5,
  ICLASS_iclass_MOVEQ32_0,
  ICLASS_iclass_MOVEQ32_1,
  ICLASS_iclass_MOVEQ32_2,
  ICLASS_iclass_MOVEQ32_3,
  ICLASS_iclass_NCO_UPDATE,
  ICLASS_iclass_POP128_0,
  ICLASS_iclass_POP128_1,
  ICLASS_iclass_POP128_2,
  ICLASS_iclass_POP128_3,
  ICLASS_iclass_POP128_4,
  ICLASS_iclass_POP128_5,
  ICLASS_iclass_POP128_2CMPQ_0,
  ICLASS_iclass_POP128_2CMPQ_1,
  ICLASS_iclass_POP128_2CMPQ_2,
  ICLASS_iclass_POP128_2CMPQ_3,
  ICLASS_iclass_POP128_2M_0,
  ICLASS_iclass_POP128_2M_1,
  ICLASS_iclass_POP128_2M_2,
  ICLASS_iclass_POP128_2M_3,
  ICLASS_iclass_POP128_2PQ_0,
  ICLASS_iclass_POP128_2PQ_1,
  ICLASS_iclass_POP128_2PQ_2,
  ICLASS_iclass_POP128_2PQ_3,
  ICLASS_iclass_POP128_2PQ_4,
  ICLASS_iclass_POP128_2PQ_5,
  ICLASS_iclass_POP2X128_2PQ_01,
  ICLASS_iclass_POP2X128_2PQ_03,
  ICLASS_iclass_POP2X128_2PQ_21,
  ICLASS_iclass_POP2X128_2PQ_23,
  ICLASS_iclass_POP32_0,
  ICLASS_iclass_POP32_1,
  ICLASS_iclass_POP32_2,
  ICLASS_iclass_POP32_3,
  ICLASS_iclass_PUSH128,
  ICLASS_iclass_PUSH128_M,
  ICLASS_iclass_PUSH128_PQ,
  ICLASS_iclass_PUSH2X128_PQ,
  ICLASS_iclass_PUSH32,
  ICLASS_iclass_QREADY,
  ICLASS_iclass_RDTIEP,
  ICLASS_iclass_SETTIEP,
  ICLASS_iclass_SMOD_LUT,
  ICLASS_iclass_WRTBSIGQ,
  ICLASS_iclass_WRTBSIGQM,
  ICLASS_iclass_WRTIEP,
  ICLASS_iclass_WRTSIGQ,
  ICLASS_iclass_ABS8,
  ICLASS_iclass_ADD16,
  ICLASS_iclass_ADD32,
  ICLASS_iclass_ADDAC_I2R,
  ICLASS_iclass_ADDAC_R2I,
  ICLASS_iclass_ADDAR2,
  ICLASS_iclass_ADDCM,
  ICLASS_iclass_ADDWRP,
  ICLASS_iclass_AND128,
  ICLASS_iclass_ARGMAX8,
  ICLASS_iclass_ASL,
  ICLASS_iclass_ASL32,
  ICLASS_iclass_ASLACM,
  ICLASS_iclass_ASLM,
  ICLASS_iclass_ASLM32,
  ICLASS_iclass_ASR,
  ICLASS_iclass_ASR32,
  ICLASS_iclass_ASRAC,
  ICLASS_iclass_ASRM,
  ICLASS_iclass_BITFEXT,
  ICLASS_iclass_BITFINS,
  ICLASS_iclass_CLB_C,
  ICLASS_iclass_CLB_R,
  ICLASS_iclass_CMP8,
  ICLASS_iclass_CMP_I,
  ICLASS_iclass_CMP_R,
  ICLASS_iclass_EXT,
  ICLASS_iclass_EXT_R,
  ICLASS_iclass_EXT32_I,
  ICLASS_iclass_EXT32_R,
  ICLASS_iclass_EXTUI4,
  ICLASS_iclass_LSLM,
  ICLASS_iclass_LSRM,
  ICLASS_iclass_MAX8,
  ICLASS_iclass_MEAN,
  ICLASS_iclass_MEAN32,
  ICLASS_iclass_MIN8,
  ICLASS_iclass_MINCLB_C,
  ICLASS_iclass_MINCLB_R,
  ICLASS_iclass_NOT128,
  ICLASS_iclass_OR128,
  ICLASS_iclass_PERM,
  ICLASS_iclass_REDAC,
  ICLASS_iclass_REDAC2,
  ICLASS_iclass_REDAC4,
  ICLASS_iclass_REDACS,
  ICLASS_iclass_SMINCLB_C,
  ICLASS_iclass_SMINCLB_R,
  ICLASS_iclass_STSWAPBM,
  ICLASS_iclass_STSWAPBMU,
  ICLASS_iclass_SUB32,
  ICLASS_iclass_SUBAC_I2R,
  ICLASS_iclass_SUBAC_R2I,
  ICLASS_iclass_SUBARX,
  ICLASS_iclass_SUBCM,
  ICLASS_iclass_SUBMEAN,
  ICLASS_iclass_SUBWRP,
  ICLASS_iclass_TRANS,
  ICLASS_iclass_XOR128,
  ICLASS_rur_expstate,
  ICLASS_wur_expstate,
  ICLASS_rur_sov,
  ICLASS_wur_sov,
  ICLASS_rur_sat_mode,
  ICLASS_wur_sat_mode,
  ICLASS_rur_sar0,
  ICLASS_wur_sar0,
  ICLASS_rur_sar1,
  ICLASS_wur_sar1,
  ICLASS_rur_sar2,
  ICLASS_wur_sar2,
  ICLASS_rur_sar3,
  ICLASS_wur_sar3,
  ICLASS_rur_hsar0,
  ICLASS_wur_hsar0,
  ICLASS_rur_hsar1,
  ICLASS_wur_hsar1,
  ICLASS_rur_hsar2,
  ICLASS_wur_hsar2,
  ICLASS_rur_hsar3,
  ICLASS_wur_hsar3,
  ICLASS_rur_max_reg_0,
  ICLASS_wur_max_reg_0,
  ICLASS_rur_max_reg_1,
  ICLASS_wur_max_reg_1,
  ICLASS_rur_max_reg_2,
  ICLASS_wur_max_reg_2,
  ICLASS_rur_max_reg_3,
  ICLASS_wur_max_reg_3,
  ICLASS_rur_arg_max_reg_0,
  ICLASS_wur_arg_max_reg_0,
  ICLASS_rur_arg_max_reg_1,
  ICLASS_wur_arg_max_reg_1,
  ICLASS_rur_arg_max_reg_2,
  ICLASS_wur_arg_max_reg_2,
  ICLASS_rur_arg_max_reg_3,
  ICLASS_wur_arg_max_reg_3,
  ICLASS_rur_nco_counter_0,
  ICLASS_wur_nco_counter_0,
  ICLASS_rur_nco_counter_1,
  ICLASS_wur_nco_counter_1,
  ICLASS_rur_nco_counter_2,
  ICLASS_wur_nco_counter_2,
  ICLASS_rur_nco_counter_3,
  ICLASS_wur_nco_counter_3,
  ICLASS_rur_interp_ext_n,
  ICLASS_wur_interp_ext_n,
  ICLASS_rur_interp_ext_l,
  ICLASS_wur_interp_ext_l,
  ICLASS_rur_llr_buf_0,
  ICLASS_wur_llr_buf_0,
  ICLASS_rur_llr_buf_1,
  ICLASS_wur_llr_buf_1,
  ICLASS_rur_llr_buf_2,
  ICLASS_wur_llr_buf_2,
  ICLASS_rur_llr_buf_3,
  ICLASS_wur_llr_buf_3,
  ICLASS_rur_llr_buf_4,
  ICLASS_wur_llr_buf_4,
  ICLASS_rur_llr_buf_5,
  ICLASS_wur_llr_buf_5,
  ICLASS_rur_llr_buf_6,
  ICLASS_wur_llr_buf_6,
  ICLASS_rur_llr_buf_7,
  ICLASS_wur_llr_buf_7,
  ICLASS_rur_llr_buf_8,
  ICLASS_wur_llr_buf_8,
  ICLASS_rur_llr_buf_9,
  ICLASS_wur_llr_buf_9,
  ICLASS_rur_llr_buf_10,
  ICLASS_wur_llr_buf_10,
  ICLASS_rur_llr_buf_11,
  ICLASS_wur_llr_buf_11,
  ICLASS_rur_llr_buf_12,
  ICLASS_wur_llr_buf_12,
  ICLASS_rur_llr_buf_13,
  ICLASS_wur_llr_buf_13,
  ICLASS_rur_llr_buf_14,
  ICLASS_wur_llr_buf_14,
  ICLASS_rur_llr_buf_15,
  ICLASS_wur_llr_buf_15,
  ICLASS_rur_llr_buf_16,
  ICLASS_wur_llr_buf_16,
  ICLASS_rur_llr_buf_17,
  ICLASS_wur_llr_buf_17,
  ICLASS_rur_llr_buf_18,
  ICLASS_wur_llr_buf_18,
  ICLASS_rur_llr_buf_19,
  ICLASS_wur_llr_buf_19,
  ICLASS_rur_llr_buf_20,
  ICLASS_wur_llr_buf_20,
  ICLASS_rur_llr_buf_21,
  ICLASS_wur_llr_buf_21,
  ICLASS_rur_llr_buf_22,
  ICLASS_wur_llr_buf_22,
  ICLASS_rur_llr_buf_23,
  ICLASS_wur_llr_buf_23,
  ICLASS_rur_smod_buf_0,
  ICLASS_wur_smod_buf_0,
  ICLASS_rur_smod_buf_1,
  ICLASS_wur_smod_buf_1,
  ICLASS_rur_smod_buf_2,
  ICLASS_wur_smod_buf_2,
  ICLASS_rur_smod_buf_3,
  ICLASS_wur_smod_buf_3,
  ICLASS_rur_smod_buf_4,
  ICLASS_wur_smod_buf_4,
  ICLASS_rur_smod_buf_5,
  ICLASS_wur_smod_buf_5,
  ICLASS_rur_smod_buf_6,
  ICLASS_wur_smod_buf_6,
  ICLASS_rur_smod_buf_7,
  ICLASS_wur_smod_buf_7,
  ICLASS_rur_weight_reg,
  ICLASS_wur_weight_reg,
  ICLASS_rur_scale_reg,
  ICLASS_wur_scale_reg,
  ICLASS_rur_llr_pos,
  ICLASS_wur_llr_pos,
  ICLASS_rur_smod_pos,
  ICLASS_wur_smod_pos,
  ICLASS_rur_perm_reg,
  ICLASS_wur_perm_reg,
  ICLASS_rur_smod_offset_table_0,
  ICLASS_wur_smod_offset_table_0,
  ICLASS_rur_smod_offset_table_1,
  ICLASS_wur_smod_offset_table_1,
  ICLASS_rur_smod_offset_table_2,
  ICLASS_wur_smod_offset_table_2,
  ICLASS_rur_smod_offset_table_3,
  ICLASS_wur_smod_offset_table_3,
  ICLASS_rur_phasor_n,
  ICLASS_wur_phasor_n,
  ICLASS_rur_phasor_offset,
  ICLASS_wur_phasor_offset
};


/*  Opcode encodings.  */

static void
Opcode_excw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2080;
}

static void
Opcode_rfe_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3000;
}

static void
Opcode_rfde_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3200;
}

static void
Opcode_syscall_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5000;
}

static void
Opcode_simcall_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5100;
}

static void
Opcode_call12_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35;
}

static void
Opcode_call8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x25;
}

static void
Opcode_call4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x15;
}

static void
Opcode_callx12_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf0;
}

static void
Opcode_callx8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe0;
}

static void
Opcode_callx4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd0;
}

static void
Opcode_entry_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x36;
}

static void
Opcode_movsp_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1000;
}

static void
Opcode_rotw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x408000;
}

static void
Opcode_retw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x90;
}

static void
Opcode_retw_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf01d;
}

static void
Opcode_rfwo_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3400;
}

static void
Opcode_rfwu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3500;
}

static void
Opcode_l32e_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x90000;
}

static void
Opcode_s32e_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x490000;
}

static void
Opcode_rsr_windowbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34800;
}

static void
Opcode_wsr_windowbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x134800;
}

static void
Opcode_xsr_windowbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x614800;
}

static void
Opcode_rsr_windowstart_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34900;
}

static void
Opcode_wsr_windowstart_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x134900;
}

static void
Opcode_xsr_windowstart_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x614900;
}

static void
Opcode_add_n_Slot_inst16a_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa;
}

static void
Opcode_addi_n_Slot_inst16a_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb;
}

static void
Opcode_addi_n_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_addi_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30001;
}

static void
Opcode_addi_n_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_addi_n_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_addi_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30001;
}

static void
Opcode_addi_n_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_addi_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30000;
}

static void
Opcode_addi_n_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_addi_n_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe0002;
}

static void
Opcode_addi_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3f8001;
}

static void
Opcode_beqz_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8c;
}

static void
Opcode_beqz_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13000;
}

static void
Opcode_beqz_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13000;
}

static void
Opcode_beqz_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4f001;
}

static void
Opcode_beqz_n_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc05001;
}

static void
Opcode_beqz_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d3001;
}

static void
Opcode_bnez_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc;
}

static void
Opcode_bnez_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x23000;
}

static void
Opcode_bnez_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x23000;
}

static void
Opcode_bnez_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x52001;
}

static void
Opcode_bnez_n_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc09001;
}

static void
Opcode_bnez_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e3001;
}

static void
Opcode_ill_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf06d;
}

static void
Opcode_l32i_n_Slot_inst16a_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8;
}

static void
Opcode_l32i_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c001;
}

static void
Opcode_l32i_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c001;
}

static void
Opcode_l32i_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c000;
}

static void
Opcode_l32i_n_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_l32i_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40c000;
}

static void
Opcode_mov_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd;
}

static void
Opcode_mov_n_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ec00;
}

static void
Opcode_mov_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd400;
}

static void
Opcode_mov_n_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd803;
}

static void
Opcode_mov_n_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e000;
}

static void
Opcode_mov_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd400;
}

static void
Opcode_mov_n_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2000;
}

static void
Opcode_mov_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x53401;
}

static void
Opcode_mov_n_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb403;
}

static void
Opcode_mov_n_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x601401;
}

static void
Opcode_mov_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c6c01;
}

static void
Opcode_movi_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc;
}

static void
Opcode_movi_n_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c000;
}

static void
Opcode_movi_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42002;
}

static void
Opcode_movi_n_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_movi_n_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30000;
}

static void
Opcode_movi_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42002;
}

static void
Opcode_movi_n_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_movi_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50001;
}

static void
Opcode_movi_n_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8003;
}

static void
Opcode_movi_n_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x402001;
}

static void
Opcode_movi_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x530000;
}

static void
Opcode_nop_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf03d;
}

static void
Opcode_ret_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf00d;
}

static void
Opcode_s32i_n_Slot_inst16a_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9;
}

static void
Opcode_s32i_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20002;
}

static void
Opcode_s32i_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20002;
}

static void
Opcode_s32i_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_s32i_n_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_s32i_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4a4000;
}

static void
Opcode_rur_threadptr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30e70;
}

static void
Opcode_wur_threadptr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf3e700;
}

static void
Opcode_addi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_addi_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2001;
}

static void
Opcode_addi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1000;
}

static void
Opcode_addmi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd002;
}

static void
Opcode_addmi_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401;
}

static void
Opcode_addmi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2000;
}

static void
Opcode_add_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800000;
}

static void
Opcode_add_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28001;
}

static void
Opcode_add_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28001;
}

static void
Opcode_add_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_add_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28000;
}

static void
Opcode_add_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd0002;
}

static void
Opcode_add_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3f0001;
}

static void
Opcode_sub_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00000;
}

static void
Opcode_sub_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x24000;
}

static void
Opcode_sub_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48002;
}

static void
Opcode_sub_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_sub_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x24000;
}

static void
Opcode_sub_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60002;
}

static void
Opcode_sub_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_sub_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c001;
}

static void
Opcode_sub_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8002;
}

static void
Opcode_sub_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1f0002;
}

static void
Opcode_sub_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4ec000;
}

static void
Opcode_addx2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x900000;
}

static void
Opcode_addx2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c001;
}

static void
Opcode_addx2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c001;
}

static void
Opcode_addx2_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c000;
}

static void
Opcode_addx2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd4002;
}

static void
Opcode_addx2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3fc001;
}

static void
Opcode_addx4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa00000;
}

static void
Opcode_addx4_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34001;
}

static void
Opcode_addx4_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34001;
}

static void
Opcode_addx4_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34000;
}

static void
Opcode_addx4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd8002;
}

static void
Opcode_addx4_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400000;
}

static void
Opcode_addx8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb00000;
}

static void
Opcode_addx8_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38001;
}

static void
Opcode_addx8_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38001;
}

static void
Opcode_addx8_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38000;
}

static void
Opcode_addx8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdc002;
}

static void
Opcode_addx8_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x404000;
}

static void
Opcode_subx2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd00000;
}

static void
Opcode_subx2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50002;
}

static void
Opcode_subx2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c002;
}

static void
Opcode_subx2_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34001;
}

static void
Opcode_subx2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1ec002;
}

static void
Opcode_subx2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4f8000;
}

static void
Opcode_subx4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe00000;
}

static void
Opcode_subx4_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60002;
}

static void
Opcode_subx4_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54002;
}

static void
Opcode_subx4_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38001;
}

static void
Opcode_subx4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1f4002;
}

static void
Opcode_subx4_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4fc000;
}

static void
Opcode_subx8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf00000;
}

static void
Opcode_subx8_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c002;
}

static void
Opcode_subx8_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58002;
}

static void
Opcode_subx8_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c001;
}

static void
Opcode_subx8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1f8002;
}

static void
Opcode_subx8_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x504000;
}

static void
Opcode_and_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100000;
}

static void
Opcode_and_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_and_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c001;
}

static void
Opcode_and_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_and_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_and_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c001;
}

static void
Opcode_and_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c000;
}

static void
Opcode_and_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_and_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe4002;
}

static void
Opcode_and_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x408000;
}

static void
Opcode_or_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200000;
}

static void
Opcode_or_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c000;
}

static void
Opcode_or_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8002;
}

static void
Opcode_or_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_or_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c000;
}

static void
Opcode_or_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8002;
}

static void
Opcode_or_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_or_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4003;
}

static void
Opcode_or_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1d4002;
}

static void
Opcode_or_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x494000;
}

static void
Opcode_xor_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300000;
}

static void
Opcode_xor_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28000;
}

static void
Opcode_xor_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54002;
}

static void
Opcode_xor_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_xor_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28000;
}

static void
Opcode_xor_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c002;
}

static void
Opcode_xor_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44001;
}

static void
Opcode_xor_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_xor_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1fc002;
}

static void
Opcode_xor_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x518000;
}

static void
Opcode_beqi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x26;
}

static void
Opcode_beqi_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40;
}

static void
Opcode_beqi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_bnei_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x66;
}

static void
Opcode_bnei_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800040;
}

static void
Opcode_bnei_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc2;
}

static void
Opcode_bgei_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe6;
}

static void
Opcode_bgei_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_bgei_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42;
}

static void
Opcode_blti_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa6;
}

static void
Opcode_blti_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_blti_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x102;
}

static void
Opcode_bbci_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6007;
}

static void
Opcode_bbci_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x801;
}

static void
Opcode_bbci_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_bbsi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe007;
}

static void
Opcode_bbsi_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1001;
}

static void
Opcode_bbsi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800;
}

static void
Opcode_bgeui_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf6;
}

static void
Opcode_bgeui_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_bgeui_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x82;
}

static void
Opcode_bltui_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb6;
}

static void
Opcode_bltui_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400040;
}

static void
Opcode_bltui_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x202;
}

static void
Opcode_beq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1007;
}

static void
Opcode_beq_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800801;
}

static void
Opcode_beq_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2800;
}

static void
Opcode_bne_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9007;
}

static void
Opcode_bne_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401001;
}

static void
Opcode_bne_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_bge_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa007;
}

static void
Opcode_bge_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400c01;
}

static void
Opcode_bge_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3000;
}

static void
Opcode_blt_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2007;
}

static void
Opcode_blt_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00801;
}

static void
Opcode_blt_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3400;
}

static void
Opcode_bgeu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb007;
}

static void
Opcode_bgeu_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800c01;
}

static void
Opcode_bgeu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c00;
}

static void
Opcode_bltu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3007;
}

static void
Opcode_bltu_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00c01;
}

static void
Opcode_bltu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3800;
}

static void
Opcode_bany_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8007;
}

static void
Opcode_bany_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800401;
}

static void
Opcode_bany_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1800;
}

static void
Opcode_bnone_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7;
}

static void
Opcode_bnone_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x801001;
}

static void
Opcode_bnone_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x403;
}

static void
Opcode_ball_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4007;
}

static void
Opcode_ball_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400401;
}

static void
Opcode_ball_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1400;
}

static void
Opcode_bnall_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc007;
}

static void
Opcode_bnall_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1801;
}

static void
Opcode_bnall_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c00;
}

static void
Opcode_bbc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5007;
}

static void
Opcode_bbc_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00401;
}

static void
Opcode_bbc_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c00;
}

static void
Opcode_bbs_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd007;
}

static void
Opcode_bbs_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400801;
}

static void
Opcode_bbs_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2400;
}

static void
Opcode_beqz_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16;
}

static void
Opcode_beqz_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80002;
}

static void
Opcode_beqz_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80001;
}

static void
Opcode_bnez_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x56;
}

static void
Opcode_bnez_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400002;
}

static void
Opcode_bnez_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0001;
}

static void
Opcode_bgez_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd6;
}

static void
Opcode_bgez_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100002;
}

static void
Opcode_bgez_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100001;
}

static void
Opcode_bltz_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x96;
}

static void
Opcode_bltz_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200002;
}

static void
Opcode_bltz_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200001;
}

static void
Opcode_call0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5;
}

static void
Opcode_callx0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0;
}

static void
Opcode_extui_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_extui_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_extui_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_extui_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_extui_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_extui_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_extui_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_extui_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_ill_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_j_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6;
}

static void
Opcode_jx_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa0;
}

static void
Opcode_l16ui_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1002;
}

static void
Opcode_l16ui_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1003;
}

static void
Opcode_l16si_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9002;
}

static void
Opcode_l16si_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x803;
}

static void
Opcode_l32i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2002;
}

static void
Opcode_l32i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2003;
}

static void
Opcode_l32r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_l32r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400001;
}

static void
Opcode_l8ui_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_l8ui_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc03;
}

static void
Opcode_loop_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8076;
}

static void
Opcode_loopnez_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9076;
}

static void
Opcode_loopgtz_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa076;
}

static void
Opcode_movi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa002;
}

static void
Opcode_movi_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800002;
}

static void
Opcode_movi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c0001;
}

static void
Opcode_moveqz_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x830000;
}

static void
Opcode_moveqz_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20000;
}

static void
Opcode_moveqz_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x74001;
}

static void
Opcode_moveqz_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20000;
}

static void
Opcode_moveqz_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x74001;
}

static void
Opcode_moveqz_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x78000;
}

static void
Opcode_moveqz_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c4002;
}

static void
Opcode_moveqz_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x454000;
}

static void
Opcode_movnez_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x930000;
}

static void
Opcode_movnez_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18000;
}

static void
Opcode_movnez_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_movnez_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18000;
}

static void
Opcode_movnez_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_movnez_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_movnez_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1e0002;
}

static void
Opcode_movnez_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x474000;
}

static void
Opcode_movltz_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa30000;
}

static void
Opcode_movltz_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14000;
}

static void
Opcode_movltz_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c001;
}

static void
Opcode_movltz_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14000;
}

static void
Opcode_movltz_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c001;
}

static void
Opcode_movltz_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_movltz_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1d0002;
}

static void
Opcode_movltz_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x470000;
}

static void
Opcode_movgez_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb30000;
}

static void
Opcode_movgez_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_movgez_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x78001;
}

static void
Opcode_movgez_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_movgez_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x78001;
}

static void
Opcode_movgez_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c000;
}

static void
Opcode_movgez_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c8002;
}

static void
Opcode_movgez_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x464000;
}

static void
Opcode_neg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x600000;
}

static void
Opcode_neg_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34034;
}

static void
Opcode_neg_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6000c;
}

static void
Opcode_neg_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x801f;
}

static void
Opcode_neg_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34020;
}

static void
Opcode_neg_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6000c;
}

static void
Opcode_neg_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_neg_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54035;
}

static void
Opcode_neg_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc01f;
}

static void
Opcode_neg_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14001a;
}

static void
Opcode_neg_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x36000d;
}

static void
Opcode_abs_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x600100;
}

static void
Opcode_abs_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34030;
}

static void
Opcode_abs_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000c;
}

static void
Opcode_abs_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34010;
}

static void
Opcode_abs_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000c;
}

static void
Opcode_abs_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54021;
}

static void
Opcode_abs_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140016;
}

static void
Opcode_abs_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34000d;
}

static void
Opcode_nop_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20f0;
}

static void
Opcode_nop_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f64;
}

static void
Opcode_nop_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc030;
}

static void
Opcode_nop_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35430;
}

static void
Opcode_nop_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xda5d4;
}

static void
Opcode_nop_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa0000;
}

static void
Opcode_nop_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa42f;
}

static void
Opcode_nop_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2eda4;
}

static void
Opcode_nop_Slot_pq_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_nop_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35430;
}

static void
Opcode_nop_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60;
}

static void
Opcode_nop_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_nop_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e40;
}

static void
Opcode_nop_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9add4;
}

static void
Opcode_nop_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x202;
}

static void
Opcode_nop_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5b015;
}

static void
Opcode_nop_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x168dd;
}

static void
Opcode_nop_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400;
}

static void
Opcode_nop_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc237;
}

static void
Opcode_nop_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa50dc;
}

static void
Opcode_nop_Slot_dual_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_nop_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec31;
}

static void
Opcode_ret_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_s16i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5002;
}

static void
Opcode_s16i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1403;
}

static void
Opcode_s32i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6002;
}

static void
Opcode_s32i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1803;
}

static void
Opcode_s8i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_s8i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c03;
}

static void
Opcode_ssr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400000;
}

static void
Opcode_ssr_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d740;
}

static void
Opcode_ssr_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d740;
}

static void
Opcode_ssr_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5df41;
}

static void
Opcode_ssr_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc8c0;
}

static void
Opcode_ssr_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ee801;
}

static void
Opcode_ssl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401000;
}

static void
Opcode_ssl_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d6c0;
}

static void
Opcode_ssl_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d6c0;
}

static void
Opcode_ssl_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dec1;
}

static void
Opcode_ssl_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc4c0;
}

static void
Opcode_ssl_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e6c01;
}

static void
Opcode_ssa8l_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x402000;
}

static void
Opcode_ssa8l_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d700;
}

static void
Opcode_ssa8l_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d700;
}

static void
Opcode_ssa8l_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5df01;
}

static void
Opcode_ssa8l_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc0c0;
}

static void
Opcode_ssa8l_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2debc1;
}

static void
Opcode_ssa8b_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x403000;
}

static void
Opcode_ssa8b_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d680;
}

static void
Opcode_ssa8b_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d680;
}

static void
Opcode_ssa8b_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5de81;
}

static void
Opcode_ssa8b_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe00c0;
}

static void
Opcode_ssa8b_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2deb81;
}

static void
Opcode_ssai_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x404000;
}

static void
Opcode_ssai_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d400;
}

static void
Opcode_ssai_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d400;
}

static void
Opcode_ssai_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dc01;
}

static void
Opcode_ssai_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x480080;
}

static void
Opcode_ssai_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6f81;
}

static void
Opcode_sll_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa10000;
}

static void
Opcode_sll_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c200;
}

static void
Opcode_sll_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c002;
}

static void
Opcode_sll_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_sll_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c000;
}

static void
Opcode_sll_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c002;
}

static void
Opcode_sll_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64001;
}

static void
Opcode_sll_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_sll_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc40c0;
}

static void
Opcode_sll_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40142;
}

static void
Opcode_src_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x810000;
}

static void
Opcode_src_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38002;
}

static void
Opcode_src_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38002;
}

static void
Opcode_src_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28001;
}

static void
Opcode_src_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1e4002;
}

static void
Opcode_src_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e0000;
}

static void
Opcode_srl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x910000;
}

static void
Opcode_srl_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5800c;
}

static void
Opcode_srl_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5800c;
}

static void
Opcode_srl_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58011;
}

static void
Opcode_srl_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x148016;
}

static void
Opcode_srl_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34c00d;
}

static void
Opcode_sra_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb10000;
}

static void
Opcode_sra_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34038;
}

static void
Opcode_sra_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5002c;
}

static void
Opcode_sra_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x802b;
}

static void
Opcode_sra_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34014;
}

static void
Opcode_sra_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5002c;
}

static void
Opcode_sra_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5403d;
}

static void
Opcode_sra_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc02b;
}

static void
Opcode_sra_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x144016;
}

static void
Opcode_sra_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34802d;
}

static void
Opcode_slli_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_slli_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_slli_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10003;
}

static void
Opcode_slli_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_slli_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_slli_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10003;
}

static void
Opcode_slli_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0002;
}

static void
Opcode_slli_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d8001;
}

static void
Opcode_srai_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x210000;
}

static void
Opcode_srai_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18001;
}

static void
Opcode_srai_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18000;
}

static void
Opcode_srai_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18001;
}

static void
Opcode_srai_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18000;
}

static void
Opcode_srai_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18000;
}

static void
Opcode_srai_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc8002;
}

static void
Opcode_srai_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e8001;
}

static void
Opcode_srli_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x410000;
}

static void
Opcode_srli_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c002;
}

static void
Opcode_srli_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c002;
}

static void
Opcode_srli_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30001;
}

static void
Opcode_srli_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1e8002;
}

static void
Opcode_srli_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4cc000;
}

static void
Opcode_memw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20c0;
}

static void
Opcode_extw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20d0;
}

static void
Opcode_isync_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2000;
}

static void
Opcode_rsync_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2010;
}

static void
Opcode_esync_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2020;
}

static void
Opcode_dsync_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2030;
}

static void
Opcode_rsil_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6000;
}

static void
Opcode_rsr_lend_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30100;
}

static void
Opcode_wsr_lend_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130100;
}

static void
Opcode_xsr_lend_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610100;
}

static void
Opcode_rsr_lcount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30200;
}

static void
Opcode_wsr_lcount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130200;
}

static void
Opcode_xsr_lcount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610200;
}

static void
Opcode_rsr_lbeg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30000;
}

static void
Opcode_wsr_lbeg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130000;
}

static void
Opcode_xsr_lbeg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610000;
}

static void
Opcode_rsr_sar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30300;
}

static void
Opcode_wsr_sar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130300;
}

static void
Opcode_xsr_sar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610300;
}

static void
Opcode_rsr_litbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30500;
}

static void
Opcode_wsr_litbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130500;
}

static void
Opcode_xsr_litbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610500;
}

static void
Opcode_rsr_176_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b000;
}

static void
Opcode_wsr_176_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b000;
}

static void
Opcode_rsr_208_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d000;
}

static void
Opcode_rsr_ps_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e600;
}

static void
Opcode_wsr_ps_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e600;
}

static void
Opcode_xsr_ps_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61e600;
}

static void
Opcode_rsr_epc1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b100;
}

static void
Opcode_wsr_epc1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b100;
}

static void
Opcode_xsr_epc1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61b100;
}

static void
Opcode_rsr_excsave1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d100;
}

static void
Opcode_wsr_excsave1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13d100;
}

static void
Opcode_xsr_excsave1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61d100;
}

static void
Opcode_rsr_epc2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b200;
}

static void
Opcode_wsr_epc2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b200;
}

static void
Opcode_xsr_epc2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61b200;
}

static void
Opcode_rsr_excsave2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d200;
}

static void
Opcode_wsr_excsave2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13d200;
}

static void
Opcode_xsr_excsave2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61d200;
}

static void
Opcode_rsr_epc3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b300;
}

static void
Opcode_wsr_epc3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b300;
}

static void
Opcode_xsr_epc3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61b300;
}

static void
Opcode_rsr_excsave3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d300;
}

static void
Opcode_wsr_excsave3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13d300;
}

static void
Opcode_xsr_excsave3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61d300;
}

static void
Opcode_rsr_epc4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b400;
}

static void
Opcode_wsr_epc4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b400;
}

static void
Opcode_xsr_epc4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61b400;
}

static void
Opcode_rsr_excsave4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d400;
}

static void
Opcode_wsr_excsave4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13d400;
}

static void
Opcode_xsr_excsave4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61d400;
}

static void
Opcode_rsr_epc5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b500;
}

static void
Opcode_wsr_epc5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b500;
}

static void
Opcode_xsr_epc5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61b500;
}

static void
Opcode_rsr_excsave5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d500;
}

static void
Opcode_wsr_excsave5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13d500;
}

static void
Opcode_xsr_excsave5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61d500;
}

static void
Opcode_rsr_epc6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b600;
}

static void
Opcode_wsr_epc6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13b600;
}

static void
Opcode_xsr_epc6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61b600;
}

static void
Opcode_rsr_excsave6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d600;
}

static void
Opcode_wsr_excsave6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13d600;
}

static void
Opcode_xsr_excsave6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61d600;
}

static void
Opcode_rsr_eps2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c200;
}

static void
Opcode_wsr_eps2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13c200;
}

static void
Opcode_xsr_eps2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61c200;
}

static void
Opcode_rsr_eps3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c300;
}

static void
Opcode_wsr_eps3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13c300;
}

static void
Opcode_xsr_eps3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61c300;
}

static void
Opcode_rsr_eps4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c400;
}

static void
Opcode_wsr_eps4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13c400;
}

static void
Opcode_xsr_eps4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61c400;
}

static void
Opcode_rsr_eps5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c500;
}

static void
Opcode_wsr_eps5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13c500;
}

static void
Opcode_xsr_eps5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61c500;
}

static void
Opcode_rsr_eps6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c600;
}

static void
Opcode_wsr_eps6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13c600;
}

static void
Opcode_xsr_eps6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61c600;
}

static void
Opcode_rsr_excvaddr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3ee00;
}

static void
Opcode_wsr_excvaddr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13ee00;
}

static void
Opcode_xsr_excvaddr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61ee00;
}

static void
Opcode_rsr_depc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c000;
}

static void
Opcode_wsr_depc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13c000;
}

static void
Opcode_xsr_depc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61c000;
}

static void
Opcode_rsr_exccause_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e800;
}

static void
Opcode_wsr_exccause_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e800;
}

static void
Opcode_xsr_exccause_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61e800;
}

static void
Opcode_rsr_prid_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3eb00;
}

static void
Opcode_rsr_vecbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e700;
}

static void
Opcode_wsr_vecbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e700;
}

static void
Opcode_xsr_vecbase_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61e700;
}

static void
Opcode_mul16u_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc10000;
}

static void
Opcode_mul16s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd10000;
}

static void
Opcode_rfi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3010;
}

static void
Opcode_waiti_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7000;
}

static void
Opcode_rsr_interrupt_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e200;
}

static void
Opcode_wsr_intset_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e200;
}

static void
Opcode_wsr_intclear_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e300;
}

static void
Opcode_rsr_intenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e400;
}

static void
Opcode_wsr_intenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e400;
}

static void
Opcode_xsr_intenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61e400;
}

static void
Opcode_break_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_break_n_Slot_inst16b_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf02d;
}

static void
Opcode_rsr_dbreaka0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39000;
}

static void
Opcode_wsr_dbreaka0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x139000;
}

static void
Opcode_xsr_dbreaka0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x619000;
}

static void
Opcode_rsr_dbreakc0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3a000;
}

static void
Opcode_wsr_dbreakc0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13a000;
}

static void
Opcode_xsr_dbreakc0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61a000;
}

static void
Opcode_rsr_dbreaka1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39100;
}

static void
Opcode_wsr_dbreaka1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x139100;
}

static void
Opcode_xsr_dbreaka1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x619100;
}

static void
Opcode_rsr_dbreakc1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3a100;
}

static void
Opcode_wsr_dbreakc1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13a100;
}

static void
Opcode_xsr_dbreakc1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61a100;
}

static void
Opcode_rsr_ibreaka0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38000;
}

static void
Opcode_wsr_ibreaka0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x138000;
}

static void
Opcode_xsr_ibreaka0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x618000;
}

static void
Opcode_rsr_ibreaka1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38100;
}

static void
Opcode_wsr_ibreaka1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x138100;
}

static void
Opcode_xsr_ibreaka1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x618100;
}

static void
Opcode_rsr_ibreakenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x36000;
}

static void
Opcode_wsr_ibreakenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x136000;
}

static void
Opcode_xsr_ibreakenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x616000;
}

static void
Opcode_rsr_debugcause_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e900;
}

static void
Opcode_wsr_debugcause_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e900;
}

static void
Opcode_xsr_debugcause_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61e900;
}

static void
Opcode_rsr_icount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3ec00;
}

static void
Opcode_wsr_icount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13ec00;
}

static void
Opcode_xsr_icount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61ec00;
}

static void
Opcode_rsr_icountlevel_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3ed00;
}

static void
Opcode_wsr_icountlevel_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13ed00;
}

static void
Opcode_xsr_icountlevel_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61ed00;
}

static void
Opcode_rsr_ddr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x36800;
}

static void
Opcode_wsr_ddr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x136800;
}

static void
Opcode_xsr_ddr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x616800;
}

static void
Opcode_rfdo_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf1e000;
}

static void
Opcode_rfdd_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf1e010;
}

static void
Opcode_wsr_mmid_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x135900;
}

static void
Opcode_andb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20000;
}

static void
Opcode_andb_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48001;
}

static void
Opcode_andb_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48001;
}

static void
Opcode_andb_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48000;
}

static void
Opcode_andb_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe8002;
}

static void
Opcode_andb_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x410000;
}

static void
Opcode_andbc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x120000;
}

static void
Opcode_orb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x220000;
}

static void
Opcode_orb_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_orb_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_orb_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20001;
}

static void
Opcode_orb_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1d8002;
}

static void
Opcode_orb_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x498000;
}

static void
Opcode_orbc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x320000;
}

static void
Opcode_xorb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x420000;
}

static void
Opcode_xorb_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58002;
}

static void
Opcode_xorb_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64002;
}

static void
Opcode_xorb_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48001;
}

static void
Opcode_xorb_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x240002;
}

static void
Opcode_xorb_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x51c000;
}

static void
Opcode_any4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_any4_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35400;
}

static void
Opcode_any4_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35400;
}

static void
Opcode_any4_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58421;
}

static void
Opcode_any4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x481421;
}

static void
Opcode_any4_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2cec21;
}

static void
Opcode_all4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9000;
}

static void
Opcode_any8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa000;
}

static void
Opcode_any8_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d420;
}

static void
Opcode_any8_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d420;
}

static void
Opcode_any8_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58019;
}

static void
Opcode_any8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x481419;
}

static void
Opcode_any8_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2cec31;
}

static void
Opcode_all8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb000;
}

static void
Opcode_bf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x76;
}

static void
Opcode_bt_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1076;
}

static void
Opcode_movf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc30000;
}

static void
Opcode_movt_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd30000;
}

static void
Opcode_movt_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_movt_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_movt_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_movt_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1cc002;
}

static void
Opcode_movt_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x47c000;
}

static void
Opcode_rsr_br_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30400;
}

static void
Opcode_wsr_br_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130400;
}

static void
Opcode_xsr_br_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610400;
}

static void
Opcode_rsr_ccount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3ea00;
}

static void
Opcode_wsr_ccount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13ea00;
}

static void
Opcode_xsr_ccount_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61ea00;
}

static void
Opcode_rsr_ccompare0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3f000;
}

static void
Opcode_wsr_ccompare0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13f000;
}

static void
Opcode_xsr_ccompare0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61f000;
}

static void
Opcode_rsr_ccompare1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3f100;
}

static void
Opcode_wsr_ccompare1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13f100;
}

static void
Opcode_xsr_ccompare1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61f100;
}

static void
Opcode_ipf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70c2;
}

static void
Opcode_ihi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70e2;
}

static void
Opcode_ipfl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70d2;
}

static void
Opcode_ihu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x270d2;
}

static void
Opcode_iiu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x370d2;
}

static void
Opcode_iii_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70f2;
}

static void
Opcode_lict_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf10000;
}

static void
Opcode_licw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf12000;
}

static void
Opcode_sict_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf11000;
}

static void
Opcode_sicw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf13000;
}

static void
Opcode_dhwb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7042;
}

static void
Opcode_dhwbi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7052;
}

static void
Opcode_diwb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x47082;
}

static void
Opcode_diwbi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x57082;
}

static void
Opcode_dhi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7062;
}

static void
Opcode_dii_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7072;
}

static void
Opcode_dpfr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7002;
}

static void
Opcode_dpfw_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7012;
}

static void
Opcode_dpfro_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7022;
}

static void
Opcode_dpfwo_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7032;
}

static void
Opcode_dpfl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7082;
}

static void
Opcode_dhu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x27082;
}

static void
Opcode_diu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x37082;
}

static void
Opcode_sdct_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf19000;
}

static void
Opcode_ldct_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf18000;
}

static void
Opcode_idtlb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50c000;
}

static void
Opcode_pdtlb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50d000;
}

static void
Opcode_rdtlb0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50b000;
}

static void
Opcode_rdtlb1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50f000;
}

static void
Opcode_wdtlb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50e000;
}

static void
Opcode_iitlb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x504000;
}

static void
Opcode_pitlb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x505000;
}

static void
Opcode_ritlb0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x503000;
}

static void
Opcode_ritlb1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x507000;
}

static void
Opcode_witlb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x506000;
}

static void
Opcode_rsr_cpenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e000;
}

static void
Opcode_wsr_cpenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13e000;
}

static void
Opcode_xsr_cpenable_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61e000;
}

static void
Opcode_clamps_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x330000;
}

static void
Opcode_clamps_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50001;
}

static void
Opcode_clamps_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50001;
}

static void
Opcode_clamps_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50000;
}

static void
Opcode_clamps_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf0002;
}

static void
Opcode_clamps_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x440000;
}

static void
Opcode_min_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x430000;
}

static void
Opcode_min_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70001;
}

static void
Opcode_min_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70001;
}

static void
Opcode_min_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c000;
}

static void
Opcode_min_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfc002;
}

static void
Opcode_min_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x460000;
}

static void
Opcode_max_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x530000;
}

static void
Opcode_max_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64001;
}

static void
Opcode_max_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64001;
}

static void
Opcode_max_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68000;
}

static void
Opcode_max_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf4002;
}

static void
Opcode_max_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x448000;
}

static void
Opcode_minu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x630000;
}

static void
Opcode_minu_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c001;
}

static void
Opcode_minu_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c001;
}

static void
Opcode_minu_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x74000;
}

static void
Opcode_minu_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c0002;
}

static void
Opcode_minu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44c000;
}

static void
Opcode_maxu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x730000;
}

static void
Opcode_maxu_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68001;
}

static void
Opcode_maxu_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68001;
}

static void
Opcode_maxu_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70000;
}

static void
Opcode_maxu_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf8002;
}

static void
Opcode_maxu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x450000;
}

static void
Opcode_nsa_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40e000;
}

static void
Opcode_nsa_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x15400;
}

static void
Opcode_nsa_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x15400;
}

static void
Opcode_nsa_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x53801;
}

static void
Opcode_nsa_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c1401;
}

static void
Opcode_nsa_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ce801;
}

static void
Opcode_nsau_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40f000;
}

static void
Opcode_nsau_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x25400;
}

static void
Opcode_nsau_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x25400;
}

static void
Opcode_nsau_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x53c01;
}

static void
Opcode_nsau_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c5401;
}

static void
Opcode_nsau_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6801;
}

static void
Opcode_sext_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x230000;
}

static void
Opcode_sext_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34002;
}

static void
Opcode_sext_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34002;
}

static void
Opcode_sext_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x24001;
}

static void
Opcode_sext_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1dc002;
}

static void
Opcode_sext_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d0000;
}

static void
Opcode_l32ai_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb002;
}

static void
Opcode_s32ri_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf002;
}

static void
Opcode_s32c1i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe002;
}

static void
Opcode_rsr_scompare1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30c00;
}

static void
Opcode_wsr_scompare1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x130c00;
}

static void
Opcode_xsr_scompare1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x610c00;
}

static void
Opcode_rsr_atomctl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x36300;
}

static void
Opcode_wsr_atomctl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x136300;
}

static void
Opcode_xsr_atomctl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x616300;
}

static void
Opcode_rer_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x406000;
}

static void
Opcode_wer_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x407000;
}

static void
Opcode_rur_fcr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30e80;
}

static void
Opcode_wur_fcr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf3e800;
}

static void
Opcode_rur_fsr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30e90;
}

static void
Opcode_wur_fsr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf3e900;
}

static void
Opcode_add_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa0000;
}

static void
Opcode_add_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3f4001;
}

static void
Opcode_sub_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1a0000;
}

static void
Opcode_sub_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4f4000;
}

static void
Opcode_mul_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2a0000;
}

static void
Opcode_mul_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x490000;
}

static void
Opcode_madd_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4a0000;
}

static void
Opcode_madd_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x444000;
}

static void
Opcode_msub_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5a0000;
}

static void
Opcode_msub_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x488000;
}

static void
Opcode_movf_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcb0000;
}

static void
Opcode_movf_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x45c000;
}

static void
Opcode_movt_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdb0000;
}

static void
Opcode_movt_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x484000;
}

static void
Opcode_moveqz_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8b0000;
}

static void
Opcode_moveqz_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x458000;
}

static void
Opcode_movnez_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9b0000;
}

static void
Opcode_movnez_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x478000;
}

static void
Opcode_movltz_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xab0000;
}

static void
Opcode_movltz_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x46c000;
}

static void
Opcode_movgez_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbb0000;
}

static void
Opcode_movgez_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x468000;
}

static void
Opcode_abs_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfa0010;
}

static void
Opcode_abs_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x142;
}

static void
Opcode_mov_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfa0000;
}

static void
Opcode_mov_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4142;
}

static void
Opcode_neg_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfa0060;
}

static void
Opcode_neg_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8142;
}

static void
Opcode_un_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1b0000;
}

static void
Opcode_un_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50c000;
}

static void
Opcode_oeq_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2b0000;
}

static void
Opcode_oeq_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4a0000;
}

static void
Opcode_ueq_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b0000;
}

static void
Opcode_ueq_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x510000;
}

static void
Opcode_olt_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4b0000;
}

static void
Opcode_olt_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48c000;
}

static void
Opcode_ult_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5b0000;
}

static void
Opcode_ult_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x580000;
}

static void
Opcode_ole_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b0000;
}

static void
Opcode_ole_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c0000;
}

static void
Opcode_ule_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7b0000;
}

static void
Opcode_ule_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x540000;
}

static void
Opcode_float_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xca0000;
}

static void
Opcode_float_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x500000;
}

static void
Opcode_ufloat_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xda0000;
}

static void
Opcode_ufloat_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x520000;
}

static void
Opcode_round_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8a0000;
}

static void
Opcode_round_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x49c000;
}

static void
Opcode_ceil_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xba0000;
}

static void
Opcode_ceil_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x420000;
}

static void
Opcode_floor_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xaa0000;
}

static void
Opcode_floor_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x600000;
}

static void
Opcode_trunc_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9a0000;
}

static void
Opcode_trunc_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x508000;
}

static void
Opcode_utrunc_s_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xea0000;
}

static void
Opcode_utrunc_s_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x514000;
}

static void
Opcode_rfr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfa0040;
}

static void
Opcode_rfr_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20142;
}

static void
Opcode_wfr_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfa0050;
}

static void
Opcode_lsi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_lsi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140001;
}

static void
Opcode_lsiu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8003;
}

static void
Opcode_lsiu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180001;
}

static void
Opcode_lsx_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80000;
}

static void
Opcode_lsxu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180000;
}

static void
Opcode_lsxu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x430000;
}

static void
Opcode_ssi_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4003;
}

static void
Opcode_ssi_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x240001;
}

static void
Opcode_ssiu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc003;
}

static void
Opcode_ssiu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x280001;
}

static void
Opcode_ssx_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x480000;
}

static void
Opcode_ssx_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d4000;
}

static void
Opcode_ssxu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x580000;
}

static void
Opcode_ssxu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d8000;
}

static void
Opcode_get_argmax_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b700;
}

static void
Opcode_get_argmax_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3407c;
}

static void
Opcode_get_argmax_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x181d4;
}

static void
Opcode_get_argmax_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3401c;
}

static void
Opcode_get_argmax_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8294;
}

static void
Opcode_get_argmax_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdd;
}

static void
Opcode_get_argmax_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x840ec;
}

static void
Opcode_get_hsar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6ba00;
}

static void
Opcode_get_hsar_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340bc;
}

static void
Opcode_get_hsar_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d434;
}

static void
Opcode_get_hsar_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x281d4;
}

static void
Opcode_get_hsar_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x802f;
}

static void
Opcode_get_hsar_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34058;
}

static void
Opcode_get_hsar_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d434;
}

static void
Opcode_get_hsar_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8314;
}

static void
Opcode_get_hsar_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58415;
}

static void
Opcode_get_hsar_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40d9;
}

static void
Opcode_get_hsar_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc83b;
}

static void
Opcode_get_hsar_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x880ec;
}

static void
Opcode_get_hsar_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec05;
}

static void
Opcode_get_hsar2sar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6bc00;
}

static void
Opcode_get_hsar2sar_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3413c;
}

static void
Opcode_get_hsar2sar_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d438;
}

static void
Opcode_get_hsar2sar_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x481d4;
}

static void
Opcode_get_hsar2sar_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x842f;
}

static void
Opcode_get_hsar2sar_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34098;
}

static void
Opcode_get_hsar2sar_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d438;
}

static void
Opcode_get_hsar2sar_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18254;
}

static void
Opcode_get_hsar2sar_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58815;
}

static void
Opcode_get_hsar2sar_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80d9;
}

static void
Opcode_get_hsar2sar_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd03b;
}

static void
Opcode_get_hsar2sar_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x900ec;
}

static void
Opcode_get_hsar2sar_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec09;
}

static void
Opcode_get_interp_ext_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70b90;
}

static void
Opcode_get_interp_ext_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6806c;
}

static void
Opcode_get_interp_ext_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6806c;
}

static void
Opcode_get_interp_ext_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64101;
}

static void
Opcode_get_interp_ext_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3540ed;
}

static void
Opcode_get_interp_ext_l_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70b80;
}

static void
Opcode_get_interp_ext_l_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6802c;
}

static void
Opcode_get_interp_ext_l_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6802c;
}

static void
Opcode_get_interp_ext_l_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64081;
}

static void
Opcode_get_interp_ext_l_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35406d;
}

static void
Opcode_get_llr_buf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x462000;
}

static void
Opcode_get_llr_buf_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d400;
}

static void
Opcode_get_llr_buf_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d400;
}

static void
Opcode_get_llr_buf_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58021;
}

static void
Opcode_get_llr_buf_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14000;
}

static void
Opcode_get_llr_buf_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2cec01;
}

static void
Opcode_get_llr_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70ba0;
}

static void
Opcode_get_llr_pos_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x680ac;
}

static void
Opcode_get_llr_pos_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x680ac;
}

static void
Opcode_get_llr_pos_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64201;
}

static void
Opcode_get_llr_pos_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc06f;
}

static void
Opcode_get_llr_pos_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35416d;
}

static void
Opcode_get_max_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6bb00;
}

static void
Opcode_get_max_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3423c;
}

static void
Opcode_get_max_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x881d4;
}

static void
Opcode_get_max_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34118;
}

static void
Opcode_get_max_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28254;
}

static void
Opcode_get_max_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100d9;
}

static void
Opcode_get_max_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa00ec;
}

static void
Opcode_get_nco_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6bd00;
}

static void
Opcode_get_nco_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340fc;
}

static void
Opcode_get_nco_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x381d4;
}

static void
Opcode_get_nco_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34218;
}

static void
Opcode_get_nco_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48254;
}

static void
Opcode_get_nco_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200d9;
}

static void
Opcode_get_nco_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8c0ec;
}

static void
Opcode_get_perm_reg_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6812c;
}

static void
Opcode_get_perm_reg_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8037;
}

static void
Opcode_get_perm_reg_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6812c;
}

static void
Opcode_get_perm_reg_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64049;
}

static void
Opcode_get_perm_reg_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0ef;
}

static void
Opcode_get_perm_reg_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35426d;
}

static void
Opcode_get_phasor_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70bc0;
}

static void
Opcode_get_phasor_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6822c;
}

static void
Opcode_get_phasor_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6822c;
}

static void
Opcode_get_phasor_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64051;
}

static void
Opcode_get_phasor_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3541ed;
}

static void
Opcode_get_phasor_offset_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70bb0;
}

static void
Opcode_get_phasor_offset_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x680ec;
}

static void
Opcode_get_phasor_offset_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x680ec;
}

static void
Opcode_get_phasor_offset_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64061;
}

static void
Opcode_get_phasor_offset_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3542ed;
}

static void
Opcode_get_sar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6be00;
}

static void
Opcode_get_sar_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3417c;
}

static void
Opcode_get_sar_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d43c;
}

static void
Opcode_get_sar_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x581d4;
}

static void
Opcode_get_sar_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x882f;
}

static void
Opcode_get_sar_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3405c;
}

static void
Opcode_get_sar_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d43c;
}

static void
Opcode_get_sar_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x88254;
}

static void
Opcode_get_sar_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x59015;
}

static void
Opcode_get_sar_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40dd;
}

static void
Opcode_get_sar_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe03b;
}

static void
Opcode_get_sar_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x940ec;
}

static void
Opcode_get_sar_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec11;
}

static void
Opcode_get_scale_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70bd0;
}

static void
Opcode_get_scale_reg_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6816c;
}

static void
Opcode_get_scale_reg_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6816c;
}

static void
Opcode_get_scale_reg_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6404d;
}

static void
Opcode_get_scale_reg_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc16f;
}

static void
Opcode_get_scale_reg_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35436d;
}

static void
Opcode_get_smod_buf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b200;
}

static void
Opcode_get_smod_buf_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d428;
}

static void
Opcode_get_smod_buf_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d428;
}

static void
Opcode_get_smod_buf_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58419;
}

static void
Opcode_get_smod_buf_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc03b;
}

static void
Opcode_get_smod_buf_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2cec39;
}

static void
Opcode_get_smod_offset_table_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6bf00;
}

static void
Opcode_get_smod_offset_table_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35410;
}

static void
Opcode_get_smod_offset_table_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35410;
}

static void
Opcode_get_smod_offset_table_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5a015;
}

static void
Opcode_get_smod_offset_table_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc43f;
}

static void
Opcode_get_smod_offset_table_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec21;
}

static void
Opcode_get_smod_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70be0;
}

static void
Opcode_get_smod_pos_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x681ac;
}

static void
Opcode_get_smod_pos_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x681ac;
}

static void
Opcode_get_smod_pos_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64055;
}

static void
Opcode_get_smod_pos_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc26f;
}

static void
Opcode_get_smod_pos_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3543ed;
}

static void
Opcode_get_sov_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70bf0;
}

static void
Opcode_get_sov_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x681ec;
}

static void
Opcode_get_sov_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x681ec;
}

static void
Opcode_get_sov_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64059;
}

static void
Opcode_get_sov_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35802d;
}

static void
Opcode_get_wght_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b00;
}

static void
Opcode_get_wght_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6826c;
}

static void
Opcode_get_wght_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6826c;
}

static void
Opcode_get_wght_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6405d;
}

static void
Opcode_get_wght_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc1ef;
}

static void
Opcode_get_wght_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35804d;
}

static void
Opcode_set_argmax_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b30;
}

static void
Opcode_set_argmax_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400d9;
}

static void
Opcode_set_argmax_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8254;
}

static void
Opcode_set_argmax_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400d9;
}

static void
Opcode_set_argmax_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x85d4;
}

static void
Opcode_set_argmax_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11d;
}

static void
Opcode_set_argmax_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x810d0;
}

static void
Opcode_set_ext_regs_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x370b00;
}

static void
Opcode_set_ext_regs_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32400;
}

static void
Opcode_set_ext_regs_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10c01;
}

static void
Opcode_set_ext_regs_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dc00;
}

static void
Opcode_set_ext_regs_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12001;
}

static void
Opcode_set_ext_regs_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe002;
}

static void
Opcode_set_ext_regs_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e1401;
}

static void
Opcode_set_hsar_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400dd;
}

static void
Opcode_set_hsar_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8294;
}

static void
Opcode_set_hsar_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400dd;
}

static void
Opcode_set_hsar_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8dd4;
}

static void
Opcode_set_hsar_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x159;
}

static void
Opcode_set_hsar_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x820d0;
}

static void
Opcode_set_llr_buf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70300;
}

static void
Opcode_set_llr_buf_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c202;
}

static void
Opcode_set_llr_buf_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c202;
}

static void
Opcode_set_llr_buf_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70001;
}

static void
Opcode_set_llr_buf_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc02f;
}

static void
Opcode_set_llr_buf_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35402d;
}

static void
Opcode_set_llr_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1620d0;
}

static void
Opcode_set_llr_pos_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d480;
}

static void
Opcode_set_llr_pos_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d480;
}

static void
Opcode_set_llr_pos_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dc81;
}

static void
Opcode_set_llr_pos_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10200;
}

static void
Opcode_set_llr_pos_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de941;
}

static void
Opcode_set_max_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b50;
}

static void
Opcode_set_max_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x404d9;
}

static void
Opcode_set_max_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8314;
}

static void
Opcode_set_max_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x404d9;
}

static void
Opcode_set_max_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x95d4;
}

static void
Opcode_set_max_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x199;
}

static void
Opcode_set_max_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x810d4;
}

static void
Opcode_set_nco_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b60;
}

static void
Opcode_set_nco_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x408d9;
}

static void
Opcode_set_nco_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8654;
}

static void
Opcode_set_nco_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x408d9;
}

static void
Opcode_set_nco_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa5d4;
}

static void
Opcode_set_nco_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x51d;
}

static void
Opcode_set_nco_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x810d8;
}

static void
Opcode_set_perm_reg_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d500;
}

static void
Opcode_set_perm_reg_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf403;
}

static void
Opcode_set_perm_reg_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d500;
}

static void
Opcode_set_perm_reg_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dd01;
}

static void
Opcode_set_perm_reg_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10040;
}

static void
Opcode_set_perm_reg_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de981;
}

static void
Opcode_set_phasor_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1620e0;
}

static void
Opcode_set_phasor_n_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d600;
}

static void
Opcode_set_phasor_n_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d600;
}

static void
Opcode_set_phasor_n_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5de01;
}

static void
Opcode_set_phasor_n_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de9c1;
}

static void
Opcode_set_phasor_offset_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1630d0;
}

static void
Opcode_set_phasor_offset_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d4c0;
}

static void
Opcode_set_phasor_offset_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d4c0;
}

static void
Opcode_set_phasor_offset_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dcc1;
}

static void
Opcode_set_phasor_offset_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dea41;
}

static void
Opcode_set_sar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b70;
}

static void
Opcode_set_sar_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x410d9;
}

static void
Opcode_set_sar_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x683ac;
}

static void
Opcode_set_sar_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8a54;
}

static void
Opcode_set_sar_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8237;
}

static void
Opcode_set_sar_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x410d9;
}

static void
Opcode_set_sar_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x683ac;
}

static void
Opcode_set_sar_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9dd4;
}

static void
Opcode_set_sar_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64075;
}

static void
Opcode_set_sar_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x91d;
}

static void
Opcode_set_sar_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0b7;
}

static void
Opcode_set_sar_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x814d0;
}

static void
Opcode_set_sar_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3581ad;
}

static void
Opcode_set_scale_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1620f0;
}

static void
Opcode_set_scale_reg_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d540;
}

static void
Opcode_set_scale_reg_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d540;
}

static void
Opcode_set_scale_reg_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dd41;
}

static void
Opcode_set_scale_reg_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10080;
}

static void
Opcode_set_scale_reg_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dea81;
}

static void
Opcode_set_smod_buf_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x370e00;
}

static void
Opcode_set_smod_buf_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c008;
}

static void
Opcode_set_smod_buf_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c008;
}

static void
Opcode_set_smod_buf_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64041;
}

static void
Opcode_set_smod_buf_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc073;
}

static void
Opcode_set_smod_buf_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x350039;
}

static void
Opcode_set_smod_offset_table_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b90;
}

static void
Opcode_set_smod_offset_table_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x683ec;
}

static void
Opcode_set_smod_offset_table_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x683ec;
}

static void
Opcode_set_smod_offset_table_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64079;
}

static void
Opcode_set_smod_offset_table_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc137;
}

static void
Opcode_set_smod_offset_table_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3582ad;
}

static void
Opcode_set_smod_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1630e0;
}

static void
Opcode_set_smod_pos_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d580;
}

static void
Opcode_set_smod_pos_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d580;
}

static void
Opcode_set_smod_pos_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5dd81;
}

static void
Opcode_set_smod_pos_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10440;
}

static void
Opcode_set_smod_pos_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2deb01;
}

static void
Opcode_set_sov_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1630f0;
}

static void
Opcode_set_sov_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d5c0;
}

static void
Opcode_set_sov_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d5c0;
}

static void
Opcode_set_sov_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5ddc1;
}

static void
Opcode_set_sov_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2deac1;
}

static void
Opcode_set_wght_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16a010;
}

static void
Opcode_set_wght_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d640;
}

static void
Opcode_set_wght_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d640;
}

static void
Opcode_set_wght_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5de41;
}

static void
Opcode_set_wght_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10840;
}

static void
Opcode_set_wght_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2deb41;
}

static void
Opcode_lac2x32_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60000;
}

static void
Opcode_lac2x32_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_lac2x32_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_lac2x32_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300001;
}

static void
Opcode_lac2x64_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x82003;
}

static void
Opcode_lac2x64_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c00;
}

static void
Opcode_lac2x64_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c00;
}

static void
Opcode_lac2x64_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0c01;
}

static void
Opcode_lac2x64_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32003;
}

static void
Opcode_lac2x64_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c00;
}

static void
Opcode_lac2x64_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c00;
}

static void
Opcode_lac2x64_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c2801;
}

static void
Opcode_lac2x64_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x132003;
}

static void
Opcode_lac2x64_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1400;
}

static void
Opcode_lac2x64_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1400;
}

static void
Opcode_lac2x64_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c4c01;
}

static void
Opcode_lac2x64_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x52003;
}

static void
Opcode_lac2x64_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1800;
}

static void
Opcode_lac2x64_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1800;
}

static void
Opcode_lac2x64_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c2c01;
}

static void
Opcode_lac32_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x61000;
}

static void
Opcode_lac32_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400;
}

static void
Opcode_lac32_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400;
}

static void
Opcode_lac32_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0001;
}

static void
Opcode_lac_ih_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2003;
}

static void
Opcode_lac_ih_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800;
}

static void
Opcode_lac_ih_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800;
}

static void
Opcode_lac_ih_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0401;
}

static void
Opcode_lac_il_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12003;
}

static void
Opcode_lac_il_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1000;
}

static void
Opcode_lac_il_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1000;
}

static void
Opcode_lac_il_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0801;
}

static void
Opcode_lac_rh_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x22003;
}

static void
Opcode_lac_rh_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00;
}

static void
Opcode_lac_rh_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00;
}

static void
Opcode_lac_rh_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c1001;
}

static void
Opcode_lac_rl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42003;
}

static void
Opcode_lac_rl_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2800;
}

static void
Opcode_lac_rl_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2800;
}

static void
Opcode_lac_rl_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c2401;
}

static void
Opcode_lcm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70300;
}

static void
Opcode_lcm_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40008;
}

static void
Opcode_lcm_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_lcm_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40008;
}

static void
Opcode_lcm_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54000;
}

static void
Opcode_lcm_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_lcm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340011;
}

static void
Opcode_lcm_pinc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70000;
}

static void
Opcode_lcm_pinc_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_lcm_pinc_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_lcm_pinc_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_lcm_pinc_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340001;
}

static void
Opcode_lcm_pinc_x_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x680000;
}

static void
Opcode_lcm_pinc_x_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54001;
}

static void
Opcode_lcm_pinc_x_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54001;
}

static void
Opcode_lcm_pinc_x_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58000;
}

static void
Opcode_lcm_pinc_x_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x414000;
}

static void
Opcode_lcm_u_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70100;
}

static void
Opcode_lcm_u_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40004;
}

static void
Opcode_lcm_u_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_lcm_u_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40004;
}

static void
Opcode_lcm_u_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_lcm_u_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_lcm_u_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x380001;
}

static void
Opcode_lcm_x_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x880000;
}

static void
Opcode_lcm_x_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58001;
}

static void
Opcode_lcm_x_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_lcm_x_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58001;
}

static void
Opcode_lcm_x_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_lcm_x_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c000;
}

static void
Opcode_lcm_x_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_lcm_x_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x418000;
}

static void
Opcode_lcm_xu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa80000;
}

static void
Opcode_lcm_xu_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c001;
}

static void
Opcode_lcm_xu_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_lcm_xu_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c001;
}

static void
Opcode_lcm_xu_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_lcm_xu_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64000;
}

static void
Opcode_lcm_xu_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_lcm_xu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x41c000;
}

static void
Opcode_lp_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70200;
}

static void
Opcode_lp_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340005;
}

static void
Opcode_lp_x_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc80000;
}

static void
Opcode_lp_x_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x424000;
}

static void
Opcode_lq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70400;
}

static void
Opcode_lq_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340009;
}

static void
Opcode_lq_x_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe80000;
}

static void
Opcode_lq_x_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x428000;
}

static void
Opcode_lut0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d0000;
}

static void
Opcode_lut0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42c000;
}

static void
Opcode_lut1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d0000;
}

static void
Opcode_lut1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x434000;
}

static void
Opcode_lut2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d0000;
}

static void
Opcode_lut2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x438000;
}

static void
Opcode_lut3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c0000;
}

static void
Opcode_lut3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x43c000;
}

static void
Opcode_sac2x32_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0000;
}

static void
Opcode_sac2x32_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_sac2x32_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_sac2x32_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c0001;
}

static void
Opcode_sac2x64_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf88000;
}

static void
Opcode_sac2x64_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c002;
}

static void
Opcode_sac2x64_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68002;
}

static void
Opcode_sac2x64_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x52a000;
}

static void
Opcode_sac2x64_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8d8000;
}

static void
Opcode_sac2x64_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5e002;
}

static void
Opcode_sac2x64_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70002;
}

static void
Opcode_sac2x64_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x52c000;
}

static void
Opcode_sac2x64_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9d8000;
}

static void
Opcode_sac2x64_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64002;
}

static void
Opcode_sac2x64_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a002;
}

static void
Opcode_sac2x64_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x52e000;
}

static void
Opcode_sac2x64_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xac8000;
}

static void
Opcode_sac2x64_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68002;
}

static void
Opcode_sac2x64_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c002;
}

static void
Opcode_sac2x64_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x532000;
}

static void
Opcode_sac32_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd0000;
}

static void
Opcode_sac32_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_sac32_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_sac32_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c8001;
}

static void
Opcode_sac_ih_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6d0000;
}

static void
Opcode_sac_ih_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40002;
}

static void
Opcode_sac_ih_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40002;
}

static void
Opcode_sac_ih_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4a8000;
}

static void
Opcode_sac_il_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6d8000;
}

static void
Opcode_sac_il_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18002;
}

static void
Opcode_sac_il_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18002;
}

static void
Opcode_sac_il_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4b0000;
}

static void
Opcode_sac_rh_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8d0000;
}

static void
Opcode_sac_rh_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1a002;
}

static void
Opcode_sac_rh_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1a002;
}

static void
Opcode_sac_rh_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4aa000;
}

static void
Opcode_sac_rl_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xac0000;
}

static void
Opcode_sac_rl_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28002;
}

static void
Opcode_sac_rl_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28002;
}

static void
Opcode_sac_rl_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4b2000;
}

static void
Opcode_scm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5d0000;
}

static void
Opcode_scm_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_scm_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4003;
}

static void
Opcode_scm_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_scm_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_scm_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_scm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4b8000;
}

static void
Opcode_scm_pinc_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0000;
}

static void
Opcode_scm_pinc_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_scm_pinc_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_scm_pinc_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_scm_pinc_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d0001;
}

static void
Opcode_scm_pinc_x_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x780000;
}

static void
Opcode_scm_pinc_x_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14002;
}

static void
Opcode_scm_pinc_x_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14002;
}

static void
Opcode_scm_pinc_x_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14001;
}

static void
Opcode_scm_pinc_x_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4bc000;
}

static void
Opcode_scm_u_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c0000;
}

static void
Opcode_scm_u_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20001;
}

static void
Opcode_scm_u_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_scm_u_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20001;
}

static void
Opcode_scm_u_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20000;
}

static void
Opcode_scm_u_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_scm_u_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3e0001;
}

static void
Opcode_scm_x_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x980000;
}

static void
Opcode_scm_x_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30002;
}

static void
Opcode_scm_x_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_scm_x_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30002;
}

static void
Opcode_scm_x_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4003;
}

static void
Opcode_scm_x_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18001;
}

static void
Opcode_scm_x_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_scm_x_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c4000;
}

static void
Opcode_scm_xu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb80000;
}

static void
Opcode_scm_xu_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x24002;
}

static void
Opcode_scm_xu_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_scm_xu_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x24002;
}

static void
Opcode_scm_xu_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_scm_xu_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c001;
}

static void
Opcode_scm_xu_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_scm_xu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c8000;
}

static void
Opcode_store_p_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c0000;
}

static void
Opcode_store_p_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48002;
}

static void
Opcode_store_p_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4dc000;
}

static void
Opcode_store_q_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc0000;
}

static void
Opcode_store_q_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x50002;
}

static void
Opcode_store_q_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e4000;
}

static void
Opcode_ar2cm_dup_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a000;
}

static void
Opcode_ar2cm_dup_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c080;
}

static void
Opcode_ar2cm_dup_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5400;
}

static void
Opcode_ar2cm_dup_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_ar2cm_dup_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf002;
}

static void
Opcode_ar2cm_dup_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38100;
}

static void
Opcode_ar2cm_dup_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5400;
}

static void
Opcode_ar2cm_dup_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_ar2cm_dup_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x53001;
}

static void
Opcode_ar2cm_dup_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4102;
}

static void
Opcode_ar2cm_dup_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb003;
}

static void
Opcode_ar2cm_dup_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1000c0;
}

static void
Opcode_ar2cm_dup_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c6801;
}

static void
Opcode_ar2cm_ln_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64000;
}

static void
Opcode_ar2cm_ln_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e000;
}

static void
Opcode_ar2cm_ln_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3000;
}

static void
Opcode_ar2cm_ln_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_ar2cm_ln_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe002;
}

static void
Opcode_ar2cm_ln_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c000;
}

static void
Opcode_ar2cm_ln_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3000;
}

static void
Opcode_ar2cm_ln_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_ar2cm_ln_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c001;
}

static void
Opcode_ar2cm_ln_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_ar2cm_ln_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa003;
}

static void
Opcode_ar2cm_ln_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc01001;
}

static void
Opcode_ar2cm_ln_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c3001;
}

static void
Opcode_ar2cm_ln_i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c000;
}

static void
Opcode_ar2cm_ln_i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7000;
}

static void
Opcode_ar2cm_ln_i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7000;
}

static void
Opcode_ar2cm_ln_i_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d001;
}

static void
Opcode_ar2cm_ln_i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c7001;
}

static void
Opcode_ar2cm_ln_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x164000;
}

static void
Opcode_ar2cm_ln_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb000;
}

static void
Opcode_ar2cm_ln_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb000;
}

static void
Opcode_ar2cm_ln_r_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e001;
}

static void
Opcode_ar2cm_ln_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2cb001;
}

static void
Opcode_ar2pq_ln_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x30000;
}

static void
Opcode_ar2pq_ln_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_ar2pq_ln_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_ar2sar_dup_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32e00;
}

static void
Opcode_ar2sar_dup_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12401;
}

static void
Opcode_ar2sar_dup_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed00;
}

static void
Opcode_ar2sar_dup_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11c01;
}

static void
Opcode_ar2sar_dup_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4942;
}

static void
Opcode_ar2sar_dup_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd00c0;
}

static void
Opcode_clrac_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16a080;
}

static void
Opcode_clrac_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d780;
}

static void
Opcode_clrac_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf803;
}

static void
Opcode_clrac_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d780;
}

static void
Opcode_clrac_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2f6801;
}

static void
Opcode_clrcm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b600;
}

static void
Opcode_clrcm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3403c;
}

static void
Opcode_clrcm_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d430;
}

static void
Opcode_clrcm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x81d4;
}

static void
Opcode_clrcm_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8033;
}

static void
Opcode_clrcm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34018;
}

static void
Opcode_clrcm_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d430;
}

static void
Opcode_clrcm_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd;
}

static void
Opcode_clrcm_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2b00;
}

static void
Opcode_clrcm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8254;
}

static void
Opcode_clrcm_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58015;
}

static void
Opcode_clrcm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd9;
}

static void
Opcode_clrcm_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc43b;
}

static void
Opcode_clrcm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800ec;
}

static void
Opcode_clrcm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec01;
}

static void
Opcode_cm2ar_ln_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbd0000;
}

static void
Opcode_cm2ar_ln_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34000;
}

static void
Opcode_cm2ar_ln_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40010;
}

static void
Opcode_cm2ar_ln_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc010;
}

static void
Opcode_cm2ar_ln_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8003;
}

static void
Opcode_cm2ar_ln_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34000;
}

static void
Opcode_cm2ar_ln_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40010;
}

static void
Opcode_cm2ar_ln_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc010;
}

static void
Opcode_cm2ar_ln_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54001;
}

static void
Opcode_cm2ar_ln_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8002;
}

static void
Opcode_cm2ar_ln_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc003;
}

static void
Opcode_cm2ar_ln_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140032;
}

static void
Opcode_cm2ar_ln_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x344011;
}

static void
Opcode_cm2ar_ln_i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbd0400;
}

static void
Opcode_cm2ar_ln_i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40030;
}

static void
Opcode_cm2ar_ln_i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40030;
}

static void
Opcode_cm2ar_ln_i_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58001;
}

static void
Opcode_cm2ar_ln_i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x344031;
}

static void
Opcode_cm2ar_ln_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbd0800;
}

static void
Opcode_cm2ar_ln_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44010;
}

static void
Opcode_cm2ar_ln_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44010;
}

static void
Opcode_cm2ar_ln_r_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54011;
}

static void
Opcode_cm2ar_ln_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34c011;
}

static void
Opcode_comb_ar_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8c0000;
}

static void
Opcode_comb_ar_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60001;
}

static void
Opcode_comb_ar_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60001;
}

static void
Opcode_comb_ar_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60000;
}

static void
Opcode_comb_ar_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x480000;
}

static void
Opcode_conj_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x170300;
}

static void
Opcode_conj_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001d;
}

static void
Opcode_conj_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44008;
}

static void
Opcode_conj_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8054;
}

static void
Opcode_conj_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8013;
}

static void
Opcode_conj_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001d;
}

static void
Opcode_conj_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44008;
}

static void
Opcode_conj_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_conj_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x81;
}

static void
Opcode_conj_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8054;
}

static void
Opcode_conj_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54025;
}

static void
Opcode_conj_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1d;
}

static void
Opcode_conj_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc013;
}

static void
Opcode_conj_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800c4;
}

static void
Opcode_conj_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34002d;
}

static void
Opcode_mov2ac32_i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdd0020;
}

static void
Opcode_mov2ac32_i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c042;
}

static void
Opcode_mov2ac32_i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c042;
}

static void
Opcode_mov2ac32_i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100142;
}

static void
Opcode_mov2ac32_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdd0040;
}

static void
Opcode_mov2ac32_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c082;
}

static void
Opcode_mov2ac32_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c082;
}

static void
Opcode_mov2ac32_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200142;
}

static void
Opcode_mov2cm2pq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x801;
}

static void
Opcode_mov2cm2pq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c0;
}

static void
Opcode_mov2cm2pq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c0;
}

static void
Opcode_movac_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x970b80;
}

static void
Opcode_movac_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a042;
}

static void
Opcode_movac_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6e202;
}

static void
Opcode_movac_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc142;
}

static void
Opcode_movac_i_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_movac_i2r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x978b80;
}

static void
Opcode_movac_r_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2020;
}

static void
Opcode_movac_r2i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70380;
}

static void
Opcode_movar2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c100;
}

static void
Opcode_movar2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80001;
}

static void
Opcode_movar2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38200;
}

static void
Opcode_movar2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80001;
}

static void
Opcode_movar2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc402;
}

static void
Opcode_movar2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00c0;
}

static void
Opcode_movcm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x170b00;
}

static void
Opcode_movcm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40099;
}

static void
Opcode_movcm_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002c;
}

static void
Opcode_movcm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8114;
}

static void
Opcode_movcm_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8023;
}

static void
Opcode_movcm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40099;
}

static void
Opcode_movcm_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002c;
}

static void
Opcode_movcm_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9;
}

static void
Opcode_movcm_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x101;
}

static void
Opcode_movcm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8114;
}

static void
Opcode_movcm_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54029;
}

static void
Opcode_movcm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x99;
}

static void
Opcode_movcm_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc023;
}

static void
Opcode_movcm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800e0;
}

static void
Opcode_movcm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34400d;
}

static void
Opcode_movcm2pq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70600;
}

static void
Opcode_movcm2pq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc02;
}

static void
Opcode_movcm2pq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4181;
}

static void
Opcode_movcm2pq_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_movcm2pq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4181;
}

static void
Opcode_movcm2pq_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x182;
}

static void
Opcode_movcnd_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3001;
}

static void
Opcode_movcnd_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4240;
}

static void
Opcode_movcnd_0_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2801;
}

static void
Opcode_movcnd_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4240;
}

static void
Opcode_movcnd_0_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c3;
}

static void
Opcode_movcnd_0_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x281c01;
}

static void
Opcode_movcnd_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c01;
}

static void
Opcode_movcnd_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4280;
}

static void
Opcode_movcnd_1_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3001;
}

static void
Opcode_movcnd_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4280;
}

static void
Opcode_movcnd_1_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x243;
}

static void
Opcode_movcnd_1_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x301c01;
}

static void
Opcode_movcnd_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3401;
}

static void
Opcode_movcnd_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4300;
}

static void
Opcode_movcnd_2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c01;
}

static void
Opcode_movcnd_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4300;
}

static void
Opcode_movcnd_2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x283;
}

static void
Opcode_movcnd_2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c1c01;
}

static void
Opcode_movcnd_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3801;
}

static void
Opcode_movcnd_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42c0;
}

static void
Opcode_movcnd_3_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3401;
}

static void
Opcode_movcnd_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x42c0;
}

static void
Opcode_movcnd_3_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x303;
}

static void
Opcode_movcnd_3_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x341c01;
}

static void
Opcode_movcnd_4_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c01;
}

static void
Opcode_movcnd_4_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4340;
}

static void
Opcode_movcnd_4_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3801;
}

static void
Opcode_movcnd_4_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4340;
}

static void
Opcode_movcnd_4_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c3;
}

static void
Opcode_movcnd_4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x381c01;
}

static void
Opcode_movcnd_5_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_movcnd_5_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4380;
}

static void
Opcode_movcnd_5_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c01;
}

static void
Opcode_movcnd_5_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4380;
}

static void
Opcode_movcnd_5_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x343;
}

static void
Opcode_movcnd_5_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c1c01;
}

static void
Opcode_movcnd_6_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x402;
}

static void
Opcode_movcnd_6_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x43c0;
}

static void
Opcode_movcnd_6_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_movcnd_6_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x43c0;
}

static void
Opcode_movcnd_6_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x383;
}

static void
Opcode_movcnd_6_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401401;
}

static void
Opcode_movcnd_7_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x802;
}

static void
Opcode_movcnd_7_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_movcnd_7_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x402;
}

static void
Opcode_movcnd_7_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_movcnd_7_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c3;
}

static void
Opcode_movcnd_7_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401801;
}

static void
Opcode_movcnd8_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1001;
}

static void
Opcode_movcnd8_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4040;
}

static void
Opcode_movcnd8_0_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x801;
}

static void
Opcode_movcnd8_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4040;
}

static void
Opcode_movcnd8_0_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_movcnd8_0_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x81c01;
}

static void
Opcode_movcnd8_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2001;
}

static void
Opcode_movcnd8_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4080;
}

static void
Opcode_movcnd8_1_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1001;
}

static void
Opcode_movcnd8_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4080;
}

static void
Opcode_movcnd8_1_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x43;
}

static void
Opcode_movcnd8_1_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x101c01;
}

static void
Opcode_movcnd8_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc01;
}

static void
Opcode_movcnd8_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4100;
}

static void
Opcode_movcnd8_2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2001;
}

static void
Opcode_movcnd8_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4100;
}

static void
Opcode_movcnd8_2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x83;
}

static void
Opcode_movcnd8_2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x201c01;
}

static void
Opcode_movcnd8_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1401;
}

static void
Opcode_movcnd8_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4200;
}

static void
Opcode_movcnd8_3_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc01;
}

static void
Opcode_movcnd8_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4200;
}

static void
Opcode_movcnd8_3_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x103;
}

static void
Opcode_movcnd8_3_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc1c01;
}

static void
Opcode_movcnd8_4_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1801;
}

static void
Opcode_movcnd8_4_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40c0;
}

static void
Opcode_movcnd8_4_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1401;
}

static void
Opcode_movcnd8_4_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40c0;
}

static void
Opcode_movcnd8_4_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x203;
}

static void
Opcode_movcnd8_4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x141c01;
}

static void
Opcode_movcnd8_5_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c01;
}

static void
Opcode_movcnd8_5_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4140;
}

static void
Opcode_movcnd8_5_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1801;
}

static void
Opcode_movcnd8_5_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4140;
}

static void
Opcode_movcnd8_5_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc3;
}

static void
Opcode_movcnd8_5_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x181c01;
}

static void
Opcode_movcnd8_6_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2401;
}

static void
Opcode_movcnd8_6_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4180;
}

static void
Opcode_movcnd8_6_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c01;
}

static void
Opcode_movcnd8_6_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4180;
}

static void
Opcode_movcnd8_6_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x143;
}

static void
Opcode_movcnd8_6_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c1c01;
}

static void
Opcode_movcnd8_7_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2801;
}

static void
Opcode_movcnd8_7_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x41c0;
}

static void
Opcode_movcnd8_7_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2401;
}

static void
Opcode_movcnd8_7_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x41c0;
}

static void
Opcode_movcnd8_7_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x183;
}

static void
Opcode_movcnd8_7_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x241c01;
}

static void
Opcode_mov_i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x370300;
}

static void
Opcode_mov_i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4800c;
}

static void
Opcode_mov_i_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8017;
}

static void
Opcode_mov_i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4800c;
}

static void
Opcode_mov_i_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54031;
}

static void
Opcode_mov_i_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc017;
}

static void
Opcode_mov_i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34800d;
}

static void
Opcode_movpq2pq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2f400;
}

static void
Opcode_movpq2pq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10801;
}

static void
Opcode_movpq2pq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10801;
}

static void
Opcode_mov_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x570300;
}

static void
Opcode_mov_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5000c;
}

static void
Opcode_mov_r_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x801b;
}

static void
Opcode_mov_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5000c;
}

static void
Opcode_mov_r_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5402d;
}

static void
Opcode_mov_r_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc01b;
}

static void
Opcode_mov_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35000d;
}

static void
Opcode_negcm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x970300;
}

static void
Opcode_negcm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40219;
}

static void
Opcode_negcm_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4802c;
}

static void
Opcode_negcm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80d4;
}

static void
Opcode_negcm_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8027;
}

static void
Opcode_negcm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40219;
}

static void
Opcode_negcm_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4802c;
}

static void
Opcode_negcm_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x21;
}

static void
Opcode_negcm_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x201;
}

static void
Opcode_negcm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80d4;
}

static void
Opcode_negcm_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54039;
}

static void
Opcode_negcm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x219;
}

static void
Opcode_negcm_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc027;
}

static void
Opcode_negcm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800e4;
}

static void
Opcode_negcm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34402d;
}

static void
Opcode_pop16llr_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16a180;
}

static void
Opcode_pop16llr_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d7a0;
}

static void
Opcode_pop16llr_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3d7a0;
}

static void
Opcode_pop16llr_1_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11040;
}

static void
Opcode_pop16llr_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e6c41;
}

static void
Opcode_pq2cm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x62000;
}

static void
Opcode_pq2cm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34028;
}

static void
Opcode_pq2cm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x88018;
}

static void
Opcode_pq2cm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38000;
}

static void
Opcode_pq2cm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x88018;
}

static void
Opcode_pq2cm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_pq2cm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x440080;
}

static void
Opcode_swapac_r_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_swapac_ri_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb70b00;
}

static void
Opcode_swapac_ri_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a082;
}

static void
Opcode_swapac_ri_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6e142;
}

static void
Opcode_swapac_ri_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc162;
}

static void
Opcode_swapb_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdd0010;
}

static void
Opcode_swapb_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c0c0;
}

static void
Opcode_swapb_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11801;
}

static void
Opcode_swapb_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38140;
}

static void
Opcode_swapb_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11401;
}

static void
Opcode_swapb_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd402;
}

static void
Opcode_swapb_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc80c0;
}

static void
Opcode_add2ac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_addac_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_cdot_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_cdotac_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_cdotacs_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_cmac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_cmac_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_cmacs_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2000;
}

static void
Opcode_cmpy_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_cmpy_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_cmpy2cm_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_cmpy2cm_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_cmpy2pq_Slot_pq_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_cmpys_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2001;
}

static void
Opcode_cmpyxp2pq_Slot_pq_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_comb32_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_dot_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_dotac_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20000;
}

static void
Opcode_dotacs_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20001;
}

static void
Opcode_lin_int_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_llrpre1_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_llrpre2_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2000;
}

static void
Opcode_mac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_mac_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6001;
}

static void
Opcode_mac8_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2002;
}

static void
Opcode_macd8_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20002;
}

static void
Opcode_macpqxp_0_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20003;
}

static void
Opcode_macpqxp_1_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_macpqxp_2_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_macpqxp_3_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40002;
}

static void
Opcode_macs_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_macxp2_0_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40003;
}

static void
Opcode_macxp2_1_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60000;
}

static void
Opcode_macxp_0_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2003;
}

static void
Opcode_macxp_1_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_macxp_2_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6000;
}

static void
Opcode_macxp_3_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_mov2ac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc010;
}

static void
Opcode_mpy_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6001;
}

static void
Opcode_mpy_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa001;
}

static void
Opcode_mpy2cm_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc003;
}

static void
Opcode_mpy2cm_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_mpy2pq_Slot_pq_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_mpy8_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_mpyadd8_2cm_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_mpyadd8_2cm_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_mpyd8_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60001;
}

static void
Opcode_mpypqxp_0_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60002;
}

static void
Opcode_mpypqxp_1_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60003;
}

static void
Opcode_mpypqxp_2_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80000;
}

static void
Opcode_mpypqxp_3_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80001;
}

static void
Opcode_mpys_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6002;
}

static void
Opcode_mpyxp2pq_Slot_pq_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_mpyxp2_0_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80002;
}

static void
Opcode_mpyxp2_1_Slot_dot_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80003;
}

static void
Opcode_mpyxp_0_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4003;
}

static void
Opcode_mpyxp_1_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6003;
}

static void
Opcode_mpyxp_2_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_mpyxp_3_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa000;
}

static void
Opcode_normacd_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_normacd_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_normacpq_i_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe000;
}

static void
Opcode_normacpq_r_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe040;
}

static void
Opcode_normd_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa001;
}

static void
Opcode_normd_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe001;
}

static void
Opcode_normpypq_i_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe080;
}

static void
Opcode_normpypq_r_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe100;
}

static void
Opcode_rcmac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8002;
}

static void
Opcode_rcmpy_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa002;
}

static void
Opcode_rcmpy2cm_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_rcmpy2cm_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_rfir_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_rfira_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_rfird_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_rfirda_Slot_acc2_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_rmac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8003;
}

static void
Opcode_rmpy_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa003;
}

static void
Opcode_rmpy2cm_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10002;
}

static void
Opcode_rmpy2cm_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_smod_align_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_smod_scr_Slot_smod_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_sub2ac_Slot_gp_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc020;
}

static void
Opcode_wght32_Slot_llr_slot1_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_clrtiep_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b000;
}

static void
Opcode_clrtiep_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32c00;
}

static void
Opcode_clrtiep_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12801;
}

static void
Opcode_clrtiep_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2f000;
}

static void
Opcode_clrtiep_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12801;
}

static void
Opcode_clrtiep_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4202;
}

static void
Opcode_clrtiep_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x280;
}

static void
Opcode_ext_2fifo_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x470600;
}

static void
Opcode_ext_2fifo_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54008;
}

static void
Opcode_ext_2fifo_0_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfc02;
}

static void
Opcode_ext_2fifo_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54008;
}

static void
Opcode_ext_2fifo_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x342019;
}

static void
Opcode_ext_2fifo_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x870600;
}

static void
Opcode_ext_2fifo_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64008;
}

static void
Opcode_ext_2fifo_1_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc003;
}

static void
Opcode_ext_2fifo_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64008;
}

static void
Opcode_ext_2fifo_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x348019;
}

static void
Opcode_ext_2fifo_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x170e00;
}

static void
Opcode_ext_2fifo_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x46028;
}

static void
Opcode_ext_2fifo_2_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc403;
}

static void
Opcode_ext_2fifo_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x46028;
}

static void
Opcode_ext_2fifo_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x350019;
}

static void
Opcode_ext_2fifo_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x178600;
}

static void
Opcode_ext_2fifo_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c028;
}

static void
Opcode_ext_2fifo_3_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc803;
}

static void
Opcode_ext_2fifo_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c028;
}

static void
Opcode_ext_2fifo_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x360019;
}

static void
Opcode_ext_r2fifo_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x178e00;
}

static void
Opcode_ext_r2fifo_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e008;
}

static void
Opcode_ext_r2fifo_0_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd003;
}

static void
Opcode_ext_r2fifo_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e008;
}

static void
Opcode_ext_r2fifo_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x342039;
}

static void
Opcode_ext_r2fifo_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x270e00;
}

static void
Opcode_ext_r2fifo_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e028;
}

static void
Opcode_ext_r2fifo_1_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe003;
}

static void
Opcode_ext_r2fifo_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e028;
}

static void
Opcode_ext_r2fifo_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x348039;
}

static void
Opcode_ext_r2fifo_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x278600;
}

static void
Opcode_ext_r2fifo_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54028;
}

static void
Opcode_ext_r2fifo_2_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc03;
}

static void
Opcode_ext_r2fifo_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x54028;
}

static void
Opcode_ext_r2fifo_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34a019;
}

static void
Opcode_ext_r2fifo_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x370600;
}

static void
Opcode_ext_r2fifo_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x56008;
}

static void
Opcode_ext_r2fifo_3_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd403;
}

static void
Opcode_ext_r2fifo_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x56008;
}

static void
Opcode_ext_r2fifo_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34a039;
}

static void
Opcode_lut_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40180;
}

static void
Opcode_lut_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c0;
}

static void
Opcode_lut_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40180;
}

static void
Opcode_lut_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c0;
}

static void
Opcode_lut_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x240;
}

static void
Opcode_lut_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd80040;
}

static void
Opcode_lut_ar_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_lut_ar_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_lut_ar_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10000;
}

static void
Opcode_lut_ar_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_lut_ar_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_lut_ar_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xec002;
}

static void
Opcode_lut_iext_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401c0;
}

static void
Opcode_lut_iext_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x240;
}

static void
Opcode_lut_iext_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401c0;
}

static void
Opcode_lut_iext_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4002;
}

static void
Opcode_lut_iext_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x240;
}

static void
Opcode_lut_iext_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x280;
}

static void
Opcode_lut_iext_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdc0040;
}

static void
Opcode_lut_phasor_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40059;
}

static void
Opcode_lut_phasor_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8094;
}

static void
Opcode_lut_phasor_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40059;
}

static void
Opcode_lut_phasor_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5;
}

static void
Opcode_lut_phasor_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8094;
}

static void
Opcode_lut_phasor_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x59;
}

static void
Opcode_lut_phasor_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800c8;
}

static void
Opcode_lut_rext_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40240;
}

static void
Opcode_lut_rext_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x280;
}

static void
Opcode_lut_rext_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40240;
}

static void
Opcode_lut_rext_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x280;
}

static void
Opcode_lut_rext_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300;
}

static void
Opcode_lut_rext_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe40040;
}

static void
Opcode_lut_write_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40431;
}

static void
Opcode_lut_write_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_lut_write_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x41031;
}

static void
Opcode_lut_write_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_lut_write_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x131;
}

static void
Opcode_lut_write_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800d0;
}

static void
Opcode_moveq128_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b010;
}

static void
Opcode_moveq128_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f40;
}

static void
Opcode_moveq128_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd81d4;
}

static void
Opcode_moveq128_0_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed80;
}

static void
Opcode_moveq128_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xadd4;
}

static void
Opcode_moveq128_0_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140dd;
}

static void
Opcode_moveq128_0_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x818d0;
}

static void
Opcode_moveq128_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b110;
}

static void
Opcode_moveq128_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f80;
}

static void
Opcode_moveq128_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe81d4;
}

static void
Opcode_moveq128_1_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed84;
}

static void
Opcode_moveq128_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb5d4;
}

static void
Opcode_moveq128_1_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x144dd;
}

static void
Opcode_moveq128_1_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x810dc;
}

static void
Opcode_moveq128_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b210;
}

static void
Opcode_moveq128_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f44;
}

static void
Opcode_moveq128_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd85d4;
}

static void
Opcode_moveq128_2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed88;
}

static void
Opcode_moveq128_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1add4;
}

static void
Opcode_moveq128_2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x148dd;
}

static void
Opcode_moveq128_2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x850dc;
}

static void
Opcode_moveq128_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b410;
}

static void
Opcode_moveq128_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f48;
}

static void
Opcode_moveq128_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd89d4;
}

static void
Opcode_moveq128_3_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed90;
}

static void
Opcode_moveq128_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2add4;
}

static void
Opcode_moveq128_3_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x150dd;
}

static void
Opcode_moveq128_3_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x890dc;
}

static void
Opcode_moveq128_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b810;
}

static void
Opcode_moveq128_4_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f50;
}

static void
Opcode_moveq128_4_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd91d4;
}

static void
Opcode_moveq128_4_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2eda0;
}

static void
Opcode_moveq128_4_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4add4;
}

static void
Opcode_moveq128_4_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x160dd;
}

static void
Opcode_moveq128_4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x910dc;
}

static void
Opcode_moveq128_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b310;
}

static void
Opcode_moveq128_5_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f60;
}

static void
Opcode_moveq128_5_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xda1d4;
}

static void
Opcode_moveq128_5_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2edc0;
}

static void
Opcode_moveq128_5_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8add4;
}

static void
Opcode_moveq128_5_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14cdd;
}

static void
Opcode_moveq128_5_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa10dc;
}

static void
Opcode_moveq32_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b510;
}

static void
Opcode_moveq32_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f4c;
}

static void
Opcode_moveq32_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd8dd4;
}

static void
Opcode_moveq32_0_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed8c;
}

static void
Opcode_moveq32_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3add4;
}

static void
Opcode_moveq32_0_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x154dd;
}

static void
Opcode_moveq32_0_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8d0dc;
}

static void
Opcode_moveq32_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b610;
}

static void
Opcode_moveq32_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f54;
}

static void
Opcode_moveq32_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd95d4;
}

static void
Opcode_moveq32_1_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed94;
}

static void
Opcode_moveq32_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5add4;
}

static void
Opcode_moveq32_1_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x158dd;
}

static void
Opcode_moveq32_1_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x950dc;
}

static void
Opcode_moveq32_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b710;
}

static void
Opcode_moveq32_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f58;
}

static void
Opcode_moveq32_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd99d4;
}

static void
Opcode_moveq32_2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed98;
}

static void
Opcode_moveq32_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6add4;
}

static void
Opcode_moveq32_2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x15cdd;
}

static void
Opcode_moveq32_2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x990dc;
}

static void
Opcode_moveq32_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16b910;
}

static void
Opcode_moveq32_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f5c;
}

static void
Opcode_moveq32_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd9dd4;
}

static void
Opcode_moveq32_3_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed9c;
}

static void
Opcode_moveq32_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7add4;
}

static void
Opcode_moveq32_3_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x164dd;
}

static void
Opcode_moveq32_3_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9d0dc;
}

static void
Opcode_nco_update_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40119;
}

static void
Opcode_nco_update_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8214;
}

static void
Opcode_nco_update_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40119;
}

static void
Opcode_nco_update_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11;
}

static void
Opcode_nco_update_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8214;
}

static void
Opcode_nco_update_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x119;
}

static void
Opcode_nco_update_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800cc;
}

static void
Opcode_pop128_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263000;
}

static void
Opcode_pop128_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x341bc;
}

static void
Opcode_pop128_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35420;
}

static void
Opcode_pop128_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x681d4;
}

static void
Opcode_pop128_0_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x902f;
}

static void
Opcode_pop128_0_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3409c;
}

static void
Opcode_pop128_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35420;
}

static void
Opcode_pop128_0_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40d;
}

static void
Opcode_pop128_0_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2b40;
}

static void
Opcode_pop128_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38254;
}

static void
Opcode_pop128_0_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58c15;
}

static void
Opcode_pop128_0_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80dd;
}

static void
Opcode_pop128_0_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc83f;
}

static void
Opcode_pop128_0_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x980ec;
}

static void
Opcode_pop128_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec0d;
}

static void
Opcode_pop128_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x26a000;
}

static void
Opcode_pop128_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x341fc;
}

static void
Opcode_pop128_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35414;
}

static void
Opcode_pop128_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x781d4;
}

static void
Opcode_pop128_1_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa02f;
}

static void
Opcode_pop128_1_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340d8;
}

static void
Opcode_pop128_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35414;
}

static void
Opcode_pop128_1_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80d;
}

static void
Opcode_pop128_1_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2b80;
}

static void
Opcode_pop128_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58254;
}

static void
Opcode_pop128_1_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x59415;
}

static void
Opcode_pop128_1_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0d9;
}

static void
Opcode_pop128_1_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc3b;
}

static void
Opcode_pop128_1_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9c0ec;
}

static void
Opcode_pop128_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec15;
}

static void
Opcode_pop128_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x362000;
}

static void
Opcode_pop128_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3427c;
}

static void
Opcode_pop128_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35418;
}

static void
Opcode_pop128_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x981d4;
}

static void
Opcode_pop128_2_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8c2f;
}

static void
Opcode_pop128_2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340dc;
}

static void
Opcode_pop128_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35418;
}

static void
Opcode_pop128_2_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100d;
}

static void
Opcode_pop128_2_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2bc0;
}

static void
Opcode_pop128_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68254;
}

static void
Opcode_pop128_2_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x59815;
}

static void
Opcode_pop128_2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0dd;
}

static void
Opcode_pop128_2_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc3f;
}

static void
Opcode_pop128_2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa40ec;
}

static void
Opcode_pop128_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec19;
}

static void
Opcode_pop128_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263100;
}

static void
Opcode_pop128_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x342bc;
}

static void
Opcode_pop128_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3541c;
}

static void
Opcode_pop128_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa81d4;
}

static void
Opcode_pop128_3_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x942f;
}

static void
Opcode_pop128_3_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3411c;
}

static void
Opcode_pop128_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3541c;
}

static void
Opcode_pop128_3_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200d;
}

static void
Opcode_pop128_3_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c80;
}

static void
Opcode_pop128_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x78254;
}

static void
Opcode_pop128_3_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x59c15;
}

static void
Opcode_pop128_3_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100dd;
}

static void
Opcode_pop128_3_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd03f;
}

static void
Opcode_pop128_3_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa80ec;
}

static void
Opcode_pop128_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec1d;
}

static void
Opcode_pop128_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263200;
}

static void
Opcode_pop128_4_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3433c;
}

static void
Opcode_pop128_4_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35424;
}

static void
Opcode_pop128_4_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc81d4;
}

static void
Opcode_pop128_4_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x982f;
}

static void
Opcode_pop128_4_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34158;
}

static void
Opcode_pop128_4_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35424;
}

static void
Opcode_pop128_4_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0d;
}

static void
Opcode_pop128_4_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d00;
}

static void
Opcode_pop128_4_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x98254;
}

static void
Opcode_pop128_4_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5a415;
}

static void
Opcode_pop128_4_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140d9;
}

static void
Opcode_pop128_4_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd43b;
}

static void
Opcode_pop128_4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb00ec;
}

static void
Opcode_pop128_4_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec25;
}

static void
Opcode_pop128_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263400;
}

static void
Opcode_pop128_5_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x342fc;
}

static void
Opcode_pop128_5_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35428;
}

static void
Opcode_pop128_5_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb81d4;
}

static void
Opcode_pop128_5_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9c2f;
}

static void
Opcode_pop128_5_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34198;
}

static void
Opcode_pop128_5_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35428;
}

static void
Opcode_pop128_5_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140d;
}

static void
Opcode_pop128_5_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e00;
}

static void
Opcode_pop128_5_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa8254;
}

static void
Opcode_pop128_5_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5a815;
}

static void
Opcode_pop128_5_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180d9;
}

static void
Opcode_pop128_5_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd83b;
}

static void
Opcode_pop128_5_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xac0ec;
}

static void
Opcode_pop128_5_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dec29;
}

static void
Opcode_pop128_2cmpq_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38000;
}

static void
Opcode_pop128_2cmpq_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8018;
}

static void
Opcode_pop128_2cmpq_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8018;
}

static void
Opcode_pop128_2cmpq_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34010;
}

static void
Opcode_pop128_2cmpq_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18018;
}

static void
Opcode_pop128_2cmpq_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x18018;
}

static void
Opcode_pop128_2cmpq_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34020;
}

static void
Opcode_pop128_2cmpq_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28018;
}

static void
Opcode_pop128_2cmpq_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x28018;
}

static void
Opcode_pop128_2cmpq_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34018;
}

static void
Opcode_pop128_2cmpq_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48018;
}

static void
Opcode_pop128_2cmpq_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x48018;
}

static void
Opcode_pop128_2m_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcd0000;
}

static void
Opcode_pop128_2m_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x66002;
}

static void
Opcode_pop128_2m_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x72002;
}

static void
Opcode_pop128_2m_0_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c001;
}

static void
Opcode_pop128_2m_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300401;
}

static void
Opcode_pop128_2m_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdc0000;
}

static void
Opcode_pop128_2m_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x66402;
}

static void
Opcode_pop128_2m_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x74002;
}

static void
Opcode_pop128_2m_1_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c401;
}

static void
Opcode_pop128_2m_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300801;
}

static void
Opcode_pop128_2m_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xec0000;
}

static void
Opcode_pop128_2m_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x66802;
}

static void
Opcode_pop128_2m_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x78002;
}

static void
Opcode_pop128_2m_2_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5c801;
}

static void
Opcode_pop128_2m_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x302401;
}

static void
Opcode_pop128_2m_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcd1000;
}

static void
Opcode_pop128_2m_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x66c02;
}

static void
Opcode_pop128_2m_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x72402;
}

static void
Opcode_pop128_2m_3_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5e001;
}

static void
Opcode_pop128_2m_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x304401;
}

static void
Opcode_pop128_2pq_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x162080;
}

static void
Opcode_pop128_2pq_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33400;
}

static void
Opcode_pop128_2pq_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38818;
}

static void
Opcode_pop128_2pq_0_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3000;
}

static void
Opcode_pop128_2pq_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38418;
}

static void
Opcode_pop128_2pq_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de801;
}

static void
Opcode_pop128_2pq_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16a000;
}

static void
Opcode_pop128_2pq_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33408;
}

static void
Opcode_pop128_2pq_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39018;
}

static void
Opcode_pop128_2pq_1_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2900;
}

static void
Opcode_pop128_2pq_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38c18;
}

static void
Opcode_pop128_2pq_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6d01;
}

static void
Opcode_pop128_2pq_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x162090;
}

static void
Opcode_pop128_2pq_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33410;
}

static void
Opcode_pop128_2pq_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3a018;
}

static void
Opcode_pop128_2pq_2_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2a00;
}

static void
Opcode_pop128_2pq_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39418;
}

static void
Opcode_pop128_2pq_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6e01;
}

static void
Opcode_pop128_2pq_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1620a0;
}

static void
Opcode_pop128_2pq_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33420;
}

static void
Opcode_pop128_2pq_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38c18;
}

static void
Opcode_pop128_2pq_3_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c00;
}

static void
Opcode_pop128_2pq_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3a418;
}

static void
Opcode_pop128_2pq_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6d81;
}

static void
Opcode_pop128_2pq_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1620c0;
}

static void
Opcode_pop128_2pq_4_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33418;
}

static void
Opcode_pop128_2pq_4_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39418;
}

static void
Opcode_pop128_2pq_4_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2980;
}

static void
Opcode_pop128_2pq_4_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39c18;
}

static void
Opcode_pop128_2pq_4_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6e81;
}

static void
Opcode_pop128_2pq_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1620b0;
}

static void
Opcode_pop128_2pq_5_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33428;
}

static void
Opcode_pop128_2pq_5_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39818;
}

static void
Opcode_pop128_2pq_5_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2a80;
}

static void
Opcode_pop128_2pq_5_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3ac18;
}

static void
Opcode_pop128_2pq_5_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6f01;
}

static void
Opcode_pop2x128_2pq_01_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263800;
}

static void
Opcode_pop2x128_2pq_01_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32d80;
}

static void
Opcode_pop2x128_2pq_01_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12441;
}

static void
Opcode_pop2x128_2pq_01_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2cc0;
}

static void
Opcode_pop2x128_2pq_01_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11c41;
}

static void
Opcode_pop2x128_2pq_01_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de881;
}

static void
Opcode_pop2x128_2pq_03_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263300;
}

static void
Opcode_pop2x128_2pq_03_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32dc0;
}

static void
Opcode_pop2x128_2pq_03_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12481;
}

static void
Opcode_pop2x128_2pq_03_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d40;
}

static void
Opcode_pop2x128_2pq_03_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11c81;
}

static void
Opcode_pop2x128_2pq_03_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de901;
}

static void
Opcode_pop2x128_2pq_21_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263500;
}

static void
Opcode_pop2x128_2pq_21_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32e40;
}

static void
Opcode_pop2x128_2pq_21_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12501;
}

static void
Opcode_pop2x128_2pq_21_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d80;
}

static void
Opcode_pop2x128_2pq_21_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11d01;
}

static void
Opcode_pop2x128_2pq_21_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dea01;
}

static void
Opcode_pop2x128_2pq_23_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x263600;
}

static void
Opcode_pop2x128_2pq_23_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32e80;
}

static void
Opcode_pop2x128_2pq_23_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12601;
}

static void
Opcode_pop2x128_2pq_23_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2dc0;
}

static void
Opcode_pop2x128_2pq_23_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11e01;
}

static void
Opcode_pop2x128_2pq_23_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2de8c1;
}

static void
Opcode_pop32_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf70300;
}

static void
Opcode_pop32_0_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3437c;
}

static void
Opcode_pop32_0_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x682ac;
}

static void
Opcode_pop32_0_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39c18;
}

static void
Opcode_pop32_0_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x803b;
}

static void
Opcode_pop32_0_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3415c;
}

static void
Opcode_pop32_0_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x682ac;
}

static void
Opcode_pop32_0_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_pop32_0_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc1;
}

static void
Opcode_pop32_0_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b418;
}

static void
Opcode_pop32_0_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64065;
}

static void
Opcode_pop32_0_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x29;
}

static void
Opcode_pop32_0_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc2ef;
}

static void
Opcode_pop32_0_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb40ec;
}

static void
Opcode_pop32_0_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35c00d;
}

static void
Opcode_pop32_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b10;
}

static void
Opcode_pop32_1_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x343bc;
}

static void
Opcode_pop32_1_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6832c;
}

static void
Opcode_pop32_1_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x39c1c;
}

static void
Opcode_pop32_1_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8077;
}

static void
Opcode_pop32_1_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3419c;
}

static void
Opcode_pop32_1_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6832c;
}

static void
Opcode_pop32_1_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_pop32_1_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc5;
}

static void
Opcode_pop32_1_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b41c;
}

static void
Opcode_pop32_1_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64069;
}

static void
Opcode_pop32_1_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x425;
}

static void
Opcode_pop32_1_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc36f;
}

static void
Opcode_pop32_1_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb80ec;
}

static void
Opcode_pop32_1_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3580ad;
}

static void
Opcode_pop32_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b20;
}

static void
Opcode_pop32_2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x343fc;
}

static void
Opcode_pop32_2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x682ec;
}

static void
Opcode_pop32_2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3a418;
}

static void
Opcode_pop32_2_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80b7;
}

static void
Opcode_pop32_2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x341d8;
}

static void
Opcode_pop32_2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x682ec;
}

static void
Opcode_pop32_2_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_pop32_2_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc9;
}

static void
Opcode_pop32_2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3bc18;
}

static void
Opcode_pop32_2_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x64071;
}

static void
Opcode_pop32_2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x825;
}

static void
Opcode_pop32_2_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc3ef;
}

static void
Opcode_pop32_2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbc0ec;
}

static void
Opcode_pop32_2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35812d;
}

static void
Opcode_pop32_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b40;
}

static void
Opcode_pop32_3_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38008;
}

static void
Opcode_pop32_3_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6836c;
}

static void
Opcode_pop32_3_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3a818;
}

static void
Opcode_pop32_3_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8137;
}

static void
Opcode_pop32_3_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x341dc;
}

static void
Opcode_pop32_3_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6836c;
}

static void
Opcode_pop32_3_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0;
}

static void
Opcode_pop32_3_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd1;
}

static void
Opcode_pop32_3_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3bc1c;
}

static void
Opcode_pop32_3_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6406d;
}

static void
Opcode_pop32_3_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1025;
}

static void
Opcode_pop32_3_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc037;
}

static void
Opcode_pop32_3_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800f4;
}

static void
Opcode_pop32_3_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35822d;
}

static void
Opcode_push128_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x970b00;
}

static void
Opcode_push128_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd02;
}

static void
Opcode_push128_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6c102;
}

static void
Opcode_push128_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4541;
}

static void
Opcode_push128_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10040;
}

static void
Opcode_push128_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40025;
}

static void
Opcode_push128_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7c102;
}

static void
Opcode_push128_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40;
}

static void
Opcode_push128_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6;
}

static void
Opcode_push128_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x81d4;
}

static void
Opcode_push128_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68001;
}

static void
Opcode_push128_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x25;
}

static void
Opcode_push128_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc033;
}

static void
Opcode_push128_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x481411;
}

static void
Opcode_push128_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35800d;
}

static void
Opcode_push128_m_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf80000;
}

static void
Opcode_push128_m_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x46002;
}

static void
Opcode_push128_m_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x46002;
}

static void
Opcode_push128_m_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x60001;
}

static void
Opcode_push128_m_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x526000;
}

static void
Opcode_push128_pq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x162000;
}

static void
Opcode_push128_pq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2fc00;
}

static void
Opcode_push128_pq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11001;
}

static void
Opcode_push128_pq_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d400;
}

static void
Opcode_push128_pq_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2400;
}

static void
Opcode_push128_pq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38018;
}

static void
Opcode_push128_pq_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc802;
}

static void
Opcode_push128_pq_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb803;
}

static void
Opcode_push128_pq_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c9401;
}

static void
Opcode_push128_pq_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e6801;
}

static void
Opcode_push2x128_pq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x862000;
}

static void
Opcode_push2x128_pq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33800;
}

static void
Opcode_push2x128_pq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38018;
}

static void
Opcode_push2x128_pq_Slot_acc2_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2800;
}

static void
Opcode_push2x128_pq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x58018;
}

static void
Opcode_push2x128_pq_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10100;
}

static void
Opcode_push2x128_pq_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d6c01;
}

static void
Opcode_push32_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdd0000;
}

static void
Opcode_push32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32000;
}

static void
Opcode_push32_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1d400;
}

static void
Opcode_push32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12001;
}

static void
Opcode_push32_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xdc03;
}

static void
Opcode_push32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d800;
}

static void
Opcode_push32_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1d400;
}

static void
Opcode_push32_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2;
}

static void
Opcode_push32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11001;
}

static void
Opcode_push32_Slot_smod_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5cc01;
}

static void
Opcode_push32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd002;
}

static void
Opcode_push32_Slot_llr_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbc03;
}

static void
Opcode_push32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d1401;
}

static void
Opcode_push32_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10142;
}

static void
Opcode_qready_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70e00;
}

static void
Opcode_qready_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c000;
}

static void
Opcode_qready_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_qready_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38080;
}

static void
Opcode_qready_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10001;
}

static void
Opcode_qready_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20002;
}

static void
Opcode_qready_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x240;
}

static void
Opcode_rdtiep_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70b80;
}

static void
Opcode_rdtiep_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x38010;
}

static void
Opcode_rdtiep_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3b018;
}

static void
Opcode_rdtiep_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3421c;
}

static void
Opcode_rdtiep_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x68018;
}

static void
Opcode_rdtiep_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2025;
}

static void
Opcode_rdtiep_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800f8;
}

static void
Opcode_settiep_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b400;
}

static void
Opcode_settiep_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32c80;
}

static void
Opcode_settiep_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13001;
}

static void
Opcode_settiep_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ec00;
}

static void
Opcode_settiep_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x13001;
}

static void
Opcode_settiep_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4142;
}

static void
Opcode_settiep_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300;
}

static void
Opcode_smod_lut_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4041;
}

static void
Opcode_wrtbsigq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16a020;
}

static void
Opcode_wrtbsigq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32f00;
}

static void
Opcode_wrtbsigq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x124c1;
}

static void
Opcode_wrtbsigq_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ee00;
}

static void
Opcode_wrtbsigq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11cc1;
}

static void
Opcode_wrtbsigq_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5142;
}

static void
Opcode_wrtbsigq_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcd0c0;
}

static void
Opcode_wrtbsigqm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6b800;
}

static void
Opcode_wrtbsigqm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32d00;
}

static void
Opcode_wrtbsigqm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12841;
}

static void
Opcode_wrtbsigqm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ec80;
}

static void
Opcode_wrtbsigqm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12841;
}

static void
Opcode_wrtbsigqm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4182;
}

static void
Opcode_wrtbsigqm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x244;
}

static void
Opcode_wrtiep_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x262000;
}

static void
Opcode_wrtiep_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x33000;
}

static void
Opcode_wrtiep_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11c01;
}

static void
Opcode_wrtiep_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e800;
}

static void
Opcode_wrtiep_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11801;
}

static void
Opcode_wrtiep_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd802;
}

static void
Opcode_wrtiep_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4d5401;
}

static void
Opcode_wrtsigq_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x16a040;
}

static void
Opcode_wrtsigq_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32ec0;
}

static void
Opcode_wrtsigq_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x12541;
}

static void
Opcode_wrtsigq_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2ed40;
}

static void
Opcode_wrtsigq_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11d41;
}

static void
Opcode_wrtsigq_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6142;
}

static void
Opcode_wrtsigq_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xce0c0;
}

static void
Opcode_abs8_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40019;
}

static void
Opcode_abs8_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8014;
}

static void
Opcode_abs8_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40019;
}

static void
Opcode_abs8_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8014;
}

static void
Opcode_abs8_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x19;
}

static void
Opcode_abs8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800c0;
}

static void
Opcode_add16_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40031;
}

static void
Opcode_add16_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8030;
}

static void
Opcode_add16_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40031;
}

static void
Opcode_add16_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8030;
}

static void
Opcode_add16_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x31;
}

static void
Opcode_add16_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x481401;
}

static void
Opcode_add32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_add32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_add32_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3;
}

static void
Opcode_add32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_add32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc00040;
}

static void
Opcode_addac_i2r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb78380;
}

static void
Opcode_addac_i2r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a00c;
}

static void
Opcode_addac_i2r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a00c;
}

static void
Opcode_addac_i2r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35832d;
}

static void
Opcode_addac_r2i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb78390;
}

static void
Opcode_addac_r2i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a04c;
}

static void
Opcode_addac_r2i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a04c;
}

static void
Opcode_addac_r2i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3583ad;
}

static void
Opcode_addar2_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e400;
}

static void
Opcode_addar2_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20001;
}

static void
Opcode_addar2_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2d000;
}

static void
Opcode_addar2_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x20001;
}

static void
Opcode_addar2_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc002;
}

static void
Opcode_addar2_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x501401;
}

static void
Opcode_addcm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40040;
}

static void
Opcode_addcm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40;
}

static void
Opcode_addcm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40040;
}

static void
Opcode_addcm_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_addcm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40;
}

static void
Opcode_addcm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40;
}

static void
Opcode_addcm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc40040;
}

static void
Opcode_addwrp_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40080;
}

static void
Opcode_addwrp_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_addwrp_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40080;
}

static void
Opcode_addwrp_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_addwrp_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80;
}

static void
Opcode_addwrp_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc80040;
}

static void
Opcode_and128_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40100;
}

static void
Opcode_and128_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_and128_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40100;
}

static void
Opcode_and128_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_and128_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x100;
}

static void
Opcode_and128_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd00040;
}

static void
Opcode_argmax8_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc82;
}

static void
Opcode_argmax8_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4141;
}

static void
Opcode_argmax8_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2002;
}

static void
Opcode_argmax8_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_argmax8_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4581;
}

static void
Opcode_argmax8_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40140;
}

static void
Opcode_argmax8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2000c0;
}

static void
Opcode_asl_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40200;
}

static void
Opcode_asl_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_asl_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40200;
}

static void
Opcode_asl_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_asl_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x200;
}

static void
Opcode_asl_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe00040;
}

static void
Opcode_asl32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_asl32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_asl32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_asl32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000;
}

static void
Opcode_asl32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0;
}

static void
Opcode_asl32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c01;
}

static void
Opcode_aslacm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xbc8000;
}

static void
Opcode_aslacm_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x70002;
}

static void
Opcode_aslacm_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6e002;
}

static void
Opcode_aslacm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x302;
}

static void
Opcode_aslm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_aslm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_aslm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_aslm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8000;
}

static void
Opcode_aslm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_aslm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140002;
}

static void
Opcode_aslm32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2002;
}

static void
Opcode_aslm32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4301;
}

static void
Opcode_aslm32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1002;
}

static void
Opcode_aslm32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4301;
}

static void
Opcode_aslm32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40300;
}

static void
Opcode_aslm32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400100;
}

static void
Opcode_asr_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400c0;
}

static void
Opcode_asr_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0;
}

static void
Opcode_asr_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400c0;
}

static void
Opcode_asr_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4001;
}

static void
Opcode_asr_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0;
}

static void
Opcode_asr_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140;
}

static void
Opcode_asr_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc0040;
}

static void
Opcode_asr32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401;
}

static void
Opcode_asr32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140;
}

static void
Opcode_asr32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x401;
}

static void
Opcode_asr32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140;
}

static void
Opcode_asr32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180;
}

static void
Opcode_asr32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x41c01;
}

static void
Opcode_asrac_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x570b00;
}

static void
Opcode_asrac_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a002;
}

static void
Opcode_asrac_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6e102;
}

static void
Opcode_asrac_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x80142;
}

static void
Opcode_asrm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40005;
}

static void
Opcode_asrm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8004;
}

static void
Opcode_asrm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40005;
}

static void
Opcode_asrm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8004;
}

static void
Opcode_asrm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5;
}

static void
Opcode_asrm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180002;
}

static void
Opcode_bitfext_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0;
}

static void
Opcode_bitfins_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1;
}

static void
Opcode_clb_c_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x770300;
}

static void
Opcode_clb_c_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5802c;
}

static void
Opcode_clb_c_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe403;
}

static void
Opcode_clb_c_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5802c;
}

static void
Opcode_clb_c_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34c02d;
}

static void
Opcode_clb_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x578b00;
}

static void
Opcode_clb_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5a02c;
}

static void
Opcode_clb_r_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe803;
}

static void
Opcode_clb_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5a02c;
}

static void
Opcode_clb_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x34e02d;
}

static void
Opcode_cmp8_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40009;
}

static void
Opcode_cmp8_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8008;
}

static void
Opcode_cmp8_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40009;
}

static void
Opcode_cmp8_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8008;
}

static void
Opcode_cmp8_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9;
}

static void
Opcode_cmp8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140006;
}

static void
Opcode_cmp_i_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40011;
}

static void
Opcode_cmp_i_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8010;
}

static void
Opcode_cmp_i_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40011;
}

static void
Opcode_cmp_i_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8010;
}

static void
Opcode_cmp_i_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11;
}

static void
Opcode_cmp_i_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14000a;
}

static void
Opcode_cmp_r_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40021;
}

static void
Opcode_cmp_r_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8020;
}

static void
Opcode_cmp_r_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40021;
}

static void
Opcode_cmp_r_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8020;
}

static void
Opcode_cmp_r_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x21;
}

static void
Opcode_cmp_r_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140012;
}

static void
Opcode_ext_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd80000;
}

static void
Opcode_ext_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2a002;
}

static void
Opcode_ext_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8002;
}

static void
Opcode_ext_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2a002;
}

static void
Opcode_ext_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x524000;
}

static void
Opcode_ext_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd88000;
}

static void
Opcode_ext_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e002;
}

static void
Opcode_ext_r_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xa002;
}

static void
Opcode_ext_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e002;
}

static void
Opcode_ext_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x528000;
}

static void
Opcode_ext32_i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x170600;
}

static void
Opcode_ext32_i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44028;
}

static void
Opcode_ext32_i_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf402;
}

static void
Opcode_ext32_i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x44028;
}

static void
Opcode_ext32_i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340019;
}

static void
Opcode_ext32_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x270600;
}

static void
Opcode_ext32_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c008;
}

static void
Opcode_ext32_r_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf802;
}

static void
Opcode_ext32_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4c008;
}

static void
Opcode_ext32_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340039;
}

static void
Opcode_extui4_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40140;
}

static void
Opcode_extui4_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180;
}

static void
Opcode_extui4_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40140;
}

static void
Opcode_extui4_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x180;
}

static void
Opcode_extui4_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1c0;
}

static void
Opcode_extui4_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd40040;
}

static void
Opcode_lslm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000d;
}

static void
Opcode_lslm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_lslm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4000d;
}

static void
Opcode_lslm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc000;
}

static void
Opcode_lslm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd;
}

static void
Opcode_lslm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x140022;
}

static void
Opcode_lsrm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40015;
}

static void
Opcode_lsrm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800c;
}

static void
Opcode_lsrm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40015;
}

static void
Opcode_lsrm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800c;
}

static void
Opcode_lsrm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x15;
}

static void
Opcode_lsrm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x14000e;
}

static void
Opcode_max8_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40280;
}

static void
Opcode_max8_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300;
}

static void
Opcode_max8_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40280;
}

static void
Opcode_max8_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x300;
}

static void
Opcode_max8_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0;
}

static void
Opcode_max8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe80040;
}

static void
Opcode_mean_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40300;
}

static void
Opcode_mean_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0;
}

static void
Opcode_mean_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40300;
}

static void
Opcode_mean_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2c0;
}

static void
Opcode_mean_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340;
}

static void
Opcode_mean_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf00040;
}

static void
Opcode_mean32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x402c0;
}

static void
Opcode_mean32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340;
}

static void
Opcode_mean32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x402c0;
}

static void
Opcode_mean32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x340;
}

static void
Opcode_mean32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x380;
}

static void
Opcode_mean32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xec0040;
}

static void
Opcode_min8_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40340;
}

static void
Opcode_min8_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x380;
}

static void
Opcode_min8_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40340;
}

static void
Opcode_min8_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x380;
}

static void
Opcode_min8_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3c0;
}

static void
Opcode_min8_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf40040;
}

static void
Opcode_minclb_c_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x770b00;
}

static void
Opcode_minclb_c_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6002c;
}

static void
Opcode_minclb_c_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf003;
}

static void
Opcode_minclb_c_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6002c;
}

static void
Opcode_minclb_c_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35002d;
}

static void
Opcode_minclb_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x778300;
}

static void
Opcode_minclb_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6800c;
}

static void
Opcode_minclb_r_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe407;
}

static void
Opcode_minclb_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6800c;
}

static void
Opcode_minclb_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35400d;
}

static void
Opcode_not128_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4005d;
}

static void
Opcode_not128_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8154;
}

static void
Opcode_not128_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4005d;
}

static void
Opcode_not128_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8154;
}

static void
Opcode_not128_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x5d;
}

static void
Opcode_not128_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800e8;
}

static void
Opcode_or128_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40380;
}

static void
Opcode_or128_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_or128_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40380;
}

static void
Opcode_or128_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8001;
}

static void
Opcode_or128_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40000;
}

static void
Opcode_or128_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf80040;
}

static void
Opcode_perm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x1002;
}

static void
Opcode_perm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_perm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x802;
}

static void
Opcode_perm_Slot_acc2_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4003;
}

static void
Opcode_perm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc001;
}

static void
Opcode_perm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40001;
}

static void
Opcode_perm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x441401;
}

static void
Opcode_redac_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb783a0;
}

static void
Opcode_redac_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a08c;
}

static void
Opcode_redac_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a08c;
}

static void
Opcode_redac_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35a32d;
}

static void
Opcode_redac2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb783c0;
}

static void
Opcode_redac2_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a10c;
}

static void
Opcode_redac2_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a10c;
}

static void
Opcode_redac2_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35a3ad;
}

static void
Opcode_redac4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb783b0;
}

static void
Opcode_redac4_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a20c;
}

static void
Opcode_redac4_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a20c;
}

static void
Opcode_redac4_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35806d;
}

static void
Opcode_redacs_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb783d0;
}

static void
Opcode_redacs_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a0cc;
}

static void
Opcode_redacs_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a0cc;
}

static void
Opcode_redacs_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x3580cd;
}

static void
Opcode_sminclb_c_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x778b00;
}

static void
Opcode_sminclb_c_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7000c;
}

static void
Opcode_sminclb_c_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe807;
}

static void
Opcode_sminclb_c_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x7000c;
}

static void
Opcode_sminclb_c_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35202d;
}

static void
Opcode_sminclb_r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xd70300;
}

static void
Opcode_sminclb_r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6202c;
}

static void
Opcode_sminclb_r_Slot_dot_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xec03;
}

static void
Opcode_sminclb_r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6202c;
}

static void
Opcode_sminclb_r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35600d;
}

static void
Opcode_stswapbm_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9c0000;
}

static void
Opcode_stswapbm_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4e8000;
}

static void
Opcode_stswapbmu_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xad0000;
}

static void
Opcode_stswapbmu_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4f0000;
}

static void
Opcode_sub32_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x403c0;
}

static void
Opcode_sub32_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4041;
}

static void
Opcode_sub32_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x403c0;
}

static void
Opcode_sub32_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4081;
}

static void
Opcode_sub32_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40040;
}

static void
Opcode_sub32_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xfc0040;
}

static void
Opcode_subac_i2r_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb783e0;
}

static void
Opcode_subac_i2r_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a14c;
}

static void
Opcode_subac_i2r_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a14c;
}

static void
Opcode_subac_i2r_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35814d;
}

static void
Opcode_subac_r2i_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xb783f0;
}

static void
Opcode_subac_r2i_Slot_gp_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a18c;
}

static void
Opcode_subac_r2i_Slot_pq_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x6a18c;
}

static void
Opcode_subac_r2i_Slot_dual_slot0_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x35824d;
}

static void
Opcode_subarx_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x32800;
}

static void
Opcode_subarx_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x11401;
}

static void
Opcode_subarx_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x2e400;
}

static void
Opcode_subarx_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x10c01;
}

static void
Opcode_subarx_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xcc02;
}

static void
Opcode_subarx_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4cd401;
}

static void
Opcode_subcm_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40002;
}

static void
Opcode_subcm_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4081;
}

static void
Opcode_subcm_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40002;
}

static void
Opcode_subcm_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4101;
}

static void
Opcode_subcm_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40080;
}

static void
Opcode_subcm_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xc0;
}

static void
Opcode_submean_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40042;
}

static void
Opcode_submean_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4101;
}

static void
Opcode_submean_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40042;
}

static void
Opcode_submean_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4201;
}

static void
Opcode_submean_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40100;
}

static void
Opcode_submean_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400080;
}

static void
Opcode_subwrp_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40082;
}

static void
Opcode_subwrp_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4201;
}

static void
Opcode_subwrp_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40082;
}

static void
Opcode_subwrp_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40c1;
}

static void
Opcode_subwrp_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40200;
}

static void
Opcode_subwrp_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800080;
}

static void
Opcode_trans_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4009d;
}

static void
Opcode_trans_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8194;
}

static void
Opcode_trans_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4009d;
}

static void
Opcode_trans_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x8194;
}

static void
Opcode_trans_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x9d;
}

static void
Opcode_trans_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x800f0;
}

static void
Opcode_xor128_Slot_gp_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40102;
}

static void
Opcode_xor128_Slot_dot_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40c1;
}

static void
Opcode_xor128_Slot_pq_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x40102;
}

static void
Opcode_xor128_Slot_smod_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x4141;
}

static void
Opcode_xor128_Slot_llr_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400c0;
}

static void
Opcode_xor128_Slot_dual_slot2_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0x400c0;
}

static void
Opcode_rur_expstate_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30000;
}

static void
Opcode_wur_expstate_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30000;
}

static void
Opcode_rur_sov_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30010;
}

static void
Opcode_wur_sov_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30100;
}

static void
Opcode_rur_sat_mode_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30020;
}

static void
Opcode_wur_sat_mode_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30200;
}

static void
Opcode_rur_sar0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30030;
}

static void
Opcode_wur_sar0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30300;
}

static void
Opcode_rur_sar1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30040;
}

static void
Opcode_wur_sar1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30400;
}

static void
Opcode_rur_sar2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30050;
}

static void
Opcode_wur_sar2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30500;
}

static void
Opcode_rur_sar3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30060;
}

static void
Opcode_wur_sar3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30600;
}

static void
Opcode_rur_hsar0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30070;
}

static void
Opcode_wur_hsar0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30700;
}

static void
Opcode_rur_hsar1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30080;
}

static void
Opcode_wur_hsar1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30800;
}

static void
Opcode_rur_hsar2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30090;
}

static void
Opcode_wur_hsar2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30900;
}

static void
Opcode_rur_hsar3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe300a0;
}

static void
Opcode_wur_hsar3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30a00;
}

static void
Opcode_rur_max_reg_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe300b0;
}

static void
Opcode_wur_max_reg_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30b00;
}

static void
Opcode_rur_max_reg_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe300c0;
}

static void
Opcode_wur_max_reg_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30c00;
}

static void
Opcode_rur_max_reg_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe300d0;
}

static void
Opcode_wur_max_reg_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30d00;
}

static void
Opcode_rur_max_reg_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe300e0;
}

static void
Opcode_wur_max_reg_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30e00;
}

static void
Opcode_rur_arg_max_reg_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe300f0;
}

static void
Opcode_wur_arg_max_reg_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf30f00;
}

static void
Opcode_rur_arg_max_reg_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30100;
}

static void
Opcode_wur_arg_max_reg_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31000;
}

static void
Opcode_rur_arg_max_reg_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30110;
}

static void
Opcode_wur_arg_max_reg_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31100;
}

static void
Opcode_rur_arg_max_reg_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30120;
}

static void
Opcode_wur_arg_max_reg_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31200;
}

static void
Opcode_rur_nco_counter_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30130;
}

static void
Opcode_wur_nco_counter_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31300;
}

static void
Opcode_rur_nco_counter_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30140;
}

static void
Opcode_wur_nco_counter_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31400;
}

static void
Opcode_rur_nco_counter_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30150;
}

static void
Opcode_wur_nco_counter_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31500;
}

static void
Opcode_rur_nco_counter_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30160;
}

static void
Opcode_wur_nco_counter_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31600;
}

static void
Opcode_rur_interp_ext_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30170;
}

static void
Opcode_wur_interp_ext_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31700;
}

static void
Opcode_rur_interp_ext_l_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30180;
}

static void
Opcode_wur_interp_ext_l_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31800;
}

static void
Opcode_rur_llr_buf_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30190;
}

static void
Opcode_wur_llr_buf_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31900;
}

static void
Opcode_rur_llr_buf_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe301a0;
}

static void
Opcode_wur_llr_buf_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31a00;
}

static void
Opcode_rur_llr_buf_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe301b0;
}

static void
Opcode_wur_llr_buf_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31b00;
}

static void
Opcode_rur_llr_buf_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe301c0;
}

static void
Opcode_wur_llr_buf_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31c00;
}

static void
Opcode_rur_llr_buf_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe301d0;
}

static void
Opcode_wur_llr_buf_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31d00;
}

static void
Opcode_rur_llr_buf_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe301e0;
}

static void
Opcode_wur_llr_buf_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31e00;
}

static void
Opcode_rur_llr_buf_6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe301f0;
}

static void
Opcode_wur_llr_buf_6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf31f00;
}

static void
Opcode_rur_llr_buf_7_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30200;
}

static void
Opcode_wur_llr_buf_7_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32000;
}

static void
Opcode_rur_llr_buf_8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30210;
}

static void
Opcode_wur_llr_buf_8_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32100;
}

static void
Opcode_rur_llr_buf_9_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30220;
}

static void
Opcode_wur_llr_buf_9_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32200;
}

static void
Opcode_rur_llr_buf_10_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30230;
}

static void
Opcode_wur_llr_buf_10_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32300;
}

static void
Opcode_rur_llr_buf_11_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30240;
}

static void
Opcode_wur_llr_buf_11_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32400;
}

static void
Opcode_rur_llr_buf_12_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30250;
}

static void
Opcode_wur_llr_buf_12_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32500;
}

static void
Opcode_rur_llr_buf_13_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30260;
}

static void
Opcode_wur_llr_buf_13_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32600;
}

static void
Opcode_rur_llr_buf_14_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30270;
}

static void
Opcode_wur_llr_buf_14_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32700;
}

static void
Opcode_rur_llr_buf_15_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30280;
}

static void
Opcode_wur_llr_buf_15_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32800;
}

static void
Opcode_rur_llr_buf_16_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30290;
}

static void
Opcode_wur_llr_buf_16_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32900;
}

static void
Opcode_rur_llr_buf_17_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe302a0;
}

static void
Opcode_wur_llr_buf_17_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32a00;
}

static void
Opcode_rur_llr_buf_18_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe302b0;
}

static void
Opcode_wur_llr_buf_18_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32b00;
}

static void
Opcode_rur_llr_buf_19_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe302c0;
}

static void
Opcode_wur_llr_buf_19_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32c00;
}

static void
Opcode_rur_llr_buf_20_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe302d0;
}

static void
Opcode_wur_llr_buf_20_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32d00;
}

static void
Opcode_rur_llr_buf_21_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe302e0;
}

static void
Opcode_wur_llr_buf_21_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32e00;
}

static void
Opcode_rur_llr_buf_22_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe302f0;
}

static void
Opcode_wur_llr_buf_22_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf32f00;
}

static void
Opcode_rur_llr_buf_23_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30300;
}

static void
Opcode_wur_llr_buf_23_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33000;
}

static void
Opcode_rur_smod_buf_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30310;
}

static void
Opcode_wur_smod_buf_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33100;
}

static void
Opcode_rur_smod_buf_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30320;
}

static void
Opcode_wur_smod_buf_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33200;
}

static void
Opcode_rur_smod_buf_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30330;
}

static void
Opcode_wur_smod_buf_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33300;
}

static void
Opcode_rur_smod_buf_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30340;
}

static void
Opcode_wur_smod_buf_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33400;
}

static void
Opcode_rur_smod_buf_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30350;
}

static void
Opcode_wur_smod_buf_4_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33500;
}

static void
Opcode_rur_smod_buf_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30360;
}

static void
Opcode_wur_smod_buf_5_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33600;
}

static void
Opcode_rur_smod_buf_6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30370;
}

static void
Opcode_wur_smod_buf_6_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33700;
}

static void
Opcode_rur_smod_buf_7_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30380;
}

static void
Opcode_wur_smod_buf_7_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33800;
}

static void
Opcode_rur_weight_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30390;
}

static void
Opcode_wur_weight_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33900;
}

static void
Opcode_rur_scale_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe303a0;
}

static void
Opcode_wur_scale_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33a00;
}

static void
Opcode_rur_llr_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe303b0;
}

static void
Opcode_wur_llr_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33b00;
}

static void
Opcode_rur_smod_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe303c0;
}

static void
Opcode_wur_smod_pos_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33c00;
}

static void
Opcode_rur_perm_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe303d0;
}

static void
Opcode_wur_perm_reg_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33d00;
}

static void
Opcode_rur_smod_offset_table_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe303e0;
}

static void
Opcode_wur_smod_offset_table_0_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33e00;
}

static void
Opcode_rur_smod_offset_table_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe303f0;
}

static void
Opcode_wur_smod_offset_table_1_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf33f00;
}

static void
Opcode_rur_smod_offset_table_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30400;
}

static void
Opcode_wur_smod_offset_table_2_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf34000;
}

static void
Opcode_rur_smod_offset_table_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30410;
}

static void
Opcode_wur_smod_offset_table_3_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf34100;
}

static void
Opcode_rur_phasor_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30420;
}

static void
Opcode_wur_phasor_n_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf34200;
}

static void
Opcode_rur_phasor_offset_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xe30430;
}

static void
Opcode_wur_phasor_offset_Slot_inst_encode (xtensa_insnbuf slotbuf)
{
  slotbuf[0] = 0xf34300;
}

static xtensa_opcode_encode_fn Opcode_excw_encode_fns[] = {
  Opcode_excw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfe_encode_fns[] = {
  Opcode_rfe_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfde_encode_fns[] = {
  Opcode_rfde_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_syscall_encode_fns[] = {
  Opcode_syscall_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_simcall_encode_fns[] = {
  Opcode_simcall_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_call12_encode_fns[] = {
  Opcode_call12_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_call8_encode_fns[] = {
  Opcode_call8_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_call4_encode_fns[] = {
  Opcode_call4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_callx12_encode_fns[] = {
  Opcode_callx12_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_callx8_encode_fns[] = {
  Opcode_callx8_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_callx4_encode_fns[] = {
  Opcode_callx4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_entry_encode_fns[] = {
  Opcode_entry_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movsp_encode_fns[] = {
  Opcode_movsp_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rotw_encode_fns[] = {
  Opcode_rotw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_retw_encode_fns[] = {
  Opcode_retw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_retw_n_encode_fns[] = {
  0, 0, Opcode_retw_n_Slot_inst16b_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfwo_encode_fns[] = {
  Opcode_rfwo_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfwu_encode_fns[] = {
  Opcode_rfwu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_l32e_encode_fns[] = {
  Opcode_l32e_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_s32e_encode_fns[] = {
  Opcode_s32e_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_windowbase_encode_fns[] = {
  Opcode_rsr_windowbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_windowbase_encode_fns[] = {
  Opcode_wsr_windowbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_windowbase_encode_fns[] = {
  Opcode_xsr_windowbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_windowstart_encode_fns[] = {
  Opcode_rsr_windowstart_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_windowstart_encode_fns[] = {
  Opcode_wsr_windowstart_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_windowstart_encode_fns[] = {
  Opcode_xsr_windowstart_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_add_n_encode_fns[] = {
  0, Opcode_add_n_Slot_inst16a_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_addi_n_encode_fns[] = {
  0, Opcode_addi_n_Slot_inst16a_encode, 0, Opcode_addi_n_Slot_gp_slot2_encode, 0, Opcode_addi_n_Slot_gp_slot0_encode, 0, 0, Opcode_addi_n_Slot_dot_slot0_encode, Opcode_addi_n_Slot_pq_slot2_encode, 0, Opcode_addi_n_Slot_pq_slot0_encode, 0, 0, Opcode_addi_n_Slot_acc2_slot0_encode, 0, 0, Opcode_addi_n_Slot_smod_slot0_encode, 0, 0, Opcode_addi_n_Slot_llr_slot0_encode, Opcode_addi_n_Slot_dual_slot2_encode, 0, Opcode_addi_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_beqz_n_encode_fns[] = {
  0, 0, Opcode_beqz_n_Slot_inst16b_encode, 0, 0, Opcode_beqz_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_beqz_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_beqz_n_Slot_smod_slot0_encode, 0, 0, 0, Opcode_beqz_n_Slot_dual_slot2_encode, 0, Opcode_beqz_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bnez_n_encode_fns[] = {
  0, 0, Opcode_bnez_n_Slot_inst16b_encode, 0, 0, Opcode_bnez_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_bnez_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_bnez_n_Slot_smod_slot0_encode, 0, 0, 0, Opcode_bnez_n_Slot_dual_slot2_encode, 0, Opcode_bnez_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ill_n_encode_fns[] = {
  0, 0, Opcode_ill_n_Slot_inst16b_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_l32i_n_encode_fns[] = {
  0, Opcode_l32i_n_Slot_inst16a_encode, 0, 0, 0, Opcode_l32i_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_l32i_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_l32i_n_Slot_smod_slot0_encode, 0, 0, Opcode_l32i_n_Slot_llr_slot0_encode, 0, 0, Opcode_l32i_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_mov_n_encode_fns[] = {
  0, 0, Opcode_mov_n_Slot_inst16b_encode, Opcode_mov_n_Slot_gp_slot2_encode, 0, Opcode_mov_n_Slot_gp_slot0_encode, 0, 0, Opcode_mov_n_Slot_dot_slot0_encode, Opcode_mov_n_Slot_pq_slot2_encode, 0, Opcode_mov_n_Slot_pq_slot0_encode, 0, 0, Opcode_mov_n_Slot_acc2_slot0_encode, 0, 0, Opcode_mov_n_Slot_smod_slot0_encode, 0, 0, Opcode_mov_n_Slot_llr_slot0_encode, Opcode_mov_n_Slot_dual_slot2_encode, 0, Opcode_mov_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movi_n_encode_fns[] = {
  0, 0, Opcode_movi_n_Slot_inst16b_encode, Opcode_movi_n_Slot_gp_slot2_encode, 0, Opcode_movi_n_Slot_gp_slot0_encode, 0, 0, Opcode_movi_n_Slot_dot_slot0_encode, Opcode_movi_n_Slot_pq_slot2_encode, 0, Opcode_movi_n_Slot_pq_slot0_encode, 0, 0, Opcode_movi_n_Slot_acc2_slot0_encode, 0, 0, Opcode_movi_n_Slot_smod_slot0_encode, 0, 0, Opcode_movi_n_Slot_llr_slot0_encode, Opcode_movi_n_Slot_dual_slot2_encode, 0, Opcode_movi_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_nop_n_encode_fns[] = {
  0, 0, Opcode_nop_n_Slot_inst16b_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ret_n_encode_fns[] = {
  0, 0, Opcode_ret_n_Slot_inst16b_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_s32i_n_encode_fns[] = {
  0, Opcode_s32i_n_Slot_inst16a_encode, 0, 0, 0, Opcode_s32i_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_s32i_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_s32i_n_Slot_smod_slot0_encode, 0, 0, Opcode_s32i_n_Slot_llr_slot0_encode, 0, 0, Opcode_s32i_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_rur_threadptr_encode_fns[] = {
  Opcode_rur_threadptr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_threadptr_encode_fns[] = {
  Opcode_wur_threadptr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_addi_encode_fns[] = {
  Opcode_addi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_addi_Slot_dual_slot2_encode, 0, Opcode_addi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_addmi_encode_fns[] = {
  Opcode_addmi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_addmi_Slot_dual_slot2_encode, 0, Opcode_addmi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_add_encode_fns[] = {
  Opcode_add_Slot_inst_encode, 0, 0, Opcode_add_Slot_gp_slot2_encode, 0, Opcode_add_Slot_gp_slot0_encode, 0, 0, Opcode_add_Slot_dot_slot0_encode, Opcode_add_Slot_pq_slot2_encode, 0, Opcode_add_Slot_pq_slot0_encode, 0, 0, Opcode_add_Slot_acc2_slot0_encode, 0, 0, Opcode_add_Slot_smod_slot0_encode, 0, 0, Opcode_add_Slot_llr_slot0_encode, Opcode_add_Slot_dual_slot2_encode, 0, Opcode_add_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sub_encode_fns[] = {
  Opcode_sub_Slot_inst_encode, 0, 0, Opcode_sub_Slot_gp_slot2_encode, 0, Opcode_sub_Slot_gp_slot0_encode, 0, 0, Opcode_sub_Slot_dot_slot0_encode, Opcode_sub_Slot_pq_slot2_encode, 0, Opcode_sub_Slot_pq_slot0_encode, 0, 0, Opcode_sub_Slot_acc2_slot0_encode, 0, 0, Opcode_sub_Slot_smod_slot0_encode, 0, 0, Opcode_sub_Slot_llr_slot0_encode, Opcode_sub_Slot_dual_slot2_encode, 0, Opcode_sub_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_addx2_encode_fns[] = {
  Opcode_addx2_Slot_inst_encode, 0, 0, 0, 0, Opcode_addx2_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_addx2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_addx2_Slot_smod_slot0_encode, 0, 0, 0, Opcode_addx2_Slot_dual_slot2_encode, 0, Opcode_addx2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_addx4_encode_fns[] = {
  Opcode_addx4_Slot_inst_encode, 0, 0, 0, 0, Opcode_addx4_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_addx4_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_addx4_Slot_smod_slot0_encode, 0, 0, 0, Opcode_addx4_Slot_dual_slot2_encode, 0, Opcode_addx4_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_addx8_encode_fns[] = {
  Opcode_addx8_Slot_inst_encode, 0, 0, 0, 0, Opcode_addx8_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_addx8_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_addx8_Slot_smod_slot0_encode, 0, 0, 0, Opcode_addx8_Slot_dual_slot2_encode, 0, Opcode_addx8_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_subx2_encode_fns[] = {
  Opcode_subx2_Slot_inst_encode, 0, 0, 0, 0, Opcode_subx2_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_subx2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_subx2_Slot_smod_slot0_encode, 0, 0, 0, Opcode_subx2_Slot_dual_slot2_encode, 0, Opcode_subx2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_subx4_encode_fns[] = {
  Opcode_subx4_Slot_inst_encode, 0, 0, 0, 0, Opcode_subx4_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_subx4_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_subx4_Slot_smod_slot0_encode, 0, 0, 0, Opcode_subx4_Slot_dual_slot2_encode, 0, Opcode_subx4_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_subx8_encode_fns[] = {
  Opcode_subx8_Slot_inst_encode, 0, 0, 0, 0, Opcode_subx8_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_subx8_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_subx8_Slot_smod_slot0_encode, 0, 0, 0, Opcode_subx8_Slot_dual_slot2_encode, 0, Opcode_subx8_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_and_encode_fns[] = {
  Opcode_and_Slot_inst_encode, 0, 0, Opcode_and_Slot_gp_slot2_encode, 0, Opcode_and_Slot_gp_slot0_encode, 0, 0, Opcode_and_Slot_dot_slot0_encode, Opcode_and_Slot_pq_slot2_encode, 0, Opcode_and_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_and_Slot_smod_slot0_encode, 0, 0, Opcode_and_Slot_llr_slot0_encode, Opcode_and_Slot_dual_slot2_encode, 0, Opcode_and_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_or_encode_fns[] = {
  Opcode_or_Slot_inst_encode, 0, 0, Opcode_or_Slot_gp_slot2_encode, 0, Opcode_or_Slot_gp_slot0_encode, 0, 0, Opcode_or_Slot_dot_slot0_encode, Opcode_or_Slot_pq_slot2_encode, 0, Opcode_or_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_or_Slot_smod_slot0_encode, 0, 0, Opcode_or_Slot_llr_slot0_encode, Opcode_or_Slot_dual_slot2_encode, 0, Opcode_or_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_xor_encode_fns[] = {
  Opcode_xor_Slot_inst_encode, 0, 0, Opcode_xor_Slot_gp_slot2_encode, 0, Opcode_xor_Slot_gp_slot0_encode, 0, 0, Opcode_xor_Slot_dot_slot0_encode, Opcode_xor_Slot_pq_slot2_encode, 0, Opcode_xor_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_xor_Slot_smod_slot0_encode, 0, 0, Opcode_xor_Slot_llr_slot0_encode, Opcode_xor_Slot_dual_slot2_encode, 0, Opcode_xor_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_beqi_encode_fns[] = {
  Opcode_beqi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_beqi_Slot_dual_slot2_encode, 0, Opcode_beqi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bnei_encode_fns[] = {
  Opcode_bnei_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bnei_Slot_dual_slot2_encode, 0, Opcode_bnei_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bgei_encode_fns[] = {
  Opcode_bgei_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bgei_Slot_dual_slot2_encode, 0, Opcode_bgei_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_blti_encode_fns[] = {
  Opcode_blti_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_blti_Slot_dual_slot2_encode, 0, Opcode_blti_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bbci_encode_fns[] = {
  Opcode_bbci_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bbci_Slot_dual_slot2_encode, 0, Opcode_bbci_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bbsi_encode_fns[] = {
  Opcode_bbsi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bbsi_Slot_dual_slot2_encode, 0, Opcode_bbsi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bgeui_encode_fns[] = {
  Opcode_bgeui_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bgeui_Slot_dual_slot2_encode, 0, Opcode_bgeui_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bltui_encode_fns[] = {
  Opcode_bltui_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bltui_Slot_dual_slot2_encode, 0, Opcode_bltui_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_beq_encode_fns[] = {
  Opcode_beq_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_beq_Slot_dual_slot2_encode, 0, Opcode_beq_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bne_encode_fns[] = {
  Opcode_bne_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bne_Slot_dual_slot2_encode, 0, Opcode_bne_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bge_encode_fns[] = {
  Opcode_bge_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bge_Slot_dual_slot2_encode, 0, Opcode_bge_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_blt_encode_fns[] = {
  Opcode_blt_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_blt_Slot_dual_slot2_encode, 0, Opcode_blt_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bgeu_encode_fns[] = {
  Opcode_bgeu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bgeu_Slot_dual_slot2_encode, 0, Opcode_bgeu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bltu_encode_fns[] = {
  Opcode_bltu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bltu_Slot_dual_slot2_encode, 0, Opcode_bltu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bany_encode_fns[] = {
  Opcode_bany_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bany_Slot_dual_slot2_encode, 0, Opcode_bany_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bnone_encode_fns[] = {
  Opcode_bnone_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bnone_Slot_dual_slot2_encode, 0, Opcode_bnone_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ball_encode_fns[] = {
  Opcode_ball_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ball_Slot_dual_slot2_encode, 0, Opcode_ball_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bnall_encode_fns[] = {
  Opcode_bnall_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bnall_Slot_dual_slot2_encode, 0, Opcode_bnall_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bbc_encode_fns[] = {
  Opcode_bbc_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bbc_Slot_dual_slot2_encode, 0, Opcode_bbc_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bbs_encode_fns[] = {
  Opcode_bbs_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bbs_Slot_dual_slot2_encode, 0, Opcode_bbs_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_beqz_encode_fns[] = {
  Opcode_beqz_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_beqz_Slot_dual_slot2_encode, 0, Opcode_beqz_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bnez_encode_fns[] = {
  Opcode_bnez_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bnez_Slot_dual_slot2_encode, 0, Opcode_bnez_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bgez_encode_fns[] = {
  Opcode_bgez_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bgez_Slot_dual_slot2_encode, 0, Opcode_bgez_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_bltz_encode_fns[] = {
  Opcode_bltz_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bltz_Slot_dual_slot2_encode, 0, Opcode_bltz_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_call0_encode_fns[] = {
  Opcode_call0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_callx0_encode_fns[] = {
  Opcode_callx0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_extui_encode_fns[] = {
  Opcode_extui_Slot_inst_encode, 0, 0, Opcode_extui_Slot_gp_slot2_encode, 0, Opcode_extui_Slot_gp_slot0_encode, 0, 0, 0, Opcode_extui_Slot_pq_slot2_encode, 0, Opcode_extui_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_extui_Slot_smod_slot0_encode, 0, 0, 0, Opcode_extui_Slot_dual_slot2_encode, 0, Opcode_extui_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ill_encode_fns[] = {
  Opcode_ill_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_j_encode_fns[] = {
  Opcode_j_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_jx_encode_fns[] = {
  Opcode_jx_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_l16ui_encode_fns[] = {
  Opcode_l16ui_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_l16ui_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_l16si_encode_fns[] = {
  Opcode_l16si_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_l16si_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_l32i_encode_fns[] = {
  Opcode_l32i_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_l32i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_l32r_encode_fns[] = {
  Opcode_l32r_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_l32r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_l8ui_encode_fns[] = {
  Opcode_l8ui_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_l8ui_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_loop_encode_fns[] = {
  Opcode_loop_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_loopnez_encode_fns[] = {
  Opcode_loopnez_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_loopgtz_encode_fns[] = {
  Opcode_loopgtz_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movi_encode_fns[] = {
  Opcode_movi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movi_Slot_dual_slot2_encode, 0, Opcode_movi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_moveqz_encode_fns[] = {
  Opcode_moveqz_Slot_inst_encode, 0, 0, Opcode_moveqz_Slot_gp_slot2_encode, 0, Opcode_moveqz_Slot_gp_slot0_encode, 0, 0, 0, Opcode_moveqz_Slot_pq_slot2_encode, 0, Opcode_moveqz_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_moveqz_Slot_smod_slot0_encode, 0, 0, 0, Opcode_moveqz_Slot_dual_slot2_encode, 0, Opcode_moveqz_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movnez_encode_fns[] = {
  Opcode_movnez_Slot_inst_encode, 0, 0, Opcode_movnez_Slot_gp_slot2_encode, 0, Opcode_movnez_Slot_gp_slot0_encode, 0, 0, 0, Opcode_movnez_Slot_pq_slot2_encode, 0, Opcode_movnez_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_movnez_Slot_smod_slot0_encode, 0, 0, 0, Opcode_movnez_Slot_dual_slot2_encode, 0, Opcode_movnez_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movltz_encode_fns[] = {
  Opcode_movltz_Slot_inst_encode, 0, 0, Opcode_movltz_Slot_gp_slot2_encode, 0, Opcode_movltz_Slot_gp_slot0_encode, 0, 0, 0, Opcode_movltz_Slot_pq_slot2_encode, 0, Opcode_movltz_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_movltz_Slot_smod_slot0_encode, 0, 0, 0, Opcode_movltz_Slot_dual_slot2_encode, 0, Opcode_movltz_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movgez_encode_fns[] = {
  Opcode_movgez_Slot_inst_encode, 0, 0, Opcode_movgez_Slot_gp_slot2_encode, 0, Opcode_movgez_Slot_gp_slot0_encode, 0, 0, 0, Opcode_movgez_Slot_pq_slot2_encode, 0, Opcode_movgez_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_movgez_Slot_smod_slot0_encode, 0, 0, 0, Opcode_movgez_Slot_dual_slot2_encode, 0, Opcode_movgez_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_neg_encode_fns[] = {
  Opcode_neg_Slot_inst_encode, 0, 0, Opcode_neg_Slot_gp_slot2_encode, 0, Opcode_neg_Slot_gp_slot0_encode, 0, 0, Opcode_neg_Slot_dot_slot0_encode, Opcode_neg_Slot_pq_slot2_encode, 0, Opcode_neg_Slot_pq_slot0_encode, 0, 0, Opcode_neg_Slot_acc2_slot0_encode, 0, 0, Opcode_neg_Slot_smod_slot0_encode, 0, 0, Opcode_neg_Slot_llr_slot0_encode, Opcode_neg_Slot_dual_slot2_encode, 0, Opcode_neg_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_abs_encode_fns[] = {
  Opcode_abs_Slot_inst_encode, 0, 0, Opcode_abs_Slot_gp_slot2_encode, 0, Opcode_abs_Slot_gp_slot0_encode, 0, 0, 0, Opcode_abs_Slot_pq_slot2_encode, 0, Opcode_abs_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_abs_Slot_smod_slot0_encode, 0, 0, 0, Opcode_abs_Slot_dual_slot2_encode, 0, Opcode_abs_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_nop_encode_fns[] = {
  Opcode_nop_Slot_inst_encode, 0, 0, Opcode_nop_Slot_gp_slot2_encode, Opcode_nop_Slot_gp_slot1_encode, Opcode_nop_Slot_gp_slot0_encode, Opcode_nop_Slot_dot_slot2_encode, Opcode_nop_Slot_dot_slot1_encode, Opcode_nop_Slot_dot_slot0_encode, Opcode_nop_Slot_pq_slot2_encode, Opcode_nop_Slot_pq_slot1_encode, Opcode_nop_Slot_pq_slot0_encode, Opcode_nop_Slot_acc2_slot2_encode, Opcode_nop_Slot_acc2_slot1_encode, Opcode_nop_Slot_acc2_slot0_encode, Opcode_nop_Slot_smod_slot2_encode, Opcode_nop_Slot_smod_slot1_encode, Opcode_nop_Slot_smod_slot0_encode, Opcode_nop_Slot_llr_slot2_encode, Opcode_nop_Slot_llr_slot1_encode, Opcode_nop_Slot_llr_slot0_encode, Opcode_nop_Slot_dual_slot2_encode, Opcode_nop_Slot_dual_slot1_encode, Opcode_nop_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ret_encode_fns[] = {
  Opcode_ret_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_s16i_encode_fns[] = {
  Opcode_s16i_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_s16i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_s32i_encode_fns[] = {
  Opcode_s32i_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_s32i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_s8i_encode_fns[] = {
  Opcode_s8i_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_s8i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssr_encode_fns[] = {
  Opcode_ssr_Slot_inst_encode, 0, 0, 0, 0, Opcode_ssr_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssr_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssr_Slot_smod_slot0_encode, 0, 0, 0, Opcode_ssr_Slot_dual_slot2_encode, 0, Opcode_ssr_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssl_encode_fns[] = {
  Opcode_ssl_Slot_inst_encode, 0, 0, 0, 0, Opcode_ssl_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssl_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssl_Slot_smod_slot0_encode, 0, 0, 0, Opcode_ssl_Slot_dual_slot2_encode, 0, Opcode_ssl_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssa8l_encode_fns[] = {
  Opcode_ssa8l_Slot_inst_encode, 0, 0, 0, 0, Opcode_ssa8l_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssa8l_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssa8l_Slot_smod_slot0_encode, 0, 0, 0, Opcode_ssa8l_Slot_dual_slot2_encode, 0, Opcode_ssa8l_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssa8b_encode_fns[] = {
  Opcode_ssa8b_Slot_inst_encode, 0, 0, 0, 0, Opcode_ssa8b_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssa8b_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssa8b_Slot_smod_slot0_encode, 0, 0, 0, Opcode_ssa8b_Slot_dual_slot2_encode, 0, Opcode_ssa8b_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssai_encode_fns[] = {
  Opcode_ssai_Slot_inst_encode, 0, 0, 0, 0, Opcode_ssai_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssai_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ssai_Slot_smod_slot0_encode, 0, 0, 0, Opcode_ssai_Slot_dual_slot2_encode, 0, Opcode_ssai_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sll_encode_fns[] = {
  Opcode_sll_Slot_inst_encode, 0, 0, Opcode_sll_Slot_gp_slot2_encode, 0, Opcode_sll_Slot_gp_slot0_encode, 0, 0, Opcode_sll_Slot_dot_slot0_encode, Opcode_sll_Slot_pq_slot2_encode, 0, Opcode_sll_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_sll_Slot_smod_slot0_encode, 0, 0, Opcode_sll_Slot_llr_slot0_encode, Opcode_sll_Slot_dual_slot2_encode, 0, Opcode_sll_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_src_encode_fns[] = {
  Opcode_src_Slot_inst_encode, 0, 0, 0, 0, Opcode_src_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_src_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_src_Slot_smod_slot0_encode, 0, 0, 0, Opcode_src_Slot_dual_slot2_encode, 0, Opcode_src_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_srl_encode_fns[] = {
  Opcode_srl_Slot_inst_encode, 0, 0, 0, 0, Opcode_srl_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_srl_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_srl_Slot_smod_slot0_encode, 0, 0, 0, Opcode_srl_Slot_dual_slot2_encode, 0, Opcode_srl_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sra_encode_fns[] = {
  Opcode_sra_Slot_inst_encode, 0, 0, Opcode_sra_Slot_gp_slot2_encode, 0, Opcode_sra_Slot_gp_slot0_encode, 0, 0, Opcode_sra_Slot_dot_slot0_encode, Opcode_sra_Slot_pq_slot2_encode, 0, Opcode_sra_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_sra_Slot_smod_slot0_encode, 0, 0, Opcode_sra_Slot_llr_slot0_encode, Opcode_sra_Slot_dual_slot2_encode, 0, Opcode_sra_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_slli_encode_fns[] = {
  Opcode_slli_Slot_inst_encode, 0, 0, 0, 0, Opcode_slli_Slot_gp_slot0_encode, 0, 0, Opcode_slli_Slot_dot_slot0_encode, 0, 0, Opcode_slli_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_slli_Slot_smod_slot0_encode, 0, 0, Opcode_slli_Slot_llr_slot0_encode, Opcode_slli_Slot_dual_slot2_encode, 0, Opcode_slli_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_srai_encode_fns[] = {
  Opcode_srai_Slot_inst_encode, 0, 0, 0, 0, Opcode_srai_Slot_gp_slot0_encode, 0, 0, Opcode_srai_Slot_dot_slot0_encode, 0, 0, Opcode_srai_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_srai_Slot_smod_slot0_encode, 0, 0, Opcode_srai_Slot_llr_slot0_encode, Opcode_srai_Slot_dual_slot2_encode, 0, Opcode_srai_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_srli_encode_fns[] = {
  Opcode_srli_Slot_inst_encode, 0, 0, 0, 0, Opcode_srli_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_srli_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_srli_Slot_smod_slot0_encode, 0, 0, 0, Opcode_srli_Slot_dual_slot2_encode, 0, Opcode_srli_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_memw_encode_fns[] = {
  Opcode_memw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_extw_encode_fns[] = {
  Opcode_extw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_isync_encode_fns[] = {
  Opcode_isync_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsync_encode_fns[] = {
  Opcode_rsync_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_esync_encode_fns[] = {
  Opcode_esync_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dsync_encode_fns[] = {
  Opcode_dsync_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsil_encode_fns[] = {
  Opcode_rsil_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_lend_encode_fns[] = {
  Opcode_rsr_lend_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_lend_encode_fns[] = {
  Opcode_wsr_lend_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_lend_encode_fns[] = {
  Opcode_xsr_lend_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_lcount_encode_fns[] = {
  Opcode_rsr_lcount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_lcount_encode_fns[] = {
  Opcode_wsr_lcount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_lcount_encode_fns[] = {
  Opcode_xsr_lcount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_lbeg_encode_fns[] = {
  Opcode_rsr_lbeg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_lbeg_encode_fns[] = {
  Opcode_wsr_lbeg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_lbeg_encode_fns[] = {
  Opcode_xsr_lbeg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_sar_encode_fns[] = {
  Opcode_rsr_sar_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_sar_encode_fns[] = {
  Opcode_wsr_sar_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_sar_encode_fns[] = {
  Opcode_xsr_sar_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_litbase_encode_fns[] = {
  Opcode_rsr_litbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_litbase_encode_fns[] = {
  Opcode_wsr_litbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_litbase_encode_fns[] = {
  Opcode_xsr_litbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_176_encode_fns[] = {
  Opcode_rsr_176_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_176_encode_fns[] = {
  Opcode_wsr_176_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_208_encode_fns[] = {
  Opcode_rsr_208_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ps_encode_fns[] = {
  Opcode_rsr_ps_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ps_encode_fns[] = {
  Opcode_wsr_ps_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ps_encode_fns[] = {
  Opcode_xsr_ps_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_epc1_encode_fns[] = {
  Opcode_rsr_epc1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_epc1_encode_fns[] = {
  Opcode_wsr_epc1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_epc1_encode_fns[] = {
  Opcode_xsr_epc1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excsave1_encode_fns[] = {
  Opcode_rsr_excsave1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excsave1_encode_fns[] = {
  Opcode_wsr_excsave1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excsave1_encode_fns[] = {
  Opcode_xsr_excsave1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_epc2_encode_fns[] = {
  Opcode_rsr_epc2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_epc2_encode_fns[] = {
  Opcode_wsr_epc2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_epc2_encode_fns[] = {
  Opcode_xsr_epc2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excsave2_encode_fns[] = {
  Opcode_rsr_excsave2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excsave2_encode_fns[] = {
  Opcode_wsr_excsave2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excsave2_encode_fns[] = {
  Opcode_xsr_excsave2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_epc3_encode_fns[] = {
  Opcode_rsr_epc3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_epc3_encode_fns[] = {
  Opcode_wsr_epc3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_epc3_encode_fns[] = {
  Opcode_xsr_epc3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excsave3_encode_fns[] = {
  Opcode_rsr_excsave3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excsave3_encode_fns[] = {
  Opcode_wsr_excsave3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excsave3_encode_fns[] = {
  Opcode_xsr_excsave3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_epc4_encode_fns[] = {
  Opcode_rsr_epc4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_epc4_encode_fns[] = {
  Opcode_wsr_epc4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_epc4_encode_fns[] = {
  Opcode_xsr_epc4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excsave4_encode_fns[] = {
  Opcode_rsr_excsave4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excsave4_encode_fns[] = {
  Opcode_wsr_excsave4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excsave4_encode_fns[] = {
  Opcode_xsr_excsave4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_epc5_encode_fns[] = {
  Opcode_rsr_epc5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_epc5_encode_fns[] = {
  Opcode_wsr_epc5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_epc5_encode_fns[] = {
  Opcode_xsr_epc5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excsave5_encode_fns[] = {
  Opcode_rsr_excsave5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excsave5_encode_fns[] = {
  Opcode_wsr_excsave5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excsave5_encode_fns[] = {
  Opcode_xsr_excsave5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_epc6_encode_fns[] = {
  Opcode_rsr_epc6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_epc6_encode_fns[] = {
  Opcode_wsr_epc6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_epc6_encode_fns[] = {
  Opcode_xsr_epc6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excsave6_encode_fns[] = {
  Opcode_rsr_excsave6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excsave6_encode_fns[] = {
  Opcode_wsr_excsave6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excsave6_encode_fns[] = {
  Opcode_xsr_excsave6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_eps2_encode_fns[] = {
  Opcode_rsr_eps2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_eps2_encode_fns[] = {
  Opcode_wsr_eps2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_eps2_encode_fns[] = {
  Opcode_xsr_eps2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_eps3_encode_fns[] = {
  Opcode_rsr_eps3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_eps3_encode_fns[] = {
  Opcode_wsr_eps3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_eps3_encode_fns[] = {
  Opcode_xsr_eps3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_eps4_encode_fns[] = {
  Opcode_rsr_eps4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_eps4_encode_fns[] = {
  Opcode_wsr_eps4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_eps4_encode_fns[] = {
  Opcode_xsr_eps4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_eps5_encode_fns[] = {
  Opcode_rsr_eps5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_eps5_encode_fns[] = {
  Opcode_wsr_eps5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_eps5_encode_fns[] = {
  Opcode_xsr_eps5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_eps6_encode_fns[] = {
  Opcode_rsr_eps6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_eps6_encode_fns[] = {
  Opcode_wsr_eps6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_eps6_encode_fns[] = {
  Opcode_xsr_eps6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_excvaddr_encode_fns[] = {
  Opcode_rsr_excvaddr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_excvaddr_encode_fns[] = {
  Opcode_wsr_excvaddr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_excvaddr_encode_fns[] = {
  Opcode_xsr_excvaddr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_depc_encode_fns[] = {
  Opcode_rsr_depc_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_depc_encode_fns[] = {
  Opcode_wsr_depc_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_depc_encode_fns[] = {
  Opcode_xsr_depc_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_exccause_encode_fns[] = {
  Opcode_rsr_exccause_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_exccause_encode_fns[] = {
  Opcode_wsr_exccause_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_exccause_encode_fns[] = {
  Opcode_xsr_exccause_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_prid_encode_fns[] = {
  Opcode_rsr_prid_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_vecbase_encode_fns[] = {
  Opcode_rsr_vecbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_vecbase_encode_fns[] = {
  Opcode_wsr_vecbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_vecbase_encode_fns[] = {
  Opcode_xsr_vecbase_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mul16u_encode_fns[] = {
  Opcode_mul16u_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mul16s_encode_fns[] = {
  Opcode_mul16s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfi_encode_fns[] = {
  Opcode_rfi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_waiti_encode_fns[] = {
  Opcode_waiti_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_interrupt_encode_fns[] = {
  Opcode_rsr_interrupt_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_intset_encode_fns[] = {
  Opcode_wsr_intset_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_intclear_encode_fns[] = {
  Opcode_wsr_intclear_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_intenable_encode_fns[] = {
  Opcode_rsr_intenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_intenable_encode_fns[] = {
  Opcode_wsr_intenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_intenable_encode_fns[] = {
  Opcode_xsr_intenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_break_encode_fns[] = {
  Opcode_break_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_break_n_encode_fns[] = {
  0, 0, Opcode_break_n_Slot_inst16b_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_dbreaka0_encode_fns[] = {
  Opcode_rsr_dbreaka0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_dbreaka0_encode_fns[] = {
  Opcode_wsr_dbreaka0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_dbreaka0_encode_fns[] = {
  Opcode_xsr_dbreaka0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_dbreakc0_encode_fns[] = {
  Opcode_rsr_dbreakc0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_dbreakc0_encode_fns[] = {
  Opcode_wsr_dbreakc0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_dbreakc0_encode_fns[] = {
  Opcode_xsr_dbreakc0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_dbreaka1_encode_fns[] = {
  Opcode_rsr_dbreaka1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_dbreaka1_encode_fns[] = {
  Opcode_wsr_dbreaka1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_dbreaka1_encode_fns[] = {
  Opcode_xsr_dbreaka1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_dbreakc1_encode_fns[] = {
  Opcode_rsr_dbreakc1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_dbreakc1_encode_fns[] = {
  Opcode_wsr_dbreakc1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_dbreakc1_encode_fns[] = {
  Opcode_xsr_dbreakc1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ibreaka0_encode_fns[] = {
  Opcode_rsr_ibreaka0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ibreaka0_encode_fns[] = {
  Opcode_wsr_ibreaka0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ibreaka0_encode_fns[] = {
  Opcode_xsr_ibreaka0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ibreaka1_encode_fns[] = {
  Opcode_rsr_ibreaka1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ibreaka1_encode_fns[] = {
  Opcode_wsr_ibreaka1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ibreaka1_encode_fns[] = {
  Opcode_xsr_ibreaka1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ibreakenable_encode_fns[] = {
  Opcode_rsr_ibreakenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ibreakenable_encode_fns[] = {
  Opcode_wsr_ibreakenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ibreakenable_encode_fns[] = {
  Opcode_xsr_ibreakenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_debugcause_encode_fns[] = {
  Opcode_rsr_debugcause_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_debugcause_encode_fns[] = {
  Opcode_wsr_debugcause_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_debugcause_encode_fns[] = {
  Opcode_xsr_debugcause_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_icount_encode_fns[] = {
  Opcode_rsr_icount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_icount_encode_fns[] = {
  Opcode_wsr_icount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_icount_encode_fns[] = {
  Opcode_xsr_icount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_icountlevel_encode_fns[] = {
  Opcode_rsr_icountlevel_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_icountlevel_encode_fns[] = {
  Opcode_wsr_icountlevel_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_icountlevel_encode_fns[] = {
  Opcode_xsr_icountlevel_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ddr_encode_fns[] = {
  Opcode_rsr_ddr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ddr_encode_fns[] = {
  Opcode_wsr_ddr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ddr_encode_fns[] = {
  Opcode_xsr_ddr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfdo_encode_fns[] = {
  Opcode_rfdo_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfdd_encode_fns[] = {
  Opcode_rfdd_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_mmid_encode_fns[] = {
  Opcode_wsr_mmid_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_andb_encode_fns[] = {
  Opcode_andb_Slot_inst_encode, 0, 0, 0, 0, Opcode_andb_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_andb_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_andb_Slot_smod_slot0_encode, 0, 0, 0, Opcode_andb_Slot_dual_slot2_encode, 0, Opcode_andb_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_andbc_encode_fns[] = {
  Opcode_andbc_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_orb_encode_fns[] = {
  Opcode_orb_Slot_inst_encode, 0, 0, 0, 0, Opcode_orb_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_orb_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_orb_Slot_smod_slot0_encode, 0, 0, 0, Opcode_orb_Slot_dual_slot2_encode, 0, Opcode_orb_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_orbc_encode_fns[] = {
  Opcode_orbc_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xorb_encode_fns[] = {
  Opcode_xorb_Slot_inst_encode, 0, 0, 0, 0, Opcode_xorb_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_xorb_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_xorb_Slot_smod_slot0_encode, 0, 0, 0, Opcode_xorb_Slot_dual_slot2_encode, 0, Opcode_xorb_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_any4_encode_fns[] = {
  Opcode_any4_Slot_inst_encode, 0, 0, 0, 0, Opcode_any4_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_any4_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_any4_Slot_smod_slot0_encode, 0, 0, 0, Opcode_any4_Slot_dual_slot2_encode, 0, Opcode_any4_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_all4_encode_fns[] = {
  Opcode_all4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_any8_encode_fns[] = {
  Opcode_any8_Slot_inst_encode, 0, 0, 0, 0, Opcode_any8_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_any8_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_any8_Slot_smod_slot0_encode, 0, 0, 0, Opcode_any8_Slot_dual_slot2_encode, 0, Opcode_any8_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_all8_encode_fns[] = {
  Opcode_all8_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_bf_encode_fns[] = {
  Opcode_bf_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_bt_encode_fns[] = {
  Opcode_bt_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movf_encode_fns[] = {
  Opcode_movf_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movt_encode_fns[] = {
  Opcode_movt_Slot_inst_encode, 0, 0, 0, 0, Opcode_movt_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_movt_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_movt_Slot_smod_slot0_encode, 0, 0, 0, Opcode_movt_Slot_dual_slot2_encode, 0, Opcode_movt_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_rsr_br_encode_fns[] = {
  Opcode_rsr_br_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_br_encode_fns[] = {
  Opcode_wsr_br_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_br_encode_fns[] = {
  Opcode_xsr_br_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ccount_encode_fns[] = {
  Opcode_rsr_ccount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ccount_encode_fns[] = {
  Opcode_wsr_ccount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ccount_encode_fns[] = {
  Opcode_xsr_ccount_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ccompare0_encode_fns[] = {
  Opcode_rsr_ccompare0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ccompare0_encode_fns[] = {
  Opcode_wsr_ccompare0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ccompare0_encode_fns[] = {
  Opcode_xsr_ccompare0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_ccompare1_encode_fns[] = {
  Opcode_rsr_ccompare1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_ccompare1_encode_fns[] = {
  Opcode_wsr_ccompare1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_ccompare1_encode_fns[] = {
  Opcode_xsr_ccompare1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ipf_encode_fns[] = {
  Opcode_ipf_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ihi_encode_fns[] = {
  Opcode_ihi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ipfl_encode_fns[] = {
  Opcode_ipfl_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ihu_encode_fns[] = {
  Opcode_ihu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_iiu_encode_fns[] = {
  Opcode_iiu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_iii_encode_fns[] = {
  Opcode_iii_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lict_encode_fns[] = {
  Opcode_lict_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_licw_encode_fns[] = {
  Opcode_licw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_sict_encode_fns[] = {
  Opcode_sict_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_sicw_encode_fns[] = {
  Opcode_sicw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dhwb_encode_fns[] = {
  Opcode_dhwb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dhwbi_encode_fns[] = {
  Opcode_dhwbi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_diwb_encode_fns[] = {
  Opcode_diwb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_diwbi_encode_fns[] = {
  Opcode_diwbi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dhi_encode_fns[] = {
  Opcode_dhi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dii_encode_fns[] = {
  Opcode_dii_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dpfr_encode_fns[] = {
  Opcode_dpfr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dpfw_encode_fns[] = {
  Opcode_dpfw_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dpfro_encode_fns[] = {
  Opcode_dpfro_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dpfwo_encode_fns[] = {
  Opcode_dpfwo_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dpfl_encode_fns[] = {
  Opcode_dpfl_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dhu_encode_fns[] = {
  Opcode_dhu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_diu_encode_fns[] = {
  Opcode_diu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_sdct_encode_fns[] = {
  Opcode_sdct_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ldct_encode_fns[] = {
  Opcode_ldct_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_idtlb_encode_fns[] = {
  Opcode_idtlb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pdtlb_encode_fns[] = {
  Opcode_pdtlb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rdtlb0_encode_fns[] = {
  Opcode_rdtlb0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rdtlb1_encode_fns[] = {
  Opcode_rdtlb1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wdtlb_encode_fns[] = {
  Opcode_wdtlb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_iitlb_encode_fns[] = {
  Opcode_iitlb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pitlb_encode_fns[] = {
  Opcode_pitlb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ritlb0_encode_fns[] = {
  Opcode_ritlb0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ritlb1_encode_fns[] = {
  Opcode_ritlb1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_witlb_encode_fns[] = {
  Opcode_witlb_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_cpenable_encode_fns[] = {
  Opcode_rsr_cpenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_cpenable_encode_fns[] = {
  Opcode_wsr_cpenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_cpenable_encode_fns[] = {
  Opcode_xsr_cpenable_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_clamps_encode_fns[] = {
  Opcode_clamps_Slot_inst_encode, 0, 0, 0, 0, Opcode_clamps_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_clamps_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_clamps_Slot_smod_slot0_encode, 0, 0, 0, Opcode_clamps_Slot_dual_slot2_encode, 0, Opcode_clamps_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_min_encode_fns[] = {
  Opcode_min_Slot_inst_encode, 0, 0, 0, 0, Opcode_min_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_min_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_min_Slot_smod_slot0_encode, 0, 0, 0, Opcode_min_Slot_dual_slot2_encode, 0, Opcode_min_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_max_encode_fns[] = {
  Opcode_max_Slot_inst_encode, 0, 0, 0, 0, Opcode_max_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_max_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_max_Slot_smod_slot0_encode, 0, 0, 0, Opcode_max_Slot_dual_slot2_encode, 0, Opcode_max_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_minu_encode_fns[] = {
  Opcode_minu_Slot_inst_encode, 0, 0, 0, 0, Opcode_minu_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_minu_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_minu_Slot_smod_slot0_encode, 0, 0, 0, Opcode_minu_Slot_dual_slot2_encode, 0, Opcode_minu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_maxu_encode_fns[] = {
  Opcode_maxu_Slot_inst_encode, 0, 0, 0, 0, Opcode_maxu_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_maxu_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_maxu_Slot_smod_slot0_encode, 0, 0, 0, Opcode_maxu_Slot_dual_slot2_encode, 0, Opcode_maxu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_nsa_encode_fns[] = {
  Opcode_nsa_Slot_inst_encode, 0, 0, 0, 0, Opcode_nsa_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_nsa_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_nsa_Slot_smod_slot0_encode, 0, 0, 0, Opcode_nsa_Slot_dual_slot2_encode, 0, Opcode_nsa_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_nsau_encode_fns[] = {
  Opcode_nsau_Slot_inst_encode, 0, 0, 0, 0, Opcode_nsau_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_nsau_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_nsau_Slot_smod_slot0_encode, 0, 0, 0, Opcode_nsau_Slot_dual_slot2_encode, 0, Opcode_nsau_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sext_encode_fns[] = {
  Opcode_sext_Slot_inst_encode, 0, 0, 0, 0, Opcode_sext_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sext_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_sext_Slot_smod_slot0_encode, 0, 0, 0, Opcode_sext_Slot_dual_slot2_encode, 0, Opcode_sext_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_l32ai_encode_fns[] = {
  Opcode_l32ai_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_s32ri_encode_fns[] = {
  Opcode_s32ri_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_s32c1i_encode_fns[] = {
  Opcode_s32c1i_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_scompare1_encode_fns[] = {
  Opcode_rsr_scompare1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_scompare1_encode_fns[] = {
  Opcode_wsr_scompare1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_scompare1_encode_fns[] = {
  Opcode_xsr_scompare1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rsr_atomctl_encode_fns[] = {
  Opcode_rsr_atomctl_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wsr_atomctl_encode_fns[] = {
  Opcode_wsr_atomctl_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xsr_atomctl_encode_fns[] = {
  Opcode_xsr_atomctl_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rer_encode_fns[] = {
  Opcode_rer_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wer_encode_fns[] = {
  Opcode_wer_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_fcr_encode_fns[] = {
  Opcode_rur_fcr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_fcr_encode_fns[] = {
  Opcode_wur_fcr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_fsr_encode_fns[] = {
  Opcode_rur_fsr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_fsr_encode_fns[] = {
  Opcode_wur_fsr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_add_s_encode_fns[] = {
  Opcode_add_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_add_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sub_s_encode_fns[] = {
  Opcode_sub_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sub_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_mul_s_encode_fns[] = {
  Opcode_mul_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mul_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_madd_s_encode_fns[] = {
  Opcode_madd_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_madd_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_msub_s_encode_fns[] = {
  Opcode_msub_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_msub_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movf_s_encode_fns[] = {
  Opcode_movf_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movf_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movt_s_encode_fns[] = {
  Opcode_movt_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movt_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_moveqz_s_encode_fns[] = {
  Opcode_moveqz_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_moveqz_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movnez_s_encode_fns[] = {
  Opcode_movnez_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movnez_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movltz_s_encode_fns[] = {
  Opcode_movltz_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movltz_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movgez_s_encode_fns[] = {
  Opcode_movgez_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movgez_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_abs_s_encode_fns[] = {
  Opcode_abs_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_abs_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_mov_s_encode_fns[] = {
  Opcode_mov_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mov_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_neg_s_encode_fns[] = {
  Opcode_neg_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_neg_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_un_s_encode_fns[] = {
  Opcode_un_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_un_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_oeq_s_encode_fns[] = {
  Opcode_oeq_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_oeq_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ueq_s_encode_fns[] = {
  Opcode_ueq_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ueq_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_olt_s_encode_fns[] = {
  Opcode_olt_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_olt_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ult_s_encode_fns[] = {
  Opcode_ult_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ult_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ole_s_encode_fns[] = {
  Opcode_ole_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ole_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ule_s_encode_fns[] = {
  Opcode_ule_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ule_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_float_s_encode_fns[] = {
  Opcode_float_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_float_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ufloat_s_encode_fns[] = {
  Opcode_ufloat_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ufloat_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_round_s_encode_fns[] = {
  Opcode_round_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_round_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ceil_s_encode_fns[] = {
  Opcode_ceil_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ceil_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_floor_s_encode_fns[] = {
  Opcode_floor_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_floor_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_trunc_s_encode_fns[] = {
  Opcode_trunc_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_trunc_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_utrunc_s_encode_fns[] = {
  Opcode_utrunc_s_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_utrunc_s_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_rfr_encode_fns[] = {
  Opcode_rfr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rfr_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_wfr_encode_fns[] = {
  Opcode_wfr_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lsi_encode_fns[] = {
  Opcode_lsi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lsi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lsiu_encode_fns[] = {
  Opcode_lsiu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lsiu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lsx_encode_fns[] = {
  Opcode_lsx_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lsxu_encode_fns[] = {
  Opcode_lsxu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lsxu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssi_encode_fns[] = {
  Opcode_ssi_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ssi_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssiu_encode_fns[] = {
  Opcode_ssiu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ssiu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssx_encode_fns[] = {
  Opcode_ssx_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ssx_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ssxu_encode_fns[] = {
  Opcode_ssxu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ssxu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_argmax_encode_fns[] = {
  Opcode_get_argmax_Slot_inst_encode, 0, 0, Opcode_get_argmax_Slot_gp_slot2_encode, 0, 0, Opcode_get_argmax_Slot_dot_slot2_encode, 0, 0, Opcode_get_argmax_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_get_argmax_Slot_smod_slot2_encode, 0, 0, Opcode_get_argmax_Slot_llr_slot2_encode, 0, 0, Opcode_get_argmax_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_get_hsar_encode_fns[] = {
  Opcode_get_hsar_Slot_inst_encode, 0, 0, Opcode_get_hsar_Slot_gp_slot2_encode, 0, Opcode_get_hsar_Slot_gp_slot0_encode, Opcode_get_hsar_Slot_dot_slot2_encode, 0, Opcode_get_hsar_Slot_dot_slot0_encode, Opcode_get_hsar_Slot_pq_slot2_encode, 0, Opcode_get_hsar_Slot_pq_slot0_encode, 0, 0, 0, Opcode_get_hsar_Slot_smod_slot2_encode, 0, Opcode_get_hsar_Slot_smod_slot0_encode, Opcode_get_hsar_Slot_llr_slot2_encode, 0, Opcode_get_hsar_Slot_llr_slot0_encode, Opcode_get_hsar_Slot_dual_slot2_encode, 0, Opcode_get_hsar_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_hsar2sar_encode_fns[] = {
  Opcode_get_hsar2sar_Slot_inst_encode, 0, 0, Opcode_get_hsar2sar_Slot_gp_slot2_encode, 0, Opcode_get_hsar2sar_Slot_gp_slot0_encode, Opcode_get_hsar2sar_Slot_dot_slot2_encode, 0, Opcode_get_hsar2sar_Slot_dot_slot0_encode, Opcode_get_hsar2sar_Slot_pq_slot2_encode, 0, Opcode_get_hsar2sar_Slot_pq_slot0_encode, 0, 0, 0, Opcode_get_hsar2sar_Slot_smod_slot2_encode, 0, Opcode_get_hsar2sar_Slot_smod_slot0_encode, Opcode_get_hsar2sar_Slot_llr_slot2_encode, 0, Opcode_get_hsar2sar_Slot_llr_slot0_encode, Opcode_get_hsar2sar_Slot_dual_slot2_encode, 0, Opcode_get_hsar2sar_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_interp_ext_n_encode_fns[] = {
  Opcode_get_interp_ext_n_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_interp_ext_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_interp_ext_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_interp_ext_n_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_interp_ext_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_interp_ext_l_encode_fns[] = {
  Opcode_get_interp_ext_l_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_interp_ext_l_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_interp_ext_l_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_interp_ext_l_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_interp_ext_l_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_llr_buf_encode_fns[] = {
  Opcode_get_llr_buf_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_llr_buf_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_llr_buf_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_llr_buf_Slot_smod_slot0_encode, 0, 0, Opcode_get_llr_buf_Slot_llr_slot0_encode, 0, 0, Opcode_get_llr_buf_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_llr_pos_encode_fns[] = {
  Opcode_get_llr_pos_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_llr_pos_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_llr_pos_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_llr_pos_Slot_smod_slot0_encode, 0, 0, Opcode_get_llr_pos_Slot_llr_slot0_encode, 0, 0, Opcode_get_llr_pos_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_max_encode_fns[] = {
  Opcode_get_max_Slot_inst_encode, 0, 0, Opcode_get_max_Slot_gp_slot2_encode, 0, 0, Opcode_get_max_Slot_dot_slot2_encode, 0, 0, Opcode_get_max_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_get_max_Slot_smod_slot2_encode, 0, 0, Opcode_get_max_Slot_llr_slot2_encode, 0, 0, Opcode_get_max_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_get_nco_encode_fns[] = {
  Opcode_get_nco_Slot_inst_encode, 0, 0, Opcode_get_nco_Slot_gp_slot2_encode, 0, 0, Opcode_get_nco_Slot_dot_slot2_encode, 0, 0, Opcode_get_nco_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_get_nco_Slot_smod_slot2_encode, 0, 0, Opcode_get_nco_Slot_llr_slot2_encode, 0, 0, Opcode_get_nco_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_get_perm_reg_encode_fns[] = {
  0, 0, 0, 0, 0, Opcode_get_perm_reg_Slot_gp_slot0_encode, 0, 0, Opcode_get_perm_reg_Slot_dot_slot0_encode, 0, 0, Opcode_get_perm_reg_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_perm_reg_Slot_smod_slot0_encode, 0, 0, Opcode_get_perm_reg_Slot_llr_slot0_encode, 0, 0, Opcode_get_perm_reg_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_phasor_n_encode_fns[] = {
  Opcode_get_phasor_n_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_phasor_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_phasor_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_phasor_n_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_phasor_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_phasor_offset_encode_fns[] = {
  Opcode_get_phasor_offset_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_phasor_offset_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_phasor_offset_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_phasor_offset_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_phasor_offset_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_sar_encode_fns[] = {
  Opcode_get_sar_Slot_inst_encode, 0, 0, Opcode_get_sar_Slot_gp_slot2_encode, 0, Opcode_get_sar_Slot_gp_slot0_encode, Opcode_get_sar_Slot_dot_slot2_encode, 0, Opcode_get_sar_Slot_dot_slot0_encode, Opcode_get_sar_Slot_pq_slot2_encode, 0, Opcode_get_sar_Slot_pq_slot0_encode, 0, 0, 0, Opcode_get_sar_Slot_smod_slot2_encode, 0, Opcode_get_sar_Slot_smod_slot0_encode, Opcode_get_sar_Slot_llr_slot2_encode, 0, Opcode_get_sar_Slot_llr_slot0_encode, Opcode_get_sar_Slot_dual_slot2_encode, 0, Opcode_get_sar_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_scale_reg_encode_fns[] = {
  Opcode_get_scale_reg_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_scale_reg_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_scale_reg_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_scale_reg_Slot_smod_slot0_encode, 0, 0, Opcode_get_scale_reg_Slot_llr_slot0_encode, 0, 0, Opcode_get_scale_reg_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_smod_buf_encode_fns[] = {
  Opcode_get_smod_buf_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_smod_buf_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_smod_buf_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_smod_buf_Slot_smod_slot0_encode, 0, 0, Opcode_get_smod_buf_Slot_llr_slot0_encode, 0, 0, Opcode_get_smod_buf_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_smod_offset_table_encode_fns[] = {
  Opcode_get_smod_offset_table_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_smod_offset_table_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_smod_offset_table_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_smod_offset_table_Slot_smod_slot0_encode, 0, 0, Opcode_get_smod_offset_table_Slot_llr_slot0_encode, 0, 0, Opcode_get_smod_offset_table_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_smod_pos_encode_fns[] = {
  Opcode_get_smod_pos_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_smod_pos_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_smod_pos_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_smod_pos_Slot_smod_slot0_encode, 0, 0, Opcode_get_smod_pos_Slot_llr_slot0_encode, 0, 0, Opcode_get_smod_pos_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_sov_encode_fns[] = {
  Opcode_get_sov_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_sov_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_sov_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_sov_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_sov_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_get_wght_encode_fns[] = {
  Opcode_get_wght_Slot_inst_encode, 0, 0, 0, 0, Opcode_get_wght_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_wght_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_get_wght_Slot_smod_slot0_encode, 0, 0, Opcode_get_wght_Slot_llr_slot0_encode, 0, 0, Opcode_get_wght_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_argmax_encode_fns[] = {
  Opcode_set_argmax_Slot_inst_encode, 0, 0, Opcode_set_argmax_Slot_gp_slot2_encode, 0, 0, Opcode_set_argmax_Slot_dot_slot2_encode, 0, 0, Opcode_set_argmax_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_set_argmax_Slot_smod_slot2_encode, 0, 0, Opcode_set_argmax_Slot_llr_slot2_encode, 0, 0, Opcode_set_argmax_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_set_ext_regs_encode_fns[] = {
  Opcode_set_ext_regs_Slot_inst_encode, 0, 0, Opcode_set_ext_regs_Slot_gp_slot2_encode, 0, 0, Opcode_set_ext_regs_Slot_dot_slot2_encode, 0, 0, Opcode_set_ext_regs_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_set_ext_regs_Slot_smod_slot2_encode, 0, 0, Opcode_set_ext_regs_Slot_llr_slot2_encode, 0, 0, Opcode_set_ext_regs_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_set_hsar_encode_fns[] = {
  0, 0, 0, Opcode_set_hsar_Slot_gp_slot2_encode, 0, 0, Opcode_set_hsar_Slot_dot_slot2_encode, 0, 0, Opcode_set_hsar_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_set_hsar_Slot_smod_slot2_encode, 0, 0, Opcode_set_hsar_Slot_llr_slot2_encode, 0, 0, Opcode_set_hsar_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_set_llr_buf_encode_fns[] = {
  Opcode_set_llr_buf_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_llr_buf_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_llr_buf_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_llr_buf_Slot_smod_slot0_encode, 0, 0, Opcode_set_llr_buf_Slot_llr_slot0_encode, 0, 0, Opcode_set_llr_buf_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_llr_pos_encode_fns[] = {
  Opcode_set_llr_pos_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_llr_pos_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_llr_pos_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_llr_pos_Slot_smod_slot0_encode, 0, 0, Opcode_set_llr_pos_Slot_llr_slot0_encode, 0, 0, Opcode_set_llr_pos_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_max_encode_fns[] = {
  Opcode_set_max_Slot_inst_encode, 0, 0, Opcode_set_max_Slot_gp_slot2_encode, 0, 0, Opcode_set_max_Slot_dot_slot2_encode, 0, 0, Opcode_set_max_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_set_max_Slot_smod_slot2_encode, 0, 0, Opcode_set_max_Slot_llr_slot2_encode, 0, 0, Opcode_set_max_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_set_nco_encode_fns[] = {
  Opcode_set_nco_Slot_inst_encode, 0, 0, Opcode_set_nco_Slot_gp_slot2_encode, 0, 0, Opcode_set_nco_Slot_dot_slot2_encode, 0, 0, Opcode_set_nco_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_set_nco_Slot_smod_slot2_encode, 0, 0, Opcode_set_nco_Slot_llr_slot2_encode, 0, 0, Opcode_set_nco_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_set_perm_reg_encode_fns[] = {
  0, 0, 0, 0, 0, Opcode_set_perm_reg_Slot_gp_slot0_encode, 0, 0, Opcode_set_perm_reg_Slot_dot_slot0_encode, 0, 0, Opcode_set_perm_reg_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_perm_reg_Slot_smod_slot0_encode, 0, 0, Opcode_set_perm_reg_Slot_llr_slot0_encode, 0, 0, Opcode_set_perm_reg_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_phasor_n_encode_fns[] = {
  Opcode_set_phasor_n_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_phasor_n_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_phasor_n_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_phasor_n_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_phasor_n_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_phasor_offset_encode_fns[] = {
  Opcode_set_phasor_offset_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_phasor_offset_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_phasor_offset_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_phasor_offset_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_phasor_offset_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_sar_encode_fns[] = {
  Opcode_set_sar_Slot_inst_encode, 0, 0, Opcode_set_sar_Slot_gp_slot2_encode, 0, Opcode_set_sar_Slot_gp_slot0_encode, Opcode_set_sar_Slot_dot_slot2_encode, 0, Opcode_set_sar_Slot_dot_slot0_encode, Opcode_set_sar_Slot_pq_slot2_encode, 0, Opcode_set_sar_Slot_pq_slot0_encode, 0, 0, 0, Opcode_set_sar_Slot_smod_slot2_encode, 0, Opcode_set_sar_Slot_smod_slot0_encode, Opcode_set_sar_Slot_llr_slot2_encode, 0, Opcode_set_sar_Slot_llr_slot0_encode, Opcode_set_sar_Slot_dual_slot2_encode, 0, Opcode_set_sar_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_scale_reg_encode_fns[] = {
  Opcode_set_scale_reg_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_scale_reg_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_scale_reg_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_scale_reg_Slot_smod_slot0_encode, 0, 0, Opcode_set_scale_reg_Slot_llr_slot0_encode, 0, 0, Opcode_set_scale_reg_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_smod_buf_encode_fns[] = {
  Opcode_set_smod_buf_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_smod_buf_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_smod_buf_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_smod_buf_Slot_smod_slot0_encode, 0, 0, Opcode_set_smod_buf_Slot_llr_slot0_encode, 0, 0, Opcode_set_smod_buf_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_smod_offset_table_encode_fns[] = {
  Opcode_set_smod_offset_table_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_smod_offset_table_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_smod_offset_table_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_smod_offset_table_Slot_smod_slot0_encode, 0, 0, Opcode_set_smod_offset_table_Slot_llr_slot0_encode, 0, 0, Opcode_set_smod_offset_table_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_smod_pos_encode_fns[] = {
  Opcode_set_smod_pos_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_smod_pos_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_smod_pos_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_smod_pos_Slot_smod_slot0_encode, 0, 0, Opcode_set_smod_pos_Slot_llr_slot0_encode, 0, 0, Opcode_set_smod_pos_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_sov_encode_fns[] = {
  Opcode_set_sov_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_sov_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_sov_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_sov_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_sov_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_set_wght_encode_fns[] = {
  Opcode_set_wght_Slot_inst_encode, 0, 0, 0, 0, Opcode_set_wght_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_wght_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_set_wght_Slot_smod_slot0_encode, 0, 0, Opcode_set_wght_Slot_llr_slot0_encode, 0, 0, Opcode_set_wght_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac2x32_encode_fns[] = {
  Opcode_lac2x32_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac2x32_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac2x32_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac2x32_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac2x64_0_encode_fns[] = {
  Opcode_lac2x64_0_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac2x64_0_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac2x64_0_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac2x64_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac2x64_1_encode_fns[] = {
  Opcode_lac2x64_1_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac2x64_1_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac2x64_1_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac2x64_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac2x64_2_encode_fns[] = {
  Opcode_lac2x64_2_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac2x64_2_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac2x64_2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac2x64_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac2x64_3_encode_fns[] = {
  Opcode_lac2x64_3_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac2x64_3_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac2x64_3_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac2x64_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac32_r_encode_fns[] = {
  Opcode_lac32_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac32_r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac32_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac32_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac_ih_encode_fns[] = {
  Opcode_lac_ih_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac_ih_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac_ih_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac_ih_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac_il_encode_fns[] = {
  Opcode_lac_il_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac_il_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac_il_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac_il_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac_rh_encode_fns[] = {
  Opcode_lac_rh_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac_rh_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac_rh_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac_rh_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lac_rl_encode_fns[] = {
  Opcode_lac_rl_Slot_inst_encode, 0, 0, 0, 0, Opcode_lac_rl_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lac_rl_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lac_rl_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lcm_encode_fns[] = {
  Opcode_lcm_Slot_inst_encode, 0, 0, 0, 0, Opcode_lcm_Slot_gp_slot0_encode, 0, 0, Opcode_lcm_Slot_dot_slot0_encode, 0, 0, Opcode_lcm_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_Slot_smod_slot0_encode, 0, 0, Opcode_lcm_Slot_llr_slot0_encode, 0, 0, Opcode_lcm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lcm_pinc_encode_fns[] = {
  Opcode_lcm_pinc_Slot_inst_encode, 0, 0, 0, 0, Opcode_lcm_pinc_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_pinc_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_pinc_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_pinc_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lcm_pinc_x_encode_fns[] = {
  Opcode_lcm_pinc_x_Slot_inst_encode, 0, 0, 0, 0, Opcode_lcm_pinc_x_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_pinc_x_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_pinc_x_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_pinc_x_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lcm_u_encode_fns[] = {
  Opcode_lcm_u_Slot_inst_encode, 0, 0, 0, 0, Opcode_lcm_u_Slot_gp_slot0_encode, 0, 0, Opcode_lcm_u_Slot_dot_slot0_encode, 0, 0, Opcode_lcm_u_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_lcm_u_Slot_smod_slot0_encode, 0, 0, Opcode_lcm_u_Slot_llr_slot0_encode, 0, 0, Opcode_lcm_u_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lcm_x_encode_fns[] = {
  Opcode_lcm_x_Slot_inst_encode, 0, 0, 0, 0, Opcode_lcm_x_Slot_gp_slot0_encode, 0, 0, Opcode_lcm_x_Slot_dot_slot0_encode, 0, 0, Opcode_lcm_x_Slot_pq_slot0_encode, 0, 0, Opcode_lcm_x_Slot_acc2_slot0_encode, 0, 0, Opcode_lcm_x_Slot_smod_slot0_encode, 0, 0, Opcode_lcm_x_Slot_llr_slot0_encode, 0, 0, Opcode_lcm_x_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lcm_xu_encode_fns[] = {
  Opcode_lcm_xu_Slot_inst_encode, 0, 0, 0, 0, Opcode_lcm_xu_Slot_gp_slot0_encode, 0, 0, Opcode_lcm_xu_Slot_dot_slot0_encode, 0, 0, Opcode_lcm_xu_Slot_pq_slot0_encode, 0, 0, Opcode_lcm_xu_Slot_acc2_slot0_encode, 0, 0, Opcode_lcm_xu_Slot_smod_slot0_encode, 0, 0, Opcode_lcm_xu_Slot_llr_slot0_encode, 0, 0, Opcode_lcm_xu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lp_encode_fns[] = {
  Opcode_lp_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lp_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lp_x_encode_fns[] = {
  Opcode_lp_x_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lp_x_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lq_encode_fns[] = {
  Opcode_lq_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lq_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lq_x_encode_fns[] = {
  Opcode_lq_x_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lq_x_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lut0_encode_fns[] = {
  Opcode_lut0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lut0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lut1_encode_fns[] = {
  Opcode_lut1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lut1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lut2_encode_fns[] = {
  Opcode_lut2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lut2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lut3_encode_fns[] = {
  Opcode_lut3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_lut3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac2x32_encode_fns[] = {
  Opcode_sac2x32_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac2x32_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac2x32_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac2x32_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac2x64_0_encode_fns[] = {
  Opcode_sac2x64_0_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac2x64_0_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac2x64_0_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac2x64_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac2x64_1_encode_fns[] = {
  Opcode_sac2x64_1_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac2x64_1_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac2x64_1_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac2x64_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac2x64_2_encode_fns[] = {
  Opcode_sac2x64_2_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac2x64_2_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac2x64_2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac2x64_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac2x64_3_encode_fns[] = {
  Opcode_sac2x64_3_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac2x64_3_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac2x64_3_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac2x64_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac32_r_encode_fns[] = {
  Opcode_sac32_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac32_r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac32_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac32_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac_ih_encode_fns[] = {
  Opcode_sac_ih_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac_ih_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac_ih_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac_ih_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac_il_encode_fns[] = {
  Opcode_sac_il_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac_il_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac_il_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac_il_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac_rh_encode_fns[] = {
  Opcode_sac_rh_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac_rh_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac_rh_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac_rh_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sac_rl_encode_fns[] = {
  Opcode_sac_rl_Slot_inst_encode, 0, 0, 0, 0, Opcode_sac_rl_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_sac_rl_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sac_rl_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_scm_encode_fns[] = {
  Opcode_scm_Slot_inst_encode, 0, 0, 0, 0, Opcode_scm_Slot_gp_slot0_encode, 0, 0, Opcode_scm_Slot_dot_slot0_encode, 0, 0, Opcode_scm_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_Slot_smod_slot0_encode, 0, 0, Opcode_scm_Slot_llr_slot0_encode, 0, 0, Opcode_scm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_scm_pinc_encode_fns[] = {
  Opcode_scm_pinc_Slot_inst_encode, 0, 0, 0, 0, Opcode_scm_pinc_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_pinc_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_pinc_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_pinc_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_scm_pinc_x_encode_fns[] = {
  Opcode_scm_pinc_x_Slot_inst_encode, 0, 0, 0, 0, Opcode_scm_pinc_x_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_pinc_x_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_pinc_x_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_pinc_x_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_scm_u_encode_fns[] = {
  Opcode_scm_u_Slot_inst_encode, 0, 0, 0, 0, Opcode_scm_u_Slot_gp_slot0_encode, 0, 0, Opcode_scm_u_Slot_dot_slot0_encode, 0, 0, Opcode_scm_u_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_scm_u_Slot_smod_slot0_encode, 0, 0, Opcode_scm_u_Slot_llr_slot0_encode, 0, 0, Opcode_scm_u_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_scm_x_encode_fns[] = {
  Opcode_scm_x_Slot_inst_encode, 0, 0, 0, 0, Opcode_scm_x_Slot_gp_slot0_encode, 0, 0, Opcode_scm_x_Slot_dot_slot0_encode, 0, 0, Opcode_scm_x_Slot_pq_slot0_encode, 0, 0, Opcode_scm_x_Slot_acc2_slot0_encode, 0, 0, Opcode_scm_x_Slot_smod_slot0_encode, 0, 0, Opcode_scm_x_Slot_llr_slot0_encode, 0, 0, Opcode_scm_x_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_scm_xu_encode_fns[] = {
  Opcode_scm_xu_Slot_inst_encode, 0, 0, 0, 0, Opcode_scm_xu_Slot_gp_slot0_encode, 0, 0, Opcode_scm_xu_Slot_dot_slot0_encode, 0, 0, Opcode_scm_xu_Slot_pq_slot0_encode, 0, 0, Opcode_scm_xu_Slot_acc2_slot0_encode, 0, 0, Opcode_scm_xu_Slot_smod_slot0_encode, 0, 0, Opcode_scm_xu_Slot_llr_slot0_encode, 0, 0, Opcode_scm_xu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_store_p_encode_fns[] = {
  Opcode_store_p_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_store_p_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_store_p_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_store_q_encode_fns[] = {
  Opcode_store_q_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_store_q_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_store_q_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ar2cm_dup_encode_fns[] = {
  Opcode_ar2cm_dup_Slot_inst_encode, 0, 0, Opcode_ar2cm_dup_Slot_gp_slot2_encode, 0, Opcode_ar2cm_dup_Slot_gp_slot0_encode, Opcode_ar2cm_dup_Slot_dot_slot2_encode, 0, Opcode_ar2cm_dup_Slot_dot_slot0_encode, Opcode_ar2cm_dup_Slot_pq_slot2_encode, 0, Opcode_ar2cm_dup_Slot_pq_slot0_encode, 0, 0, 0, Opcode_ar2cm_dup_Slot_smod_slot2_encode, 0, Opcode_ar2cm_dup_Slot_smod_slot0_encode, Opcode_ar2cm_dup_Slot_llr_slot2_encode, 0, Opcode_ar2cm_dup_Slot_llr_slot0_encode, Opcode_ar2cm_dup_Slot_dual_slot2_encode, 0, Opcode_ar2cm_dup_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ar2cm_ln_encode_fns[] = {
  Opcode_ar2cm_ln_Slot_inst_encode, 0, 0, Opcode_ar2cm_ln_Slot_gp_slot2_encode, 0, Opcode_ar2cm_ln_Slot_gp_slot0_encode, Opcode_ar2cm_ln_Slot_dot_slot2_encode, 0, Opcode_ar2cm_ln_Slot_dot_slot0_encode, Opcode_ar2cm_ln_Slot_pq_slot2_encode, 0, Opcode_ar2cm_ln_Slot_pq_slot0_encode, 0, 0, 0, Opcode_ar2cm_ln_Slot_smod_slot2_encode, 0, Opcode_ar2cm_ln_Slot_smod_slot0_encode, Opcode_ar2cm_ln_Slot_llr_slot2_encode, 0, Opcode_ar2cm_ln_Slot_llr_slot0_encode, Opcode_ar2cm_ln_Slot_dual_slot2_encode, 0, Opcode_ar2cm_ln_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ar2cm_ln_i_encode_fns[] = {
  Opcode_ar2cm_ln_i_Slot_inst_encode, 0, 0, 0, 0, Opcode_ar2cm_ln_i_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ar2cm_ln_i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ar2cm_ln_i_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_ar2cm_ln_i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ar2cm_ln_r_encode_fns[] = {
  Opcode_ar2cm_ln_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_ar2cm_ln_r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_ar2cm_ln_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_ar2cm_ln_r_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_ar2cm_ln_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ar2pq_ln_encode_fns[] = {
  0, 0, 0, Opcode_ar2pq_ln_Slot_gp_slot2_encode, 0, 0, Opcode_ar2pq_ln_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ar2pq_ln_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ar2sar_dup_encode_fns[] = {
  0, 0, 0, Opcode_ar2sar_dup_Slot_gp_slot2_encode, 0, 0, Opcode_ar2sar_dup_Slot_dot_slot2_encode, 0, 0, Opcode_ar2sar_dup_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_ar2sar_dup_Slot_smod_slot2_encode, 0, 0, Opcode_ar2sar_dup_Slot_llr_slot2_encode, 0, 0, Opcode_ar2sar_dup_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_clrac_encode_fns[] = {
  Opcode_clrac_Slot_inst_encode, 0, 0, 0, 0, Opcode_clrac_Slot_gp_slot0_encode, 0, 0, Opcode_clrac_Slot_dot_slot0_encode, 0, 0, Opcode_clrac_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_clrac_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_clrcm_encode_fns[] = {
  Opcode_clrcm_Slot_inst_encode, 0, 0, Opcode_clrcm_Slot_gp_slot2_encode, 0, Opcode_clrcm_Slot_gp_slot0_encode, Opcode_clrcm_Slot_dot_slot2_encode, 0, Opcode_clrcm_Slot_dot_slot0_encode, Opcode_clrcm_Slot_pq_slot2_encode, 0, Opcode_clrcm_Slot_pq_slot0_encode, Opcode_clrcm_Slot_acc2_slot2_encode, 0, Opcode_clrcm_Slot_acc2_slot0_encode, Opcode_clrcm_Slot_smod_slot2_encode, 0, Opcode_clrcm_Slot_smod_slot0_encode, Opcode_clrcm_Slot_llr_slot2_encode, 0, Opcode_clrcm_Slot_llr_slot0_encode, Opcode_clrcm_Slot_dual_slot2_encode, 0, Opcode_clrcm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_cm2ar_ln_encode_fns[] = {
  Opcode_cm2ar_ln_Slot_inst_encode, 0, 0, Opcode_cm2ar_ln_Slot_gp_slot2_encode, 0, Opcode_cm2ar_ln_Slot_gp_slot0_encode, Opcode_cm2ar_ln_Slot_dot_slot2_encode, 0, Opcode_cm2ar_ln_Slot_dot_slot0_encode, Opcode_cm2ar_ln_Slot_pq_slot2_encode, 0, Opcode_cm2ar_ln_Slot_pq_slot0_encode, 0, 0, 0, Opcode_cm2ar_ln_Slot_smod_slot2_encode, 0, Opcode_cm2ar_ln_Slot_smod_slot0_encode, Opcode_cm2ar_ln_Slot_llr_slot2_encode, 0, Opcode_cm2ar_ln_Slot_llr_slot0_encode, Opcode_cm2ar_ln_Slot_dual_slot2_encode, 0, Opcode_cm2ar_ln_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_cm2ar_ln_i_encode_fns[] = {
  Opcode_cm2ar_ln_i_Slot_inst_encode, 0, 0, 0, 0, Opcode_cm2ar_ln_i_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_cm2ar_ln_i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_cm2ar_ln_i_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_cm2ar_ln_i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_cm2ar_ln_r_encode_fns[] = {
  Opcode_cm2ar_ln_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_cm2ar_ln_r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_cm2ar_ln_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_cm2ar_ln_r_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_cm2ar_ln_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_comb_ar_encode_fns[] = {
  Opcode_comb_ar_Slot_inst_encode, 0, 0, 0, 0, Opcode_comb_ar_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_comb_ar_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_comb_ar_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_comb_ar_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_conj_encode_fns[] = {
  Opcode_conj_Slot_inst_encode, 0, 0, Opcode_conj_Slot_gp_slot2_encode, 0, Opcode_conj_Slot_gp_slot0_encode, Opcode_conj_Slot_dot_slot2_encode, 0, Opcode_conj_Slot_dot_slot0_encode, Opcode_conj_Slot_pq_slot2_encode, 0, Opcode_conj_Slot_pq_slot0_encode, Opcode_conj_Slot_acc2_slot2_encode, 0, Opcode_conj_Slot_acc2_slot0_encode, Opcode_conj_Slot_smod_slot2_encode, 0, Opcode_conj_Slot_smod_slot0_encode, Opcode_conj_Slot_llr_slot2_encode, 0, Opcode_conj_Slot_llr_slot0_encode, Opcode_conj_Slot_dual_slot2_encode, 0, Opcode_conj_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_mov2ac32_i_encode_fns[] = {
  Opcode_mov2ac32_i_Slot_inst_encode, 0, 0, 0, 0, Opcode_mov2ac32_i_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_mov2ac32_i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mov2ac32_i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_mov2ac32_r_encode_fns[] = {
  Opcode_mov2ac32_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_mov2ac32_r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_mov2ac32_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mov2ac32_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_mov2cm2pq_encode_fns[] = {
  0, 0, 0, Opcode_mov2cm2pq_Slot_gp_slot2_encode, 0, 0, Opcode_mov2cm2pq_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mov2cm2pq_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movac_encode_fns[] = {
  Opcode_movac_Slot_inst_encode, 0, 0, 0, 0, Opcode_movac_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_movac_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movac_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movac_i_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movac_i_Slot_llr_slot1_encode, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movac_i2r_encode_fns[] = {
  Opcode_movac_i2r_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movac_r_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movac_r_Slot_llr_slot1_encode, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movac_r2i_encode_fns[] = {
  Opcode_movac_r2i_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movar2_encode_fns[] = {
  0, 0, 0, Opcode_movar2_Slot_gp_slot2_encode, 0, 0, Opcode_movar2_Slot_dot_slot2_encode, 0, 0, Opcode_movar2_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movar2_Slot_smod_slot2_encode, 0, 0, Opcode_movar2_Slot_llr_slot2_encode, 0, 0, Opcode_movar2_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcm_encode_fns[] = {
  Opcode_movcm_Slot_inst_encode, 0, 0, Opcode_movcm_Slot_gp_slot2_encode, 0, Opcode_movcm_Slot_gp_slot0_encode, Opcode_movcm_Slot_dot_slot2_encode, 0, Opcode_movcm_Slot_dot_slot0_encode, Opcode_movcm_Slot_pq_slot2_encode, 0, Opcode_movcm_Slot_pq_slot0_encode, Opcode_movcm_Slot_acc2_slot2_encode, 0, Opcode_movcm_Slot_acc2_slot0_encode, Opcode_movcm_Slot_smod_slot2_encode, 0, Opcode_movcm_Slot_smod_slot0_encode, Opcode_movcm_Slot_llr_slot2_encode, 0, Opcode_movcm_Slot_llr_slot0_encode, Opcode_movcm_Slot_dual_slot2_encode, 0, Opcode_movcm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movcm2pq_encode_fns[] = {
  Opcode_movcm2pq_Slot_inst_encode, 0, 0, Opcode_movcm2pq_Slot_gp_slot2_encode, 0, 0, Opcode_movcm2pq_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_movcm2pq_Slot_acc2_slot0_encode, Opcode_movcm2pq_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_movcm2pq_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movcnd_0_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_0_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_0_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_0_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_0_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_0_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_0_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_1_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_1_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_1_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_1_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_1_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_1_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_1_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_2_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_2_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_2_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_2_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_2_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_2_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_2_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_3_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_3_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_3_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_3_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_3_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_3_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_3_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_4_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_4_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_4_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_4_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_4_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_4_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_4_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_5_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_5_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_5_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_5_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_5_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_5_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_5_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_6_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_6_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_6_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_6_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_6_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_6_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_6_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd_7_encode_fns[] = {
  0, 0, 0, Opcode_movcnd_7_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd_7_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd_7_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd_7_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd_7_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd_7_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_0_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_0_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_0_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_0_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_0_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_0_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_0_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_1_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_1_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_1_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_1_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_1_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_1_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_1_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_2_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_2_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_2_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_2_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_2_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_2_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_2_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_3_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_3_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_3_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_3_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_3_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_3_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_3_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_4_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_4_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_4_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_4_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_4_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_4_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_4_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_5_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_5_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_5_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_5_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_5_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_5_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_5_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_6_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_6_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_6_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_6_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_6_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_6_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_6_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_movcnd8_7_encode_fns[] = {
  0, 0, 0, Opcode_movcnd8_7_Slot_gp_slot2_encode, 0, 0, Opcode_movcnd8_7_Slot_dot_slot2_encode, 0, 0, Opcode_movcnd8_7_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_movcnd8_7_Slot_smod_slot2_encode, 0, 0, Opcode_movcnd8_7_Slot_llr_slot2_encode, 0, 0, Opcode_movcnd8_7_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mov_i_encode_fns[] = {
  Opcode_mov_i_Slot_inst_encode, 0, 0, 0, 0, Opcode_mov_i_Slot_gp_slot0_encode, 0, 0, Opcode_mov_i_Slot_dot_slot0_encode, 0, 0, Opcode_mov_i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_mov_i_Slot_smod_slot0_encode, 0, 0, Opcode_mov_i_Slot_llr_slot0_encode, 0, 0, Opcode_mov_i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_movpq2pq_encode_fns[] = {
  0, 0, 0, Opcode_movpq2pq_Slot_gp_slot2_encode, 0, 0, Opcode_movpq2pq_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_movpq2pq_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mov_r_encode_fns[] = {
  Opcode_mov_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_mov_r_Slot_gp_slot0_encode, 0, 0, Opcode_mov_r_Slot_dot_slot0_encode, 0, 0, Opcode_mov_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_mov_r_Slot_smod_slot0_encode, 0, 0, Opcode_mov_r_Slot_llr_slot0_encode, 0, 0, Opcode_mov_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_negcm_encode_fns[] = {
  Opcode_negcm_Slot_inst_encode, 0, 0, Opcode_negcm_Slot_gp_slot2_encode, 0, Opcode_negcm_Slot_gp_slot0_encode, Opcode_negcm_Slot_dot_slot2_encode, 0, Opcode_negcm_Slot_dot_slot0_encode, Opcode_negcm_Slot_pq_slot2_encode, 0, Opcode_negcm_Slot_pq_slot0_encode, Opcode_negcm_Slot_acc2_slot2_encode, 0, Opcode_negcm_Slot_acc2_slot0_encode, Opcode_negcm_Slot_smod_slot2_encode, 0, Opcode_negcm_Slot_smod_slot0_encode, Opcode_negcm_Slot_llr_slot2_encode, 0, Opcode_negcm_Slot_llr_slot0_encode, Opcode_negcm_Slot_dual_slot2_encode, 0, Opcode_negcm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop16llr_1_encode_fns[] = {
  Opcode_pop16llr_1_Slot_inst_encode, 0, 0, 0, 0, Opcode_pop16llr_1_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop16llr_1_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_pop16llr_1_Slot_llr_slot0_encode, 0, 0, Opcode_pop16llr_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pq2cm_encode_fns[] = {
  Opcode_pq2cm_Slot_inst_encode, 0, 0, Opcode_pq2cm_Slot_gp_slot2_encode, 0, 0, Opcode_pq2cm_Slot_dot_slot2_encode, 0, 0, Opcode_pq2cm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_pq2cm_Slot_smod_slot2_encode, 0, 0, Opcode_pq2cm_Slot_llr_slot2_encode, 0, 0, Opcode_pq2cm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_swapac_r_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_swapac_r_Slot_acc2_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_swapac_ri_encode_fns[] = {
  Opcode_swapac_ri_Slot_inst_encode, 0, 0, 0, 0, Opcode_swapac_ri_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_swapac_ri_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_swapac_ri_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_swapb_encode_fns[] = {
  Opcode_swapb_Slot_inst_encode, 0, 0, Opcode_swapb_Slot_gp_slot2_encode, 0, 0, Opcode_swapb_Slot_dot_slot2_encode, 0, 0, Opcode_swapb_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_swapb_Slot_smod_slot2_encode, 0, 0, Opcode_swapb_Slot_llr_slot2_encode, 0, 0, Opcode_swapb_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_add2ac_encode_fns[] = {
  0, 0, 0, 0, Opcode_add2ac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_addac_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_addac_Slot_llr_slot1_encode, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cdot_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_cdot_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cdotac_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_cdotac_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cdotacs_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_cdotacs_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmac_encode_fns[] = {
  0, 0, 0, 0, Opcode_cmac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_cmac_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmacs_encode_fns[] = {
  0, 0, 0, 0, Opcode_cmacs_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmpy_encode_fns[] = {
  0, 0, 0, 0, Opcode_cmpy_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_cmpy_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmpy2cm_encode_fns[] = {
  0, 0, 0, 0, Opcode_cmpy2cm_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_cmpy2cm_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmpy2pq_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_cmpy2pq_Slot_pq_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmpys_encode_fns[] = {
  0, 0, 0, 0, Opcode_cmpys_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmpyxp2pq_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_cmpyxp2pq_Slot_pq_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_comb32_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_comb32_Slot_llr_slot1_encode, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dot_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_dot_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dotac_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_dotac_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_dotacs_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_dotacs_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lin_int_encode_fns[] = {
  0, 0, 0, 0, Opcode_lin_int_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_llrpre1_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_llrpre1_Slot_acc2_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_llrpre2_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_llrpre2_Slot_llr_slot1_encode, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mac_encode_fns[] = {
  0, 0, 0, 0, Opcode_mac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mac_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mac8_encode_fns[] = {
  0, 0, 0, 0, Opcode_mac8_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macd8_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macd8_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macpqxp_0_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macpqxp_0_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macpqxp_1_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macpqxp_1_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macpqxp_2_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macpqxp_2_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macpqxp_3_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macpqxp_3_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macs_encode_fns[] = {
  0, 0, 0, 0, Opcode_macs_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macxp2_0_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macxp2_0_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macxp2_1_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_macxp2_1_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macxp_0_encode_fns[] = {
  0, 0, 0, 0, Opcode_macxp_0_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macxp_1_encode_fns[] = {
  0, 0, 0, 0, Opcode_macxp_1_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macxp_2_encode_fns[] = {
  0, 0, 0, 0, Opcode_macxp_2_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_macxp_3_encode_fns[] = {
  0, 0, 0, 0, Opcode_macxp_3_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mov2ac_encode_fns[] = {
  0, 0, 0, 0, Opcode_mov2ac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpy_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpy_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mpy_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpy2cm_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpy2cm_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mpy2cm_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpy2pq_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mpy2pq_Slot_pq_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpy8_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpy8_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyadd8_2cm_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpyadd8_2cm_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mpyadd8_2cm_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyd8_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpyd8_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpypqxp_0_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpypqxp_0_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpypqxp_1_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpypqxp_1_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpypqxp_2_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpypqxp_2_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpypqxp_3_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpypqxp_3_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpys_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpys_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp2pq_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_mpyxp2pq_Slot_pq_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp2_0_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpyxp2_0_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp2_1_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, Opcode_mpyxp2_1_Slot_dot_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp_0_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpyxp_0_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp_1_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpyxp_1_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp_2_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpyxp_2_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mpyxp_3_encode_fns[] = {
  0, 0, 0, 0, Opcode_mpyxp_3_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_normacd_encode_fns[] = {
  0, 0, 0, 0, Opcode_normacd_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_normacd_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_normacpq_i_encode_fns[] = {
  0, 0, 0, 0, Opcode_normacpq_i_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_normacpq_r_encode_fns[] = {
  0, 0, 0, 0, Opcode_normacpq_r_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_normd_encode_fns[] = {
  0, 0, 0, 0, Opcode_normd_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_normd_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_normpypq_i_encode_fns[] = {
  0, 0, 0, 0, Opcode_normpypq_i_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_normpypq_r_encode_fns[] = {
  0, 0, 0, 0, Opcode_normpypq_r_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rcmac_encode_fns[] = {
  0, 0, 0, 0, Opcode_rcmac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rcmpy_encode_fns[] = {
  0, 0, 0, 0, Opcode_rcmpy_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rcmpy2cm_encode_fns[] = {
  0, 0, 0, 0, Opcode_rcmpy2cm_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rcmpy2cm_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfir_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rfir_Slot_acc2_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfira_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rfira_Slot_acc2_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfird_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rfird_Slot_acc2_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rfirda_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rfirda_Slot_acc2_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rmac_encode_fns[] = {
  0, 0, 0, 0, Opcode_rmac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rmpy_encode_fns[] = {
  0, 0, 0, 0, Opcode_rmpy_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rmpy2cm_encode_fns[] = {
  0, 0, 0, 0, Opcode_rmpy2cm_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_rmpy2cm_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_smod_align_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_smod_align_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_smod_scr_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_smod_scr_Slot_smod_slot1_encode, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_sub2ac_encode_fns[] = {
  0, 0, 0, 0, Opcode_sub2ac_Slot_gp_slot1_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wght32_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_wght32_Slot_llr_slot1_encode, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_clrtiep_encode_fns[] = {
  Opcode_clrtiep_Slot_inst_encode, 0, 0, Opcode_clrtiep_Slot_gp_slot2_encode, 0, 0, Opcode_clrtiep_Slot_dot_slot2_encode, 0, 0, Opcode_clrtiep_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_clrtiep_Slot_smod_slot2_encode, 0, 0, Opcode_clrtiep_Slot_llr_slot2_encode, 0, 0, Opcode_clrtiep_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ext_2fifo_0_encode_fns[] = {
  Opcode_ext_2fifo_0_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_2fifo_0_Slot_gp_slot0_encode, 0, 0, Opcode_ext_2fifo_0_Slot_dot_slot0_encode, 0, 0, Opcode_ext_2fifo_0_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_2fifo_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_2fifo_1_encode_fns[] = {
  Opcode_ext_2fifo_1_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_2fifo_1_Slot_gp_slot0_encode, 0, 0, Opcode_ext_2fifo_1_Slot_dot_slot0_encode, 0, 0, Opcode_ext_2fifo_1_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_2fifo_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_2fifo_2_encode_fns[] = {
  Opcode_ext_2fifo_2_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_2fifo_2_Slot_gp_slot0_encode, 0, 0, Opcode_ext_2fifo_2_Slot_dot_slot0_encode, 0, 0, Opcode_ext_2fifo_2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_2fifo_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_2fifo_3_encode_fns[] = {
  Opcode_ext_2fifo_3_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_2fifo_3_Slot_gp_slot0_encode, 0, 0, Opcode_ext_2fifo_3_Slot_dot_slot0_encode, 0, 0, Opcode_ext_2fifo_3_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_2fifo_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_r2fifo_0_encode_fns[] = {
  Opcode_ext_r2fifo_0_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_r2fifo_0_Slot_gp_slot0_encode, 0, 0, Opcode_ext_r2fifo_0_Slot_dot_slot0_encode, 0, 0, Opcode_ext_r2fifo_0_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_r2fifo_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_r2fifo_1_encode_fns[] = {
  Opcode_ext_r2fifo_1_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_r2fifo_1_Slot_gp_slot0_encode, 0, 0, Opcode_ext_r2fifo_1_Slot_dot_slot0_encode, 0, 0, Opcode_ext_r2fifo_1_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_r2fifo_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_r2fifo_2_encode_fns[] = {
  Opcode_ext_r2fifo_2_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_r2fifo_2_Slot_gp_slot0_encode, 0, 0, Opcode_ext_r2fifo_2_Slot_dot_slot0_encode, 0, 0, Opcode_ext_r2fifo_2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_r2fifo_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_r2fifo_3_encode_fns[] = {
  Opcode_ext_r2fifo_3_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_r2fifo_3_Slot_gp_slot0_encode, 0, 0, Opcode_ext_r2fifo_3_Slot_dot_slot0_encode, 0, 0, Opcode_ext_r2fifo_3_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_r2fifo_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_lut_encode_fns[] = {
  0, 0, 0, Opcode_lut_Slot_gp_slot2_encode, 0, 0, Opcode_lut_Slot_dot_slot2_encode, 0, 0, Opcode_lut_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_lut_Slot_smod_slot2_encode, 0, 0, Opcode_lut_Slot_llr_slot2_encode, 0, 0, Opcode_lut_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lut_ar_encode_fns[] = {
  0, 0, 0, Opcode_lut_ar_Slot_gp_slot2_encode, 0, 0, Opcode_lut_ar_Slot_dot_slot2_encode, 0, 0, Opcode_lut_ar_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_lut_ar_Slot_smod_slot2_encode, 0, 0, Opcode_lut_ar_Slot_llr_slot2_encode, 0, 0, Opcode_lut_ar_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lut_iext_encode_fns[] = {
  0, 0, 0, Opcode_lut_iext_Slot_gp_slot2_encode, 0, 0, Opcode_lut_iext_Slot_dot_slot2_encode, 0, 0, Opcode_lut_iext_Slot_pq_slot2_encode, 0, 0, Opcode_lut_iext_Slot_acc2_slot2_encode, 0, 0, Opcode_lut_iext_Slot_smod_slot2_encode, 0, 0, Opcode_lut_iext_Slot_llr_slot2_encode, 0, 0, Opcode_lut_iext_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lut_phasor_encode_fns[] = {
  0, 0, 0, Opcode_lut_phasor_Slot_gp_slot2_encode, 0, 0, Opcode_lut_phasor_Slot_dot_slot2_encode, 0, 0, Opcode_lut_phasor_Slot_pq_slot2_encode, 0, 0, Opcode_lut_phasor_Slot_acc2_slot2_encode, 0, 0, Opcode_lut_phasor_Slot_smod_slot2_encode, 0, 0, Opcode_lut_phasor_Slot_llr_slot2_encode, 0, 0, Opcode_lut_phasor_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lut_rext_encode_fns[] = {
  0, 0, 0, Opcode_lut_rext_Slot_gp_slot2_encode, 0, 0, Opcode_lut_rext_Slot_dot_slot2_encode, 0, 0, Opcode_lut_rext_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_lut_rext_Slot_smod_slot2_encode, 0, 0, Opcode_lut_rext_Slot_llr_slot2_encode, 0, 0, Opcode_lut_rext_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lut_write_encode_fns[] = {
  0, 0, 0, Opcode_lut_write_Slot_gp_slot2_encode, 0, 0, Opcode_lut_write_Slot_dot_slot2_encode, 0, 0, Opcode_lut_write_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_lut_write_Slot_smod_slot2_encode, 0, 0, Opcode_lut_write_Slot_llr_slot2_encode, 0, 0, Opcode_lut_write_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq128_0_encode_fns[] = {
  Opcode_moveq128_0_Slot_inst_encode, 0, 0, Opcode_moveq128_0_Slot_gp_slot2_encode, 0, 0, Opcode_moveq128_0_Slot_dot_slot2_encode, 0, 0, Opcode_moveq128_0_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq128_0_Slot_smod_slot2_encode, 0, 0, Opcode_moveq128_0_Slot_llr_slot2_encode, 0, 0, Opcode_moveq128_0_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq128_1_encode_fns[] = {
  Opcode_moveq128_1_Slot_inst_encode, 0, 0, Opcode_moveq128_1_Slot_gp_slot2_encode, 0, 0, Opcode_moveq128_1_Slot_dot_slot2_encode, 0, 0, Opcode_moveq128_1_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq128_1_Slot_smod_slot2_encode, 0, 0, Opcode_moveq128_1_Slot_llr_slot2_encode, 0, 0, Opcode_moveq128_1_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq128_2_encode_fns[] = {
  Opcode_moveq128_2_Slot_inst_encode, 0, 0, Opcode_moveq128_2_Slot_gp_slot2_encode, 0, 0, Opcode_moveq128_2_Slot_dot_slot2_encode, 0, 0, Opcode_moveq128_2_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq128_2_Slot_smod_slot2_encode, 0, 0, Opcode_moveq128_2_Slot_llr_slot2_encode, 0, 0, Opcode_moveq128_2_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq128_3_encode_fns[] = {
  Opcode_moveq128_3_Slot_inst_encode, 0, 0, Opcode_moveq128_3_Slot_gp_slot2_encode, 0, 0, Opcode_moveq128_3_Slot_dot_slot2_encode, 0, 0, Opcode_moveq128_3_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq128_3_Slot_smod_slot2_encode, 0, 0, Opcode_moveq128_3_Slot_llr_slot2_encode, 0, 0, Opcode_moveq128_3_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq128_4_encode_fns[] = {
  Opcode_moveq128_4_Slot_inst_encode, 0, 0, Opcode_moveq128_4_Slot_gp_slot2_encode, 0, 0, Opcode_moveq128_4_Slot_dot_slot2_encode, 0, 0, Opcode_moveq128_4_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq128_4_Slot_smod_slot2_encode, 0, 0, Opcode_moveq128_4_Slot_llr_slot2_encode, 0, 0, Opcode_moveq128_4_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq128_5_encode_fns[] = {
  Opcode_moveq128_5_Slot_inst_encode, 0, 0, Opcode_moveq128_5_Slot_gp_slot2_encode, 0, 0, Opcode_moveq128_5_Slot_dot_slot2_encode, 0, 0, Opcode_moveq128_5_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq128_5_Slot_smod_slot2_encode, 0, 0, Opcode_moveq128_5_Slot_llr_slot2_encode, 0, 0, Opcode_moveq128_5_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq32_0_encode_fns[] = {
  Opcode_moveq32_0_Slot_inst_encode, 0, 0, Opcode_moveq32_0_Slot_gp_slot2_encode, 0, 0, Opcode_moveq32_0_Slot_dot_slot2_encode, 0, 0, Opcode_moveq32_0_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq32_0_Slot_smod_slot2_encode, 0, 0, Opcode_moveq32_0_Slot_llr_slot2_encode, 0, 0, Opcode_moveq32_0_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq32_1_encode_fns[] = {
  Opcode_moveq32_1_Slot_inst_encode, 0, 0, Opcode_moveq32_1_Slot_gp_slot2_encode, 0, 0, Opcode_moveq32_1_Slot_dot_slot2_encode, 0, 0, Opcode_moveq32_1_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq32_1_Slot_smod_slot2_encode, 0, 0, Opcode_moveq32_1_Slot_llr_slot2_encode, 0, 0, Opcode_moveq32_1_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq32_2_encode_fns[] = {
  Opcode_moveq32_2_Slot_inst_encode, 0, 0, Opcode_moveq32_2_Slot_gp_slot2_encode, 0, 0, Opcode_moveq32_2_Slot_dot_slot2_encode, 0, 0, Opcode_moveq32_2_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq32_2_Slot_smod_slot2_encode, 0, 0, Opcode_moveq32_2_Slot_llr_slot2_encode, 0, 0, Opcode_moveq32_2_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_moveq32_3_encode_fns[] = {
  Opcode_moveq32_3_Slot_inst_encode, 0, 0, Opcode_moveq32_3_Slot_gp_slot2_encode, 0, 0, Opcode_moveq32_3_Slot_dot_slot2_encode, 0, 0, Opcode_moveq32_3_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_moveq32_3_Slot_smod_slot2_encode, 0, 0, Opcode_moveq32_3_Slot_llr_slot2_encode, 0, 0, Opcode_moveq32_3_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_nco_update_encode_fns[] = {
  0, 0, 0, Opcode_nco_update_Slot_gp_slot2_encode, 0, 0, Opcode_nco_update_Slot_dot_slot2_encode, 0, 0, Opcode_nco_update_Slot_pq_slot2_encode, 0, 0, Opcode_nco_update_Slot_acc2_slot2_encode, 0, 0, Opcode_nco_update_Slot_smod_slot2_encode, 0, 0, Opcode_nco_update_Slot_llr_slot2_encode, 0, 0, Opcode_nco_update_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pop128_0_encode_fns[] = {
  Opcode_pop128_0_Slot_inst_encode, 0, 0, Opcode_pop128_0_Slot_gp_slot2_encode, 0, Opcode_pop128_0_Slot_gp_slot0_encode, Opcode_pop128_0_Slot_dot_slot2_encode, 0, Opcode_pop128_0_Slot_dot_slot0_encode, Opcode_pop128_0_Slot_pq_slot2_encode, 0, Opcode_pop128_0_Slot_pq_slot0_encode, Opcode_pop128_0_Slot_acc2_slot2_encode, 0, Opcode_pop128_0_Slot_acc2_slot0_encode, Opcode_pop128_0_Slot_smod_slot2_encode, 0, Opcode_pop128_0_Slot_smod_slot0_encode, Opcode_pop128_0_Slot_llr_slot2_encode, 0, Opcode_pop128_0_Slot_llr_slot0_encode, Opcode_pop128_0_Slot_dual_slot2_encode, 0, Opcode_pop128_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_1_encode_fns[] = {
  Opcode_pop128_1_Slot_inst_encode, 0, 0, Opcode_pop128_1_Slot_gp_slot2_encode, 0, Opcode_pop128_1_Slot_gp_slot0_encode, Opcode_pop128_1_Slot_dot_slot2_encode, 0, Opcode_pop128_1_Slot_dot_slot0_encode, Opcode_pop128_1_Slot_pq_slot2_encode, 0, Opcode_pop128_1_Slot_pq_slot0_encode, Opcode_pop128_1_Slot_acc2_slot2_encode, 0, Opcode_pop128_1_Slot_acc2_slot0_encode, Opcode_pop128_1_Slot_smod_slot2_encode, 0, Opcode_pop128_1_Slot_smod_slot0_encode, Opcode_pop128_1_Slot_llr_slot2_encode, 0, Opcode_pop128_1_Slot_llr_slot0_encode, Opcode_pop128_1_Slot_dual_slot2_encode, 0, Opcode_pop128_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2_encode_fns[] = {
  Opcode_pop128_2_Slot_inst_encode, 0, 0, Opcode_pop128_2_Slot_gp_slot2_encode, 0, Opcode_pop128_2_Slot_gp_slot0_encode, Opcode_pop128_2_Slot_dot_slot2_encode, 0, Opcode_pop128_2_Slot_dot_slot0_encode, Opcode_pop128_2_Slot_pq_slot2_encode, 0, Opcode_pop128_2_Slot_pq_slot0_encode, Opcode_pop128_2_Slot_acc2_slot2_encode, 0, Opcode_pop128_2_Slot_acc2_slot0_encode, Opcode_pop128_2_Slot_smod_slot2_encode, 0, Opcode_pop128_2_Slot_smod_slot0_encode, Opcode_pop128_2_Slot_llr_slot2_encode, 0, Opcode_pop128_2_Slot_llr_slot0_encode, Opcode_pop128_2_Slot_dual_slot2_encode, 0, Opcode_pop128_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_3_encode_fns[] = {
  Opcode_pop128_3_Slot_inst_encode, 0, 0, Opcode_pop128_3_Slot_gp_slot2_encode, 0, Opcode_pop128_3_Slot_gp_slot0_encode, Opcode_pop128_3_Slot_dot_slot2_encode, 0, Opcode_pop128_3_Slot_dot_slot0_encode, Opcode_pop128_3_Slot_pq_slot2_encode, 0, Opcode_pop128_3_Slot_pq_slot0_encode, Opcode_pop128_3_Slot_acc2_slot2_encode, 0, Opcode_pop128_3_Slot_acc2_slot0_encode, Opcode_pop128_3_Slot_smod_slot2_encode, 0, Opcode_pop128_3_Slot_smod_slot0_encode, Opcode_pop128_3_Slot_llr_slot2_encode, 0, Opcode_pop128_3_Slot_llr_slot0_encode, Opcode_pop128_3_Slot_dual_slot2_encode, 0, Opcode_pop128_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_4_encode_fns[] = {
  Opcode_pop128_4_Slot_inst_encode, 0, 0, Opcode_pop128_4_Slot_gp_slot2_encode, 0, Opcode_pop128_4_Slot_gp_slot0_encode, Opcode_pop128_4_Slot_dot_slot2_encode, 0, Opcode_pop128_4_Slot_dot_slot0_encode, Opcode_pop128_4_Slot_pq_slot2_encode, 0, Opcode_pop128_4_Slot_pq_slot0_encode, Opcode_pop128_4_Slot_acc2_slot2_encode, 0, Opcode_pop128_4_Slot_acc2_slot0_encode, Opcode_pop128_4_Slot_smod_slot2_encode, 0, Opcode_pop128_4_Slot_smod_slot0_encode, Opcode_pop128_4_Slot_llr_slot2_encode, 0, Opcode_pop128_4_Slot_llr_slot0_encode, Opcode_pop128_4_Slot_dual_slot2_encode, 0, Opcode_pop128_4_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_5_encode_fns[] = {
  Opcode_pop128_5_Slot_inst_encode, 0, 0, Opcode_pop128_5_Slot_gp_slot2_encode, 0, Opcode_pop128_5_Slot_gp_slot0_encode, Opcode_pop128_5_Slot_dot_slot2_encode, 0, Opcode_pop128_5_Slot_dot_slot0_encode, Opcode_pop128_5_Slot_pq_slot2_encode, 0, Opcode_pop128_5_Slot_pq_slot0_encode, Opcode_pop128_5_Slot_acc2_slot2_encode, 0, Opcode_pop128_5_Slot_acc2_slot0_encode, Opcode_pop128_5_Slot_smod_slot2_encode, 0, Opcode_pop128_5_Slot_smod_slot0_encode, Opcode_pop128_5_Slot_llr_slot2_encode, 0, Opcode_pop128_5_Slot_llr_slot0_encode, Opcode_pop128_5_Slot_dual_slot2_encode, 0, Opcode_pop128_5_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2cmpq_0_encode_fns[] = {
  0, 0, 0, Opcode_pop128_2cmpq_0_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2cmpq_0_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2cmpq_0_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pop128_2cmpq_1_encode_fns[] = {
  0, 0, 0, Opcode_pop128_2cmpq_1_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2cmpq_1_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2cmpq_1_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pop128_2cmpq_2_encode_fns[] = {
  0, 0, 0, Opcode_pop128_2cmpq_2_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2cmpq_2_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2cmpq_2_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pop128_2cmpq_3_encode_fns[] = {
  0, 0, 0, Opcode_pop128_2cmpq_3_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2cmpq_3_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2cmpq_3_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_pop128_2m_0_encode_fns[] = {
  Opcode_pop128_2m_0_Slot_inst_encode, 0, 0, 0, 0, Opcode_pop128_2m_0_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_0_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_0_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2m_1_encode_fns[] = {
  Opcode_pop128_2m_1_Slot_inst_encode, 0, 0, 0, 0, Opcode_pop128_2m_1_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_1_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_1_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2m_2_encode_fns[] = {
  Opcode_pop128_2m_2_Slot_inst_encode, 0, 0, 0, 0, Opcode_pop128_2m_2_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_2_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2m_3_encode_fns[] = {
  Opcode_pop128_2m_3_Slot_inst_encode, 0, 0, 0, 0, Opcode_pop128_2m_3_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_3_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_3_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_pop128_2m_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2pq_0_encode_fns[] = {
  Opcode_pop128_2pq_0_Slot_inst_encode, 0, 0, Opcode_pop128_2pq_0_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2pq_0_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_0_Slot_acc2_slot0_encode, Opcode_pop128_2pq_0_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2pq_1_encode_fns[] = {
  Opcode_pop128_2pq_1_Slot_inst_encode, 0, 0, Opcode_pop128_2pq_1_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2pq_1_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_1_Slot_acc2_slot0_encode, Opcode_pop128_2pq_1_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2pq_2_encode_fns[] = {
  Opcode_pop128_2pq_2_Slot_inst_encode, 0, 0, Opcode_pop128_2pq_2_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2pq_2_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_2_Slot_acc2_slot0_encode, Opcode_pop128_2pq_2_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2pq_3_encode_fns[] = {
  Opcode_pop128_2pq_3_Slot_inst_encode, 0, 0, Opcode_pop128_2pq_3_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2pq_3_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_3_Slot_acc2_slot0_encode, Opcode_pop128_2pq_3_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2pq_4_encode_fns[] = {
  Opcode_pop128_2pq_4_Slot_inst_encode, 0, 0, Opcode_pop128_2pq_4_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2pq_4_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_4_Slot_acc2_slot0_encode, Opcode_pop128_2pq_4_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_4_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop128_2pq_5_encode_fns[] = {
  Opcode_pop128_2pq_5_Slot_inst_encode, 0, 0, Opcode_pop128_2pq_5_Slot_gp_slot2_encode, 0, 0, Opcode_pop128_2pq_5_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_5_Slot_acc2_slot0_encode, Opcode_pop128_2pq_5_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop128_2pq_5_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop2x128_2pq_01_encode_fns[] = {
  Opcode_pop2x128_2pq_01_Slot_inst_encode, 0, 0, Opcode_pop2x128_2pq_01_Slot_gp_slot2_encode, 0, 0, Opcode_pop2x128_2pq_01_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_01_Slot_acc2_slot0_encode, Opcode_pop2x128_2pq_01_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_01_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop2x128_2pq_03_encode_fns[] = {
  Opcode_pop2x128_2pq_03_Slot_inst_encode, 0, 0, Opcode_pop2x128_2pq_03_Slot_gp_slot2_encode, 0, 0, Opcode_pop2x128_2pq_03_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_03_Slot_acc2_slot0_encode, Opcode_pop2x128_2pq_03_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_03_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop2x128_2pq_21_encode_fns[] = {
  Opcode_pop2x128_2pq_21_Slot_inst_encode, 0, 0, Opcode_pop2x128_2pq_21_Slot_gp_slot2_encode, 0, 0, Opcode_pop2x128_2pq_21_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_21_Slot_acc2_slot0_encode, Opcode_pop2x128_2pq_21_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_21_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop2x128_2pq_23_encode_fns[] = {
  Opcode_pop2x128_2pq_23_Slot_inst_encode, 0, 0, Opcode_pop2x128_2pq_23_Slot_gp_slot2_encode, 0, 0, Opcode_pop2x128_2pq_23_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_23_Slot_acc2_slot0_encode, Opcode_pop2x128_2pq_23_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_pop2x128_2pq_23_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop32_0_encode_fns[] = {
  Opcode_pop32_0_Slot_inst_encode, 0, 0, Opcode_pop32_0_Slot_gp_slot2_encode, 0, Opcode_pop32_0_Slot_gp_slot0_encode, Opcode_pop32_0_Slot_dot_slot2_encode, 0, Opcode_pop32_0_Slot_dot_slot0_encode, Opcode_pop32_0_Slot_pq_slot2_encode, 0, Opcode_pop32_0_Slot_pq_slot0_encode, Opcode_pop32_0_Slot_acc2_slot2_encode, 0, Opcode_pop32_0_Slot_acc2_slot0_encode, Opcode_pop32_0_Slot_smod_slot2_encode, 0, Opcode_pop32_0_Slot_smod_slot0_encode, Opcode_pop32_0_Slot_llr_slot2_encode, 0, Opcode_pop32_0_Slot_llr_slot0_encode, Opcode_pop32_0_Slot_dual_slot2_encode, 0, Opcode_pop32_0_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop32_1_encode_fns[] = {
  Opcode_pop32_1_Slot_inst_encode, 0, 0, Opcode_pop32_1_Slot_gp_slot2_encode, 0, Opcode_pop32_1_Slot_gp_slot0_encode, Opcode_pop32_1_Slot_dot_slot2_encode, 0, Opcode_pop32_1_Slot_dot_slot0_encode, Opcode_pop32_1_Slot_pq_slot2_encode, 0, Opcode_pop32_1_Slot_pq_slot0_encode, Opcode_pop32_1_Slot_acc2_slot2_encode, 0, Opcode_pop32_1_Slot_acc2_slot0_encode, Opcode_pop32_1_Slot_smod_slot2_encode, 0, Opcode_pop32_1_Slot_smod_slot0_encode, Opcode_pop32_1_Slot_llr_slot2_encode, 0, Opcode_pop32_1_Slot_llr_slot0_encode, Opcode_pop32_1_Slot_dual_slot2_encode, 0, Opcode_pop32_1_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop32_2_encode_fns[] = {
  Opcode_pop32_2_Slot_inst_encode, 0, 0, Opcode_pop32_2_Slot_gp_slot2_encode, 0, Opcode_pop32_2_Slot_gp_slot0_encode, Opcode_pop32_2_Slot_dot_slot2_encode, 0, Opcode_pop32_2_Slot_dot_slot0_encode, Opcode_pop32_2_Slot_pq_slot2_encode, 0, Opcode_pop32_2_Slot_pq_slot0_encode, Opcode_pop32_2_Slot_acc2_slot2_encode, 0, Opcode_pop32_2_Slot_acc2_slot0_encode, Opcode_pop32_2_Slot_smod_slot2_encode, 0, Opcode_pop32_2_Slot_smod_slot0_encode, Opcode_pop32_2_Slot_llr_slot2_encode, 0, Opcode_pop32_2_Slot_llr_slot0_encode, Opcode_pop32_2_Slot_dual_slot2_encode, 0, Opcode_pop32_2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_pop32_3_encode_fns[] = {
  Opcode_pop32_3_Slot_inst_encode, 0, 0, Opcode_pop32_3_Slot_gp_slot2_encode, 0, Opcode_pop32_3_Slot_gp_slot0_encode, Opcode_pop32_3_Slot_dot_slot2_encode, 0, Opcode_pop32_3_Slot_dot_slot0_encode, Opcode_pop32_3_Slot_pq_slot2_encode, 0, Opcode_pop32_3_Slot_pq_slot0_encode, Opcode_pop32_3_Slot_acc2_slot2_encode, 0, Opcode_pop32_3_Slot_acc2_slot0_encode, Opcode_pop32_3_Slot_smod_slot2_encode, 0, Opcode_pop32_3_Slot_smod_slot0_encode, Opcode_pop32_3_Slot_llr_slot2_encode, 0, Opcode_pop32_3_Slot_llr_slot0_encode, Opcode_pop32_3_Slot_dual_slot2_encode, 0, Opcode_pop32_3_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_push128_encode_fns[] = {
  Opcode_push128_Slot_inst_encode, 0, 0, Opcode_push128_Slot_gp_slot2_encode, 0, Opcode_push128_Slot_gp_slot0_encode, Opcode_push128_Slot_dot_slot2_encode, 0, Opcode_push128_Slot_dot_slot0_encode, Opcode_push128_Slot_pq_slot2_encode, 0, Opcode_push128_Slot_pq_slot0_encode, Opcode_push128_Slot_acc2_slot2_encode, 0, Opcode_push128_Slot_acc2_slot0_encode, Opcode_push128_Slot_smod_slot2_encode, 0, Opcode_push128_Slot_smod_slot0_encode, Opcode_push128_Slot_llr_slot2_encode, 0, Opcode_push128_Slot_llr_slot0_encode, Opcode_push128_Slot_dual_slot2_encode, 0, Opcode_push128_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_push128_m_encode_fns[] = {
  Opcode_push128_m_Slot_inst_encode, 0, 0, 0, 0, Opcode_push128_m_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_push128_m_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, Opcode_push128_m_Slot_smod_slot0_encode, 0, 0, 0, 0, 0, Opcode_push128_m_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_push128_pq_encode_fns[] = {
  Opcode_push128_pq_Slot_inst_encode, 0, 0, Opcode_push128_pq_Slot_gp_slot2_encode, 0, 0, Opcode_push128_pq_Slot_dot_slot2_encode, 0, 0, Opcode_push128_pq_Slot_pq_slot2_encode, 0, 0, 0, 0, Opcode_push128_pq_Slot_acc2_slot0_encode, Opcode_push128_pq_Slot_smod_slot2_encode, 0, 0, Opcode_push128_pq_Slot_llr_slot2_encode, 0, Opcode_push128_pq_Slot_llr_slot0_encode, Opcode_push128_pq_Slot_dual_slot2_encode, 0, Opcode_push128_pq_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_push2x128_pq_encode_fns[] = {
  Opcode_push2x128_pq_Slot_inst_encode, 0, 0, Opcode_push2x128_pq_Slot_gp_slot2_encode, 0, 0, Opcode_push2x128_pq_Slot_dot_slot2_encode, 0, 0, 0, 0, 0, 0, 0, Opcode_push2x128_pq_Slot_acc2_slot0_encode, Opcode_push2x128_pq_Slot_smod_slot2_encode, 0, 0, 0, 0, Opcode_push2x128_pq_Slot_llr_slot0_encode, 0, 0, Opcode_push2x128_pq_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_push32_encode_fns[] = {
  Opcode_push32_Slot_inst_encode, 0, 0, Opcode_push32_Slot_gp_slot2_encode, 0, Opcode_push32_Slot_gp_slot0_encode, Opcode_push32_Slot_dot_slot2_encode, 0, Opcode_push32_Slot_dot_slot0_encode, Opcode_push32_Slot_pq_slot2_encode, 0, Opcode_push32_Slot_pq_slot0_encode, Opcode_push32_Slot_acc2_slot2_encode, 0, 0, Opcode_push32_Slot_smod_slot2_encode, 0, Opcode_push32_Slot_smod_slot0_encode, Opcode_push32_Slot_llr_slot2_encode, 0, Opcode_push32_Slot_llr_slot0_encode, Opcode_push32_Slot_dual_slot2_encode, 0, Opcode_push32_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_qready_encode_fns[] = {
  Opcode_qready_Slot_inst_encode, 0, 0, Opcode_qready_Slot_gp_slot2_encode, 0, 0, Opcode_qready_Slot_dot_slot2_encode, 0, 0, Opcode_qready_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_qready_Slot_smod_slot2_encode, 0, 0, Opcode_qready_Slot_llr_slot2_encode, 0, 0, Opcode_qready_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rdtiep_encode_fns[] = {
  Opcode_rdtiep_Slot_inst_encode, 0, 0, Opcode_rdtiep_Slot_gp_slot2_encode, 0, 0, Opcode_rdtiep_Slot_dot_slot2_encode, 0, 0, Opcode_rdtiep_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_rdtiep_Slot_smod_slot2_encode, 0, 0, Opcode_rdtiep_Slot_llr_slot2_encode, 0, 0, Opcode_rdtiep_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_settiep_encode_fns[] = {
  Opcode_settiep_Slot_inst_encode, 0, 0, Opcode_settiep_Slot_gp_slot2_encode, 0, 0, Opcode_settiep_Slot_dot_slot2_encode, 0, 0, Opcode_settiep_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_settiep_Slot_smod_slot2_encode, 0, 0, Opcode_settiep_Slot_llr_slot2_encode, 0, 0, Opcode_settiep_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_smod_lut_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_smod_lut_Slot_smod_slot2_encode, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wrtbsigq_encode_fns[] = {
  Opcode_wrtbsigq_Slot_inst_encode, 0, 0, Opcode_wrtbsigq_Slot_gp_slot2_encode, 0, 0, Opcode_wrtbsigq_Slot_dot_slot2_encode, 0, 0, Opcode_wrtbsigq_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_wrtbsigq_Slot_smod_slot2_encode, 0, 0, Opcode_wrtbsigq_Slot_llr_slot2_encode, 0, 0, Opcode_wrtbsigq_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wrtbsigqm_encode_fns[] = {
  Opcode_wrtbsigqm_Slot_inst_encode, 0, 0, Opcode_wrtbsigqm_Slot_gp_slot2_encode, 0, 0, Opcode_wrtbsigqm_Slot_dot_slot2_encode, 0, 0, Opcode_wrtbsigqm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_wrtbsigqm_Slot_smod_slot2_encode, 0, 0, Opcode_wrtbsigqm_Slot_llr_slot2_encode, 0, 0, Opcode_wrtbsigqm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wrtiep_encode_fns[] = {
  Opcode_wrtiep_Slot_inst_encode, 0, 0, Opcode_wrtiep_Slot_gp_slot2_encode, 0, 0, Opcode_wrtiep_Slot_dot_slot2_encode, 0, 0, Opcode_wrtiep_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_wrtiep_Slot_smod_slot2_encode, 0, 0, Opcode_wrtiep_Slot_llr_slot2_encode, 0, 0, Opcode_wrtiep_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wrtsigq_encode_fns[] = {
  Opcode_wrtsigq_Slot_inst_encode, 0, 0, Opcode_wrtsigq_Slot_gp_slot2_encode, 0, 0, Opcode_wrtsigq_Slot_dot_slot2_encode, 0, 0, Opcode_wrtsigq_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_wrtsigq_Slot_smod_slot2_encode, 0, 0, Opcode_wrtsigq_Slot_llr_slot2_encode, 0, 0, Opcode_wrtsigq_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_abs8_encode_fns[] = {
  0, 0, 0, Opcode_abs8_Slot_gp_slot2_encode, 0, 0, Opcode_abs8_Slot_dot_slot2_encode, 0, 0, Opcode_abs8_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_abs8_Slot_smod_slot2_encode, 0, 0, Opcode_abs8_Slot_llr_slot2_encode, 0, 0, Opcode_abs8_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_add16_encode_fns[] = {
  0, 0, 0, Opcode_add16_Slot_gp_slot2_encode, 0, 0, Opcode_add16_Slot_dot_slot2_encode, 0, 0, Opcode_add16_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_add16_Slot_smod_slot2_encode, 0, 0, Opcode_add16_Slot_llr_slot2_encode, 0, 0, Opcode_add16_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_add32_encode_fns[] = {
  0, 0, 0, Opcode_add32_Slot_gp_slot2_encode, 0, 0, Opcode_add32_Slot_dot_slot2_encode, 0, 0, Opcode_add32_Slot_pq_slot2_encode, 0, 0, Opcode_add32_Slot_acc2_slot2_encode, 0, 0, Opcode_add32_Slot_smod_slot2_encode, 0, 0, Opcode_add32_Slot_llr_slot2_encode, 0, 0, Opcode_add32_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_addac_i2r_encode_fns[] = {
  Opcode_addac_i2r_Slot_inst_encode, 0, 0, 0, 0, Opcode_addac_i2r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_addac_i2r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_addac_i2r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_addac_r2i_encode_fns[] = {
  Opcode_addac_r2i_Slot_inst_encode, 0, 0, 0, 0, Opcode_addac_r2i_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_addac_r2i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_addac_r2i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_addar2_encode_fns[] = {
  0, 0, 0, Opcode_addar2_Slot_gp_slot2_encode, 0, 0, Opcode_addar2_Slot_dot_slot2_encode, 0, 0, Opcode_addar2_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_addar2_Slot_smod_slot2_encode, 0, 0, Opcode_addar2_Slot_llr_slot2_encode, 0, 0, Opcode_addar2_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_addcm_encode_fns[] = {
  0, 0, 0, Opcode_addcm_Slot_gp_slot2_encode, 0, 0, Opcode_addcm_Slot_dot_slot2_encode, 0, 0, Opcode_addcm_Slot_pq_slot2_encode, 0, 0, Opcode_addcm_Slot_acc2_slot2_encode, 0, 0, Opcode_addcm_Slot_smod_slot2_encode, 0, 0, Opcode_addcm_Slot_llr_slot2_encode, 0, 0, Opcode_addcm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_addwrp_encode_fns[] = {
  0, 0, 0, Opcode_addwrp_Slot_gp_slot2_encode, 0, 0, Opcode_addwrp_Slot_dot_slot2_encode, 0, 0, Opcode_addwrp_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_addwrp_Slot_smod_slot2_encode, 0, 0, Opcode_addwrp_Slot_llr_slot2_encode, 0, 0, Opcode_addwrp_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_and128_encode_fns[] = {
  0, 0, 0, Opcode_and128_Slot_gp_slot2_encode, 0, 0, Opcode_and128_Slot_dot_slot2_encode, 0, 0, Opcode_and128_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_and128_Slot_smod_slot2_encode, 0, 0, Opcode_and128_Slot_llr_slot2_encode, 0, 0, Opcode_and128_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_argmax8_encode_fns[] = {
  0, 0, 0, Opcode_argmax8_Slot_gp_slot2_encode, 0, 0, Opcode_argmax8_Slot_dot_slot2_encode, 0, 0, Opcode_argmax8_Slot_pq_slot2_encode, 0, 0, Opcode_argmax8_Slot_acc2_slot2_encode, 0, 0, Opcode_argmax8_Slot_smod_slot2_encode, 0, 0, Opcode_argmax8_Slot_llr_slot2_encode, 0, 0, Opcode_argmax8_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_asl_encode_fns[] = {
  0, 0, 0, Opcode_asl_Slot_gp_slot2_encode, 0, 0, Opcode_asl_Slot_dot_slot2_encode, 0, 0, Opcode_asl_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_asl_Slot_smod_slot2_encode, 0, 0, Opcode_asl_Slot_llr_slot2_encode, 0, 0, Opcode_asl_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_asl32_encode_fns[] = {
  0, 0, 0, Opcode_asl32_Slot_gp_slot2_encode, 0, 0, Opcode_asl32_Slot_dot_slot2_encode, 0, 0, Opcode_asl32_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_asl32_Slot_smod_slot2_encode, 0, 0, Opcode_asl32_Slot_llr_slot2_encode, 0, 0, Opcode_asl32_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_aslacm_encode_fns[] = {
  Opcode_aslacm_Slot_inst_encode, 0, 0, 0, 0, Opcode_aslacm_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_aslacm_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_aslacm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_aslm_encode_fns[] = {
  0, 0, 0, Opcode_aslm_Slot_gp_slot2_encode, 0, 0, Opcode_aslm_Slot_dot_slot2_encode, 0, 0, Opcode_aslm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_aslm_Slot_smod_slot2_encode, 0, 0, Opcode_aslm_Slot_llr_slot2_encode, 0, 0, Opcode_aslm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_aslm32_encode_fns[] = {
  0, 0, 0, Opcode_aslm32_Slot_gp_slot2_encode, 0, 0, Opcode_aslm32_Slot_dot_slot2_encode, 0, 0, Opcode_aslm32_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_aslm32_Slot_smod_slot2_encode, 0, 0, Opcode_aslm32_Slot_llr_slot2_encode, 0, 0, Opcode_aslm32_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_asr_encode_fns[] = {
  0, 0, 0, Opcode_asr_Slot_gp_slot2_encode, 0, 0, Opcode_asr_Slot_dot_slot2_encode, 0, 0, Opcode_asr_Slot_pq_slot2_encode, 0, 0, Opcode_asr_Slot_acc2_slot2_encode, 0, 0, Opcode_asr_Slot_smod_slot2_encode, 0, 0, Opcode_asr_Slot_llr_slot2_encode, 0, 0, Opcode_asr_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_asr32_encode_fns[] = {
  0, 0, 0, Opcode_asr32_Slot_gp_slot2_encode, 0, 0, Opcode_asr32_Slot_dot_slot2_encode, 0, 0, Opcode_asr32_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_asr32_Slot_smod_slot2_encode, 0, 0, Opcode_asr32_Slot_llr_slot2_encode, 0, 0, Opcode_asr32_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_asrac_encode_fns[] = {
  Opcode_asrac_Slot_inst_encode, 0, 0, 0, 0, Opcode_asrac_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_asrac_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_asrac_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_asrm_encode_fns[] = {
  0, 0, 0, Opcode_asrm_Slot_gp_slot2_encode, 0, 0, Opcode_asrm_Slot_dot_slot2_encode, 0, 0, Opcode_asrm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_asrm_Slot_smod_slot2_encode, 0, 0, Opcode_asrm_Slot_llr_slot2_encode, 0, 0, Opcode_asrm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_bitfext_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bitfext_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_bitfins_encode_fns[] = {
  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_bitfins_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_clb_c_encode_fns[] = {
  Opcode_clb_c_Slot_inst_encode, 0, 0, 0, 0, Opcode_clb_c_Slot_gp_slot0_encode, 0, 0, Opcode_clb_c_Slot_dot_slot0_encode, 0, 0, Opcode_clb_c_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_clb_c_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_clb_r_encode_fns[] = {
  Opcode_clb_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_clb_r_Slot_gp_slot0_encode, 0, 0, Opcode_clb_r_Slot_dot_slot0_encode, 0, 0, Opcode_clb_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_clb_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_cmp8_encode_fns[] = {
  0, 0, 0, Opcode_cmp8_Slot_gp_slot2_encode, 0, 0, Opcode_cmp8_Slot_dot_slot2_encode, 0, 0, Opcode_cmp8_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_cmp8_Slot_smod_slot2_encode, 0, 0, Opcode_cmp8_Slot_llr_slot2_encode, 0, 0, Opcode_cmp8_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmp_i_encode_fns[] = {
  0, 0, 0, Opcode_cmp_i_Slot_gp_slot2_encode, 0, 0, Opcode_cmp_i_Slot_dot_slot2_encode, 0, 0, Opcode_cmp_i_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_cmp_i_Slot_smod_slot2_encode, 0, 0, Opcode_cmp_i_Slot_llr_slot2_encode, 0, 0, Opcode_cmp_i_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_cmp_r_encode_fns[] = {
  0, 0, 0, Opcode_cmp_r_Slot_gp_slot2_encode, 0, 0, Opcode_cmp_r_Slot_dot_slot2_encode, 0, 0, Opcode_cmp_r_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_cmp_r_Slot_smod_slot2_encode, 0, 0, Opcode_cmp_r_Slot_llr_slot2_encode, 0, 0, Opcode_cmp_r_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_ext_encode_fns[] = {
  Opcode_ext_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_Slot_gp_slot0_encode, 0, 0, Opcode_ext_Slot_dot_slot0_encode, 0, 0, Opcode_ext_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext_r_encode_fns[] = {
  Opcode_ext_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext_r_Slot_gp_slot0_encode, 0, 0, Opcode_ext_r_Slot_dot_slot0_encode, 0, 0, Opcode_ext_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext32_i_encode_fns[] = {
  Opcode_ext32_i_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext32_i_Slot_gp_slot0_encode, 0, 0, Opcode_ext32_i_Slot_dot_slot0_encode, 0, 0, Opcode_ext32_i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext32_i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_ext32_r_encode_fns[] = {
  Opcode_ext32_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_ext32_r_Slot_gp_slot0_encode, 0, 0, Opcode_ext32_r_Slot_dot_slot0_encode, 0, 0, Opcode_ext32_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_ext32_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_extui4_encode_fns[] = {
  0, 0, 0, Opcode_extui4_Slot_gp_slot2_encode, 0, 0, Opcode_extui4_Slot_dot_slot2_encode, 0, 0, Opcode_extui4_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_extui4_Slot_smod_slot2_encode, 0, 0, Opcode_extui4_Slot_llr_slot2_encode, 0, 0, Opcode_extui4_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lslm_encode_fns[] = {
  0, 0, 0, Opcode_lslm_Slot_gp_slot2_encode, 0, 0, Opcode_lslm_Slot_dot_slot2_encode, 0, 0, Opcode_lslm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_lslm_Slot_smod_slot2_encode, 0, 0, Opcode_lslm_Slot_llr_slot2_encode, 0, 0, Opcode_lslm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_lsrm_encode_fns[] = {
  0, 0, 0, Opcode_lsrm_Slot_gp_slot2_encode, 0, 0, Opcode_lsrm_Slot_dot_slot2_encode, 0, 0, Opcode_lsrm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_lsrm_Slot_smod_slot2_encode, 0, 0, Opcode_lsrm_Slot_llr_slot2_encode, 0, 0, Opcode_lsrm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_max8_encode_fns[] = {
  0, 0, 0, Opcode_max8_Slot_gp_slot2_encode, 0, 0, Opcode_max8_Slot_dot_slot2_encode, 0, 0, Opcode_max8_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_max8_Slot_smod_slot2_encode, 0, 0, Opcode_max8_Slot_llr_slot2_encode, 0, 0, Opcode_max8_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mean_encode_fns[] = {
  0, 0, 0, Opcode_mean_Slot_gp_slot2_encode, 0, 0, Opcode_mean_Slot_dot_slot2_encode, 0, 0, Opcode_mean_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_mean_Slot_smod_slot2_encode, 0, 0, Opcode_mean_Slot_llr_slot2_encode, 0, 0, Opcode_mean_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_mean32_encode_fns[] = {
  0, 0, 0, Opcode_mean32_Slot_gp_slot2_encode, 0, 0, Opcode_mean32_Slot_dot_slot2_encode, 0, 0, Opcode_mean32_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_mean32_Slot_smod_slot2_encode, 0, 0, Opcode_mean32_Slot_llr_slot2_encode, 0, 0, Opcode_mean32_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_min8_encode_fns[] = {
  0, 0, 0, Opcode_min8_Slot_gp_slot2_encode, 0, 0, Opcode_min8_Slot_dot_slot2_encode, 0, 0, Opcode_min8_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_min8_Slot_smod_slot2_encode, 0, 0, Opcode_min8_Slot_llr_slot2_encode, 0, 0, Opcode_min8_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_minclb_c_encode_fns[] = {
  Opcode_minclb_c_Slot_inst_encode, 0, 0, 0, 0, Opcode_minclb_c_Slot_gp_slot0_encode, 0, 0, Opcode_minclb_c_Slot_dot_slot0_encode, 0, 0, Opcode_minclb_c_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_minclb_c_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_minclb_r_encode_fns[] = {
  Opcode_minclb_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_minclb_r_Slot_gp_slot0_encode, 0, 0, Opcode_minclb_r_Slot_dot_slot0_encode, 0, 0, Opcode_minclb_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_minclb_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_not128_encode_fns[] = {
  0, 0, 0, Opcode_not128_Slot_gp_slot2_encode, 0, 0, Opcode_not128_Slot_dot_slot2_encode, 0, 0, Opcode_not128_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_not128_Slot_smod_slot2_encode, 0, 0, Opcode_not128_Slot_llr_slot2_encode, 0, 0, Opcode_not128_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_or128_encode_fns[] = {
  0, 0, 0, Opcode_or128_Slot_gp_slot2_encode, 0, 0, Opcode_or128_Slot_dot_slot2_encode, 0, 0, Opcode_or128_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_or128_Slot_smod_slot2_encode, 0, 0, Opcode_or128_Slot_llr_slot2_encode, 0, 0, Opcode_or128_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_perm_encode_fns[] = {
  0, 0, 0, Opcode_perm_Slot_gp_slot2_encode, 0, 0, Opcode_perm_Slot_dot_slot2_encode, 0, 0, Opcode_perm_Slot_pq_slot2_encode, 0, 0, Opcode_perm_Slot_acc2_slot2_encode, 0, 0, Opcode_perm_Slot_smod_slot2_encode, 0, 0, Opcode_perm_Slot_llr_slot2_encode, 0, 0, Opcode_perm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_redac_encode_fns[] = {
  Opcode_redac_Slot_inst_encode, 0, 0, 0, 0, Opcode_redac_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_redac_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_redac_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_redac2_encode_fns[] = {
  Opcode_redac2_Slot_inst_encode, 0, 0, 0, 0, Opcode_redac2_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_redac2_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_redac2_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_redac4_encode_fns[] = {
  Opcode_redac4_Slot_inst_encode, 0, 0, 0, 0, Opcode_redac4_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_redac4_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_redac4_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_redacs_encode_fns[] = {
  Opcode_redacs_Slot_inst_encode, 0, 0, 0, 0, Opcode_redacs_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_redacs_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_redacs_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sminclb_c_encode_fns[] = {
  Opcode_sminclb_c_Slot_inst_encode, 0, 0, 0, 0, Opcode_sminclb_c_Slot_gp_slot0_encode, 0, 0, Opcode_sminclb_c_Slot_dot_slot0_encode, 0, 0, Opcode_sminclb_c_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sminclb_c_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sminclb_r_encode_fns[] = {
  Opcode_sminclb_r_Slot_inst_encode, 0, 0, 0, 0, Opcode_sminclb_r_Slot_gp_slot0_encode, 0, 0, Opcode_sminclb_r_Slot_dot_slot0_encode, 0, 0, Opcode_sminclb_r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_sminclb_r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_stswapbm_encode_fns[] = {
  Opcode_stswapbm_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_stswapbm_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_stswapbmu_encode_fns[] = {
  Opcode_stswapbmu_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_stswapbmu_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_sub32_encode_fns[] = {
  0, 0, 0, Opcode_sub32_Slot_gp_slot2_encode, 0, 0, Opcode_sub32_Slot_dot_slot2_encode, 0, 0, Opcode_sub32_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_sub32_Slot_smod_slot2_encode, 0, 0, Opcode_sub32_Slot_llr_slot2_encode, 0, 0, Opcode_sub32_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_subac_i2r_encode_fns[] = {
  Opcode_subac_i2r_Slot_inst_encode, 0, 0, 0, 0, Opcode_subac_i2r_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_subac_i2r_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_subac_i2r_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_subac_r2i_encode_fns[] = {
  Opcode_subac_r2i_Slot_inst_encode, 0, 0, 0, 0, Opcode_subac_r2i_Slot_gp_slot0_encode, 0, 0, 0, 0, 0, Opcode_subac_r2i_Slot_pq_slot0_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, Opcode_subac_r2i_Slot_dual_slot0_encode
};

static xtensa_opcode_encode_fn Opcode_subarx_encode_fns[] = {
  0, 0, 0, Opcode_subarx_Slot_gp_slot2_encode, 0, 0, Opcode_subarx_Slot_dot_slot2_encode, 0, 0, Opcode_subarx_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_subarx_Slot_smod_slot2_encode, 0, 0, Opcode_subarx_Slot_llr_slot2_encode, 0, 0, Opcode_subarx_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_subcm_encode_fns[] = {
  0, 0, 0, Opcode_subcm_Slot_gp_slot2_encode, 0, 0, Opcode_subcm_Slot_dot_slot2_encode, 0, 0, Opcode_subcm_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_subcm_Slot_smod_slot2_encode, 0, 0, Opcode_subcm_Slot_llr_slot2_encode, 0, 0, Opcode_subcm_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_submean_encode_fns[] = {
  0, 0, 0, Opcode_submean_Slot_gp_slot2_encode, 0, 0, Opcode_submean_Slot_dot_slot2_encode, 0, 0, Opcode_submean_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_submean_Slot_smod_slot2_encode, 0, 0, Opcode_submean_Slot_llr_slot2_encode, 0, 0, Opcode_submean_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_subwrp_encode_fns[] = {
  0, 0, 0, Opcode_subwrp_Slot_gp_slot2_encode, 0, 0, Opcode_subwrp_Slot_dot_slot2_encode, 0, 0, Opcode_subwrp_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_subwrp_Slot_smod_slot2_encode, 0, 0, Opcode_subwrp_Slot_llr_slot2_encode, 0, 0, Opcode_subwrp_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_trans_encode_fns[] = {
  0, 0, 0, Opcode_trans_Slot_gp_slot2_encode, 0, 0, Opcode_trans_Slot_dot_slot2_encode, 0, 0, Opcode_trans_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_trans_Slot_smod_slot2_encode, 0, 0, Opcode_trans_Slot_llr_slot2_encode, 0, 0, Opcode_trans_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_xor128_encode_fns[] = {
  0, 0, 0, Opcode_xor128_Slot_gp_slot2_encode, 0, 0, Opcode_xor128_Slot_dot_slot2_encode, 0, 0, Opcode_xor128_Slot_pq_slot2_encode, 0, 0, 0, 0, 0, Opcode_xor128_Slot_smod_slot2_encode, 0, 0, Opcode_xor128_Slot_llr_slot2_encode, 0, 0, Opcode_xor128_Slot_dual_slot2_encode, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_expstate_encode_fns[] = {
  Opcode_rur_expstate_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_expstate_encode_fns[] = {
  Opcode_wur_expstate_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_sov_encode_fns[] = {
  Opcode_rur_sov_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_sov_encode_fns[] = {
  Opcode_wur_sov_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_sat_mode_encode_fns[] = {
  Opcode_rur_sat_mode_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_sat_mode_encode_fns[] = {
  Opcode_wur_sat_mode_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_sar0_encode_fns[] = {
  Opcode_rur_sar0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_sar0_encode_fns[] = {
  Opcode_wur_sar0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_sar1_encode_fns[] = {
  Opcode_rur_sar1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_sar1_encode_fns[] = {
  Opcode_wur_sar1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_sar2_encode_fns[] = {
  Opcode_rur_sar2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_sar2_encode_fns[] = {
  Opcode_wur_sar2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_sar3_encode_fns[] = {
  Opcode_rur_sar3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_sar3_encode_fns[] = {
  Opcode_wur_sar3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_hsar0_encode_fns[] = {
  Opcode_rur_hsar0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_hsar0_encode_fns[] = {
  Opcode_wur_hsar0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_hsar1_encode_fns[] = {
  Opcode_rur_hsar1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_hsar1_encode_fns[] = {
  Opcode_wur_hsar1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_hsar2_encode_fns[] = {
  Opcode_rur_hsar2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_hsar2_encode_fns[] = {
  Opcode_wur_hsar2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_hsar3_encode_fns[] = {
  Opcode_rur_hsar3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_hsar3_encode_fns[] = {
  Opcode_wur_hsar3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_max_reg_0_encode_fns[] = {
  Opcode_rur_max_reg_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_max_reg_0_encode_fns[] = {
  Opcode_wur_max_reg_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_max_reg_1_encode_fns[] = {
  Opcode_rur_max_reg_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_max_reg_1_encode_fns[] = {
  Opcode_wur_max_reg_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_max_reg_2_encode_fns[] = {
  Opcode_rur_max_reg_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_max_reg_2_encode_fns[] = {
  Opcode_wur_max_reg_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_max_reg_3_encode_fns[] = {
  Opcode_rur_max_reg_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_max_reg_3_encode_fns[] = {
  Opcode_wur_max_reg_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_arg_max_reg_0_encode_fns[] = {
  Opcode_rur_arg_max_reg_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_arg_max_reg_0_encode_fns[] = {
  Opcode_wur_arg_max_reg_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_arg_max_reg_1_encode_fns[] = {
  Opcode_rur_arg_max_reg_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_arg_max_reg_1_encode_fns[] = {
  Opcode_wur_arg_max_reg_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_arg_max_reg_2_encode_fns[] = {
  Opcode_rur_arg_max_reg_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_arg_max_reg_2_encode_fns[] = {
  Opcode_wur_arg_max_reg_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_arg_max_reg_3_encode_fns[] = {
  Opcode_rur_arg_max_reg_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_arg_max_reg_3_encode_fns[] = {
  Opcode_wur_arg_max_reg_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_nco_counter_0_encode_fns[] = {
  Opcode_rur_nco_counter_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_nco_counter_0_encode_fns[] = {
  Opcode_wur_nco_counter_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_nco_counter_1_encode_fns[] = {
  Opcode_rur_nco_counter_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_nco_counter_1_encode_fns[] = {
  Opcode_wur_nco_counter_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_nco_counter_2_encode_fns[] = {
  Opcode_rur_nco_counter_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_nco_counter_2_encode_fns[] = {
  Opcode_wur_nco_counter_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_nco_counter_3_encode_fns[] = {
  Opcode_rur_nco_counter_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_nco_counter_3_encode_fns[] = {
  Opcode_wur_nco_counter_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_interp_ext_n_encode_fns[] = {
  Opcode_rur_interp_ext_n_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_interp_ext_n_encode_fns[] = {
  Opcode_wur_interp_ext_n_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_interp_ext_l_encode_fns[] = {
  Opcode_rur_interp_ext_l_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_interp_ext_l_encode_fns[] = {
  Opcode_wur_interp_ext_l_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_0_encode_fns[] = {
  Opcode_rur_llr_buf_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_0_encode_fns[] = {
  Opcode_wur_llr_buf_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_1_encode_fns[] = {
  Opcode_rur_llr_buf_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_1_encode_fns[] = {
  Opcode_wur_llr_buf_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_2_encode_fns[] = {
  Opcode_rur_llr_buf_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_2_encode_fns[] = {
  Opcode_wur_llr_buf_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_3_encode_fns[] = {
  Opcode_rur_llr_buf_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_3_encode_fns[] = {
  Opcode_wur_llr_buf_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_4_encode_fns[] = {
  Opcode_rur_llr_buf_4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_4_encode_fns[] = {
  Opcode_wur_llr_buf_4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_5_encode_fns[] = {
  Opcode_rur_llr_buf_5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_5_encode_fns[] = {
  Opcode_wur_llr_buf_5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_6_encode_fns[] = {
  Opcode_rur_llr_buf_6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_6_encode_fns[] = {
  Opcode_wur_llr_buf_6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_7_encode_fns[] = {
  Opcode_rur_llr_buf_7_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_7_encode_fns[] = {
  Opcode_wur_llr_buf_7_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_8_encode_fns[] = {
  Opcode_rur_llr_buf_8_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_8_encode_fns[] = {
  Opcode_wur_llr_buf_8_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_9_encode_fns[] = {
  Opcode_rur_llr_buf_9_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_9_encode_fns[] = {
  Opcode_wur_llr_buf_9_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_10_encode_fns[] = {
  Opcode_rur_llr_buf_10_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_10_encode_fns[] = {
  Opcode_wur_llr_buf_10_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_11_encode_fns[] = {
  Opcode_rur_llr_buf_11_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_11_encode_fns[] = {
  Opcode_wur_llr_buf_11_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_12_encode_fns[] = {
  Opcode_rur_llr_buf_12_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_12_encode_fns[] = {
  Opcode_wur_llr_buf_12_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_13_encode_fns[] = {
  Opcode_rur_llr_buf_13_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_13_encode_fns[] = {
  Opcode_wur_llr_buf_13_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_14_encode_fns[] = {
  Opcode_rur_llr_buf_14_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_14_encode_fns[] = {
  Opcode_wur_llr_buf_14_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_15_encode_fns[] = {
  Opcode_rur_llr_buf_15_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_15_encode_fns[] = {
  Opcode_wur_llr_buf_15_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_16_encode_fns[] = {
  Opcode_rur_llr_buf_16_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_16_encode_fns[] = {
  Opcode_wur_llr_buf_16_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_17_encode_fns[] = {
  Opcode_rur_llr_buf_17_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_17_encode_fns[] = {
  Opcode_wur_llr_buf_17_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_18_encode_fns[] = {
  Opcode_rur_llr_buf_18_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_18_encode_fns[] = {
  Opcode_wur_llr_buf_18_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_19_encode_fns[] = {
  Opcode_rur_llr_buf_19_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_19_encode_fns[] = {
  Opcode_wur_llr_buf_19_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_20_encode_fns[] = {
  Opcode_rur_llr_buf_20_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_20_encode_fns[] = {
  Opcode_wur_llr_buf_20_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_21_encode_fns[] = {
  Opcode_rur_llr_buf_21_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_21_encode_fns[] = {
  Opcode_wur_llr_buf_21_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_22_encode_fns[] = {
  Opcode_rur_llr_buf_22_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_22_encode_fns[] = {
  Opcode_wur_llr_buf_22_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_buf_23_encode_fns[] = {
  Opcode_rur_llr_buf_23_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_buf_23_encode_fns[] = {
  Opcode_wur_llr_buf_23_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_0_encode_fns[] = {
  Opcode_rur_smod_buf_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_0_encode_fns[] = {
  Opcode_wur_smod_buf_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_1_encode_fns[] = {
  Opcode_rur_smod_buf_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_1_encode_fns[] = {
  Opcode_wur_smod_buf_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_2_encode_fns[] = {
  Opcode_rur_smod_buf_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_2_encode_fns[] = {
  Opcode_wur_smod_buf_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_3_encode_fns[] = {
  Opcode_rur_smod_buf_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_3_encode_fns[] = {
  Opcode_wur_smod_buf_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_4_encode_fns[] = {
  Opcode_rur_smod_buf_4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_4_encode_fns[] = {
  Opcode_wur_smod_buf_4_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_5_encode_fns[] = {
  Opcode_rur_smod_buf_5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_5_encode_fns[] = {
  Opcode_wur_smod_buf_5_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_6_encode_fns[] = {
  Opcode_rur_smod_buf_6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_6_encode_fns[] = {
  Opcode_wur_smod_buf_6_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_buf_7_encode_fns[] = {
  Opcode_rur_smod_buf_7_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_buf_7_encode_fns[] = {
  Opcode_wur_smod_buf_7_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_weight_reg_encode_fns[] = {
  Opcode_rur_weight_reg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_weight_reg_encode_fns[] = {
  Opcode_wur_weight_reg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_scale_reg_encode_fns[] = {
  Opcode_rur_scale_reg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_scale_reg_encode_fns[] = {
  Opcode_wur_scale_reg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_llr_pos_encode_fns[] = {
  Opcode_rur_llr_pos_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_llr_pos_encode_fns[] = {
  Opcode_wur_llr_pos_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_pos_encode_fns[] = {
  Opcode_rur_smod_pos_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_pos_encode_fns[] = {
  Opcode_wur_smod_pos_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_perm_reg_encode_fns[] = {
  Opcode_rur_perm_reg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_perm_reg_encode_fns[] = {
  Opcode_wur_perm_reg_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_offset_table_0_encode_fns[] = {
  Opcode_rur_smod_offset_table_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_offset_table_0_encode_fns[] = {
  Opcode_wur_smod_offset_table_0_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_offset_table_1_encode_fns[] = {
  Opcode_rur_smod_offset_table_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_offset_table_1_encode_fns[] = {
  Opcode_wur_smod_offset_table_1_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_offset_table_2_encode_fns[] = {
  Opcode_rur_smod_offset_table_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_offset_table_2_encode_fns[] = {
  Opcode_wur_smod_offset_table_2_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_smod_offset_table_3_encode_fns[] = {
  Opcode_rur_smod_offset_table_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_smod_offset_table_3_encode_fns[] = {
  Opcode_wur_smod_offset_table_3_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_phasor_n_encode_fns[] = {
  Opcode_rur_phasor_n_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_phasor_n_encode_fns[] = {
  Opcode_wur_phasor_n_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_rur_phasor_offset_encode_fns[] = {
  Opcode_rur_phasor_offset_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};

static xtensa_opcode_encode_fn Opcode_wur_phasor_offset_encode_fns[] = {
  Opcode_wur_phasor_offset_Slot_inst_encode, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};


/* Opcode table.  */

static xtensa_opcode_internal opcodes[] = {
  { "excw", ICLASS_xt_iclass_excw,
    0,
    Opcode_excw_encode_fns, 0, 0 },
  { "rfe", ICLASS_xt_iclass_rfe,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfe_encode_fns, 0, 0 },
  { "rfde", ICLASS_xt_iclass_rfde,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfde_encode_fns, 0, 0 },
  { "syscall", ICLASS_xt_iclass_syscall,
    0,
    Opcode_syscall_encode_fns, 0, 0 },
  { "simcall", ICLASS_xt_iclass_simcall,
    0,
    Opcode_simcall_encode_fns, 0, 0 },
  { "call12", ICLASS_xt_iclass_call12,
    XTENSA_OPCODE_IS_CALL,
    Opcode_call12_encode_fns, 0, 0 },
  { "call8", ICLASS_xt_iclass_call8,
    XTENSA_OPCODE_IS_CALL,
    Opcode_call8_encode_fns, 0, 0 },
  { "call4", ICLASS_xt_iclass_call4,
    XTENSA_OPCODE_IS_CALL,
    Opcode_call4_encode_fns, 0, 0 },
  { "callx12", ICLASS_xt_iclass_callx12,
    XTENSA_OPCODE_IS_CALL,
    Opcode_callx12_encode_fns, 0, 0 },
  { "callx8", ICLASS_xt_iclass_callx8,
    XTENSA_OPCODE_IS_CALL,
    Opcode_callx8_encode_fns, 0, 0 },
  { "callx4", ICLASS_xt_iclass_callx4,
    XTENSA_OPCODE_IS_CALL,
    Opcode_callx4_encode_fns, 0, 0 },
  { "entry", ICLASS_xt_iclass_entry,
    0,
    Opcode_entry_encode_fns, 0, 0 },
  { "movsp", ICLASS_xt_iclass_movsp,
    0,
    Opcode_movsp_encode_fns, 0, 0 },
  { "rotw", ICLASS_xt_iclass_rotw,
    0,
    Opcode_rotw_encode_fns, 0, 0 },
  { "retw", ICLASS_xt_iclass_retw,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_retw_encode_fns, 0, 0 },
  { "retw.n", ICLASS_xt_iclass_retw,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_retw_n_encode_fns, 0, 0 },
  { "rfwo", ICLASS_xt_iclass_rfwou,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfwo_encode_fns, 0, 0 },
  { "rfwu", ICLASS_xt_iclass_rfwou,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfwu_encode_fns, 0, 0 },
  { "l32e", ICLASS_xt_iclass_l32e,
    0,
    Opcode_l32e_encode_fns, 0, 0 },
  { "s32e", ICLASS_xt_iclass_s32e,
    0,
    Opcode_s32e_encode_fns, 0, 0 },
  { "rsr.windowbase", ICLASS_xt_iclass_rsr_windowbase,
    0,
    Opcode_rsr_windowbase_encode_fns, 0, 0 },
  { "wsr.windowbase", ICLASS_xt_iclass_wsr_windowbase,
    0,
    Opcode_wsr_windowbase_encode_fns, 0, 0 },
  { "xsr.windowbase", ICLASS_xt_iclass_xsr_windowbase,
    0,
    Opcode_xsr_windowbase_encode_fns, 0, 0 },
  { "rsr.windowstart", ICLASS_xt_iclass_rsr_windowstart,
    0,
    Opcode_rsr_windowstart_encode_fns, 0, 0 },
  { "wsr.windowstart", ICLASS_xt_iclass_wsr_windowstart,
    0,
    Opcode_wsr_windowstart_encode_fns, 0, 0 },
  { "xsr.windowstart", ICLASS_xt_iclass_xsr_windowstart,
    0,
    Opcode_xsr_windowstart_encode_fns, 0, 0 },
  { "add.n", ICLASS_xt_iclass_add_n,
    0,
    Opcode_add_n_encode_fns, 0, 0 },
  { "addi.n", ICLASS_xt_iclass_addi_n,
    0,
    Opcode_addi_n_encode_fns, 0, 0 },
  { "beqz.n", ICLASS_xt_iclass_bz6,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_beqz_n_encode_fns, 0, 0 },
  { "bnez.n", ICLASS_xt_iclass_bz6,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bnez_n_encode_fns, 0, 0 },
  { "ill.n", ICLASS_xt_iclass_ill_n,
    0,
    Opcode_ill_n_encode_fns, 0, 0 },
  { "l32i.n", ICLASS_xt_iclass_loadi4,
    0,
    Opcode_l32i_n_encode_fns, 0, 0 },
  { "mov.n", ICLASS_xt_iclass_mov_n,
    0,
    Opcode_mov_n_encode_fns, 0, 0 },
  { "movi.n", ICLASS_xt_iclass_movi_n,
    0,
    Opcode_movi_n_encode_fns, 0, 0 },
  { "nop.n", ICLASS_xt_iclass_nopn,
    0,
    Opcode_nop_n_encode_fns, 0, 0 },
  { "ret.n", ICLASS_xt_iclass_retn,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_ret_n_encode_fns, 0, 0 },
  { "s32i.n", ICLASS_xt_iclass_storei4,
    0,
    Opcode_s32i_n_encode_fns, 0, 0 },
  { "rur.threadptr", ICLASS_rur_threadptr,
    0,
    Opcode_rur_threadptr_encode_fns, 0, 0 },
  { "wur.threadptr", ICLASS_wur_threadptr,
    0,
    Opcode_wur_threadptr_encode_fns, 0, 0 },
  { "addi", ICLASS_xt_iclass_addi,
    0,
    Opcode_addi_encode_fns, 0, 0 },
  { "addmi", ICLASS_xt_iclass_addmi,
    0,
    Opcode_addmi_encode_fns, 0, 0 },
  { "add", ICLASS_xt_iclass_addsub,
    0,
    Opcode_add_encode_fns, 0, 0 },
  { "sub", ICLASS_xt_iclass_addsub,
    0,
    Opcode_sub_encode_fns, 0, 0 },
  { "addx2", ICLASS_xt_iclass_addsub,
    0,
    Opcode_addx2_encode_fns, 0, 0 },
  { "addx4", ICLASS_xt_iclass_addsub,
    0,
    Opcode_addx4_encode_fns, 0, 0 },
  { "addx8", ICLASS_xt_iclass_addsub,
    0,
    Opcode_addx8_encode_fns, 0, 0 },
  { "subx2", ICLASS_xt_iclass_addsub,
    0,
    Opcode_subx2_encode_fns, 0, 0 },
  { "subx4", ICLASS_xt_iclass_addsub,
    0,
    Opcode_subx4_encode_fns, 0, 0 },
  { "subx8", ICLASS_xt_iclass_addsub,
    0,
    Opcode_subx8_encode_fns, 0, 0 },
  { "and", ICLASS_xt_iclass_bit,
    0,
    Opcode_and_encode_fns, 0, 0 },
  { "or", ICLASS_xt_iclass_bit,
    0,
    Opcode_or_encode_fns, 0, 0 },
  { "xor", ICLASS_xt_iclass_bit,
    0,
    Opcode_xor_encode_fns, 0, 0 },
  { "beqi", ICLASS_xt_iclass_bsi8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_beqi_encode_fns, 0, 0 },
  { "bnei", ICLASS_xt_iclass_bsi8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bnei_encode_fns, 0, 0 },
  { "bgei", ICLASS_xt_iclass_bsi8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bgei_encode_fns, 0, 0 },
  { "blti", ICLASS_xt_iclass_bsi8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_blti_encode_fns, 0, 0 },
  { "bbci", ICLASS_xt_iclass_bsi8b,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bbci_encode_fns, 0, 0 },
  { "bbsi", ICLASS_xt_iclass_bsi8b,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bbsi_encode_fns, 0, 0 },
  { "bgeui", ICLASS_xt_iclass_bsi8u,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bgeui_encode_fns, 0, 0 },
  { "bltui", ICLASS_xt_iclass_bsi8u,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bltui_encode_fns, 0, 0 },
  { "beq", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_beq_encode_fns, 0, 0 },
  { "bne", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bne_encode_fns, 0, 0 },
  { "bge", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bge_encode_fns, 0, 0 },
  { "blt", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_blt_encode_fns, 0, 0 },
  { "bgeu", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bgeu_encode_fns, 0, 0 },
  { "bltu", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bltu_encode_fns, 0, 0 },
  { "bany", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bany_encode_fns, 0, 0 },
  { "bnone", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bnone_encode_fns, 0, 0 },
  { "ball", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_ball_encode_fns, 0, 0 },
  { "bnall", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bnall_encode_fns, 0, 0 },
  { "bbc", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bbc_encode_fns, 0, 0 },
  { "bbs", ICLASS_xt_iclass_bst8,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bbs_encode_fns, 0, 0 },
  { "beqz", ICLASS_xt_iclass_bsz12,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_beqz_encode_fns, 0, 0 },
  { "bnez", ICLASS_xt_iclass_bsz12,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bnez_encode_fns, 0, 0 },
  { "bgez", ICLASS_xt_iclass_bsz12,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bgez_encode_fns, 0, 0 },
  { "bltz", ICLASS_xt_iclass_bsz12,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bltz_encode_fns, 0, 0 },
  { "call0", ICLASS_xt_iclass_call0,
    XTENSA_OPCODE_IS_CALL,
    Opcode_call0_encode_fns, 0, 0 },
  { "callx0", ICLASS_xt_iclass_callx0,
    XTENSA_OPCODE_IS_CALL,
    Opcode_callx0_encode_fns, 0, 0 },
  { "extui", ICLASS_xt_iclass_exti,
    0,
    Opcode_extui_encode_fns, 0, 0 },
  { "ill", ICLASS_xt_iclass_ill,
    0,
    Opcode_ill_encode_fns, 0, 0 },
  { "j", ICLASS_xt_iclass_jump,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_j_encode_fns, 0, 0 },
  { "jx", ICLASS_xt_iclass_jumpx,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_jx_encode_fns, 0, 0 },
  { "l16ui", ICLASS_xt_iclass_l16ui,
    0,
    Opcode_l16ui_encode_fns, 0, 0 },
  { "l16si", ICLASS_xt_iclass_l16si,
    0,
    Opcode_l16si_encode_fns, 0, 0 },
  { "l32i", ICLASS_xt_iclass_l32i,
    0,
    Opcode_l32i_encode_fns, 0, 0 },
  { "l32r", ICLASS_xt_iclass_l32r,
    0,
    Opcode_l32r_encode_fns, 0, 0 },
  { "l8ui", ICLASS_xt_iclass_l8i,
    0,
    Opcode_l8ui_encode_fns, 0, 0 },
  { "loop", ICLASS_xt_iclass_loop,
    XTENSA_OPCODE_IS_LOOP,
    Opcode_loop_encode_fns, 0, 0 },
  { "loopnez", ICLASS_xt_iclass_loopz,
    XTENSA_OPCODE_IS_LOOP,
    Opcode_loopnez_encode_fns, 0, 0 },
  { "loopgtz", ICLASS_xt_iclass_loopz,
    XTENSA_OPCODE_IS_LOOP,
    Opcode_loopgtz_encode_fns, 0, 0 },
  { "movi", ICLASS_xt_iclass_movi,
    0,
    Opcode_movi_encode_fns, 0, 0 },
  { "moveqz", ICLASS_xt_iclass_movz,
    0,
    Opcode_moveqz_encode_fns, 0, 0 },
  { "movnez", ICLASS_xt_iclass_movz,
    0,
    Opcode_movnez_encode_fns, 0, 0 },
  { "movltz", ICLASS_xt_iclass_movz,
    0,
    Opcode_movltz_encode_fns, 0, 0 },
  { "movgez", ICLASS_xt_iclass_movz,
    0,
    Opcode_movgez_encode_fns, 0, 0 },
  { "neg", ICLASS_xt_iclass_neg,
    0,
    Opcode_neg_encode_fns, 0, 0 },
  { "abs", ICLASS_xt_iclass_neg,
    0,
    Opcode_abs_encode_fns, 0, 0 },
  { "nop", ICLASS_xt_iclass_nop,
    0,
    Opcode_nop_encode_fns, 0, 0 },
  { "ret", ICLASS_xt_iclass_return,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_ret_encode_fns, 0, 0 },
  { "s16i", ICLASS_xt_iclass_s16i,
    0,
    Opcode_s16i_encode_fns, 0, 0 },
  { "s32i", ICLASS_xt_iclass_s32i,
    0,
    Opcode_s32i_encode_fns, 0, 0 },
  { "s8i", ICLASS_xt_iclass_s8i,
    0,
    Opcode_s8i_encode_fns, 0, 0 },
  { "ssr", ICLASS_xt_iclass_sar,
    0,
    Opcode_ssr_encode_fns, 0, 0 },
  { "ssl", ICLASS_xt_iclass_sar,
    0,
    Opcode_ssl_encode_fns, 0, 0 },
  { "ssa8l", ICLASS_xt_iclass_sar,
    0,
    Opcode_ssa8l_encode_fns, 0, 0 },
  { "ssa8b", ICLASS_xt_iclass_sar,
    0,
    Opcode_ssa8b_encode_fns, 0, 0 },
  { "ssai", ICLASS_xt_iclass_sari,
    0,
    Opcode_ssai_encode_fns, 0, 0 },
  { "sll", ICLASS_xt_iclass_shifts,
    0,
    Opcode_sll_encode_fns, 0, 0 },
  { "src", ICLASS_xt_iclass_shiftst,
    0,
    Opcode_src_encode_fns, 0, 0 },
  { "srl", ICLASS_xt_iclass_shiftt,
    0,
    Opcode_srl_encode_fns, 0, 0 },
  { "sra", ICLASS_xt_iclass_shiftt,
    0,
    Opcode_sra_encode_fns, 0, 0 },
  { "slli", ICLASS_xt_iclass_slli,
    0,
    Opcode_slli_encode_fns, 0, 0 },
  { "srai", ICLASS_xt_iclass_srai,
    0,
    Opcode_srai_encode_fns, 0, 0 },
  { "srli", ICLASS_xt_iclass_srli,
    0,
    Opcode_srli_encode_fns, 0, 0 },
  { "memw", ICLASS_xt_iclass_memw,
    0,
    Opcode_memw_encode_fns, 0, 0 },
  { "extw", ICLASS_xt_iclass_extw,
    0,
    Opcode_extw_encode_fns, 0, 0 },
  { "isync", ICLASS_xt_iclass_isync,
    0,
    Opcode_isync_encode_fns, 0, 0 },
  { "rsync", ICLASS_xt_iclass_sync,
    0,
    Opcode_rsync_encode_fns, 0, 0 },
  { "esync", ICLASS_xt_iclass_sync,
    0,
    Opcode_esync_encode_fns, 0, 0 },
  { "dsync", ICLASS_xt_iclass_sync,
    0,
    Opcode_dsync_encode_fns, 0, 0 },
  { "rsil", ICLASS_xt_iclass_rsil,
    0,
    Opcode_rsil_encode_fns, 0, 0 },
  { "rsr.lend", ICLASS_xt_iclass_rsr_lend,
    0,
    Opcode_rsr_lend_encode_fns, 0, 0 },
  { "wsr.lend", ICLASS_xt_iclass_wsr_lend,
    0,
    Opcode_wsr_lend_encode_fns, 0, 0 },
  { "xsr.lend", ICLASS_xt_iclass_xsr_lend,
    0,
    Opcode_xsr_lend_encode_fns, 0, 0 },
  { "rsr.lcount", ICLASS_xt_iclass_rsr_lcount,
    0,
    Opcode_rsr_lcount_encode_fns, 0, 0 },
  { "wsr.lcount", ICLASS_xt_iclass_wsr_lcount,
    0,
    Opcode_wsr_lcount_encode_fns, 0, 0 },
  { "xsr.lcount", ICLASS_xt_iclass_xsr_lcount,
    0,
    Opcode_xsr_lcount_encode_fns, 0, 0 },
  { "rsr.lbeg", ICLASS_xt_iclass_rsr_lbeg,
    0,
    Opcode_rsr_lbeg_encode_fns, 0, 0 },
  { "wsr.lbeg", ICLASS_xt_iclass_wsr_lbeg,
    0,
    Opcode_wsr_lbeg_encode_fns, 0, 0 },
  { "xsr.lbeg", ICLASS_xt_iclass_xsr_lbeg,
    0,
    Opcode_xsr_lbeg_encode_fns, 0, 0 },
  { "rsr.sar", ICLASS_xt_iclass_rsr_sar,
    0,
    Opcode_rsr_sar_encode_fns, 0, 0 },
  { "wsr.sar", ICLASS_xt_iclass_wsr_sar,
    0,
    Opcode_wsr_sar_encode_fns, 0, 0 },
  { "xsr.sar", ICLASS_xt_iclass_xsr_sar,
    0,
    Opcode_xsr_sar_encode_fns, 0, 0 },
  { "rsr.litbase", ICLASS_xt_iclass_rsr_litbase,
    0,
    Opcode_rsr_litbase_encode_fns, 0, 0 },
  { "wsr.litbase", ICLASS_xt_iclass_wsr_litbase,
    0,
    Opcode_wsr_litbase_encode_fns, 0, 0 },
  { "xsr.litbase", ICLASS_xt_iclass_xsr_litbase,
    0,
    Opcode_xsr_litbase_encode_fns, 0, 0 },
  { "rsr.176", ICLASS_xt_iclass_rsr_176,
    0,
    Opcode_rsr_176_encode_fns, 0, 0 },
  { "wsr.176", ICLASS_xt_iclass_wsr_176,
    0,
    Opcode_wsr_176_encode_fns, 0, 0 },
  { "rsr.208", ICLASS_xt_iclass_rsr_208,
    0,
    Opcode_rsr_208_encode_fns, 0, 0 },
  { "rsr.ps", ICLASS_xt_iclass_rsr_ps,
    0,
    Opcode_rsr_ps_encode_fns, 0, 0 },
  { "wsr.ps", ICLASS_xt_iclass_wsr_ps,
    0,
    Opcode_wsr_ps_encode_fns, 0, 0 },
  { "xsr.ps", ICLASS_xt_iclass_xsr_ps,
    0,
    Opcode_xsr_ps_encode_fns, 0, 0 },
  { "rsr.epc1", ICLASS_xt_iclass_rsr_epc1,
    0,
    Opcode_rsr_epc1_encode_fns, 0, 0 },
  { "wsr.epc1", ICLASS_xt_iclass_wsr_epc1,
    0,
    Opcode_wsr_epc1_encode_fns, 0, 0 },
  { "xsr.epc1", ICLASS_xt_iclass_xsr_epc1,
    0,
    Opcode_xsr_epc1_encode_fns, 0, 0 },
  { "rsr.excsave1", ICLASS_xt_iclass_rsr_excsave1,
    0,
    Opcode_rsr_excsave1_encode_fns, 0, 0 },
  { "wsr.excsave1", ICLASS_xt_iclass_wsr_excsave1,
    0,
    Opcode_wsr_excsave1_encode_fns, 0, 0 },
  { "xsr.excsave1", ICLASS_xt_iclass_xsr_excsave1,
    0,
    Opcode_xsr_excsave1_encode_fns, 0, 0 },
  { "rsr.epc2", ICLASS_xt_iclass_rsr_epc2,
    0,
    Opcode_rsr_epc2_encode_fns, 0, 0 },
  { "wsr.epc2", ICLASS_xt_iclass_wsr_epc2,
    0,
    Opcode_wsr_epc2_encode_fns, 0, 0 },
  { "xsr.epc2", ICLASS_xt_iclass_xsr_epc2,
    0,
    Opcode_xsr_epc2_encode_fns, 0, 0 },
  { "rsr.excsave2", ICLASS_xt_iclass_rsr_excsave2,
    0,
    Opcode_rsr_excsave2_encode_fns, 0, 0 },
  { "wsr.excsave2", ICLASS_xt_iclass_wsr_excsave2,
    0,
    Opcode_wsr_excsave2_encode_fns, 0, 0 },
  { "xsr.excsave2", ICLASS_xt_iclass_xsr_excsave2,
    0,
    Opcode_xsr_excsave2_encode_fns, 0, 0 },
  { "rsr.epc3", ICLASS_xt_iclass_rsr_epc3,
    0,
    Opcode_rsr_epc3_encode_fns, 0, 0 },
  { "wsr.epc3", ICLASS_xt_iclass_wsr_epc3,
    0,
    Opcode_wsr_epc3_encode_fns, 0, 0 },
  { "xsr.epc3", ICLASS_xt_iclass_xsr_epc3,
    0,
    Opcode_xsr_epc3_encode_fns, 0, 0 },
  { "rsr.excsave3", ICLASS_xt_iclass_rsr_excsave3,
    0,
    Opcode_rsr_excsave3_encode_fns, 0, 0 },
  { "wsr.excsave3", ICLASS_xt_iclass_wsr_excsave3,
    0,
    Opcode_wsr_excsave3_encode_fns, 0, 0 },
  { "xsr.excsave3", ICLASS_xt_iclass_xsr_excsave3,
    0,
    Opcode_xsr_excsave3_encode_fns, 0, 0 },
  { "rsr.epc4", ICLASS_xt_iclass_rsr_epc4,
    0,
    Opcode_rsr_epc4_encode_fns, 0, 0 },
  { "wsr.epc4", ICLASS_xt_iclass_wsr_epc4,
    0,
    Opcode_wsr_epc4_encode_fns, 0, 0 },
  { "xsr.epc4", ICLASS_xt_iclass_xsr_epc4,
    0,
    Opcode_xsr_epc4_encode_fns, 0, 0 },
  { "rsr.excsave4", ICLASS_xt_iclass_rsr_excsave4,
    0,
    Opcode_rsr_excsave4_encode_fns, 0, 0 },
  { "wsr.excsave4", ICLASS_xt_iclass_wsr_excsave4,
    0,
    Opcode_wsr_excsave4_encode_fns, 0, 0 },
  { "xsr.excsave4", ICLASS_xt_iclass_xsr_excsave4,
    0,
    Opcode_xsr_excsave4_encode_fns, 0, 0 },
  { "rsr.epc5", ICLASS_xt_iclass_rsr_epc5,
    0,
    Opcode_rsr_epc5_encode_fns, 0, 0 },
  { "wsr.epc5", ICLASS_xt_iclass_wsr_epc5,
    0,
    Opcode_wsr_epc5_encode_fns, 0, 0 },
  { "xsr.epc5", ICLASS_xt_iclass_xsr_epc5,
    0,
    Opcode_xsr_epc5_encode_fns, 0, 0 },
  { "rsr.excsave5", ICLASS_xt_iclass_rsr_excsave5,
    0,
    Opcode_rsr_excsave5_encode_fns, 0, 0 },
  { "wsr.excsave5", ICLASS_xt_iclass_wsr_excsave5,
    0,
    Opcode_wsr_excsave5_encode_fns, 0, 0 },
  { "xsr.excsave5", ICLASS_xt_iclass_xsr_excsave5,
    0,
    Opcode_xsr_excsave5_encode_fns, 0, 0 },
  { "rsr.epc6", ICLASS_xt_iclass_rsr_epc6,
    0,
    Opcode_rsr_epc6_encode_fns, 0, 0 },
  { "wsr.epc6", ICLASS_xt_iclass_wsr_epc6,
    0,
    Opcode_wsr_epc6_encode_fns, 0, 0 },
  { "xsr.epc6", ICLASS_xt_iclass_xsr_epc6,
    0,
    Opcode_xsr_epc6_encode_fns, 0, 0 },
  { "rsr.excsave6", ICLASS_xt_iclass_rsr_excsave6,
    0,
    Opcode_rsr_excsave6_encode_fns, 0, 0 },
  { "wsr.excsave6", ICLASS_xt_iclass_wsr_excsave6,
    0,
    Opcode_wsr_excsave6_encode_fns, 0, 0 },
  { "xsr.excsave6", ICLASS_xt_iclass_xsr_excsave6,
    0,
    Opcode_xsr_excsave6_encode_fns, 0, 0 },
  { "rsr.eps2", ICLASS_xt_iclass_rsr_eps2,
    0,
    Opcode_rsr_eps2_encode_fns, 0, 0 },
  { "wsr.eps2", ICLASS_xt_iclass_wsr_eps2,
    0,
    Opcode_wsr_eps2_encode_fns, 0, 0 },
  { "xsr.eps2", ICLASS_xt_iclass_xsr_eps2,
    0,
    Opcode_xsr_eps2_encode_fns, 0, 0 },
  { "rsr.eps3", ICLASS_xt_iclass_rsr_eps3,
    0,
    Opcode_rsr_eps3_encode_fns, 0, 0 },
  { "wsr.eps3", ICLASS_xt_iclass_wsr_eps3,
    0,
    Opcode_wsr_eps3_encode_fns, 0, 0 },
  { "xsr.eps3", ICLASS_xt_iclass_xsr_eps3,
    0,
    Opcode_xsr_eps3_encode_fns, 0, 0 },
  { "rsr.eps4", ICLASS_xt_iclass_rsr_eps4,
    0,
    Opcode_rsr_eps4_encode_fns, 0, 0 },
  { "wsr.eps4", ICLASS_xt_iclass_wsr_eps4,
    0,
    Opcode_wsr_eps4_encode_fns, 0, 0 },
  { "xsr.eps4", ICLASS_xt_iclass_xsr_eps4,
    0,
    Opcode_xsr_eps4_encode_fns, 0, 0 },
  { "rsr.eps5", ICLASS_xt_iclass_rsr_eps5,
    0,
    Opcode_rsr_eps5_encode_fns, 0, 0 },
  { "wsr.eps5", ICLASS_xt_iclass_wsr_eps5,
    0,
    Opcode_wsr_eps5_encode_fns, 0, 0 },
  { "xsr.eps5", ICLASS_xt_iclass_xsr_eps5,
    0,
    Opcode_xsr_eps5_encode_fns, 0, 0 },
  { "rsr.eps6", ICLASS_xt_iclass_rsr_eps6,
    0,
    Opcode_rsr_eps6_encode_fns, 0, 0 },
  { "wsr.eps6", ICLASS_xt_iclass_wsr_eps6,
    0,
    Opcode_wsr_eps6_encode_fns, 0, 0 },
  { "xsr.eps6", ICLASS_xt_iclass_xsr_eps6,
    0,
    Opcode_xsr_eps6_encode_fns, 0, 0 },
  { "rsr.excvaddr", ICLASS_xt_iclass_rsr_excvaddr,
    0,
    Opcode_rsr_excvaddr_encode_fns, 0, 0 },
  { "wsr.excvaddr", ICLASS_xt_iclass_wsr_excvaddr,
    0,
    Opcode_wsr_excvaddr_encode_fns, 0, 0 },
  { "xsr.excvaddr", ICLASS_xt_iclass_xsr_excvaddr,
    0,
    Opcode_xsr_excvaddr_encode_fns, 0, 0 },
  { "rsr.depc", ICLASS_xt_iclass_rsr_depc,
    0,
    Opcode_rsr_depc_encode_fns, 0, 0 },
  { "wsr.depc", ICLASS_xt_iclass_wsr_depc,
    0,
    Opcode_wsr_depc_encode_fns, 0, 0 },
  { "xsr.depc", ICLASS_xt_iclass_xsr_depc,
    0,
    Opcode_xsr_depc_encode_fns, 0, 0 },
  { "rsr.exccause", ICLASS_xt_iclass_rsr_exccause,
    0,
    Opcode_rsr_exccause_encode_fns, 0, 0 },
  { "wsr.exccause", ICLASS_xt_iclass_wsr_exccause,
    0,
    Opcode_wsr_exccause_encode_fns, 0, 0 },
  { "xsr.exccause", ICLASS_xt_iclass_xsr_exccause,
    0,
    Opcode_xsr_exccause_encode_fns, 0, 0 },
  { "rsr.prid", ICLASS_xt_iclass_rsr_prid,
    0,
    Opcode_rsr_prid_encode_fns, 0, 0 },
  { "rsr.vecbase", ICLASS_xt_iclass_rsr_vecbase,
    0,
    Opcode_rsr_vecbase_encode_fns, 0, 0 },
  { "wsr.vecbase", ICLASS_xt_iclass_wsr_vecbase,
    0,
    Opcode_wsr_vecbase_encode_fns, 0, 0 },
  { "xsr.vecbase", ICLASS_xt_iclass_xsr_vecbase,
    0,
    Opcode_xsr_vecbase_encode_fns, 0, 0 },
  { "mul16u", ICLASS_xt_mul16,
    0,
    Opcode_mul16u_encode_fns, 0, 0 },
  { "mul16s", ICLASS_xt_mul16,
    0,
    Opcode_mul16s_encode_fns, 0, 0 },
  { "rfi", ICLASS_xt_iclass_rfi,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfi_encode_fns, 0, 0 },
  { "waiti", ICLASS_xt_iclass_wait,
    0,
    Opcode_waiti_encode_fns, 0, 0 },
  { "rsr.interrupt", ICLASS_xt_iclass_rsr_interrupt,
    0,
    Opcode_rsr_interrupt_encode_fns, 0, 0 },
  { "wsr.intset", ICLASS_xt_iclass_wsr_intset,
    0,
    Opcode_wsr_intset_encode_fns, 0, 0 },
  { "wsr.intclear", ICLASS_xt_iclass_wsr_intclear,
    0,
    Opcode_wsr_intclear_encode_fns, 0, 0 },
  { "rsr.intenable", ICLASS_xt_iclass_rsr_intenable,
    0,
    Opcode_rsr_intenable_encode_fns, 0, 0 },
  { "wsr.intenable", ICLASS_xt_iclass_wsr_intenable,
    0,
    Opcode_wsr_intenable_encode_fns, 0, 0 },
  { "xsr.intenable", ICLASS_xt_iclass_xsr_intenable,
    0,
    Opcode_xsr_intenable_encode_fns, 0, 0 },
  { "break", ICLASS_xt_iclass_break,
    0,
    Opcode_break_encode_fns, 0, 0 },
  { "break.n", ICLASS_xt_iclass_break_n,
    0,
    Opcode_break_n_encode_fns, 0, 0 },
  { "rsr.dbreaka0", ICLASS_xt_iclass_rsr_dbreaka0,
    0,
    Opcode_rsr_dbreaka0_encode_fns, 0, 0 },
  { "wsr.dbreaka0", ICLASS_xt_iclass_wsr_dbreaka0,
    0,
    Opcode_wsr_dbreaka0_encode_fns, 0, 0 },
  { "xsr.dbreaka0", ICLASS_xt_iclass_xsr_dbreaka0,
    0,
    Opcode_xsr_dbreaka0_encode_fns, 0, 0 },
  { "rsr.dbreakc0", ICLASS_xt_iclass_rsr_dbreakc0,
    0,
    Opcode_rsr_dbreakc0_encode_fns, 0, 0 },
  { "wsr.dbreakc0", ICLASS_xt_iclass_wsr_dbreakc0,
    0,
    Opcode_wsr_dbreakc0_encode_fns, 0, 0 },
  { "xsr.dbreakc0", ICLASS_xt_iclass_xsr_dbreakc0,
    0,
    Opcode_xsr_dbreakc0_encode_fns, 0, 0 },
  { "rsr.dbreaka1", ICLASS_xt_iclass_rsr_dbreaka1,
    0,
    Opcode_rsr_dbreaka1_encode_fns, 0, 0 },
  { "wsr.dbreaka1", ICLASS_xt_iclass_wsr_dbreaka1,
    0,
    Opcode_wsr_dbreaka1_encode_fns, 0, 0 },
  { "xsr.dbreaka1", ICLASS_xt_iclass_xsr_dbreaka1,
    0,
    Opcode_xsr_dbreaka1_encode_fns, 0, 0 },
  { "rsr.dbreakc1", ICLASS_xt_iclass_rsr_dbreakc1,
    0,
    Opcode_rsr_dbreakc1_encode_fns, 0, 0 },
  { "wsr.dbreakc1", ICLASS_xt_iclass_wsr_dbreakc1,
    0,
    Opcode_wsr_dbreakc1_encode_fns, 0, 0 },
  { "xsr.dbreakc1", ICLASS_xt_iclass_xsr_dbreakc1,
    0,
    Opcode_xsr_dbreakc1_encode_fns, 0, 0 },
  { "rsr.ibreaka0", ICLASS_xt_iclass_rsr_ibreaka0,
    0,
    Opcode_rsr_ibreaka0_encode_fns, 0, 0 },
  { "wsr.ibreaka0", ICLASS_xt_iclass_wsr_ibreaka0,
    0,
    Opcode_wsr_ibreaka0_encode_fns, 0, 0 },
  { "xsr.ibreaka0", ICLASS_xt_iclass_xsr_ibreaka0,
    0,
    Opcode_xsr_ibreaka0_encode_fns, 0, 0 },
  { "rsr.ibreaka1", ICLASS_xt_iclass_rsr_ibreaka1,
    0,
    Opcode_rsr_ibreaka1_encode_fns, 0, 0 },
  { "wsr.ibreaka1", ICLASS_xt_iclass_wsr_ibreaka1,
    0,
    Opcode_wsr_ibreaka1_encode_fns, 0, 0 },
  { "xsr.ibreaka1", ICLASS_xt_iclass_xsr_ibreaka1,
    0,
    Opcode_xsr_ibreaka1_encode_fns, 0, 0 },
  { "rsr.ibreakenable", ICLASS_xt_iclass_rsr_ibreakenable,
    0,
    Opcode_rsr_ibreakenable_encode_fns, 0, 0 },
  { "wsr.ibreakenable", ICLASS_xt_iclass_wsr_ibreakenable,
    0,
    Opcode_wsr_ibreakenable_encode_fns, 0, 0 },
  { "xsr.ibreakenable", ICLASS_xt_iclass_xsr_ibreakenable,
    0,
    Opcode_xsr_ibreakenable_encode_fns, 0, 0 },
  { "rsr.debugcause", ICLASS_xt_iclass_rsr_debugcause,
    0,
    Opcode_rsr_debugcause_encode_fns, 0, 0 },
  { "wsr.debugcause", ICLASS_xt_iclass_wsr_debugcause,
    0,
    Opcode_wsr_debugcause_encode_fns, 0, 0 },
  { "xsr.debugcause", ICLASS_xt_iclass_xsr_debugcause,
    0,
    Opcode_xsr_debugcause_encode_fns, 0, 0 },
  { "rsr.icount", ICLASS_xt_iclass_rsr_icount,
    0,
    Opcode_rsr_icount_encode_fns, 0, 0 },
  { "wsr.icount", ICLASS_xt_iclass_wsr_icount,
    0,
    Opcode_wsr_icount_encode_fns, 0, 0 },
  { "xsr.icount", ICLASS_xt_iclass_xsr_icount,
    0,
    Opcode_xsr_icount_encode_fns, 0, 0 },
  { "rsr.icountlevel", ICLASS_xt_iclass_rsr_icountlevel,
    0,
    Opcode_rsr_icountlevel_encode_fns, 0, 0 },
  { "wsr.icountlevel", ICLASS_xt_iclass_wsr_icountlevel,
    0,
    Opcode_wsr_icountlevel_encode_fns, 0, 0 },
  { "xsr.icountlevel", ICLASS_xt_iclass_xsr_icountlevel,
    0,
    Opcode_xsr_icountlevel_encode_fns, 0, 0 },
  { "rsr.ddr", ICLASS_xt_iclass_rsr_ddr,
    0,
    Opcode_rsr_ddr_encode_fns, 0, 0 },
  { "wsr.ddr", ICLASS_xt_iclass_wsr_ddr,
    0,
    Opcode_wsr_ddr_encode_fns, 0, 0 },
  { "xsr.ddr", ICLASS_xt_iclass_xsr_ddr,
    0,
    Opcode_xsr_ddr_encode_fns, 0, 0 },
  { "rfdo", ICLASS_xt_iclass_rfdo,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfdo_encode_fns, 0, 0 },
  { "rfdd", ICLASS_xt_iclass_rfdd,
    XTENSA_OPCODE_IS_JUMP,
    Opcode_rfdd_encode_fns, 0, 0 },
  { "wsr.mmid", ICLASS_xt_iclass_wsr_mmid,
    0,
    Opcode_wsr_mmid_encode_fns, 0, 0 },
  { "andb", ICLASS_xt_iclass_bbool1,
    0,
    Opcode_andb_encode_fns, 0, 0 },
  { "andbc", ICLASS_xt_iclass_bbool1,
    0,
    Opcode_andbc_encode_fns, 0, 0 },
  { "orb", ICLASS_xt_iclass_bbool1,
    0,
    Opcode_orb_encode_fns, 0, 0 },
  { "orbc", ICLASS_xt_iclass_bbool1,
    0,
    Opcode_orbc_encode_fns, 0, 0 },
  { "xorb", ICLASS_xt_iclass_bbool1,
    0,
    Opcode_xorb_encode_fns, 0, 0 },
  { "any4", ICLASS_xt_iclass_bbool4,
    0,
    Opcode_any4_encode_fns, 0, 0 },
  { "all4", ICLASS_xt_iclass_bbool4,
    0,
    Opcode_all4_encode_fns, 0, 0 },
  { "any8", ICLASS_xt_iclass_bbool8,
    0,
    Opcode_any8_encode_fns, 0, 0 },
  { "all8", ICLASS_xt_iclass_bbool8,
    0,
    Opcode_all8_encode_fns, 0, 0 },
  { "bf", ICLASS_xt_iclass_bbranch,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bf_encode_fns, 0, 0 },
  { "bt", ICLASS_xt_iclass_bbranch,
    XTENSA_OPCODE_IS_BRANCH,
    Opcode_bt_encode_fns, 0, 0 },
  { "movf", ICLASS_xt_iclass_bmove,
    0,
    Opcode_movf_encode_fns, 0, 0 },
  { "movt", ICLASS_xt_iclass_bmove,
    0,
    Opcode_movt_encode_fns, 0, 0 },
  { "rsr.br", ICLASS_xt_iclass_RSR_BR,
    0,
    Opcode_rsr_br_encode_fns, 0, 0 },
  { "wsr.br", ICLASS_xt_iclass_WSR_BR,
    0,
    Opcode_wsr_br_encode_fns, 0, 0 },
  { "xsr.br", ICLASS_xt_iclass_XSR_BR,
    0,
    Opcode_xsr_br_encode_fns, 0, 0 },
  { "rsr.ccount", ICLASS_xt_iclass_rsr_ccount,
    0,
    Opcode_rsr_ccount_encode_fns, 0, 0 },
  { "wsr.ccount", ICLASS_xt_iclass_wsr_ccount,
    0,
    Opcode_wsr_ccount_encode_fns, 0, 0 },
  { "xsr.ccount", ICLASS_xt_iclass_xsr_ccount,
    0,
    Opcode_xsr_ccount_encode_fns, 0, 0 },
  { "rsr.ccompare0", ICLASS_xt_iclass_rsr_ccompare0,
    0,
    Opcode_rsr_ccompare0_encode_fns, 0, 0 },
  { "wsr.ccompare0", ICLASS_xt_iclass_wsr_ccompare0,
    0,
    Opcode_wsr_ccompare0_encode_fns, 0, 0 },
  { "xsr.ccompare0", ICLASS_xt_iclass_xsr_ccompare0,
    0,
    Opcode_xsr_ccompare0_encode_fns, 0, 0 },
  { "rsr.ccompare1", ICLASS_xt_iclass_rsr_ccompare1,
    0,
    Opcode_rsr_ccompare1_encode_fns, 0, 0 },
  { "wsr.ccompare1", ICLASS_xt_iclass_wsr_ccompare1,
    0,
    Opcode_wsr_ccompare1_encode_fns, 0, 0 },
  { "xsr.ccompare1", ICLASS_xt_iclass_xsr_ccompare1,
    0,
    Opcode_xsr_ccompare1_encode_fns, 0, 0 },
  { "ipf", ICLASS_xt_iclass_icache,
    0,
    Opcode_ipf_encode_fns, 0, 0 },
  { "ihi", ICLASS_xt_iclass_icache,
    0,
    Opcode_ihi_encode_fns, 0, 0 },
  { "ipfl", ICLASS_xt_iclass_icache_lock,
    0,
    Opcode_ipfl_encode_fns, 0, 0 },
  { "ihu", ICLASS_xt_iclass_icache_lock,
    0,
    Opcode_ihu_encode_fns, 0, 0 },
  { "iiu", ICLASS_xt_iclass_icache_lock,
    0,
    Opcode_iiu_encode_fns, 0, 0 },
  { "iii", ICLASS_xt_iclass_icache_inv,
    0,
    Opcode_iii_encode_fns, 0, 0 },
  { "lict", ICLASS_xt_iclass_licx,
    0,
    Opcode_lict_encode_fns, 0, 0 },
  { "licw", ICLASS_xt_iclass_licx,
    0,
    Opcode_licw_encode_fns, 0, 0 },
  { "sict", ICLASS_xt_iclass_sicx,
    0,
    Opcode_sict_encode_fns, 0, 0 },
  { "sicw", ICLASS_xt_iclass_sicx,
    0,
    Opcode_sicw_encode_fns, 0, 0 },
  { "dhwb", ICLASS_xt_iclass_dcache,
    0,
    Opcode_dhwb_encode_fns, 0, 0 },
  { "dhwbi", ICLASS_xt_iclass_dcache,
    0,
    Opcode_dhwbi_encode_fns, 0, 0 },
  { "diwb", ICLASS_xt_iclass_dcache_ind,
    0,
    Opcode_diwb_encode_fns, 0, 0 },
  { "diwbi", ICLASS_xt_iclass_dcache_ind,
    0,
    Opcode_diwbi_encode_fns, 0, 0 },
  { "dhi", ICLASS_xt_iclass_dcache_inv,
    0,
    Opcode_dhi_encode_fns, 0, 0 },
  { "dii", ICLASS_xt_iclass_dcache_inv,
    0,
    Opcode_dii_encode_fns, 0, 0 },
  { "dpfr", ICLASS_xt_iclass_dpf,
    0,
    Opcode_dpfr_encode_fns, 0, 0 },
  { "dpfw", ICLASS_xt_iclass_dpf,
    0,
    Opcode_dpfw_encode_fns, 0, 0 },
  { "dpfro", ICLASS_xt_iclass_dpf,
    0,
    Opcode_dpfro_encode_fns, 0, 0 },
  { "dpfwo", ICLASS_xt_iclass_dpf,
    0,
    Opcode_dpfwo_encode_fns, 0, 0 },
  { "dpfl", ICLASS_xt_iclass_dcache_lock,
    0,
    Opcode_dpfl_encode_fns, 0, 0 },
  { "dhu", ICLASS_xt_iclass_dcache_lock,
    0,
    Opcode_dhu_encode_fns, 0, 0 },
  { "diu", ICLASS_xt_iclass_dcache_lock,
    0,
    Opcode_diu_encode_fns, 0, 0 },
  { "sdct", ICLASS_xt_iclass_sdct,
    0,
    Opcode_sdct_encode_fns, 0, 0 },
  { "ldct", ICLASS_xt_iclass_ldct,
    0,
    Opcode_ldct_encode_fns, 0, 0 },
  { "idtlb", ICLASS_xt_iclass_idtlb,
    0,
    Opcode_idtlb_encode_fns, 0, 0 },
  { "pdtlb", ICLASS_xt_iclass_rdtlb,
    0,
    Opcode_pdtlb_encode_fns, 0, 0 },
  { "rdtlb0", ICLASS_xt_iclass_rdtlb,
    0,
    Opcode_rdtlb0_encode_fns, 0, 0 },
  { "rdtlb1", ICLASS_xt_iclass_rdtlb,
    0,
    Opcode_rdtlb1_encode_fns, 0, 0 },
  { "wdtlb", ICLASS_xt_iclass_wdtlb,
    0,
    Opcode_wdtlb_encode_fns, 0, 0 },
  { "iitlb", ICLASS_xt_iclass_iitlb,
    0,
    Opcode_iitlb_encode_fns, 0, 0 },
  { "pitlb", ICLASS_xt_iclass_ritlb,
    0,
    Opcode_pitlb_encode_fns, 0, 0 },
  { "ritlb0", ICLASS_xt_iclass_ritlb,
    0,
    Opcode_ritlb0_encode_fns, 0, 0 },
  { "ritlb1", ICLASS_xt_iclass_ritlb,
    0,
    Opcode_ritlb1_encode_fns, 0, 0 },
  { "witlb", ICLASS_xt_iclass_witlb,
    0,
    Opcode_witlb_encode_fns, 0, 0 },
  { "rsr.cpenable", ICLASS_xt_iclass_rsr_cpenable,
    0,
    Opcode_rsr_cpenable_encode_fns, 0, 0 },
  { "wsr.cpenable", ICLASS_xt_iclass_wsr_cpenable,
    0,
    Opcode_wsr_cpenable_encode_fns, 0, 0 },
  { "xsr.cpenable", ICLASS_xt_iclass_xsr_cpenable,
    0,
    Opcode_xsr_cpenable_encode_fns, 0, 0 },
  { "clamps", ICLASS_xt_iclass_clamp,
    0,
    Opcode_clamps_encode_fns, 0, 0 },
  { "min", ICLASS_xt_iclass_minmax,
    0,
    Opcode_min_encode_fns, 0, 0 },
  { "max", ICLASS_xt_iclass_minmax,
    0,
    Opcode_max_encode_fns, 0, 0 },
  { "minu", ICLASS_xt_iclass_minmax,
    0,
    Opcode_minu_encode_fns, 0, 0 },
  { "maxu", ICLASS_xt_iclass_minmax,
    0,
    Opcode_maxu_encode_fns, 0, 0 },
  { "nsa", ICLASS_xt_iclass_nsa,
    0,
    Opcode_nsa_encode_fns, 0, 0 },
  { "nsau", ICLASS_xt_iclass_nsa,
    0,
    Opcode_nsau_encode_fns, 0, 0 },
  { "sext", ICLASS_xt_iclass_sx,
    0,
    Opcode_sext_encode_fns, 0, 0 },
  { "l32ai", ICLASS_xt_iclass_l32ai,
    0,
    Opcode_l32ai_encode_fns, 0, 0 },
  { "s32ri", ICLASS_xt_iclass_s32ri,
    0,
    Opcode_s32ri_encode_fns, 0, 0 },
  { "s32c1i", ICLASS_xt_iclass_s32c1i,
    0,
    Opcode_s32c1i_encode_fns, 0, 0 },
  { "rsr.scompare1", ICLASS_xt_iclass_rsr_scompare1,
    0,
    Opcode_rsr_scompare1_encode_fns, 0, 0 },
  { "wsr.scompare1", ICLASS_xt_iclass_wsr_scompare1,
    0,
    Opcode_wsr_scompare1_encode_fns, 0, 0 },
  { "xsr.scompare1", ICLASS_xt_iclass_xsr_scompare1,
    0,
    Opcode_xsr_scompare1_encode_fns, 0, 0 },
  { "rsr.atomctl", ICLASS_xt_iclass_rsr_atomctl,
    0,
    Opcode_rsr_atomctl_encode_fns, 0, 0 },
  { "wsr.atomctl", ICLASS_xt_iclass_wsr_atomctl,
    0,
    Opcode_wsr_atomctl_encode_fns, 0, 0 },
  { "xsr.atomctl", ICLASS_xt_iclass_xsr_atomctl,
    0,
    Opcode_xsr_atomctl_encode_fns, 0, 0 },
  { "rer", ICLASS_xt_iclass_rer,
    0,
    Opcode_rer_encode_fns, 0, 0 },
  { "wer", ICLASS_xt_iclass_wer,
    0,
    Opcode_wer_encode_fns, 0, 0 },
  { "rur.fcr", ICLASS_rur_fcr,
    0,
    Opcode_rur_fcr_encode_fns, 0, 0 },
  { "wur.fcr", ICLASS_wur_fcr,
    0,
    Opcode_wur_fcr_encode_fns, 0, 0 },
  { "rur.fsr", ICLASS_rur_fsr,
    0,
    Opcode_rur_fsr_encode_fns, 0, 0 },
  { "wur.fsr", ICLASS_wur_fsr,
    0,
    Opcode_wur_fsr_encode_fns, 0, 0 },
  { "add.s", ICLASS_fp,
    0,
    Opcode_add_s_encode_fns, 0, 0 },
  { "sub.s", ICLASS_fp,
    0,
    Opcode_sub_s_encode_fns, 0, 0 },
  { "mul.s", ICLASS_fp,
    0,
    Opcode_mul_s_encode_fns, 0, 0 },
  { "madd.s", ICLASS_fp_mac,
    0,
    Opcode_madd_s_encode_fns, 0, 0 },
  { "msub.s", ICLASS_fp_mac,
    0,
    Opcode_msub_s_encode_fns, 0, 0 },
  { "movf.s", ICLASS_fp_cmov,
    0,
    Opcode_movf_s_encode_fns, 0, 0 },
  { "movt.s", ICLASS_fp_cmov,
    0,
    Opcode_movt_s_encode_fns, 0, 0 },
  { "moveqz.s", ICLASS_fp_mov,
    0,
    Opcode_moveqz_s_encode_fns, 0, 0 },
  { "movnez.s", ICLASS_fp_mov,
    0,
    Opcode_movnez_s_encode_fns, 0, 0 },
  { "movltz.s", ICLASS_fp_mov,
    0,
    Opcode_movltz_s_encode_fns, 0, 0 },
  { "movgez.s", ICLASS_fp_mov,
    0,
    Opcode_movgez_s_encode_fns, 0, 0 },
  { "abs.s", ICLASS_fp_mov2,
    0,
    Opcode_abs_s_encode_fns, 0, 0 },
  { "mov.s", ICLASS_fp_mov2,
    0,
    Opcode_mov_s_encode_fns, 0, 0 },
  { "neg.s", ICLASS_fp_mov2,
    0,
    Opcode_neg_s_encode_fns, 0, 0 },
  { "un.s", ICLASS_fp_cmp,
    0,
    Opcode_un_s_encode_fns, 0, 0 },
  { "oeq.s", ICLASS_fp_cmp,
    0,
    Opcode_oeq_s_encode_fns, 0, 0 },
  { "ueq.s", ICLASS_fp_cmp,
    0,
    Opcode_ueq_s_encode_fns, 0, 0 },
  { "olt.s", ICLASS_fp_cmp,
    0,
    Opcode_olt_s_encode_fns, 0, 0 },
  { "ult.s", ICLASS_fp_cmp,
    0,
    Opcode_ult_s_encode_fns, 0, 0 },
  { "ole.s", ICLASS_fp_cmp,
    0,
    Opcode_ole_s_encode_fns, 0, 0 },
  { "ule.s", ICLASS_fp_cmp,
    0,
    Opcode_ule_s_encode_fns, 0, 0 },
  { "float.s", ICLASS_fp_float,
    0,
    Opcode_float_s_encode_fns, 0, 0 },
  { "ufloat.s", ICLASS_fp_float,
    0,
    Opcode_ufloat_s_encode_fns, 0, 0 },
  { "round.s", ICLASS_fp_int,
    0,
    Opcode_round_s_encode_fns, 0, 0 },
  { "ceil.s", ICLASS_fp_int,
    0,
    Opcode_ceil_s_encode_fns, 0, 0 },
  { "floor.s", ICLASS_fp_int,
    0,
    Opcode_floor_s_encode_fns, 0, 0 },
  { "trunc.s", ICLASS_fp_int,
    0,
    Opcode_trunc_s_encode_fns, 0, 0 },
  { "utrunc.s", ICLASS_fp_int,
    0,
    Opcode_utrunc_s_encode_fns, 0, 0 },
  { "rfr", ICLASS_fp_rfr,
    0,
    Opcode_rfr_encode_fns, 0, 0 },
  { "wfr", ICLASS_fp_wfr,
    0,
    Opcode_wfr_encode_fns, 0, 0 },
  { "lsi", ICLASS_fp_lsi,
    0,
    Opcode_lsi_encode_fns, 0, 0 },
  { "lsiu", ICLASS_fp_lsiu,
    0,
    Opcode_lsiu_encode_fns, 0, 0 },
  { "lsx", ICLASS_fp_lsx,
    0,
    Opcode_lsx_encode_fns, 0, 0 },
  { "lsxu", ICLASS_fp_lsxu,
    0,
    Opcode_lsxu_encode_fns, 0, 0 },
  { "ssi", ICLASS_fp_ssi,
    0,
    Opcode_ssi_encode_fns, 0, 0 },
  { "ssiu", ICLASS_fp_ssiu,
    0,
    Opcode_ssiu_encode_fns, 0, 0 },
  { "ssx", ICLASS_fp_ssx,
    0,
    Opcode_ssx_encode_fns, 0, 0 },
  { "ssxu", ICLASS_fp_ssxu,
    0,
    Opcode_ssxu_encode_fns, 0, 0 },
  { "get_argmax", ICLASS_iclass_GET_ARGMAX,
    0,
    Opcode_get_argmax_encode_fns, 0, 0 },
  { "get_hsar", ICLASS_iclass_GET_HSAR,
    0,
    Opcode_get_hsar_encode_fns, 0, 0 },
  { "get_hsar2sar", ICLASS_iclass_GET_HSAR2SAR,
    0,
    Opcode_get_hsar2sar_encode_fns, 0, 0 },
  { "get_interp_ext_n", ICLASS_iclass_GET_INTERP_EXT_N,
    0,
    Opcode_get_interp_ext_n_encode_fns, 0, 0 },
  { "get_interp_ext_l", ICLASS_iclass_GET_INTERP_EXT_L,
    0,
    Opcode_get_interp_ext_l_encode_fns, 0, 0 },
  { "get_llr_buf", ICLASS_iclass_GET_LLR_BUF,
    0,
    Opcode_get_llr_buf_encode_fns, 0, 0 },
  { "get_llr_pos", ICLASS_iclass_GET_LLR_POS,
    0,
    Opcode_get_llr_pos_encode_fns, 0, 0 },
  { "get_max", ICLASS_iclass_GET_MAX,
    0,
    Opcode_get_max_encode_fns, 0, 0 },
  { "get_nco", ICLASS_iclass_GET_NCO,
    0,
    Opcode_get_nco_encode_fns, 0, 0 },
  { "get_perm_reg", ICLASS_iclass_GET_PERM_REG,
    0,
    Opcode_get_perm_reg_encode_fns, 0, 0 },
  { "get_phasor_n", ICLASS_iclass_GET_PHASOR_N,
    0,
    Opcode_get_phasor_n_encode_fns, 0, 0 },
  { "get_phasor_offset", ICLASS_iclass_GET_PHASOR_OFFSET,
    0,
    Opcode_get_phasor_offset_encode_fns, 0, 0 },
  { "get_sar", ICLASS_iclass_GET_SAR,
    0,
    Opcode_get_sar_encode_fns, 0, 0 },
  { "get_scale_reg", ICLASS_iclass_GET_SCALE_REG,
    0,
    Opcode_get_scale_reg_encode_fns, 0, 0 },
  { "get_smod_buf", ICLASS_iclass_GET_SMOD_BUF,
    0,
    Opcode_get_smod_buf_encode_fns, 0, 0 },
  { "get_smod_offset_table", ICLASS_iclass_GET_SMOD_OFFSET_TABLE,
    0,
    Opcode_get_smod_offset_table_encode_fns, 0, 0 },
  { "get_smod_pos", ICLASS_iclass_GET_SMOD_POS,
    0,
    Opcode_get_smod_pos_encode_fns, 0, 0 },
  { "get_sov", ICLASS_iclass_GET_SOV,
    0,
    Opcode_get_sov_encode_fns, 0, 0 },
  { "get_wght", ICLASS_iclass_GET_WGHT,
    0,
    Opcode_get_wght_encode_fns, 0, 0 },
  { "set_argmax", ICLASS_iclass_SET_ARGMAX,
    0,
    Opcode_set_argmax_encode_fns, 0, 0 },
  { "set_ext_regs", ICLASS_iclass_SET_EXT_REGS,
    0,
    Opcode_set_ext_regs_encode_fns, 0, 0 },
  { "set_hsar", ICLASS_iclass_SET_HSAR,
    0,
    Opcode_set_hsar_encode_fns, 0, 0 },
  { "set_llr_buf", ICLASS_iclass_SET_LLR_BUF,
    0,
    Opcode_set_llr_buf_encode_fns, 0, 0 },
  { "set_llr_pos", ICLASS_iclass_SET_LLR_POS,
    0,
    Opcode_set_llr_pos_encode_fns, 0, 0 },
  { "set_max", ICLASS_iclass_SET_MAX,
    0,
    Opcode_set_max_encode_fns, 0, 0 },
  { "set_nco", ICLASS_iclass_SET_NCO,
    0,
    Opcode_set_nco_encode_fns, 0, 0 },
  { "set_perm_reg", ICLASS_iclass_SET_PERM_REG,
    0,
    Opcode_set_perm_reg_encode_fns, 0, 0 },
  { "set_phasor_n", ICLASS_iclass_SET_PHASOR_N,
    0,
    Opcode_set_phasor_n_encode_fns, 0, 0 },
  { "set_phasor_offset", ICLASS_iclass_SET_PHASOR_OFFSET,
    0,
    Opcode_set_phasor_offset_encode_fns, 0, 0 },
  { "set_sar", ICLASS_iclass_SET_SAR,
    0,
    Opcode_set_sar_encode_fns, 0, 0 },
  { "set_scale_reg", ICLASS_iclass_SET_SCALE_REG,
    0,
    Opcode_set_scale_reg_encode_fns, 0, 0 },
  { "set_smod_buf", ICLASS_iclass_SET_SMOD_BUF,
    0,
    Opcode_set_smod_buf_encode_fns, 0, 0 },
  { "set_smod_offset_table", ICLASS_iclass_SET_SMOD_OFFSET_TABLE,
    0,
    Opcode_set_smod_offset_table_encode_fns, 0, 0 },
  { "set_smod_pos", ICLASS_iclass_SET_SMOD_POS,
    0,
    Opcode_set_smod_pos_encode_fns, 0, 0 },
  { "set_sov", ICLASS_iclass_SET_SOV,
    0,
    Opcode_set_sov_encode_fns, 0, 0 },
  { "set_wght", ICLASS_iclass_SET_WGHT,
    0,
    Opcode_set_wght_encode_fns, 0, 0 },
  { "lac2x32", ICLASS_iclass_LAC2X32,
    0,
    Opcode_lac2x32_encode_fns, 0, 0 },
  { "lac2x64_0", ICLASS_iclass_LAC2X64_0,
    0,
    Opcode_lac2x64_0_encode_fns, 0, 0 },
  { "lac2x64_1", ICLASS_iclass_LAC2X64_1,
    0,
    Opcode_lac2x64_1_encode_fns, 0, 0 },
  { "lac2x64_2", ICLASS_iclass_LAC2X64_2,
    0,
    Opcode_lac2x64_2_encode_fns, 0, 0 },
  { "lac2x64_3", ICLASS_iclass_LAC2X64_3,
    0,
    Opcode_lac2x64_3_encode_fns, 0, 0 },
  { "lac32_r", ICLASS_iclass_LAC32_R,
    0,
    Opcode_lac32_r_encode_fns, 0, 0 },
  { "lac_ih", ICLASS_iclass_LAC_IH,
    0,
    Opcode_lac_ih_encode_fns, 0, 0 },
  { "lac_il", ICLASS_iclass_LAC_IL,
    0,
    Opcode_lac_il_encode_fns, 0, 0 },
  { "lac_rh", ICLASS_iclass_LAC_RH,
    0,
    Opcode_lac_rh_encode_fns, 0, 0 },
  { "lac_rl", ICLASS_iclass_LAC_RL,
    0,
    Opcode_lac_rl_encode_fns, 0, 0 },
  { "lcm", ICLASS_iclass_LCM,
    0,
    Opcode_lcm_encode_fns, 0, 0 },
  { "lcm_pinc", ICLASS_iclass_LCM_PINC,
    0,
    Opcode_lcm_pinc_encode_fns, 0, 0 },
  { "lcm_pinc_x", ICLASS_iclass_LCM_PINC_X,
    0,
    Opcode_lcm_pinc_x_encode_fns, 0, 0 },
  { "lcm_u", ICLASS_iclass_LCM_U,
    0,
    Opcode_lcm_u_encode_fns, 0, 0 },
  { "lcm_x", ICLASS_iclass_LCM_X,
    0,
    Opcode_lcm_x_encode_fns, 0, 0 },
  { "lcm_xu", ICLASS_iclass_LCM_XU,
    0,
    Opcode_lcm_xu_encode_fns, 0, 0 },
  { "lp", ICLASS_iclass_LP,
    0,
    Opcode_lp_encode_fns, 0, 0 },
  { "lp_x", ICLASS_iclass_LP_X,
    0,
    Opcode_lp_x_encode_fns, 0, 0 },
  { "lq", ICLASS_iclass_LQ,
    0,
    Opcode_lq_encode_fns, 0, 0 },
  { "lq_x", ICLASS_iclass_LQ_X,
    0,
    Opcode_lq_x_encode_fns, 0, 0 },
  { "lut0", ICLASS_iclass_LUT0,
    0,
    Opcode_lut0_encode_fns, 0, 0 },
  { "lut1", ICLASS_iclass_LUT1,
    0,
    Opcode_lut1_encode_fns, 0, 0 },
  { "lut2", ICLASS_iclass_LUT2,
    0,
    Opcode_lut2_encode_fns, 0, 0 },
  { "lut3", ICLASS_iclass_LUT3,
    0,
    Opcode_lut3_encode_fns, 0, 0 },
  { "sac2x32", ICLASS_iclass_SAC2X32,
    0,
    Opcode_sac2x32_encode_fns, 0, 0 },
  { "sac2x64_0", ICLASS_iclass_SAC2X64_0,
    0,
    Opcode_sac2x64_0_encode_fns, 0, 0 },
  { "sac2x64_1", ICLASS_iclass_SAC2X64_1,
    0,
    Opcode_sac2x64_1_encode_fns, 0, 0 },
  { "sac2x64_2", ICLASS_iclass_SAC2X64_2,
    0,
    Opcode_sac2x64_2_encode_fns, 0, 0 },
  { "sac2x64_3", ICLASS_iclass_SAC2X64_3,
    0,
    Opcode_sac2x64_3_encode_fns, 0, 0 },
  { "sac32_r", ICLASS_iclass_SAC32_R,
    0,
    Opcode_sac32_r_encode_fns, 0, 0 },
  { "sac_ih", ICLASS_iclass_SAC_IH,
    0,
    Opcode_sac_ih_encode_fns, 0, 0 },
  { "sac_il", ICLASS_iclass_SAC_IL,
    0,
    Opcode_sac_il_encode_fns, 0, 0 },
  { "sac_rh", ICLASS_iclass_SAC_RH,
    0,
    Opcode_sac_rh_encode_fns, 0, 0 },
  { "sac_rl", ICLASS_iclass_SAC_RL,
    0,
    Opcode_sac_rl_encode_fns, 0, 0 },
  { "scm", ICLASS_iclass_SCM,
    0,
    Opcode_scm_encode_fns, 0, 0 },
  { "scm_pinc", ICLASS_iclass_SCM_PINC,
    0,
    Opcode_scm_pinc_encode_fns, 0, 0 },
  { "scm_pinc_x", ICLASS_iclass_SCM_PINC_X,
    0,
    Opcode_scm_pinc_x_encode_fns, 0, 0 },
  { "scm_u", ICLASS_iclass_SCM_U,
    0,
    Opcode_scm_u_encode_fns, 0, 0 },
  { "scm_x", ICLASS_iclass_SCM_X,
    0,
    Opcode_scm_x_encode_fns, 0, 0 },
  { "scm_xu", ICLASS_iclass_SCM_XU,
    0,
    Opcode_scm_xu_encode_fns, 0, 0 },
  { "store_p", ICLASS_iclass_STORE_P,
    0,
    Opcode_store_p_encode_fns, 0, 0 },
  { "store_q", ICLASS_iclass_STORE_Q,
    0,
    Opcode_store_q_encode_fns, 0, 0 },
  { "ar2cm_dup", ICLASS_iclass_AR2CM_DUP,
    0,
    Opcode_ar2cm_dup_encode_fns, 0, 0 },
  { "ar2cm_ln", ICLASS_iclass_AR2CM_LN,
    0,
    Opcode_ar2cm_ln_encode_fns, 0, 0 },
  { "ar2cm_ln_i", ICLASS_iclass_AR2CM_LN_I,
    0,
    Opcode_ar2cm_ln_i_encode_fns, 0, 0 },
  { "ar2cm_ln_r", ICLASS_iclass_AR2CM_LN_R,
    0,
    Opcode_ar2cm_ln_r_encode_fns, 0, 0 },
  { "ar2pq_ln", ICLASS_iclass_AR2PQ_LN,
    0,
    Opcode_ar2pq_ln_encode_fns, 0, 0 },
  { "ar2sar_dup", ICLASS_iclass_AR2SAR_DUP,
    0,
    Opcode_ar2sar_dup_encode_fns, 0, 0 },
  { "clrac", ICLASS_iclass_CLRAC,
    0,
    Opcode_clrac_encode_fns, 0, 0 },
  { "clrcm", ICLASS_iclass_CLRCM,
    0,
    Opcode_clrcm_encode_fns, 0, 0 },
  { "cm2ar_ln", ICLASS_iclass_CM2AR_LN,
    0,
    Opcode_cm2ar_ln_encode_fns, 0, 0 },
  { "cm2ar_ln_i", ICLASS_iclass_CM2AR_LN_I,
    0,
    Opcode_cm2ar_ln_i_encode_fns, 0, 0 },
  { "cm2ar_ln_r", ICLASS_iclass_CM2AR_LN_R,
    0,
    Opcode_cm2ar_ln_r_encode_fns, 0, 0 },
  { "comb_ar", ICLASS_iclass_COMB_AR,
    0,
    Opcode_comb_ar_encode_fns, 0, 0 },
  { "conj", ICLASS_iclass_CONJ,
    0,
    Opcode_conj_encode_fns, 0, 0 },
  { "mov2ac32_i", ICLASS_iclass_MOV2AC32_I,
    0,
    Opcode_mov2ac32_i_encode_fns, 0, 0 },
  { "mov2ac32_r", ICLASS_iclass_MOV2AC32_R,
    0,
    Opcode_mov2ac32_r_encode_fns, 0, 0 },
  { "mov2cm2pq", ICLASS_iclass_MOV2CM2PQ,
    0,
    Opcode_mov2cm2pq_encode_fns, 0, 0 },
  { "movac", ICLASS_iclass_MOVAC,
    0,
    Opcode_movac_encode_fns, 0, 0 },
  { "movac_i", ICLASS_iclass_MOVAC_I,
    0,
    Opcode_movac_i_encode_fns, 0, 0 },
  { "movac_i2r", ICLASS_iclass_MOVAC_I2R,
    0,
    Opcode_movac_i2r_encode_fns, 0, 0 },
  { "movac_r", ICLASS_iclass_MOVAC_R,
    0,
    Opcode_movac_r_encode_fns, 0, 0 },
  { "movac_r2i", ICLASS_iclass_MOVAC_R2I,
    0,
    Opcode_movac_r2i_encode_fns, 0, 0 },
  { "movar2", ICLASS_iclass_MOVAR2,
    0,
    Opcode_movar2_encode_fns, 0, 0 },
  { "movcm", ICLASS_iclass_MOVCM,
    0,
    Opcode_movcm_encode_fns, 0, 0 },
  { "movcm2pq", ICLASS_iclass_MOVCM2PQ,
    0,
    Opcode_movcm2pq_encode_fns, 0, 0 },
  { "movcnd_0", ICLASS_iclass_MOVCND_0,
    0,
    Opcode_movcnd_0_encode_fns, 0, 0 },
  { "movcnd_1", ICLASS_iclass_MOVCND_1,
    0,
    Opcode_movcnd_1_encode_fns, 0, 0 },
  { "movcnd_2", ICLASS_iclass_MOVCND_2,
    0,
    Opcode_movcnd_2_encode_fns, 0, 0 },
  { "movcnd_3", ICLASS_iclass_MOVCND_3,
    0,
    Opcode_movcnd_3_encode_fns, 0, 0 },
  { "movcnd_4", ICLASS_iclass_MOVCND_4,
    0,
    Opcode_movcnd_4_encode_fns, 0, 0 },
  { "movcnd_5", ICLASS_iclass_MOVCND_5,
    0,
    Opcode_movcnd_5_encode_fns, 0, 0 },
  { "movcnd_6", ICLASS_iclass_MOVCND_6,
    0,
    Opcode_movcnd_6_encode_fns, 0, 0 },
  { "movcnd_7", ICLASS_iclass_MOVCND_7,
    0,
    Opcode_movcnd_7_encode_fns, 0, 0 },
  { "movcnd8_0", ICLASS_iclass_MOVCND8_0,
    0,
    Opcode_movcnd8_0_encode_fns, 0, 0 },
  { "movcnd8_1", ICLASS_iclass_MOVCND8_1,
    0,
    Opcode_movcnd8_1_encode_fns, 0, 0 },
  { "movcnd8_2", ICLASS_iclass_MOVCND8_2,
    0,
    Opcode_movcnd8_2_encode_fns, 0, 0 },
  { "movcnd8_3", ICLASS_iclass_MOVCND8_3,
    0,
    Opcode_movcnd8_3_encode_fns, 0, 0 },
  { "movcnd8_4", ICLASS_iclass_MOVCND8_4,
    0,
    Opcode_movcnd8_4_encode_fns, 0, 0 },
  { "movcnd8_5", ICLASS_iclass_MOVCND8_5,
    0,
    Opcode_movcnd8_5_encode_fns, 0, 0 },
  { "movcnd8_6", ICLASS_iclass_MOVCND8_6,
    0,
    Opcode_movcnd8_6_encode_fns, 0, 0 },
  { "movcnd8_7", ICLASS_iclass_MOVCND8_7,
    0,
    Opcode_movcnd8_7_encode_fns, 0, 0 },
  { "mov_i", ICLASS_iclass_MOV_I,
    0,
    Opcode_mov_i_encode_fns, 0, 0 },
  { "movpq2pq", ICLASS_iclass_MOVPQ2PQ,
    0,
    Opcode_movpq2pq_encode_fns, 0, 0 },
  { "mov_r", ICLASS_iclass_MOV_R,
    0,
    Opcode_mov_r_encode_fns, 0, 0 },
  { "negcm", ICLASS_iclass_NEGCM,
    0,
    Opcode_negcm_encode_fns, 0, 0 },
  { "pop16llr_1", ICLASS_iclass_POP16LLR_1,
    0,
    Opcode_pop16llr_1_encode_fns, 0, 0 },
  { "pq2cm", ICLASS_iclass_PQ2CM,
    0,
    Opcode_pq2cm_encode_fns, 0, 0 },
  { "swapac_r", ICLASS_iclass_SWAPAC_R,
    0,
    Opcode_swapac_r_encode_fns, 0, 0 },
  { "swapac_ri", ICLASS_iclass_SWAPAC_RI,
    0,
    Opcode_swapac_ri_encode_fns, 0, 0 },
  { "swapb", ICLASS_iclass_SWAPB,
    0,
    Opcode_swapb_encode_fns, 0, 0 },
  { "add2ac", ICLASS_iclass_ADD2AC,
    0,
    Opcode_add2ac_encode_fns, 0, 0 },
  { "addac", ICLASS_iclass_ADDAC,
    0,
    Opcode_addac_encode_fns, 0, 0 },
  { "cdot", ICLASS_iclass_CDOT,
    0,
    Opcode_cdot_encode_fns, 0, 0 },
  { "cdotac", ICLASS_iclass_CDOTAC,
    0,
    Opcode_cdotac_encode_fns, 0, 0 },
  { "cdotacs", ICLASS_iclass_CDOTACS,
    0,
    Opcode_cdotacs_encode_fns, 0, 0 },
  { "cmac", ICLASS_iclass_CMAC,
    0,
    Opcode_cmac_encode_fns, 0, 0 },
  { "cmacs", ICLASS_iclass_CMACS,
    0,
    Opcode_cmacs_encode_fns, 0, 0 },
  { "cmpy", ICLASS_iclass_CMPY,
    0,
    Opcode_cmpy_encode_fns, 0, 0 },
  { "cmpy2cm", ICLASS_iclass_CMPY2CM,
    0,
    Opcode_cmpy2cm_encode_fns, 0, 0 },
  { "cmpy2pq", ICLASS_iclass_CMPY2PQ,
    0,
    Opcode_cmpy2pq_encode_fns, 0, 0 },
  { "cmpys", ICLASS_iclass_CMPYS,
    0,
    Opcode_cmpys_encode_fns, 0, 0 },
  { "cmpyxp2pq", ICLASS_iclass_CMPYXP2PQ,
    0,
    Opcode_cmpyxp2pq_encode_fns, 0, 0 },
  { "comb32", ICLASS_iclass_COMB32,
    0,
    Opcode_comb32_encode_fns, 0, 0 },
  { "dot", ICLASS_iclass_DOT,
    0,
    Opcode_dot_encode_fns, 0, 0 },
  { "dotac", ICLASS_iclass_DOTAC,
    0,
    Opcode_dotac_encode_fns, 0, 0 },
  { "dotacs", ICLASS_iclass_DOTACS,
    0,
    Opcode_dotacs_encode_fns, 0, 0 },
  { "lin_int", ICLASS_iclass_LIN_INT,
    0,
    Opcode_lin_int_encode_fns, 0, 0 },
  { "llrpre1", ICLASS_iclass_LLRPRE1,
    0,
    Opcode_llrpre1_encode_fns, 0, 0 },
  { "llrpre2", ICLASS_iclass_LLRPRE2,
    0,
    Opcode_llrpre2_encode_fns, 0, 0 },
  { "mac", ICLASS_iclass_MAC,
    0,
    Opcode_mac_encode_fns, 0, 0 },
  { "mac8", ICLASS_iclass_MAC8,
    0,
    Opcode_mac8_encode_fns, 0, 0 },
  { "macd8", ICLASS_iclass_MACD8,
    0,
    Opcode_macd8_encode_fns, 0, 0 },
  { "macpqxp_0", ICLASS_iclass_MACPQXP_0,
    0,
    Opcode_macpqxp_0_encode_fns, 0, 0 },
  { "macpqxp_1", ICLASS_iclass_MACPQXP_1,
    0,
    Opcode_macpqxp_1_encode_fns, 0, 0 },
  { "macpqxp_2", ICLASS_iclass_MACPQXP_2,
    0,
    Opcode_macpqxp_2_encode_fns, 0, 0 },
  { "macpqxp_3", ICLASS_iclass_MACPQXP_3,
    0,
    Opcode_macpqxp_3_encode_fns, 0, 0 },
  { "macs", ICLASS_iclass_MACS,
    0,
    Opcode_macs_encode_fns, 0, 0 },
  { "macxp2_0", ICLASS_iclass_MACXP2_0,
    0,
    Opcode_macxp2_0_encode_fns, 0, 0 },
  { "macxp2_1", ICLASS_iclass_MACXP2_1,
    0,
    Opcode_macxp2_1_encode_fns, 0, 0 },
  { "macxp_0", ICLASS_iclass_MACXP_0,
    0,
    Opcode_macxp_0_encode_fns, 0, 0 },
  { "macxp_1", ICLASS_iclass_MACXP_1,
    0,
    Opcode_macxp_1_encode_fns, 0, 0 },
  { "macxp_2", ICLASS_iclass_MACXP_2,
    0,
    Opcode_macxp_2_encode_fns, 0, 0 },
  { "macxp_3", ICLASS_iclass_MACXP_3,
    0,
    Opcode_macxp_3_encode_fns, 0, 0 },
  { "mov2ac", ICLASS_iclass_MOV2AC,
    0,
    Opcode_mov2ac_encode_fns, 0, 0 },
  { "mpy", ICLASS_iclass_MPY,
    0,
    Opcode_mpy_encode_fns, 0, 0 },
  { "mpy2cm", ICLASS_iclass_MPY2CM,
    0,
    Opcode_mpy2cm_encode_fns, 0, 0 },
  { "mpy2pq", ICLASS_iclass_MPY2PQ,
    0,
    Opcode_mpy2pq_encode_fns, 0, 0 },
  { "mpy8", ICLASS_iclass_MPY8,
    0,
    Opcode_mpy8_encode_fns, 0, 0 },
  { "mpyadd8_2cm", ICLASS_iclass_MPYADD8_2CM,
    0,
    Opcode_mpyadd8_2cm_encode_fns, 0, 0 },
  { "mpyd8", ICLASS_iclass_MPYD8,
    0,
    Opcode_mpyd8_encode_fns, 0, 0 },
  { "mpypqxp_0", ICLASS_iclass_MPYPQXP_0,
    0,
    Opcode_mpypqxp_0_encode_fns, 0, 0 },
  { "mpypqxp_1", ICLASS_iclass_MPYPQXP_1,
    0,
    Opcode_mpypqxp_1_encode_fns, 0, 0 },
  { "mpypqxp_2", ICLASS_iclass_MPYPQXP_2,
    0,
    Opcode_mpypqxp_2_encode_fns, 0, 0 },
  { "mpypqxp_3", ICLASS_iclass_MPYPQXP_3,
    0,
    Opcode_mpypqxp_3_encode_fns, 0, 0 },
  { "mpys", ICLASS_iclass_MPYS,
    0,
    Opcode_mpys_encode_fns, 0, 0 },
  { "mpyxp2pq", ICLASS_iclass_MPYXP2PQ,
    0,
    Opcode_mpyxp2pq_encode_fns, 0, 0 },
  { "mpyxp2_0", ICLASS_iclass_MPYXP2_0,
    0,
    Opcode_mpyxp2_0_encode_fns, 0, 0 },
  { "mpyxp2_1", ICLASS_iclass_MPYXP2_1,
    0,
    Opcode_mpyxp2_1_encode_fns, 0, 0 },
  { "mpyxp_0", ICLASS_iclass_MPYXP_0,
    0,
    Opcode_mpyxp_0_encode_fns, 0, 0 },
  { "mpyxp_1", ICLASS_iclass_MPYXP_1,
    0,
    Opcode_mpyxp_1_encode_fns, 0, 0 },
  { "mpyxp_2", ICLASS_iclass_MPYXP_2,
    0,
    Opcode_mpyxp_2_encode_fns, 0, 0 },
  { "mpyxp_3", ICLASS_iclass_MPYXP_3,
    0,
    Opcode_mpyxp_3_encode_fns, 0, 0 },
  { "normacd", ICLASS_iclass_NORMACD,
    0,
    Opcode_normacd_encode_fns, 0, 0 },
  { "normacpq_i", ICLASS_iclass_NORMACPQ_I,
    0,
    Opcode_normacpq_i_encode_fns, 0, 0 },
  { "normacpq_r", ICLASS_iclass_NORMACPQ_R,
    0,
    Opcode_normacpq_r_encode_fns, 0, 0 },
  { "normd", ICLASS_iclass_NORMD,
    0,
    Opcode_normd_encode_fns, 0, 0 },
  { "normpypq_i", ICLASS_iclass_NORMPYPQ_I,
    0,
    Opcode_normpypq_i_encode_fns, 0, 0 },
  { "normpypq_r", ICLASS_iclass_NORMPYPQ_R,
    0,
    Opcode_normpypq_r_encode_fns, 0, 0 },
  { "rcmac", ICLASS_iclass_RCMAC,
    0,
    Opcode_rcmac_encode_fns, 0, 0 },
  { "rcmpy", ICLASS_iclass_RCMPY,
    0,
    Opcode_rcmpy_encode_fns, 0, 0 },
  { "rcmpy2cm", ICLASS_iclass_RCMPY2CM,
    0,
    Opcode_rcmpy2cm_encode_fns, 0, 0 },
  { "rfir", ICLASS_iclass_RFIR,
    0,
    Opcode_rfir_encode_fns, 0, 0 },
  { "rfira", ICLASS_iclass_RFIRA,
    0,
    Opcode_rfira_encode_fns, 0, 0 },
  { "rfird", ICLASS_iclass_RFIRD,
    0,
    Opcode_rfird_encode_fns, 0, 0 },
  { "rfirda", ICLASS_iclass_RFIRDA,
    0,
    Opcode_rfirda_encode_fns, 0, 0 },
  { "rmac", ICLASS_iclass_RMAC,
    0,
    Opcode_rmac_encode_fns, 0, 0 },
  { "rmpy", ICLASS_iclass_RMPY,
    0,
    Opcode_rmpy_encode_fns, 0, 0 },
  { "rmpy2cm", ICLASS_iclass_RMPY2CM,
    0,
    Opcode_rmpy2cm_encode_fns, 0, 0 },
  { "smod_align", ICLASS_iclass_SMOD_ALIGN,
    0,
    Opcode_smod_align_encode_fns, 0, 0 },
  { "smod_scr", ICLASS_iclass_SMOD_SCR,
    0,
    Opcode_smod_scr_encode_fns, 0, 0 },
  { "sub2ac", ICLASS_iclass_SUB2AC,
    0,
    Opcode_sub2ac_encode_fns, 0, 0 },
  { "wght32", ICLASS_iclass_WGHT32,
    0,
    Opcode_wght32_encode_fns, 0, 0 },
  { "clrtiep", ICLASS_iclass_CLRTIEP,
    0,
    Opcode_clrtiep_encode_fns, 0, 0 },
  { "ext_2fifo_0", ICLASS_iclass_EXT_2FIFO_0,
    0,
    Opcode_ext_2fifo_0_encode_fns, 0, 0 },
  { "ext_2fifo_1", ICLASS_iclass_EXT_2FIFO_1,
    0,
    Opcode_ext_2fifo_1_encode_fns, 0, 0 },
  { "ext_2fifo_2", ICLASS_iclass_EXT_2FIFO_2,
    0,
    Opcode_ext_2fifo_2_encode_fns, 0, 0 },
  { "ext_2fifo_3", ICLASS_iclass_EXT_2FIFO_3,
    0,
    Opcode_ext_2fifo_3_encode_fns, 0, 0 },
  { "ext_r2fifo_0", ICLASS_iclass_EXT_R2FIFO_0,
    0,
    Opcode_ext_r2fifo_0_encode_fns, 0, 0 },
  { "ext_r2fifo_1", ICLASS_iclass_EXT_R2FIFO_1,
    0,
    Opcode_ext_r2fifo_1_encode_fns, 0, 0 },
  { "ext_r2fifo_2", ICLASS_iclass_EXT_R2FIFO_2,
    0,
    Opcode_ext_r2fifo_2_encode_fns, 0, 0 },
  { "ext_r2fifo_3", ICLASS_iclass_EXT_R2FIFO_3,
    0,
    Opcode_ext_r2fifo_3_encode_fns, 0, 0 },
  { "lut", ICLASS_iclass_LUT,
    0,
    Opcode_lut_encode_fns, 0, 0 },
  { "lut_ar", ICLASS_iclass_LUT_AR,
    0,
    Opcode_lut_ar_encode_fns, 0, 0 },
  { "lut_iext", ICLASS_iclass_LUT_IEXT,
    0,
    Opcode_lut_iext_encode_fns, 0, 0 },
  { "lut_phasor", ICLASS_iclass_LUT_PHASOR,
    0,
    Opcode_lut_phasor_encode_fns, 0, 0 },
  { "lut_rext", ICLASS_iclass_LUT_REXT,
    0,
    Opcode_lut_rext_encode_fns, 0, 0 },
  { "lut_write", ICLASS_iclass_LUT_WRITE,
    0,
    Opcode_lut_write_encode_fns, 0, 0 },
  { "moveq128_0", ICLASS_iclass_MOVEQ128_0,
    0,
    Opcode_moveq128_0_encode_fns, 0, 0 },
  { "moveq128_1", ICLASS_iclass_MOVEQ128_1,
    0,
    Opcode_moveq128_1_encode_fns, 0, 0 },
  { "moveq128_2", ICLASS_iclass_MOVEQ128_2,
    0,
    Opcode_moveq128_2_encode_fns, 0, 0 },
  { "moveq128_3", ICLASS_iclass_MOVEQ128_3,
    0,
    Opcode_moveq128_3_encode_fns, 0, 0 },
  { "moveq128_4", ICLASS_iclass_MOVEQ128_4,
    0,
    Opcode_moveq128_4_encode_fns, 0, 0 },
  { "moveq128_5", ICLASS_iclass_MOVEQ128_5,
    0,
    Opcode_moveq128_5_encode_fns, 0, 0 },
  { "moveq32_0", ICLASS_iclass_MOVEQ32_0,
    0,
    Opcode_moveq32_0_encode_fns, 0, 0 },
  { "moveq32_1", ICLASS_iclass_MOVEQ32_1,
    0,
    Opcode_moveq32_1_encode_fns, 0, 0 },
  { "moveq32_2", ICLASS_iclass_MOVEQ32_2,
    0,
    Opcode_moveq32_2_encode_fns, 0, 0 },
  { "moveq32_3", ICLASS_iclass_MOVEQ32_3,
    0,
    Opcode_moveq32_3_encode_fns, 0, 0 },
  { "nco_update", ICLASS_iclass_NCO_UPDATE,
    0,
    Opcode_nco_update_encode_fns, 0, 0 },
  { "pop128_0", ICLASS_iclass_POP128_0,
    0,
    Opcode_pop128_0_encode_fns, 0, 0 },
  { "pop128_1", ICLASS_iclass_POP128_1,
    0,
    Opcode_pop128_1_encode_fns, 0, 0 },
  { "pop128_2", ICLASS_iclass_POP128_2,
    0,
    Opcode_pop128_2_encode_fns, 0, 0 },
  { "pop128_3", ICLASS_iclass_POP128_3,
    0,
    Opcode_pop128_3_encode_fns, 0, 0 },
  { "pop128_4", ICLASS_iclass_POP128_4,
    0,
    Opcode_pop128_4_encode_fns, 0, 0 },
  { "pop128_5", ICLASS_iclass_POP128_5,
    0,
    Opcode_pop128_5_encode_fns, 0, 0 },
  { "pop128_2cmpq_0", ICLASS_iclass_POP128_2CMPQ_0,
    0,
    Opcode_pop128_2cmpq_0_encode_fns, 0, 0 },
  { "pop128_2cmpq_1", ICLASS_iclass_POP128_2CMPQ_1,
    0,
    Opcode_pop128_2cmpq_1_encode_fns, 0, 0 },
  { "pop128_2cmpq_2", ICLASS_iclass_POP128_2CMPQ_2,
    0,
    Opcode_pop128_2cmpq_2_encode_fns, 0, 0 },
  { "pop128_2cmpq_3", ICLASS_iclass_POP128_2CMPQ_3,
    0,
    Opcode_pop128_2cmpq_3_encode_fns, 0, 0 },
  { "pop128_2m_0", ICLASS_iclass_POP128_2M_0,
    0,
    Opcode_pop128_2m_0_encode_fns, 0, 0 },
  { "pop128_2m_1", ICLASS_iclass_POP128_2M_1,
    0,
    Opcode_pop128_2m_1_encode_fns, 0, 0 },
  { "pop128_2m_2", ICLASS_iclass_POP128_2M_2,
    0,
    Opcode_pop128_2m_2_encode_fns, 0, 0 },
  { "pop128_2m_3", ICLASS_iclass_POP128_2M_3,
    0,
    Opcode_pop128_2m_3_encode_fns, 0, 0 },
  { "pop128_2pq_0", ICLASS_iclass_POP128_2PQ_0,
    0,
    Opcode_pop128_2pq_0_encode_fns, 0, 0 },
  { "pop128_2pq_1", ICLASS_iclass_POP128_2PQ_1,
    0,
    Opcode_pop128_2pq_1_encode_fns, 0, 0 },
  { "pop128_2pq_2", ICLASS_iclass_POP128_2PQ_2,
    0,
    Opcode_pop128_2pq_2_encode_fns, 0, 0 },
  { "pop128_2pq_3", ICLASS_iclass_POP128_2PQ_3,
    0,
    Opcode_pop128_2pq_3_encode_fns, 0, 0 },
  { "pop128_2pq_4", ICLASS_iclass_POP128_2PQ_4,
    0,
    Opcode_pop128_2pq_4_encode_fns, 0, 0 },
  { "pop128_2pq_5", ICLASS_iclass_POP128_2PQ_5,
    0,
    Opcode_pop128_2pq_5_encode_fns, 0, 0 },
  { "pop2x128_2pq_01", ICLASS_iclass_POP2X128_2PQ_01,
    0,
    Opcode_pop2x128_2pq_01_encode_fns, 0, 0 },
  { "pop2x128_2pq_03", ICLASS_iclass_POP2X128_2PQ_03,
    0,
    Opcode_pop2x128_2pq_03_encode_fns, 0, 0 },
  { "pop2x128_2pq_21", ICLASS_iclass_POP2X128_2PQ_21,
    0,
    Opcode_pop2x128_2pq_21_encode_fns, 0, 0 },
  { "pop2x128_2pq_23", ICLASS_iclass_POP2X128_2PQ_23,
    0,
    Opcode_pop2x128_2pq_23_encode_fns, 0, 0 },
  { "pop32_0", ICLASS_iclass_POP32_0,
    0,
    Opcode_pop32_0_encode_fns, 0, 0 },
  { "pop32_1", ICLASS_iclass_POP32_1,
    0,
    Opcode_pop32_1_encode_fns, 0, 0 },
  { "pop32_2", ICLASS_iclass_POP32_2,
    0,
    Opcode_pop32_2_encode_fns, 0, 0 },
  { "pop32_3", ICLASS_iclass_POP32_3,
    0,
    Opcode_pop32_3_encode_fns, 0, 0 },
  { "push128", ICLASS_iclass_PUSH128,
    0,
    Opcode_push128_encode_fns, 0, 0 },
  { "push128_m", ICLASS_iclass_PUSH128_M,
    0,
    Opcode_push128_m_encode_fns, 0, 0 },
  { "push128_pq", ICLASS_iclass_PUSH128_PQ,
    0,
    Opcode_push128_pq_encode_fns, 0, 0 },
  { "push2x128_pq", ICLASS_iclass_PUSH2X128_PQ,
    0,
    Opcode_push2x128_pq_encode_fns, 0, 0 },
  { "push32", ICLASS_iclass_PUSH32,
    0,
    Opcode_push32_encode_fns, 0, 0 },
  { "qready", ICLASS_iclass_QREADY,
    0,
    Opcode_qready_encode_fns, 0, 0 },
  { "rdtiep", ICLASS_iclass_RDTIEP,
    0,
    Opcode_rdtiep_encode_fns, 0, 0 },
  { "settiep", ICLASS_iclass_SETTIEP,
    0,
    Opcode_settiep_encode_fns, 0, 0 },
  { "smod_lut", ICLASS_iclass_SMOD_LUT,
    0,
    Opcode_smod_lut_encode_fns, 0, 0 },
  { "wrtbsigq", ICLASS_iclass_WRTBSIGQ,
    0,
    Opcode_wrtbsigq_encode_fns, 0, 0 },
  { "wrtbsigqm", ICLASS_iclass_WRTBSIGQM,
    0,
    Opcode_wrtbsigqm_encode_fns, 0, 0 },
  { "wrtiep", ICLASS_iclass_WRTIEP,
    0,
    Opcode_wrtiep_encode_fns, 0, 0 },
  { "wrtsigq", ICLASS_iclass_WRTSIGQ,
    0,
    Opcode_wrtsigq_encode_fns, 0, 0 },
  { "abs8", ICLASS_iclass_ABS8,
    0,
    Opcode_abs8_encode_fns, 0, 0 },
  { "add16", ICLASS_iclass_ADD16,
    0,
    Opcode_add16_encode_fns, 0, 0 },
  { "add32", ICLASS_iclass_ADD32,
    0,
    Opcode_add32_encode_fns, 0, 0 },
  { "addac_i2r", ICLASS_iclass_ADDAC_I2R,
    0,
    Opcode_addac_i2r_encode_fns, 0, 0 },
  { "addac_r2i", ICLASS_iclass_ADDAC_R2I,
    0,
    Opcode_addac_r2i_encode_fns, 0, 0 },
  { "addar2", ICLASS_iclass_ADDAR2,
    0,
    Opcode_addar2_encode_fns, 0, 0 },
  { "addcm", ICLASS_iclass_ADDCM,
    0,
    Opcode_addcm_encode_fns, 0, 0 },
  { "addwrp", ICLASS_iclass_ADDWRP,
    0,
    Opcode_addwrp_encode_fns, 0, 0 },
  { "and128", ICLASS_iclass_AND128,
    0,
    Opcode_and128_encode_fns, 0, 0 },
  { "argmax8", ICLASS_iclass_ARGMAX8,
    0,
    Opcode_argmax8_encode_fns, 0, 0 },
  { "asl", ICLASS_iclass_ASL,
    0,
    Opcode_asl_encode_fns, 0, 0 },
  { "asl32", ICLASS_iclass_ASL32,
    0,
    Opcode_asl32_encode_fns, 0, 0 },
  { "aslacm", ICLASS_iclass_ASLACM,
    0,
    Opcode_aslacm_encode_fns, 0, 0 },
  { "aslm", ICLASS_iclass_ASLM,
    0,
    Opcode_aslm_encode_fns, 0, 0 },
  { "aslm32", ICLASS_iclass_ASLM32,
    0,
    Opcode_aslm32_encode_fns, 0, 0 },
  { "asr", ICLASS_iclass_ASR,
    0,
    Opcode_asr_encode_fns, 0, 0 },
  { "asr32", ICLASS_iclass_ASR32,
    0,
    Opcode_asr32_encode_fns, 0, 0 },
  { "asrac", ICLASS_iclass_ASRAC,
    0,
    Opcode_asrac_encode_fns, 0, 0 },
  { "asrm", ICLASS_iclass_ASRM,
    0,
    Opcode_asrm_encode_fns, 0, 0 },
  { "bitfext", ICLASS_iclass_BITFEXT,
    0,
    Opcode_bitfext_encode_fns, 0, 0 },
  { "bitfins", ICLASS_iclass_BITFINS,
    0,
    Opcode_bitfins_encode_fns, 0, 0 },
  { "clb_c", ICLASS_iclass_CLB_C,
    0,
    Opcode_clb_c_encode_fns, 0, 0 },
  { "clb_r", ICLASS_iclass_CLB_R,
    0,
    Opcode_clb_r_encode_fns, 0, 0 },
  { "cmp8", ICLASS_iclass_CMP8,
    0,
    Opcode_cmp8_encode_fns, 0, 0 },
  { "cmp_i", ICLASS_iclass_CMP_I,
    0,
    Opcode_cmp_i_encode_fns, 0, 0 },
  { "cmp_r", ICLASS_iclass_CMP_R,
    0,
    Opcode_cmp_r_encode_fns, 0, 0 },
  { "ext", ICLASS_iclass_EXT,
    0,
    Opcode_ext_encode_fns, 0, 0 },
  { "ext_r", ICLASS_iclass_EXT_R,
    0,
    Opcode_ext_r_encode_fns, 0, 0 },
  { "ext32_i", ICLASS_iclass_EXT32_I,
    0,
    Opcode_ext32_i_encode_fns, 0, 0 },
  { "ext32_r", ICLASS_iclass_EXT32_R,
    0,
    Opcode_ext32_r_encode_fns, 0, 0 },
  { "extui4", ICLASS_iclass_EXTUI4,
    0,
    Opcode_extui4_encode_fns, 0, 0 },
  { "lslm", ICLASS_iclass_LSLM,
    0,
    Opcode_lslm_encode_fns, 0, 0 },
  { "lsrm", ICLASS_iclass_LSRM,
    0,
    Opcode_lsrm_encode_fns, 0, 0 },
  { "max8", ICLASS_iclass_MAX8,
    0,
    Opcode_max8_encode_fns, 0, 0 },
  { "mean", ICLASS_iclass_MEAN,
    0,
    Opcode_mean_encode_fns, 0, 0 },
  { "mean32", ICLASS_iclass_MEAN32,
    0,
    Opcode_mean32_encode_fns, 0, 0 },
  { "min8", ICLASS_iclass_MIN8,
    0,
    Opcode_min8_encode_fns, 0, 0 },
  { "minclb_c", ICLASS_iclass_MINCLB_C,
    0,
    Opcode_minclb_c_encode_fns, 0, 0 },
  { "minclb_r", ICLASS_iclass_MINCLB_R,
    0,
    Opcode_minclb_r_encode_fns, 0, 0 },
  { "not128", ICLASS_iclass_NOT128,
    0,
    Opcode_not128_encode_fns, 0, 0 },
  { "or128", ICLASS_iclass_OR128,
    0,
    Opcode_or128_encode_fns, 0, 0 },
  { "perm", ICLASS_iclass_PERM,
    0,
    Opcode_perm_encode_fns, 0, 0 },
  { "redac", ICLASS_iclass_REDAC,
    0,
    Opcode_redac_encode_fns, 0, 0 },
  { "redac2", ICLASS_iclass_REDAC2,
    0,
    Opcode_redac2_encode_fns, 0, 0 },
  { "redac4", ICLASS_iclass_REDAC4,
    0,
    Opcode_redac4_encode_fns, 0, 0 },
  { "redacs", ICLASS_iclass_REDACS,
    0,
    Opcode_redacs_encode_fns, 0, 0 },
  { "sminclb_c", ICLASS_iclass_SMINCLB_C,
    0,
    Opcode_sminclb_c_encode_fns, 0, 0 },
  { "sminclb_r", ICLASS_iclass_SMINCLB_R,
    0,
    Opcode_sminclb_r_encode_fns, 0, 0 },
  { "stswapbm", ICLASS_iclass_STSWAPBM,
    0,
    Opcode_stswapbm_encode_fns, 0, 0 },
  { "stswapbmu", ICLASS_iclass_STSWAPBMU,
    0,
    Opcode_stswapbmu_encode_fns, 0, 0 },
  { "sub32", ICLASS_iclass_SUB32,
    0,
    Opcode_sub32_encode_fns, 0, 0 },
  { "subac_i2r", ICLASS_iclass_SUBAC_I2R,
    0,
    Opcode_subac_i2r_encode_fns, 0, 0 },
  { "subac_r2i", ICLASS_iclass_SUBAC_R2I,
    0,
    Opcode_subac_r2i_encode_fns, 0, 0 },
  { "subarx", ICLASS_iclass_SUBARX,
    0,
    Opcode_subarx_encode_fns, 0, 0 },
  { "subcm", ICLASS_iclass_SUBCM,
    0,
    Opcode_subcm_encode_fns, 0, 0 },
  { "submean", ICLASS_iclass_SUBMEAN,
    0,
    Opcode_submean_encode_fns, 0, 0 },
  { "subwrp", ICLASS_iclass_SUBWRP,
    0,
    Opcode_subwrp_encode_fns, 0, 0 },
  { "trans", ICLASS_iclass_TRANS,
    0,
    Opcode_trans_encode_fns, 0, 0 },
  { "xor128", ICLASS_iclass_XOR128,
    0,
    Opcode_xor128_encode_fns, 0, 0 },
  { "rur.expstate", ICLASS_rur_expstate,
    0,
    Opcode_rur_expstate_encode_fns, 0, 0 },
  { "wur.expstate", ICLASS_wur_expstate,
    0,
    Opcode_wur_expstate_encode_fns, 0, 0 },
  { "rur.sov", ICLASS_rur_sov,
    0,
    Opcode_rur_sov_encode_fns, 0, 0 },
  { "wur.sov", ICLASS_wur_sov,
    0,
    Opcode_wur_sov_encode_fns, 0, 0 },
  { "rur.sat_mode", ICLASS_rur_sat_mode,
    0,
    Opcode_rur_sat_mode_encode_fns, 0, 0 },
  { "wur.sat_mode", ICLASS_wur_sat_mode,
    0,
    Opcode_wur_sat_mode_encode_fns, 0, 0 },
  { "rur.sar0", ICLASS_rur_sar0,
    0,
    Opcode_rur_sar0_encode_fns, 0, 0 },
  { "wur.sar0", ICLASS_wur_sar0,
    0,
    Opcode_wur_sar0_encode_fns, 0, 0 },
  { "rur.sar1", ICLASS_rur_sar1,
    0,
    Opcode_rur_sar1_encode_fns, 0, 0 },
  { "wur.sar1", ICLASS_wur_sar1,
    0,
    Opcode_wur_sar1_encode_fns, 0, 0 },
  { "rur.sar2", ICLASS_rur_sar2,
    0,
    Opcode_rur_sar2_encode_fns, 0, 0 },
  { "wur.sar2", ICLASS_wur_sar2,
    0,
    Opcode_wur_sar2_encode_fns, 0, 0 },
  { "rur.sar3", ICLASS_rur_sar3,
    0,
    Opcode_rur_sar3_encode_fns, 0, 0 },
  { "wur.sar3", ICLASS_wur_sar3,
    0,
    Opcode_wur_sar3_encode_fns, 0, 0 },
  { "rur.hsar0", ICLASS_rur_hsar0,
    0,
    Opcode_rur_hsar0_encode_fns, 0, 0 },
  { "wur.hsar0", ICLASS_wur_hsar0,
    0,
    Opcode_wur_hsar0_encode_fns, 0, 0 },
  { "rur.hsar1", ICLASS_rur_hsar1,
    0,
    Opcode_rur_hsar1_encode_fns, 0, 0 },
  { "wur.hsar1", ICLASS_wur_hsar1,
    0,
    Opcode_wur_hsar1_encode_fns, 0, 0 },
  { "rur.hsar2", ICLASS_rur_hsar2,
    0,
    Opcode_rur_hsar2_encode_fns, 0, 0 },
  { "wur.hsar2", ICLASS_wur_hsar2,
    0,
    Opcode_wur_hsar2_encode_fns, 0, 0 },
  { "rur.hsar3", ICLASS_rur_hsar3,
    0,
    Opcode_rur_hsar3_encode_fns, 0, 0 },
  { "wur.hsar3", ICLASS_wur_hsar3,
    0,
    Opcode_wur_hsar3_encode_fns, 0, 0 },
  { "rur.max_reg_0", ICLASS_rur_max_reg_0,
    0,
    Opcode_rur_max_reg_0_encode_fns, 0, 0 },
  { "wur.max_reg_0", ICLASS_wur_max_reg_0,
    0,
    Opcode_wur_max_reg_0_encode_fns, 0, 0 },
  { "rur.max_reg_1", ICLASS_rur_max_reg_1,
    0,
    Opcode_rur_max_reg_1_encode_fns, 0, 0 },
  { "wur.max_reg_1", ICLASS_wur_max_reg_1,
    0,
    Opcode_wur_max_reg_1_encode_fns, 0, 0 },
  { "rur.max_reg_2", ICLASS_rur_max_reg_2,
    0,
    Opcode_rur_max_reg_2_encode_fns, 0, 0 },
  { "wur.max_reg_2", ICLASS_wur_max_reg_2,
    0,
    Opcode_wur_max_reg_2_encode_fns, 0, 0 },
  { "rur.max_reg_3", ICLASS_rur_max_reg_3,
    0,
    Opcode_rur_max_reg_3_encode_fns, 0, 0 },
  { "wur.max_reg_3", ICLASS_wur_max_reg_3,
    0,
    Opcode_wur_max_reg_3_encode_fns, 0, 0 },
  { "rur.arg_max_reg_0", ICLASS_rur_arg_max_reg_0,
    0,
    Opcode_rur_arg_max_reg_0_encode_fns, 0, 0 },
  { "wur.arg_max_reg_0", ICLASS_wur_arg_max_reg_0,
    0,
    Opcode_wur_arg_max_reg_0_encode_fns, 0, 0 },
  { "rur.arg_max_reg_1", ICLASS_rur_arg_max_reg_1,
    0,
    Opcode_rur_arg_max_reg_1_encode_fns, 0, 0 },
  { "wur.arg_max_reg_1", ICLASS_wur_arg_max_reg_1,
    0,
    Opcode_wur_arg_max_reg_1_encode_fns, 0, 0 },
  { "rur.arg_max_reg_2", ICLASS_rur_arg_max_reg_2,
    0,
    Opcode_rur_arg_max_reg_2_encode_fns, 0, 0 },
  { "wur.arg_max_reg_2", ICLASS_wur_arg_max_reg_2,
    0,
    Opcode_wur_arg_max_reg_2_encode_fns, 0, 0 },
  { "rur.arg_max_reg_3", ICLASS_rur_arg_max_reg_3,
    0,
    Opcode_rur_arg_max_reg_3_encode_fns, 0, 0 },
  { "wur.arg_max_reg_3", ICLASS_wur_arg_max_reg_3,
    0,
    Opcode_wur_arg_max_reg_3_encode_fns, 0, 0 },
  { "rur.nco_counter_0", ICLASS_rur_nco_counter_0,
    0,
    Opcode_rur_nco_counter_0_encode_fns, 0, 0 },
  { "wur.nco_counter_0", ICLASS_wur_nco_counter_0,
    0,
    Opcode_wur_nco_counter_0_encode_fns, 0, 0 },
  { "rur.nco_counter_1", ICLASS_rur_nco_counter_1,
    0,
    Opcode_rur_nco_counter_1_encode_fns, 0, 0 },
  { "wur.nco_counter_1", ICLASS_wur_nco_counter_1,
    0,
    Opcode_wur_nco_counter_1_encode_fns, 0, 0 },
  { "rur.nco_counter_2", ICLASS_rur_nco_counter_2,
    0,
    Opcode_rur_nco_counter_2_encode_fns, 0, 0 },
  { "wur.nco_counter_2", ICLASS_wur_nco_counter_2,
    0,
    Opcode_wur_nco_counter_2_encode_fns, 0, 0 },
  { "rur.nco_counter_3", ICLASS_rur_nco_counter_3,
    0,
    Opcode_rur_nco_counter_3_encode_fns, 0, 0 },
  { "wur.nco_counter_3", ICLASS_wur_nco_counter_3,
    0,
    Opcode_wur_nco_counter_3_encode_fns, 0, 0 },
  { "rur.interp_ext_n", ICLASS_rur_interp_ext_n,
    0,
    Opcode_rur_interp_ext_n_encode_fns, 0, 0 },
  { "wur.interp_ext_n", ICLASS_wur_interp_ext_n,
    0,
    Opcode_wur_interp_ext_n_encode_fns, 0, 0 },
  { "rur.interp_ext_l", ICLASS_rur_interp_ext_l,
    0,
    Opcode_rur_interp_ext_l_encode_fns, 0, 0 },
  { "wur.interp_ext_l", ICLASS_wur_interp_ext_l,
    0,
    Opcode_wur_interp_ext_l_encode_fns, 0, 0 },
  { "rur.llr_buf_0", ICLASS_rur_llr_buf_0,
    0,
    Opcode_rur_llr_buf_0_encode_fns, 0, 0 },
  { "wur.llr_buf_0", ICLASS_wur_llr_buf_0,
    0,
    Opcode_wur_llr_buf_0_encode_fns, 0, 0 },
  { "rur.llr_buf_1", ICLASS_rur_llr_buf_1,
    0,
    Opcode_rur_llr_buf_1_encode_fns, 0, 0 },
  { "wur.llr_buf_1", ICLASS_wur_llr_buf_1,
    0,
    Opcode_wur_llr_buf_1_encode_fns, 0, 0 },
  { "rur.llr_buf_2", ICLASS_rur_llr_buf_2,
    0,
    Opcode_rur_llr_buf_2_encode_fns, 0, 0 },
  { "wur.llr_buf_2", ICLASS_wur_llr_buf_2,
    0,
    Opcode_wur_llr_buf_2_encode_fns, 0, 0 },
  { "rur.llr_buf_3", ICLASS_rur_llr_buf_3,
    0,
    Opcode_rur_llr_buf_3_encode_fns, 0, 0 },
  { "wur.llr_buf_3", ICLASS_wur_llr_buf_3,
    0,
    Opcode_wur_llr_buf_3_encode_fns, 0, 0 },
  { "rur.llr_buf_4", ICLASS_rur_llr_buf_4,
    0,
    Opcode_rur_llr_buf_4_encode_fns, 0, 0 },
  { "wur.llr_buf_4", ICLASS_wur_llr_buf_4,
    0,
    Opcode_wur_llr_buf_4_encode_fns, 0, 0 },
  { "rur.llr_buf_5", ICLASS_rur_llr_buf_5,
    0,
    Opcode_rur_llr_buf_5_encode_fns, 0, 0 },
  { "wur.llr_buf_5", ICLASS_wur_llr_buf_5,
    0,
    Opcode_wur_llr_buf_5_encode_fns, 0, 0 },
  { "rur.llr_buf_6", ICLASS_rur_llr_buf_6,
    0,
    Opcode_rur_llr_buf_6_encode_fns, 0, 0 },
  { "wur.llr_buf_6", ICLASS_wur_llr_buf_6,
    0,
    Opcode_wur_llr_buf_6_encode_fns, 0, 0 },
  { "rur.llr_buf_7", ICLASS_rur_llr_buf_7,
    0,
    Opcode_rur_llr_buf_7_encode_fns, 0, 0 },
  { "wur.llr_buf_7", ICLASS_wur_llr_buf_7,
    0,
    Opcode_wur_llr_buf_7_encode_fns, 0, 0 },
  { "rur.llr_buf_8", ICLASS_rur_llr_buf_8,
    0,
    Opcode_rur_llr_buf_8_encode_fns, 0, 0 },
  { "wur.llr_buf_8", ICLASS_wur_llr_buf_8,
    0,
    Opcode_wur_llr_buf_8_encode_fns, 0, 0 },
  { "rur.llr_buf_9", ICLASS_rur_llr_buf_9,
    0,
    Opcode_rur_llr_buf_9_encode_fns, 0, 0 },
  { "wur.llr_buf_9", ICLASS_wur_llr_buf_9,
    0,
    Opcode_wur_llr_buf_9_encode_fns, 0, 0 },
  { "rur.llr_buf_10", ICLASS_rur_llr_buf_10,
    0,
    Opcode_rur_llr_buf_10_encode_fns, 0, 0 },
  { "wur.llr_buf_10", ICLASS_wur_llr_buf_10,
    0,
    Opcode_wur_llr_buf_10_encode_fns, 0, 0 },
  { "rur.llr_buf_11", ICLASS_rur_llr_buf_11,
    0,
    Opcode_rur_llr_buf_11_encode_fns, 0, 0 },
  { "wur.llr_buf_11", ICLASS_wur_llr_buf_11,
    0,
    Opcode_wur_llr_buf_11_encode_fns, 0, 0 },
  { "rur.llr_buf_12", ICLASS_rur_llr_buf_12,
    0,
    Opcode_rur_llr_buf_12_encode_fns, 0, 0 },
  { "wur.llr_buf_12", ICLASS_wur_llr_buf_12,
    0,
    Opcode_wur_llr_buf_12_encode_fns, 0, 0 },
  { "rur.llr_buf_13", ICLASS_rur_llr_buf_13,
    0,
    Opcode_rur_llr_buf_13_encode_fns, 0, 0 },
  { "wur.llr_buf_13", ICLASS_wur_llr_buf_13,
    0,
    Opcode_wur_llr_buf_13_encode_fns, 0, 0 },
  { "rur.llr_buf_14", ICLASS_rur_llr_buf_14,
    0,
    Opcode_rur_llr_buf_14_encode_fns, 0, 0 },
  { "wur.llr_buf_14", ICLASS_wur_llr_buf_14,
    0,
    Opcode_wur_llr_buf_14_encode_fns, 0, 0 },
  { "rur.llr_buf_15", ICLASS_rur_llr_buf_15,
    0,
    Opcode_rur_llr_buf_15_encode_fns, 0, 0 },
  { "wur.llr_buf_15", ICLASS_wur_llr_buf_15,
    0,
    Opcode_wur_llr_buf_15_encode_fns, 0, 0 },
  { "rur.llr_buf_16", ICLASS_rur_llr_buf_16,
    0,
    Opcode_rur_llr_buf_16_encode_fns, 0, 0 },
  { "wur.llr_buf_16", ICLASS_wur_llr_buf_16,
    0,
    Opcode_wur_llr_buf_16_encode_fns, 0, 0 },
  { "rur.llr_buf_17", ICLASS_rur_llr_buf_17,
    0,
    Opcode_rur_llr_buf_17_encode_fns, 0, 0 },
  { "wur.llr_buf_17", ICLASS_wur_llr_buf_17,
    0,
    Opcode_wur_llr_buf_17_encode_fns, 0, 0 },
  { "rur.llr_buf_18", ICLASS_rur_llr_buf_18,
    0,
    Opcode_rur_llr_buf_18_encode_fns, 0, 0 },
  { "wur.llr_buf_18", ICLASS_wur_llr_buf_18,
    0,
    Opcode_wur_llr_buf_18_encode_fns, 0, 0 },
  { "rur.llr_buf_19", ICLASS_rur_llr_buf_19,
    0,
    Opcode_rur_llr_buf_19_encode_fns, 0, 0 },
  { "wur.llr_buf_19", ICLASS_wur_llr_buf_19,
    0,
    Opcode_wur_llr_buf_19_encode_fns, 0, 0 },
  { "rur.llr_buf_20", ICLASS_rur_llr_buf_20,
    0,
    Opcode_rur_llr_buf_20_encode_fns, 0, 0 },
  { "wur.llr_buf_20", ICLASS_wur_llr_buf_20,
    0,
    Opcode_wur_llr_buf_20_encode_fns, 0, 0 },
  { "rur.llr_buf_21", ICLASS_rur_llr_buf_21,
    0,
    Opcode_rur_llr_buf_21_encode_fns, 0, 0 },
  { "wur.llr_buf_21", ICLASS_wur_llr_buf_21,
    0,
    Opcode_wur_llr_buf_21_encode_fns, 0, 0 },
  { "rur.llr_buf_22", ICLASS_rur_llr_buf_22,
    0,
    Opcode_rur_llr_buf_22_encode_fns, 0, 0 },
  { "wur.llr_buf_22", ICLASS_wur_llr_buf_22,
    0,
    Opcode_wur_llr_buf_22_encode_fns, 0, 0 },
  { "rur.llr_buf_23", ICLASS_rur_llr_buf_23,
    0,
    Opcode_rur_llr_buf_23_encode_fns, 0, 0 },
  { "wur.llr_buf_23", ICLASS_wur_llr_buf_23,
    0,
    Opcode_wur_llr_buf_23_encode_fns, 0, 0 },
  { "rur.smod_buf_0", ICLASS_rur_smod_buf_0,
    0,
    Opcode_rur_smod_buf_0_encode_fns, 0, 0 },
  { "wur.smod_buf_0", ICLASS_wur_smod_buf_0,
    0,
    Opcode_wur_smod_buf_0_encode_fns, 0, 0 },
  { "rur.smod_buf_1", ICLASS_rur_smod_buf_1,
    0,
    Opcode_rur_smod_buf_1_encode_fns, 0, 0 },
  { "wur.smod_buf_1", ICLASS_wur_smod_buf_1,
    0,
    Opcode_wur_smod_buf_1_encode_fns, 0, 0 },
  { "rur.smod_buf_2", ICLASS_rur_smod_buf_2,
    0,
    Opcode_rur_smod_buf_2_encode_fns, 0, 0 },
  { "wur.smod_buf_2", ICLASS_wur_smod_buf_2,
    0,
    Opcode_wur_smod_buf_2_encode_fns, 0, 0 },
  { "rur.smod_buf_3", ICLASS_rur_smod_buf_3,
    0,
    Opcode_rur_smod_buf_3_encode_fns, 0, 0 },
  { "wur.smod_buf_3", ICLASS_wur_smod_buf_3,
    0,
    Opcode_wur_smod_buf_3_encode_fns, 0, 0 },
  { "rur.smod_buf_4", ICLASS_rur_smod_buf_4,
    0,
    Opcode_rur_smod_buf_4_encode_fns, 0, 0 },
  { "wur.smod_buf_4", ICLASS_wur_smod_buf_4,
    0,
    Opcode_wur_smod_buf_4_encode_fns, 0, 0 },
  { "rur.smod_buf_5", ICLASS_rur_smod_buf_5,
    0,
    Opcode_rur_smod_buf_5_encode_fns, 0, 0 },
  { "wur.smod_buf_5", ICLASS_wur_smod_buf_5,
    0,
    Opcode_wur_smod_buf_5_encode_fns, 0, 0 },
  { "rur.smod_buf_6", ICLASS_rur_smod_buf_6,
    0,
    Opcode_rur_smod_buf_6_encode_fns, 0, 0 },
  { "wur.smod_buf_6", ICLASS_wur_smod_buf_6,
    0,
    Opcode_wur_smod_buf_6_encode_fns, 0, 0 },
  { "rur.smod_buf_7", ICLASS_rur_smod_buf_7,
    0,
    Opcode_rur_smod_buf_7_encode_fns, 0, 0 },
  { "wur.smod_buf_7", ICLASS_wur_smod_buf_7,
    0,
    Opcode_wur_smod_buf_7_encode_fns, 0, 0 },
  { "rur.weight_reg", ICLASS_rur_weight_reg,
    0,
    Opcode_rur_weight_reg_encode_fns, 0, 0 },
  { "wur.weight_reg", ICLASS_wur_weight_reg,
    0,
    Opcode_wur_weight_reg_encode_fns, 0, 0 },
  { "rur.scale_reg", ICLASS_rur_scale_reg,
    0,
    Opcode_rur_scale_reg_encode_fns, 0, 0 },
  { "wur.scale_reg", ICLASS_wur_scale_reg,
    0,
    Opcode_wur_scale_reg_encode_fns, 0, 0 },
  { "rur.llr_pos", ICLASS_rur_llr_pos,
    0,
    Opcode_rur_llr_pos_encode_fns, 0, 0 },
  { "wur.llr_pos", ICLASS_wur_llr_pos,
    0,
    Opcode_wur_llr_pos_encode_fns, 0, 0 },
  { "rur.smod_pos", ICLASS_rur_smod_pos,
    0,
    Opcode_rur_smod_pos_encode_fns, 0, 0 },
  { "wur.smod_pos", ICLASS_wur_smod_pos,
    0,
    Opcode_wur_smod_pos_encode_fns, 0, 0 },
  { "rur.perm_reg", ICLASS_rur_perm_reg,
    0,
    Opcode_rur_perm_reg_encode_fns, 0, 0 },
  { "wur.perm_reg", ICLASS_wur_perm_reg,
    0,
    Opcode_wur_perm_reg_encode_fns, 0, 0 },
  { "rur.smod_offset_table_0", ICLASS_rur_smod_offset_table_0,
    0,
    Opcode_rur_smod_offset_table_0_encode_fns, 0, 0 },
  { "wur.smod_offset_table_0", ICLASS_wur_smod_offset_table_0,
    0,
    Opcode_wur_smod_offset_table_0_encode_fns, 0, 0 },
  { "rur.smod_offset_table_1", ICLASS_rur_smod_offset_table_1,
    0,
    Opcode_rur_smod_offset_table_1_encode_fns, 0, 0 },
  { "wur.smod_offset_table_1", ICLASS_wur_smod_offset_table_1,
    0,
    Opcode_wur_smod_offset_table_1_encode_fns, 0, 0 },
  { "rur.smod_offset_table_2", ICLASS_rur_smod_offset_table_2,
    0,
    Opcode_rur_smod_offset_table_2_encode_fns, 0, 0 },
  { "wur.smod_offset_table_2", ICLASS_wur_smod_offset_table_2,
    0,
    Opcode_wur_smod_offset_table_2_encode_fns, 0, 0 },
  { "rur.smod_offset_table_3", ICLASS_rur_smod_offset_table_3,
    0,
    Opcode_rur_smod_offset_table_3_encode_fns, 0, 0 },
  { "wur.smod_offset_table_3", ICLASS_wur_smod_offset_table_3,
    0,
    Opcode_wur_smod_offset_table_3_encode_fns, 0, 0 },
  { "rur.phasor_n", ICLASS_rur_phasor_n,
    0,
    Opcode_rur_phasor_n_encode_fns, 0, 0 },
  { "wur.phasor_n", ICLASS_wur_phasor_n,
    0,
    Opcode_wur_phasor_n_encode_fns, 0, 0 },
  { "rur.phasor_offset", ICLASS_rur_phasor_offset,
    0,
    Opcode_rur_phasor_offset_encode_fns, 0, 0 },
  { "wur.phasor_offset", ICLASS_wur_phasor_offset,
    0,
    Opcode_wur_phasor_offset_encode_fns, 0, 0 }
};

enum xtensa_opcode_id {
  OPCODE_EXCW,
  OPCODE_RFE,
  OPCODE_RFDE,
  OPCODE_SYSCALL,
  OPCODE_SIMCALL,
  OPCODE_CALL12,
  OPCODE_CALL8,
  OPCODE_CALL4,
  OPCODE_CALLX12,
  OPCODE_CALLX8,
  OPCODE_CALLX4,
  OPCODE_ENTRY,
  OPCODE_MOVSP,
  OPCODE_ROTW,
  OPCODE_RETW,
  OPCODE_RETW_N,
  OPCODE_RFWO,
  OPCODE_RFWU,
  OPCODE_L32E,
  OPCODE_S32E,
  OPCODE_RSR_WINDOWBASE,
  OPCODE_WSR_WINDOWBASE,
  OPCODE_XSR_WINDOWBASE,
  OPCODE_RSR_WINDOWSTART,
  OPCODE_WSR_WINDOWSTART,
  OPCODE_XSR_WINDOWSTART,
  OPCODE_ADD_N,
  OPCODE_ADDI_N,
  OPCODE_BEQZ_N,
  OPCODE_BNEZ_N,
  OPCODE_ILL_N,
  OPCODE_L32I_N,
  OPCODE_MOV_N,
  OPCODE_MOVI_N,
  OPCODE_NOP_N,
  OPCODE_RET_N,
  OPCODE_S32I_N,
  OPCODE_RUR_THREADPTR,
  OPCODE_WUR_THREADPTR,
  OPCODE_ADDI,
  OPCODE_ADDMI,
  OPCODE_ADD,
  OPCODE_SUB,
  OPCODE_ADDX2,
  OPCODE_ADDX4,
  OPCODE_ADDX8,
  OPCODE_SUBX2,
  OPCODE_SUBX4,
  OPCODE_SUBX8,
  OPCODE_AND,
  OPCODE_OR,
  OPCODE_XOR,
  OPCODE_BEQI,
  OPCODE_BNEI,
  OPCODE_BGEI,
  OPCODE_BLTI,
  OPCODE_BBCI,
  OPCODE_BBSI,
  OPCODE_BGEUI,
  OPCODE_BLTUI,
  OPCODE_BEQ,
  OPCODE_BNE,
  OPCODE_BGE,
  OPCODE_BLT,
  OPCODE_BGEU,
  OPCODE_BLTU,
  OPCODE_BANY,
  OPCODE_BNONE,
  OPCODE_BALL,
  OPCODE_BNALL,
  OPCODE_BBC,
  OPCODE_BBS,
  OPCODE_BEQZ,
  OPCODE_BNEZ,
  OPCODE_BGEZ,
  OPCODE_BLTZ,
  OPCODE_CALL0,
  OPCODE_CALLX0,
  OPCODE_EXTUI,
  OPCODE_ILL,
  OPCODE_J,
  OPCODE_JX,
  OPCODE_L16UI,
  OPCODE_L16SI,
  OPCODE_L32I,
  OPCODE_L32R,
  OPCODE_L8UI,
  OPCODE_LOOP,
  OPCODE_LOOPNEZ,
  OPCODE_LOOPGTZ,
  OPCODE_MOVI,
  OPCODE_MOVEQZ,
  OPCODE_MOVNEZ,
  OPCODE_MOVLTZ,
  OPCODE_MOVGEZ,
  OPCODE_NEG,
  OPCODE_ABS,
  OPCODE_NOP,
  OPCODE_RET,
  OPCODE_S16I,
  OPCODE_S32I,
  OPCODE_S8I,
  OPCODE_SSR,
  OPCODE_SSL,
  OPCODE_SSA8L,
  OPCODE_SSA8B,
  OPCODE_SSAI,
  OPCODE_SLL,
  OPCODE_SRC,
  OPCODE_SRL,
  OPCODE_SRA,
  OPCODE_SLLI,
  OPCODE_SRAI,
  OPCODE_SRLI,
  OPCODE_MEMW,
  OPCODE_EXTW,
  OPCODE_ISYNC,
  OPCODE_RSYNC,
  OPCODE_ESYNC,
  OPCODE_DSYNC,
  OPCODE_RSIL,
  OPCODE_RSR_LEND,
  OPCODE_WSR_LEND,
  OPCODE_XSR_LEND,
  OPCODE_RSR_LCOUNT,
  OPCODE_WSR_LCOUNT,
  OPCODE_XSR_LCOUNT,
  OPCODE_RSR_LBEG,
  OPCODE_WSR_LBEG,
  OPCODE_XSR_LBEG,
  OPCODE_RSR_SAR,
  OPCODE_WSR_SAR,
  OPCODE_XSR_SAR,
  OPCODE_RSR_LITBASE,
  OPCODE_WSR_LITBASE,
  OPCODE_XSR_LITBASE,
  OPCODE_RSR_176,
  OPCODE_WSR_176,
  OPCODE_RSR_208,
  OPCODE_RSR_PS,
  OPCODE_WSR_PS,
  OPCODE_XSR_PS,
  OPCODE_RSR_EPC1,
  OPCODE_WSR_EPC1,
  OPCODE_XSR_EPC1,
  OPCODE_RSR_EXCSAVE1,
  OPCODE_WSR_EXCSAVE1,
  OPCODE_XSR_EXCSAVE1,
  OPCODE_RSR_EPC2,
  OPCODE_WSR_EPC2,
  OPCODE_XSR_EPC2,
  OPCODE_RSR_EXCSAVE2,
  OPCODE_WSR_EXCSAVE2,
  OPCODE_XSR_EXCSAVE2,
  OPCODE_RSR_EPC3,
  OPCODE_WSR_EPC3,
  OPCODE_XSR_EPC3,
  OPCODE_RSR_EXCSAVE3,
  OPCODE_WSR_EXCSAVE3,
  OPCODE_XSR_EXCSAVE3,
  OPCODE_RSR_EPC4,
  OPCODE_WSR_EPC4,
  OPCODE_XSR_EPC4,
  OPCODE_RSR_EXCSAVE4,
  OPCODE_WSR_EXCSAVE4,
  OPCODE_XSR_EXCSAVE4,
  OPCODE_RSR_EPC5,
  OPCODE_WSR_EPC5,
  OPCODE_XSR_EPC5,
  OPCODE_RSR_EXCSAVE5,
  OPCODE_WSR_EXCSAVE5,
  OPCODE_XSR_EXCSAVE5,
  OPCODE_RSR_EPC6,
  OPCODE_WSR_EPC6,
  OPCODE_XSR_EPC6,
  OPCODE_RSR_EXCSAVE6,
  OPCODE_WSR_EXCSAVE6,
  OPCODE_XSR_EXCSAVE6,
  OPCODE_RSR_EPS2,
  OPCODE_WSR_EPS2,
  OPCODE_XSR_EPS2,
  OPCODE_RSR_EPS3,
  OPCODE_WSR_EPS3,
  OPCODE_XSR_EPS3,
  OPCODE_RSR_EPS4,
  OPCODE_WSR_EPS4,
  OPCODE_XSR_EPS4,
  OPCODE_RSR_EPS5,
  OPCODE_WSR_EPS5,
  OPCODE_XSR_EPS5,
  OPCODE_RSR_EPS6,
  OPCODE_WSR_EPS6,
  OPCODE_XSR_EPS6,
  OPCODE_RSR_EXCVADDR,
  OPCODE_WSR_EXCVADDR,
  OPCODE_XSR_EXCVADDR,
  OPCODE_RSR_DEPC,
  OPCODE_WSR_DEPC,
  OPCODE_XSR_DEPC,
  OPCODE_RSR_EXCCAUSE,
  OPCODE_WSR_EXCCAUSE,
  OPCODE_XSR_EXCCAUSE,
  OPCODE_RSR_PRID,
  OPCODE_RSR_VECBASE,
  OPCODE_WSR_VECBASE,
  OPCODE_XSR_VECBASE,
  OPCODE_MUL16U,
  OPCODE_MUL16S,
  OPCODE_RFI,
  OPCODE_WAITI,
  OPCODE_RSR_INTERRUPT,
  OPCODE_WSR_INTSET,
  OPCODE_WSR_INTCLEAR,
  OPCODE_RSR_INTENABLE,
  OPCODE_WSR_INTENABLE,
  OPCODE_XSR_INTENABLE,
  OPCODE_BREAK,
  OPCODE_BREAK_N,
  OPCODE_RSR_DBREAKA0,
  OPCODE_WSR_DBREAKA0,
  OPCODE_XSR_DBREAKA0,
  OPCODE_RSR_DBREAKC0,
  OPCODE_WSR_DBREAKC0,
  OPCODE_XSR_DBREAKC0,
  OPCODE_RSR_DBREAKA1,
  OPCODE_WSR_DBREAKA1,
  OPCODE_XSR_DBREAKA1,
  OPCODE_RSR_DBREAKC1,
  OPCODE_WSR_DBREAKC1,
  OPCODE_XSR_DBREAKC1,
  OPCODE_RSR_IBREAKA0,
  OPCODE_WSR_IBREAKA0,
  OPCODE_XSR_IBREAKA0,
  OPCODE_RSR_IBREAKA1,
  OPCODE_WSR_IBREAKA1,
  OPCODE_XSR_IBREAKA1,
  OPCODE_RSR_IBREAKENABLE,
  OPCODE_WSR_IBREAKENABLE,
  OPCODE_XSR_IBREAKENABLE,
  OPCODE_RSR_DEBUGCAUSE,
  OPCODE_WSR_DEBUGCAUSE,
  OPCODE_XSR_DEBUGCAUSE,
  OPCODE_RSR_ICOUNT,
  OPCODE_WSR_ICOUNT,
  OPCODE_XSR_ICOUNT,
  OPCODE_RSR_ICOUNTLEVEL,
  OPCODE_WSR_ICOUNTLEVEL,
  OPCODE_XSR_ICOUNTLEVEL,
  OPCODE_RSR_DDR,
  OPCODE_WSR_DDR,
  OPCODE_XSR_DDR,
  OPCODE_RFDO,
  OPCODE_RFDD,
  OPCODE_WSR_MMID,
  OPCODE_ANDB,
  OPCODE_ANDBC,
  OPCODE_ORB,
  OPCODE_ORBC,
  OPCODE_XORB,
  OPCODE_ANY4,
  OPCODE_ALL4,
  OPCODE_ANY8,
  OPCODE_ALL8,
  OPCODE_BF,
  OPCODE_BT,
  OPCODE_MOVF,
  OPCODE_MOVT,
  OPCODE_RSR_BR,
  OPCODE_WSR_BR,
  OPCODE_XSR_BR,
  OPCODE_RSR_CCOUNT,
  OPCODE_WSR_CCOUNT,
  OPCODE_XSR_CCOUNT,
  OPCODE_RSR_CCOMPARE0,
  OPCODE_WSR_CCOMPARE0,
  OPCODE_XSR_CCOMPARE0,
  OPCODE_RSR_CCOMPARE1,
  OPCODE_WSR_CCOMPARE1,
  OPCODE_XSR_CCOMPARE1,
  OPCODE_IPF,
  OPCODE_IHI,
  OPCODE_IPFL,
  OPCODE_IHU,
  OPCODE_IIU,
  OPCODE_III,
  OPCODE_LICT,
  OPCODE_LICW,
  OPCODE_SICT,
  OPCODE_SICW,
  OPCODE_DHWB,
  OPCODE_DHWBI,
  OPCODE_DIWB,
  OPCODE_DIWBI,
  OPCODE_DHI,
  OPCODE_DII,
  OPCODE_DPFR,
  OPCODE_DPFW,
  OPCODE_DPFRO,
  OPCODE_DPFWO,
  OPCODE_DPFL,
  OPCODE_DHU,
  OPCODE_DIU,
  OPCODE_SDCT,
  OPCODE_LDCT,
  OPCODE_IDTLB,
  OPCODE_PDTLB,
  OPCODE_RDTLB0,
  OPCODE_RDTLB1,
  OPCODE_WDTLB,
  OPCODE_IITLB,
  OPCODE_PITLB,
  OPCODE_RITLB0,
  OPCODE_RITLB1,
  OPCODE_WITLB,
  OPCODE_RSR_CPENABLE,
  OPCODE_WSR_CPENABLE,
  OPCODE_XSR_CPENABLE,
  OPCODE_CLAMPS,
  OPCODE_MIN,
  OPCODE_MAX,
  OPCODE_MINU,
  OPCODE_MAXU,
  OPCODE_NSA,
  OPCODE_NSAU,
  OPCODE_SEXT,
  OPCODE_L32AI,
  OPCODE_S32RI,
  OPCODE_S32C1I,
  OPCODE_RSR_SCOMPARE1,
  OPCODE_WSR_SCOMPARE1,
  OPCODE_XSR_SCOMPARE1,
  OPCODE_RSR_ATOMCTL,
  OPCODE_WSR_ATOMCTL,
  OPCODE_XSR_ATOMCTL,
  OPCODE_RER,
  OPCODE_WER,
  OPCODE_RUR_FCR,
  OPCODE_WUR_FCR,
  OPCODE_RUR_FSR,
  OPCODE_WUR_FSR,
  OPCODE_ADD_S,
  OPCODE_SUB_S,
  OPCODE_MUL_S,
  OPCODE_MADD_S,
  OPCODE_MSUB_S,
  OPCODE_MOVF_S,
  OPCODE_MOVT_S,
  OPCODE_MOVEQZ_S,
  OPCODE_MOVNEZ_S,
  OPCODE_MOVLTZ_S,
  OPCODE_MOVGEZ_S,
  OPCODE_ABS_S,
  OPCODE_MOV_S,
  OPCODE_NEG_S,
  OPCODE_UN_S,
  OPCODE_OEQ_S,
  OPCODE_UEQ_S,
  OPCODE_OLT_S,
  OPCODE_ULT_S,
  OPCODE_OLE_S,
  OPCODE_ULE_S,
  OPCODE_FLOAT_S,
  OPCODE_UFLOAT_S,
  OPCODE_ROUND_S,
  OPCODE_CEIL_S,
  OPCODE_FLOOR_S,
  OPCODE_TRUNC_S,
  OPCODE_UTRUNC_S,
  OPCODE_RFR,
  OPCODE_WFR,
  OPCODE_LSI,
  OPCODE_LSIU,
  OPCODE_LSX,
  OPCODE_LSXU,
  OPCODE_SSI,
  OPCODE_SSIU,
  OPCODE_SSX,
  OPCODE_SSXU,
  OPCODE_GET_ARGMAX,
  OPCODE_GET_HSAR,
  OPCODE_GET_HSAR2SAR,
  OPCODE_GET_INTERP_EXT_N,
  OPCODE_GET_INTERP_EXT_L,
  OPCODE_GET_LLR_BUF,
  OPCODE_GET_LLR_POS,
  OPCODE_GET_MAX,
  OPCODE_GET_NCO,
  OPCODE_GET_PERM_REG,
  OPCODE_GET_PHASOR_N,
  OPCODE_GET_PHASOR_OFFSET,
  OPCODE_GET_SAR,
  OPCODE_GET_SCALE_REG,
  OPCODE_GET_SMOD_BUF,
  OPCODE_GET_SMOD_OFFSET_TABLE,
  OPCODE_GET_SMOD_POS,
  OPCODE_GET_SOV,
  OPCODE_GET_WGHT,
  OPCODE_SET_ARGMAX,
  OPCODE_SET_EXT_REGS,
  OPCODE_SET_HSAR,
  OPCODE_SET_LLR_BUF,
  OPCODE_SET_LLR_POS,
  OPCODE_SET_MAX,
  OPCODE_SET_NCO,
  OPCODE_SET_PERM_REG,
  OPCODE_SET_PHASOR_N,
  OPCODE_SET_PHASOR_OFFSET,
  OPCODE_SET_SAR,
  OPCODE_SET_SCALE_REG,
  OPCODE_SET_SMOD_BUF,
  OPCODE_SET_SMOD_OFFSET_TABLE,
  OPCODE_SET_SMOD_POS,
  OPCODE_SET_SOV,
  OPCODE_SET_WGHT,
  OPCODE_LAC2X32,
  OPCODE_LAC2X64_0,
  OPCODE_LAC2X64_1,
  OPCODE_LAC2X64_2,
  OPCODE_LAC2X64_3,
  OPCODE_LAC32_R,
  OPCODE_LAC_IH,
  OPCODE_LAC_IL,
  OPCODE_LAC_RH,
  OPCODE_LAC_RL,
  OPCODE_LCM,
  OPCODE_LCM_PINC,
  OPCODE_LCM_PINC_X,
  OPCODE_LCM_U,
  OPCODE_LCM_X,
  OPCODE_LCM_XU,
  OPCODE_LP,
  OPCODE_LP_X,
  OPCODE_LQ,
  OPCODE_LQ_X,
  OPCODE_LUT0,
  OPCODE_LUT1,
  OPCODE_LUT2,
  OPCODE_LUT3,
  OPCODE_SAC2X32,
  OPCODE_SAC2X64_0,
  OPCODE_SAC2X64_1,
  OPCODE_SAC2X64_2,
  OPCODE_SAC2X64_3,
  OPCODE_SAC32_R,
  OPCODE_SAC_IH,
  OPCODE_SAC_IL,
  OPCODE_SAC_RH,
  OPCODE_SAC_RL,
  OPCODE_SCM,
  OPCODE_SCM_PINC,
  OPCODE_SCM_PINC_X,
  OPCODE_SCM_U,
  OPCODE_SCM_X,
  OPCODE_SCM_XU,
  OPCODE_STORE_P,
  OPCODE_STORE_Q,
  OPCODE_AR2CM_DUP,
  OPCODE_AR2CM_LN,
  OPCODE_AR2CM_LN_I,
  OPCODE_AR2CM_LN_R,
  OPCODE_AR2PQ_LN,
  OPCODE_AR2SAR_DUP,
  OPCODE_CLRAC,
  OPCODE_CLRCM,
  OPCODE_CM2AR_LN,
  OPCODE_CM2AR_LN_I,
  OPCODE_CM2AR_LN_R,
  OPCODE_COMB_AR,
  OPCODE_CONJ,
  OPCODE_MOV2AC32_I,
  OPCODE_MOV2AC32_R,
  OPCODE_MOV2CM2PQ,
  OPCODE_MOVAC,
  OPCODE_MOVAC_I,
  OPCODE_MOVAC_I2R,
  OPCODE_MOVAC_R,
  OPCODE_MOVAC_R2I,
  OPCODE_MOVAR2,
  OPCODE_MOVCM,
  OPCODE_MOVCM2PQ,
  OPCODE_MOVCND_0,
  OPCODE_MOVCND_1,
  OPCODE_MOVCND_2,
  OPCODE_MOVCND_3,
  OPCODE_MOVCND_4,
  OPCODE_MOVCND_5,
  OPCODE_MOVCND_6,
  OPCODE_MOVCND_7,
  OPCODE_MOVCND8_0,
  OPCODE_MOVCND8_1,
  OPCODE_MOVCND8_2,
  OPCODE_MOVCND8_3,
  OPCODE_MOVCND8_4,
  OPCODE_MOVCND8_5,
  OPCODE_MOVCND8_6,
  OPCODE_MOVCND8_7,
  OPCODE_MOV_I,
  OPCODE_MOVPQ2PQ,
  OPCODE_MOV_R,
  OPCODE_NEGCM,
  OPCODE_POP16LLR_1,
  OPCODE_PQ2CM,
  OPCODE_SWAPAC_R,
  OPCODE_SWAPAC_RI,
  OPCODE_SWAPB,
  OPCODE_ADD2AC,
  OPCODE_ADDAC,
  OPCODE_CDOT,
  OPCODE_CDOTAC,
  OPCODE_CDOTACS,
  OPCODE_CMAC,
  OPCODE_CMACS,
  OPCODE_CMPY,
  OPCODE_CMPY2CM,
  OPCODE_CMPY2PQ,
  OPCODE_CMPYS,
  OPCODE_CMPYXP2PQ,
  OPCODE_COMB32,
  OPCODE_DOT,
  OPCODE_DOTAC,
  OPCODE_DOTACS,
  OPCODE_LIN_INT,
  OPCODE_LLRPRE1,
  OPCODE_LLRPRE2,
  OPCODE_MAC,
  OPCODE_MAC8,
  OPCODE_MACD8,
  OPCODE_MACPQXP_0,
  OPCODE_MACPQXP_1,
  OPCODE_MACPQXP_2,
  OPCODE_MACPQXP_3,
  OPCODE_MACS,
  OPCODE_MACXP2_0,
  OPCODE_MACXP2_1,
  OPCODE_MACXP_0,
  OPCODE_MACXP_1,
  OPCODE_MACXP_2,
  OPCODE_MACXP_3,
  OPCODE_MOV2AC,
  OPCODE_MPY,
  OPCODE_MPY2CM,
  OPCODE_MPY2PQ,
  OPCODE_MPY8,
  OPCODE_MPYADD8_2CM,
  OPCODE_MPYD8,
  OPCODE_MPYPQXP_0,
  OPCODE_MPYPQXP_1,
  OPCODE_MPYPQXP_2,
  OPCODE_MPYPQXP_3,
  OPCODE_MPYS,
  OPCODE_MPYXP2PQ,
  OPCODE_MPYXP2_0,
  OPCODE_MPYXP2_1,
  OPCODE_MPYXP_0,
  OPCODE_MPYXP_1,
  OPCODE_MPYXP_2,
  OPCODE_MPYXP_3,
  OPCODE_NORMACD,
  OPCODE_NORMACPQ_I,
  OPCODE_NORMACPQ_R,
  OPCODE_NORMD,
  OPCODE_NORMPYPQ_I,
  OPCODE_NORMPYPQ_R,
  OPCODE_RCMAC,
  OPCODE_RCMPY,
  OPCODE_RCMPY2CM,
  OPCODE_RFIR,
  OPCODE_RFIRA,
  OPCODE_RFIRD,
  OPCODE_RFIRDA,
  OPCODE_RMAC,
  OPCODE_RMPY,
  OPCODE_RMPY2CM,
  OPCODE_SMOD_ALIGN,
  OPCODE_SMOD_SCR,
  OPCODE_SUB2AC,
  OPCODE_WGHT32,
  OPCODE_CLRTIEP,
  OPCODE_EXT_2FIFO_0,
  OPCODE_EXT_2FIFO_1,
  OPCODE_EXT_2FIFO_2,
  OPCODE_EXT_2FIFO_3,
  OPCODE_EXT_R2FIFO_0,
  OPCODE_EXT_R2FIFO_1,
  OPCODE_EXT_R2FIFO_2,
  OPCODE_EXT_R2FIFO_3,
  OPCODE_LUT,
  OPCODE_LUT_AR,
  OPCODE_LUT_IEXT,
  OPCODE_LUT_PHASOR,
  OPCODE_LUT_REXT,
  OPCODE_LUT_WRITE,
  OPCODE_MOVEQ128_0,
  OPCODE_MOVEQ128_1,
  OPCODE_MOVEQ128_2,
  OPCODE_MOVEQ128_3,
  OPCODE_MOVEQ128_4,
  OPCODE_MOVEQ128_5,
  OPCODE_MOVEQ32_0,
  OPCODE_MOVEQ32_1,
  OPCODE_MOVEQ32_2,
  OPCODE_MOVEQ32_3,
  OPCODE_NCO_UPDATE,
  OPCODE_POP128_0,
  OPCODE_POP128_1,
  OPCODE_POP128_2,
  OPCODE_POP128_3,
  OPCODE_POP128_4,
  OPCODE_POP128_5,
  OPCODE_POP128_2CMPQ_0,
  OPCODE_POP128_2CMPQ_1,
  OPCODE_POP128_2CMPQ_2,
  OPCODE_POP128_2CMPQ_3,
  OPCODE_POP128_2M_0,
  OPCODE_POP128_2M_1,
  OPCODE_POP128_2M_2,
  OPCODE_POP128_2M_3,
  OPCODE_POP128_2PQ_0,
  OPCODE_POP128_2PQ_1,
  OPCODE_POP128_2PQ_2,
  OPCODE_POP128_2PQ_3,
  OPCODE_POP128_2PQ_4,
  OPCODE_POP128_2PQ_5,
  OPCODE_POP2X128_2PQ_01,
  OPCODE_POP2X128_2PQ_03,
  OPCODE_POP2X128_2PQ_21,
  OPCODE_POP2X128_2PQ_23,
  OPCODE_POP32_0,
  OPCODE_POP32_1,
  OPCODE_POP32_2,
  OPCODE_POP32_3,
  OPCODE_PUSH128,
  OPCODE_PUSH128_M,
  OPCODE_PUSH128_PQ,
  OPCODE_PUSH2X128_PQ,
  OPCODE_PUSH32,
  OPCODE_QREADY,
  OPCODE_RDTIEP,
  OPCODE_SETTIEP,
  OPCODE_SMOD_LUT,
  OPCODE_WRTBSIGQ,
  OPCODE_WRTBSIGQM,
  OPCODE_WRTIEP,
  OPCODE_WRTSIGQ,
  OPCODE_ABS8,
  OPCODE_ADD16,
  OPCODE_ADD32,
  OPCODE_ADDAC_I2R,
  OPCODE_ADDAC_R2I,
  OPCODE_ADDAR2,
  OPCODE_ADDCM,
  OPCODE_ADDWRP,
  OPCODE_AND128,
  OPCODE_ARGMAX8,
  OPCODE_ASL,
  OPCODE_ASL32,
  OPCODE_ASLACM,
  OPCODE_ASLM,
  OPCODE_ASLM32,
  OPCODE_ASR,
  OPCODE_ASR32,
  OPCODE_ASRAC,
  OPCODE_ASRM,
  OPCODE_BITFEXT,
  OPCODE_BITFINS,
  OPCODE_CLB_C,
  OPCODE_CLB_R,
  OPCODE_CMP8,
  OPCODE_CMP_I,
  OPCODE_CMP_R,
  OPCODE_EXT,
  OPCODE_EXT_R,
  OPCODE_EXT32_I,
  OPCODE_EXT32_R,
  OPCODE_EXTUI4,
  OPCODE_LSLM,
  OPCODE_LSRM,
  OPCODE_MAX8,
  OPCODE_MEAN,
  OPCODE_MEAN32,
  OPCODE_MIN8,
  OPCODE_MINCLB_C,
  OPCODE_MINCLB_R,
  OPCODE_NOT128,
  OPCODE_OR128,
  OPCODE_PERM,
  OPCODE_REDAC,
  OPCODE_REDAC2,
  OPCODE_REDAC4,
  OPCODE_REDACS,
  OPCODE_SMINCLB_C,
  OPCODE_SMINCLB_R,
  OPCODE_STSWAPBM,
  OPCODE_STSWAPBMU,
  OPCODE_SUB32,
  OPCODE_SUBAC_I2R,
  OPCODE_SUBAC_R2I,
  OPCODE_SUBARX,
  OPCODE_SUBCM,
  OPCODE_SUBMEAN,
  OPCODE_SUBWRP,
  OPCODE_TRANS,
  OPCODE_XOR128,
  OPCODE_RUR_EXPSTATE,
  OPCODE_WUR_EXPSTATE,
  OPCODE_RUR_SOV,
  OPCODE_WUR_SOV,
  OPCODE_RUR_SAT_MODE,
  OPCODE_WUR_SAT_MODE,
  OPCODE_RUR_SAR0,
  OPCODE_WUR_SAR0,
  OPCODE_RUR_SAR1,
  OPCODE_WUR_SAR1,
  OPCODE_RUR_SAR2,
  OPCODE_WUR_SAR2,
  OPCODE_RUR_SAR3,
  OPCODE_WUR_SAR3,
  OPCODE_RUR_HSAR0,
  OPCODE_WUR_HSAR0,
  OPCODE_RUR_HSAR1,
  OPCODE_WUR_HSAR1,
  OPCODE_RUR_HSAR2,
  OPCODE_WUR_HSAR2,
  OPCODE_RUR_HSAR3,
  OPCODE_WUR_HSAR3,
  OPCODE_RUR_MAX_REG_0,
  OPCODE_WUR_MAX_REG_0,
  OPCODE_RUR_MAX_REG_1,
  OPCODE_WUR_MAX_REG_1,
  OPCODE_RUR_MAX_REG_2,
  OPCODE_WUR_MAX_REG_2,
  OPCODE_RUR_MAX_REG_3,
  OPCODE_WUR_MAX_REG_3,
  OPCODE_RUR_ARG_MAX_REG_0,
  OPCODE_WUR_ARG_MAX_REG_0,
  OPCODE_RUR_ARG_MAX_REG_1,
  OPCODE_WUR_ARG_MAX_REG_1,
  OPCODE_RUR_ARG_MAX_REG_2,
  OPCODE_WUR_ARG_MAX_REG_2,
  OPCODE_RUR_ARG_MAX_REG_3,
  OPCODE_WUR_ARG_MAX_REG_3,
  OPCODE_RUR_NCO_COUNTER_0,
  OPCODE_WUR_NCO_COUNTER_0,
  OPCODE_RUR_NCO_COUNTER_1,
  OPCODE_WUR_NCO_COUNTER_1,
  OPCODE_RUR_NCO_COUNTER_2,
  OPCODE_WUR_NCO_COUNTER_2,
  OPCODE_RUR_NCO_COUNTER_3,
  OPCODE_WUR_NCO_COUNTER_3,
  OPCODE_RUR_INTERP_EXT_N,
  OPCODE_WUR_INTERP_EXT_N,
  OPCODE_RUR_INTERP_EXT_L,
  OPCODE_WUR_INTERP_EXT_L,
  OPCODE_RUR_LLR_BUF_0,
  OPCODE_WUR_LLR_BUF_0,
  OPCODE_RUR_LLR_BUF_1,
  OPCODE_WUR_LLR_BUF_1,
  OPCODE_RUR_LLR_BUF_2,
  OPCODE_WUR_LLR_BUF_2,
  OPCODE_RUR_LLR_BUF_3,
  OPCODE_WUR_LLR_BUF_3,
  OPCODE_RUR_LLR_BUF_4,
  OPCODE_WUR_LLR_BUF_4,
  OPCODE_RUR_LLR_BUF_5,
  OPCODE_WUR_LLR_BUF_5,
  OPCODE_RUR_LLR_BUF_6,
  OPCODE_WUR_LLR_BUF_6,
  OPCODE_RUR_LLR_BUF_7,
  OPCODE_WUR_LLR_BUF_7,
  OPCODE_RUR_LLR_BUF_8,
  OPCODE_WUR_LLR_BUF_8,
  OPCODE_RUR_LLR_BUF_9,
  OPCODE_WUR_LLR_BUF_9,
  OPCODE_RUR_LLR_BUF_10,
  OPCODE_WUR_LLR_BUF_10,
  OPCODE_RUR_LLR_BUF_11,
  OPCODE_WUR_LLR_BUF_11,
  OPCODE_RUR_LLR_BUF_12,
  OPCODE_WUR_LLR_BUF_12,
  OPCODE_RUR_LLR_BUF_13,
  OPCODE_WUR_LLR_BUF_13,
  OPCODE_RUR_LLR_BUF_14,
  OPCODE_WUR_LLR_BUF_14,
  OPCODE_RUR_LLR_BUF_15,
  OPCODE_WUR_LLR_BUF_15,
  OPCODE_RUR_LLR_BUF_16,
  OPCODE_WUR_LLR_BUF_16,
  OPCODE_RUR_LLR_BUF_17,
  OPCODE_WUR_LLR_BUF_17,
  OPCODE_RUR_LLR_BUF_18,
  OPCODE_WUR_LLR_BUF_18,
  OPCODE_RUR_LLR_BUF_19,
  OPCODE_WUR_LLR_BUF_19,
  OPCODE_RUR_LLR_BUF_20,
  OPCODE_WUR_LLR_BUF_20,
  OPCODE_RUR_LLR_BUF_21,
  OPCODE_WUR_LLR_BUF_21,
  OPCODE_RUR_LLR_BUF_22,
  OPCODE_WUR_LLR_BUF_22,
  OPCODE_RUR_LLR_BUF_23,
  OPCODE_WUR_LLR_BUF_23,
  OPCODE_RUR_SMOD_BUF_0,
  OPCODE_WUR_SMOD_BUF_0,
  OPCODE_RUR_SMOD_BUF_1,
  OPCODE_WUR_SMOD_BUF_1,
  OPCODE_RUR_SMOD_BUF_2,
  OPCODE_WUR_SMOD_BUF_2,
  OPCODE_RUR_SMOD_BUF_3,
  OPCODE_WUR_SMOD_BUF_3,
  OPCODE_RUR_SMOD_BUF_4,
  OPCODE_WUR_SMOD_BUF_4,
  OPCODE_RUR_SMOD_BUF_5,
  OPCODE_WUR_SMOD_BUF_5,
  OPCODE_RUR_SMOD_BUF_6,
  OPCODE_WUR_SMOD_BUF_6,
  OPCODE_RUR_SMOD_BUF_7,
  OPCODE_WUR_SMOD_BUF_7,
  OPCODE_RUR_WEIGHT_REG,
  OPCODE_WUR_WEIGHT_REG,
  OPCODE_RUR_SCALE_REG,
  OPCODE_WUR_SCALE_REG,
  OPCODE_RUR_LLR_POS,
  OPCODE_WUR_LLR_POS,
  OPCODE_RUR_SMOD_POS,
  OPCODE_WUR_SMOD_POS,
  OPCODE_RUR_PERM_REG,
  OPCODE_WUR_PERM_REG,
  OPCODE_RUR_SMOD_OFFSET_TABLE_0,
  OPCODE_WUR_SMOD_OFFSET_TABLE_0,
  OPCODE_RUR_SMOD_OFFSET_TABLE_1,
  OPCODE_WUR_SMOD_OFFSET_TABLE_1,
  OPCODE_RUR_SMOD_OFFSET_TABLE_2,
  OPCODE_WUR_SMOD_OFFSET_TABLE_2,
  OPCODE_RUR_SMOD_OFFSET_TABLE_3,
  OPCODE_WUR_SMOD_OFFSET_TABLE_3,
  OPCODE_RUR_PHASOR_N,
  OPCODE_WUR_PHASOR_N,
  OPCODE_RUR_PHASOR_OFFSET,
  OPCODE_WUR_PHASOR_OFFSET
};


/* Slot-specific opcode decode functions.  */

static int
Slot_inst_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2021_Slot_inst_get (insn))
    {
    case 3:
      if (Field_sa4_Slot_inst_get (insn) == 0 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LCM_PINC_X;
      if (Field_sa4_Slot_inst_get (insn) == 1 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_SCM_PINC_X;
      break;
    case 4:
      if (Field_sa4_Slot_inst_get (insn) == 0 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LCM_X;
      if (Field_sa4_Slot_inst_get (insn) == 1 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_SCM_X;
      break;
    case 5:
      if (Field_sa4_Slot_inst_get (insn) == 0 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LCM_XU;
      if (Field_sa4_Slot_inst_get (insn) == 1 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_SCM_XU;
      break;
    case 6:
      if (Field_sa4_Slot_inst_get (insn) == 0 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LP_X;
      break;
    case 7:
      if (Field_sa4_Slot_inst_get (insn) == 0 &&
	  Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LQ_X;
      break;
    }
  if (Field_dsp340050b49a6c_fld2035_Slot_inst_get (insn) == 2 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2047_Slot_inst_get (insn) == 0)
    return OPCODE_LQ;
  switch (Field_dsp340050b49a6c_fld2037_Slot_inst_get (insn))
    {
    case 0:
      if (Field_sae4_Slot_inst_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LCM_PINC;
      break;
    case 1:
      if (Field_sae4_Slot_inst_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LCM_U;
      break;
    case 2:
      if (Field_sae4_Slot_inst_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LP;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2048_Slot_inst_get (insn))
    {
    case 0:
      if (Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LAC2X32;
      break;
    case 1:
      if (Field_sae4_Slot_inst_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
	  Field_op0_Slot_inst_get (insn) == 0)
	return OPCODE_LAC32_R;
      break;
    }
  if (Field_dsp340050b49a6c_fld2082inst_Slot_inst_get (insn) == 0 &&
      Field_r2_Slot_inst_get (insn) == 1 &&
      Field_bbi4_Slot_inst_get (insn) == 0 &&
      Field_op0_Slot_inst_get (insn) == 3)
    return OPCODE_LAC_IH;
  if (Field_dsp340050b49a6c_fld2083inst_Slot_inst_get (insn) == 1 &&
      Field_r2_Slot_inst_get (insn) == 1 &&
      Field_bbi4_Slot_inst_get (insn) == 0 &&
      Field_op0_Slot_inst_get (insn) == 3)
    return OPCODE_LAC_IL;
  if (Field_dsp340050b49a6c_fld2084inst_Slot_inst_get (insn) == 2 &&
      Field_r2_Slot_inst_get (insn) == 1 &&
      Field_bbi4_Slot_inst_get (insn) == 0 &&
      Field_op0_Slot_inst_get (insn) == 3)
    return OPCODE_LAC_RH;
  switch (Field_dsp340050b49a6c_fld2085inst_Slot_inst_get (insn))
    {
    case 3:
      if (Field_r2_Slot_inst_get (insn) == 1 &&
	  Field_bbi4_Slot_inst_get (insn) == 0 &&
	  Field_op0_Slot_inst_get (insn) == 3)
	return OPCODE_LAC2X64_1;
      break;
    case 19:
      if (Field_r2_Slot_inst_get (insn) == 1 &&
	  Field_bbi4_Slot_inst_get (insn) == 0 &&
	  Field_op0_Slot_inst_get (insn) == 3)
	return OPCODE_LAC2X64_2;
      break;
    }
  if (Field_dsp340050b49a6c_fld2086inst_Slot_inst_get (insn) == 2 &&
      Field_r2_Slot_inst_get (insn) == 1 &&
      Field_bbi4_Slot_inst_get (insn) == 0 &&
      Field_op0_Slot_inst_get (insn) == 3 &&
      Field_dsp340050b49a6c_fld3634_Slot_inst_get (insn) == 0)
    return OPCODE_LAC_RL;
  if (Field_dsp340050b49a6c_fld2088inst_Slot_inst_get (insn) == 3 &&
      Field_r2_Slot_inst_get (insn) == 1 &&
      Field_bbi4_Slot_inst_get (insn) == 0 &&
      Field_op0_Slot_inst_get (insn) == 3 &&
      Field_dsp340050b49a6c_fld3633inst_Slot_inst_get (insn) == 0)
    return OPCODE_LAC2X64_3;
  if (Field_dsp340050b49a6c_fld2089inst_Slot_inst_get (insn) == 1 &&
      Field_r2_Slot_inst_get (insn) == 1 &&
      Field_bbi4_Slot_inst_get (insn) == 0 &&
      Field_op0_Slot_inst_get (insn) == 3 &&
      Field_dsp340050b49a6c_fld3631inst_Slot_inst_get (insn) == 0)
    return OPCODE_LAC2X64_0;
  if (Field_dsp340050b49a6c_fld2090inst_Slot_inst_get (insn) == 1 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld2091inst_Slot_inst_get (insn) == 10 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld2092inst_Slot_inst_get (insn) == 88 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CLRTIEP;
  if (Field_dsp340050b49a6c_fld2094inst_Slot_inst_get (insn) == 89 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld2095inst_Slot_inst_get (insn) == 90 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SETTIEP;
  if (Field_dsp340050b49a6c_fld2096inst_Slot_inst_get (insn) == 182 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2098inst_Slot_inst_get (insn) == 183 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2099inst_Slot_inst_get (insn) == 92 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_WRTBSIGQM;
  if (Field_dsp340050b49a6c_fld2100inst_Slot_inst_get (insn) == 186 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2101inst_Slot_inst_get (insn) == 187 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld2102inst_Slot_inst_get (insn) == 188 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2103inst_Slot_inst_get (insn) == 189 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld2104inst_Slot_inst_get (insn) == 190 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2105inst_Slot_inst_get (insn) == 191 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld2106inst_Slot_inst_get (insn) == 18 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_PUSH128_PQ;
  if (Field_dsp340050b49a6c_fld2107inst_Slot_inst_get (insn) == 152 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2PQ_0;
  if (Field_dsp340050b49a6c_fld2108inst_Slot_inst_get (insn) == 153 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2PQ_2;
  if (Field_dsp340050b49a6c_fld2109inst_Slot_inst_get (insn) == 154 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2PQ_3;
  if (Field_dsp340050b49a6c_fld2110inst_Slot_inst_get (insn) == 155 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2PQ_5;
  if (Field_dsp340050b49a6c_fld2111inst_Slot_inst_get (insn) == 156 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2PQ_4;
  if (Field_dsp340050b49a6c_fld2112inst_Slot_inst_get (insn) == 301 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_LLR_POS;
  if (Field_dsp340050b49a6c_fld2113inst_Slot_inst_get (insn) == 317 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_PHASOR_OFFSET;
  if (Field_dsp340050b49a6c_fld2114inst_Slot_inst_get (insn) == 302 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_PHASOR_N;
  if (Field_dsp340050b49a6c_fld2115inst_Slot_inst_get (insn) == 303 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld2116inst_Slot_inst_get (insn) == 318 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld2117inst_Slot_inst_get (insn) == 319 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_SOV;
  if (Field_dsp340050b49a6c_fld2118inst_Slot_inst_get (insn) == 208 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2PQ_1;
  if (Field_dsp340050b49a6c_fld2119inst_Slot_inst_get (insn) == 417 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_WGHT;
  if (Field_dsp340050b49a6c_fld2120inst_Slot_inst_get (insn) == 6913 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ128_0;
  if (Field_dsp340050b49a6c_fld2122inst_Slot_inst_get (insn) == 6929 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ128_1;
  if (Field_dsp340050b49a6c_fld2123inst_Slot_inst_get (insn) == 6945 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ128_2;
  if (Field_dsp340050b49a6c_fld2124inst_Slot_inst_get (insn) == 6961 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ128_5;
  if (Field_dsp340050b49a6c_fld2125inst_Slot_inst_get (insn) == 6977 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ128_3;
  if (Field_dsp340050b49a6c_fld2126inst_Slot_inst_get (insn) == 6993 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ32_0;
  if (Field_dsp340050b49a6c_fld2127inst_Slot_inst_get (insn) == 7009 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ32_1;
  if (Field_dsp340050b49a6c_fld2128inst_Slot_inst_get (insn) == 7025 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ32_2;
  if (Field_dsp340050b49a6c_fld2129inst_Slot_inst_get (insn) == 1761 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2035_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ128_4;
  if (Field_dsp340050b49a6c_fld2131inst_Slot_inst_get (insn) == 1777 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2035_Slot_inst_get (insn) == 0)
    return OPCODE_MOVEQ32_3;
  if (Field_dsp340050b49a6c_fld2132inst_Slot_inst_get (insn) == 105 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3659inst_Slot_inst_get (insn) == 0)
    return OPCODE_WRTBSIGQ;
  if (Field_dsp340050b49a6c_fld2133inst_Slot_inst_get (insn) == 53 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3660inst_Slot_inst_get (insn) == 0)
    return OPCODE_WRTSIGQ;
  if (Field_dsp340050b49a6c_fld2134inst_Slot_inst_get (insn) == 53 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2050_Slot_inst_get (insn) == 0)
    return OPCODE_CLRAC;
  if (Field_dsp340050b49a6c_fld2136inst_Slot_inst_get (insn) == 55 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2050_Slot_inst_get (insn) == 0)
    return OPCODE_POP16LLR_1;
  if (Field_dsp340050b49a6c_fld2137inst_Slot_inst_get (insn) == 34 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_WRTIEP;
  if (Field_dsp340050b49a6c_fld2138inst_Slot_inst_get (insn) == 560 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2139inst_Slot_inst_get (insn) == 561 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2140inst_Slot_inst_get (insn) == 562 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2141inst_Slot_inst_get (insn) == 563 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_03;
  if (Field_dsp340050b49a6c_fld2142inst_Slot_inst_get (insn) == 564 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2143inst_Slot_inst_get (insn) == 565 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_21;
  if (Field_dsp340050b49a6c_fld2144inst_Slot_inst_get (insn) == 283 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2047_Slot_inst_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_23;
  if (Field_dsp340050b49a6c_fld2145inst_Slot_inst_get (insn) == 71 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2037_Slot_inst_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_01;
  if (Field_dsp340050b49a6c_fld2146inst_Slot_inst_get (insn) == 21 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3649inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2147inst_Slot_inst_get (insn) == 13 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3650inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2149inst_Slot_inst_get (insn) == 5 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3627inst_Slot_inst_get (insn) == 0)
    return OPCODE_GET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld2151inst_Slot_inst_get (insn) == 5 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3657inst_Slot_inst_get (insn) == 0)
    return OPCODE_PUSH2X128_PQ;
  if (Field_dsp340050b49a6c_fld2153inst_Slot_inst_get (insn) == 1 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3635inst_Slot_inst_get (insn) == 0)
    return OPCODE_AR2CM_LN;
  if (Field_dsp340050b49a6c_fld2154inst_Slot_inst_get (insn) == 3 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3636inst_Slot_inst_get (insn) == 0)
    return OPCODE_AR2CM_LN_I;
  if (Field_dsp340050b49a6c_fld2155inst_Slot_inst_get (insn) == 3 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3637inst_Slot_inst_get (insn) == 0)
    return OPCODE_AR2CM_LN_R;
  if (Field_dsp340050b49a6c_fld2156inst_Slot_inst_get (insn) == 3 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_LCM;
  if (Field_dsp340050b49a6c_fld2157inst_Slot_inst_get (insn) == 19 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2158inst_Slot_inst_get (insn) == 27 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2159inst_Slot_inst_get (insn) == 51 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld2160inst_Slot_inst_get (insn) == 59 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_EXT_REGS;
  if (Field_dsp340050b49a6c_fld2161inst_Slot_inst_get (insn) == 83 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld2162inst_Slot_inst_get (insn) == 171 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_ASRAC;
  if (Field_dsp340050b49a6c_fld2163inst_Slot_inst_get (insn) == 187 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CLB_R;
  if (Field_dsp340050b49a6c_fld2164inst_Slot_inst_get (insn) == 227 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CLB_C;
  if (Field_dsp340050b49a6c_fld2165inst_Slot_inst_get (insn) == 235 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MINCLB_C;
  if (Field_dsp340050b49a6c_fld2166inst_Slot_inst_get (insn) == 243 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MINCLB_R;
  if (Field_dsp340050b49a6c_fld2167inst_Slot_inst_get (insn) == 251 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SMINCLB_C;
  if (Field_dsp340050b49a6c_fld2168inst_Slot_inst_get (insn) == 147 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2169inst_Slot_inst_get (insn) == 310 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld2171inst_Slot_inst_get (insn) == 599 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVAC;
  if (Field_dsp340050b49a6c_fld2172inst_Slot_inst_get (insn) == 631 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVAC_I2R;
  if (Field_dsp340050b49a6c_fld2173inst_Slot_inst_get (insn) == 358 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld2174inst_Slot_inst_get (insn) == 711 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVAC_R2I;
  if (Field_dsp340050b49a6c_fld2175inst_Slot_inst_get (insn) == 5944 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_ADDAC_I2R;
  if (Field_dsp340050b49a6c_fld2177inst_Slot_inst_get (insn) == 5945 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_ADDAC_R2I;
  if (Field_dsp340050b49a6c_fld2178inst_Slot_inst_get (insn) == 5946 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_REDAC;
  if (Field_dsp340050b49a6c_fld2179inst_Slot_inst_get (insn) == 5947 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_REDAC4;
  if (Field_dsp340050b49a6c_fld2180inst_Slot_inst_get (insn) == 5948 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_REDAC2;
  if (Field_dsp340050b49a6c_fld2181inst_Slot_inst_get (insn) == 5949 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_REDACS;
  if (Field_dsp340050b49a6c_fld2182inst_Slot_inst_get (insn) == 5950 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SUBAC_I2R;
  if (Field_dsp340050b49a6c_fld2183inst_Slot_inst_get (insn) == 5951 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SUBAC_R2I;
  if (Field_dsp340050b49a6c_fld2184inst_Slot_inst_get (insn) == 374 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3642inst_Slot_inst_get (insn) == 0)
    return OPCODE_SWAPAC_RI;
  if (Field_dsp340050b49a6c_fld2185inst_Slot_inst_get (insn) == 3000 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_L;
  if (Field_dsp340050b49a6c_fld2186inst_Slot_inst_get (insn) == 3001 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_N;
  if (Field_dsp340050b49a6c_fld2187inst_Slot_inst_get (insn) == 3002 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_LLR_POS;
  if (Field_dsp340050b49a6c_fld2188inst_Slot_inst_get (insn) == 3003 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_PHASOR_OFFSET;
  if (Field_dsp340050b49a6c_fld2189inst_Slot_inst_get (insn) == 3004 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_PHASOR_N;
  if (Field_dsp340050b49a6c_fld2190inst_Slot_inst_get (insn) == 3005 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld2191inst_Slot_inst_get (insn) == 3006 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld2192inst_Slot_inst_get (insn) == 3007 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_SOV;
  if (Field_dsp340050b49a6c_fld2193inst_Slot_inst_get (insn) == 211 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3662inst_Slot_inst_get (insn) == 0)
    return OPCODE_SMINCLB_R;
  if (Field_dsp340050b49a6c_fld2194inst_Slot_inst_get (insn) == 3504 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_GET_WGHT;
  if (Field_dsp340050b49a6c_fld2195inst_Slot_inst_get (insn) == 3505 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2196inst_Slot_inst_get (insn) == 3506 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2197inst_Slot_inst_get (insn) == 3507 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2198inst_Slot_inst_get (insn) == 3508 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2199inst_Slot_inst_get (insn) == 3509 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_MAX;
  if (Field_dsp340050b49a6c_fld2200inst_Slot_inst_get (insn) == 3510 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_NCO;
  if (Field_dsp340050b49a6c_fld2201inst_Slot_inst_get (insn) == 3511 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2202inst_Slot_inst_get (insn) == 878 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2254_Slot_inst_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld2203inst_Slot_inst_get (insn) == 879 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2254_Slot_inst_get (insn) == 0)
    return OPCODE_SET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld2204inst_Slot_inst_get (insn) == 123 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3656inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2205inst_Slot_inst_get (insn) == 3 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_MOVCM2PQ;
  if (Field_dsp340050b49a6c_fld2206inst_Slot_inst_get (insn) == 7 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_QREADY;
  if (Field_dsp340050b49a6c_fld2207inst_Slot_inst_get (insn) == 19 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT32_I;
  if (Field_dsp340050b49a6c_fld2208inst_Slot_inst_get (insn) == 23 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_2FIFO_2;
  if (Field_dsp340050b49a6c_fld2209inst_Slot_inst_get (insn) == 27 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_2FIFO_3;
  if (Field_dsp340050b49a6c_fld2210inst_Slot_inst_get (insn) == 31 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_0;
  if (Field_dsp340050b49a6c_fld2211inst_Slot_inst_get (insn) == 35 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT32_R;
  if (Field_dsp340050b49a6c_fld2212inst_Slot_inst_get (insn) == 39 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_1;
  if (Field_dsp340050b49a6c_fld2213inst_Slot_inst_get (insn) == 23 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3647inst_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_2;
  if (Field_dsp340050b49a6c_fld2214inst_Slot_inst_get (insn) == 27 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3648inst_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_3;
  if (Field_dsp340050b49a6c_fld2215inst_Slot_inst_get (insn) == 31 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3630inst_Slot_inst_get (insn) == 0)
    return OPCODE_SET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld2216inst_Slot_inst_get (insn) == 7 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3644inst_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_2FIFO_0;
  if (Field_dsp340050b49a6c_fld2217inst_Slot_inst_get (insn) == 7 &&
      Field_sae4_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 3 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3645inst_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_2FIFO_1;
  if (Field_dsp340050b49a6c_fld2218inst_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2021_Slot_inst_get (insn) == 6 &&
      Field_sa4_Slot_inst_get (insn) == 1 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT;
  if (Field_dsp340050b49a6c_fld2219inst_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2021_Slot_inst_get (insn) == 6 &&
      Field_sa4_Slot_inst_get (insn) == 1 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_EXT_R;
  if (Field_dsp340050b49a6c_fld2220inst_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2021_Slot_inst_get (insn) == 7 &&
      Field_sa4_Slot_inst_get (insn) == 1 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_PUSH128_M;
  if (Field_dsp340050b49a6c_fld2221inst_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2021_Slot_inst_get (insn) == 7 &&
      Field_sa4_Slot_inst_get (insn) == 1 &&
      Field_sae4_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 4 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC2X64_0;
  if (Field_dsp340050b49a6c_fld2222inst_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC2X32;
  if (Field_dsp340050b49a6c_fld2223inst_Slot_inst_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC32_R;
  if (Field_dsp340050b49a6c_fld2224inst_Slot_inst_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SCM_PINC;
  if (Field_dsp340050b49a6c_fld2225inst_Slot_inst_get (insn) == 5 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_LUT0;
  if (Field_dsp340050b49a6c_fld2226inst_Slot_inst_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_LUT1;
  if (Field_dsp340050b49a6c_fld2227inst_Slot_inst_get (insn) == 4 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SCM_U;
  if (Field_dsp340050b49a6c_fld2228inst_Slot_inst_get (insn) == 9 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_LUT2;
  if (Field_dsp340050b49a6c_fld2229inst_Slot_inst_get (insn) == 11 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SCM;
  if (Field_dsp340050b49a6c_fld2230inst_Slot_inst_get (insn) == 12 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_LUT3;
  if (Field_dsp340050b49a6c_fld2231inst_Slot_inst_get (insn) == 14 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_STORE_P;
  if (Field_dsp340050b49a6c_fld2232inst_Slot_inst_get (insn) == 14 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC_IH;
  if (Field_dsp340050b49a6c_fld2234inst_Slot_inst_get (insn) == 15 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC_IL;
  if (Field_dsp340050b49a6c_fld2235inst_Slot_inst_get (insn) == 16 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_COMB_AR;
  if (Field_dsp340050b49a6c_fld2236inst_Slot_inst_get (insn) == 18 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_STSWAPBM;
  if (Field_dsp340050b49a6c_fld2237inst_Slot_inst_get (insn) == 18 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC_RH;
  if (Field_dsp340050b49a6c_fld2238inst_Slot_inst_get (insn) == 35 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC2X64_1;
  if (Field_dsp340050b49a6c_fld2239inst_Slot_inst_get (insn) == 39 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC2X64_2;
  if (Field_dsp340050b49a6c_fld2240inst_Slot_inst_get (insn) == 20 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC_RL;
  if (Field_dsp340050b49a6c_fld2241inst_Slot_inst_get (insn) == 41 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_SAC2X64_3;
  if (Field_dsp340050b49a6c_fld2242inst_Slot_inst_get (insn) == 45 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3661inst_Slot_inst_get (insn) == 0)
    return OPCODE_ASLACM;
  if (Field_dsp340050b49a6c_fld2243inst_Slot_inst_get (insn) == 21 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_STSWAPBMU;
  if (Field_dsp340050b49a6c_fld2244inst_Slot_inst_get (insn) == 92 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld2245inst_Slot_inst_get (insn) == 93 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_CM2AR_LN_I;
  if (Field_dsp340050b49a6c_fld2246inst_Slot_inst_get (insn) == 47 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3638inst_Slot_inst_get (insn) == 0)
    return OPCODE_CM2AR_LN_R;
  if (Field_dsp340050b49a6c_fld2247inst_Slot_inst_get (insn) == 24 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0)
    return OPCODE_STORE_Q;
  if (Field_dsp340050b49a6c_fld2248inst_Slot_inst_get (insn) == 50 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3651inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2M_0;
  if (Field_dsp340050b49a6c_fld2249inst_Slot_inst_get (insn) == 51 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3655inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2M_3;
  if (Field_dsp340050b49a6c_fld2250inst_Slot_inst_get (insn) == 26 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3653inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2M_1;
  if (Field_dsp340050b49a6c_fld2251inst_Slot_inst_get (insn) == 216 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3658inst_Slot_inst_get (insn) == 0)
    return OPCODE_PUSH32;
  if (Field_dsp340050b49a6c_fld2252inst_Slot_inst_get (insn) == 217 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3643inst_Slot_inst_get (insn) == 0)
    return OPCODE_SWAPB;
  if (Field_dsp340050b49a6c_fld2253inst_Slot_inst_get (insn) == 109 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3639inst_Slot_inst_get (insn) == 0)
    return OPCODE_MOV2AC32_I;
  if (Field_dsp340050b49a6c_fld2255inst_Slot_inst_get (insn) == 55 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3640inst_Slot_inst_get (insn) == 0)
    return OPCODE_MOV2AC32_R;
  if (Field_dsp340050b49a6c_fld2257inst_Slot_inst_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld2019_Slot_inst_get (insn) == 6 &&
      Field_op0_Slot_inst_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3654inst_Slot_inst_get (insn) == 0)
    return OPCODE_POP128_2M_2;
  switch (Field_op0_Slot_inst_get (insn))
    {
    case 0:
      switch (Field_op1_Slot_inst_get (insn))
	{
	case 0:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	      switch (Field_r_Slot_inst_get (insn))
		{
		case 0:
		  switch (Field_m_Slot_inst_get (insn))
		    {
		    case 0:
		      if (Field_s_Slot_inst_get (insn) == 0 &&
			  Field_n_Slot_inst_get (insn) == 0)
			return OPCODE_ILL;
		      break;
		    case 2:
		      switch (Field_n_Slot_inst_get (insn))
			{
			case 0:
			  return OPCODE_RET;
			case 1:
			  return OPCODE_RETW;
			case 2:
			  return OPCODE_JX;
			}
		      break;
		    case 3:
		      switch (Field_n_Slot_inst_get (insn))
			{
			case 0:
			  return OPCODE_CALLX0;
			case 1:
			  return OPCODE_CALLX4;
			case 2:
			  return OPCODE_CALLX8;
			case 3:
			  return OPCODE_CALLX12;
			}
		      break;
		    }
		  break;
		case 1:
		  return OPCODE_MOVSP;
		case 2:
		  if (Field_s_Slot_inst_get (insn) == 0)
		    {
		      switch (Field_t_Slot_inst_get (insn))
			{
			case 0:
			  return OPCODE_ISYNC;
			case 1:
			  return OPCODE_RSYNC;
			case 2:
			  return OPCODE_ESYNC;
			case 3:
			  return OPCODE_DSYNC;
			case 8:
			  return OPCODE_EXCW;
			case 12:
			  return OPCODE_MEMW;
			case 13:
			  return OPCODE_EXTW;
			case 15:
			  return OPCODE_NOP;
			}
		    }
		  break;
		case 3:
		  switch (Field_t_Slot_inst_get (insn))
		    {
		    case 0:
		      switch (Field_s_Slot_inst_get (insn))
			{
			case 0:
			  return OPCODE_RFE;
			case 2:
			  return OPCODE_RFDE;
			case 4:
			  return OPCODE_RFWO;
			case 5:
			  return OPCODE_RFWU;
			}
		      break;
		    case 1:
		      return OPCODE_RFI;
		    }
		  break;
		case 4:
		  return OPCODE_BREAK;
		case 5:
		  switch (Field_s_Slot_inst_get (insn))
		    {
		    case 0:
		      if (Field_t_Slot_inst_get (insn) == 0)
			return OPCODE_SYSCALL;
		      break;
		    case 1:
		      if (Field_t_Slot_inst_get (insn) == 0)
			return OPCODE_SIMCALL;
		      break;
		    }
		  break;
		case 6:
		  return OPCODE_RSIL;
		case 7:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_WAITI;
		  break;
		case 8:
		  return OPCODE_ANY4;
		case 9:
		  return OPCODE_ALL4;
		case 10:
		  return OPCODE_ANY8;
		case 11:
		  return OPCODE_ALL8;
		}
	      break;
	    case 1:
	      return OPCODE_AND;
	    case 2:
	      return OPCODE_OR;
	    case 3:
	      return OPCODE_XOR;
	    case 4:
	      switch (Field_r_Slot_inst_get (insn))
		{
		case 0:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_SSR;
		  break;
		case 1:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_SSL;
		  break;
		case 2:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_SSA8L;
		  break;
		case 3:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_SSA8B;
		  break;
		case 4:
		  if (Field_thi3_Slot_inst_get (insn) == 0)
		    return OPCODE_SSAI;
		  break;
		case 6:
		  return OPCODE_RER;
		case 7:
		  return OPCODE_WER;
		case 8:
		  if (Field_s_Slot_inst_get (insn) == 0)
		    return OPCODE_ROTW;
		  break;
		case 14:
		  return OPCODE_NSA;
		case 15:
		  return OPCODE_NSAU;
		}
	      break;
	    case 5:
	      switch (Field_r_Slot_inst_get (insn))
		{
		case 3:
		  return OPCODE_RITLB0;
		case 4:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_IITLB;
		  break;
		case 5:
		  return OPCODE_PITLB;
		case 6:
		  return OPCODE_WITLB;
		case 7:
		  return OPCODE_RITLB1;
		case 11:
		  return OPCODE_RDTLB0;
		case 12:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_IDTLB;
		  break;
		case 13:
		  return OPCODE_PDTLB;
		case 14:
		  return OPCODE_WDTLB;
		case 15:
		  return OPCODE_RDTLB1;
		}
	      break;
	    case 6:
	      switch (Field_s_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_NEG;
		case 1:
		  return OPCODE_ABS;
		}
	      break;
	    case 8:
	      return OPCODE_ADD;
	    case 9:
	      return OPCODE_ADDX2;
	    case 10:
	      return OPCODE_ADDX4;
	    case 11:
	      return OPCODE_ADDX8;
	    case 12:
	      return OPCODE_SUB;
	    case 13:
	      return OPCODE_SUBX2;
	    case 14:
	      return OPCODE_SUBX4;
	    case 15:
	      return OPCODE_SUBX8;
	    }
	  break;
	case 1:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	    case 1:
	      return OPCODE_SLLI;
	    case 2:
	    case 3:
	      return OPCODE_SRAI;
	    case 4:
	      return OPCODE_SRLI;
	    case 6:
	      switch (Field_sr_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_XSR_LBEG;
		case 1:
		  return OPCODE_XSR_LEND;
		case 2:
		  return OPCODE_XSR_LCOUNT;
		case 3:
		  return OPCODE_XSR_SAR;
		case 4:
		  return OPCODE_XSR_BR;
		case 5:
		  return OPCODE_XSR_LITBASE;
		case 12:
		  return OPCODE_XSR_SCOMPARE1;
		case 72:
		  return OPCODE_XSR_WINDOWBASE;
		case 73:
		  return OPCODE_XSR_WINDOWSTART;
		case 96:
		  return OPCODE_XSR_IBREAKENABLE;
		case 99:
		  return OPCODE_XSR_ATOMCTL;
		case 104:
		  return OPCODE_XSR_DDR;
		case 128:
		  return OPCODE_XSR_IBREAKA0;
		case 129:
		  return OPCODE_XSR_IBREAKA1;
		case 144:
		  return OPCODE_XSR_DBREAKA0;
		case 145:
		  return OPCODE_XSR_DBREAKA1;
		case 160:
		  return OPCODE_XSR_DBREAKC0;
		case 161:
		  return OPCODE_XSR_DBREAKC1;
		case 177:
		  return OPCODE_XSR_EPC1;
		case 178:
		  return OPCODE_XSR_EPC2;
		case 179:
		  return OPCODE_XSR_EPC3;
		case 180:
		  return OPCODE_XSR_EPC4;
		case 181:
		  return OPCODE_XSR_EPC5;
		case 182:
		  return OPCODE_XSR_EPC6;
		case 192:
		  return OPCODE_XSR_DEPC;
		case 194:
		  return OPCODE_XSR_EPS2;
		case 195:
		  return OPCODE_XSR_EPS3;
		case 196:
		  return OPCODE_XSR_EPS4;
		case 197:
		  return OPCODE_XSR_EPS5;
		case 198:
		  return OPCODE_XSR_EPS6;
		case 209:
		  return OPCODE_XSR_EXCSAVE1;
		case 210:
		  return OPCODE_XSR_EXCSAVE2;
		case 211:
		  return OPCODE_XSR_EXCSAVE3;
		case 212:
		  return OPCODE_XSR_EXCSAVE4;
		case 213:
		  return OPCODE_XSR_EXCSAVE5;
		case 214:
		  return OPCODE_XSR_EXCSAVE6;
		case 224:
		  return OPCODE_XSR_CPENABLE;
		case 228:
		  return OPCODE_XSR_INTENABLE;
		case 230:
		  return OPCODE_XSR_PS;
		case 231:
		  return OPCODE_XSR_VECBASE;
		case 232:
		  return OPCODE_XSR_EXCCAUSE;
		case 233:
		  return OPCODE_XSR_DEBUGCAUSE;
		case 234:
		  return OPCODE_XSR_CCOUNT;
		case 236:
		  return OPCODE_XSR_ICOUNT;
		case 237:
		  return OPCODE_XSR_ICOUNTLEVEL;
		case 238:
		  return OPCODE_XSR_EXCVADDR;
		case 240:
		  return OPCODE_XSR_CCOMPARE0;
		case 241:
		  return OPCODE_XSR_CCOMPARE1;
		}
	      break;
	    case 8:
	      return OPCODE_SRC;
	    case 9:
	      if (Field_s_Slot_inst_get (insn) == 0)
		return OPCODE_SRL;
	      break;
	    case 10:
	      if (Field_t_Slot_inst_get (insn) == 0)
		return OPCODE_SLL;
	      break;
	    case 11:
	      if (Field_s_Slot_inst_get (insn) == 0)
		return OPCODE_SRA;
	      break;
	    case 12:
	      return OPCODE_MUL16U;
	    case 13:
	      return OPCODE_MUL16S;
	    case 15:
	      switch (Field_r_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_LICT;
		case 1:
		  return OPCODE_SICT;
		case 2:
		  return OPCODE_LICW;
		case 3:
		  return OPCODE_SICW;
		case 8:
		  return OPCODE_LDCT;
		case 9:
		  return OPCODE_SDCT;
		case 14:
		  if (Field_t_Slot_inst_get (insn) == 0)
		    return OPCODE_RFDO;
		  if (Field_t_Slot_inst_get (insn) == 1)
		    return OPCODE_RFDD;
		  break;
		}
	      break;
	    }
	  break;
	case 2:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_ANDB;
	    case 1:
	      return OPCODE_ANDBC;
	    case 2:
	      return OPCODE_ORB;
	    case 3:
	      return OPCODE_ORBC;
	    case 4:
	      return OPCODE_XORB;
	    }
	  break;
	case 3:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	      switch (Field_sr_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_RSR_LBEG;
		case 1:
		  return OPCODE_RSR_LEND;
		case 2:
		  return OPCODE_RSR_LCOUNT;
		case 3:
		  return OPCODE_RSR_SAR;
		case 4:
		  return OPCODE_RSR_BR;
		case 5:
		  return OPCODE_RSR_LITBASE;
		case 12:
		  return OPCODE_RSR_SCOMPARE1;
		case 72:
		  return OPCODE_RSR_WINDOWBASE;
		case 73:
		  return OPCODE_RSR_WINDOWSTART;
		case 96:
		  return OPCODE_RSR_IBREAKENABLE;
		case 99:
		  return OPCODE_RSR_ATOMCTL;
		case 104:
		  return OPCODE_RSR_DDR;
		case 128:
		  return OPCODE_RSR_IBREAKA0;
		case 129:
		  return OPCODE_RSR_IBREAKA1;
		case 144:
		  return OPCODE_RSR_DBREAKA0;
		case 145:
		  return OPCODE_RSR_DBREAKA1;
		case 160:
		  return OPCODE_RSR_DBREAKC0;
		case 161:
		  return OPCODE_RSR_DBREAKC1;
		case 176:
		  return OPCODE_RSR_176;
		case 177:
		  return OPCODE_RSR_EPC1;
		case 178:
		  return OPCODE_RSR_EPC2;
		case 179:
		  return OPCODE_RSR_EPC3;
		case 180:
		  return OPCODE_RSR_EPC4;
		case 181:
		  return OPCODE_RSR_EPC5;
		case 182:
		  return OPCODE_RSR_EPC6;
		case 192:
		  return OPCODE_RSR_DEPC;
		case 194:
		  return OPCODE_RSR_EPS2;
		case 195:
		  return OPCODE_RSR_EPS3;
		case 196:
		  return OPCODE_RSR_EPS4;
		case 197:
		  return OPCODE_RSR_EPS5;
		case 198:
		  return OPCODE_RSR_EPS6;
		case 208:
		  return OPCODE_RSR_208;
		case 209:
		  return OPCODE_RSR_EXCSAVE1;
		case 210:
		  return OPCODE_RSR_EXCSAVE2;
		case 211:
		  return OPCODE_RSR_EXCSAVE3;
		case 212:
		  return OPCODE_RSR_EXCSAVE4;
		case 213:
		  return OPCODE_RSR_EXCSAVE5;
		case 214:
		  return OPCODE_RSR_EXCSAVE6;
		case 224:
		  return OPCODE_RSR_CPENABLE;
		case 226:
		  return OPCODE_RSR_INTERRUPT;
		case 228:
		  return OPCODE_RSR_INTENABLE;
		case 230:
		  return OPCODE_RSR_PS;
		case 231:
		  return OPCODE_RSR_VECBASE;
		case 232:
		  return OPCODE_RSR_EXCCAUSE;
		case 233:
		  return OPCODE_RSR_DEBUGCAUSE;
		case 234:
		  return OPCODE_RSR_CCOUNT;
		case 235:
		  return OPCODE_RSR_PRID;
		case 236:
		  return OPCODE_RSR_ICOUNT;
		case 237:
		  return OPCODE_RSR_ICOUNTLEVEL;
		case 238:
		  return OPCODE_RSR_EXCVADDR;
		case 240:
		  return OPCODE_RSR_CCOMPARE0;
		case 241:
		  return OPCODE_RSR_CCOMPARE1;
		}
	      break;
	    case 1:
	      switch (Field_sr_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_WSR_LBEG;
		case 1:
		  return OPCODE_WSR_LEND;
		case 2:
		  return OPCODE_WSR_LCOUNT;
		case 3:
		  return OPCODE_WSR_SAR;
		case 4:
		  return OPCODE_WSR_BR;
		case 5:
		  return OPCODE_WSR_LITBASE;
		case 12:
		  return OPCODE_WSR_SCOMPARE1;
		case 72:
		  return OPCODE_WSR_WINDOWBASE;
		case 73:
		  return OPCODE_WSR_WINDOWSTART;
		case 89:
		  return OPCODE_WSR_MMID;
		case 96:
		  return OPCODE_WSR_IBREAKENABLE;
		case 99:
		  return OPCODE_WSR_ATOMCTL;
		case 104:
		  return OPCODE_WSR_DDR;
		case 128:
		  return OPCODE_WSR_IBREAKA0;
		case 129:
		  return OPCODE_WSR_IBREAKA1;
		case 144:
		  return OPCODE_WSR_DBREAKA0;
		case 145:
		  return OPCODE_WSR_DBREAKA1;
		case 160:
		  return OPCODE_WSR_DBREAKC0;
		case 161:
		  return OPCODE_WSR_DBREAKC1;
		case 176:
		  return OPCODE_WSR_176;
		case 177:
		  return OPCODE_WSR_EPC1;
		case 178:
		  return OPCODE_WSR_EPC2;
		case 179:
		  return OPCODE_WSR_EPC3;
		case 180:
		  return OPCODE_WSR_EPC4;
		case 181:
		  return OPCODE_WSR_EPC5;
		case 182:
		  return OPCODE_WSR_EPC6;
		case 192:
		  return OPCODE_WSR_DEPC;
		case 194:
		  return OPCODE_WSR_EPS2;
		case 195:
		  return OPCODE_WSR_EPS3;
		case 196:
		  return OPCODE_WSR_EPS4;
		case 197:
		  return OPCODE_WSR_EPS5;
		case 198:
		  return OPCODE_WSR_EPS6;
		case 209:
		  return OPCODE_WSR_EXCSAVE1;
		case 210:
		  return OPCODE_WSR_EXCSAVE2;
		case 211:
		  return OPCODE_WSR_EXCSAVE3;
		case 212:
		  return OPCODE_WSR_EXCSAVE4;
		case 213:
		  return OPCODE_WSR_EXCSAVE5;
		case 214:
		  return OPCODE_WSR_EXCSAVE6;
		case 224:
		  return OPCODE_WSR_CPENABLE;
		case 226:
		  return OPCODE_WSR_INTSET;
		case 227:
		  return OPCODE_WSR_INTCLEAR;
		case 228:
		  return OPCODE_WSR_INTENABLE;
		case 230:
		  return OPCODE_WSR_PS;
		case 231:
		  return OPCODE_WSR_VECBASE;
		case 232:
		  return OPCODE_WSR_EXCCAUSE;
		case 233:
		  return OPCODE_WSR_DEBUGCAUSE;
		case 234:
		  return OPCODE_WSR_CCOUNT;
		case 236:
		  return OPCODE_WSR_ICOUNT;
		case 237:
		  return OPCODE_WSR_ICOUNTLEVEL;
		case 238:
		  return OPCODE_WSR_EXCVADDR;
		case 240:
		  return OPCODE_WSR_CCOMPARE0;
		case 241:
		  return OPCODE_WSR_CCOMPARE1;
		}
	      break;
	    case 2:
	      return OPCODE_SEXT;
	    case 3:
	      return OPCODE_CLAMPS;
	    case 4:
	      return OPCODE_MIN;
	    case 5:
	      return OPCODE_MAX;
	    case 6:
	      return OPCODE_MINU;
	    case 7:
	      return OPCODE_MAXU;
	    case 8:
	      return OPCODE_MOVEQZ;
	    case 9:
	      return OPCODE_MOVNEZ;
	    case 10:
	      return OPCODE_MOVLTZ;
	    case 11:
	      return OPCODE_MOVGEZ;
	    case 12:
	      return OPCODE_MOVF;
	    case 13:
	      return OPCODE_MOVT;
	    case 14:
	      switch (Field_st_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_RUR_EXPSTATE;
		case 1:
		  return OPCODE_RUR_SOV;
		case 2:
		  return OPCODE_RUR_SAT_MODE;
		case 3:
		  return OPCODE_RUR_SAR0;
		case 4:
		  return OPCODE_RUR_SAR1;
		case 5:
		  return OPCODE_RUR_SAR2;
		case 6:
		  return OPCODE_RUR_SAR3;
		case 7:
		  return OPCODE_RUR_HSAR0;
		case 8:
		  return OPCODE_RUR_HSAR1;
		case 9:
		  return OPCODE_RUR_HSAR2;
		case 10:
		  return OPCODE_RUR_HSAR3;
		case 11:
		  return OPCODE_RUR_MAX_REG_0;
		case 12:
		  return OPCODE_RUR_MAX_REG_1;
		case 13:
		  return OPCODE_RUR_MAX_REG_2;
		case 14:
		  return OPCODE_RUR_MAX_REG_3;
		case 15:
		  return OPCODE_RUR_ARG_MAX_REG_0;
		case 16:
		  return OPCODE_RUR_ARG_MAX_REG_1;
		case 17:
		  return OPCODE_RUR_ARG_MAX_REG_2;
		case 18:
		  return OPCODE_RUR_ARG_MAX_REG_3;
		case 19:
		  return OPCODE_RUR_NCO_COUNTER_0;
		case 20:
		  return OPCODE_RUR_NCO_COUNTER_1;
		case 21:
		  return OPCODE_RUR_NCO_COUNTER_2;
		case 22:
		  return OPCODE_RUR_NCO_COUNTER_3;
		case 23:
		  return OPCODE_RUR_INTERP_EXT_N;
		case 24:
		  return OPCODE_RUR_INTERP_EXT_L;
		case 25:
		  return OPCODE_RUR_LLR_BUF_0;
		case 26:
		  return OPCODE_RUR_LLR_BUF_1;
		case 27:
		  return OPCODE_RUR_LLR_BUF_2;
		case 28:
		  return OPCODE_RUR_LLR_BUF_3;
		case 29:
		  return OPCODE_RUR_LLR_BUF_4;
		case 30:
		  return OPCODE_RUR_LLR_BUF_5;
		case 31:
		  return OPCODE_RUR_LLR_BUF_6;
		case 32:
		  return OPCODE_RUR_LLR_BUF_7;
		case 33:
		  return OPCODE_RUR_LLR_BUF_8;
		case 34:
		  return OPCODE_RUR_LLR_BUF_9;
		case 35:
		  return OPCODE_RUR_LLR_BUF_10;
		case 36:
		  return OPCODE_RUR_LLR_BUF_11;
		case 37:
		  return OPCODE_RUR_LLR_BUF_12;
		case 38:
		  return OPCODE_RUR_LLR_BUF_13;
		case 39:
		  return OPCODE_RUR_LLR_BUF_14;
		case 40:
		  return OPCODE_RUR_LLR_BUF_15;
		case 41:
		  return OPCODE_RUR_LLR_BUF_16;
		case 42:
		  return OPCODE_RUR_LLR_BUF_17;
		case 43:
		  return OPCODE_RUR_LLR_BUF_18;
		case 44:
		  return OPCODE_RUR_LLR_BUF_19;
		case 45:
		  return OPCODE_RUR_LLR_BUF_20;
		case 46:
		  return OPCODE_RUR_LLR_BUF_21;
		case 47:
		  return OPCODE_RUR_LLR_BUF_22;
		case 48:
		  return OPCODE_RUR_LLR_BUF_23;
		case 49:
		  return OPCODE_RUR_SMOD_BUF_0;
		case 50:
		  return OPCODE_RUR_SMOD_BUF_1;
		case 51:
		  return OPCODE_RUR_SMOD_BUF_2;
		case 52:
		  return OPCODE_RUR_SMOD_BUF_3;
		case 53:
		  return OPCODE_RUR_SMOD_BUF_4;
		case 54:
		  return OPCODE_RUR_SMOD_BUF_5;
		case 55:
		  return OPCODE_RUR_SMOD_BUF_6;
		case 56:
		  return OPCODE_RUR_SMOD_BUF_7;
		case 57:
		  return OPCODE_RUR_WEIGHT_REG;
		case 58:
		  return OPCODE_RUR_SCALE_REG;
		case 59:
		  return OPCODE_RUR_LLR_POS;
		case 60:
		  return OPCODE_RUR_SMOD_POS;
		case 61:
		  return OPCODE_RUR_PERM_REG;
		case 62:
		  return OPCODE_RUR_SMOD_OFFSET_TABLE_0;
		case 63:
		  return OPCODE_RUR_SMOD_OFFSET_TABLE_1;
		case 64:
		  return OPCODE_RUR_SMOD_OFFSET_TABLE_2;
		case 65:
		  return OPCODE_RUR_SMOD_OFFSET_TABLE_3;
		case 66:
		  return OPCODE_RUR_PHASOR_N;
		case 67:
		  return OPCODE_RUR_PHASOR_OFFSET;
		case 231:
		  return OPCODE_RUR_THREADPTR;
		case 232:
		  return OPCODE_RUR_FCR;
		case 233:
		  return OPCODE_RUR_FSR;
		}
	      break;
	    case 15:
	      switch (Field_sr_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_WUR_EXPSTATE;
		case 1:
		  return OPCODE_WUR_SOV;
		case 2:
		  return OPCODE_WUR_SAT_MODE;
		case 3:
		  return OPCODE_WUR_SAR0;
		case 4:
		  return OPCODE_WUR_SAR1;
		case 5:
		  return OPCODE_WUR_SAR2;
		case 6:
		  return OPCODE_WUR_SAR3;
		case 7:
		  return OPCODE_WUR_HSAR0;
		case 8:
		  return OPCODE_WUR_HSAR1;
		case 9:
		  return OPCODE_WUR_HSAR2;
		case 10:
		  return OPCODE_WUR_HSAR3;
		case 11:
		  return OPCODE_WUR_MAX_REG_0;
		case 12:
		  return OPCODE_WUR_MAX_REG_1;
		case 13:
		  return OPCODE_WUR_MAX_REG_2;
		case 14:
		  return OPCODE_WUR_MAX_REG_3;
		case 15:
		  return OPCODE_WUR_ARG_MAX_REG_0;
		case 16:
		  return OPCODE_WUR_ARG_MAX_REG_1;
		case 17:
		  return OPCODE_WUR_ARG_MAX_REG_2;
		case 18:
		  return OPCODE_WUR_ARG_MAX_REG_3;
		case 19:
		  return OPCODE_WUR_NCO_COUNTER_0;
		case 20:
		  return OPCODE_WUR_NCO_COUNTER_1;
		case 21:
		  return OPCODE_WUR_NCO_COUNTER_2;
		case 22:
		  return OPCODE_WUR_NCO_COUNTER_3;
		case 23:
		  return OPCODE_WUR_INTERP_EXT_N;
		case 24:
		  return OPCODE_WUR_INTERP_EXT_L;
		case 25:
		  return OPCODE_WUR_LLR_BUF_0;
		case 26:
		  return OPCODE_WUR_LLR_BUF_1;
		case 27:
		  return OPCODE_WUR_LLR_BUF_2;
		case 28:
		  return OPCODE_WUR_LLR_BUF_3;
		case 29:
		  return OPCODE_WUR_LLR_BUF_4;
		case 30:
		  return OPCODE_WUR_LLR_BUF_5;
		case 31:
		  return OPCODE_WUR_LLR_BUF_6;
		case 32:
		  return OPCODE_WUR_LLR_BUF_7;
		case 33:
		  return OPCODE_WUR_LLR_BUF_8;
		case 34:
		  return OPCODE_WUR_LLR_BUF_9;
		case 35:
		  return OPCODE_WUR_LLR_BUF_10;
		case 36:
		  return OPCODE_WUR_LLR_BUF_11;
		case 37:
		  return OPCODE_WUR_LLR_BUF_12;
		case 38:
		  return OPCODE_WUR_LLR_BUF_13;
		case 39:
		  return OPCODE_WUR_LLR_BUF_14;
		case 40:
		  return OPCODE_WUR_LLR_BUF_15;
		case 41:
		  return OPCODE_WUR_LLR_BUF_16;
		case 42:
		  return OPCODE_WUR_LLR_BUF_17;
		case 43:
		  return OPCODE_WUR_LLR_BUF_18;
		case 44:
		  return OPCODE_WUR_LLR_BUF_19;
		case 45:
		  return OPCODE_WUR_LLR_BUF_20;
		case 46:
		  return OPCODE_WUR_LLR_BUF_21;
		case 47:
		  return OPCODE_WUR_LLR_BUF_22;
		case 48:
		  return OPCODE_WUR_LLR_BUF_23;
		case 49:
		  return OPCODE_WUR_SMOD_BUF_0;
		case 50:
		  return OPCODE_WUR_SMOD_BUF_1;
		case 51:
		  return OPCODE_WUR_SMOD_BUF_2;
		case 52:
		  return OPCODE_WUR_SMOD_BUF_3;
		case 53:
		  return OPCODE_WUR_SMOD_BUF_4;
		case 54:
		  return OPCODE_WUR_SMOD_BUF_5;
		case 55:
		  return OPCODE_WUR_SMOD_BUF_6;
		case 56:
		  return OPCODE_WUR_SMOD_BUF_7;
		case 57:
		  return OPCODE_WUR_WEIGHT_REG;
		case 58:
		  return OPCODE_WUR_SCALE_REG;
		case 59:
		  return OPCODE_WUR_LLR_POS;
		case 60:
		  return OPCODE_WUR_SMOD_POS;
		case 61:
		  return OPCODE_WUR_PERM_REG;
		case 62:
		  return OPCODE_WUR_SMOD_OFFSET_TABLE_0;
		case 63:
		  return OPCODE_WUR_SMOD_OFFSET_TABLE_1;
		case 64:
		  return OPCODE_WUR_SMOD_OFFSET_TABLE_2;
		case 65:
		  return OPCODE_WUR_SMOD_OFFSET_TABLE_3;
		case 66:
		  return OPCODE_WUR_PHASOR_N;
		case 67:
		  return OPCODE_WUR_PHASOR_OFFSET;
		case 231:
		  return OPCODE_WUR_THREADPTR;
		case 232:
		  return OPCODE_WUR_FCR;
		case 233:
		  return OPCODE_WUR_FSR;
		}
	      break;
	    }
	  break;
	case 4:
	case 5:
	  return OPCODE_EXTUI;
	case 8:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_LSX;
	    case 1:
	      return OPCODE_LSXU;
	    case 4:
	      return OPCODE_SSX;
	    case 5:
	      return OPCODE_SSXU;
	    }
	  break;
	case 9:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_L32E;
	    case 4:
	      return OPCODE_S32E;
	    }
	  break;
	case 10:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_ADD_S;
	    case 1:
	      return OPCODE_SUB_S;
	    case 2:
	      return OPCODE_MUL_S;
	    case 4:
	      return OPCODE_MADD_S;
	    case 5:
	      return OPCODE_MSUB_S;
	    case 8:
	      return OPCODE_ROUND_S;
	    case 9:
	      return OPCODE_TRUNC_S;
	    case 10:
	      return OPCODE_FLOOR_S;
	    case 11:
	      return OPCODE_CEIL_S;
	    case 12:
	      return OPCODE_FLOAT_S;
	    case 13:
	      return OPCODE_UFLOAT_S;
	    case 14:
	      return OPCODE_UTRUNC_S;
	    case 15:
	      switch (Field_t_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_MOV_S;
		case 1:
		  return OPCODE_ABS_S;
		case 4:
		  return OPCODE_RFR;
		case 5:
		  return OPCODE_WFR;
		case 6:
		  return OPCODE_NEG_S;
		}
	      break;
	    }
	  break;
	case 11:
	  switch (Field_op2_Slot_inst_get (insn))
	    {
	    case 1:
	      return OPCODE_UN_S;
	    case 2:
	      return OPCODE_OEQ_S;
	    case 3:
	      return OPCODE_UEQ_S;
	    case 4:
	      return OPCODE_OLT_S;
	    case 5:
	      return OPCODE_ULT_S;
	    case 6:
	      return OPCODE_OLE_S;
	    case 7:
	      return OPCODE_ULE_S;
	    case 8:
	      return OPCODE_MOVEQZ_S;
	    case 9:
	      return OPCODE_MOVNEZ_S;
	    case 10:
	      return OPCODE_MOVLTZ_S;
	    case 11:
	      return OPCODE_MOVGEZ_S;
	    case 12:
	      return OPCODE_MOVF_S;
	    case 13:
	      return OPCODE_MOVT_S;
	    }
	  break;
	}
      break;
    case 1:
      return OPCODE_L32R;
    case 2:
      switch (Field_r_Slot_inst_get (insn))
	{
	case 0:
	  return OPCODE_L8UI;
	case 1:
	  return OPCODE_L16UI;
	case 2:
	  return OPCODE_L32I;
	case 4:
	  return OPCODE_S8I;
	case 5:
	  return OPCODE_S16I;
	case 6:
	  return OPCODE_S32I;
	case 7:
	  switch (Field_t_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_DPFR;
	    case 1:
	      return OPCODE_DPFW;
	    case 2:
	      return OPCODE_DPFRO;
	    case 3:
	      return OPCODE_DPFWO;
	    case 4:
	      return OPCODE_DHWB;
	    case 5:
	      return OPCODE_DHWBI;
	    case 6:
	      return OPCODE_DHI;
	    case 7:
	      return OPCODE_DII;
	    case 8:
	      switch (Field_op1_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_DPFL;
		case 2:
		  return OPCODE_DHU;
		case 3:
		  return OPCODE_DIU;
		case 4:
		  return OPCODE_DIWB;
		case 5:
		  return OPCODE_DIWBI;
		}
	      break;
	    case 12:
	      return OPCODE_IPF;
	    case 13:
	      switch (Field_op1_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_IPFL;
		case 2:
		  return OPCODE_IHU;
		case 3:
		  return OPCODE_IIU;
		}
	      break;
	    case 14:
	      return OPCODE_IHI;
	    case 15:
	      return OPCODE_III;
	    }
	  break;
	case 9:
	  return OPCODE_L16SI;
	case 10:
	  return OPCODE_MOVI;
	case 11:
	  return OPCODE_L32AI;
	case 12:
	  return OPCODE_ADDI;
	case 13:
	  return OPCODE_ADDMI;
	case 14:
	  return OPCODE_S32C1I;
	case 15:
	  return OPCODE_S32RI;
	}
      break;
    case 3:
      switch (Field_r_Slot_inst_get (insn))
	{
	case 0:
	  return OPCODE_LSI;
	case 4:
	  return OPCODE_SSI;
	case 8:
	  return OPCODE_LSIU;
	case 12:
	  return OPCODE_SSIU;
	}
      break;
    case 5:
      switch (Field_n_Slot_inst_get (insn))
	{
	case 0:
	  return OPCODE_CALL0;
	case 1:
	  return OPCODE_CALL4;
	case 2:
	  return OPCODE_CALL8;
	case 3:
	  return OPCODE_CALL12;
	}
      break;
    case 6:
      switch (Field_n_Slot_inst_get (insn))
	{
	case 0:
	  return OPCODE_J;
	case 1:
	  switch (Field_m_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_BEQZ;
	    case 1:
	      return OPCODE_BNEZ;
	    case 2:
	      return OPCODE_BLTZ;
	    case 3:
	      return OPCODE_BGEZ;
	    }
	  break;
	case 2:
	  switch (Field_m_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_BEQI;
	    case 1:
	      return OPCODE_BNEI;
	    case 2:
	      return OPCODE_BLTI;
	    case 3:
	      return OPCODE_BGEI;
	    }
	  break;
	case 3:
	  switch (Field_m_Slot_inst_get (insn))
	    {
	    case 0:
	      return OPCODE_ENTRY;
	    case 1:
	      switch (Field_r_Slot_inst_get (insn))
		{
		case 0:
		  return OPCODE_BF;
		case 1:
		  return OPCODE_BT;
		case 8:
		  return OPCODE_LOOP;
		case 9:
		  return OPCODE_LOOPNEZ;
		case 10:
		  return OPCODE_LOOPGTZ;
		}
	      break;
	    case 2:
	      return OPCODE_BLTUI;
	    case 3:
	      return OPCODE_BGEUI;
	    }
	  break;
	}
      break;
    case 7:
      switch (Field_r_Slot_inst_get (insn))
	{
	case 0:
	  return OPCODE_BNONE;
	case 1:
	  return OPCODE_BEQ;
	case 2:
	  return OPCODE_BLT;
	case 3:
	  return OPCODE_BLTU;
	case 4:
	  return OPCODE_BALL;
	case 5:
	  return OPCODE_BBC;
	case 6:
	case 7:
	  return OPCODE_BBCI;
	case 8:
	  return OPCODE_BANY;
	case 9:
	  return OPCODE_BNE;
	case 10:
	  return OPCODE_BGE;
	case 11:
	  return OPCODE_BGEU;
	case 12:
	  return OPCODE_BNALL;
	case 13:
	  return OPCODE_BBS;
	case 14:
	case 15:
	  return OPCODE_BBSI;
	}
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_inst16b_decode (const xtensa_insnbuf insn)
{
  switch (Field_op0_Slot_inst16b_get (insn))
    {
    case 12:
      switch (Field_i_Slot_inst16b_get (insn))
	{
	case 0:
	  return OPCODE_MOVI_N;
	case 1:
	  switch (Field_z_Slot_inst16b_get (insn))
	    {
	    case 0:
	      return OPCODE_BEQZ_N;
	    case 1:
	      return OPCODE_BNEZ_N;
	    }
	  break;
	}
      break;
    case 13:
      switch (Field_r_Slot_inst16b_get (insn))
	{
	case 0:
	  return OPCODE_MOV_N;
	case 15:
	  switch (Field_t_Slot_inst16b_get (insn))
	    {
	    case 0:
	      return OPCODE_RET_N;
	    case 1:
	      return OPCODE_RETW_N;
	    case 2:
	      return OPCODE_BREAK_N;
	    case 3:
	      if (Field_s_Slot_inst16b_get (insn) == 0)
		return OPCODE_NOP_N;
	      break;
	    case 6:
	      if (Field_s_Slot_inst16b_get (insn) == 0)
		return OPCODE_ILL_N;
	      break;
	    }
	  break;
	}
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_inst16a_decode (const xtensa_insnbuf insn)
{
  switch (Field_op0_Slot_inst16a_get (insn))
    {
    case 8:
      return OPCODE_L32I_N;
    case 9:
      return OPCODE_S32I_N;
    case 10:
      return OPCODE_ADD_N;
    case 11:
      return OPCODE_ADDI_N;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_gp_slot2_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2258gp_slot2_Slot_gp_slot2_get (insn))
    {
    case 22:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVI_N;
      break;
    case 24:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_AR2PQ_LN;
      break;
    }
  if (Field_dsp340050b49a6c_fld2259gp_slot2_Slot_gp_slot2_get (insn) == 46 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_AR2CM_LN;
  switch (Field_dsp340050b49a6c_fld2260gp_slot2_Slot_gp_slot2_get (insn))
    {
    case 185:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_ADDAR2;
      break;
    case 187:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOV_N;
      break;
    case 189:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVPQ2PQ;
      break;
    case 191:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_PUSH128_PQ;
      break;
    case 200:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_PUSH32;
      break;
    case 201:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_SET_EXT_REGS;
      break;
    case 202:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_SUBARX;
      break;
    case 204:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_WRTIEP;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2261gp_slot2_Slot_gp_slot2_get (insn))
    {
    case 1624:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_CLRTIEP;
      break;
    case 1625:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_SETTIEP;
      break;
    case 1626:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_WRTBSIGQM;
      break;
    case 1631:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2044_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_1;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2262gp_slot2_Slot_gp_slot2_get (insn))
    {
    case 3254:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_01;
      break;
    case 3255:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_03;
      break;
    case 3256:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_AR2SAR_DUP;
      break;
    case 3257:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_21;
      break;
    case 3258:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_23;
      break;
    case 3259:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_WRTSIGQ;
      break;
    case 3260:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_WRTBSIGQ;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2263gp_slot2_Slot_gp_slot2_get (insn))
    {
    case 52176:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_0;
      break;
    case 52177:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_2;
      break;
    case 52178:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_3;
      break;
    case 52179:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_0;
      break;
    case 52180:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_4;
      break;
    case 52181:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_1;
      break;
    case 52182:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_2;
      break;
    case 52183:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_3;
      break;
    }
  if (Field_dsp340050b49a6c_fld2264gp_slot2_Slot_gp_slot2_get (insn) == 13046 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2302_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_5;
  if (Field_dsp340050b49a6c_fld2266gp_slot2_Slot_gp_slot2_get (insn) == 13047 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2302_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2267gp_slot2_Slot_gp_slot2_get (insn) == 1640 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_0;
  if (Field_dsp340050b49a6c_fld2268gp_slot2_Slot_gp_slot2_get (insn) == 1641 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_1;
  if (Field_dsp340050b49a6c_fld2269gp_slot2_Slot_gp_slot2_get (insn) == 1642 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_2;
  if (Field_dsp340050b49a6c_fld2270gp_slot2_Slot_gp_slot2_get (insn) == 1643 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_4;
  if (Field_dsp340050b49a6c_fld2271gp_slot2_Slot_gp_slot2_get (insn) == 822 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2305_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_3;
  if (Field_dsp340050b49a6c_fld2273gp_slot2_Slot_gp_slot2_get (insn) == 823 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2305_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_5;
  if (Field_dsp340050b49a6c_fld2274gp_slot2_Slot_gp_slot2_get (insn) == 103 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2056_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_PUSH2X128_PQ;
  if (Field_dsp340050b49a6c_fld2275gp_slot2_Slot_gp_slot2_get (insn) == 52 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld2277gp_slot2_Slot_gp_slot2_get (insn) == 106 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_1;
  if (Field_dsp340050b49a6c_fld2278gp_slot2_Slot_gp_slot2_get (insn) == 107 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_3;
  if (Field_dsp340050b49a6c_fld2279gp_slot2_Slot_gp_slot2_get (insn) == 108 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_2;
  if (Field_dsp340050b49a6c_fld2280gp_slot2_Slot_gp_slot2_get (insn) == 109 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld2281gp_slot2_Slot_gp_slot2_get (insn) == 220 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld2282gp_slot2_Slot_gp_slot2_get (insn) == 221 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld2283gp_slot2_Slot_gp_slot2_get (insn) == 222 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld2284gp_slot2_Slot_gp_slot2_get (insn) == 3343 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2286gp_slot2_Slot_gp_slot2_get (insn) == 3359 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2287gp_slot2_Slot_gp_slot2_get (insn) == 3375 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2288gp_slot2_Slot_gp_slot2_get (insn) == 3391 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld2289gp_slot2_Slot_gp_slot2_get (insn) == 3407 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2290gp_slot2_Slot_gp_slot2_get (insn) == 3423 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2291gp_slot2_Slot_gp_slot2_get (insn) == 3439 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2292gp_slot2_Slot_gp_slot2_get (insn) == 3455 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2293gp_slot2_Slot_gp_slot2_get (insn) == 3471 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld2294gp_slot2_Slot_gp_slot2_get (insn) == 3487 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2295gp_slot2_Slot_gp_slot2_get (insn) == 3503 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2296gp_slot2_Slot_gp_slot2_get (insn) == 3519 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2297gp_slot2_Slot_gp_slot2_get (insn) == 3535 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2298gp_slot2_Slot_gp_slot2_get (insn) == 3551 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2299gp_slot2_Slot_gp_slot2_get (insn) == 3567 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2300gp_slot2_Slot_gp_slot2_get (insn) == 3583 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2301gp_slot2_Slot_gp_slot2_get (insn) == 56 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2272_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_0;
  if (Field_dsp340050b49a6c_fld2303gp_slot2_Slot_gp_slot2_get (insn) == 57 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3671gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2304gp_slot2_Slot_gp_slot2_get (insn) == 29 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3674gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld2306gp_slot2_Slot_gp_slot2_get (insn) == 120 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_QREADY;
  if (Field_dsp340050b49a6c_fld2308gp_slot2_Slot_gp_slot2_get (insn) == 242 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld2309gp_slot2_Slot_gp_slot2_get (insn) == 243 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SWAPB;
  if (Field_dsp340050b49a6c_fld2310gp_slot2_Slot_gp_slot2_get (insn) == 61 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3667gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MOVAR2;
  if (Field_dsp340050b49a6c_fld2312gp_slot2_Slot_gp_slot2_get (insn) == 31 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2386_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld2314gp_slot2_Slot_gp_slot2_get (insn) == 16 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ADD32;
  if (Field_dsp340050b49a6c_fld2316gp_slot2_Slot_gp_slot2_get (insn) == 17 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ADDCM;
  if (Field_dsp340050b49a6c_fld2317gp_slot2_Slot_gp_slot2_get (insn) == 18 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ADDWRP;
  if (Field_dsp340050b49a6c_fld2318gp_slot2_Slot_gp_slot2_get (insn) == 19 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ASR;
  if (Field_dsp340050b49a6c_fld2319gp_slot2_Slot_gp_slot2_get (insn) == 20 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_AND128;
  if (Field_dsp340050b49a6c_fld2320gp_slot2_Slot_gp_slot2_get (insn) == 21 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_EXTUI4;
  if (Field_dsp340050b49a6c_fld2321gp_slot2_Slot_gp_slot2_get (insn) == 22 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_LUT;
  if (Field_dsp340050b49a6c_fld2322gp_slot2_Slot_gp_slot2_get (insn) == 23 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_LUT_IEXT;
  if (Field_dsp340050b49a6c_fld2323gp_slot2_Slot_gp_slot2_get (insn) == 24 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ASL;
  if (Field_dsp340050b49a6c_fld2324gp_slot2_Slot_gp_slot2_get (insn) == 25 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_LUT_REXT;
  if (Field_dsp340050b49a6c_fld2325gp_slot2_Slot_gp_slot2_get (insn) == 26 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MAX8;
  if (Field_dsp340050b49a6c_fld2326gp_slot2_Slot_gp_slot2_get (insn) == 27 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MEAN32;
  if (Field_dsp340050b49a6c_fld2327gp_slot2_Slot_gp_slot2_get (insn) == 28 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MEAN;
  if (Field_dsp340050b49a6c_fld2328gp_slot2_Slot_gp_slot2_get (insn) == 29 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MIN8;
  if (Field_dsp340050b49a6c_fld2329gp_slot2_Slot_gp_slot2_get (insn) == 30 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_OR128;
  if (Field_dsp340050b49a6c_fld2330gp_slot2_Slot_gp_slot2_get (insn) == 31 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SUB32;
  if (Field_dsp340050b49a6c_fld2331gp_slot2_Slot_gp_slot2_get (insn) == 0 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_ASL32;
  if (Field_dsp340050b49a6c_fld2332gp_slot2_Slot_gp_slot2_get (insn) == 1 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_ASR32;
  if (Field_dsp340050b49a6c_fld2333gp_slot2_Slot_gp_slot2_get (insn) == 2 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOV2CM2PQ;
  if (Field_dsp340050b49a6c_fld2334gp_slot2_Slot_gp_slot2_get (insn) == 3 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_2;
  if (Field_dsp340050b49a6c_fld2335gp_slot2_Slot_gp_slot2_get (insn) == 4 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_0;
  if (Field_dsp340050b49a6c_fld2336gp_slot2_Slot_gp_slot2_get (insn) == 5 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_3;
  if (Field_dsp340050b49a6c_fld2337gp_slot2_Slot_gp_slot2_get (insn) == 6 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_4;
  if (Field_dsp340050b49a6c_fld2338gp_slot2_Slot_gp_slot2_get (insn) == 7 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_5;
  if (Field_dsp340050b49a6c_fld2339gp_slot2_Slot_gp_slot2_get (insn) == 8 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_1;
  if (Field_dsp340050b49a6c_fld2340gp_slot2_Slot_gp_slot2_get (insn) == 9 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_6;
  if (Field_dsp340050b49a6c_fld2341gp_slot2_Slot_gp_slot2_get (insn) == 10 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_7;
  if (Field_dsp340050b49a6c_fld2342gp_slot2_Slot_gp_slot2_get (insn) == 11 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND_1;
  if (Field_dsp340050b49a6c_fld2343gp_slot2_Slot_gp_slot2_get (insn) == 12 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND_0;
  if (Field_dsp340050b49a6c_fld2344gp_slot2_Slot_gp_slot2_get (insn) == 13 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND_2;
  if (Field_dsp340050b49a6c_fld2345gp_slot2_Slot_gp_slot2_get (insn) == 14 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND_3;
  if (Field_dsp340050b49a6c_fld2346gp_slot2_Slot_gp_slot2_get (insn) == 15 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCND_4;
  if (Field_dsp340050b49a6c_fld2347gp_slot2_Slot_gp_slot2_get (insn) == 16 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_ASLM;
  if (Field_dsp340050b49a6c_fld2348gp_slot2_Slot_gp_slot2_get (insn) == 17 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_ASRM;
  if (Field_dsp340050b49a6c_fld2349gp_slot2_Slot_gp_slot2_get (insn) == 18 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_CMP8;
  if (Field_dsp340050b49a6c_fld2350gp_slot2_Slot_gp_slot2_get (insn) == 19 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_LSLM;
  if (Field_dsp340050b49a6c_fld2351gp_slot2_Slot_gp_slot2_get (insn) == 20 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_CMP_I;
  if (Field_dsp340050b49a6c_fld2352gp_slot2_Slot_gp_slot2_get (insn) == 21 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_LSRM;
  if (Field_dsp340050b49a6c_fld2353gp_slot2_Slot_gp_slot2_get (insn) == 262 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_ABS8;
  if (Field_dsp340050b49a6c_fld2354gp_slot2_Slot_gp_slot2_get (insn) == 263 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2355gp_slot2_Slot_gp_slot2_get (insn) == 278 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_LUT_PHASOR;
  if (Field_dsp340050b49a6c_fld2356gp_slot2_Slot_gp_slot2_get (insn) == 279 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_NOT128;
  if (Field_dsp340050b49a6c_fld2357gp_slot2_Slot_gp_slot2_get (insn) == 294 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2358gp_slot2_Slot_gp_slot2_get (insn) == 295 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1)
    return OPCODE_TRANS;
  if (Field_dsp340050b49a6c_fld2359gp_slot2_Slot_gp_slot2_get (insn) == 2102 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2384_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2361gp_slot2_Slot_gp_slot2_get (insn) == 2103 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2384_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SET_HSAR;
  if (Field_dsp340050b49a6c_fld2362gp_slot2_Slot_gp_slot2_get (insn) == 1179 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3663gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SET_MAX;
  if (Field_dsp340050b49a6c_fld2364gp_slot2_Slot_gp_slot2_get (insn) == 667 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3664gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SET_NCO;
  if (Field_dsp340050b49a6c_fld2366gp_slot2_Slot_gp_slot2_get (insn) == 411 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3665gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2368gp_slot2_Slot_gp_slot2_get (insn) == 43 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3670gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_NCO_UPDATE;
  if (Field_dsp340050b49a6c_fld2369gp_slot2_Slot_gp_slot2_get (insn) == 27 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3668gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2370gp_slot2_Slot_gp_slot2_get (insn) == 6 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2032_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_CMP_R;
  if (Field_dsp340050b49a6c_fld2371gp_slot2_Slot_gp_slot2_get (insn) == 11 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3675gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ADD16;
  if (Field_dsp340050b49a6c_fld2372gp_slot2_Slot_gp_slot2_get (insn) == 15 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3669gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_LUT_WRITE;
  if (Field_dsp340050b49a6c_fld2373gp_slot2_Slot_gp_slot2_get (insn) == 0 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2)
    return OPCODE_MOVCND_5;
  if (Field_dsp340050b49a6c_fld2374gp_slot2_Slot_gp_slot2_get (insn) == 1 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2)
    return OPCODE_MOVCND_6;
  if (Field_dsp340050b49a6c_fld2375gp_slot2_Slot_gp_slot2_get (insn) == 2 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2)
    return OPCODE_MOVCND_7;
  if (Field_dsp340050b49a6c_fld2376gp_slot2_Slot_gp_slot2_get (insn) == 12 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_MOVCM2PQ;
  if (Field_dsp340050b49a6c_fld2378gp_slot2_Slot_gp_slot2_get (insn) == 13 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3676gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ARGMAX8;
  if (Field_dsp340050b49a6c_fld2379gp_slot2_Slot_gp_slot2_get (insn) == 7 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3673gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld2381gp_slot2_Slot_gp_slot2_get (insn) == 1 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3666_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_PERM;
  if (Field_dsp340050b49a6c_fld2383gp_slot2_Slot_gp_slot2_get (insn) == 1 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3678gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_ASLM32;
  if (Field_dsp340050b49a6c_fld2385gp_slot2_Slot_gp_slot2_get (insn) == 8 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SUBCM;
  if (Field_dsp340050b49a6c_fld2387gp_slot2_Slot_gp_slot2_get (insn) == 9 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SUBMEAN;
  if (Field_dsp340050b49a6c_fld2388gp_slot2_Slot_gp_slot2_get (insn) == 5 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3679gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_SUBWRP;
  if (Field_dsp340050b49a6c_fld2389gp_slot2_Slot_gp_slot2_get (insn) == 3 &&
      Field_op0_s3_Slot_gp_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3680gp_slot2_Slot_gp_slot2_get (insn) == 0)
    return OPCODE_XOR128;
  if (Field_op0_s3_Slot_gp_slot2_get (insn) == 3)
    return OPCODE_EXTUI;
  switch (Field_sae_Slot_gp_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_ADD;
      break;
    case 1:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_ADDI_N;
      break;
    case 2:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_AND;
      break;
    case 3:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVGEZ;
      break;
    case 4:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_LUT_AR;
      break;
    case 5:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVLTZ;
      break;
    case 6:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVNEZ;
      break;
    case 7:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_OR;
      break;
    case 8:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_MOVEQZ;
      break;
    case 9:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_SUB;
      break;
    case 10:
      if (Field_op0_s3_Slot_gp_slot2_get (insn) == 0)
	return OPCODE_XOR;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_gp_slot0_decode (const xtensa_insnbuf insn)
{
  if (Field_dsp340050b49a6c_fld2407gp_slot0_Slot_gp_slot0_get (insn) == 0 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC2X32;
  if (Field_dsp340050b49a6c_fld2409gp_slot0_Slot_gp_slot0_get (insn) == 1 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC32_R;
  if (Field_dsp340050b49a6c_fld2410gp_slot0_Slot_gp_slot0_get (insn) == 2 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC_IH;
  if (Field_dsp340050b49a6c_fld2411gp_slot0_Slot_gp_slot0_get (insn) == 3 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC_RH;
  if (Field_dsp340050b49a6c_fld2412gp_slot0_Slot_gp_slot0_get (insn) == 10 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC_RL;
  if (Field_dsp340050b49a6c_fld2413gp_slot0_Slot_gp_slot0_get (insn) == 11 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_0;
  if (Field_dsp340050b49a6c_fld2415gp_slot0_Slot_gp_slot0_get (insn) == 27 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_1;
  if (Field_dsp340050b49a6c_fld2416gp_slot0_Slot_gp_slot0_get (insn) == 4 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC_IL;
  if (Field_dsp340050b49a6c_fld2417gp_slot0_Slot_gp_slot0_get (insn) == 5 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_2;
  switch (Field_dsp340050b49a6c_fld2418gp_slot0_Slot_gp_slot0_get (insn))
    {
    case 21:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_AR2CM_DUP;
      break;
    case 53:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_MOV_N;
      break;
    case 85:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_NSA;
      break;
    case 117:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_PUSH32;
      break;
    case 149:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_NSAU;
      break;
    }
  if (Field_dsp340050b49a6c_fld2419gp_slot0_Slot_gp_slot0_get (insn) == 362 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld2420gp_slot0_Slot_gp_slot0_get (insn) == 1452 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ANY8;
  if (Field_dsp340050b49a6c_fld2422gp_slot0_Slot_gp_slot0_get (insn) == 1453 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld2423gp_slot0_Slot_gp_slot0_get (insn) == 2908 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2424gp_slot0_Slot_gp_slot0_get (insn) == 2909 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2425gp_slot0_Slot_gp_slot0_get (insn) == 2910 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2426gp_slot0_Slot_gp_slot0_get (insn) == 2911 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2427gp_slot0_Slot_gp_slot0_get (insn) == 852 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ANY4;
  if (Field_dsp340050b49a6c_fld2429gp_slot0_Slot_gp_slot0_get (insn) == 3412 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld2430gp_slot0_Slot_gp_slot0_get (insn) == 3413 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2431gp_slot0_Slot_gp_slot0_get (insn) == 3414 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2432gp_slot0_Slot_gp_slot0_get (insn) == 3415 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2433gp_slot0_Slot_gp_slot0_get (insn) == 3416 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2434gp_slot0_Slot_gp_slot0_get (insn) == 3417 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2435gp_slot0_Slot_gp_slot0_get (insn) == 1709 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_s8_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2436gp_slot0_Slot_gp_slot0_get (insn) == 855 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3690gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2437gp_slot0_Slot_gp_slot0_get (insn) == 1960 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SSAI;
  switch (Field_dsp340050b49a6c_fld2438gp_slot0_Slot_gp_slot0_get (insn))
    {
    case 3922:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_LLR_POS;
      break;
    case 3923:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_PHASOR_OFFSET;
      break;
    case 3924:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_PERM_REG;
      break;
    case 3925:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_SCALE_REG;
      break;
    case 3926:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_SMOD_POS;
      break;
    case 3927:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_SOV;
      break;
    case 3928:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_PHASOR_N;
      break;
    case 3929:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SET_WGHT;
      break;
    case 3930:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SSA8B;
      break;
    case 3931:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SSL;
      break;
    case 3932:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SSA8L;
      break;
    case 3933:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_SSR;
      break;
    }
  if (Field_dsp340050b49a6c_fld2439gp_slot0_Slot_gp_slot0_get (insn) == 3934 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CLRAC;
  if (Field_dsp340050b49a6c_fld2440gp_slot0_Slot_gp_slot0_get (insn) == 3935 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP16LLR_1;
  if (Field_dsp340050b49a6c_fld2441gp_slot0_Slot_gp_slot0_get (insn) == 3 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3693gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_3;
  switch (Field_dsp340050b49a6c_fld2443gp_slot0_Slot_gp_slot0_get (insn))
    {
    case 3:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_AR2CM_LN;
      break;
    case 7:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
	return OPCODE_AR2CM_LN_I;
      break;
    }
  if (Field_dsp340050b49a6c_fld2444gp_slot0_Slot_gp_slot0_get (insn) == 7 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2067_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_AR2CM_LN_R;
  switch (Field_dsp340050b49a6c_fld2445_Slot_gp_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_SAC2X32;
      break;
    case 1:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_SAC32_R;
      break;
    case 2:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_SCM_PINC;
      break;
    case 3:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_SRAI;
      break;
    case 4:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_SCM_U;
      break;
    case 8:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_SLLI;
      break;
    }
  if (Field_dsp340050b49a6c_fld2447gp_slot0_Slot_gp_slot0_get (insn) == 7 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3688gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_BEQZ_N;
  if (Field_dsp340050b49a6c_fld2448_Slot_gp_slot0_get (insn) == 7 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3703gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ASLACM;
  if (Field_dsp340050b49a6c_fld2449gp_slot0_Slot_gp_slot0_get (insn) == 7 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3689gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_BNEZ_N;
  if (Field_dsp340050b49a6c_fld2451gp_slot0_Slot_gp_slot0_get (insn) == 8 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LCM_PINC;
  if (Field_dsp340050b49a6c_fld2452gp_slot0_Slot_gp_slot0_get (insn) == 9 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LCM_U;
  if (Field_dsp340050b49a6c_fld2453gp_slot0_Slot_gp_slot0_get (insn) == 18 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_LCM;
  if (Field_dsp340050b49a6c_fld2454gp_slot0_Slot_gp_slot0_get (insn) == 259 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld2455gp_slot0_Slot_gp_slot0_get (insn) == 267 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2456gp_slot0_Slot_gp_slot0_get (insn) == 291 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld2457gp_slot0_Slot_gp_slot0_get (insn) == 299 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2458gp_slot0_Slot_gp_slot0_get (insn) == 323 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld2459gp_slot0_Slot_gp_slot0_get (insn) == 331 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld2460gp_slot0_Slot_gp_slot0_get (insn) == 355 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SRL;
  if (Field_dsp340050b49a6c_fld2461gp_slot0_Slot_gp_slot0_get (insn) == 715 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CLB_C;
  switch (Field_dsp340050b49a6c_fld2462_Slot_gp_slot0_get (insn))
    {
    case 21:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_EXT;
      break;
    case 23:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_EXT_R;
      break;
    case 33:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_MOVI_N;
      break;
    case 35:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_PUSH128_M;
      break;
    case 46:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_0;
      break;
    case 47:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_1;
      break;
    case 50:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_2;
      break;
    case 52:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_3;
      break;
    }
  if (Field_dsp340050b49a6c_fld2463gp_slot0_Slot_gp_slot0_get (insn) == 731 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CLB_R;
  if (Field_dsp340050b49a6c_fld2464gp_slot0_Slot_gp_slot0_get (insn) == 387 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld2465gp_slot0_Slot_gp_slot0_get (insn) == 779 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MINCLB_C;
  if (Field_dsp340050b49a6c_fld2466gp_slot0_Slot_gp_slot0_get (insn) == 795 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SMINCLB_R;
  if (Field_dsp340050b49a6c_fld2467gp_slot0_Slot_gp_slot0_get (insn) == 835 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MINCLB_R;
  if (Field_dsp340050b49a6c_fld2468gp_slot0_Slot_gp_slot0_get (insn) == 13571 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ADDAC_I2R;
  if (Field_dsp340050b49a6c_fld2470gp_slot0_Slot_gp_slot0_get (insn) == 13587 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ADDAC_R2I;
  if (Field_dsp340050b49a6c_fld2471gp_slot0_Slot_gp_slot0_get (insn) == 13603 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_REDAC;
  if (Field_dsp340050b49a6c_fld2472gp_slot0_Slot_gp_slot0_get (insn) == 13619 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_REDACS;
  if (Field_dsp340050b49a6c_fld2473gp_slot0_Slot_gp_slot0_get (insn) == 13635 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_REDAC2;
  if (Field_dsp340050b49a6c_fld2474gp_slot0_Slot_gp_slot0_get (insn) == 13651 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SUBAC_I2R;
  if (Field_dsp340050b49a6c_fld2475gp_slot0_Slot_gp_slot0_get (insn) == 6835 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SUBAC_R2I;
  if (Field_dsp340050b49a6c_fld2477gp_slot0_Slot_gp_slot0_get (insn) == 1715 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3705gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_REDAC4;
  if (Field_dsp340050b49a6c_fld2479gp_slot0_Slot_gp_slot0_get (insn) == 6667 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_L;
  if (Field_dsp340050b49a6c_fld2480gp_slot0_Slot_gp_slot0_get (insn) == 6683 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_N;
  if (Field_dsp340050b49a6c_fld2481gp_slot0_Slot_gp_slot0_get (insn) == 6699 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_LLR_POS;
  if (Field_dsp340050b49a6c_fld2482gp_slot0_Slot_gp_slot0_get (insn) == 6715 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_PHASOR_OFFSET;
  if (Field_dsp340050b49a6c_fld2483gp_slot0_Slot_gp_slot0_get (insn) == 6731 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_PERM_REG;
  if (Field_dsp340050b49a6c_fld2484gp_slot0_Slot_gp_slot0_get (insn) == 6747 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld2485gp_slot0_Slot_gp_slot0_get (insn) == 6763 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld2486gp_slot0_Slot_gp_slot0_get (insn) == 6779 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_SOV;
  if (Field_dsp340050b49a6c_fld2487gp_slot0_Slot_gp_slot0_get (insn) == 6795 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_PHASOR_N;
  if (Field_dsp340050b49a6c_fld2488gp_slot0_Slot_gp_slot0_get (insn) == 6811 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_GET_WGHT;
  if (Field_dsp340050b49a6c_fld2489gp_slot0_Slot_gp_slot0_get (insn) == 6827 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2490gp_slot0_Slot_gp_slot0_get (insn) == 6843 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2491gp_slot0_Slot_gp_slot0_get (insn) == 6859 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2492gp_slot0_Slot_gp_slot0_get (insn) == 6875 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2493gp_slot0_Slot_gp_slot0_get (insn) == 6891 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2494gp_slot0_Slot_gp_slot0_get (insn) == 6907 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld2495gp_slot0_Slot_gp_slot0_get (insn) == 115 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3706gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SMINCLB_C;
  if (Field_dsp340050b49a6c_fld2496gp_slot0_Slot_gp_slot0_get (insn) == 137 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_s8_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2497gp_slot0_Slot_gp_slot0_get (insn) == 277 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT32_I;
  if (Field_dsp340050b49a6c_fld2498gp_slot0_Slot_gp_slot0_get (insn) == 285 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_2;
  if (Field_dsp340050b49a6c_fld2499gp_slot0_Slot_gp_slot0_get (insn) == 305 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT32_R;
  if (Field_dsp340050b49a6c_fld2500gp_slot0_Slot_gp_slot0_get (insn) == 309 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_3;
  if (Field_dsp340050b49a6c_fld2501gp_slot0_Slot_gp_slot0_get (insn) == 313 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_0;
  if (Field_dsp340050b49a6c_fld2502gp_slot0_Slot_gp_slot0_get (insn) == 317 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_1;
  if (Field_dsp340050b49a6c_fld2503gp_slot0_Slot_gp_slot0_get (insn) == 337 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_0;
  if (Field_dsp340050b49a6c_fld2504gp_slot0_Slot_gp_slot0_get (insn) == 341 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_2;
  if (Field_dsp340050b49a6c_fld2505gp_slot0_Slot_gp_slot0_get (insn) == 173 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2036_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_3;
  if (Field_dsp340050b49a6c_fld2506gp_slot0_Slot_gp_slot0_get (insn) == 93 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3692gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld2507gp_slot0_Slot_gp_slot0_get (insn) == 29 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3700gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_1;
  if (Field_dsp340050b49a6c_fld2508gp_slot0_Slot_gp_slot0_get (insn) == 9 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2058_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld2509gp_slot0_Slot_gp_slot0_get (insn) == 11 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2058_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN_I;
  if (Field_dsp340050b49a6c_fld2510gp_slot0_Slot_gp_slot0_get (insn) == 7 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3696gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN_R;
  if (Field_dsp340050b49a6c_fld2512gp_slot0_Slot_gp_slot0_get (insn) == 6 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_SAC_IL;
  if (Field_dsp340050b49a6c_fld2514gp_slot0_Slot_gp_slot0_get (insn) == 7 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_SAC_RH;
  if (Field_dsp340050b49a6c_fld2515gp_slot0_Slot_gp_slot0_get (insn) == 10 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_SAC_RL;
  if (Field_dsp340050b49a6c_fld2516gp_slot0_Slot_gp_slot0_get (insn) == 16 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_SAC_IH;
  if (Field_dsp340050b49a6c_fld2517gp_slot0_Slot_gp_slot0_get (insn) == 204 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_POP128_2M_0;
  if (Field_dsp340050b49a6c_fld2518gp_slot0_Slot_gp_slot0_get (insn) == 205 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_POP128_2M_1;
  if (Field_dsp340050b49a6c_fld2519gp_slot0_Slot_gp_slot0_get (insn) == 206 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_POP128_2M_2;
  if (Field_dsp340050b49a6c_fld2520gp_slot0_Slot_gp_slot0_get (insn) == 207 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_POP128_2M_3;
  if (Field_dsp340050b49a6c_fld2521gp_slot0_Slot_gp_slot0_get (insn) == 212 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2530_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_ASRAC;
  if (Field_dsp340050b49a6c_fld2523gp_slot0_Slot_gp_slot0_get (insn) == 213 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3698gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MOVAC;
  if (Field_dsp340050b49a6c_fld2524gp_slot0_Slot_gp_slot0_get (insn) == 107 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3699gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SWAPAC_RI;
  if (Field_dsp340050b49a6c_fld2526gp_slot0_Slot_gp_slot0_get (insn) == 432 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld2527gp_slot0_Slot_gp_slot0_get (insn) == 433 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2036_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MOV2AC32_I;
  if (Field_dsp340050b49a6c_fld2528gp_slot0_Slot_gp_slot0_get (insn) == 217 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3697gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_MOV2AC32_R;
  if (Field_dsp340050b49a6c_fld2529gp_slot0_Slot_gp_slot0_get (insn) == 109 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3702gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld2531gp_slot0_Slot_gp_slot0_get (insn) == 55 &&
      Field_op0_s5_Slot_gp_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3691gp_slot0_Slot_gp_slot0_get (insn) == 0)
    return OPCODE_SET_LLR_BUF;
  if (Field_op0_s5_Slot_gp_slot0_get (insn) == 3)
    return OPCODE_EXTUI;
  switch (Field_sae_Slot_gp_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_MOVNEZ;
      break;
    case 1:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_MOVT;
      break;
    case 2:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_OR;
      break;
    case 3:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SCM;
      break;
    case 4:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_ORB;
      break;
    case 5:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SCM_PINC_X;
      break;
    case 8:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_S32I_N;
      break;
    case 9:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SCM_XU;
      break;
    case 10:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_ADD;
      break;
    case 11:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_ADDX2;
      break;
    case 12:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_ADDI_N;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SCM_X;
      break;
    case 13:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_ADDX4;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SEXT;
      break;
    case 14:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_ADDX8;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SRC;
      break;
    case 15:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_AND;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SRLI;
      break;
    case 18:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SUB;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_ANDB;
      break;
    case 19:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_L32I_N;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SUBX8;
      break;
    case 20:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SUBX2;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_CLAMPS;
      break;
    case 21:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_XOR;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_LCM_PINC_X;
      break;
    case 22:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_XORB;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_LCM_X;
      break;
    case 23:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_LCM_XU;
      break;
    case 24:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 2)
	return OPCODE_SUBX4;
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_COMB_AR;
      break;
    case 25:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MAX;
      break;
    case 26:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MAXU;
      break;
    case 27:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MINU;
      break;
    case 28:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MIN;
      break;
    case 29:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MOVEQZ;
      break;
    case 30:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MOVGEZ;
      break;
    case 31:
      if (Field_op0_s5_Slot_gp_slot0_get (insn) == 1)
	return OPCODE_MOVLTZ;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_dot_slot0_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2068_Slot_dot_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_ADD;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 5)
	return OPCODE_SUB;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 1)
	return OPCODE_AND;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 3)
	return OPCODE_OR;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 2)
	return OPCODE_LCM_X;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 4)
	return OPCODE_SCM_X;
      break;
    case 1:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_ADDI_N;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 5)
	return OPCODE_XOR;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 1)
	return OPCODE_LCM;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 2)
	return OPCODE_LCM_XU;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 3)
	return OPCODE_SCM;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 4)
	return OPCODE_SCM_XU;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2666dot_slot0_Slot_dot_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_EXT;
      break;
    case 1:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_EXT_R;
      break;
    case 2:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_MOVI_N;
      break;
    }
  if (Field_dsp340050b49a6c_fld2667dot_slot0_Slot_dot_slot0_get (insn) == 6 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
    return OPCODE_AR2CM_LN;
  switch (Field_dsp340050b49a6c_fld2668dot_slot0_Slot_dot_slot0_get (insn))
    {
    case 16:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_2FIFO_1;
      break;
    case 17:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_2FIFO_2;
      break;
    case 18:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_2FIFO_3;
      break;
    case 19:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_R2FIFO_2;
      break;
    case 20:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_R2FIFO_0;
      break;
    case 21:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_R2FIFO_3;
      break;
    case 22:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_MOV_N;
      break;
    case 23:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_PUSH32;
      break;
    case 24:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
	return OPCODE_EXT_R2FIFO_1;
      break;
    case 27:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
	  Field_dsp340050b49a6c_fld2047_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_SMINCLB_R;
      break;
    case 28:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_AR2CM_DUP;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
	  Field_dsp340050b49a6c_fld2047_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_MINCLB_C;
      break;
    case 29:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
	  Field_t_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_SET_PERM_REG;
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_EXT32_I;
      break;
    case 30:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_EXT32_R;
      break;
    case 31:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 6)
	return OPCODE_EXT_2FIFO_0;
      break;
    }
  if (Field_dsp340050b49a6c_fld2669dot_slot0_Slot_dot_slot0_get (insn) == 0 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld2671dot_slot0_Slot_dot_slot0_get (insn) == 4 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2672dot_slot0_Slot_dot_slot0_get (insn) == 5 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld2673dot_slot0_Slot_dot_slot0_get (insn) == 6 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld2674dot_slot0_Slot_dot_slot0_get (insn) == 7 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld2675dot_slot0_Slot_dot_slot0_get (insn) == 8 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2676dot_slot0_Slot_dot_slot0_get (insn) == 9 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2677dot_slot0_Slot_dot_slot0_get (insn) == 10 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld2678dot_slot0_Slot_dot_slot0_get (insn) == 11 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2679dot_slot0_Slot_dot_slot0_get (insn) == 27 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2680dot_slot0_Slot_dot_slot0_get (insn) == 43 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2681dot_slot0_Slot_dot_slot0_get (insn) == 59 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2682dot_slot0_Slot_dot_slot0_get (insn) == 75 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2683dot_slot0_Slot_dot_slot0_get (insn) == 91 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2684dot_slot0_Slot_dot_slot0_get (insn) == 107 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2685dot_slot0_Slot_dot_slot0_get (insn) == 123 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2686dot_slot0_Slot_dot_slot0_get (insn) == 43 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3736_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2688dot_slot0_Slot_dot_slot0_get (insn) == 59 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3735dot_slot0_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2689dot_slot0_Slot_dot_slot0_get (insn) == 12 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_r_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2690dot_slot0_Slot_dot_slot0_get (insn) == 13 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_GET_PERM_REG;
  if (Field_dsp340050b49a6c_fld2692dot_slot0_Slot_dot_slot0_get (insn) == 29 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2693dot_slot0_Slot_dot_slot0_get (insn) == 29 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld2705dot_slot0_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2695dot_slot0_Slot_dot_slot0_get (insn) == 29 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3741dot_slot0_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2697dot_slot0_Slot_dot_slot0_get (insn) == 29 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3738dot_slot0_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2699dot_slot0_Slot_dot_slot0_get (insn) == 7 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3740dot_slot0_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2700dot_slot0_Slot_dot_slot0_get (insn) == 50 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_CLB_C;
  if (Field_dsp340050b49a6c_fld2701dot_slot0_Slot_dot_slot0_get (insn) == 51 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_MINCLB_R;
  if (Field_dsp340050b49a6c_fld2702dot_slot0_Slot_dot_slot0_get (insn) == 52 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_CLB_R;
  if (Field_dsp340050b49a6c_fld2703dot_slot0_Slot_dot_slot0_get (insn) == 53 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7)
    return OPCODE_SMINCLB_C;
  if (Field_dsp340050b49a6c_fld2704dot_slot0_Slot_dot_slot0_get (insn) == 15 &&
      Field_op0_s8_Slot_dot_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3739dot_slot0_Slot_dot_slot0_get (insn) == 0)
    return OPCODE_CLRAC;
  switch (Field_dsp340050b49a6c_fld2705dot_slot0_Slot_dot_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 8 &&
	  Field_dsp340050b49a6c_fld3737dot_slot0_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_SLL;
      break;
    case 1:
      if (Field_op0_s8_Slot_dot_slot0_get (insn) == 8 &&
	  Field_dsp340050b49a6c_fld3742dot_slot0_Slot_dot_slot0_get (insn) == 0)
	return OPCODE_PUSH128;
      break;
    }
  switch (Field_op0_s8_Slot_dot_slot0_get (insn))
    {
    case 9:
      return OPCODE_LCM_U;
    case 10:
      return OPCODE_SCM_U;
    case 11:
      return OPCODE_SLLI;
    case 12:
      return OPCODE_SRAI;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_pq_slot2_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2706pq_slot2_Slot_pq_slot2_get (insn))
    {
    case 44:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_AR2CM_LN;
      break;
    case 47:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld3752pq_slot2_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_CLRTIEP;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2707pq_slot2_Slot_pq_slot2_get (insn))
    {
    case 180:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_ADDAR2;
      break;
    case 181:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_PUSH128_PQ;
      break;
    case 182:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_PUSH32;
      break;
    case 183:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_SET_EXT_REGS;
      break;
    case 184:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOV_N;
      break;
    case 185:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_SUBARX;
      break;
    case 186:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_WRTIEP;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2708pq_slot2_Slot_pq_slot2_get (insn))
    {
    case 1496:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_SETTIEP;
      break;
    case 1497:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_WRTBSIGQM;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2709pq_slot2_Slot_pq_slot2_get (insn))
    {
    case 2996:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_AR2SAR_DUP;
      break;
    case 2997:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_WRTSIGQ;
      break;
    case 2999:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
	  Field_s_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_5;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2710pq_slot2_Slot_pq_slot2_get (insn))
    {
    case 47968:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_0;
      break;
    case 47969:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_1;
      break;
    case 47970:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_2;
      break;
    case 47971:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_0;
      break;
    case 47972:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_3;
      break;
    case 47973:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_1;
      break;
    case 47974:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_2;
      break;
    case 47975:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_3;
      break;
    }
  if (Field_dsp340050b49a6c_fld2711pq_slot2_Slot_pq_slot2_get (insn) == 11994 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3745pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_4;
  if (Field_dsp340050b49a6c_fld2713pq_slot2_Slot_pq_slot2_get (insn) == 11995 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3745pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2714pq_slot2_Slot_pq_slot2_get (insn) == 375 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQ;
  if (Field_dsp340050b49a6c_fld2715pq_slot2_Slot_pq_slot2_get (insn) == 52 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld2717pq_slot2_Slot_pq_slot2_get (insn) == 212 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld2718pq_slot2_Slot_pq_slot2_get (insn) == 213 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld2719pq_slot2_Slot_pq_slot2_get (insn) == 3334 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2721pq_slot2_Slot_pq_slot2_get (insn) == 3335 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2722pq_slot2_Slot_pq_slot2_get (insn) == 3350 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2723pq_slot2_Slot_pq_slot2_get (insn) == 3351 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2724pq_slot2_Slot_pq_slot2_get (insn) == 3366 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2725pq_slot2_Slot_pq_slot2_get (insn) == 3367 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2726pq_slot2_Slot_pq_slot2_get (insn) == 3382 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2727pq_slot2_Slot_pq_slot2_get (insn) == 3383 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2728pq_slot2_Slot_pq_slot2_get (insn) == 3398 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld2729pq_slot2_Slot_pq_slot2_get (insn) == 3399 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2730pq_slot2_Slot_pq_slot2_get (insn) == 3414 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2731pq_slot2_Slot_pq_slot2_get (insn) == 3415 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2732pq_slot2_Slot_pq_slot2_get (insn) == 3430 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2733pq_slot2_Slot_pq_slot2_get (insn) == 3431 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2734pq_slot2_Slot_pq_slot2_get (insn) == 3446 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2735pq_slot2_Slot_pq_slot2_get (insn) == 3447 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2736pq_slot2_Slot_pq_slot2_get (insn) == 438 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld2738pq_slot2_Slot_pq_slot2_get (insn) == 439 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld2739pq_slot2_Slot_pq_slot2_get (insn) == 27 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3744pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld2741pq_slot2_Slot_pq_slot2_get (insn) == 112 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld2742pq_slot2_Slot_pq_slot2_get (insn) == 113 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_QREADY;
  if (Field_dsp340050b49a6c_fld2743pq_slot2_Slot_pq_slot2_get (insn) == 114 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3750pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld2746pq_slot2_Slot_pq_slot2_get (insn) == 115 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3750pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SWAPB;
  if (Field_dsp340050b49a6c_fld2747pq_slot2_Slot_pq_slot2_get (insn) == 29 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_MOVAR2;
  if (Field_dsp340050b49a6c_fld2748pq_slot2_Slot_pq_slot2_get (insn) == 16 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ADD32;
  if (Field_dsp340050b49a6c_fld2750pq_slot2_Slot_pq_slot2_get (insn) == 17 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ADDCM;
  if (Field_dsp340050b49a6c_fld2751pq_slot2_Slot_pq_slot2_get (insn) == 18 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ADDWRP;
  if (Field_dsp340050b49a6c_fld2752pq_slot2_Slot_pq_slot2_get (insn) == 19 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ASR;
  if (Field_dsp340050b49a6c_fld2753pq_slot2_Slot_pq_slot2_get (insn) == 20 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_AND128;
  if (Field_dsp340050b49a6c_fld2754pq_slot2_Slot_pq_slot2_get (insn) == 21 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_EXTUI4;
  if (Field_dsp340050b49a6c_fld2755pq_slot2_Slot_pq_slot2_get (insn) == 22 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_LUT;
  if (Field_dsp340050b49a6c_fld2756pq_slot2_Slot_pq_slot2_get (insn) == 23 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_LUT_IEXT;
  if (Field_dsp340050b49a6c_fld2757pq_slot2_Slot_pq_slot2_get (insn) == 24 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ASL;
  if (Field_dsp340050b49a6c_fld2758pq_slot2_Slot_pq_slot2_get (insn) == 25 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_LUT_REXT;
  if (Field_dsp340050b49a6c_fld2759pq_slot2_Slot_pq_slot2_get (insn) == 26 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_MAX8;
  if (Field_dsp340050b49a6c_fld2760pq_slot2_Slot_pq_slot2_get (insn) == 27 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_MEAN32;
  if (Field_dsp340050b49a6c_fld2761pq_slot2_Slot_pq_slot2_get (insn) == 28 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_MEAN;
  if (Field_dsp340050b49a6c_fld2762pq_slot2_Slot_pq_slot2_get (insn) == 29 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_MIN8;
  if (Field_dsp340050b49a6c_fld2763pq_slot2_Slot_pq_slot2_get (insn) == 30 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_OR128;
  if (Field_dsp340050b49a6c_fld2764pq_slot2_Slot_pq_slot2_get (insn) == 31 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SUB32;
  if (Field_dsp340050b49a6c_fld2765pq_slot2_Slot_pq_slot2_get (insn) == 0 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_ASL32;
  if (Field_dsp340050b49a6c_fld2766pq_slot2_Slot_pq_slot2_get (insn) == 1 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_ASR32;
  if (Field_dsp340050b49a6c_fld2767pq_slot2_Slot_pq_slot2_get (insn) == 2 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_0;
  if (Field_dsp340050b49a6c_fld2768pq_slot2_Slot_pq_slot2_get (insn) == 3 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_3;
  if (Field_dsp340050b49a6c_fld2769pq_slot2_Slot_pq_slot2_get (insn) == 4 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_1;
  if (Field_dsp340050b49a6c_fld2770pq_slot2_Slot_pq_slot2_get (insn) == 5 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_4;
  if (Field_dsp340050b49a6c_fld2771pq_slot2_Slot_pq_slot2_get (insn) == 6 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_5;
  if (Field_dsp340050b49a6c_fld2772pq_slot2_Slot_pq_slot2_get (insn) == 7 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_6;
  if (Field_dsp340050b49a6c_fld2773pq_slot2_Slot_pq_slot2_get (insn) == 8 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_2;
  if (Field_dsp340050b49a6c_fld2774pq_slot2_Slot_pq_slot2_get (insn) == 9 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_7;
  if (Field_dsp340050b49a6c_fld2775pq_slot2_Slot_pq_slot2_get (insn) == 10 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND_0;
  if (Field_dsp340050b49a6c_fld2776pq_slot2_Slot_pq_slot2_get (insn) == 11 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND_2;
  if (Field_dsp340050b49a6c_fld2777pq_slot2_Slot_pq_slot2_get (insn) == 12 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND_1;
  if (Field_dsp340050b49a6c_fld2778pq_slot2_Slot_pq_slot2_get (insn) == 13 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND_3;
  if (Field_dsp340050b49a6c_fld2779pq_slot2_Slot_pq_slot2_get (insn) == 14 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND_4;
  if (Field_dsp340050b49a6c_fld2780pq_slot2_Slot_pq_slot2_get (insn) == 15 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCND_5;
  if (Field_dsp340050b49a6c_fld2781pq_slot2_Slot_pq_slot2_get (insn) == 16 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_ASLM;
  if (Field_dsp340050b49a6c_fld2782pq_slot2_Slot_pq_slot2_get (insn) == 17 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_ASRM;
  if (Field_dsp340050b49a6c_fld2783pq_slot2_Slot_pq_slot2_get (insn) == 18 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_CMP8;
  if (Field_dsp340050b49a6c_fld2784pq_slot2_Slot_pq_slot2_get (insn) == 19 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_LSLM;
  if (Field_dsp340050b49a6c_fld2785pq_slot2_Slot_pq_slot2_get (insn) == 20 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_CMP_I;
  if (Field_dsp340050b49a6c_fld2786pq_slot2_Slot_pq_slot2_get (insn) == 21 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_LSRM;
  if (Field_dsp340050b49a6c_fld2787pq_slot2_Slot_pq_slot2_get (insn) == 262 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_ABS8;
  if (Field_dsp340050b49a6c_fld2788pq_slot2_Slot_pq_slot2_get (insn) == 263 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2789pq_slot2_Slot_pq_slot2_get (insn) == 278 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_LUT_PHASOR;
  if (Field_dsp340050b49a6c_fld2790pq_slot2_Slot_pq_slot2_get (insn) == 279 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_NOT128;
  if (Field_dsp340050b49a6c_fld2791pq_slot2_Slot_pq_slot2_get (insn) == 294 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2792pq_slot2_Slot_pq_slot2_get (insn) == 295 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1)
    return OPCODE_TRANS;
  if (Field_dsp340050b49a6c_fld2793pq_slot2_Slot_pq_slot2_get (insn) == 2102 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2795pq_slot2_Slot_pq_slot2_get (insn) == 2103 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SET_HSAR;
  if (Field_dsp340050b49a6c_fld2796pq_slot2_Slot_pq_slot2_get (insn) == 1179 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3746pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SET_MAX;
  if (Field_dsp340050b49a6c_fld2798pq_slot2_Slot_pq_slot2_get (insn) == 667 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3747pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SET_NCO;
  if (Field_dsp340050b49a6c_fld2801pq_slot2_Slot_pq_slot2_get (insn) == 411 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3749pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2803pq_slot2_Slot_pq_slot2_get (insn) == 43 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3754pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_NCO_UPDATE;
  if (Field_dsp340050b49a6c_fld2805pq_slot2_Slot_pq_slot2_get (insn) == 27 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3751pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2806pq_slot2_Slot_pq_slot2_get (insn) == 12 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2025_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_CMP_R;
  if (Field_dsp340050b49a6c_fld2807pq_slot2_Slot_pq_slot2_get (insn) == 13 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3756pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld2808pq_slot2_Slot_pq_slot2_get (insn) == 11 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3757pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ADD16;
  if (Field_dsp340050b49a6c_fld2809pq_slot2_Slot_pq_slot2_get (insn) == 15 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3753pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_LUT_WRITE;
  if (Field_dsp340050b49a6c_fld2810pq_slot2_Slot_pq_slot2_get (insn) == 0 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2)
    return OPCODE_MOVCND_6;
  if (Field_dsp340050b49a6c_fld2811pq_slot2_Slot_pq_slot2_get (insn) == 1 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2)
    return OPCODE_MOVCND_7;
  if (Field_dsp340050b49a6c_fld2812pq_slot2_Slot_pq_slot2_get (insn) == 1 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3748_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_PERM;
  if (Field_dsp340050b49a6c_fld2814pq_slot2_Slot_pq_slot2_get (insn) == 1 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3758pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ASLM32;
  if (Field_dsp340050b49a6c_fld2816pq_slot2_Slot_pq_slot2_get (insn) == 1 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_imm7_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_ARGMAX8;
  if (Field_dsp340050b49a6c_fld2818pq_slot2_Slot_pq_slot2_get (insn) == 8 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2737_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SUBCM;
  if (Field_dsp340050b49a6c_fld2820pq_slot2_Slot_pq_slot2_get (insn) == 9 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2737_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SUBMEAN;
  if (Field_dsp340050b49a6c_fld2821pq_slot2_Slot_pq_slot2_get (insn) == 5 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3759pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_SUBWRP;
  if (Field_dsp340050b49a6c_fld2823pq_slot2_Slot_pq_slot2_get (insn) == 3 &&
      Field_op0_s9_Slot_pq_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3760pq_slot2_Slot_pq_slot2_get (insn) == 0)
    return OPCODE_XOR128;
  if (Field_op0_s9_Slot_pq_slot2_get (insn) == 3)
    return OPCODE_EXTUI;
  switch (Field_sae_Slot_pq_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_ADD;
      break;
    case 1:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_ADDI_N;
      break;
    case 2:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_AND;
      break;
    case 3:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVGEZ;
      break;
    case 4:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_LUT_AR;
      break;
    case 5:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVLTZ;
      break;
    case 6:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVNEZ;
      break;
    case 7:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_OR;
      break;
    case 8:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVEQZ;
      break;
    case 9:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_SUB;
      break;
    case 10:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_XOR;
      break;
    case 12:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_MOVI_N;
      break;
    case 15:
      if (Field_op0_s9_Slot_pq_slot2_get (insn) == 0 &&
	  Field_t_Slot_pq_slot2_get (insn) == 0)
	return OPCODE_SLL;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_pq_slot0_decode (const xtensa_insnbuf insn)
{
  if (Field_dsp340050b49a6c_fld2827pq_slot0_Slot_pq_slot0_get (insn) == 0 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC2X32;
  if (Field_dsp340050b49a6c_fld2829pq_slot0_Slot_pq_slot0_get (insn) == 1 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC32_R;
  if (Field_dsp340050b49a6c_fld2830pq_slot0_Slot_pq_slot0_get (insn) == 2 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC_IH;
  if (Field_dsp340050b49a6c_fld2831pq_slot0_Slot_pq_slot0_get (insn) == 3 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC_RH;
  if (Field_dsp340050b49a6c_fld2832pq_slot0_Slot_pq_slot0_get (insn) == 10 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC_RL;
  if (Field_dsp340050b49a6c_fld2833pq_slot0_Slot_pq_slot0_get (insn) == 11 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_0;
  if (Field_dsp340050b49a6c_fld2835pq_slot0_Slot_pq_slot0_get (insn) == 27 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_1;
  if (Field_dsp340050b49a6c_fld2836pq_slot0_Slot_pq_slot0_get (insn) == 4 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC_IL;
  if (Field_dsp340050b49a6c_fld2837pq_slot0_Slot_pq_slot0_get (insn) == 5 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_2;
  switch (Field_dsp340050b49a6c_fld2838pq_slot0_Slot_pq_slot0_get (insn))
    {
    case 21:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_AR2CM_DUP;
      break;
    case 53:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_MOV_N;
      break;
    case 85:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_NSA;
      break;
    case 117:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_PUSH32;
      break;
    case 149:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_NSAU;
      break;
    }
  if (Field_dsp340050b49a6c_fld2839pq_slot0_Slot_pq_slot0_get (insn) == 362 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld2840pq_slot0_Slot_pq_slot0_get (insn) == 1452 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_ANY8;
  if (Field_dsp340050b49a6c_fld2842pq_slot0_Slot_pq_slot0_get (insn) == 1453 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld2843pq_slot0_Slot_pq_slot0_get (insn) == 2908 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2844pq_slot0_Slot_pq_slot0_get (insn) == 2909 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2845pq_slot0_Slot_pq_slot0_get (insn) == 2910 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2846pq_slot0_Slot_pq_slot0_get (insn) == 2911 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2847pq_slot0_Slot_pq_slot0_get (insn) == 852 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_ANY4;
  if (Field_dsp340050b49a6c_fld2849pq_slot0_Slot_pq_slot0_get (insn) == 3412 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld2850pq_slot0_Slot_pq_slot0_get (insn) == 3413 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2851pq_slot0_Slot_pq_slot0_get (insn) == 3414 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2852pq_slot0_Slot_pq_slot0_get (insn) == 3415 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2853pq_slot0_Slot_pq_slot0_get (insn) == 3416 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2854pq_slot0_Slot_pq_slot0_get (insn) == 3417 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2855pq_slot0_Slot_pq_slot0_get (insn) == 1709 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_s8_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2856pq_slot0_Slot_pq_slot0_get (insn) == 855 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3765pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2857pq_slot0_Slot_pq_slot0_get (insn) == 1960 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SSAI;
  switch (Field_dsp340050b49a6c_fld2858pq_slot0_Slot_pq_slot0_get (insn))
    {
    case 3922:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_LLR_POS;
      break;
    case 3923:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_PHASOR_OFFSET;
      break;
    case 3924:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_PERM_REG;
      break;
    case 3925:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_SCALE_REG;
      break;
    case 3926:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_SMOD_POS;
      break;
    case 3927:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_SOV;
      break;
    case 3928:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_PHASOR_N;
      break;
    case 3929:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SET_WGHT;
      break;
    case 3930:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SSA8B;
      break;
    case 3931:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SSL;
      break;
    case 3932:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SSA8L;
      break;
    case 3933:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_SSR;
      break;
    }
  if (Field_dsp340050b49a6c_fld2859pq_slot0_Slot_pq_slot0_get (insn) == 3934 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CLRAC;
  if (Field_dsp340050b49a6c_fld2860pq_slot0_Slot_pq_slot0_get (insn) == 3935 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP16LLR_1;
  if (Field_dsp340050b49a6c_fld2861pq_slot0_Slot_pq_slot0_get (insn) == 3 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3768pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LAC2X64_3;
  switch (Field_dsp340050b49a6c_fld2863pq_slot0_Slot_pq_slot0_get (insn))
    {
    case 3:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_AR2CM_LN;
      break;
    case 7:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_AR2CM_LN_I;
      break;
    }
  if (Field_dsp340050b49a6c_fld2864pq_slot0_Slot_pq_slot0_get (insn) == 7 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2069_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_AR2CM_LN_R;
  switch (Field_dsp340050b49a6c_fld2865_Slot_pq_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_SAC2X32;
      break;
    case 1:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_SAC32_R;
      break;
    case 2:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_SCM_PINC;
      break;
    case 3:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_SRAI;
      break;
    case 4:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_SCM_U;
      break;
    case 8:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_SLLI;
      break;
    }
  if (Field_dsp340050b49a6c_fld2867pq_slot0_Slot_pq_slot0_get (insn) == 7 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3763pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_BEQZ_N;
  if (Field_dsp340050b49a6c_fld2869pq_slot0_Slot_pq_slot0_get (insn) == 7 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3764pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_BNEZ_N;
  if (Field_dsp340050b49a6c_fld2871pq_slot0_Slot_pq_slot0_get (insn) == 8 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LCM_PINC;
  if (Field_dsp340050b49a6c_fld2872pq_slot0_Slot_pq_slot0_get (insn) == 9 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LCM_U;
  if (Field_dsp340050b49a6c_fld2873pq_slot0_Slot_pq_slot0_get (insn) == 18 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_LCM;
  if (Field_dsp340050b49a6c_fld2874pq_slot0_Slot_pq_slot0_get (insn) == 259 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld2875pq_slot0_Slot_pq_slot0_get (insn) == 267 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2876pq_slot0_Slot_pq_slot0_get (insn) == 291 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld2877pq_slot0_Slot_pq_slot0_get (insn) == 299 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2878pq_slot0_Slot_pq_slot0_get (insn) == 323 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld2879pq_slot0_Slot_pq_slot0_get (insn) == 331 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld2880pq_slot0_Slot_pq_slot0_get (insn) == 355 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SRL;
  if (Field_dsp340050b49a6c_fld2881pq_slot0_Slot_pq_slot0_get (insn) == 715 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CLB_C;
  switch (Field_dsp340050b49a6c_fld2882_Slot_pq_slot0_get (insn))
    {
    case 21:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_EXT;
      break;
    case 23:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_EXT_R;
      break;
    case 33:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_MOVI_N;
      break;
    case 35:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_PUSH128_M;
      break;
    case 52:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_0;
      break;
    case 53:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_2;
      break;
    case 54:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_3;
      break;
    case 56:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SAC2X64_1;
      break;
    }
  if (Field_dsp340050b49a6c_fld2883pq_slot0_Slot_pq_slot0_get (insn) == 731 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CLB_R;
  if (Field_dsp340050b49a6c_fld2884pq_slot0_Slot_pq_slot0_get (insn) == 387 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld2885pq_slot0_Slot_pq_slot0_get (insn) == 779 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MINCLB_C;
  if (Field_dsp340050b49a6c_fld2886pq_slot0_Slot_pq_slot0_get (insn) == 795 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SMINCLB_R;
  if (Field_dsp340050b49a6c_fld2887pq_slot0_Slot_pq_slot0_get (insn) == 835 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MINCLB_R;
  if (Field_dsp340050b49a6c_fld2888pq_slot0_Slot_pq_slot0_get (insn) == 13571 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_ADDAC_I2R;
  if (Field_dsp340050b49a6c_fld2890pq_slot0_Slot_pq_slot0_get (insn) == 13587 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_ADDAC_R2I;
  if (Field_dsp340050b49a6c_fld2891pq_slot0_Slot_pq_slot0_get (insn) == 13603 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_REDAC;
  if (Field_dsp340050b49a6c_fld2892pq_slot0_Slot_pq_slot0_get (insn) == 13619 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_REDACS;
  if (Field_dsp340050b49a6c_fld2893pq_slot0_Slot_pq_slot0_get (insn) == 13635 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_REDAC2;
  if (Field_dsp340050b49a6c_fld2894pq_slot0_Slot_pq_slot0_get (insn) == 13651 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SUBAC_I2R;
  if (Field_dsp340050b49a6c_fld2895pq_slot0_Slot_pq_slot0_get (insn) == 6835 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SUBAC_R2I;
  if (Field_dsp340050b49a6c_fld2897pq_slot0_Slot_pq_slot0_get (insn) == 1715 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3779pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_REDAC4;
  if (Field_dsp340050b49a6c_fld2899pq_slot0_Slot_pq_slot0_get (insn) == 6667 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_L;
  if (Field_dsp340050b49a6c_fld2900pq_slot0_Slot_pq_slot0_get (insn) == 6683 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_N;
  if (Field_dsp340050b49a6c_fld2901pq_slot0_Slot_pq_slot0_get (insn) == 6699 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_LLR_POS;
  if (Field_dsp340050b49a6c_fld2902pq_slot0_Slot_pq_slot0_get (insn) == 6715 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_PHASOR_OFFSET;
  if (Field_dsp340050b49a6c_fld2903pq_slot0_Slot_pq_slot0_get (insn) == 6731 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_PERM_REG;
  if (Field_dsp340050b49a6c_fld2904pq_slot0_Slot_pq_slot0_get (insn) == 6747 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld2905pq_slot0_Slot_pq_slot0_get (insn) == 6763 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld2906pq_slot0_Slot_pq_slot0_get (insn) == 6779 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_SOV;
  if (Field_dsp340050b49a6c_fld2907pq_slot0_Slot_pq_slot0_get (insn) == 6795 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_PHASOR_N;
  if (Field_dsp340050b49a6c_fld2908pq_slot0_Slot_pq_slot0_get (insn) == 6811 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_GET_WGHT;
  if (Field_dsp340050b49a6c_fld2909pq_slot0_Slot_pq_slot0_get (insn) == 6827 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2910pq_slot0_Slot_pq_slot0_get (insn) == 6843 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2911pq_slot0_Slot_pq_slot0_get (insn) == 6859 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2912pq_slot0_Slot_pq_slot0_get (insn) == 6875 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2913pq_slot0_Slot_pq_slot0_get (insn) == 6891 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2914pq_slot0_Slot_pq_slot0_get (insn) == 6907 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld2915pq_slot0_Slot_pq_slot0_get (insn) == 115 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3780pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SMINCLB_C;
  if (Field_dsp340050b49a6c_fld2916pq_slot0_Slot_pq_slot0_get (insn) == 137 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_s8_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2917pq_slot0_Slot_pq_slot0_get (insn) == 277 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT32_I;
  if (Field_dsp340050b49a6c_fld2918pq_slot0_Slot_pq_slot0_get (insn) == 285 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_2;
  if (Field_dsp340050b49a6c_fld2919pq_slot0_Slot_pq_slot0_get (insn) == 305 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT32_R;
  if (Field_dsp340050b49a6c_fld2920pq_slot0_Slot_pq_slot0_get (insn) == 309 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_3;
  if (Field_dsp340050b49a6c_fld2921pq_slot0_Slot_pq_slot0_get (insn) == 313 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_0;
  if (Field_dsp340050b49a6c_fld2922pq_slot0_Slot_pq_slot0_get (insn) == 317 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_1;
  if (Field_dsp340050b49a6c_fld2923pq_slot0_Slot_pq_slot0_get (insn) == 337 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_0;
  if (Field_dsp340050b49a6c_fld2924pq_slot0_Slot_pq_slot0_get (insn) == 341 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_2;
  if (Field_dsp340050b49a6c_fld2925pq_slot0_Slot_pq_slot0_get (insn) == 173 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2036_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_R2FIFO_3;
  if (Field_dsp340050b49a6c_fld2926pq_slot0_Slot_pq_slot0_get (insn) == 93 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3767pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld2927pq_slot0_Slot_pq_slot0_get (insn) == 29 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3773pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_1;
  if (Field_dsp340050b49a6c_fld2928pq_slot0_Slot_pq_slot0_get (insn) == 9 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2059_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld2929pq_slot0_Slot_pq_slot0_get (insn) == 11 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2059_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN_I;
  if (Field_dsp340050b49a6c_fld2930pq_slot0_Slot_pq_slot0_get (insn) == 7 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3769pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN_R;
  if (Field_dsp340050b49a6c_fld2932pq_slot0_Slot_pq_slot0_get (insn) == 6 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
    return OPCODE_SAC_IL;
  if (Field_dsp340050b49a6c_fld2934pq_slot0_Slot_pq_slot0_get (insn) == 7 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
    return OPCODE_SAC_RH;
  if (Field_dsp340050b49a6c_fld2935pq_slot0_Slot_pq_slot0_get (insn) == 10 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
    return OPCODE_SAC_RL;
  if (Field_dsp340050b49a6c_fld2936pq_slot0_Slot_pq_slot0_get (insn) == 16 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
    return OPCODE_SAC_IH;
  if (Field_dsp340050b49a6c_fld2937pq_slot0_Slot_pq_slot0_get (insn) == 220 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
    return OPCODE_ASLACM;
  if (Field_dsp340050b49a6c_fld2939pq_slot0_Slot_pq_slot0_get (insn) == 442 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3772_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_ASRAC;
  if (Field_dsp340050b49a6c_fld2941pq_slot0_Slot_pq_slot0_get (insn) == 443 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3771pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SWAPAC_RI;
  if (Field_dsp340050b49a6c_fld2942pq_slot0_Slot_pq_slot0_get (insn) == 111 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3766pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MOVAC;
  if (Field_dsp340050b49a6c_fld2943pq_slot0_Slot_pq_slot0_get (insn) == 114 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3775pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_2M_0;
  if (Field_dsp340050b49a6c_fld2945pq_slot0_Slot_pq_slot0_get (insn) == 115 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3775pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_POP128_2M_3;
  if (Field_dsp340050b49a6c_fld2946pq_slot0_Slot_pq_slot0_get (insn) == 496 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld2947pq_slot0_Slot_pq_slot0_get (insn) == 497 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2036_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MOV2AC32_I;
  if (Field_dsp340050b49a6c_fld2948pq_slot0_Slot_pq_slot0_get (insn) == 249 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3770pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_MOV2AC32_R;
  if (Field_dsp340050b49a6c_fld2949pq_slot0_Slot_pq_slot0_get (insn) == 125 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3778pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld2950pq_slot0_Slot_pq_slot0_get (insn) == 63 &&
      Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3766pq_slot0_Slot_pq_slot0_get (insn) == 0)
    return OPCODE_SET_LLR_BUF;
  if (Field_op0_s11_Slot_pq_slot0_get (insn) == 3)
    return OPCODE_EXTUI;
  switch (Field_sae_Slot_pq_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_MOVNEZ;
      break;
    case 1:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_MOVT;
      break;
    case 2:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_OR;
      break;
    case 3:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SCM;
      break;
    case 4:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_ORB;
      break;
    case 5:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SCM_PINC_X;
      break;
    case 8:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_S32I_N;
      break;
    case 9:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SCM_XU;
      break;
    case 10:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_ADD;
      break;
    case 11:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_ADDX2;
      break;
    case 12:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_ADDI_N;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SCM_X;
      break;
    case 13:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_ADDX4;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SEXT;
      break;
    case 14:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_ADDX8;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SRC;
      break;
    case 15:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_AND;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SRLI;
      break;
    case 18:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_ANDB;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_STORE_P;
      break;
    case 19:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_L32I_N;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SUBX2;
      break;
    case 20:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_CLAMPS;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_STORE_Q;
      break;
    case 21:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SUBX4;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_LCM_PINC_X;
      break;
    case 22:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SUBX8;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_LCM_X;
      break;
    case 23:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_XOR;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_LCM_XU;
      break;
    case 24:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_SUB;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_COMB_AR;
      break;
    case 25:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2)
	return OPCODE_XORB;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MAX;
      break;
    case 26:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MAXU;
      break;
    case 27:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MINU;
      break;
    case 28:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MIN;
      break;
    case 29:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MOVEQZ;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3776pq_slot0_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_POP128_2M_1;
      break;
    case 30:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MOVGEZ;
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3777pq_slot0_Slot_pq_slot0_get (insn) == 0)
	return OPCODE_POP128_2M_2;
      break;
    case 31:
      if (Field_op0_s11_Slot_pq_slot0_get (insn) == 1)
	return OPCODE_MOVLTZ;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_acc2_slot0_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2047_Slot_acc2_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3795acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_NEG;
      break;
    case 1:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld2039_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_PUSH128;
      break;
    }
  if (Field_dsp340050b49a6c_fld2056_Slot_acc2_slot0_get (insn) == 0 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1)
    return OPCODE_MOVCM2PQ;
  if (Field_dsp340050b49a6c_fld2973acc2_slot0_Slot_acc2_slot0_get (insn) == 0 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_MOVI_N;
  if (Field_dsp340050b49a6c_fld2974acc2_slot0_Slot_acc2_slot0_get (insn) == 40 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_PUSH2X128_PQ;
  switch (Field_dsp340050b49a6c_fld2975acc2_slot0_Slot_acc2_slot0_get (insn))
    {
    case 82:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_2PQ_1;
      break;
    case 83:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_2PQ_4;
      break;
    case 84:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_2PQ_2;
      break;
    case 85:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_2PQ_5;
      break;
    case 88:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_2PQ_3;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2976acc2_slot0_Slot_acc2_slot0_get (insn))
    {
    case 172:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_CLRCM;
      break;
    case 173:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_0;
      break;
    case 174:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_1;
      break;
    case 175:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_2;
      break;
    case 178:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_3;
      break;
    case 179:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_01;
      break;
    case 180:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP128_4;
      break;
    case 181:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_03;
      break;
    case 182:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_21;
      break;
    case 183:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_POP2X128_2PQ_23;
      break;
    }
  if (Field_dsp340050b49a6c_fld2977acc2_slot0_Slot_acc2_slot0_get (insn) == 46 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3797_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2980acc2_slot0_Slot_acc2_slot0_get (insn) == 47 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3796acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2981acc2_slot0_Slot_acc2_slot0_get (insn) == 3 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3800acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_POP128_2PQ_0;
  if (Field_dsp340050b49a6c_fld2982acc2_slot0_Slot_acc2_slot0_get (insn) == 24 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3801acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2984acc2_slot0_Slot_acc2_slot0_get (insn) == 25 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3801acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2985acc2_slot0_Slot_acc2_slot0_get (insn) == 13 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3802acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2987acc2_slot0_Slot_acc2_slot0_get (insn) == 7 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3803acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2989acc2_slot0_Slot_acc2_slot0_get (insn) == 1 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3798acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2990acc2_slot0_Slot_acc2_slot0_get (insn) == 1 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3799acc2_slot0_Slot_acc2_slot0_get (insn) == 0)
    return OPCODE_NEGCM;
  switch (Field_op0_s14_Slot_acc2_slot0_get (insn))
    {
    case 3:
      return OPCODE_ADD;
    case 4:
      return OPCODE_ADDI_N;
    case 5:
      return OPCODE_LCM_X;
    case 6:
      return OPCODE_LCM_XU;
    case 7:
      return OPCODE_SCM_X;
    case 8:
      return OPCODE_SCM_XU;
    case 9:
      return OPCODE_SUB;
    }
  switch (Field_r_Slot_acc2_slot0_get (insn))
    {
    case 8:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_MOV_N;
      break;
    case 9:
      if (Field_op0_s14_Slot_acc2_slot0_get (insn) == 0)
	return OPCODE_PUSH128_PQ;
      break;
    }
  if (Field_t_Slot_acc2_slot0_get (insn) == 2 &&
      Field_op0_s14_Slot_acc2_slot0_get (insn) == 1)
    return OPCODE_CONJ;
  return XTENSA_UNDEFINED;
}

static int
Slot_smod_slot0_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld3119smod_slot0_Slot_smod_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_LCM_PINC;
      break;
    case 1:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_LCM_U;
      break;
    case 2:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_SCM_PINC;
      break;
    case 3:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_SRAI;
      break;
    case 4:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_SCM_U;
      break;
    case 8:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_SLLI;
      break;
    case 13:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld3841smod_slot0_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_PUSH128;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3120smod_slot0_Slot_smod_slot0_get (insn))
    {
    case 76:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_AR2CM_LN;
      break;
    case 77:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_AR2CM_LN_I;
      break;
    case 78:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_AR2CM_LN_R;
      break;
    case 79:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_BEQZ_N;
      break;
    case 82:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_BNEZ_N;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3121smod_slot0_Slot_smod_slot0_get (insn))
    {
    case 40:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_MOVI_N;
      break;
    case 47:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2049_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_POP128_2M_3;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3122smod_slot0_Slot_smod_slot0_get (insn))
    {
    case 332:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_AR2CM_DUP;
      break;
    case 333:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_MOV_N;
      break;
    case 334:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_NSA;
      break;
    case 335:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_NSAU;
      break;
    case 371:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_PUSH32;
      break;
    }
  if (Field_dsp340050b49a6c_fld3123smod_slot0_Slot_smod_slot0_get (insn) == 84 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld3125smod_slot0_Slot_smod_slot0_get (insn) == 85 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_CM2AR_LN_R;
  if (Field_dsp340050b49a6c_fld3126smod_slot0_Slot_smod_slot0_get (insn) == 344 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld3127smod_slot0_Slot_smod_slot0_get (insn) == 345 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld3128smod_slot0_Slot_smod_slot0_get (insn) == 346 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld3129smod_slot0_Slot_smod_slot0_get (insn) == 347 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld3130smod_slot0_Slot_smod_slot0_get (insn) == 348 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld3131smod_slot0_Slot_smod_slot0_get (insn) == 349 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld3132smod_slot0_Slot_smod_slot0_get (insn) == 350 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld3133smod_slot0_Slot_smod_slot0_get (insn) == 351 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld3134smod_slot0_Slot_smod_slot0_get (insn) == 88 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_CM2AR_LN_I;
  if (Field_dsp340050b49a6c_fld3135smod_slot0_Slot_smod_slot0_get (insn) == 356 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_SRL;
  if (Field_dsp340050b49a6c_fld3136smod_slot0_Slot_smod_slot0_get (insn) == 5637 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld3137smod_slot0_Slot_smod_slot0_get (insn) == 5653 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld3138smod_slot0_Slot_smod_slot0_get (insn) == 5669 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld3139smod_slot0_Slot_smod_slot0_get (insn) == 5685 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld3140smod_slot0_Slot_smod_slot0_get (insn) == 5701 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld3141smod_slot0_Slot_smod_slot0_get (insn) == 5717 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld3142smod_slot0_Slot_smod_slot0_get (insn) == 5733 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld3143smod_slot0_Slot_smod_slot0_get (insn) == 5749 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld3144smod_slot0_Slot_smod_slot0_get (insn) == 5765 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld3145smod_slot0_Slot_smod_slot0_get (insn) == 5781 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld3146smod_slot0_Slot_smod_slot0_get (insn) == 2901 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3150_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld3148smod_slot0_Slot_smod_slot0_get (insn) == 1461 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_imm6_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3149smod_slot0_Slot_smod_slot0_get (insn) == 355 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_ANY8;
  if (Field_dsp340050b49a6c_fld3152smod_slot0_Slot_smod_slot0_get (insn) == 363 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_GET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld3153smod_slot0_Slot_smod_slot0_get (insn) == 89 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_GET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld3155smod_slot0_Slot_smod_slot0_get (insn) == 91 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3833smod_slot0_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_ANY4;
  if (Field_dsp340050b49a6c_fld3156smod_slot0_Slot_smod_slot0_get (insn) == 184 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_2M_0;
  if (Field_dsp340050b49a6c_fld3157smod_slot0_Slot_smod_slot0_get (insn) == 185 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_2M_1;
  if (Field_dsp340050b49a6c_fld3158smod_slot0_Slot_smod_slot0_get (insn) == 186 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP128_2M_2;
  if (Field_dsp340050b49a6c_fld3159smod_slot0_Slot_smod_slot0_get (insn) == 3000 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_SSAI;
  switch (Field_dsp340050b49a6c_fld3160smod_slot0_Slot_smod_slot0_get (insn))
    {
    case 6002:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_LLR_POS;
      break;
    case 6003:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_PHASOR_OFFSET;
      break;
    case 6004:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_PERM_REG;
      break;
    case 6005:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_SCALE_REG;
      break;
    case 6006:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_SMOD_POS;
      break;
    case 6007:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_SOV;
      break;
    case 6008:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_PHASOR_N;
      break;
    case 6009:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SET_WGHT;
      break;
    case 6010:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SSA8B;
      break;
    case 6011:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SSL;
      break;
    }
  if (Field_dsp340050b49a6c_fld3161smod_slot0_Slot_smod_slot0_get (insn) == 3006 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3832smod_slot0_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_SSA8L;
  if (Field_dsp340050b49a6c_fld3164smod_slot0_Slot_smod_slot0_get (insn) == 3007 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3832smod_slot0_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_SSR;
  if (Field_dsp340050b49a6c_fld3165smod_slot0_Slot_smod_slot0_get (insn) == 400 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld3166smod_slot0_Slot_smod_slot0_get (insn) == 3208 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_SET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld3168smod_slot0_Slot_smod_slot0_get (insn) == 6418 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_PERM_REG;
  if (Field_dsp340050b49a6c_fld3170smod_slot0_Slot_smod_slot0_get (insn) == 6419 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld3171smod_slot0_Slot_smod_slot0_get (insn) == 6420 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_PHASOR_N;
  if (Field_dsp340050b49a6c_fld3172smod_slot0_Slot_smod_slot0_get (insn) == 6421 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld3173smod_slot0_Slot_smod_slot0_get (insn) == 6422 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_SOV;
  if (Field_dsp340050b49a6c_fld3174smod_slot0_Slot_smod_slot0_get (insn) == 6423 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_WGHT;
  if (Field_dsp340050b49a6c_fld3175smod_slot0_Slot_smod_slot0_get (insn) == 6424 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_GET_PHASOR_OFFSET;
  if (Field_dsp340050b49a6c_fld3176smod_slot0_Slot_smod_slot0_get (insn) == 6425 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld3177smod_slot0_Slot_smod_slot0_get (insn) == 6426 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld3178smod_slot0_Slot_smod_slot0_get (insn) == 6427 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld3179smod_slot0_Slot_smod_slot0_get (insn) == 6428 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld3180smod_slot0_Slot_smod_slot0_get (insn) == 6429 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld3181smod_slot0_Slot_smod_slot0_get (insn) == 3215 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_s8_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld3182smod_slot0_Slot_smod_slot0_get (insn) == 201 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_sas_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_L;
  if (Field_dsp340050b49a6c_fld3184smod_slot0_Slot_smod_slot0_get (insn) == 101 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3836smod_slot0_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_GET_INTERP_EXT_N;
  if (Field_dsp340050b49a6c_fld3186smod_slot0_Slot_smod_slot0_get (insn) == 51 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3837smod_slot0_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_GET_LLR_POS;
  if (Field_dsp340050b49a6c_fld3188smod_slot0_Slot_smod_slot0_get (insn) == 7 &&
      Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3838smod_slot0_Slot_smod_slot0_get (insn) == 0)
    return OPCODE_SET_LLR_BUF;
  if (Field_op0_s17_Slot_smod_slot0_get (insn) == 2)
    return OPCODE_EXTUI;
  switch (Field_sae_Slot_smod_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_MOVLTZ;
      break;
    case 1:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_MOVNEZ;
      break;
    case 2:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_MOVT;
      break;
    case 3:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SCM;
      break;
    case 4:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_OR;
      break;
    case 5:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SCM_PINC_X;
      break;
    case 6:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SCM_X;
      break;
    case 7:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SCM_XU;
      break;
    case 8:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_ORB;
      break;
    case 9:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SEXT;
      break;
    case 10:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_ADD;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SRC;
      break;
    case 11:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SUB;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_ADDX2;
      break;
    case 12:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_ADDI_N;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SRLI;
      break;
    case 13:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_ADDX4;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SUBX2;
      break;
    case 14:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_ADDX8;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SUBX4;
      break;
    case 15:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_SUBX8;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_AND;
      break;
    case 16:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_S32I_N;
      break;
    case 17:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_XOR;
      break;
    case 18:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_ANDB;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1)
	return OPCODE_XORB;
      break;
    case 19:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_L32I_N;
      break;
    case 20:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_CLAMPS;
      break;
    case 21:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_LCM;
      break;
    case 22:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_LCM_PINC_X;
      break;
    case 23:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_LCM_X;
      break;
    case 24:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_COMB_AR;
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld3842smod_slot0_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_PUSH128_M;
      break;
    case 25:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_LCM_XU;
      break;
    case 26:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_MAX;
      break;
    case 27:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_MIN;
      break;
    case 28:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_MAXU;
      break;
    case 29:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_MINU;
      break;
    case 30:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_MOVEQZ;
      break;
    case 31:
      if (Field_op0_s17_Slot_smod_slot0_get (insn) == 0)
	return OPCODE_MOVGEZ;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_llr_slot0_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2071_Slot_llr_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 4)
	return OPCODE_S32I_N;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 0)
	return OPCODE_ADD;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 6)
	return OPCODE_SUB;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 1)
	return OPCODE_AND;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 2)
	return OPCODE_LCM;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 3)
	return OPCODE_LCM_XU;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 5)
	return OPCODE_SCM_X;
      break;
    case 1:
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 0)
	return OPCODE_ADDI_N;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 1)
	return OPCODE_L32I_N;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 3)
	return OPCODE_OR;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 6)
	return OPCODE_XOR;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
	  Field_dsp340050b49a6c_fld3881llr_slot0_Slot_llr_slot0_get (insn) == 0)
	return OPCODE_GET_LLR_BUF;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 2)
	return OPCODE_LCM_X;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 4)
	return OPCODE_SCM;
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 5)
	return OPCODE_SCM_XU;
      break;
    }
  if (Field_dsp340050b49a6c_fld3258llr_slot0_Slot_llr_slot0_get (insn) == 0 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_MOVI_N;
  if (Field_dsp340050b49a6c_fld3259llr_slot0_Slot_llr_slot0_get (insn) == 2 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_AR2CM_LN;
  switch (Field_dsp340050b49a6c_fld3260llr_slot0_Slot_llr_slot0_get (insn))
    {
    case 12:
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
	return OPCODE_AR2CM_DUP;
      break;
    case 13:
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
	return OPCODE_MOV_N;
      break;
    case 14:
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
	return OPCODE_PUSH128_PQ;
      break;
    case 15:
      if (Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
	return OPCODE_PUSH32;
      break;
    }
  if (Field_dsp340050b49a6c_fld3261llr_slot0_Slot_llr_slot0_get (insn) == 4 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld3263llr_slot0_Slot_llr_slot0_get (insn) == 20 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld3264llr_slot0_Slot_llr_slot0_get (insn) == 21 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld3265llr_slot0_Slot_llr_slot0_get (insn) == 22 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld3266llr_slot0_Slot_llr_slot0_get (insn) == 23 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld3267llr_slot0_Slot_llr_slot0_get (insn) == 24 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld3268llr_slot0_Slot_llr_slot0_get (insn) == 25 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld3269llr_slot0_Slot_llr_slot0_get (insn) == 26 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld3270llr_slot0_Slot_llr_slot0_get (insn) == 43 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_SET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld3272llr_slot0_Slot_llr_slot0_get (insn) == 283 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_LLR_POS;
  if (Field_dsp340050b49a6c_fld3274llr_slot0_Slot_llr_slot0_get (insn) == 315 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_PERM_REG;
  if (Field_dsp340050b49a6c_fld3275llr_slot0_Slot_llr_slot0_get (insn) == 347 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld3276llr_slot0_Slot_llr_slot0_get (insn) == 379 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_WGHT;
  if (Field_dsp340050b49a6c_fld3277llr_slot0_Slot_llr_slot0_get (insn) == 411 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld3278llr_slot0_Slot_llr_slot0_get (insn) == 443 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld3279llr_slot0_Slot_llr_slot0_get (insn) == 475 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld3280llr_slot0_Slot_llr_slot0_get (insn) == 507 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld3281llr_slot0_Slot_llr_slot0_get (insn) == 44 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld3282llr_slot0_Slot_llr_slot0_get (insn) == 269 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld3283llr_slot0_Slot_llr_slot0_get (insn) == 301 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld3284llr_slot0_Slot_llr_slot0_get (insn) == 173 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld2041_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld3286llr_slot0_Slot_llr_slot0_get (insn) == 109 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3879llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3288llr_slot0_Slot_llr_slot0_get (insn) == 30 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld2056_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld3289llr_slot0_Slot_llr_slot0_get (insn) == 135 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld3291llr_slot0_Slot_llr_slot0_get (insn) == 286 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld3292llr_slot0_Slot_llr_slot0_get (insn) == 287 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld3293llr_slot0_Slot_llr_slot0_get (insn) == 302 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld3294llr_slot0_Slot_llr_slot0_get (insn) == 303 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld3295llr_slot0_Slot_llr_slot0_get (insn) == 318 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld3296llr_slot0_Slot_llr_slot0_get (insn) == 319 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld3297llr_slot0_Slot_llr_slot0_get (insn) == 334 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld3298llr_slot0_Slot_llr_slot0_get (insn) == 335 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld3299llr_slot0_Slot_llr_slot0_get (insn) == 175 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld2047_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld3300llr_slot0_Slot_llr_slot0_get (insn) == 95 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3893llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld3302llr_slot0_Slot_llr_slot0_get (insn) == 31 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 7 &&
      Field_dsp340050b49a6c_fld3883llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld3303llr_slot0_Slot_llr_slot0_get (insn) == 0 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld3304llr_slot0_Slot_llr_slot0_get (insn) == 1 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_dsp340050b49a6c_fld3887llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_PERM_REG;
  if (Field_dsp340050b49a6c_fld3305llr_slot0_Slot_llr_slot0_get (insn) == 17 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_dsp340050b49a6c_fld3887llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld3306llr_slot0_Slot_llr_slot0_get (insn) == 17 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_dsp340050b49a6c_fld3890llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_WGHT;
  if (Field_dsp340050b49a6c_fld3308llr_slot0_Slot_llr_slot0_get (insn) == 17 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_dsp340050b49a6c_fld3892llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_POP16LLR_1;
  if (Field_dsp340050b49a6c_fld3310llr_slot0_Slot_llr_slot0_get (insn) == 1 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_dsp340050b49a6c_fld3888llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld3311llr_slot0_Slot_llr_slot0_get (insn) == 1 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_r_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_PUSH2X128_PQ;
  if (Field_dsp340050b49a6c_fld3312llr_slot0_Slot_llr_slot0_get (insn) == 1 &&
      Field_op0_s20_Slot_llr_slot0_get (insn) == 8 &&
      Field_dsp340050b49a6c_fld3885llr_slot0_Slot_llr_slot0_get (insn) == 0)
    return OPCODE_SET_LLR_POS;
  switch (Field_op0_s20_Slot_llr_slot0_get (insn))
    {
    case 9:
      return OPCODE_LCM_U;
    case 10:
      return OPCODE_SCM_U;
    case 11:
      return OPCODE_SLLI;
    case 12:
      return OPCODE_SRAI;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_dual_slot2_decode (const xtensa_insnbuf insn)
{
  if (Field_dsp340050b49a6c_fld2044_Slot_dual_slot2_get (insn) == 0 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
    return OPCODE_EXTUI;
  if (Field_dsp340050b49a6c_fld2056_Slot_dual_slot2_get (insn) == 5 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3923dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_CLRTIEP;
  if (Field_dsp340050b49a6c_fld3313dual_slot2_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BEQI;
  if (Field_dsp340050b49a6c_fld3314_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3904dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BNEZ;
  if (Field_dsp340050b49a6c_fld3315dual_slot2_Slot_dual_slot2_get (insn) == 17 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BLTUI;
  if (Field_dsp340050b49a6c_fld3316dual_slot2_Slot_dual_slot2_get (insn) == 33 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BNEI;
  if (Field_dsp340050b49a6c_fld3317dual_slot2_Slot_dual_slot2_get (insn) == 769 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ADD32;
  switch (Field_dsp340050b49a6c_fld3318_Slot_dual_slot2_get (insn))
    {
    case 2:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_BEQZ;
      break;
    case 4:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_BGEZ;
      break;
    case 6:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
	  Field_s_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_ASRM;
      break;
    }
  if (Field_dsp340050b49a6c_fld3319dual_slot2_Slot_dual_slot2_get (insn) == 785 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ADDCM;
  if (Field_dsp340050b49a6c_fld3320dual_slot2_Slot_dual_slot2_get (insn) == 801 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ADDWRP;
  if (Field_dsp340050b49a6c_fld3321dual_slot2_Slot_dual_slot2_get (insn) == 817 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ASR;
  if (Field_dsp340050b49a6c_fld3322dual_slot2_Slot_dual_slot2_get (insn) == 833 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_AND128;
  if (Field_dsp340050b49a6c_fld3323dual_slot2_Slot_dual_slot2_get (insn) == 849 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_EXTUI4;
  if (Field_dsp340050b49a6c_fld3324dual_slot2_Slot_dual_slot2_get (insn) == 865 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_LUT;
  if (Field_dsp340050b49a6c_fld3325dual_slot2_Slot_dual_slot2_get (insn) == 881 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_LUT_IEXT;
  if (Field_dsp340050b49a6c_fld3326dual_slot2_Slot_dual_slot2_get (insn) == 897 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ASL;
  if (Field_dsp340050b49a6c_fld3327dual_slot2_Slot_dual_slot2_get (insn) == 913 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_LUT_REXT;
  if (Field_dsp340050b49a6c_fld3328dual_slot2_Slot_dual_slot2_get (insn) == 929 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MAX8;
  if (Field_dsp340050b49a6c_fld3329dual_slot2_Slot_dual_slot2_get (insn) == 945 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MEAN32;
  if (Field_dsp340050b49a6c_fld3330dual_slot2_Slot_dual_slot2_get (insn) == 961 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MEAN;
  if (Field_dsp340050b49a6c_fld3331dual_slot2_Slot_dual_slot2_get (insn) == 977 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MIN8;
  if (Field_dsp340050b49a6c_fld3332dual_slot2_Slot_dual_slot2_get (insn) == 993 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_OR128;
  if (Field_dsp340050b49a6c_fld3333dual_slot2_Slot_dual_slot2_get (insn) == 1009 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SUB32;
  if (Field_dsp340050b49a6c_fld3334dual_slot2_Slot_dual_slot2_get (insn) == 2 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BGEI;
  if (Field_dsp340050b49a6c_fld3335dual_slot2_Slot_dual_slot2_get (insn) == 3 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SUBCM;
  if (Field_dsp340050b49a6c_fld3336dual_slot2_Slot_dual_slot2_get (insn) == 19 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_XOR128;
  if (Field_dsp340050b49a6c_fld3337dual_slot2_Slot_dual_slot2_get (insn) == 560 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ABS8;
  if (Field_dsp340050b49a6c_fld3339dual_slot2_Slot_dual_slot2_get (insn) == 561 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld3340dual_slot2_Slot_dual_slot2_get (insn) == 562 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_LUT_PHASOR;
  if (Field_dsp340050b49a6c_fld3341dual_slot2_Slot_dual_slot2_get (insn) == 563 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_NCO_UPDATE;
  if (Field_dsp340050b49a6c_fld3342dual_slot2_Slot_dual_slot2_get (insn) == 525 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_LUT_WRITE;
  if (Field_dsp340050b49a6c_fld3345dual_slot2_Slot_dual_slot2_get (insn) == 9268 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SET_ARGMAX;
  if (Field_dsp340050b49a6c_fld3347dual_slot2_Slot_dual_slot2_get (insn) == 9269 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SET_MAX;
  if (Field_dsp340050b49a6c_fld3348dual_slot2_Slot_dual_slot2_get (insn) == 9270 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SET_NCO;
  switch (Field_dsp340050b49a6c_fld3349dual_slot2_Slot_dual_slot2_get (insn))
    {
    case 132151:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_1;
      break;
    case 136247:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_2;
      break;
    case 140343:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_3;
      break;
    case 144439:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_0;
      break;
    case 148535:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_4;
      break;
    case 152631:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_1;
      break;
    case 156727:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_2;
      break;
    case 160823:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_3;
      break;
    }
  if (Field_dsp340050b49a6c_fld3350dual_slot2_Slot_dual_slot2_get (insn) == 42039 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3907dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_5;
  if (Field_dsp340050b49a6c_fld3353dual_slot2_Slot_dual_slot2_get (insn) == 46135 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3907dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3354dual_slot2_Slot_dual_slot2_get (insn) == 2381 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_s4_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld3356dual_slot2_Slot_dual_slot2_get (insn) == 1229 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3924dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_0;
  if (Field_dsp340050b49a6c_fld3358dual_slot2_Slot_dual_slot2_get (insn) == 333 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3918dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SET_HSAR;
  if (Field_dsp340050b49a6c_fld3360dual_slot2_Slot_dual_slot2_get (insn) == 568 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld3361dual_slot2_Slot_dual_slot2_get (insn) == 569 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld3362dual_slot2_Slot_dual_slot2_get (insn) == 570 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_NOT128;
  if (Field_dsp340050b49a6c_fld3363dual_slot2_Slot_dual_slot2_get (insn) == 8251 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_CLRCM;
  switch (Field_dsp340050b49a6c_fld3364_Slot_dual_slot2_get (insn))
    {
    case 52:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ADD;
      break;
    case 53:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ADDX2;
      break;
    case 54:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ADDX4;
      break;
    case 55:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ADDX8;
      break;
    case 56:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ADDI_N;
      break;
    case 57:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_AND;
      break;
    case 58:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ANDB;
      break;
    case 59:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_LUT_AR;
      break;
    case 60:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_CLAMPS;
      break;
    case 61:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MAX;
      break;
    case 62:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MAXU;
      break;
    case 63:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MIN;
      break;
    case 112:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MINU;
      break;
    case 113:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MOVEQZ;
      break;
    case 114:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MOVGEZ;
      break;
    case 115:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MOVT;
      break;
    case 116:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MOVLTZ;
      break;
    case 117:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_OR;
      break;
    case 118:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_ORB;
      break;
    case 119:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SEXT;
      break;
    case 120:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_MOVNEZ;
      break;
    case 121:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SRC;
      break;
    case 122:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SRLI;
      break;
    case 123:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SUBX2;
      break;
    case 124:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SUB;
      break;
    case 125:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SUBX4;
      break;
    case 126:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SUBX8;
      break;
    case 127:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_XOR;
      break;
    }
  if (Field_dsp340050b49a6c_fld3365dual_slot2_Slot_dual_slot2_get (insn) == 8507 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld3366dual_slot2_Slot_dual_slot2_get (insn) == 8763 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld3367dual_slot2_Slot_dual_slot2_get (insn) == 9019 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld3368dual_slot2_Slot_dual_slot2_get (insn) == 9275 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld3369dual_slot2_Slot_dual_slot2_get (insn) == 9531 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld3370dual_slot2_Slot_dual_slot2_get (insn) == 9787 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld3371dual_slot2_Slot_dual_slot2_get (insn) == 10043 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld3372dual_slot2_Slot_dual_slot2_get (insn) == 10299 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld3373dual_slot2_Slot_dual_slot2_get (insn) == 10555 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld3374dual_slot2_Slot_dual_slot2_get (insn) == 10811 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld3375dual_slot2_Slot_dual_slot2_get (insn) == 11067 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld3376dual_slot2_Slot_dual_slot2_get (insn) == 11323 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld3377dual_slot2_Slot_dual_slot2_get (insn) == 11579 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld3378dual_slot2_Slot_dual_slot2_get (insn) == 11835 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld3379dual_slot2_Slot_dual_slot2_get (insn) == 12091 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld3380dual_slot2_Slot_dual_slot2_get (insn) == 572 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_TRANS;
  if (Field_dsp340050b49a6c_fld3381dual_slot2_Slot_dual_slot2_get (insn) == 573 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld3382dual_slot2_Slot_dual_slot2_get (insn) == 287 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3928dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld3384dual_slot2_Slot_dual_slot2_get (insn) == 771 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVAR2;
  if (Field_dsp340050b49a6c_fld3385dual_slot2_Slot_dual_slot2_get (insn) == 787 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld3386dual_slot2_Slot_dual_slot2_get (insn) == 803 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SWAPB;
  switch (Field_dsp340050b49a6c_fld3387dual_slot2_Slot_dual_slot2_get (insn))
    {
    case 13059:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_SSA8L;
      break;
    case 13075:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_SSL;
      break;
    }
  if (Field_dsp340050b49a6c_fld3388dual_slot2_Slot_dual_slot2_get (insn) == 6547 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3913dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SSR;
  if (Field_dsp340050b49a6c_fld3390dual_slot2_Slot_dual_slot2_get (insn) == 3283 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2049_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQ;
  if (Field_dsp340050b49a6c_fld3392dual_slot2_Slot_dual_slot2_get (insn) == 1651 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3900_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_WRTSIGQ;
  if (Field_dsp340050b49a6c_fld3394dual_slot2_Slot_dual_slot2_get (insn) == 211 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3920dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_AR2SAR_DUP;
  if (Field_dsp340050b49a6c_fld3396dual_slot2_Slot_dual_slot2_get (insn) == 115 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3909dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SSA8B;
  if (Field_dsp340050b49a6c_fld3397dual_slot2_Slot_dual_slot2_get (insn) == 19 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3919dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld3399dual_slot2_Slot_dual_slot2_get (insn) == 19 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3931dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ARGMAX8;
  if (Field_dsp340050b49a6c_fld3401dual_slot2_Slot_dual_slot2_get (insn) == 33 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3934dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SUBMEAN;
  if (Field_dsp340050b49a6c_fld3403dual_slot2_Slot_dual_slot2_get (insn) == 41 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3922dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld3404dual_slot2_Slot_dual_slot2_get (insn) == 25 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3910dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SSAI;
  if (Field_dsp340050b49a6c_fld3406dual_slot2_Slot_dual_slot2_get (insn) == 9 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3935dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SUBWRP;
  if (Field_dsp340050b49a6c_fld3407_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3905dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVI;
  if (Field_dsp340050b49a6c_fld3408dual_slot2_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3901dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BGEUI;
  if (Field_dsp340050b49a6c_fld3410_Slot_dual_slot2_get (insn) == 3 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3929dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SETTIEP;
  if (Field_dsp340050b49a6c_fld3411dual_slot2_Slot_dual_slot2_get (insn) == 5 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3933dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ASLM32;
  if (Field_dsp340050b49a6c_fld3412dual_slot2_Slot_dual_slot2_get (insn) == 18 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3927dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_QREADY;
  if (Field_dsp340050b49a6c_fld3413dual_slot2_Slot_dual_slot2_get (insn) == 19 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3930dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQM;
  if (Field_dsp340050b49a6c_fld3414dual_slot2_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_ADDMI;
  if (Field_dsp340050b49a6c_fld3415dual_slot2_Slot_dual_slot2_get (insn) == 17 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BALL;
  if (Field_dsp340050b49a6c_fld3416dual_slot2_Slot_dual_slot2_get (insn) == 33 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BANY;
  if (Field_dsp340050b49a6c_fld3417dual_slot2_Slot_dual_slot2_get (insn) == 49 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BBC;
  if (Field_dsp340050b49a6c_fld3418dual_slot2_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BBCI;
  if (Field_dsp340050b49a6c_fld3419dual_slot2_Slot_dual_slot2_get (insn) == 18 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BBS;
  if (Field_dsp340050b49a6c_fld3420dual_slot2_Slot_dual_slot2_get (insn) == 19 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BGE;
  if (Field_dsp340050b49a6c_fld3421dual_slot2_Slot_dual_slot2_get (insn) == 34 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BEQ;
  if (Field_dsp340050b49a6c_fld3422dual_slot2_Slot_dual_slot2_get (insn) == 35 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BGEU;
  if (Field_dsp340050b49a6c_fld3423dual_slot2_Slot_dual_slot2_get (insn) == 50 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BLT;
  if (Field_dsp340050b49a6c_fld3424dual_slot2_Slot_dual_slot2_get (insn) == 51 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BLTU;
  if (Field_dsp340050b49a6c_fld3425dual_slot2_Slot_dual_slot2_get (insn) == 2 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BBSI;
  if (Field_dsp340050b49a6c_fld3426dual_slot2_Slot_dual_slot2_get (insn) == 6 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BNALL;
  if (Field_dsp340050b49a6c_fld3427dual_slot2_Slot_dual_slot2_get (insn) == 7 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_ASL32;
  if (Field_dsp340050b49a6c_fld3428dual_slot2_Slot_dual_slot2_get (insn) == 23 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_ASR32;
  if (Field_dsp340050b49a6c_fld3429dual_slot2_Slot_dual_slot2_get (insn) == 39 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_0;
  if (Field_dsp340050b49a6c_fld3430dual_slot2_Slot_dual_slot2_get (insn) == 55 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_3;
  if (Field_dsp340050b49a6c_fld3431dual_slot2_Slot_dual_slot2_get (insn) == 71 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_1;
  if (Field_dsp340050b49a6c_fld3432dual_slot2_Slot_dual_slot2_get (insn) == 87 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_4;
  if (Field_dsp340050b49a6c_fld3433dual_slot2_Slot_dual_slot2_get (insn) == 103 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_5;
  if (Field_dsp340050b49a6c_fld3434dual_slot2_Slot_dual_slot2_get (insn) == 119 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_6;
  if (Field_dsp340050b49a6c_fld3435dual_slot2_Slot_dual_slot2_get (insn) == 135 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_2;
  if (Field_dsp340050b49a6c_fld3436dual_slot2_Slot_dual_slot2_get (insn) == 151 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND8_7;
  if (Field_dsp340050b49a6c_fld3437dual_slot2_Slot_dual_slot2_get (insn) == 167 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_0;
  if (Field_dsp340050b49a6c_fld3438dual_slot2_Slot_dual_slot2_get (insn) == 183 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_2;
  if (Field_dsp340050b49a6c_fld3439dual_slot2_Slot_dual_slot2_get (insn) == 199 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_1;
  if (Field_dsp340050b49a6c_fld3440dual_slot2_Slot_dual_slot2_get (insn) == 215 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_3;
  if (Field_dsp340050b49a6c_fld3441dual_slot2_Slot_dual_slot2_get (insn) == 231 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_4;
  if (Field_dsp340050b49a6c_fld3442dual_slot2_Slot_dual_slot2_get (insn) == 247 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_5;
  if (Field_dsp340050b49a6c_fld3443dual_slot2_Slot_dual_slot2_get (insn) == 20 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BNE;
  if (Field_dsp340050b49a6c_fld3444dual_slot2_Slot_dual_slot2_get (insn) == 261 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_MOVCND_6;
  if (Field_dsp340050b49a6c_fld3445dual_slot2_Slot_dual_slot2_get (insn) == 277 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_PERM;
  if (Field_dsp340050b49a6c_fld3446dual_slot2_Slot_dual_slot2_get (insn) == 1172 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_ADD16;
  if (Field_dsp340050b49a6c_fld3448dual_slot2_Slot_dual_slot2_get (insn) == 2346 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3925dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld3450dual_slot2_Slot_dual_slot2_get (insn) == 2347 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ANY8;
  if (Field_dsp340050b49a6c_fld3451dual_slot2_Slot_dual_slot2_get (insn) == 587 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3914dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ANY4;
  switch (Field_dsp340050b49a6c_fld3453dual_slot2_Slot_dual_slot2_get (insn))
    {
    case 4869:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
	return OPCODE_NSA;
      break;
    case 4885:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
	return OPCODE_NSAU;
      break;
    case 4901:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
	return OPCODE_PUSH128_PQ;
      break;
    case 4917:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
	return OPCODE_SUBARX;
      break;
    }
  if (Field_dsp340050b49a6c_fld3454dual_slot2_Slot_dual_slot2_get (insn) == 2469 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3466_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_PUSH32;
  if (Field_dsp340050b49a6c_fld3456dual_slot2_Slot_dual_slot2_get (insn) == 2485 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3466_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_WRTIEP;
  if (Field_dsp340050b49a6c_fld3457dual_slot2_Slot_dual_slot2_get (insn) == 629 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3917dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SET_EXT_REGS;
  if (Field_dsp340050b49a6c_fld3458dual_slot2_Slot_dual_slot2_get (insn) == 85 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3919dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ADDAR2;
  if (Field_dsp340050b49a6c_fld3459dual_slot2_Slot_dual_slot2_get (insn) == 53 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3896dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOV_N;
  if (Field_dsp340050b49a6c_fld3460dual_slot2_Slot_dual_slot2_get (insn) == 11 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3921dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVCND_7;
  if (Field_dsp340050b49a6c_fld3461dual_slot2_Slot_dual_slot2_get (insn) == 9 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2049_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BNONE;
  if (Field_dsp340050b49a6c_fld3462dual_slot2_Slot_dual_slot2_get (insn) == 49 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3894dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_AR2CM_LN;
  if (Field_dsp340050b49a6c_fld3464dual_slot2_Slot_dual_slot2_get (insn) == 53 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3894dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BEQZ_N;
  if (Field_dsp340050b49a6c_fld3465dual_slot2_Slot_dual_slot2_get (insn) == 29 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3895dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BNEZ_N;
  if (Field_dsp340050b49a6c_fld3467dual_slot2_Slot_dual_slot2_get (insn) == 1 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3899dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ADDI;
  if (Field_dsp340050b49a6c_fld3468dual_slot2_Slot_dual_slot2_get (insn) == 3 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3897dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_MOVI_N;
  switch (Field_dsp340050b49a6c_fld3469dual_slot2_Slot_dual_slot2_get (insn))
    {
    case 24:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SLLI;
      break;
    case 25:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
	return OPCODE_SRAI;
      break;
    }
  if (Field_dsp340050b49a6c_fld3470dual_slot2_Slot_dual_slot2_get (insn) == 80 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
    return OPCODE_ASLM;
  if (Field_dsp340050b49a6c_fld3471dual_slot2_Slot_dual_slot2_get (insn) == 81 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
    return OPCODE_CMP8;
  if (Field_dsp340050b49a6c_fld3472dual_slot2_Slot_dual_slot2_get (insn) == 82 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
    return OPCODE_CMP_I;
  if (Field_dsp340050b49a6c_fld3473dual_slot2_Slot_dual_slot2_get (insn) == 83 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
    return OPCODE_LSRM;
  if (Field_dsp340050b49a6c_fld3474dual_slot2_Slot_dual_slot2_get (insn) == 84 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2)
    return OPCODE_CMP_R;
  if (Field_dsp340050b49a6c_fld3475dual_slot2_Slot_dual_slot2_get (insn) == 325 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3898dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld3477dual_slot2_Slot_dual_slot2_get (insn) == 341 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3898dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld3478dual_slot2_Slot_dual_slot2_get (insn) == 181 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3908dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_SRL;
  if (Field_dsp340050b49a6c_fld3479dual_slot2_Slot_dual_slot2_get (insn) == 43 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3906dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld3480dual_slot2_Slot_dual_slot2_get (insn) == 22 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_s4_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_LSLM;
  if (Field_dsp340050b49a6c_fld3481dual_slot2_Slot_dual_slot2_get (insn) == 23 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld3482dual_slot2_Slot_dual_slot2_get (insn) == 2 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3903dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_BLTZ;
  if (Field_dsp340050b49a6c_fld3484dual_slot2_Slot_dual_slot2_get (insn) == 3 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3916dual_slot2_Slot_dual_slot2_get (insn) == 0)
    return OPCODE_XORB;
  if (Field_r_Slot_dual_slot2_get (insn) == 0 &&
      Field_op0_s21_Slot_dual_slot2_get (insn) == 1)
    return OPCODE_BITFINS;
  switch (Field_t_Slot_dual_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_BITFEXT;
      break;
    case 8:
      if (Field_op0_s21_Slot_dual_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld3314_Slot_dual_slot2_get (insn) == 0)
	return OPCODE_BLTI;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_dual_slot0_decode (const xtensa_insnbuf insn)
{
  if (Field_dsp340050b49a6c_fld2056_Slot_dual_slot0_get (insn) == 3 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_imm8_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_MOVCM2PQ;
  switch (Field_dsp340050b49a6c_fld2057_Slot_dual_slot0_get (insn))
    {
    case 2842:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_AR2CM_DUP;
      break;
    case 2843:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_MOV_N;
      break;
    case 2874:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_NSA;
      break;
    case 2906:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_NSAU;
      break;
    case 2970:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_PUSH128_PQ;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3487dual_slot0_Slot_dual_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BBCI;
      break;
    case 1:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BBSI;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3488dual_slot0_Slot_dual_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_EXTUI;
      break;
    case 3:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4 &&
	  Field_sae_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_ULT_S;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3489dual_slot0_Slot_dual_slot0_get (insn))
    {
    case 2:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_BEQZ;
      break;
    case 3:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_BNEZ;
      break;
    case 4:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_BGEZ;
      break;
    case 5:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4 &&
	  Field_dsp340050b49a6c_fld2079_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_ULE_S;
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_LSI;
      break;
    case 6:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_LSIU;
      break;
    case 7:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_MOVI;
      break;
    case 8:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_BLTZ;
      break;
    case 9:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SSI;
      break;
    case 10:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SSIU;
      break;
    case 14:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2037_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_LCM_U;
      break;
    }
  if (Field_dsp340050b49a6c_fld3490dual_slot0_Slot_dual_slot0_get (insn) == 88 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC32_R;
  if (Field_dsp340050b49a6c_fld3491dual_slot0_Slot_dual_slot0_get (insn) == 177 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC_IH;
  if (Field_dsp340050b49a6c_fld3492dual_slot0_Slot_dual_slot0_get (insn) == 185 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC_RL;
  if (Field_dsp340050b49a6c_fld3493dual_slot0_Slot_dual_slot0_get (insn) == 178 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC_IL;
  if (Field_dsp340050b49a6c_fld3494dual_slot0_Slot_dual_slot0_get (insn) == 355 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC2X64_0;
  if (Field_dsp340050b49a6c_fld3496dual_slot0_Slot_dual_slot0_get (insn) == 371 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC2X64_2;
  if (Field_dsp340050b49a6c_fld3497dual_slot0_Slot_dual_slot0_get (insn) == 362 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC2X64_1;
  if (Field_dsp340050b49a6c_fld3498dual_slot0_Slot_dual_slot0_get (insn) == 363 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LAC2X64_3;
  if (Field_dsp340050b49a6c_fld3499dual_slot0_Slot_dual_slot0_get (insn) == 5750 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld3500dual_slot0_Slot_dual_slot0_get (insn) == 11502 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_ANY4;
  if (Field_dsp340050b49a6c_fld3502dual_slot0_Slot_dual_slot0_get (insn) == 23006 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_ANY8;
  if (Field_dsp340050b49a6c_fld3504dual_slot0_Slot_dual_slot0_get (insn) == 23007 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld3505dual_slot0_Slot_dual_slot0_get (insn) == 11628 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_PUSH2X128_PQ;
  switch (Field_dsp340050b49a6c_fld3506dual_slot0_Slot_dual_slot0_get (insn))
    {
    case 23258:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP128_2PQ_1;
      break;
    case 23259:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP128_2PQ_3;
      break;
    case 23260:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP128_2PQ_2;
      break;
    case 23261:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP128_2PQ_4;
      break;
    case 23262:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP128_2PQ_5;
      break;
    case 23263:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SSAI;
      break;
    case 23504:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP128_2PQ_0;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3507dual_slot0_Slot_dual_slot0_get (insn))
    {
    case 47010:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_01;
      break;
    case 47011:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_23;
      break;
    case 47012:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_03;
      break;
    case 47013:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_LLR_POS;
      break;
    case 47014:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_PERM_REG;
      break;
    case 47015:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_PHASOR_N;
      break;
    case 47016:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_21;
      break;
    case 47017:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_PHASOR_OFFSET;
      break;
    case 47018:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_SCALE_REG;
      break;
    case 47019:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_SOV;
      break;
    case 47020:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_SMOD_POS;
      break;
    case 47021:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SET_WGHT;
      break;
    case 47022:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SSA8B;
      break;
    case 47023:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SSA8L;
      break;
    }
  if (Field_dsp340050b49a6c_fld3508dual_slot0_Slot_dual_slot0_get (insn) == 47024 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld3509dual_slot0_Slot_dual_slot0_get (insn) == 47025 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld3510dual_slot0_Slot_dual_slot0_get (insn) == 47026 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld3511dual_slot0_Slot_dual_slot0_get (insn) == 47027 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld3512dual_slot0_Slot_dual_slot0_get (insn) == 47028 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld3513dual_slot0_Slot_dual_slot0_get (insn) == 47029 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld3514dual_slot0_Slot_dual_slot0_get (insn) == 47030 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld3515dual_slot0_Slot_dual_slot0_get (insn) == 47031 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld3516dual_slot0_Slot_dual_slot0_get (insn) == 47032 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld3517dual_slot0_Slot_dual_slot0_get (insn) == 47033 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld3518dual_slot0_Slot_dual_slot0_get (insn) == 23517 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_s8_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld3519dual_slot0_Slot_dual_slot0_get (insn) == 11759 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3939dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3520dual_slot0_Slot_dual_slot0_get (insn) == 5942 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2056_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SSL;
  if (Field_dsp340050b49a6c_fld3522dual_slot0_Slot_dual_slot0_get (insn) == 5943 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3950dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP16LLR_1;
  if (Field_dsp340050b49a6c_fld3523dual_slot0_Slot_dual_slot0_get (insn) == 1501 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_bbi_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SSR;
  if (Field_dsp340050b49a6c_fld3524dual_slot0_Slot_dual_slot0_get (insn) == 765 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3943dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_CLRAC;
  if (Field_dsp340050b49a6c_fld3527dual_slot0_Slot_dual_slot0_get (insn) == 45 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2049_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_LAC_RH;
  switch (Field_dsp340050b49a6c_fld3529dual_slot0_Slot_dual_slot0_get (insn))
    {
    case 707:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_AR2CM_LN;
      break;
    case 711:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_AR2CM_LN_I;
      break;
    }
  if (Field_dsp340050b49a6c_fld3530dual_slot0_Slot_dual_slot0_get (insn) == 359 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2072_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_AR2CM_LN_R;
  switch (Field_dsp340050b49a6c_fld3531_Slot_dual_slot0_get (insn))
    {
    case 120:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SAC2X32;
      break;
    case 121:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SAC32_R;
      break;
    case 122:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SCM_PINC;
      break;
    case 123:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SLLI;
      break;
    case 124:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SCM_U;
      break;
    case 125:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_SRAI;
      break;
    }
  if (Field_dsp340050b49a6c_fld3532dual_slot0_Slot_dual_slot0_get (insn) == 183 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_BEQZ_N;
  if (Field_dsp340050b49a6c_fld3533dual_slot0_Slot_dual_slot0_get (insn) == 95 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3936dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_BNEZ_N;
  if (Field_dsp340050b49a6c_fld3535dual_slot0_Slot_dual_slot0_get (insn) == 48 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2066_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_LAC2X32;
  if (Field_dsp340050b49a6c_fld3536dual_slot0_Slot_dual_slot0_get (insn) == 193 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2060_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP128_2M_0;
  if (Field_dsp340050b49a6c_fld3537dual_slot0_Slot_dual_slot0_get (insn) == 197 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2060_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP128_2M_2;
  if (Field_dsp340050b49a6c_fld3538dual_slot0_Slot_dual_slot0_get (insn) == 101 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3957dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP128_2M_3;
  if (Field_dsp340050b49a6c_fld3539dual_slot0_Slot_dual_slot0_get (insn) == 25 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3954dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP128_2M_1;
  if (Field_dsp340050b49a6c_fld3541dual_slot0_Slot_dual_slot0_get (insn) == 104 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LCM_PINC;
  if (Field_dsp340050b49a6c_fld3542dual_slot0_Slot_dual_slot0_get (insn) == 105 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LP;
  if (Field_dsp340050b49a6c_fld3543dual_slot0_Slot_dual_slot0_get (insn) == 106 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_LQ;
  if (Field_dsp340050b49a6c_fld3544dual_slot0_Slot_dual_slot0_get (insn) == 3331 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_ABS;
  if (Field_dsp340050b49a6c_fld3545dual_slot0_Slot_dual_slot0_get (insn) == 3339 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld3546dual_slot0_Slot_dual_slot0_get (insn) == 3347 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld3547dual_slot0_Slot_dual_slot0_get (insn) == 3355 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld3548dual_slot0_Slot_dual_slot0_get (insn) == 3363 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_MOV_I;
  if (Field_dsp340050b49a6c_fld3549dual_slot0_Slot_dual_slot0_get (insn) == 3371 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SRA;
  if (Field_dsp340050b49a6c_fld3550dual_slot0_Slot_dual_slot0_get (insn) == 3379 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SRL;
  if (Field_dsp340050b49a6c_fld3551dual_slot0_Slot_dual_slot0_get (insn) == 6763 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_CLB_C;
  switch (Field_dsp340050b49a6c_fld3552_Slot_dual_slot0_get (insn))
    {
    case 146:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_EXT;
      break;
    case 147:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_PUSH128_M;
      break;
    case 148:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_EXT_R;
      break;
    case 149:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SAC2X64_0;
      break;
    case 150:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SAC2X64_1;
      break;
    case 151:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SAC2X64_2;
      break;
    }
  if (Field_dsp340050b49a6c_fld3553dual_slot0_Slot_dual_slot0_get (insn) == 6779 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_CLB_R;
  if (Field_dsp340050b49a6c_fld3554dual_slot0_Slot_dual_slot0_get (insn) == 3395 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_MOV_R;
  if (Field_dsp340050b49a6c_fld3555dual_slot0_Slot_dual_slot0_get (insn) == 6795 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_MINCLB_C;
  if (Field_dsp340050b49a6c_fld3556dual_slot0_Slot_dual_slot0_get (insn) == 6811 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SMINCLB_C;
  if (Field_dsp340050b49a6c_fld3557dual_slot0_Slot_dual_slot0_get (insn) == 6819 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_MINCLB_R;
  if (Field_dsp340050b49a6c_fld3558dual_slot0_Slot_dual_slot0_get (insn) == 6835 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SMINCLB_R;
  if (Field_dsp340050b49a6c_fld3559dual_slot0_Slot_dual_slot0_get (insn) == 6827 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SET_LLR_BUF;
  if (Field_dsp340050b49a6c_fld3560dual_slot0_Slot_dual_slot0_get (insn) == 54555 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_INTERP_EXT_L;
  if (Field_dsp340050b49a6c_fld3562dual_slot0_Slot_dual_slot0_get (insn) == 54587 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_INTERP_EXT_N;
  if (Field_dsp340050b49a6c_fld3563dual_slot0_Slot_dual_slot0_get (insn) == 54619 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_LLR_POS;
  if (Field_dsp340050b49a6c_fld3564dual_slot0_Slot_dual_slot0_get (insn) == 54651 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_PHASOR_N;
  if (Field_dsp340050b49a6c_fld3565dual_slot0_Slot_dual_slot0_get (insn) == 54683 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_PERM_REG;
  if (Field_dsp340050b49a6c_fld3566dual_slot0_Slot_dual_slot0_get (insn) == 54715 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_PHASOR_OFFSET;
  if (Field_dsp340050b49a6c_fld3567dual_slot0_Slot_dual_slot0_get (insn) == 54747 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_SCALE_REG;
  if (Field_dsp340050b49a6c_fld3568dual_slot0_Slot_dual_slot0_get (insn) == 54779 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_SMOD_POS;
  if (Field_dsp340050b49a6c_fld3569dual_slot0_Slot_dual_slot0_get (insn) == 6851 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld3570dual_slot0_Slot_dual_slot0_get (insn) == 54795 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_SOV;
  if (Field_dsp340050b49a6c_fld3571dual_slot0_Slot_dual_slot0_get (insn) == 54827 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld3572dual_slot0_Slot_dual_slot0_get (insn) == 54859 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld3573dual_slot0_Slot_dual_slot0_get (insn) == 54891 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld3574dual_slot0_Slot_dual_slot0_get (insn) == 54923 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld3575dual_slot0_Slot_dual_slot0_get (insn) == 54955 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_SET_SMOD_OFFSET_TABLE;
  if (Field_dsp340050b49a6c_fld3576dual_slot0_Slot_dual_slot0_get (insn) == 109771 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_ADDAC_I2R;
  if (Field_dsp340050b49a6c_fld3577dual_slot0_Slot_dual_slot0_get (insn) == 109803 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_ADDAC_R2I;
  if (Field_dsp340050b49a6c_fld3578dual_slot0_Slot_dual_slot0_get (insn) == 110027 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_REDAC;
  if (Field_dsp340050b49a6c_fld3579dual_slot0_Slot_dual_slot0_get (insn) == 110059 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_REDAC2;
  if (Field_dsp340050b49a6c_fld3580dual_slot0_Slot_dual_slot0_get (insn) == 54803 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_GET_WGHT;
  if (Field_dsp340050b49a6c_fld3581dual_slot0_Slot_dual_slot0_get (insn) == 54811 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_REDAC4;
  if (Field_dsp340050b49a6c_fld3582dual_slot0_Slot_dual_slot0_get (insn) == 27419 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3959dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_REDACS;
  if (Field_dsp340050b49a6c_fld3583dual_slot0_Slot_dual_slot0_get (insn) == 13723 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3960dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SUBAC_I2R;
  switch (Field_dsp340050b49a6c_fld3584_Slot_dual_slot0_get (insn))
    {
    case 2:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3937dual_slot0_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BLTUI;
      break;
    case 3:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3552_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_ASLACM;
      break;
    }
  if (Field_dsp340050b49a6c_fld3585dual_slot0_Slot_dual_slot0_get (insn) == 6875 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3961dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SUBAC_R2I;
  if (Field_dsp340050b49a6c_fld3587dual_slot0_Slot_dual_slot0_get (insn) == 1723 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3610_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld3588dual_slot0_Slot_dual_slot0_get (insn) == 219 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3938dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_NEG;
  if (Field_dsp340050b49a6c_fld3589dual_slot0_Slot_dual_slot0_get (insn) == 106 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_s8_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_LCM;
  if (Field_dsp340050b49a6c_fld3590dual_slot0_Slot_dual_slot0_get (insn) == 3331 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT32_I;
  if (Field_dsp340050b49a6c_fld3591dual_slot0_Slot_dual_slot0_get (insn) == 3335 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT32_R;
  if (Field_dsp340050b49a6c_fld3592dual_slot0_Slot_dual_slot0_get (insn) == 3339 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT_2FIFO_0;
  if (Field_dsp340050b49a6c_fld3593dual_slot0_Slot_dual_slot0_get (insn) == 3343 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT_R2FIFO_0;
  if (Field_dsp340050b49a6c_fld3594dual_slot0_Slot_dual_slot0_get (insn) == 3363 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT_2FIFO_1;
  if (Field_dsp340050b49a6c_fld3595dual_slot0_Slot_dual_slot0_get (insn) == 3367 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT_R2FIFO_1;
  if (Field_dsp340050b49a6c_fld3596dual_slot0_Slot_dual_slot0_get (insn) == 3371 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT_R2FIFO_2;
  if (Field_dsp340050b49a6c_fld3597dual_slot0_Slot_dual_slot0_get (insn) == 3375 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
    return OPCODE_EXT_R2FIFO_3;
  if (Field_dsp340050b49a6c_fld3598dual_slot0_Slot_dual_slot0_get (insn) == 851 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3951dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_2;
  if (Field_dsp340050b49a6c_fld3599dual_slot0_Slot_dual_slot0_get (insn) == 855 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3941dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SET_SMOD_BUF;
  if (Field_dsp340050b49a6c_fld3600dual_slot0_Slot_dual_slot0_get (insn) == 219 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3952dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_EXT_2FIFO_3;
  if (Field_dsp340050b49a6c_fld3601dual_slot0_Slot_dual_slot0_get (insn) == 213 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3945dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  if (Field_dsp340050b49a6c_fld3603dual_slot0_Slot_dual_slot0_get (insn) == 215 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3945dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN_I;
  if (Field_dsp340050b49a6c_fld3604dual_slot0_Slot_dual_slot0_get (insn) == 111 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3946dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_CM2AR_LN_R;
  if (Field_dsp340050b49a6c_fld3606dual_slot0_Slot_dual_slot0_get (insn) == 5 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
    return OPCODE_ABS_S;
  if (Field_dsp340050b49a6c_fld3607dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
    return OPCODE_MOV_S;
  if (Field_dsp340050b49a6c_fld3608dual_slot0_Slot_dual_slot0_get (insn) == 37 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
    return OPCODE_NEG_S;
  if (Field_dsp340050b49a6c_fld3609dual_slot0_Slot_dual_slot0_get (insn) == 106 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_MOVAC;
  if (Field_dsp340050b49a6c_fld3611dual_slot0_Slot_dual_slot0_get (insn) == 107 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SWAPAC_RI;
  if (Field_dsp340050b49a6c_fld3612dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_PUSH32;
  if (Field_dsp340050b49a6c_fld3613dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3936dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_RFR;
  if (Field_dsp340050b49a6c_fld3614dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2079_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SLL;
  if (Field_dsp340050b49a6c_fld3615dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3958dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_ASRAC;
  if (Field_dsp340050b49a6c_fld3616dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3947dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_MOV2AC32_I;
  if (Field_dsp340050b49a6c_fld3618dual_slot0_Slot_dual_slot0_get (insn) == 21 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3949dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_MOV2AC32_R;
  if (Field_dsp340050b49a6c_fld3619_Slot_dual_slot0_get (insn) == 1 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4 &&
      Field_dsp340050b49a6c_fld3940dual_slot0_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_FLOOR_S;
  if (Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_get (insn) == 1 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 3 &&
      Field_dsp340050b49a6c_fld2048_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_L32I;
  if (Field_dsp340050b49a6c_fld3621dual_slot0_Slot_dual_slot0_get (insn) == 42 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
    return OPCODE_SAC_IH;
  if (Field_dsp340050b49a6c_fld3622dual_slot0_Slot_dual_slot0_get (insn) == 43 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
    return OPCODE_SAC_RH;
  if (Field_dsp340050b49a6c_fld3623dual_slot0_Slot_dual_slot0_get (insn) == 44 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
    return OPCODE_SAC_IL;
  if (Field_dsp340050b49a6c_fld3624dual_slot0_Slot_dual_slot0_get (insn) == 45 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
    return OPCODE_SAC_RL;
  if (Field_dsp340050b49a6c_fld3625dual_slot0_Slot_dual_slot0_get (insn) == 38 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4 &&
      Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_MOVI_N;
  if (Field_dsp340050b49a6c_fld3626dual_slot0_Slot_dual_slot0_get (insn) == 39 &&
      Field_op0_s23_Slot_dual_slot0_get (insn) == 4 &&
      Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_get (insn) == 0)
    return OPCODE_SAC2X64_3;
  switch (Field_imm8_Slot_dual_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_ADDX4;
      break;
    case 1:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_ADDX8;
      break;
    case 2:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_AND;
      break;
    case 3:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_L32I_N;
      break;
    case 4:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_ANDB;
      break;
    case 5:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LCM_PINC_X;
      break;
    case 6:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LCM_X;
      break;
    case 7:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LCM_XU;
      break;
    case 8:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_CEIL_S;
      break;
    case 9:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LP_X;
      break;
    case 10:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LQ_X;
      break;
    case 11:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LUT0;
      break;
    case 12:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LSXU;
      break;
    case 13:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LUT1;
      break;
    case 14:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LUT2;
      break;
    case 15:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_LUT3;
      break;
    case 16:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_CLAMPS;
      break;
    case 17:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MADD_S;
      break;
    case 18:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MAX;
      break;
    case 19:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MINU;
      break;
    case 20:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MAXU;
      break;
    case 21:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVEQZ;
      break;
    case 22:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVEQZ_S;
      break;
    case 23:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVF_S;
      break;
    case 24:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MIN;
      break;
    case 25:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVGEZ;
      break;
    case 26:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVGEZ_S;
      break;
    case 27:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVLTZ_S;
      break;
    case 28:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVLTZ;
      break;
    case 29:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVNEZ;
      break;
    case 30:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVNEZ_S;
      break;
    case 31:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVT;
      break;
    case 32:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_COMB_AR;
      break;
    case 33:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MOVT_S;
      break;
    case 34:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MSUB_S;
      break;
    case 35:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_OLT_S;
      break;
    case 36:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_MUL_S;
      break;
    case 37:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_OR;
      break;
    case 38:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_ORB;
      break;
    case 39:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_ROUND_S;
      break;
    case 40:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_OEQ_S;
      break;
    case 41:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_S32I_N;
      break;
    case 46:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SCM;
      break;
    case 47:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SCM_PINC_X;
      break;
    case 48:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_OLE_S;
      break;
    case 49:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SCM_X;
      break;
    case 50:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SCM_XU;
      break;
    case 51:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SRLI;
      break;
    case 52:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SEXT;
      break;
    case 53:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SSX;
      break;
    case 54:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SSXU;
      break;
    case 55:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_STORE_P;
      break;
    case 56:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SRC;
      break;
    case 57:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_STORE_Q;
      break;
    case 58:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_STSWAPBM;
      break;
    case 59:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SUB;
      break;
    case 60:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_STSWAPBMU;
      break;
    case 61:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SUB_S;
      break;
    case 62:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SUBX2;
      break;
    case 63:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SUBX4;
      break;
    case 64:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_FLOAT_S;
      break;
    case 65:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_SUBX8;
      break;
    case 66:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_TRUNC_S;
      break;
    case 67:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_UN_S;
      break;
    case 68:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_UEQ_S;
      break;
    case 69:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_UTRUNC_S;
      break;
    case 70:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_XOR;
      break;
    case 71:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_XORB;
      break;
    case 72:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 4)
	return OPCODE_UFLOAT_S;
      break;
    case 252:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_ADD;
      break;
    case 253:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_ADD_S;
      break;
    case 254:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_ADDI_N;
      break;
    case 255:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 1)
	return OPCODE_ADDX2;
      break;
    }
  if (Field_op0_s23_Slot_dual_slot0_get (insn) == 5)
    return OPCODE_L32R;
  switch (Field_r_Slot_dual_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_BNE;
      break;
    case 1:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_BNONE;
      break;
    case 2:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_L16SI;
      break;
    case 3:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_L8UI;
      break;
    case 4:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_ADDI;
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_L16UI;
      break;
    case 5:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BALL;
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_S16I;
      break;
    case 6:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BANY;
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_S32I;
      break;
    case 7:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BBC;
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 3)
	return OPCODE_S8I;
      break;
    case 8:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_ADDMI;
      break;
    case 9:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BBS;
      break;
    case 10:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BEQ;
      break;
    case 11:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BGEU;
      break;
    case 12:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BGE;
      break;
    case 13:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BLT;
      break;
    case 14:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BLTU;
      break;
    case 15:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 0)
	return OPCODE_BNALL;
      break;
    }
  switch (Field_t_Slot_dual_slot0_get (insn))
    {
    case 0:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
	return OPCODE_BEQI;
      break;
    case 1:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
	return OPCODE_BGEI;
      break;
    case 2:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
	return OPCODE_BGEUI;
      break;
    case 3:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
	return OPCODE_BNEI;
      break;
    case 4:
      if (Field_op0_s23_Slot_dual_slot0_get (insn) == 2)
	return OPCODE_BLTI;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_gp_slot1_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2394gp_slot1_Slot_gp_slot1_get (insn))
    {
    case 0:
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 0)
	return OPCODE_CMAC;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 1)
	return OPCODE_CMPY;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 2)
	return OPCODE_MAC;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 3)
	return OPCODE_MACS;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 4)
	return OPCODE_MACXP_1;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 5)
	return OPCODE_MACXP_3;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 6)
	return OPCODE_MPY8;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 7)
	return OPCODE_MPYXP_0;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 8)
	return OPCODE_MPYXP_2;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 9)
	return OPCODE_NORMACD;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 10)
	return OPCODE_RCMAC;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 11)
	return OPCODE_RMAC;
      break;
    case 1:
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 0)
	return OPCODE_CMACS;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 1)
	return OPCODE_CMPYS;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 2)
	return OPCODE_MAC8;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 3)
	return OPCODE_MACXP_0;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 4)
	return OPCODE_MACXP_2;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 5)
	return OPCODE_MPY;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 6)
	return OPCODE_MPYS;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 7)
	return OPCODE_MPYXP_1;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 8)
	return OPCODE_MPYXP_3;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 9)
	return OPCODE_NORMD;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 10)
	return OPCODE_RCMPY;
      if (Field_op0_s4_Slot_gp_slot1_get (insn) == 11)
	return OPCODE_RMPY;
      break;
    }
  if (Field_dsp340050b49a6c_fld2395gp_slot1_Slot_gp_slot1_get (insn) == 0 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12)
    return OPCODE_ADD2AC;
  if (Field_dsp340050b49a6c_fld2397gp_slot1_Slot_gp_slot1_get (insn) == 1 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12)
    return OPCODE_MOV2AC;
  if (Field_dsp340050b49a6c_fld2398gp_slot1_Slot_gp_slot1_get (insn) == 2 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12)
    return OPCODE_SUB2AC;
  if (Field_dsp340050b49a6c_fld2399gp_slot1_Slot_gp_slot1_get (insn) == 3 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12 &&
      Field_dsp340050b49a6c_fld3681gp_slot1_Slot_gp_slot1_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2400gp_slot1_Slot_gp_slot1_get (insn) == 8 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12 &&
      Field_dsp340050b49a6c_fld3683gp_slot1_Slot_gp_slot1_get (insn) == 0)
    return OPCODE_NORMACPQ_I;
  if (Field_dsp340050b49a6c_fld2402gp_slot1_Slot_gp_slot1_get (insn) == 9 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12 &&
      Field_dsp340050b49a6c_fld3683gp_slot1_Slot_gp_slot1_get (insn) == 0)
    return OPCODE_NORMACPQ_R;
  if (Field_dsp340050b49a6c_fld2403gp_slot1_Slot_gp_slot1_get (insn) == 5 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12 &&
      Field_dsp340050b49a6c_fld3684gp_slot1_Slot_gp_slot1_get (insn) == 0)
    return OPCODE_NORMPYPQ_I;
  if (Field_dsp340050b49a6c_fld2405gp_slot1_Slot_gp_slot1_get (insn) == 3 &&
      Field_op0_s4_Slot_gp_slot1_get (insn) == 12 &&
      Field_dsp340050b49a6c_fld3686gp_slot1_Slot_gp_slot1_get (insn) == 0)
    return OPCODE_NORMPYPQ_R;
  switch (Field_op0_s4_Slot_gp_slot1_get (insn))
    {
    case 13:
      return OPCODE_CMPY2CM;
    case 14:
      return OPCODE_LIN_INT;
    case 15:
      return OPCODE_MPY2CM;
    case 16:
      return OPCODE_MPYADD8_2CM;
    case 17:
      return OPCODE_RCMPY2CM;
    case 18:
      return OPCODE_RMPY2CM;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_dot_slot2_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2049_Slot_dot_slot2_get (insn))
    {
    case 2:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
	  Field_t_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_OR128;
      break;
    case 3:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_PERM;
      break;
    }
  if (Field_dsp340050b49a6c_fld2532dot_slot2_Slot_dot_slot2_get (insn) == 0 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ADD32;
  if (Field_dsp340050b49a6c_fld2533dot_slot2_Slot_dot_slot2_get (insn) == 1 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ADDCM;
  if (Field_dsp340050b49a6c_fld2534dot_slot2_Slot_dot_slot2_get (insn) == 2 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ADDWRP;
  if (Field_dsp340050b49a6c_fld2535dot_slot2_Slot_dot_slot2_get (insn) == 3 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASR;
  if (Field_dsp340050b49a6c_fld2536dot_slot2_Slot_dot_slot2_get (insn) == 4 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_AND128;
  if (Field_dsp340050b49a6c_fld2537dot_slot2_Slot_dot_slot2_get (insn) == 5 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASR32;
  if (Field_dsp340050b49a6c_fld2538dot_slot2_Slot_dot_slot2_get (insn) == 6 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_EXTUI4;
  if (Field_dsp340050b49a6c_fld2539dot_slot2_Slot_dot_slot2_get (insn) == 7 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LUT;
  if (Field_dsp340050b49a6c_fld2540dot_slot2_Slot_dot_slot2_get (insn) == 8 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASL;
  if (Field_dsp340050b49a6c_fld2541dot_slot2_Slot_dot_slot2_get (insn) == 9 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LUT_IEXT;
  if (Field_dsp340050b49a6c_fld2542dot_slot2_Slot_dot_slot2_get (insn) == 10 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LUT_REXT;
  if (Field_dsp340050b49a6c_fld2543dot_slot2_Slot_dot_slot2_get (insn) == 11 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MEAN;
  if (Field_dsp340050b49a6c_fld2544dot_slot2_Slot_dot_slot2_get (insn) == 12 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MAX8;
  if (Field_dsp340050b49a6c_fld2545dot_slot2_Slot_dot_slot2_get (insn) == 13 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MEAN32;
  if (Field_dsp340050b49a6c_fld2546dot_slot2_Slot_dot_slot2_get (insn) == 14 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MIN8;
  if (Field_dsp340050b49a6c_fld2547dot_slot2_Slot_dot_slot2_get (insn) == 15 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOV2CM2PQ;
  if (Field_dsp340050b49a6c_fld2548dot_slot2_Slot_dot_slot2_get (insn) == 16 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASL32;
  if (Field_dsp340050b49a6c_fld2549dot_slot2_Slot_dot_slot2_get (insn) == 17 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_0;
  if (Field_dsp340050b49a6c_fld2550dot_slot2_Slot_dot_slot2_get (insn) == 18 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_1;
  if (Field_dsp340050b49a6c_fld2551dot_slot2_Slot_dot_slot2_get (insn) == 19 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_4;
  if (Field_dsp340050b49a6c_fld2552dot_slot2_Slot_dot_slot2_get (insn) == 20 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_2;
  if (Field_dsp340050b49a6c_fld2553dot_slot2_Slot_dot_slot2_get (insn) == 21 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_5;
  if (Field_dsp340050b49a6c_fld2554dot_slot2_Slot_dot_slot2_get (insn) == 22 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_6;
  if (Field_dsp340050b49a6c_fld2555dot_slot2_Slot_dot_slot2_get (insn) == 23 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_7;
  if (Field_dsp340050b49a6c_fld2556dot_slot2_Slot_dot_slot2_get (insn) == 24 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_3;
  if (Field_dsp340050b49a6c_fld2557dot_slot2_Slot_dot_slot2_get (insn) == 25 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_0;
  if (Field_dsp340050b49a6c_fld2558dot_slot2_Slot_dot_slot2_get (insn) == 26 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_1;
  if (Field_dsp340050b49a6c_fld2559dot_slot2_Slot_dot_slot2_get (insn) == 27 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_3;
  if (Field_dsp340050b49a6c_fld2560dot_slot2_Slot_dot_slot2_get (insn) == 28 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_2;
  if (Field_dsp340050b49a6c_fld2561dot_slot2_Slot_dot_slot2_get (insn) == 29 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_4;
  if (Field_dsp340050b49a6c_fld2562dot_slot2_Slot_dot_slot2_get (insn) == 30 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_5;
  if (Field_dsp340050b49a6c_fld2563dot_slot2_Slot_dot_slot2_get (insn) == 31 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCND_6;
  if (Field_dsp340050b49a6c_fld2564dot_slot2_Slot_dot_slot2_get (insn) == 32 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASLM;
  if (Field_dsp340050b49a6c_fld2565dot_slot2_Slot_dot_slot2_get (insn) == 33 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASRM;
  if (Field_dsp340050b49a6c_fld2566dot_slot2_Slot_dot_slot2_get (insn) == 34 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CMP8;
  if (Field_dsp340050b49a6c_fld2567dot_slot2_Slot_dot_slot2_get (insn) == 35 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LSRM;
  if (Field_dsp340050b49a6c_fld2568dot_slot2_Slot_dot_slot2_get (insn) == 36 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CMP_I;
  if (Field_dsp340050b49a6c_fld2569dot_slot2_Slot_dot_slot2_get (insn) == 517 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ABS8;
  if (Field_dsp340050b49a6c_fld2571dot_slot2_Slot_dot_slot2_get (insn) == 533 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld2572dot_slot2_Slot_dot_slot2_get (insn) == 549 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LUT_PHASOR;
  if (Field_dsp340050b49a6c_fld2573dot_slot2_Slot_dot_slot2_get (insn) == 565 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld2574dot_slot2_Slot_dot_slot2_get (insn) == 581 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld2575dot_slot2_Slot_dot_slot2_get (insn) == 597 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_NOT128;
  if (Field_dsp340050b49a6c_fld2576dot_slot2_Slot_dot_slot2_get (insn) == 613 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_TRANS;
  if (Field_dsp340050b49a6c_fld2577dot_slot2_Slot_dot_slot2_get (insn) == 629 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2578_Slot_dot_slot2_get (insn) == 0 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_LUT_AR;
  if (Field_dsp340050b49a6c_fld2579dot_slot2_Slot_dot_slot2_get (insn) == 1653 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2580dot_slot2_Slot_dot_slot2_get (insn) == 2677 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld2581dot_slot2_Slot_dot_slot2_get (insn) == 3701 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld2582dot_slot2_Slot_dot_slot2_get (insn) == 4725 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld2583dot_slot2_Slot_dot_slot2_get (insn) == 5749 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld2584dot_slot2_Slot_dot_slot2_get (insn) == 6773 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2585dot_slot2_Slot_dot_slot2_get (insn) == 7797 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2586dot_slot2_Slot_dot_slot2_get (insn) == 8821 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld2587dot_slot2_Slot_dot_slot2_get (insn) == 9845 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2588dot_slot2_Slot_dot_slot2_get (insn) == 10869 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2589dot_slot2_Slot_dot_slot2_get (insn) == 11893 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2590dot_slot2_Slot_dot_slot2_get (insn) == 12917 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_4;
  switch (Field_dsp340050b49a6c_fld2591dot_slot2_Slot_dot_slot2_get (insn))
    {
    case 221301:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_0;
      break;
    case 221557:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_2;
      break;
    case 221813:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_3;
      break;
    case 222069:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_0;
      break;
    case 222325:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_4;
      break;
    case 222581:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_1;
      break;
    case 222837:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_2;
      break;
    case 223093:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_3;
      break;
    }
  if (Field_dsp340050b49a6c_fld2592dot_slot2_Slot_dot_slot2_get (insn) == 55925 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3708dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_5;
  if (Field_dsp340050b49a6c_fld2595dot_slot2_Slot_dot_slot2_get (insn) == 56181 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3708dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld2596dot_slot2_Slot_dot_slot2_get (insn) == 7797 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3724dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_1;
  if (Field_dsp340050b49a6c_fld2598dot_slot2_Slot_dot_slot2_get (insn) == 645 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_NCO_UPDATE;
  if (Field_dsp340050b49a6c_fld2599dot_slot2_Slot_dot_slot2_get (insn) == 2197 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3709dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SET_ARGMAX;
  if (Field_dsp340050b49a6c_fld2601dot_slot2_Slot_dot_slot2_get (insn) == 2453 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3709dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SET_NCO;
  if (Field_dsp340050b49a6c_fld2602dot_slot2_Slot_dot_slot2_get (insn) == 1429 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3713dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld2604dot_slot2_Slot_dot_slot2_get (insn) == 341 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3710dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SET_HSAR;
  if (Field_dsp340050b49a6c_fld2606dot_slot2_Slot_dot_slot2_get (insn) == 181 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3711dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SET_MAX;
  if (Field_dsp340050b49a6c_fld2608dot_slot2_Slot_dot_slot2_get (insn) == 19 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_0;
  if (Field_dsp340050b49a6c_fld2609dot_slot2_Slot_dot_slot2_get (insn) == 51 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_1;
  if (Field_dsp340050b49a6c_fld2610dot_slot2_Slot_dot_slot2_get (insn) == 83 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_2;
  if (Field_dsp340050b49a6c_fld2611dot_slot2_Slot_dot_slot2_get (insn) == 899 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_PUSH2X128_PQ;
  if (Field_dsp340050b49a6c_fld2612_Slot_dot_slot2_get (insn) == 32 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_QREADY;
  if (Field_dsp340050b49a6c_fld2613dot_slot2_Slot_dot_slot2_get (insn) == 1811 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_0;
  switch (Field_dsp340050b49a6c_fld2614_Slot_dot_slot2_get (insn))
    {
    case 66:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_MOVPQ2PQ;
      break;
    case 67:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_SET_EXT_REGS;
      break;
    case 68:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_PUSH128_PQ;
      break;
    case 69:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_SUBARX;
      break;
    case 70:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_SWAPB;
      break;
    case 71:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_WRTIEP;
      break;
    case 72:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_PUSH32;
      break;
    }
  if (Field_dsp340050b49a6c_fld2615dot_slot2_Slot_dot_slot2_get (insn) == 1819 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_3;
  if (Field_dsp340050b49a6c_fld2616dot_slot2_Slot_dot_slot2_get (insn) == 1827 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_1;
  if (Field_dsp340050b49a6c_fld2617dot_slot2_Slot_dot_slot2_get (insn) == 1835 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_4;
  if (Field_dsp340050b49a6c_fld2618dot_slot2_Slot_dot_slot2_get (insn) == 1843 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_5;
  if (Field_dsp340050b49a6c_fld2619dot_slot2_Slot_dot_slot2_get (insn) == 3702 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld2620dot_slot2_Slot_dot_slot2_get (insn) == 3703 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2621dot_slot2_Slot_dot_slot2_get (insn) == 1859 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_2;
  if (Field_dsp340050b49a6c_fld2622dot_slot2_Slot_dot_slot2_get (insn) == 1867 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2047_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2623dot_slot2_Slot_dot_slot2_get (insn) == 939 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3727dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld2624dot_slot2_Slot_dot_slot2_get (insn) == 475 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3729dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld2625_Slot_dot_slot2_get (insn) == 19 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3731dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SETTIEP;
  if (Field_dsp340050b49a6c_fld2626dot_slot2_Slot_dot_slot2_get (insn) == 51 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3715_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_3;
  if (Field_dsp340050b49a6c_fld2628dot_slot2_Slot_dot_slot2_get (insn) == 51 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3722_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld2630dot_slot2_Slot_dot_slot2_get (insn) == 10 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2032_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CMP_R;
  if (Field_dsp340050b49a6c_fld2632dot_slot2_Slot_dot_slot2_get (insn) == 11 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_t_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ADD16;
  if (Field_dsp340050b49a6c_fld2633dot_slot2_Slot_dot_slot2_get (insn) == 6 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3733dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LSLM;
  if (Field_dsp340050b49a6c_fld2635dot_slot2_Slot_dot_slot2_get (insn) == 7 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3719dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  switch (Field_dsp340050b49a6c_fld2636dot_slot2_Slot_dot_slot2_get (insn))
    {
    case 1168:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_AR2SAR_DUP;
      break;
    case 1169:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_01;
      break;
    case 1170:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_03;
      break;
    case 1171:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
	return OPCODE_WRTBSIGQ;
      break;
    }
  if (Field_dsp340050b49a6c_fld2637dot_slot2_Slot_dot_slot2_get (insn) == 586 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3725dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_21;
  if (Field_dsp340050b49a6c_fld2640dot_slot2_Slot_dot_slot2_get (insn) == 587 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3725dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_WRTSIGQ;
  if (Field_dsp340050b49a6c_fld2641dot_slot2_Slot_dot_slot2_get (insn) == 147 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3726dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_23;
  if (Field_dsp340050b49a6c_fld2642dot_slot2_Slot_dot_slot2_get (insn) == 74 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2605_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_CLRTIEP;
  if (Field_dsp340050b49a6c_fld2643dot_slot2_Slot_dot_slot2_get (insn) == 75 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2605_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQM;
  if (Field_dsp340050b49a6c_fld2644dot_slot2_Slot_dot_slot2_get (insn) == 4 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3732dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ADDAR2;
  if (Field_dsp340050b49a6c_fld2645dot_slot2_Slot_dot_slot2_get (insn) == 4 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3714dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld2646dot_slot2_Slot_dot_slot2_get (insn) == 4 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3721dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVAR2;
  if (Field_dsp340050b49a6c_fld2647dot_slot2_Slot_dot_slot2_get (insn) == 16 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_MOVCND_7;
  if (Field_dsp340050b49a6c_fld2648dot_slot2_Slot_dot_slot2_get (insn) == 17 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_SUB32;
  if (Field_dsp340050b49a6c_fld2649dot_slot2_Slot_dot_slot2_get (insn) == 18 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_SUBCM;
  if (Field_dsp340050b49a6c_fld2650dot_slot2_Slot_dot_slot2_get (insn) == 19 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_XOR128;
  if (Field_dsp340050b49a6c_fld2651dot_slot2_Slot_dot_slot2_get (insn) == 20 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1)
    return OPCODE_SUBMEAN;
  if (Field_dsp340050b49a6c_fld2652dot_slot2_Slot_dot_slot2_get (insn) == 37 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3718_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ARGMAX8;
  if (Field_dsp340050b49a6c_fld2654dot_slot2_Slot_dot_slot2_get (insn) == 53 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3728dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld2655dot_slot2_Slot_dot_slot2_get (insn) == 11 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_MOVCM2PQ;
  if (Field_dsp340050b49a6c_fld2656dot_slot2_Slot_dot_slot2_get (insn) == 6 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3712_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_SUBWRP;
  if (Field_dsp340050b49a6c_fld2657dot_slot2_Slot_dot_slot2_get (insn) == 7 &&
      Field_op0_s6_Slot_dot_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_ASLM32;
  switch (Field_dsp340050b49a6c_fld2658dot_slot2_Slot_dot_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3717dot_slot2_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_AR2PQ_LN;
      break;
    case 1:
      if (Field_op0_s6_Slot_dot_slot2_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3716dot_slot2_Slot_dot_slot2_get (insn) == 0)
	return OPCODE_AR2CM_LN;
      break;
    }
  if (Field_op0_s6_Slot_dot_slot2_get (insn) == 3 &&
      Field_dsp340050b49a6c_fld3723dot_slot2_Slot_dot_slot2_get (insn) == 0)
    return OPCODE_LUT_WRITE;
  return XTENSA_UNDEFINED;
}

static int
Slot_dot_slot1_decode (const xtensa_insnbuf insn)
{
  switch (Field_op0_s7_Slot_dot_slot1_get (insn))
    {
    case 0:
      return OPCODE_CDOT;
    case 1:
      return OPCODE_CDOTAC;
    case 2:
      return OPCODE_CDOTACS;
    case 3:
      return OPCODE_DOT;
    case 4:
      return OPCODE_DOTAC;
    case 5:
      return OPCODE_DOTACS;
    case 6:
      return OPCODE_MACD8;
    case 7:
      return OPCODE_MACPQXP_0;
    case 8:
      return OPCODE_MACPQXP_1;
    case 9:
      return OPCODE_MACPQXP_2;
    case 10:
      return OPCODE_MACPQXP_3;
    case 11:
      return OPCODE_MACXP2_0;
    case 12:
      return OPCODE_MACXP2_1;
    case 13:
      return OPCODE_MPYD8;
    case 14:
      return OPCODE_MPYPQXP_0;
    case 15:
      return OPCODE_MPYPQXP_1;
    case 16:
      return OPCODE_MPYPQXP_2;
    case 17:
      return OPCODE_MPYPQXP_3;
    case 18:
      return OPCODE_MPYXP2_0;
    case 19:
      return OPCODE_MPYXP2_1;
    case 20:
      if (Field_dsp340050b49a6c_fld3734dot_slot1_Slot_dot_slot1_get (insn) == 0)
	return OPCODE_NOP;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_pq_slot1_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2825pq_slot1_Slot_pq_slot1_get (insn))
    {
    case 0:
      if (Field_op0_s10_Slot_pq_slot1_get (insn) == 0)
	return OPCODE_CMPYXP2PQ;
      break;
    case 1:
      if (Field_op0_s10_Slot_pq_slot1_get (insn) == 0)
	return OPCODE_MPYXP2PQ;
      break;
    }
  if (Field_dsp340050b49a6c_fld2826pq_slot1_Slot_pq_slot1_get (insn) == 1 &&
      Field_op0_s10_Slot_pq_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3761pq_slot1_Slot_pq_slot1_get (insn) == 0)
    return OPCODE_NOP;
  switch (Field_op0_s10_Slot_pq_slot1_get (insn))
    {
    case 1:
      return OPCODE_CMPY2PQ;
    case 2:
      return OPCODE_MPY2PQ;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_acc2_slot2_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2045_Slot_acc2_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 0)
	return OPCODE_ARGMAX8;
      break;
    case 2:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_get (insn) == 0)
	return OPCODE_POP32_0;
      break;
    case 3:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_get (insn) == 0)
	return OPCODE_POP32_3;
      break;
    }
  switch (Field_dsp340050b49a6c_fld2953acc2_slot2_Slot_acc2_slot2_get (insn))
    {
    case 2:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 0)
	return OPCODE_PUSH128;
      break;
    case 3:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld3782acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
	return OPCODE_NOP;
      break;
    }
  if (Field_dsp340050b49a6c_fld2954acc2_slot2_Slot_acc2_slot2_get (insn) == 1 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3786acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld2955acc2_slot2_Slot_acc2_slot2_get (insn) == 1 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3788acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld2956acc2_slot2_Slot_acc2_slot2_get (insn) == 3 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld2957acc2_slot2_Slot_acc2_slot2_get (insn) == 19 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld2958acc2_slot2_Slot_acc2_slot2_get (insn) == 35 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld2959acc2_slot2_Slot_acc2_slot2_get (insn) == 51 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld2960acc2_slot2_Slot_acc2_slot2_get (insn) == 35 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3784acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld2963acc2_slot2_Slot_acc2_slot2_get (insn) == 51 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3784acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld2964acc2_slot2_Slot_acc2_slot2_get (insn) == 19 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3785acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld2966acc2_slot2_Slot_acc2_slot2_get (insn) == 1 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3783acc2_slot2_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_NCO_UPDATE;
  if (Field_dsp340050b49a6c_fld2967acc2_slot2_Slot_acc2_slot2_get (insn) == 1 &&
      Field_op0_s12_Slot_acc2_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2056_Slot_acc2_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  switch (Field_op0_s12_Slot_acc2_slot2_get (insn))
    {
    case 2:
      if (Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_get (insn) == 0)
	return OPCODE_PUSH32;
      break;
    case 3:
      return OPCODE_ADD32;
    case 4:
      return OPCODE_ADDCM;
    case 5:
      return OPCODE_ASR;
    case 6:
      return OPCODE_LUT_IEXT;
    case 7:
      return OPCODE_PERM;
    }
  switch (Field_s_Slot_acc2_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
	return OPCODE_CONJ;
      break;
    case 1:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
	return OPCODE_LUT_PHASOR;
      break;
    case 2:
      if (Field_op0_s12_Slot_acc2_slot2_get (insn) == 1)
	return OPCODE_MOVCM;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_acc2_slot1_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2075_Slot_acc2_slot1_get (insn))
    {
    case 0:
      if (Field_op0_s13_Slot_acc2_slot1_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2031_Slot_acc2_slot1_get (insn) == 0)
	return OPCODE_RFIRD;
      break;
    case 1:
      if (Field_op0_s13_Slot_acc2_slot1_get (insn) == 0 &&
	  Field_dsp340050b49a6c_fld2031_Slot_acc2_slot1_get (insn) == 0)
	return OPCODE_RFIRDA;
      break;
    }
  if (Field_dsp340050b49a6c_fld2968acc2_slot1_Slot_acc2_slot1_get (insn) == 1 &&
      Field_op0_s13_Slot_acc2_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3793acc2_slot1_Slot_acc2_slot1_get (insn) == 0)
    return OPCODE_SWAPAC_R;
  if (Field_dsp340050b49a6c_fld2969acc2_slot1_Slot_acc2_slot1_get (insn) == 1 &&
      Field_op0_s13_Slot_acc2_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3790acc2_slot1_Slot_acc2_slot1_get (insn) == 0)
    return OPCODE_NOP;
  switch (Field_op0_s13_Slot_acc2_slot1_get (insn))
    {
    case 1:
      return OPCODE_LLRPRE1;
    case 2:
      return OPCODE_RFIR;
    case 3:
      return OPCODE_RFIRA;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_smod_slot2_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2049_Slot_smod_slot2_get (insn))
    {
    case 2:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
	  Field_t_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_OR128;
      break;
    case 3:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2029_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_PERM;
      break;
    }
  if (Field_dsp340050b49a6c_fld2991smod_slot2_Slot_smod_slot2_get (insn) == 0 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ADD32;
  if (Field_dsp340050b49a6c_fld2992smod_slot2_Slot_smod_slot2_get (insn) == 1 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ADDCM;
  if (Field_dsp340050b49a6c_fld2993smod_slot2_Slot_smod_slot2_get (insn) == 2 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ADDWRP;
  if (Field_dsp340050b49a6c_fld2994smod_slot2_Slot_smod_slot2_get (insn) == 3 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASR;
  if (Field_dsp340050b49a6c_fld2995smod_slot2_Slot_smod_slot2_get (insn) == 4 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_AND128;
  if (Field_dsp340050b49a6c_fld2996smod_slot2_Slot_smod_slot2_get (insn) == 5 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASR32;
  if (Field_dsp340050b49a6c_fld2997smod_slot2_Slot_smod_slot2_get (insn) == 6 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_EXTUI4;
  if (Field_dsp340050b49a6c_fld2998smod_slot2_Slot_smod_slot2_get (insn) == 7 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LUT;
  if (Field_dsp340050b49a6c_fld2999smod_slot2_Slot_smod_slot2_get (insn) == 8 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASL;
  if (Field_dsp340050b49a6c_fld3000smod_slot2_Slot_smod_slot2_get (insn) == 9 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LUT_IEXT;
  if (Field_dsp340050b49a6c_fld3001smod_slot2_Slot_smod_slot2_get (insn) == 10 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LUT_REXT;
  if (Field_dsp340050b49a6c_fld3002smod_slot2_Slot_smod_slot2_get (insn) == 11 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MEAN;
  if (Field_dsp340050b49a6c_fld3003smod_slot2_Slot_smod_slot2_get (insn) == 12 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MAX8;
  if (Field_dsp340050b49a6c_fld3004smod_slot2_Slot_smod_slot2_get (insn) == 13 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MEAN32;
  if (Field_dsp340050b49a6c_fld3005smod_slot2_Slot_smod_slot2_get (insn) == 14 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MIN8;
  if (Field_dsp340050b49a6c_fld3006smod_slot2_Slot_smod_slot2_get (insn) == 15 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOV2CM2PQ;
  if (Field_dsp340050b49a6c_fld3007smod_slot2_Slot_smod_slot2_get (insn) == 16 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASL32;
  if (Field_dsp340050b49a6c_fld3008smod_slot2_Slot_smod_slot2_get (insn) == 17 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_0;
  if (Field_dsp340050b49a6c_fld3009smod_slot2_Slot_smod_slot2_get (insn) == 18 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_1;
  if (Field_dsp340050b49a6c_fld3010smod_slot2_Slot_smod_slot2_get (insn) == 19 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_4;
  if (Field_dsp340050b49a6c_fld3011smod_slot2_Slot_smod_slot2_get (insn) == 20 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_2;
  if (Field_dsp340050b49a6c_fld3012smod_slot2_Slot_smod_slot2_get (insn) == 21 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_5;
  if (Field_dsp340050b49a6c_fld3013smod_slot2_Slot_smod_slot2_get (insn) == 22 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_6;
  if (Field_dsp340050b49a6c_fld3014smod_slot2_Slot_smod_slot2_get (insn) == 23 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_7;
  if (Field_dsp340050b49a6c_fld3015smod_slot2_Slot_smod_slot2_get (insn) == 24 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND8_3;
  if (Field_dsp340050b49a6c_fld3016smod_slot2_Slot_smod_slot2_get (insn) == 25 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_0;
  if (Field_dsp340050b49a6c_fld3017smod_slot2_Slot_smod_slot2_get (insn) == 26 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_1;
  if (Field_dsp340050b49a6c_fld3018smod_slot2_Slot_smod_slot2_get (insn) == 27 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_3;
  if (Field_dsp340050b49a6c_fld3019smod_slot2_Slot_smod_slot2_get (insn) == 28 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_2;
  if (Field_dsp340050b49a6c_fld3020smod_slot2_Slot_smod_slot2_get (insn) == 29 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_4;
  if (Field_dsp340050b49a6c_fld3021smod_slot2_Slot_smod_slot2_get (insn) == 30 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_5;
  if (Field_dsp340050b49a6c_fld3022smod_slot2_Slot_smod_slot2_get (insn) == 31 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCND_6;
  if (Field_dsp340050b49a6c_fld3023smod_slot2_Slot_smod_slot2_get (insn) == 32 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASLM;
  if (Field_dsp340050b49a6c_fld3024smod_slot2_Slot_smod_slot2_get (insn) == 33 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASRM;
  if (Field_dsp340050b49a6c_fld3025smod_slot2_Slot_smod_slot2_get (insn) == 34 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CMP8;
  if (Field_dsp340050b49a6c_fld3026smod_slot2_Slot_smod_slot2_get (insn) == 35 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LSRM;
  if (Field_dsp340050b49a6c_fld3027smod_slot2_Slot_smod_slot2_get (insn) == 36 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CMP_I;
  if (Field_dsp340050b49a6c_fld3028smod_slot2_Slot_smod_slot2_get (insn) == 517 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ABS8;
  if (Field_dsp340050b49a6c_fld3030smod_slot2_Slot_smod_slot2_get (insn) == 533 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CONJ;
  if (Field_dsp340050b49a6c_fld3031smod_slot2_Slot_smod_slot2_get (insn) == 549 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LUT_PHASOR;
  if (Field_dsp340050b49a6c_fld3032smod_slot2_Slot_smod_slot2_get (insn) == 565 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld3033smod_slot2_Slot_smod_slot2_get (insn) == 581 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCM;
  if (Field_dsp340050b49a6c_fld3034smod_slot2_Slot_smod_slot2_get (insn) == 597 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_NOT128;
  if (Field_dsp340050b49a6c_fld3035smod_slot2_Slot_smod_slot2_get (insn) == 613 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_TRANS;
  if (Field_dsp340050b49a6c_fld3036smod_slot2_Slot_smod_slot2_get (insn) == 1141 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_PUSH128;
  switch (Field_dsp340050b49a6c_fld3038smod_slot2_Slot_smod_slot2_get (insn))
    {
    case 8565:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_SET_ARGMAX;
      break;
    case 9077:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_SET_HSAR;
      break;
    case 9589:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_SET_MAX;
      break;
    case 10101:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_SET_SAR;
      break;
    case 10613:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_SET_NCO;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3039smod_slot2_Slot_smod_slot2_get (insn))
    {
    case 11125:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_0;
      break;
    case 27509:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_2;
      break;
    case 43893:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_3;
      break;
    case 60277:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_0;
      break;
    case 76661:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ128_4;
      break;
    case 93045:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_1;
      break;
    case 109429:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_2;
      break;
    case 125813:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_MOVEQ32_3;
      break;
    }
  if (Field_dsp340050b49a6c_fld3040smod_slot2_Slot_smod_slot2_get (insn) == 43893 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3805smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_5;
  if (Field_dsp340050b49a6c_fld3043smod_slot2_Slot_smod_slot2_get (insn) == 60277 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3805smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3044smod_slot2_Slot_smod_slot2_get (insn) == 6005 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3819smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_1;
  if (Field_dsp340050b49a6c_fld3046smod_slot2_Slot_smod_slot2_get (insn) == 645 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_NCO_UPDATE;
  if (Field_dsp340050b49a6c_fld3047smod_slot2_Slot_smod_slot2_get (insn) == 661 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld3048_Slot_smod_slot2_get (insn) == 0 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_LUT_AR;
  if (Field_dsp340050b49a6c_fld3049smod_slot2_Slot_smod_slot2_get (insn) == 1685 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld3050smod_slot2_Slot_smod_slot2_get (insn) == 2709 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld3051smod_slot2_Slot_smod_slot2_get (insn) == 3733 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld3052smod_slot2_Slot_smod_slot2_get (insn) == 4757 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld3053smod_slot2_Slot_smod_slot2_get (insn) == 5781 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld3054smod_slot2_Slot_smod_slot2_get (insn) == 6805 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld3055smod_slot2_Slot_smod_slot2_get (insn) == 7829 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld3056smod_slot2_Slot_smod_slot2_get (insn) == 4757 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3809smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld3058smod_slot2_Slot_smod_slot2_get (insn) == 5781 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3809smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_4;
  if (Field_dsp340050b49a6c_fld3059smod_slot2_Slot_smod_slot2_get (insn) == 3733 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3821smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld3061smod_slot2_Slot_smod_slot2_get (insn) == 341 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3806smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld3063smod_slot2_Slot_smod_slot2_get (insn) == 181 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3807smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld3065smod_slot2_Slot_smod_slot2_get (insn) == 19 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_0;
  if (Field_dsp340050b49a6c_fld3066smod_slot2_Slot_smod_slot2_get (insn) == 51 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_1;
  if (Field_dsp340050b49a6c_fld3067smod_slot2_Slot_smod_slot2_get (insn) == 83 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_2;
  if (Field_dsp340050b49a6c_fld3068smod_slot2_Slot_smod_slot2_get (insn) == 227 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_PUSH128_PQ;
  if (Field_dsp340050b49a6c_fld3069smod_slot2_Slot_smod_slot2_get (insn) == 1803 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_0;
  switch (Field_dsp340050b49a6c_fld3070_Slot_smod_slot2_get (insn))
    {
    case 66:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_MOVPQ2PQ;
      break;
    case 67:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_SUBARX;
      break;
    case 68:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_PUSH32;
      break;
    case 69:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_SWAPB;
      break;
    case 70:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_WRTIEP;
      break;
    }
  if (Field_dsp340050b49a6c_fld3071smod_slot2_Slot_smod_slot2_get (insn) == 1819 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_1;
  if (Field_dsp340050b49a6c_fld3072smod_slot2_Slot_smod_slot2_get (insn) == 1835 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_2;
  if (Field_dsp340050b49a6c_fld3073smod_slot2_Slot_smod_slot2_get (insn) == 1851 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_4;
  if (Field_dsp340050b49a6c_fld3074smod_slot2_Slot_smod_slot2_get (insn) == 1867 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_3;
  if (Field_dsp340050b49a6c_fld3075smod_slot2_Slot_smod_slot2_get (insn) == 1883 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2PQ_5;
  if (Field_dsp340050b49a6c_fld3076smod_slot2_Slot_smod_slot2_get (insn) == 3798 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld3077smod_slot2_Slot_smod_slot2_get (insn) == 3799 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld3078smod_slot2_Slot_smod_slot2_get (insn) == 3830 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld3079smod_slot2_Slot_smod_slot2_get (insn) == 3831 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld3080smod_slot2_Slot_smod_slot2_get (insn) == 147 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP128_2CMPQ_3;
  if (Field_dsp340050b49a6c_fld3081smod_slot2_Slot_smod_slot2_get (insn) == 179 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2056_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_PUSH2X128_PQ;
  if (Field_dsp340050b49a6c_fld3082smod_slot2_Slot_smod_slot2_get (insn) == 115 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3824smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld3084smod_slot2_Slot_smod_slot2_get (insn) == 51 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3817_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld3085smod_slot2_Slot_smod_slot2_get (insn) == 10 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld2032_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CMP_R;
  if (Field_dsp340050b49a6c_fld3087smod_slot2_Slot_smod_slot2_get (insn) == 11 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_t_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ADD16;
  if (Field_dsp340050b49a6c_fld3088smod_slot2_Slot_smod_slot2_get (insn) == 6 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3828smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LSLM;
  if (Field_dsp340050b49a6c_fld3090smod_slot2_Slot_smod_slot2_get (insn) == 7 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3814smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CM2AR_LN;
  switch (Field_dsp340050b49a6c_fld3091smod_slot2_Slot_smod_slot2_get (insn))
    {
    case 32:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_QREADY;
      break;
    case 36:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld2041_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_SET_EXT_REGS;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3092smod_slot2_Slot_smod_slot2_get (insn))
    {
    case 1136:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_AR2SAR_DUP;
      break;
    case 1137:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_01;
      break;
    case 1138:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_POP2X128_2PQ_03;
      break;
    case 1139:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
	return OPCODE_WRTBSIGQ;
      break;
    }
  if (Field_dsp340050b49a6c_fld3093smod_slot2_Slot_smod_slot2_get (insn) == 570 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3822smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_21;
  if (Field_dsp340050b49a6c_fld3096smod_slot2_Slot_smod_slot2_get (insn) == 571 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3822smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_WRTSIGQ;
  if (Field_dsp340050b49a6c_fld3097smod_slot2_Slot_smod_slot2_get (insn) == 143 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3823smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_POP2X128_2PQ_23;
  if (Field_dsp340050b49a6c_fld3098smod_slot2_Slot_smod_slot2_get (insn) == 74 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3062_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_CLRTIEP;
  if (Field_dsp340050b49a6c_fld3099smod_slot2_Slot_smod_slot2_get (insn) == 75 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3062_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQM;
  if (Field_dsp340050b49a6c_fld3100smod_slot2_Slot_smod_slot2_get (insn) == 19 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3825smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_SETTIEP;
  if (Field_dsp340050b49a6c_fld3101smod_slot2_Slot_smod_slot2_get (insn) == 4 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3826smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ADDAR2;
  if (Field_dsp340050b49a6c_fld3102smod_slot2_Slot_smod_slot2_get (insn) == 4 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3810smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld3104smod_slot2_Slot_smod_slot2_get (insn) == 4 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3816smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVAR2;
  if (Field_dsp340050b49a6c_fld3105smod_slot2_Slot_smod_slot2_get (insn) == 16 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_MOVCND_7;
  if (Field_dsp340050b49a6c_fld3106smod_slot2_Slot_smod_slot2_get (insn) == 17 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_SMOD_LUT;
  if (Field_dsp340050b49a6c_fld3107smod_slot2_Slot_smod_slot2_get (insn) == 18 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_SUB32;
  if (Field_dsp340050b49a6c_fld3108smod_slot2_Slot_smod_slot2_get (insn) == 19 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_SUBWRP;
  if (Field_dsp340050b49a6c_fld3109smod_slot2_Slot_smod_slot2_get (insn) == 20 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_SUBCM;
  if (Field_dsp340050b49a6c_fld3110smod_slot2_Slot_smod_slot2_get (insn) == 21 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1)
    return OPCODE_XOR128;
  if (Field_dsp340050b49a6c_fld3111smod_slot2_Slot_smod_slot2_get (insn) == 19 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2056_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_MOVCM2PQ;
  if (Field_dsp340050b49a6c_fld3113smod_slot2_Slot_smod_slot2_get (insn) == 27 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3827smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ARGMAX8;
  if (Field_dsp340050b49a6c_fld3114smod_slot2_Slot_smod_slot2_get (insn) == 6 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3808_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_SUBMEAN;
  if (Field_dsp340050b49a6c_fld3115smod_slot2_Slot_smod_slot2_get (insn) == 7 &&
      Field_op0_s15_Slot_smod_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2029_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_ASLM32;
  switch (Field_dsp340050b49a6c_fld3116smod_slot2_Slot_smod_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3813smod_slot2_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_AR2PQ_LN;
      break;
    case 1:
      if (Field_op0_s15_Slot_smod_slot2_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3812smod_slot2_Slot_smod_slot2_get (insn) == 0)
	return OPCODE_AR2CM_LN;
      break;
    }
  if (Field_op0_s15_Slot_smod_slot2_get (insn) == 3 &&
      Field_dsp340050b49a6c_fld3818smod_slot2_Slot_smod_slot2_get (insn) == 0)
    return OPCODE_LUT_WRITE;
  return XTENSA_UNDEFINED;
}

static int
Slot_smod_slot1_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2080_Slot_smod_slot1_get (insn))
    {
    case 0:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 0)
	return OPCODE_CMPY2CM;
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_RMPY2CM;
      break;
    case 1:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 0)
	return OPCODE_MPY2CM;
      break;
    case 2:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 0)
	return OPCODE_MPYADD8_2CM;
      break;
    case 3:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 0)
	return OPCODE_RCMPY2CM;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3117smod_slot1_Slot_smod_slot1_get (insn))
    {
    case 2:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_CMAC;
      break;
    case 3:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_MAC;
      break;
    case 4:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_CMPY;
      break;
    case 5:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_MPY;
      break;
    case 6:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_NORMACD;
      break;
    case 7:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 1)
	return OPCODE_NORMD;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3118smod_slot1_Slot_smod_slot1_get (insn))
    {
    case 0:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3117smod_slot1_Slot_smod_slot1_get (insn) == 0)
	return OPCODE_SMOD_SCR;
      break;
    case 1:
      if (Field_op0_s16_Slot_smod_slot1_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3829smod_slot1_Slot_smod_slot1_get (insn) == 0)
	return OPCODE_NOP;
      break;
    }
  if (Field_op0_s16_Slot_smod_slot1_get (insn) == 3)
    return OPCODE_SMOD_ALIGN;
  return XTENSA_UNDEFINED;
}

static int
Slot_llr_slot2_decode (const xtensa_insnbuf insn)
{
  switch (Field_dsp340050b49a6c_fld2046_Slot_llr_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_LUT_AR;
      break;
    case 2:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
	  Field_dsp340050b49a6c_fld3230_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_CM2AR_LN;
      break;
    }
  switch (Field_dsp340050b49a6c_fld3191llr_slot2_Slot_llr_slot2_get (insn))
    {
    case 6:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_ABS8;
      break;
    case 7:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_CONJ;
      break;
    case 22:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_LUT_PHASOR;
      break;
    case 23:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_NOT128;
      break;
    case 38:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVCM;
      break;
    case 39:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_TRANS;
      break;
    case 70:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_NCO_UPDATE;
      break;
    }
  if (Field_dsp340050b49a6c_fld3192llr_slot2_Slot_llr_slot2_get (insn) == 54 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_CLRCM;
  if (Field_dsp340050b49a6c_fld3193llr_slot2_Slot_llr_slot2_get (insn) == 55 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_GET_ARGMAX;
  if (Field_dsp340050b49a6c_fld3194llr_slot2_Slot_llr_slot2_get (insn) == 310 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_GET_HSAR;
  if (Field_dsp340050b49a6c_fld3195llr_slot2_Slot_llr_slot2_get (insn) == 311 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_GET_SAR;
  if (Field_dsp340050b49a6c_fld3196llr_slot2_Slot_llr_slot2_get (insn) == 566 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_GET_HSAR2SAR;
  if (Field_dsp340050b49a6c_fld3197llr_slot2_Slot_llr_slot2_get (insn) == 567 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_POP128_0;
  if (Field_dsp340050b49a6c_fld3198llr_slot2_Slot_llr_slot2_get (insn) == 822 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_POP128_1;
  if (Field_dsp340050b49a6c_fld3199llr_slot2_Slot_llr_slot2_get (insn) == 823 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_POP128_2;
  if (Field_dsp340050b49a6c_fld3200llr_slot2_Slot_llr_slot2_get (insn) == 1078 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_GET_MAX;
  if (Field_dsp340050b49a6c_fld3201llr_slot2_Slot_llr_slot2_get (insn) == 1079 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_POP128_3;
  if (Field_dsp340050b49a6c_fld3202llr_slot2_Slot_llr_slot2_get (insn) == 1334 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
    return OPCODE_POP128_4;
  switch (Field_dsp340050b49a6c_fld3203llr_slot2_Slot_llr_slot2_get (insn))
    {
    case 20535:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ128_0;
      break;
    case 20791:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ128_1;
      break;
    case 21047:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ128_2;
      break;
    case 21303:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ128_5;
      break;
    case 21559:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ128_3;
      break;
    case 21815:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ32_0;
      break;
    case 22071:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ32_1;
      break;
    case 22327:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_MOVEQ32_2;
      break;
    }
  if (Field_dsp340050b49a6c_fld3204llr_slot2_Slot_llr_slot2_get (insn) == 11319 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3844_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_MOVEQ128_4;
  if (Field_dsp340050b49a6c_fld3205_Slot_llr_slot2_get (insn) == 7 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3847llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_SET_EXT_REGS;
  switch (Field_dsp340050b49a6c_fld3206_Slot_llr_slot2_get (insn))
    {
    case 71:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld3225_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_SET_ARGMAX;
      break;
    case 327:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
	  Field_dsp340050b49a6c_fld3225_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_SET_NCO;
      break;
    }
  if (Field_dsp340050b49a6c_fld3207llr_slot2_Slot_llr_slot2_get (insn) == 11575 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3844_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_MOVEQ32_3;
  if (Field_dsp340050b49a6c_fld3208llr_slot2_Slot_llr_slot2_get (insn) == 5943 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3843llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3210llr_slot2_Slot_llr_slot2_get (insn) == 411 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3857llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_POP128_5;
  if (Field_dsp340050b49a6c_fld3212_Slot_llr_slot2_get (insn) == 43 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3848llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_SET_HSAR;
  if (Field_dsp340050b49a6c_fld3213llr_slot2_Slot_llr_slot2_get (insn) == 155 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3845llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_GET_NCO;
  if (Field_dsp340050b49a6c_fld3214_Slot_llr_slot2_get (insn) == 1 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3863llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_QREADY;
  if (Field_dsp340050b49a6c_fld3215llr_slot2_Slot_llr_slot2_get (insn) == 327 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3850llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_SET_SAR;
  if (Field_dsp340050b49a6c_fld3216llr_slot2_Slot_llr_slot2_get (insn) == 27 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3849llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_SET_MAX;
  if (Field_dsp340050b49a6c_fld3217_Slot_llr_slot2_get (insn) == 5 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3859llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_POP32_0;
  if (Field_dsp340050b49a6c_fld3218llr_slot2_Slot_llr_slot2_get (insn) == 11 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3851llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_NEGCM;
  if (Field_dsp340050b49a6c_fld3220llr_slot2_Slot_llr_slot2_get (insn) == 9 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3246_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_PUSH128;
  if (Field_dsp340050b49a6c_fld3221llr_slot2_Slot_llr_slot2_get (insn) == 25 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld2046_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_POP32_1;
  if (Field_dsp340050b49a6c_fld3222llr_slot2_Slot_llr_slot2_get (insn) == 25 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3860llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_POP32_2;
  if (Field_dsp340050b49a6c_fld3224llr_slot2_Slot_llr_slot2_get (insn) == 25 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3861llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_POP32_3;
  if (Field_dsp340050b49a6c_fld3226llr_slot2_Slot_llr_slot2_get (insn) == 25 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3864llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_RDTIEP;
  if (Field_dsp340050b49a6c_fld3228llr_slot2_Slot_llr_slot2_get (insn) == 3 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3866llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_ADD16;
  if (Field_dsp340050b49a6c_fld3231llr_slot2_Slot_llr_slot2_get (insn) == 7 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 1 &&
      Field_dsp340050b49a6c_fld3856llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_LUT_WRITE;
  if (Field_dsp340050b49a6c_fld3232llr_slot2_Slot_llr_slot2_get (insn) == 4 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
    return OPCODE_AR2CM_LN;
  switch (Field_dsp340050b49a6c_fld3233_Slot_llr_slot2_get (insn))
    {
    case 2:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 4 &&
	  Field_dsp340050b49a6c_fld2049_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_SUBWRP;
      break;
    case 3:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 4 &&
	  Field_dsp340050b49a6c_fld2029_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ASLM32;
      break;
    }
  if (Field_dsp340050b49a6c_fld3234llr_slot2_Slot_llr_slot2_get (insn) == 20 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
    return OPCODE_AR2CM_DUP;
  if (Field_dsp340050b49a6c_fld3235llr_slot2_Slot_llr_slot2_get (insn) == 133 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
    return OPCODE_SETTIEP;
  if (Field_dsp340050b49a6c_fld3236_Slot_llr_slot2_get (insn) == 27 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2074_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_WRTIEP;
  if (Field_dsp340050b49a6c_fld3237llr_slot2_Slot_llr_slot2_get (insn) == 149 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld2074_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_AR2SAR_DUP;
  if (Field_dsp340050b49a6c_fld3238llr_slot2_Slot_llr_slot2_get (insn) == 85 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3862_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQ;
  if (Field_dsp340050b49a6c_fld3240llr_slot2_Slot_llr_slot2_get (insn) == 53 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3847llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_WRTSIGQ;
  if (Field_dsp340050b49a6c_fld3241llr_slot2_Slot_llr_slot2_get (insn) == 11 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3865llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_WRTBSIGQM;
  if (Field_dsp340050b49a6c_fld3242llr_slot2_Slot_llr_slot2_get (insn) == 3 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3855llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_CLRTIEP;
  switch (Field_dsp340050b49a6c_fld3243llr_slot2_Slot_llr_slot2_get (insn))
    {
    case 48:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_ADDAR2;
      break;
    case 49:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_MOVAR2;
      break;
    case 50:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_PUSH128_PQ;
      break;
    case 51:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_SUBARX;
      break;
    case 52:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_PUSH32;
      break;
    case 53:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 2)
	return OPCODE_SWAPB;
      break;
    }
  if (Field_dsp340050b49a6c_fld3244llr_slot2_Slot_llr_slot2_get (insn) == 1 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 2 &&
      Field_dsp340050b49a6c_fld3853llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_PQ2CM;
  if (Field_dsp340050b49a6c_fld3245llr_slot2_Slot_llr_slot2_get (insn) == 2 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 4 &&
      Field_dsp340050b49a6c_fld3868_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_SUBMEAN;
  if (Field_dsp340050b49a6c_fld3247llr_slot2_Slot_llr_slot2_get (insn) == 3 &&
      Field_op0_s18_Slot_llr_slot2_get (insn) == 4 &&
      Field_dsp340050b49a6c_fld3867llr_slot2_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_ARGMAX8;
  if (Field_op0_s18_Slot_llr_slot2_get (insn) == 5 &&
      Field_dsp340050b49a6c_fld2029_Slot_llr_slot2_get (insn) == 0)
    return OPCODE_PERM;
  switch (Field_s_Slot_llr_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_ASLM;
      break;
    case 1:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_ASRM;
      break;
    case 2:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_CMP8;
      break;
    case 3:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_LSLM;
      break;
    case 4:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_CMP_I;
      break;
    case 5:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_LSRM;
      break;
    case 8:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 1)
	return OPCODE_CMP_R;
      break;
    }
  switch (Field_t_Slot_llr_slot2_get (insn))
    {
    case 0:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_0;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ADD32;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 4)
	return OPCODE_OR128;
      break;
    case 1:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_1;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ADDCM;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 4)
	return OPCODE_SUB32;
      break;
    case 2:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_2;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ADDWRP;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 4)
	return OPCODE_SUBCM;
      break;
    case 3:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_5;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ASL32;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 4)
	return OPCODE_XOR128;
      break;
    case 4:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_3;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_AND128;
      break;
    case 5:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_6;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ASR;
      break;
    case 6:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_7;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ASR32;
      break;
    case 7:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_0;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_EXTUI4;
      break;
    case 8:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND8_4;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_ASL;
      break;
    case 9:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_1;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_LUT;
      break;
    case 10:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_2;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_LUT_IEXT;
      break;
    case 11:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_4;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_MAX8;
      break;
    case 12:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_3;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_LUT_REXT;
      break;
    case 13:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_5;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_MEAN;
      break;
    case 14:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_6;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_MEAN32;
      break;
    case 15:
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 3)
	return OPCODE_MOVCND_7;
      if (Field_op0_s18_Slot_llr_slot2_get (insn) == 0)
	return OPCODE_MIN8;
      break;
    }
  return XTENSA_UNDEFINED;
}

static int
Slot_llr_slot1_decode (const xtensa_insnbuf insn)
{
  if (Field_dsp340050b49a6c_fld3248llr_slot1_Slot_llr_slot1_get (insn) == 0 &&
      Field_op0_s19_Slot_llr_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3870_Slot_llr_slot1_get (insn) == 0)
    return OPCODE_WGHT32;
  if (Field_dsp340050b49a6c_fld3250llr_slot1_Slot_llr_slot1_get (insn) == 1 &&
      Field_op0_s19_Slot_llr_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3869llr_slot1_Slot_llr_slot1_get (insn) == 0)
    return OPCODE_NOP;
  if (Field_dsp340050b49a6c_fld3251llr_slot1_Slot_llr_slot1_get (insn) == 2 &&
      Field_op0_s19_Slot_llr_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3878llr_slot1_Slot_llr_slot1_get (insn) == 0)
    return OPCODE_LLRPRE2;
  if (Field_dsp340050b49a6c_fld3252llr_slot1_Slot_llr_slot1_get (insn) == 3 &&
      Field_op0_s19_Slot_llr_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3875llr_slot1_Slot_llr_slot1_get (insn) == 0)
    return OPCODE_MOVAC_R;
  if (Field_dsp340050b49a6c_fld3253llr_slot1_Slot_llr_slot1_get (insn) == 1 &&
      Field_op0_s19_Slot_llr_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3876llr_slot1_Slot_llr_slot1_get (insn) == 0)
    return OPCODE_ADDAC;
  if (Field_dsp340050b49a6c_fld3254llr_slot1_Slot_llr_slot1_get (insn) == 1 &&
      Field_op0_s19_Slot_llr_slot1_get (insn) == 0 &&
      Field_dsp340050b49a6c_fld3872llr_slot1_Slot_llr_slot1_get (insn) == 0)
    return OPCODE_MOVAC_I;
  if (Field_op0_s19_Slot_llr_slot1_get (insn) == 1)
    return OPCODE_COMB32;
  return XTENSA_UNDEFINED;
}

static int
Slot_dual_slot1_decode (const xtensa_insnbuf insn)
{
  if (Field_op0_s22_Slot_dual_slot1_get (insn) == 0)
    return OPCODE_NOP;
  return XTENSA_UNDEFINED;
}


/* Instruction slots.  */

static void
Slot_x24_Format_inst_0_get (const xtensa_insnbuf insn,
			    xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = (insn[0] & 0xffffff);
}

static void
Slot_x24_Format_inst_0_set (xtensa_insnbuf insn,
			    const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xffffff) | (slotbuf[0] & 0xffffff);
}

static void
Slot_x16a_Format_inst16a_0_get (const xtensa_insnbuf insn,
				xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = (insn[0] & 0xffff);
}

static void
Slot_x16a_Format_inst16a_0_set (xtensa_insnbuf insn,
				const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xffff) | (slotbuf[0] & 0xffff);
}

static void
Slot_x16b_Format_inst16b_0_get (const xtensa_insnbuf insn,
				xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = (insn[0] & 0xffff);
}

static void
Slot_x16b_Format_inst16b_0_set (xtensa_insnbuf insn,
				const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xffff) | (slotbuf[0] & 0xffff);
}

static void
Slot_gp_Format_gp_slot2_43_get (const xtensa_insnbuf insn,
			       xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[1] & 0x3ffff800) >> 11);
}

static void
Slot_gp_Format_gp_slot2_43_set (xtensa_insnbuf insn,
			       const xtensa_insnbuf slotbuf)
{
  insn[1] = (insn[1] & ~0x3ffff800) | ((slotbuf[0] & 0x7ffff) << 11);
}

static void
Slot_gp_Format_gp_slot1_26_get (const xtensa_insnbuf insn,
			       xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xfc000000) >> 26);
  slotbuf[0] = (slotbuf[0] & ~0x1ffc0) | ((insn[1] & 0x7ff) << 6);
}

static void
Slot_gp_Format_gp_slot1_26_set (xtensa_insnbuf insn,
			       const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xfc000000) | ((slotbuf[0] & 0x3f) << 26);
  insn[1] = (insn[1] & ~0x7ff) | ((slotbuf[0] & 0x1ffc0) >> 6);
}

static void
Slot_gp_Format_gp_slot0_7_get (const xtensa_insnbuf insn,
			       xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x3ffff80) >> 7);
}

static void
Slot_gp_Format_gp_slot0_7_set (xtensa_insnbuf insn,
			       const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x3ffff80) | ((slotbuf[0] & 0x7ffff) << 7);
}

static void
Slot_dot_Format_dot_slot2_44_get (const xtensa_insnbuf insn,
				 xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[1] & 0xfffff000) >> 12);
}

static void
Slot_dot_Format_dot_slot2_44_set (xtensa_insnbuf insn,
				 const xtensa_insnbuf slotbuf)
{
  insn[1] = (insn[1] & ~0xfffff000) | ((slotbuf[0] & 0xfffff) << 12);
}

static void
Slot_dot_Format_dot_slot1_24_get (const xtensa_insnbuf insn,
				 xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xff000000) >> 24);
  slotbuf[0] = (slotbuf[0] & ~0xfff00) | ((insn[1] & 0xfff) << 8);
}

static void
Slot_dot_Format_dot_slot1_24_set (xtensa_insnbuf insn,
				 const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xff000000) | ((slotbuf[0] & 0xff) << 24);
  insn[1] = (insn[1] & ~0xfff) | ((slotbuf[0] & 0xfff00) >> 8);
}

static void
Slot_dot_Format_dot_slot0_7_get (const xtensa_insnbuf insn,
				 xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xffff80) >> 7);
}

static void
Slot_dot_Format_dot_slot0_7_set (xtensa_insnbuf insn,
				 const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xffff80) | ((slotbuf[0] & 0x1ffff) << 7);
}

static void
Slot_pq_Format_pq_slot2_40_get (const xtensa_insnbuf insn,
			       xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[1] & 0x7ffff00) >> 8);
}

static void
Slot_pq_Format_pq_slot2_40_set (xtensa_insnbuf insn,
			       const xtensa_insnbuf slotbuf)
{
  insn[1] = (insn[1] & ~0x7ffff00) | ((slotbuf[0] & 0x7ffff) << 8);
}

static void
Slot_pq_Format_pq_slot1_26_get (const xtensa_insnbuf insn,
			       xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xfc000000) >> 26);
  slotbuf[0] = (slotbuf[0] & ~0x3fc0) | ((insn[1] & 0xff) << 6);
}

static void
Slot_pq_Format_pq_slot1_26_set (xtensa_insnbuf insn,
			       const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xfc000000) | ((slotbuf[0] & 0x3f) << 26);
  insn[1] = (insn[1] & ~0xff) | ((slotbuf[0] & 0x3fc0) >> 6);
}

static void
Slot_pq_Format_pq_slot0_7_get (const xtensa_insnbuf insn,
			       xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x3ffff80) >> 7);
}

static void
Slot_pq_Format_pq_slot0_7_set (xtensa_insnbuf insn,
			       const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x3ffff80) | ((slotbuf[0] & 0x7ffff) << 7);
}

static void
Slot_acc2_Format_acc2_slot2_47_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[1] & 0x3fff8000) >> 15);
}

static void
Slot_acc2_Format_acc2_slot2_47_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[1] = (insn[1] & ~0x3fff8000) | ((slotbuf[0] & 0x7fff) << 15);
}

static void
Slot_acc2_Format_acc2_slot1_23_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xff800000) >> 23);
  slotbuf[0] = (slotbuf[0] & ~0xfffe00) | ((insn[1] & 0x7fff) << 9);
}

static void
Slot_acc2_Format_acc2_slot1_23_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xff800000) | ((slotbuf[0] & 0x1ff) << 23);
  insn[1] = (insn[1] & ~0x7fff) | ((slotbuf[0] & 0xfffe00) >> 9);
}

static void
Slot_acc2_Format_acc2_slot0_7_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x7fff80) >> 7);
}

static void
Slot_acc2_Format_acc2_slot0_7_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x7fff80) | ((slotbuf[0] & 0xffff) << 7);
}

static void
Slot_smod_Format_smod_slot2_42_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[1] & 0x3ffffc00) >> 10);
}

static void
Slot_smod_Format_smod_slot2_42_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[1] = (insn[1] & ~0x3ffffc00) | ((slotbuf[0] & 0xfffff) << 10);
}

static void
Slot_smod_Format_smod_slot1_26_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xfc000000) >> 26);
  slotbuf[0] = (slotbuf[0] & ~0xffc0) | ((insn[1] & 0x3ff) << 6);
}

static void
Slot_smod_Format_smod_slot1_26_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xfc000000) | ((slotbuf[0] & 0x3f) << 26);
  insn[1] = (insn[1] & ~0x3ff) | ((slotbuf[0] & 0xffc0) >> 6);
}

static void
Slot_smod_Format_smod_slot0_7_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x3ffff80) >> 7);
}

static void
Slot_smod_Format_smod_slot0_7_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x3ffff80) | ((slotbuf[0] & 0x7ffff) << 7);
}

static void
Slot_llr_Format_llr_slot2_44_get (const xtensa_insnbuf insn,
				 xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[1] & 0x7ffff000) >> 12);
}

static void
Slot_llr_Format_llr_slot2_44_set (xtensa_insnbuf insn,
				 const xtensa_insnbuf slotbuf)
{
  insn[1] = (insn[1] & ~0x7ffff000) | ((slotbuf[0] & 0x7ffff) << 12);
}

static void
Slot_llr_Format_llr_slot1_24_get (const xtensa_insnbuf insn,
				 xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xff000000) >> 24);
  slotbuf[0] = (slotbuf[0] & ~0xfff00) | ((insn[1] & 0xfff) << 8);
}

static void
Slot_llr_Format_llr_slot1_24_set (xtensa_insnbuf insn,
				 const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xff000000) | ((slotbuf[0] & 0xff) << 24);
  insn[1] = (insn[1] & ~0xfff) | ((slotbuf[0] & 0xfff00) >> 8);
}

static void
Slot_llr_Format_llr_slot0_7_get (const xtensa_insnbuf insn,
				 xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0xffff80) >> 7);
}

static void
Slot_llr_Format_llr_slot0_7_set (xtensa_insnbuf insn,
				 const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0xffff80) | ((slotbuf[0] & 0x1ffff) << 7);
}

static void
Slot_dual_Format_dual_slot2_31_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x80000000) >> 31);
  slotbuf[0] = (slotbuf[0] & ~0xfffffe) | ((insn[1] & 0x7fffff) << 1);
}

static void
Slot_dual_Format_dual_slot2_31_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x80000000) | ((slotbuf[0] & 0x1) << 31);
  insn[1] = (insn[1] & ~0x7fffff) | ((slotbuf[0] & 0xfffffe) >> 1);
}

static void
Slot_dual_Format_dual_slot1_30_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x40000000) >> 30);
}

static void
Slot_dual_Format_dual_slot1_30_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x40000000) | ((slotbuf[0] & 0x1) << 30);
}

static void
Slot_dual_Format_dual_slot0_7_get (const xtensa_insnbuf insn,
				   xtensa_insnbuf slotbuf)
{
  slotbuf[1] = 0;
  slotbuf[0] = ((insn[0] & 0x3fffff80) >> 7);
}

static void
Slot_dual_Format_dual_slot0_7_set (xtensa_insnbuf insn,
				   const xtensa_insnbuf slotbuf)
{
  insn[0] = (insn[0] & ~0x3fffff80) | ((slotbuf[0] & 0x7fffff) << 7);
}

static xtensa_get_field_fn
Slot_inst_get_field_fns[] = {
  Field_t_Slot_inst_get,
  Field_bbi4_Slot_inst_get,
  Field_bbi_Slot_inst_get,
  Field_imm12_Slot_inst_get,
  Field_imm8_Slot_inst_get,
  Field_s_Slot_inst_get,
  Field_imm12b_Slot_inst_get,
  Field_imm16_Slot_inst_get,
  Field_m_Slot_inst_get,
  Field_n_Slot_inst_get,
  Field_offset_Slot_inst_get,
  Field_op0_Slot_inst_get,
  Field_op1_Slot_inst_get,
  Field_op2_Slot_inst_get,
  Field_r_Slot_inst_get,
  Field_sa4_Slot_inst_get,
  Field_sae4_Slot_inst_get,
  Field_sae_Slot_inst_get,
  Field_sal_Slot_inst_get,
  Field_sargt_Slot_inst_get,
  Field_sas4_Slot_inst_get,
  Field_sas_Slot_inst_get,
  Field_sr_Slot_inst_get,
  Field_st_Slot_inst_get,
  Field_thi3_Slot_inst_get,
  Field_imm4_Slot_inst_get,
  Field_mn_Slot_inst_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_t2_Slot_inst_get,
  Field_s2_Slot_inst_get,
  Field_r2_Slot_inst_get,
  Field_t4_Slot_inst_get,
  Field_s4_Slot_inst_get,
  Field_r4_Slot_inst_get,
  Field_t8_Slot_inst_get,
  Field_s8_Slot_inst_get,
  Field_r8_Slot_inst_get,
  Field_xt_wbr15_imm_Slot_inst_get,
  Field_xt_wbr18_imm_Slot_inst_get,
  Field_fimm8_Slot_inst_get,
  Field_dsp340050b49a6c_fld2019_Slot_inst_get,
  Field_dsp340050b49a6c_fld2021_Slot_inst_get,
  Field_dsp340050b49a6c_fld2029_Slot_inst_get,
  Field_dsp340050b49a6c_fld2030_Slot_inst_get,
  Field_dsp340050b49a6c_fld2032_Slot_inst_get,
  Field_dsp340050b49a6c_fld2035_Slot_inst_get,
  Field_dsp340050b49a6c_fld2036_Slot_inst_get,
  Field_dsp340050b49a6c_fld2037_Slot_inst_get,
  Field_dsp340050b49a6c_fld2038_Slot_inst_get,
  Field_dsp340050b49a6c_fld2039_Slot_inst_get,
  Field_dsp340050b49a6c_fld2040_Slot_inst_get,
  Field_dsp340050b49a6c_fld2041_Slot_inst_get,
  Field_dsp340050b49a6c_fld2042_Slot_inst_get,
  Field_dsp340050b49a6c_fld2043_Slot_inst_get,
  Field_dsp340050b49a6c_fld2044_Slot_inst_get,
  Field_dsp340050b49a6c_fld2045_Slot_inst_get,
  Field_dsp340050b49a6c_fld2046_Slot_inst_get,
  Field_dsp340050b49a6c_fld2047_Slot_inst_get,
  Field_dsp340050b49a6c_fld2048_Slot_inst_get,
  Field_dsp340050b49a6c_fld2049_Slot_inst_get,
  Field_dsp340050b49a6c_fld2050_Slot_inst_get,
  Field_dsp340050b49a6c_fld2051_Slot_inst_get,
  Field_dsp340050b49a6c_fld2052_Slot_inst_get,
  Field_dsp340050b49a6c_fld2053_Slot_inst_get,
  Field_dsp340050b49a6c_fld2054_Slot_inst_get,
  Field_dsp340050b49a6c_fld2055_Slot_inst_get,
  Field_dsp340050b49a6c_fld2056_Slot_inst_get,
  Field_dsp340050b49a6c_fld2082inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2083inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2084inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2085inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2086inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2088inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2089inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2090inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2091inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2092inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2094inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2095inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2096inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2098inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2099inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2100inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2101inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2102inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2103inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2104inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2105inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2106inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2107inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2108inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2109inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2110inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2111inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2112inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2113inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2114inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2115inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2116inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2117inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2118inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2119inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2120inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2122inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2123inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2124inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2125inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2126inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2127inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2128inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2129inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2131inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2132inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2133inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2134inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2136inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2137inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2138inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2139inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2140inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2141inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2142inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2143inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2144inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2145inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2146inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2147inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2149inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2151inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2153inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2154inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2155inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2156inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2157inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2158inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2159inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2160inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2161inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2162inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2163inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2164inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2165inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2166inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2167inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2168inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2169inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2171inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2172inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2173inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2174inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2175inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2177inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2178inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2179inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2180inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2181inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2182inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2183inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2184inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2185inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2186inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2187inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2188inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2189inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2190inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2191inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2192inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2193inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2194inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2195inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2196inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2197inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2198inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2199inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2200inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2201inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2202inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2203inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2204inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2205inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2206inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2207inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2208inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2209inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2210inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2211inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2212inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2213inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2214inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2215inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2216inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2217inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2218inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2219inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2220inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2221inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2222inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2223inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2224inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2225inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2226inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2227inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2228inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2229inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2230inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2231inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2232inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2234inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2235inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2236inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2237inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2238inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2239inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2240inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2241inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2242inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2243inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2244inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2245inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2246inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2247inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2248inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2249inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2250inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2251inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2252inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2253inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2254_Slot_inst_get,
  Field_dsp340050b49a6c_fld2255inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld2257inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3627inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3630inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3631inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3633inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3634_Slot_inst_get,
  Field_dsp340050b49a6c_fld3635inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3636inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3637inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3638inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3639inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3640inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3642inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3643inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3644inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3645inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3647inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3648inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3649inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3650inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3651inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3653inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3654inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3655inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3656inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3657inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3658inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3659inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3660inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3661inst_Slot_inst_get,
  Field_dsp340050b49a6c_fld3662inst_Slot_inst_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_inst_set_field_fns[] = {
  Field_t_Slot_inst_set,
  Field_bbi4_Slot_inst_set,
  Field_bbi_Slot_inst_set,
  Field_imm12_Slot_inst_set,
  Field_imm8_Slot_inst_set,
  Field_s_Slot_inst_set,
  Field_imm12b_Slot_inst_set,
  Field_imm16_Slot_inst_set,
  Field_m_Slot_inst_set,
  Field_n_Slot_inst_set,
  Field_offset_Slot_inst_set,
  Field_op0_Slot_inst_set,
  Field_op1_Slot_inst_set,
  Field_op2_Slot_inst_set,
  Field_r_Slot_inst_set,
  Field_sa4_Slot_inst_set,
  Field_sae4_Slot_inst_set,
  Field_sae_Slot_inst_set,
  Field_sal_Slot_inst_set,
  Field_sargt_Slot_inst_set,
  Field_sas4_Slot_inst_set,
  Field_sas_Slot_inst_set,
  Field_sr_Slot_inst_set,
  Field_st_Slot_inst_set,
  Field_thi3_Slot_inst_set,
  Field_imm4_Slot_inst_set,
  Field_mn_Slot_inst_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_t2_Slot_inst_set,
  Field_s2_Slot_inst_set,
  Field_r2_Slot_inst_set,
  Field_t4_Slot_inst_set,
  Field_s4_Slot_inst_set,
  Field_r4_Slot_inst_set,
  Field_t8_Slot_inst_set,
  Field_s8_Slot_inst_set,
  Field_r8_Slot_inst_set,
  Field_xt_wbr15_imm_Slot_inst_set,
  Field_xt_wbr18_imm_Slot_inst_set,
  Field_fimm8_Slot_inst_set,
  Field_dsp340050b49a6c_fld2019_Slot_inst_set,
  Field_dsp340050b49a6c_fld2021_Slot_inst_set,
  Field_dsp340050b49a6c_fld2029_Slot_inst_set,
  Field_dsp340050b49a6c_fld2030_Slot_inst_set,
  Field_dsp340050b49a6c_fld2032_Slot_inst_set,
  Field_dsp340050b49a6c_fld2035_Slot_inst_set,
  Field_dsp340050b49a6c_fld2036_Slot_inst_set,
  Field_dsp340050b49a6c_fld2037_Slot_inst_set,
  Field_dsp340050b49a6c_fld2038_Slot_inst_set,
  Field_dsp340050b49a6c_fld2039_Slot_inst_set,
  Field_dsp340050b49a6c_fld2040_Slot_inst_set,
  Field_dsp340050b49a6c_fld2041_Slot_inst_set,
  Field_dsp340050b49a6c_fld2042_Slot_inst_set,
  Field_dsp340050b49a6c_fld2043_Slot_inst_set,
  Field_dsp340050b49a6c_fld2044_Slot_inst_set,
  Field_dsp340050b49a6c_fld2045_Slot_inst_set,
  Field_dsp340050b49a6c_fld2046_Slot_inst_set,
  Field_dsp340050b49a6c_fld2047_Slot_inst_set,
  Field_dsp340050b49a6c_fld2048_Slot_inst_set,
  Field_dsp340050b49a6c_fld2049_Slot_inst_set,
  Field_dsp340050b49a6c_fld2050_Slot_inst_set,
  Field_dsp340050b49a6c_fld2051_Slot_inst_set,
  Field_dsp340050b49a6c_fld2052_Slot_inst_set,
  Field_dsp340050b49a6c_fld2053_Slot_inst_set,
  Field_dsp340050b49a6c_fld2054_Slot_inst_set,
  Field_dsp340050b49a6c_fld2055_Slot_inst_set,
  Field_dsp340050b49a6c_fld2056_Slot_inst_set,
  Field_dsp340050b49a6c_fld2082inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2083inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2084inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2085inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2086inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2088inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2089inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2090inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2091inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2092inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2094inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2095inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2096inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2098inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2099inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2100inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2101inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2102inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2103inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2104inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2105inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2106inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2107inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2108inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2109inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2110inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2111inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2112inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2113inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2114inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2115inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2116inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2117inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2118inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2119inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2120inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2122inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2123inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2124inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2125inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2126inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2127inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2128inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2129inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2131inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2132inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2133inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2134inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2136inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2137inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2138inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2139inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2140inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2141inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2142inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2143inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2144inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2145inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2146inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2147inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2149inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2151inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2153inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2154inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2155inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2156inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2157inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2158inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2159inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2160inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2161inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2162inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2163inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2164inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2165inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2166inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2167inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2168inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2169inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2171inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2172inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2173inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2174inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2175inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2177inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2178inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2179inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2180inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2181inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2182inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2183inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2184inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2185inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2186inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2187inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2188inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2189inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2190inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2191inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2192inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2193inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2194inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2195inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2196inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2197inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2198inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2199inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2200inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2201inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2202inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2203inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2204inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2205inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2206inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2207inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2208inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2209inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2210inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2211inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2212inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2213inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2214inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2215inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2216inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2217inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2218inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2219inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2220inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2221inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2222inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2223inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2224inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2225inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2226inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2227inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2228inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2229inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2230inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2231inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2232inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2234inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2235inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2236inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2237inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2238inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2239inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2240inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2241inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2242inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2243inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2244inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2245inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2246inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2247inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2248inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2249inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2250inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2251inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2252inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2253inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2254_Slot_inst_set,
  Field_dsp340050b49a6c_fld2255inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld2257inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3627inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3630inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3631inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3633inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3634_Slot_inst_set,
  Field_dsp340050b49a6c_fld3635inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3636inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3637inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3638inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3639inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3640inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3642inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3643inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3644inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3645inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3647inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3648inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3649inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3650inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3651inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3653inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3654inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3655inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3656inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3657inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3658inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3659inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3660inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3661inst_Slot_inst_set,
  Field_dsp340050b49a6c_fld3662inst_Slot_inst_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_inst16a_get_field_fns[] = {
  Field_t_Slot_inst16a_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_inst16a_get,
  0,
  0,
  0,
  0,
  0,
  Field_op0_Slot_inst16a_get,
  0,
  0,
  Field_r_Slot_inst16a_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_sr_Slot_inst16a_get,
  Field_st_Slot_inst16a_get,
  0,
  Field_imm4_Slot_inst16a_get,
  0,
  Field_i_Slot_inst16a_get,
  Field_imm6lo_Slot_inst16a_get,
  Field_imm6hi_Slot_inst16a_get,
  Field_imm7lo_Slot_inst16a_get,
  Field_imm7hi_Slot_inst16a_get,
  Field_z_Slot_inst16a_get,
  Field_imm6_Slot_inst16a_get,
  Field_imm7_Slot_inst16a_get,
  Field_t2_Slot_inst16a_get,
  Field_s2_Slot_inst16a_get,
  Field_r2_Slot_inst16a_get,
  Field_t4_Slot_inst16a_get,
  Field_s4_Slot_inst16a_get,
  Field_r4_Slot_inst16a_get,
  Field_t8_Slot_inst16a_get,
  Field_s8_Slot_inst16a_get,
  Field_r8_Slot_inst16a_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_inst16a_set_field_fns[] = {
  Field_t_Slot_inst16a_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_inst16a_set,
  0,
  0,
  0,
  0,
  0,
  Field_op0_Slot_inst16a_set,
  0,
  0,
  Field_r_Slot_inst16a_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_sr_Slot_inst16a_set,
  Field_st_Slot_inst16a_set,
  0,
  Field_imm4_Slot_inst16a_set,
  0,
  Field_i_Slot_inst16a_set,
  Field_imm6lo_Slot_inst16a_set,
  Field_imm6hi_Slot_inst16a_set,
  Field_imm7lo_Slot_inst16a_set,
  Field_imm7hi_Slot_inst16a_set,
  Field_z_Slot_inst16a_set,
  Field_imm6_Slot_inst16a_set,
  Field_imm7_Slot_inst16a_set,
  Field_t2_Slot_inst16a_set,
  Field_s2_Slot_inst16a_set,
  Field_r2_Slot_inst16a_set,
  Field_t4_Slot_inst16a_set,
  Field_s4_Slot_inst16a_set,
  Field_r4_Slot_inst16a_set,
  Field_t8_Slot_inst16a_set,
  Field_s8_Slot_inst16a_set,
  Field_r8_Slot_inst16a_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_inst16b_get_field_fns[] = {
  Field_t_Slot_inst16b_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_inst16b_get,
  0,
  0,
  0,
  0,
  0,
  Field_op0_Slot_inst16b_get,
  0,
  0,
  Field_r_Slot_inst16b_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_sr_Slot_inst16b_get,
  Field_st_Slot_inst16b_get,
  0,
  Field_imm4_Slot_inst16b_get,
  0,
  Field_i_Slot_inst16b_get,
  Field_imm6lo_Slot_inst16b_get,
  Field_imm6hi_Slot_inst16b_get,
  Field_imm7lo_Slot_inst16b_get,
  Field_imm7hi_Slot_inst16b_get,
  Field_z_Slot_inst16b_get,
  Field_imm6_Slot_inst16b_get,
  Field_imm7_Slot_inst16b_get,
  Field_t2_Slot_inst16b_get,
  Field_s2_Slot_inst16b_get,
  Field_r2_Slot_inst16b_get,
  Field_t4_Slot_inst16b_get,
  Field_s4_Slot_inst16b_get,
  Field_r4_Slot_inst16b_get,
  Field_t8_Slot_inst16b_get,
  Field_s8_Slot_inst16b_get,
  Field_r8_Slot_inst16b_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_inst16b_set_field_fns[] = {
  Field_t_Slot_inst16b_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_inst16b_set,
  0,
  0,
  0,
  0,
  0,
  Field_op0_Slot_inst16b_set,
  0,
  0,
  Field_r_Slot_inst16b_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_sr_Slot_inst16b_set,
  Field_st_Slot_inst16b_set,
  0,
  Field_imm4_Slot_inst16b_set,
  0,
  Field_i_Slot_inst16b_set,
  Field_imm6lo_Slot_inst16b_set,
  Field_imm6hi_Slot_inst16b_set,
  Field_imm7lo_Slot_inst16b_set,
  Field_imm7hi_Slot_inst16b_set,
  Field_z_Slot_inst16b_set,
  Field_imm6_Slot_inst16b_set,
  Field_imm7_Slot_inst16b_set,
  Field_t2_Slot_inst16b_set,
  Field_s2_Slot_inst16b_set,
  Field_r2_Slot_inst16b_set,
  Field_t4_Slot_inst16b_set,
  Field_s4_Slot_inst16b_set,
  Field_r4_Slot_inst16b_set,
  Field_t8_Slot_inst16b_set,
  Field_s8_Slot_inst16b_set,
  Field_r8_Slot_inst16b_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_gp_slot2_get_field_fns[] = {
  Field_t_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_gp_slot2_get,
  Field_r_Slot_gp_slot2_get,
  0,
  0,
  Field_sae_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2030_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2032_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_gp_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_gp_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2045_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2047_Slot_gp_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2050_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2051_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2052_Slot_gp_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2056_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s3_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2025_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2027_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2258gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2259gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2260gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2261gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2262gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2263gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2264gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2266gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2267gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2268gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2269gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2270gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2271gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2272_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2273gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2274gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2275gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2277gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2278gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2279gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2280gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2281gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2282gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2283gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2284gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2286gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2287gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2288gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2289gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2290gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2291gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2292gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2293gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2294gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2295gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2296gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2297gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2298gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2299gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2300gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2301gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2302_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2303gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2304gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2305_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2306gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2308gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2309gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2310gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2312gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2314gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2316gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2317gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2318gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2319gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2320gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2321gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2322gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2323gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2324gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2325gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2326gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2327gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2328gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2329gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2330gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2331gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2332gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2333gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2334gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2335gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2336gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2337gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2338gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2339gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2340gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2341gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2342gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2343gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2344gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2345gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2346gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2347gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2348gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2349gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2350gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2351gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2352gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2353gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2354gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2355gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2356gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2357gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2358gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2359gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2361gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2362gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2364gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2366gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2368gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2369gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2370gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2371gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2372gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2373gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2374gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2375gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2376gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2378gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2379gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2381gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2383gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2384_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2385gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2386_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2387gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2388gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld2389gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3663gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3664gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3665gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3666_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3667gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3668gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3669gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3670gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3671gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3673gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3674gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3675gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3676gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3678gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3679gp_slot2_Slot_gp_slot2_get,
  Field_dsp340050b49a6c_fld3680gp_slot2_Slot_gp_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_gp_slot2_set_field_fns[] = {
  Field_t_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_gp_slot2_set,
  Field_r_Slot_gp_slot2_set,
  0,
  0,
  Field_sae_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2030_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2032_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_gp_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_gp_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2045_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2047_Slot_gp_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2050_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2051_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2052_Slot_gp_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2056_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s3_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2025_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2027_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2258gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2259gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2260gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2261gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2262gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2263gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2264gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2266gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2267gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2268gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2269gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2270gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2271gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2272_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2273gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2274gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2275gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2277gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2278gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2279gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2280gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2281gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2282gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2283gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2284gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2286gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2287gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2288gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2289gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2290gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2291gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2292gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2293gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2294gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2295gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2296gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2297gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2298gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2299gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2300gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2301gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2302_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2303gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2304gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2305_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2306gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2308gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2309gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2310gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2312gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2313_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2314gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2316gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2317gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2318gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2319gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2320gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2321gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2322gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2323gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2324gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2325gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2326gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2327gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2328gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2329gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2330gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2331gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2332gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2333gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2334gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2335gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2336gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2337gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2338gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2339gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2340gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2341gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2342gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2343gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2344gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2345gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2346gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2347gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2348gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2349gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2350gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2351gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2352gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2353gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2354gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2355gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2356gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2357gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2358gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2359gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2361gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2362gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2364gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2366gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2368gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2369gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2370gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2371gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2372gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2373gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2374gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2375gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2376gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2378gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2379gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2381gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2383gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2384_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2385gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2386_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2387gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2388gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld2389gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3663gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3664gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3665gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3666_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3667gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3668gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3669gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3670gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3671gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3673gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3674gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3675gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3676gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3678gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3679gp_slot2_Slot_gp_slot2_set,
  Field_dsp340050b49a6c_fld3680gp_slot2_Slot_gp_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_gp_slot1_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  Field_s_Slot_gp_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2046_Slot_gp_slot1_get,
  0,
  Field_dsp340050b49a6c_fld2048_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2049_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2050_Slot_gp_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s4_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2026_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2031_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2394gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2395gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2397gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2398gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2399gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2400gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2402gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2403gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld2405gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld3681gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld3683gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld3684gp_slot1_Slot_gp_slot1_get,
  Field_dsp340050b49a6c_fld3686gp_slot1_Slot_gp_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_gp_slot1_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  Field_s_Slot_gp_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2046_Slot_gp_slot1_set,
  0,
  Field_dsp340050b49a6c_fld2048_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2049_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2050_Slot_gp_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s4_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2026_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2031_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2394gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2395gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2397gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2398gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2399gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2400gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2402gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2403gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld2405gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld3681gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld3683gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld3684gp_slot1_Slot_gp_slot1_set,
  Field_dsp340050b49a6c_fld3686gp_slot1_Slot_gp_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_gp_slot0_get_field_fns[] = {
  Field_t_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_gp_slot0_get,
  Field_r_Slot_gp_slot0_get,
  0,
  0,
  Field_sae_Slot_gp_slot0_get,
  Field_sal_Slot_gp_slot0_get,
  Field_sargt_Slot_gp_slot0_get,
  0,
  Field_sas_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_gp_slot0_get,
  Field_imm7_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  Field_s4_Slot_gp_slot0_get,
  0,
  0,
  Field_s8_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2030_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2032_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2035_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2036_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2037_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2038_Slot_gp_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2040_Slot_gp_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2042_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2043_Slot_gp_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2048_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2049_Slot_gp_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2051_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2052_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2053_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2054_Slot_gp_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s5_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2058_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2067_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2407gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2409gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2410gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2411gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2412gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2413gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2415gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2416gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2417gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2418gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2419gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2420gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2422gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2423gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2424gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2425gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2426gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2427gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2429gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2430gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2431gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2432gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2433gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2434gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2435gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2436gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2437gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2438gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2439gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2440gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2441gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2443gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2444gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2445_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2447gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2448_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2449gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2451gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2452gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2453gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2454gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2455gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2456gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2457gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2458gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2459gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2460gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2461gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2462_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2463gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2464gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2465gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2466gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2467gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2468gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2470gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2471gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2472gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2473gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2474gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2475gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2477gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2479gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2480gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2481gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2482gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2483gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2484gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2485gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2486gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2487gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2488gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2489gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2490gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2491gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2492gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2493gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2494gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2495gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2496gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2497gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2498gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2499gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2500gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2501gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2502gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2503gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2504gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2505gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2506gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2507gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2508gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2509gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2510gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2512gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2514gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2515gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2516gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2517gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2518gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2519gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2520gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2521gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2523gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2524gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2526gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2527gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2528gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2529gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2530_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld2531gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3688gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3689gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3690gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3691gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3692gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3693gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3696gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3697gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3698gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3699gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3700gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3702gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3703gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3705gp_slot0_Slot_gp_slot0_get,
  Field_dsp340050b49a6c_fld3706gp_slot0_Slot_gp_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_gp_slot0_set_field_fns[] = {
  Field_t_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_gp_slot0_set,
  Field_r_Slot_gp_slot0_set,
  0,
  0,
  Field_sae_Slot_gp_slot0_set,
  Field_sal_Slot_gp_slot0_set,
  Field_sargt_Slot_gp_slot0_set,
  0,
  Field_sas_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_gp_slot0_set,
  Field_imm7_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  Field_s4_Slot_gp_slot0_set,
  0,
  0,
  Field_s8_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2030_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2032_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2035_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2036_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2037_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2038_Slot_gp_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2040_Slot_gp_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2042_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2043_Slot_gp_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2048_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2049_Slot_gp_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2051_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2052_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2053_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2054_Slot_gp_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s5_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2058_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2067_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2407gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2409gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2410gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2411gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2412gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2413gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2415gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2416gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2417gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2418gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2419gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2420gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2422gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2423gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2424gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2425gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2426gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2427gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2429gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2430gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2431gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2432gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2433gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2434gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2435gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2436gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2437gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2438gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2439gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2440gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2441gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2443gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2444gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2445_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2447gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2448_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2449gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2451gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2452gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2453gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2454gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2455gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2456gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2457gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2458gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2459gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2460gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2461gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2462_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2463gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2464gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2465gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2466gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2467gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2468gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2470gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2471gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2472gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2473gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2474gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2475gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2477gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2479gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2480gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2481gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2482gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2483gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2484gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2485gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2486gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2487gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2488gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2489gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2490gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2491gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2492gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2493gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2494gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2495gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2496gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2497gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2498gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2499gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2500gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2501gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2502gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2503gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2504gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2505gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2506gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2507gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2508gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2509gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2510gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2512gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2514gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2515gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2516gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2517gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2518gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2519gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2520gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2521gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2523gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2524gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2526gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2527gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2528gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2529gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2530_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld2531gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3688gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3689gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3690gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3691gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3692gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3693gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3695gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3696gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3697gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3698gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3699gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3700gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3702gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3703gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3705gp_slot0_Slot_gp_slot0_set,
  Field_dsp340050b49a6c_fld3706gp_slot0_Slot_gp_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_dot_slot2_get_field_fns[] = {
  Field_t_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2030_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2032_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_dot_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_dot_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2045_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2047_Slot_dot_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2050_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2051_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2052_Slot_dot_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2056_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2027_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s6_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2532dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2533dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2534dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2535dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2536dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2537dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2538dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2539dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2540dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2541dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2542dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2543dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2544dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2545dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2546dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2547dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2548dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2549dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2550dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2551dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2552dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2553dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2554dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2555dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2556dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2557dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2558dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2559dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2560dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2561dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2562dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2563dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2564dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2565dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2566dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2567dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2568dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2569dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2571dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2572dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2573dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2574dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2575dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2576dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2577dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2578_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2579dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2580dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2581dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2582dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2583dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2584dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2585dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2586dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2587dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2588dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2589dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2590dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2591dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2592dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2595dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2596dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2598dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2599dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2601dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2602dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2604dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2605_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2606dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2608dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2609dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2610dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2611dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2612_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2613dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2614_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2615dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2616dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2617dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2618dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2619dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2620dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2621dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2622dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2623dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2624dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2625_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2626dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2628dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2630dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2632dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2633dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2635dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2636dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2637dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2640dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2641dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2642dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2643dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2644dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2645dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2646dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2647dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2648dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2649dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2650dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2651dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2652dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2654dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2655dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2656dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2657dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld2658dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3708dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3709dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3710dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3711dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3712_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3713dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3714dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3715_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3716dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3717dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3718_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3719dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3721dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3722_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3723dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3724dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3725dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3726dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3727dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3728dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3729dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3731dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3732dot_slot2_Slot_dot_slot2_get,
  Field_dsp340050b49a6c_fld3733dot_slot2_Slot_dot_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_dot_slot2_set_field_fns[] = {
  Field_t_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2030_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2032_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_dot_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_dot_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2045_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2047_Slot_dot_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2050_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2051_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2052_Slot_dot_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2056_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2027_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s6_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2532dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2533dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2534dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2535dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2536dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2537dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2538dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2539dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2540dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2541dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2542dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2543dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2544dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2545dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2546dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2547dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2548dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2549dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2550dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2551dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2552dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2553dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2554dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2555dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2556dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2557dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2558dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2559dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2560dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2561dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2562dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2563dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2564dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2565dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2566dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2567dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2568dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2569dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2571dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2572dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2573dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2574dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2575dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2576dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2577dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2578_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2579dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2580dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2581dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2582dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2583dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2584dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2585dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2586dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2587dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2588dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2589dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2590dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2591dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2592dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2595dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2596dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2598dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2599dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2601dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2602dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2604dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2605_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2606dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2608dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2609dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2610dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2611dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2612_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2613dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2614_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2615dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2616dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2617dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2618dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2619dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2620dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2621dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2622dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2623dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2624dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2625_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2626dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2628dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2630dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2632dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2633dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2635dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2636dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2637dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2640dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2641dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2642dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2643dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2644dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2645dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2646dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2647dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2648dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2649dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2650dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2651dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2652dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2654dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2655dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2656dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2657dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld2658dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3708dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3709dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3710dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3711dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3712_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3713dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3714dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3715_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3716dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3717dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3718_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3719dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3721dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3722_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3723dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3724dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3725dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3726dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3727dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3728dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3729dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3731dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3732dot_slot2_Slot_dot_slot2_set,
  Field_dsp340050b49a6c_fld3733dot_slot2_Slot_dot_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_dot_slot1_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2048_Slot_dot_slot1_get,
  0,
  Field_dsp340050b49a6c_fld2050_Slot_dot_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_dot_slot1_get,
  Field_dsp340050b49a6c_fld2031_Slot_dot_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s7_Slot_dot_slot1_get,
  Field_dsp340050b49a6c_fld3734dot_slot1_Slot_dot_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_dot_slot1_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2048_Slot_dot_slot1_set,
  0,
  Field_dsp340050b49a6c_fld2050_Slot_dot_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_dot_slot1_set,
  Field_dsp340050b49a6c_fld2031_Slot_dot_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s7_Slot_dot_slot1_set,
  Field_dsp340050b49a6c_fld3734dot_slot1_Slot_dot_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_dot_slot0_get_field_fns[] = {
  Field_t_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_dot_slot0_get,
  0,
  0,
  0,
  Field_sal_Slot_dot_slot0_get,
  Field_sargt_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dot_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2032_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2035_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2036_Slot_dot_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2038_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2043_Slot_dot_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2048_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2049_Slot_dot_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2053_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2054_Slot_dot_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s8_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2068_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2666dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2667dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2668dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2669dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2671dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2672dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2673dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2674dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2675dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2676dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2677dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2678dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2679dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2680dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2681dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2682dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2683dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2684dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2685dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2686dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2688dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2689dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2690dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2692dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2693dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2695dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2697dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2699dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2700dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2701dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2702dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2703dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2704dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld2705dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3735dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3736_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3737dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3738dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3739dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3740dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3741dot_slot0_Slot_dot_slot0_get,
  Field_dsp340050b49a6c_fld3742dot_slot0_Slot_dot_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_dot_slot0_set_field_fns[] = {
  Field_t_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_dot_slot0_set,
  0,
  0,
  0,
  Field_sal_Slot_dot_slot0_set,
  Field_sargt_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dot_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2032_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2035_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2036_Slot_dot_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2038_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2043_Slot_dot_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2048_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2049_Slot_dot_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2053_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2054_Slot_dot_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s8_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2068_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2666dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2667dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2668dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2669dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2671dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2672dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2673dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2674dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2675dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2676dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2677dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2678dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2679dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2680dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2681dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2682dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2683dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2684dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2685dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2686dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2688dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2689dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2690dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2692dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2693dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2695dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2697dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2699dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2700dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2701dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2702dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2703dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2704dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld2705dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3735dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3736_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3737dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3738dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3739dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3740dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3741dot_slot0_Slot_dot_slot0_set,
  Field_dsp340050b49a6c_fld3742dot_slot0_Slot_dot_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_pq_slot2_get_field_fns[] = {
  Field_t_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_pq_slot2_get,
  Field_r_Slot_pq_slot2_get,
  0,
  0,
  Field_sae_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2030_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2032_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2045_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2047_Slot_pq_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2050_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2051_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2052_Slot_pq_slot2_get,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2027_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s9_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2706pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2707pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2708pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2709pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2710pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2711pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2713pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2714pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2715pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2717pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2718pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2719pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2721pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2722pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2723pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2724pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2725pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2726pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2727pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2728pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2729pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2730pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2731pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2732pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2733pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2734pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2735pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2736pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2737_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2738pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2739pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2741pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2742pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2743pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2746pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2747pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2748pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2750pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2751pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2752pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2753pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2754pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2755pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2756pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2757pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2758pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2759pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2760pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2761pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2762pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2763pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2764pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2765pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2766pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2767pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2768pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2769pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2770pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2771pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2772pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2773pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2774pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2775pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2776pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2777pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2778pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2779pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2780pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2781pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2782pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2783pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2784pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2785pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2786pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2787pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2788pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2789pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2790pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2791pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2792pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2793pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2795pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2796pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2798pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2801pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2803pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2805pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2806pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2807pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2808pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2809pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2810pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2811pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2812pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2814pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2816pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2818pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2820pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2821pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld2823pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3744pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3745pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3746pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3747pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3748_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3749pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3750pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3751pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3752pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3753pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3754pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3756pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3757pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3758pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3759pq_slot2_Slot_pq_slot2_get,
  Field_dsp340050b49a6c_fld3760pq_slot2_Slot_pq_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_pq_slot2_set_field_fns[] = {
  Field_t_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_pq_slot2_set,
  Field_r_Slot_pq_slot2_set,
  0,
  0,
  Field_sae_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2030_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2032_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2045_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2047_Slot_pq_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2050_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2051_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2052_Slot_pq_slot2_set,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2027_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s9_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2706pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2707pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2708pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2709pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2710pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2711pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2713pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2714pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2715pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2717pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2718pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2719pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2721pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2722pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2723pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2724pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2725pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2726pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2727pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2728pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2729pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2730pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2731pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2732pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2733pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2734pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2735pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2736pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2737_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2738pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2739pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2741pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2742pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2743pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2746pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2747pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2748pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2750pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2751pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2752pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2753pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2754pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2755pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2756pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2757pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2758pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2759pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2760pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2761pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2762pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2763pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2764pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2765pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2766pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2767pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2768pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2769pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2770pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2771pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2772pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2773pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2774pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2775pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2776pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2777pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2778pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2779pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2780pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2781pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2782pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2783pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2784pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2785pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2786pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2787pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2788pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2789pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2790pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2791pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2792pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2793pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2795pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2796pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2798pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2801pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2803pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2805pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2806pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2807pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2808pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2809pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2810pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2811pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2812pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2814pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2816pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2817_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2818pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2819_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2820pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2821pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld2823pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3744pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3745pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3746pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3747pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3748_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3749pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3750pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3751pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3752pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3753pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3754pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3756pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3757pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3758pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3759pq_slot2_Slot_pq_slot2_set,
  Field_dsp340050b49a6c_fld3760pq_slot2_Slot_pq_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_pq_slot1_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_pq_slot1_get,
  Field_dsp340050b49a6c_fld2050_Slot_pq_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_pq_slot1_get,
  Field_dsp340050b49a6c_fld2031_Slot_pq_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s10_Slot_pq_slot1_get,
  Field_dsp340050b49a6c_fld2825pq_slot1_Slot_pq_slot1_get,
  Field_dsp340050b49a6c_fld2826pq_slot1_Slot_pq_slot1_get,
  Field_dsp340050b49a6c_fld3761pq_slot1_Slot_pq_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_pq_slot1_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_pq_slot1_set,
  Field_dsp340050b49a6c_fld2050_Slot_pq_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_pq_slot1_set,
  Field_dsp340050b49a6c_fld2031_Slot_pq_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s10_Slot_pq_slot1_set,
  Field_dsp340050b49a6c_fld2825pq_slot1_Slot_pq_slot1_set,
  Field_dsp340050b49a6c_fld2826pq_slot1_Slot_pq_slot1_set,
  Field_dsp340050b49a6c_fld3761pq_slot1_Slot_pq_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_pq_slot0_get_field_fns[] = {
  Field_t_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_pq_slot0_get,
  Field_r_Slot_pq_slot0_get,
  0,
  0,
  Field_sae_Slot_pq_slot0_get,
  Field_sal_Slot_pq_slot0_get,
  Field_sargt_Slot_pq_slot0_get,
  0,
  Field_sas_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_pq_slot0_get,
  Field_imm7_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  Field_s4_Slot_pq_slot0_get,
  0,
  0,
  Field_s8_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2030_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2032_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2035_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2036_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2037_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2038_Slot_pq_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2040_Slot_pq_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2042_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2043_Slot_pq_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2048_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2049_Slot_pq_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2051_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2052_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2053_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2054_Slot_pq_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s11_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2059_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2069_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2827pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2829pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2830pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2831pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2832pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2833pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2835pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2836pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2837pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2838pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2839pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2840pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2842pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2843pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2844pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2845pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2846pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2847pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2849pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2850pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2851pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2852pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2853pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2854pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2855pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2856pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2857pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2858pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2859pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2860pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2861pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2863pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2864pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2865_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2867pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2869pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2871pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2872pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2873pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2874pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2875pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2876pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2877pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2878pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2879pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2880pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2881pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2882_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2883pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2884pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2885pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2886pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2887pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2888pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2890pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2891pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2892pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2893pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2894pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2895pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2897pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2899pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2900pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2901pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2902pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2903pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2904pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2905pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2906pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2907pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2908pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2909pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2910pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2911pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2912pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2913pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2914pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2915pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2916pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2917pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2918pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2919pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2920pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2921pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2922pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2923pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2924pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2925pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2926pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2927pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2928pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2929pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2930pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2932pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2934pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2935pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2936pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2937pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2939pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2941pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2942pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2943pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2945pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2946pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2947pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2948pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2949pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld2950pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3763pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3764pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3765pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3766pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3767pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3768pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3769pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3770pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3771pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3772_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3773pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3775pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3776pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3777pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3778pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3779pq_slot0_Slot_pq_slot0_get,
  Field_dsp340050b49a6c_fld3780pq_slot0_Slot_pq_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_pq_slot0_set_field_fns[] = {
  Field_t_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_pq_slot0_set,
  Field_r_Slot_pq_slot0_set,
  0,
  0,
  Field_sae_Slot_pq_slot0_set,
  Field_sal_Slot_pq_slot0_set,
  Field_sargt_Slot_pq_slot0_set,
  0,
  Field_sas_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_pq_slot0_set,
  Field_imm7_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  Field_s4_Slot_pq_slot0_set,
  0,
  0,
  Field_s8_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2030_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2032_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2035_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2036_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2037_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2038_Slot_pq_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2040_Slot_pq_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2042_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2043_Slot_pq_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2048_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2049_Slot_pq_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2051_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2052_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2053_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2054_Slot_pq_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s11_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2059_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2069_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2827pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2829pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2830pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2831pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2832pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2833pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2835pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2836pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2837pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2838pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2839pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2840pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2842pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2843pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2844pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2845pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2846pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2847pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2849pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2850pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2851pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2852pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2853pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2854pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2855pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2856pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2857pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2858pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2859pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2860pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2861pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2863pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2864pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2865_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2867pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2869pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2871pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2872pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2873pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2874pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2875pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2876pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2877pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2878pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2879pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2880pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2881pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2882_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2883pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2884pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2885pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2886pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2887pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2888pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2890pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2891pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2892pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2893pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2894pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2895pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2897pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2899pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2900pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2901pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2902pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2903pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2904pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2905pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2906pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2907pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2908pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2909pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2910pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2911pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2912pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2913pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2914pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2915pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2916pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2917pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2918pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2919pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2920pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2921pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2922pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2923pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2924pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2925pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2926pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2927pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2928pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2929pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2930pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2932pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2934pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2935pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2936pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2937pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2939pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2940_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2941pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2942pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2943pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2945pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2946pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2947pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2948pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2949pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld2950pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3763pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3764pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3765pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3766pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3767pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3768pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3769pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3770pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3771pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3772_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3773pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3775pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3776pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3777pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3778pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3779pq_slot0_Slot_pq_slot0_set,
  Field_dsp340050b49a6c_fld3780pq_slot0_Slot_pq_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_acc2_slot2_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  Field_s_Slot_acc2_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_acc2_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_get,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_acc2_slot2_get,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_acc2_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2027_Slot_acc2_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s12_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2953acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2954acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2955acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2956acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2957acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2958acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2959acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2960acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2963acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2964acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2966acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld2967acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld3782acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld3783acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld3784acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld3785acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld3786acc2_slot2_Slot_acc2_slot2_get,
  Field_dsp340050b49a6c_fld3788acc2_slot2_Slot_acc2_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_acc2_slot2_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  Field_s_Slot_acc2_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_acc2_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_acc2_slot2_set,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_acc2_slot2_set,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_acc2_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2027_Slot_acc2_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s12_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2953acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2954acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2955acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2956acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2957acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2958acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2959acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2960acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2963acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2964acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2966acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld2967acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld3782acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld3783acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld3784acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld3785acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld3786acc2_slot2_Slot_acc2_slot2_set,
  Field_dsp340050b49a6c_fld3788acc2_slot2_Slot_acc2_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_acc2_slot1_get_field_fns[] = {
  Field_t_Slot_acc2_slot1_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_acc2_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2047_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld2048_Slot_acc2_slot1_get,
  0,
  Field_dsp340050b49a6c_fld2050_Slot_acc2_slot1_get,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_acc2_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld2031_Slot_acc2_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s13_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld2028_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld2075_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld2968acc2_slot1_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld2969acc2_slot1_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld3790acc2_slot1_Slot_acc2_slot1_get,
  Field_dsp340050b49a6c_fld3793acc2_slot1_Slot_acc2_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_acc2_slot1_set_field_fns[] = {
  Field_t_Slot_acc2_slot1_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_acc2_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2047_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld2048_Slot_acc2_slot1_set,
  0,
  Field_dsp340050b49a6c_fld2050_Slot_acc2_slot1_set,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_acc2_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld2031_Slot_acc2_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s13_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld2028_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld2075_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld2968acc2_slot1_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld2969acc2_slot1_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld3790acc2_slot1_Slot_acc2_slot1_set,
  Field_dsp340050b49a6c_fld3793acc2_slot1_Slot_acc2_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_acc2_slot0_get_field_fns[] = {
  Field_t_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_acc2_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_acc2_slot0_get,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_acc2_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2050_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2056_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s14_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2973acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2974acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2975acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2976acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2977acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2980acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2981acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2982acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2984acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2985acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2987acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2989acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld2990acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3795acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3796acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3797_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3798acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3799acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3800acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3801acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3802acc2_slot0_Slot_acc2_slot0_get,
  Field_dsp340050b49a6c_fld3803acc2_slot0_Slot_acc2_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_acc2_slot0_set_field_fns[] = {
  Field_t_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_acc2_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_acc2_slot0_set,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_acc2_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2050_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2056_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s14_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2973acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2974acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2975acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2976acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2977acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2980acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2981acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2982acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2984acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2985acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2987acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2989acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld2990acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3795acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3796acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3797_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3798acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3799acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3800acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3801acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3802acc2_slot0_Slot_acc2_slot0_set,
  Field_dsp340050b49a6c_fld3803acc2_slot0_Slot_acc2_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_smod_slot2_get_field_fns[] = {
  Field_t_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2030_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2032_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_smod_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_smod_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2045_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2047_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2048_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2049_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2050_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2051_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2052_Slot_smod_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2056_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2027_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s15_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2991smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2992smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2993smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2994smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2995smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2996smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2997smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2998smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld2999smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3000smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3001smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3002smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3003smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3004smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3005smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3006smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3007smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3008smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3009smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3010smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3011smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3012smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3013smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3014smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3015smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3016smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3017smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3018smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3019smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3020smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3021smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3022smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3023smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3024smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3025smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3026smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3027smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3028smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3030smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3031smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3032smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3033smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3034smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3035smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3036smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3038smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3039smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3040smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3043smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3044smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3046smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3047smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3048_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3049smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3050smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3051smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3052smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3053smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3054smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3055smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3056smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3058smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3059smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3061smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3062_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3063smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3065smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3066smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3067smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3068smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3069smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3070_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3071smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3072smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3073smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3074smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3075smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3076smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3077smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3078smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3079smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3080smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3081smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3082smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3084smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3085smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3087smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3088smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3090smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3091smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3092smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3093smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3096smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3097smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3098smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3099smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3100smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3101smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3102smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3104smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3105smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3106smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3107smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3108smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3109smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3110smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3111smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3113smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3114smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3115smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3116smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3805smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3806smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3807smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3808_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3809smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3810smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3812smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3813smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3814smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3816smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3817_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3818smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3819smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3821smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3822smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3823smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3824smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3825smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3826smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3827smod_slot2_Slot_smod_slot2_get,
  Field_dsp340050b49a6c_fld3828smod_slot2_Slot_smod_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_smod_slot2_set_field_fns[] = {
  Field_t_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2030_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2032_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2039_Slot_smod_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_smod_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2045_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2047_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2048_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2049_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2050_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2051_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2052_Slot_smod_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2056_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2027_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s15_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2991smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2992smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2993smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2994smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2995smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2996smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2997smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2998smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld2999smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3000smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3001smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3002smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3003smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3004smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3005smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3006smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3007smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3008smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3009smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3010smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3011smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3012smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3013smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3014smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3015smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3016smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3017smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3018smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3019smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3020smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3021smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3022smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3023smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3024smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3025smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3026smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3027smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3028smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3030smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3031smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3032smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3033smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3034smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3035smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3036smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3038smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3039smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3040smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3043smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3044smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3046smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3047smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3048_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3049smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3050smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3051smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3052smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3053smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3054smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3055smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3056smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3058smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3059smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3061smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3062_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3063smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3065smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3066smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3067smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3068smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3069smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3070_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3071smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3072smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3073smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3074smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3075smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3076smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3077smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3078smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3079smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3080smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3081smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3082smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3084smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3085smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3087smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3088smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3090smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3091smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3092smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3093smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3096smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3097smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3098smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3099smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3100smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3101smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3102smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3104smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3105smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3106smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3107smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3108smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3109smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3110smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3111smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3113smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3114smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3115smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3116smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3805smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3806smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3807smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3808_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3809smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3810smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3812smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3813smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3814smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3816smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3817_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3818smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3819smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3821smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3822smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3823smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3824smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3825smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3826smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3827smod_slot2_Slot_smod_slot2_set,
  Field_dsp340050b49a6c_fld3828smod_slot2_Slot_smod_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_smod_slot1_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  Field_s_Slot_smod_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2046_Slot_smod_slot1_get,
  0,
  Field_dsp340050b49a6c_fld2048_Slot_smod_slot1_get,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2054_Slot_smod_slot1_get,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_smod_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_smod_slot1_get,
  Field_dsp340050b49a6c_fld2031_Slot_smod_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s16_Slot_smod_slot1_get,
  Field_dsp340050b49a6c_fld2033_Slot_smod_slot1_get,
  Field_dsp340050b49a6c_fld2080_Slot_smod_slot1_get,
  Field_dsp340050b49a6c_fld3117smod_slot1_Slot_smod_slot1_get,
  Field_dsp340050b49a6c_fld3118smod_slot1_Slot_smod_slot1_get,
  Field_dsp340050b49a6c_fld3829smod_slot1_Slot_smod_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_smod_slot1_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  Field_s_Slot_smod_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2046_Slot_smod_slot1_set,
  0,
  Field_dsp340050b49a6c_fld2048_Slot_smod_slot1_set,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2054_Slot_smod_slot1_set,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_smod_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_smod_slot1_set,
  Field_dsp340050b49a6c_fld2031_Slot_smod_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s16_Slot_smod_slot1_set,
  Field_dsp340050b49a6c_fld2033_Slot_smod_slot1_set,
  Field_dsp340050b49a6c_fld2080_Slot_smod_slot1_set,
  Field_dsp340050b49a6c_fld3117smod_slot1_Slot_smod_slot1_set,
  Field_dsp340050b49a6c_fld3118smod_slot1_Slot_smod_slot1_set,
  Field_dsp340050b49a6c_fld3829smod_slot1_Slot_smod_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_smod_slot0_get_field_fns[] = {
  Field_t_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_smod_slot0_get,
  Field_r_Slot_smod_slot0_get,
  0,
  0,
  Field_sae_Slot_smod_slot0_get,
  Field_sal_Slot_smod_slot0_get,
  Field_sargt_Slot_smod_slot0_get,
  0,
  Field_sas_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_smod_slot0_get,
  Field_imm7_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  Field_s4_Slot_smod_slot0_get,
  0,
  0,
  Field_s8_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_smod_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2032_Slot_smod_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2037_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld2038_Slot_smod_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2040_Slot_smod_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2043_Slot_smod_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_smod_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_smod_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld2053_Slot_smod_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s17_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3119smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3120smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3121smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3122smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3123smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3125smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3126smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3127smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3128smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3129smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3130smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3131smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3132smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3133smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3134smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3135smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3136smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3137smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3138smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3139smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3140smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3141smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3142smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3143smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3144smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3145smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3146smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3148smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3149smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3150_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3152smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3153smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3155smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3156smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3157smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3158smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3159smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3160smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3161smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3164smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3165smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3166smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3168smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3170smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3171smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3172smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3173smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3174smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3175smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3176smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3177smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3178smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3179smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3180smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3181smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3182smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3184smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3186smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3188smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3832smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3833smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3836smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3837smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3838smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3841smod_slot0_Slot_smod_slot0_get,
  Field_dsp340050b49a6c_fld3842smod_slot0_Slot_smod_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_smod_slot0_set_field_fns[] = {
  Field_t_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_smod_slot0_set,
  Field_r_Slot_smod_slot0_set,
  0,
  0,
  Field_sae_Slot_smod_slot0_set,
  Field_sal_Slot_smod_slot0_set,
  Field_sargt_Slot_smod_slot0_set,
  0,
  Field_sas_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_smod_slot0_set,
  Field_imm7_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  Field_s4_Slot_smod_slot0_set,
  0,
  0,
  Field_s8_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_smod_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2032_Slot_smod_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2037_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld2038_Slot_smod_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2040_Slot_smod_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2043_Slot_smod_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_smod_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_smod_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld2053_Slot_smod_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s17_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3119smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3120smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3121smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3122smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3123smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3125smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3126smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3127smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3128smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3129smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3130smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3131smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3132smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3133smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3134smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3135smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3136smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3137smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3138smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3139smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3140smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3141smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3142smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3143smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3144smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3145smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3146smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3148smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3149smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3150_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3152smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3153smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3155smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3156smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3157smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3158smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3159smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3160smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3161smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3164smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3165smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3166smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3168smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3170smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3171smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3172smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3173smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3174smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3175smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3176smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3177smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3178smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3179smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3180smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3181smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3182smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3184smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3186smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3188smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3832smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3833smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3834_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3836smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3837smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3838smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3841smod_slot0_Slot_smod_slot0_set,
  Field_dsp340050b49a6c_fld3842smod_slot0_Slot_smod_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_llr_slot2_get_field_fns[] = {
  Field_t_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2030_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2032_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2045_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2047_Slot_llr_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2050_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2051_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2052_Slot_llr_slot2_get,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2027_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s18_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld2074_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3191llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3192llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3193llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3194llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3195llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3196llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3197llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3198llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3199llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3200llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3201llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3202llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3203llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3204llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3205_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3206_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3207llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3208llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3210llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3212_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3213llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3214_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3215llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3216llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3217_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3218llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3220llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3221llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3222llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3224llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3225_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3226llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3228llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3230_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3231llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3232llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3233_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3234llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3235llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3236_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3237llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3238llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3240llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3241llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3242llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3243llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3244llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3245llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3246_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3247llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3843llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3844_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3845llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3847llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3848llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3849llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3850llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3851llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3853llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3855llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3856llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3857llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3859llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3860llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3861llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3862_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3863llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3864llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3865llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3866llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3867llr_slot2_Slot_llr_slot2_get,
  Field_dsp340050b49a6c_fld3868_Slot_llr_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_llr_slot2_set_field_fns[] = {
  Field_t_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2030_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2032_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2045_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2047_Slot_llr_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2050_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2051_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2052_Slot_llr_slot2_set,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2027_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s18_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld2074_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3191llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3192llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3193llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3194llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3195llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3196llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3197llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3198llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3199llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3200llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3201llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3202llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3203llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3204llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3205_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3206_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3207llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3208llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3210llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3212_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3213llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3214_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3215llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3216llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3217_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3218llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3220llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3221llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3222llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3224llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3225_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3226llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3228llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3230_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3231llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3232llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3233_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3234llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3235llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3236_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3237llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3238llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3240llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3241llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3242llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3243llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3244llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3245llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3246_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3247llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3843llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3844_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3845llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3847llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3848llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3849llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3850llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3851llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3853llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3855llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3856llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3857llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3859llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3860llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3861llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3862_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3863llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3864llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3865llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3866llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3867llr_slot2_Slot_llr_slot2_set,
  Field_dsp340050b49a6c_fld3868_Slot_llr_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_llr_slot1_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2047_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld2048_Slot_llr_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld2056_Slot_llr_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld2031_Slot_llr_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2028_Slot_llr_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2033_Slot_llr_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s19_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld2034_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3248llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3250llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3251llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3252llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3253llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3254llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3869llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3870_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3872llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3875llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3876llr_slot1_Slot_llr_slot1_get,
  Field_dsp340050b49a6c_fld3878llr_slot1_Slot_llr_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_llr_slot1_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2047_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld2048_Slot_llr_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2055_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld2056_Slot_llr_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2026_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld2031_Slot_llr_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2028_Slot_llr_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2033_Slot_llr_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s19_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld2034_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3248llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3250llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3251llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3252llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3253llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3254llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3869llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3870_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3872llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3875llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3876llr_slot1_Slot_llr_slot1_set,
  Field_dsp340050b49a6c_fld3878llr_slot1_Slot_llr_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_llr_slot0_get_field_fns[] = {
  Field_t_Slot_llr_slot0_get,
  0,
  0,
  0,
  0,
  Field_s_Slot_llr_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_llr_slot0_get,
  0,
  0,
  0,
  Field_sal_Slot_llr_slot0_get,
  Field_sargt_Slot_llr_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_llr_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_llr_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2032_Slot_llr_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2037_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2038_Slot_llr_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_llr_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2043_Slot_llr_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_llr_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2050_Slot_llr_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2053_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2054_Slot_llr_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_llr_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s20_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld2071_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3258llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3259llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3260llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3261llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3263llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3264llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3265llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3266llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3267llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3268llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3269llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3270llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3272llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3274llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3275llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3276llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3277llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3278llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3279llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3280llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3281llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3282llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3283llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3284llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3286llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3288llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3289llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3291llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3292llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3293llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3294llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3295llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3296llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3297llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3298llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3299llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3300llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3302llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3303llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3304llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3305llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3306llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3308llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3310llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3311llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3312llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3879llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3881llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3883llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3885llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3887llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3888llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3890llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3892llr_slot0_Slot_llr_slot0_get,
  Field_dsp340050b49a6c_fld3893llr_slot0_Slot_llr_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_llr_slot0_set_field_fns[] = {
  Field_t_Slot_llr_slot0_set,
  0,
  0,
  0,
  0,
  Field_s_Slot_llr_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_r_Slot_llr_slot0_set,
  0,
  0,
  0,
  Field_sal_Slot_llr_slot0_set,
  Field_sargt_Slot_llr_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm7_Slot_llr_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_llr_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2032_Slot_llr_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2037_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2038_Slot_llr_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2041_Slot_llr_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2043_Slot_llr_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_llr_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2050_Slot_llr_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2052_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2053_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2054_Slot_llr_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_llr_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s20_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld2071_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3258llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3259llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3260llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3261llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3263llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3264llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3265llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3266llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3267llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3268llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3269llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3270llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3272llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3274llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3275llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3276llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3277llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3278llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3279llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3280llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3281llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3282llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3283llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3284llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3286llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3288llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3289llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3291llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3292llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3293llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3294llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3295llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3296llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3297llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3298llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3299llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3300llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3302llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3303llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3304llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3305llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3306llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3308llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3310llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3311llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3312llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3879llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3881llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3883llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3885llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3887llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3888llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3890llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3892llr_slot0_Slot_llr_slot0_set,
  Field_dsp340050b49a6c_fld3893llr_slot0_Slot_llr_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_dual_slot2_get_field_fns[] = {
  Field_t_Slot_dual_slot2_get,
  0,
  Field_bbi_Slot_dual_slot2_get,
  Field_imm12_Slot_dual_slot2_get,
  Field_imm8_Slot_dual_slot2_get,
  Field_s_Slot_dual_slot2_get,
  Field_imm12b_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_dual_slot2_get,
  Field_r_Slot_dual_slot2_get,
  0,
  0,
  Field_sae_Slot_dual_slot2_get,
  Field_sal_Slot_dual_slot2_get,
  Field_sargt_Slot_dual_slot2_get,
  0,
  Field_sas_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_dual_slot2_get,
  Field_imm7_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  Field_s4_Slot_dual_slot2_get,
  0,
  0,
  Field_s8_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2030_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2032_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2045_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2047_Slot_dual_slot2_get,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2050_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2051_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2052_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2053_Slot_dual_slot2_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld2027_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s21_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3313dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3314_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3315dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3316dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3317dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3318_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3319dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3320dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3321dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3322dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3323dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3324dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3325dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3326dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3327dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3328dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3329dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3330dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3331dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3332dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3333dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3334dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3335dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3336dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3337dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3339dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3340dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3341dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3342dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3345dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3347dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3348dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3349dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3350dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3353dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3354dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3356dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3358dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3360dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3361dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3362dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3363dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3364_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3365dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3366dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3367dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3368dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3369dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3370dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3371dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3372dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3373dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3374dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3375dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3376dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3377dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3378dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3379dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3380dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3381dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3382dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3384dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3385dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3386dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3387dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3388dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3390dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3392dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3394dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3396dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3397dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3399dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3401dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3403dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3404dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3406dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3407_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3408dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3410_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3411dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3412dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3413dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3414dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3415dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3416dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3417dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3418dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3419dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3420dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3421dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3422dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3423dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3424dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3425dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3426dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3427dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3428dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3429dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3430dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3431dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3432dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3433dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3434dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3435dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3436dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3437dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3438dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3439dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3440dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3441dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3442dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3443dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3444dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3445dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3446dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3448dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3450dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3451dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3453dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3454dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3456dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3457dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3458dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3459dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3460dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3461dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3462dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3464dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3465dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3466_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3467dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3468dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3469dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3470dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3471dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3472dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3473dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3474dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3475dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3477dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3478dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3479dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3480dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3481dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3482dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3484dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3894dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3895dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3896dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3897dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3898dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3899dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3900_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3901dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3903dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3904dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3905dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3906dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3907dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3908dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3909dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3910dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3913dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3914dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3916dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3917dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3918dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3919dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3920dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3921dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3922dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3923dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3924dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3925dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3927dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3928dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3929dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3930dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3931dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3933dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3934dual_slot2_Slot_dual_slot2_get,
  Field_dsp340050b49a6c_fld3935dual_slot2_Slot_dual_slot2_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_dual_slot2_set_field_fns[] = {
  Field_t_Slot_dual_slot2_set,
  0,
  Field_bbi_Slot_dual_slot2_set,
  Field_imm12_Slot_dual_slot2_set,
  Field_imm8_Slot_dual_slot2_set,
  Field_s_Slot_dual_slot2_set,
  Field_imm12b_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_dual_slot2_set,
  Field_r_Slot_dual_slot2_set,
  0,
  0,
  Field_sae_Slot_dual_slot2_set,
  Field_sal_Slot_dual_slot2_set,
  Field_sargt_Slot_dual_slot2_set,
  0,
  Field_sas_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_dual_slot2_set,
  Field_imm7_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  Field_s4_Slot_dual_slot2_set,
  0,
  0,
  Field_s8_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2030_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2032_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2044_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2045_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2046_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2047_Slot_dual_slot2_set,
  0,
  Field_dsp340050b49a6c_fld2049_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2050_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2051_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2052_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2053_Slot_dual_slot2_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2056_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_dsp340050b49a6c_fld2025_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld2027_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s21_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3313dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3314_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3315dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3316dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3317dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3318_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3319dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3320dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3321dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3322dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3323dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3324dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3325dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3326dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3327dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3328dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3329dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3330dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3331dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3332dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3333dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3334dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3335dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3336dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3337dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3339dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3340dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3341dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3342dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3345dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3347dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3348dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3349dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3350dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3353dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3354dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3356dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3358dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3360dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3361dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3362dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3363dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3364_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3365dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3366dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3367dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3368dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3369dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3370dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3371dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3372dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3373dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3374dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3375dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3376dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3377dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3378dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3379dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3380dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3381dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3382dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3384dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3385dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3386dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3387dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3388dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3390dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3392dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3394dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3396dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3397dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3399dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3401dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3403dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3404dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3406dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3407_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3408dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3410_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3411dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3412dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3413dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3414dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3415dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3416dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3417dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3418dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3419dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3420dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3421dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3422dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3423dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3424dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3425dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3426dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3427dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3428dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3429dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3430dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3431dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3432dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3433dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3434dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3435dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3436dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3437dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3438dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3439dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3440dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3441dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3442dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3443dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3444dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3445dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3446dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3448dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3450dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3451dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3453dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3454dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3456dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3457dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3458dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3459dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3460dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3461dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3462dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3464dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3465dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3466_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3467dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3468dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3469dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3470dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3471dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3472dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3473dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3474dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3475dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3477dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3478dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3479dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3480dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3481dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3482dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3484dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3894dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3895dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3896dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3897dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3898dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3899dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3900_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3901dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3903dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3904dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3905dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3906dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3907dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3908dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3909dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3910dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3913dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3914dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3916dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3917dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3918dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3919dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3920dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3921dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3922dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3923dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3924dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3925dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3927dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3928dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3929dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3930dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3931dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3933dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3934dual_slot2_Slot_dual_slot2_set,
  Field_dsp340050b49a6c_fld3935dual_slot2_Slot_dual_slot2_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_dual_slot1_get_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s22_Slot_dual_slot1_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_dual_slot1_set_field_fns[] = {
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s22_Slot_dual_slot1_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_get_field_fn
Slot_dual_slot0_get_field_fns[] = {
  Field_t_Slot_dual_slot0_get,
  0,
  Field_bbi_Slot_dual_slot0_get,
  Field_imm12_Slot_dual_slot0_get,
  Field_imm8_Slot_dual_slot0_get,
  Field_s_Slot_dual_slot0_get,
  Field_imm12b_Slot_dual_slot0_get,
  Field_imm16_Slot_dual_slot0_get,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_dual_slot0_get,
  Field_r_Slot_dual_slot0_get,
  0,
  0,
  Field_sae_Slot_dual_slot0_get,
  Field_sal_Slot_dual_slot0_get,
  Field_sargt_Slot_dual_slot0_get,
  0,
  Field_sas_Slot_dual_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_dual_slot0_get,
  Field_imm7_Slot_dual_slot0_get,
  0,
  0,
  0,
  0,
  Field_s4_Slot_dual_slot0_get,
  0,
  0,
  Field_s8_Slot_dual_slot0_get,
  0,
  0,
  0,
  Field_fimm8_Slot_dual_slot0_get,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2030_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2032_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2035_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2036_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2037_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2038_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2039_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2040_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2041_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2042_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2043_Slot_dual_slot0_get,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2046_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2047_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2048_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2049_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2050_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2051_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2052_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2053_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2054_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2055_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2056_Slot_dual_slot0_get,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s23_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2057_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2060_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2066_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2072_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld2079_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3487dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3488dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3489dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3490dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3491dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3492dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3493dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3494dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3496dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3497dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3498dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3499dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3500dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3502dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3504dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3505dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3506dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3507dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3508dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3509dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3510dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3511dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3512dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3513dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3514dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3515dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3516dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3517dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3518dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3519dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3520dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3522dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3523dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3524dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3527dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3529dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3530dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3531_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3532dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3533dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3535dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3536dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3537dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3538dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3539dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3541dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3542dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3543dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3544dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3545dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3546dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3547dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3548dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3549dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3550dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3551dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3552_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3553dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3554dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3555dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3556dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3557dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3558dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3559dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3560dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3562dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3563dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3564dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3565dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3566dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3567dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3568dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3569dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3570dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3571dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3572dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3573dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3574dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3575dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3576dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3577dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3578dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3579dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3580dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3581dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3582dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3583dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3584_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3585dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3587dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3588dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3589dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3590dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3591dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3592dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3593dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3594dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3595dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3596dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3597dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3598dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3599dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3600dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3601dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3603dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3604dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3606dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3607dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3608dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3609dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3610_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3611dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3612dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3613dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3614dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3615dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3616dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3618dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3619_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3621dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3622dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3623dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3624dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3625dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3626dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3936dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3937dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3938dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3939dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3940dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3941dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3943dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3945dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3946dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3947dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3949dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3950dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3951dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3952dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3954dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3957dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3958dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3959dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3960dual_slot0_Slot_dual_slot0_get,
  Field_dsp340050b49a6c_fld3961dual_slot0_Slot_dual_slot0_get,
  Implicit_Field_ar0_get,
  Implicit_Field_ar4_get,
  Implicit_Field_ar8_get,
  Implicit_Field_ar12_get,
  Implicit_Field_bt16_get,
  Implicit_Field_bs16_get,
  Implicit_Field_br16_get,
  Implicit_Field_brall_get
};

static xtensa_set_field_fn
Slot_dual_slot0_set_field_fns[] = {
  Field_t_Slot_dual_slot0_set,
  0,
  Field_bbi_Slot_dual_slot0_set,
  Field_imm12_Slot_dual_slot0_set,
  Field_imm8_Slot_dual_slot0_set,
  Field_s_Slot_dual_slot0_set,
  Field_imm12b_Slot_dual_slot0_set,
  Field_imm16_Slot_dual_slot0_set,
  0,
  0,
  0,
  0,
  0,
  Field_op2_Slot_dual_slot0_set,
  Field_r_Slot_dual_slot0_set,
  0,
  0,
  Field_sae_Slot_dual_slot0_set,
  Field_sal_Slot_dual_slot0_set,
  Field_sargt_Slot_dual_slot0_set,
  0,
  Field_sas_Slot_dual_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_imm6_Slot_dual_slot0_set,
  Field_imm7_Slot_dual_slot0_set,
  0,
  0,
  0,
  0,
  Field_s4_Slot_dual_slot0_set,
  0,
  0,
  Field_s8_Slot_dual_slot0_set,
  0,
  0,
  0,
  Field_fimm8_Slot_dual_slot0_set,
  0,
  0,
  Field_dsp340050b49a6c_fld2029_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2030_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2032_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2035_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2036_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2037_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2038_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2039_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2040_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2041_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2042_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2043_Slot_dual_slot0_set,
  0,
  Field_dsp340050b49a6c_fld2045_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2046_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2047_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2048_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2049_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2050_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2051_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2052_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2053_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2054_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2055_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2056_Slot_dual_slot0_set,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  0,
  Field_op0_s23_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2057_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2060_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2066_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2072_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld2079_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3487dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3488dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3489dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3490dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3491dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3492dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3493dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3494dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3496dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3497dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3498dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3499dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3500dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3502dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3504dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3505dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3506dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3507dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3508dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3509dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3510dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3511dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3512dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3513dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3514dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3515dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3516dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3517dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3518dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3519dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3520dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3522dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3523dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3524dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3527dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3529dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3530dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3531_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3532dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3533dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3535dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3536dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3537dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3538dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3539dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3541dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3542dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3543dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3544dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3545dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3546dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3547dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3548dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3549dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3550dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3551dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3552_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3553dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3554dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3555dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3556dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3557dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3558dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3559dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3560dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3562dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3563dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3564dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3565dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3566dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3567dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3568dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3569dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3570dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3571dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3572dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3573dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3574dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3575dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3576dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3577dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3578dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3579dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3580dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3581dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3582dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3583dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3584_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3585dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3587dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3588dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3589dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3590dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3591dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3592dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3593dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3594dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3595dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3596dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3597dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3598dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3599dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3600dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3601dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3602_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3603dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3604dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3606dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3607dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3608dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3609dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3610_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3611dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3612dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3613dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3614dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3615dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3616dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3618dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3619_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3620dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3621dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3622dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3623dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3624dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3625dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3626dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3936dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3937dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3938dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3939dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3940dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3941dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3943dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3945dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3946dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3947dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3949dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3950dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3951dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3952dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3954dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3957dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3958dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3959dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3960dual_slot0_Slot_dual_slot0_set,
  Field_dsp340050b49a6c_fld3961dual_slot0_Slot_dual_slot0_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set,
  Implicit_Field_set
};

static xtensa_slot_internal slots[] = {
  { "Inst", "x24", 0,
    Slot_x24_Format_inst_0_get, Slot_x24_Format_inst_0_set,
    Slot_inst_get_field_fns, Slot_inst_set_field_fns,
    Slot_inst_decode, "nop" },
  { "Inst16a", "x16a", 0,
    Slot_x16a_Format_inst16a_0_get, Slot_x16a_Format_inst16a_0_set,
    Slot_inst16a_get_field_fns, Slot_inst16a_set_field_fns,
    Slot_inst16a_decode, "" },
  { "Inst16b", "x16b", 0,
    Slot_x16b_Format_inst16b_0_get, Slot_x16b_Format_inst16b_0_set,
    Slot_inst16b_get_field_fns, Slot_inst16b_set_field_fns,
    Slot_inst16b_decode, "nop.n" },
  { "GP_slot2", "GP", 2,
    Slot_gp_Format_gp_slot2_43_get, Slot_gp_Format_gp_slot2_43_set,
    Slot_gp_slot2_get_field_fns, Slot_gp_slot2_set_field_fns,
    Slot_gp_slot2_decode, "nop" },
  { "GP_slot1", "GP", 1,
    Slot_gp_Format_gp_slot1_26_get, Slot_gp_Format_gp_slot1_26_set,
    Slot_gp_slot1_get_field_fns, Slot_gp_slot1_set_field_fns,
    Slot_gp_slot1_decode, "nop" },
  { "GP_slot0", "GP", 0,
    Slot_gp_Format_gp_slot0_7_get, Slot_gp_Format_gp_slot0_7_set,
    Slot_gp_slot0_get_field_fns, Slot_gp_slot0_set_field_fns,
    Slot_gp_slot0_decode, "nop" },
  { "DOT_slot2", "DOT", 2,
    Slot_dot_Format_dot_slot2_44_get, Slot_dot_Format_dot_slot2_44_set,
    Slot_dot_slot2_get_field_fns, Slot_dot_slot2_set_field_fns,
    Slot_dot_slot2_decode, "nop" },
  { "DOT_slot1", "DOT", 1,
    Slot_dot_Format_dot_slot1_24_get, Slot_dot_Format_dot_slot1_24_set,
    Slot_dot_slot1_get_field_fns, Slot_dot_slot1_set_field_fns,
    Slot_dot_slot1_decode, "nop" },
  { "DOT_slot0", "DOT", 0,
    Slot_dot_Format_dot_slot0_7_get, Slot_dot_Format_dot_slot0_7_set,
    Slot_dot_slot0_get_field_fns, Slot_dot_slot0_set_field_fns,
    Slot_dot_slot0_decode, "nop" },
  { "PQ_slot2", "PQ", 2,
    Slot_pq_Format_pq_slot2_40_get, Slot_pq_Format_pq_slot2_40_set,
    Slot_pq_slot2_get_field_fns, Slot_pq_slot2_set_field_fns,
    Slot_pq_slot2_decode, "nop" },
  { "PQ_slot1", "PQ", 1,
    Slot_pq_Format_pq_slot1_26_get, Slot_pq_Format_pq_slot1_26_set,
    Slot_pq_slot1_get_field_fns, Slot_pq_slot1_set_field_fns,
    Slot_pq_slot1_decode, "nop" },
  { "PQ_slot0", "PQ", 0,
    Slot_pq_Format_pq_slot0_7_get, Slot_pq_Format_pq_slot0_7_set,
    Slot_pq_slot0_get_field_fns, Slot_pq_slot0_set_field_fns,
    Slot_pq_slot0_decode, "nop" },
  { "ACC2_slot2", "ACC2", 2,
    Slot_acc2_Format_acc2_slot2_47_get, Slot_acc2_Format_acc2_slot2_47_set,
    Slot_acc2_slot2_get_field_fns, Slot_acc2_slot2_set_field_fns,
    Slot_acc2_slot2_decode, "nop" },
  { "ACC2_slot1", "ACC2", 1,
    Slot_acc2_Format_acc2_slot1_23_get, Slot_acc2_Format_acc2_slot1_23_set,
    Slot_acc2_slot1_get_field_fns, Slot_acc2_slot1_set_field_fns,
    Slot_acc2_slot1_decode, "nop" },
  { "ACC2_slot0", "ACC2", 0,
    Slot_acc2_Format_acc2_slot0_7_get, Slot_acc2_Format_acc2_slot0_7_set,
    Slot_acc2_slot0_get_field_fns, Slot_acc2_slot0_set_field_fns,
    Slot_acc2_slot0_decode, "nop" },
  { "SMOD_slot2", "SMOD", 2,
    Slot_smod_Format_smod_slot2_42_get, Slot_smod_Format_smod_slot2_42_set,
    Slot_smod_slot2_get_field_fns, Slot_smod_slot2_set_field_fns,
    Slot_smod_slot2_decode, "nop" },
  { "SMOD_slot1", "SMOD", 1,
    Slot_smod_Format_smod_slot1_26_get, Slot_smod_Format_smod_slot1_26_set,
    Slot_smod_slot1_get_field_fns, Slot_smod_slot1_set_field_fns,
    Slot_smod_slot1_decode, "nop" },
  { "SMOD_slot0", "SMOD", 0,
    Slot_smod_Format_smod_slot0_7_get, Slot_smod_Format_smod_slot0_7_set,
    Slot_smod_slot0_get_field_fns, Slot_smod_slot0_set_field_fns,
    Slot_smod_slot0_decode, "nop" },
  { "LLR_slot2", "LLR", 2,
    Slot_llr_Format_llr_slot2_44_get, Slot_llr_Format_llr_slot2_44_set,
    Slot_llr_slot2_get_field_fns, Slot_llr_slot2_set_field_fns,
    Slot_llr_slot2_decode, "nop" },
  { "LLR_slot1", "LLR", 1,
    Slot_llr_Format_llr_slot1_24_get, Slot_llr_Format_llr_slot1_24_set,
    Slot_llr_slot1_get_field_fns, Slot_llr_slot1_set_field_fns,
    Slot_llr_slot1_decode, "nop" },
  { "LLR_slot0", "LLR", 0,
    Slot_llr_Format_llr_slot0_7_get, Slot_llr_Format_llr_slot0_7_set,
    Slot_llr_slot0_get_field_fns, Slot_llr_slot0_set_field_fns,
    Slot_llr_slot0_decode, "nop" },
  { "DUAL_slot2", "DUAL", 2,
    Slot_dual_Format_dual_slot2_31_get, Slot_dual_Format_dual_slot2_31_set,
    Slot_dual_slot2_get_field_fns, Slot_dual_slot2_set_field_fns,
    Slot_dual_slot2_decode, "nop" },
  { "DUAL_slot1", "DUAL", 1,
    Slot_dual_Format_dual_slot1_30_get, Slot_dual_Format_dual_slot1_30_set,
    Slot_dual_slot1_get_field_fns, Slot_dual_slot1_set_field_fns,
    Slot_dual_slot1_decode, "nop" },
  { "DUAL_slot0", "DUAL", 0,
    Slot_dual_Format_dual_slot0_7_get, Slot_dual_Format_dual_slot0_7_set,
    Slot_dual_slot0_get_field_fns, Slot_dual_slot0_set_field_fns,
    Slot_dual_slot0_decode, "nop" }
};


/* Instruction formats.  */

static void
Format_x24_encode (xtensa_insnbuf insn)
{
  insn[0] = 0;
  insn[1] = 0;
}

static void
Format_x16a_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x8;
  insn[1] = 0;
}

static void
Format_x16b_encode (xtensa_insnbuf insn)
{
  insn[0] = 0xc;
  insn[1] = 0;
}

static void
Format_GP_encode (xtensa_insnbuf insn)
{
  insn[0] = 0xf;
  insn[1] = 0;
}

static void
Format_DOT_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x1f;
  insn[1] = 0;
}

static void
Format_PQ_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x2f;
  insn[1] = 0;
}

static void
Format_ACC2_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x3f;
  insn[1] = 0;
}

static void
Format_SMOD_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x4f;
  insn[1] = 0;
}

static void
Format_LLR_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x5f;
  insn[1] = 0;
}

static void
Format_DUAL_encode (xtensa_insnbuf insn)
{
  insn[0] = 0x6f;
  insn[1] = 0;
}

static int Format_x24_slots[] = { 0 };

static int Format_x16a_slots[] = { 1 };

static int Format_x16b_slots[] = { 2 };

static int Format_GP_slots[] = { 5, 4, 3 };

static int Format_DOT_slots[] = { 8, 7, 6 };

static int Format_PQ_slots[] = { 11, 10, 9 };

static int Format_ACC2_slots[] = { 14, 13, 12 };

static int Format_SMOD_slots[] = { 17, 16, 15 };

static int Format_LLR_slots[] = { 20, 19, 18 };

static int Format_DUAL_slots[] = { 23, 22, 21 };

static xtensa_format_internal formats[] = {
  { "x24", 3, Format_x24_encode, 1, Format_x24_slots },
  { "x16a", 2, Format_x16a_encode, 1, Format_x16a_slots },
  { "x16b", 2, Format_x16b_encode, 1, Format_x16b_slots },
  { "GP", 8, Format_GP_encode, 3, Format_GP_slots },
  { "DOT", 8, Format_DOT_encode, 3, Format_DOT_slots },
  { "PQ", 8, Format_PQ_encode, 3, Format_PQ_slots },
  { "ACC2", 8, Format_ACC2_encode, 3, Format_ACC2_slots },
  { "SMOD", 8, Format_SMOD_encode, 3, Format_SMOD_slots },
  { "LLR", 8, Format_LLR_encode, 3, Format_LLR_slots },
  { "DUAL", 8, Format_DUAL_encode, 3, Format_DUAL_slots }
};


static int
format_decoder (const xtensa_insnbuf insn)
{
  if ((insn[0] & 0x8) == 0 && (insn[1] & 0) == 0)
    return 0; /* x24 */
  if ((insn[0] & 0xc) == 0x8 && (insn[1] & 0) == 0)
    return 1; /* x16a */
  if ((insn[0] & 0xe) == 0xc && (insn[1] & 0) == 0)
    return 2; /* x16b */
  if ((insn[0] & 0x7f) == 0xf && (insn[1] & 0) == 0)
    return 3; /* GP */
  if ((insn[0] & 0x7f) == 0x1f && (insn[1] & 0) == 0)
    return 4; /* DOT */
  if ((insn[0] & 0x7f) == 0x2f && (insn[1] & 0) == 0)
    return 5; /* PQ */
  if ((insn[0] & 0x7f) == 0x3f && (insn[1] & 0) == 0)
    return 6; /* ACC2 */
  if ((insn[0] & 0x7f) == 0x4f && (insn[1] & 0) == 0)
    return 7; /* SMOD */
  if ((insn[0] & 0x7f) == 0x5f && (insn[1] & 0) == 0)
    return 8; /* LLR */
  if ((insn[0] & 0x7f) == 0x6f && (insn[1] & 0) == 0)
    return 9; /* DUAL */
  return -1;
}

static int length_table[16] = {
  3,
  3,
  3,
  3,
  3,
  3,
  3,
  3,
  2,
  2,
  2,
  2,
  2,
  2,
  -1,
  8
};

static int
length_decoder (const unsigned char *insn)
{
  int op0 = insn[0] & 0xf;
  return length_table[op0];
}


/* Top-level ISA structure.  */

xtensa_isa_internal xtensa_modules = {
  0 /* little-endian */,
  8 /* insn_size */, 0,
  10, formats, format_decoder, length_decoder,
  24, slots,
  1733 /* num_fields */,
  1828, operands,
  747, iclasses,
  839, opcodes, 0,
  10, regfiles,
  NUM_STATES, states, 0,
  NUM_SYSREGS, sysregs, 0,
  { MAX_SPECIAL_REG, MAX_USER_REG }, { 0, 0 },
  95, interfaces, 0,
  0, funcUnits, 0
};
